Package: spsurvey
Title: Spatial Sampling Design and Analysis
Version: 5.6.1
Authors@R: c(
    person("Michael", "Dumelle", role=c("aut","cre"),
           email = "Dumelle.Michael@epa.gov", comment = c(ORCID = "0000-0002-3393-5529")),
    person("Tom", "Kincaid", role="aut"),
    person("Anthony (Tony) R.", "Olsen",  role="aut"),
    person("Marc", "Weber",  role="aut"),
    person("Don", "Stevens",  role="ctb"),
    person("Denis", "White",  role="ctb"),
    person("Amanda M.", "Nahlik",  role="ctb"),
    person("Sarah", "Lehmann",  role="ctb")
    )
Maintainer: Michael Dumelle <Dumelle.Michael@epa.gov>
Description: A design-based approach to statistical inference, with a focus on spatial data. Spatially balanced samples are selected using the Generalized Random Tessellation Stratified (GRTS) algorithm. The GRTS algorithm can be applied to finite resources (point geometries) and infinite resources (linear / linestring and areal / polygon geometries) and flexibly accommodates a diverse set of sampling design features, including stratification, unequal inclusion probabilities, proportional (to size) inclusion probabilities, legacy (historical) sites, a minimum distance between sites, and two options for replacement sites (reverse hierarchical order and nearest neighbor). Data are analyzed using a wide range of analysis functions that perform categorical variable analysis, continuous variable analysis, attributable risk analysis, risk difference analysis, relative risk analysis, change analysis, and trend analysis. spsurvey can also be used to summarize objects, visualize objects, select samples that are not spatially balanced, select panel samples, measure the amount of spatial balance in a sample, adjust design weights, and more. For additional details, see Dumelle et al. (2023) <doi:10.18637/jss.v105.i03>.
Depends: R (>= 3.5.0), sf, survey (>= 4.1-1)
Imports: boot, crossdes, deldir, graphics, grDevices, lme4, MASS,
        sampling, stats, units
Suggests: knitr, testthat, rmarkdown
License: GPL (>= 3)
URL: https://usepa.github.io/spsurvey/,
        https://github.com/USEPA/spsurvey
BugReports: https://github.com/USEPA/spsurvey/issues
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-04 19:25:28 UTC; MDUMELLE
Author: Michael Dumelle [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3393-5529>),
  Tom Kincaid [aut],
  Anthony (Tony) R. Olsen [aut],
  Marc Weber [aut],
  Don Stevens [ctb],
  Denis White [ctb],
  Amanda M. Nahlik [ctb],
  Sarah Lehmann [ctb]
Repository: CRAN
Date/Publication: 2026-05-04 21:10:02 UTC
Built: R 4.6.0; ; 2026-05-05 00:13:35 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,sp_CDF)
S3method(plot,sp_design)
S3method(plot,sp_frame)
S3method(print,sp_design)
S3method(print,sp_summary.sp_design)
S3method(print,sp_summary.sp_frame)
S3method(print,summary.sp_design)
S3method(print,summary.sp_frame)
S3method(sp_plot,default)
S3method(sp_plot,sp_design)
S3method(sp_summary,default)
S3method(sp_summary,sp_design)
S3method(summary,sp_design)
S3method(summary,sp_frame)
export(adjwgt)
export(adjwgtNR)
export(ash1_wgt)
export(attrisk_analysis)
export(cat_analysis)
export(cdf_plot)
export(change_analysis)
export(cont_analysis)
export(cont_cdfplot)
export(cont_cdftest)
export(cov_panel_dsgn)
export(diffrisk_analysis)
export(errorprnt)
export(grts)
export(irs)
export(localmean_cov)
export(localmean_var)
export(localmean_weight)
export(pd_summary)
export(power_dsgn)
export(ppd_plot)
export(relrisk_analysis)
export(revisit_bibd)
export(revisit_dsgn)
export(revisit_rand)
export(sp_balance)
export(sp_frame)
export(sp_plot)
export(sp_rbind)
export(sp_summary)
export(sp_unframe)
export(stopprnt)
export(trend_analysis)
export(warnprnt)
import(sf)
import(survey)
importFrom(MASS,ginv)
importFrom(boot,boot)
importFrom(crossdes,find.BIB)
importFrom(deldir,deldir)
importFrom(deldir,tile.list)
importFrom(grDevices,graphics.off)
importFrom(grDevices,pdf)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(lme4,VarCorr)
importFrom(lme4,lmer)
importFrom(sampling,UPpivotal)
importFrom(stats,addmargins)
importFrom(stats,chisq.test)
importFrom(stats,confint)
importFrom(stats,dist)
importFrom(stats,dnorm)
importFrom(stats,extractAIC)
importFrom(stats,ftable)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,optimize)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,weights)
importFrom(units,set_units)

# spsurvey <img src="man/figures/logo.png" align="right" height="130" alt="" />

<!-- badges: start -->
[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN](http://www.r-pkg.org/badges/version/spsurvey)](https://cran.r-project.org/package=spsurvey)
[![cran checks](https://badges.cranchecks.info/worst/spsurvey.svg)](https://cran.r-project.org/web/checks/check_results_spsurvey.html)
[![R-CMD-check](https://github.com/USEPA/spsurvey/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/USEPA/spsurvey/actions/workflows/R-CMD-check.yaml)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/spsurvey)](https://cran.r-project.org/package=spsurvey)

<!-- badges: end -->

# Overview

`spsurvey` is an R package that implements a design-based approach to statistical inference,
with a focus on spatial data.
Spatially balanced samples are selected using the
Generalized Random Tessellation Stratified (GRTS) algorithm.
The GRTS algorithm can be applied to finite resources (point geometries) and
infinite resources (linear / linestring and areal / polygon geometries) and flexibly
accommodates a diverse set of sampling design features, including
stratification, unequal inclusion probabilities, proportional (to size)
inclusion probabilities, legacy (historical) sites, a minimum distance between
sites, and two options for replacement sites (reverse hierarchical order and
nearest neighbor). Data are analyzed using a wide
range of analysis functions that perform categorical variable analysis, continuous
variable analysis, attributable risk analysis, risk difference analysis, relative
risk analysis, change analysis, and trend analysis. `spsurvey` can also be used to
summarize objects, visualize objects, select samples that are not spatially balanced,
select panel samples, measure the amount of spatial balance in a sample,
adjust design weights, and more. For additional details, see Dumelle et al. (2023) <doi:10.18637/jss.v105.i03>.

## Installation

You can install and load the most recent approved version from CRAN by running

```r
# install the most recent approved version from CRAN
install.packages("spsurvey")
# load the most recent approved version from CRAN
library(spsurvey)
```

You can install and load the most recent development version of`spsurvey` from GitHub by running:

```r
# Installing from GitHub requires you first install the remotes package
install.packages("remotes")

# install the most recent development version from GitHub
remotes::install_github("USEPA/spsurvey", ref = "main")
# load the most recent development version from GitHub
library(spsurvey)
```

You can install the most recent development version of `spsurvey` from GitHub with the package vignette by running:
```r
install the most recent development version from GitHub with package vignettes
devtools::install_github("USEPA/spsurvey", build_vignettes=TRUE)
```

To view the vignette in RStudio, run
```r
vignette("start-here", "spsurvey") # start with this vignette for an spsurvey overview
```

There are several additional vignettes (six in total) in web format. To access these vignettes, visit the "Articles" tab on our website [here](https://usepa.github.io/spsurvey/).

Further detail regarding `spsurvey` is contained in the package's documentation manual available for download [here](https://cran.r-project.org/package=spsurvey).


## Citation

If you used `spsurvey` in your work, please cite it. You can view the most recent citation by running
```r
citation(package = "spsurvey")
```

```
#> To cite spsurvey in publications use:
#> 
#>   Michael Dumelle, Tom Kincaid, Anthony R. Olsen, Marc Weber (2023).
#>   spsurvey: Spatial Sampling Design and Analysis in R. Journal of
#>   Statistical Software, 105(3), 1-29. doi:10.18637/jss.v105.i03
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {{spsurvey}: Spatial Sampling Design and Analysis in {R}},
#>     author = {Michael Dumelle and Tom Kincaid and Anthony R. Olsen and Marc Weber},
#>     journal = {Journal of Statistical Software},
#>     year = {2023},
#>     volume = {105},
#>     number = {3},
#>     pages = {1--29},
#>     doi = {10.18637/jss.v105.i03},
#>   }
```

## Package Contributions

We encourage users to submit issues and enhancement requests so we may
continue to improve `spsurvey`.

## Acknowledgements

The United States Environmental Protection Agency (EPA) GitHub project code is provided on an "as is" basis and the user assumes responsibility for its use. EPA has relinquished control of the information and no longer has responsibility to protect the integrity , confidentiality, or availability of the information. Any reference to specific commercial products, processes, or services by service mark, trademark, manufacturer, or otherwise, does not constitute or imply their endorsement, recommendation or favoring by EPA. The EPA seal and logo shall not be used in any manner to imply endorsement of any commercial product or activity by EPA or the United States Government.


A special thank you to Virginia Vichi-Miller for creating the `spsurvey` hex sticker (i.e., graphic identifier). The sticker represents the environmental sampling context for which `spsurvey` was created, with an emphasis on aquatic resources. The sticker shows a sunny day filled with a fish swimming in water, aquatic vegetation, and rolling hills and trees. The word `spsurvey` is set against the backdrop of the clear sky.

### License

This project is licensed under the GNU General Public License, [GPL-3](https://cran.r-project.org/web/licenses/GPL-3).  

################################################################################
# Function: cat_localmean_prop (not exported)
# Programmer: Tom Kincaid
# Date: July 23, 2020
# Revised: October 23, 2020 to correct a coding error when calculating
#          proportion estimates for a stratum
# Revised: January 28, 2021 to replace "pcfactor.ind" with "pcfactor_ind"
# Revised: April 27, 2021 to check whether the local mean variance estimator
#          produced negative estimates and to use the SRS variance estimator
#          when that situation occurs
# Revised: June 8, 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
#
#' Local Mean Variance Estimates of Estimated Proportions for Categorical Data
#'
#' This function organizes input and output for calculation of the local mean
#' variance estimator for estimated proportions for categorical data.
#'
#' @param itype Character value that identifies a factor variable in the design
#'   argument containing subpopulation (domain) values.
#'
#' @param lev_itype Character vector that provides levels of the subpopulation
#'   variable.
#'
#' @param nlev_itype Numeric value that provides the number of levels of the
#'   subpopulation variable.
#'
#' @param ivar Character value that identifies a factor variable in the design
#'   argument containing categorical response values.
#'
#' @param lev_ivar Character vector that provides levels of the categorical
#'   response variable.
#'
#' @param nlev_ivar Numeric value that provides the number of levels of the
#'   categorical response variable.
#'
#' @param design Object of class \code{survey.design} that specifies a complex
#'   survey design.
#'
#' @param design_names Character vector that provides names of survey design
#'   variables in the design argument.
#'
#' @param rslt_P Data frame that provides proportion estimates for the
#'   categorical response variable.
#'
#' @param mult Numeric value that provides the Normal distribution confidence
#'   bound multiplier.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @return A list containing the following objects:
#'   \itemize{
#'     \item{\code{stderr_P}}{data frame containing standard error estimates}
#'     \item{\code{confval_P}}{data frame containing confidence bound estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

cat_localmean_prop <- function(itype, lev_itype, nlev_itype, ivar, lev_ivar,
                               nlev_ivar, design, design_names, rslt_P,
                               mult, warn_ind, warn_df) {

  # Assign a value to the function name variable

  fname <- "cat_localmean_prop"

  # For variables that exist in the design$variables data frame, assign survey
  # design variables

  dframe <- design$variables
  for (i in names(design_names)) {
    if (is.null(design_names[[i]])) {
      eval(parse(text = paste0(i, " <- NULL")))
    } else {
      eval(parse(text = paste0(i, " <- dframe[, \"", design_names[[i]], "\"]")))
    }
  }

  # Assign values to the categorical response variable vector, catvar

  catvar <- dframe[, ivar]

  # Assign a value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Assign values to weight variables

  if (cluster_ind) {
    wgt1 <- dframe$wgt1
    wgt2 <- dframe$wgt2
  } else {
    wgt <- dframe$wgt
  }

  # Create the output data frames for standard error estimates and confidence
  # bound estimates

  stderr_P <- data.frame(array(0, c(nlev_itype, nlev_ivar + 1)))
  dimnames(stderr_P) <- list(lev_itype, c(lev_ivar, "Total"))
  temp <- nlev_ivar + 1
  confval_P <- data.frame(array(0, c(nlev_itype * temp, 2)))
  confval_P[seq(temp, by = temp, length = nlev_itype), ] <- c(1.0, 1.0)
  dimnames(confval_P) <- list(
    paste(rep(lev_itype, rep(temp, nlev_itype)),
      rep(c(lev_ivar, "Total"), nlev_itype),
      sep = ":"
    ),
    c("LCB", "UCB")
  )

  # Loop through all subpopulations

  for (isubpop in 1:nlev_itype) {
    tst <- !is.na(dframe[, ivar]) &
      (dframe[, itype] %in% lev_itype[isubpop])

    # Assign values to the vector of category proportion estimates, prop

    prop <- unlist(rslt_P[isubpop, 1:nlev_ivar])
    prop_names <- lev_ivar

    # Assign values to the warn_vec vector

    warn_vec <- c(itype, lev_itype[isubpop], ivar)

    # Assign a value to the indicator variable for a stratified sample

    stratum_ind <- !is.null(stratumID)

    # For a stratified design, determine whether the subpopulation contains a
    # single stratum

    if (stratum_ind) {
      stratum <- factor(stratumID[tst])
      stratum_levels <- levels(stratum)
      nstrata <- length(stratum_levels)
      if (nstrata == 1) {
        stratum_ind <- FALSE
      }
    }

    # Branch for a stratified sample

    if (stratum_ind) {

      # Calculate values required for weighting strata

      if (cluster_ind) {
        popsize_hat <- tapply(wgt1[tst] * wgt2[tst], stratum, sum)
        sum_popsize_hat <- sum(wgt1[tst] * wgt2[tst])
      } else {
        popsize_hat <- tapply(wgt[tst], stratum, sum)
        sum_popsize_hat <- sum(wgt[tst])
      }

      # Begin the subsection for individual strata

      for (i in 1:nstrata) {

        # Calculate proportion estimates for the stratum

        stratum_i <- tst & stratumID == stratum_levels[i]
        prop_st <- t(as.data.frame(svymean(make.formula(ivar),
          design = subset(design, stratum_i), na.rm = TRUE
        ))[1])

        # Calculate variance estimates

        if (cluster_ind) {
          temp <- catvar_prop(
            factor(catvar[stratum_i]), wgt2[stratum_i],
            xcoord[stratum_i], ycoord[stratum_i], prop_st, prop_names,
            stratum_ind, stratum_levels[i], cluster_ind, clusterID[stratum_i],
            wgt1[stratum_i], xcoord1[stratum_i], ycoord1[stratum_i],
            warn_ind, warn_df, warn_vec
          )
        } else {
          temp <- catvar_prop(factor(catvar[stratum_i]), wgt[stratum_i],
            xcoord[stratum_i], ycoord[stratum_i], prop_st, prop_names,
            stratum_ind, stratum_levels[i], cluster_ind,
            warn_ind = warn_ind, warn_df = warn_df, warn_vec = warn_vec
          )
        }
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
        if (any(temp$varest < 0)) {
          temp$vartype <- "SRS"
          warn_ind <- TRUE
          act <- "The simple random sampling variance estimator for an infinite population was used.\n"
          warn <- paste0("The local mean variance estimator produced one or more  negative variance estimates in \nstratum \"", stratum_levels[i], "\", the simple random sampling variance estimator for an infinite \npopulation was used to calculate variance of the category proportion estimates.\n")
          warn_df <-
            rbind(
              warn_df,
              data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = I(stratum_levels[i]),
                warning = I(warn), action = I(act)
              )
            )
        }
        if (temp$vartype == "SRS") {
          rslt_svy <- svymean(make.formula(ivar),
            design = subset(design, stratum_i), na.rm = TRUE
          )
          varest <- SE(rslt_svy)^2
          names(varest) <- prop_names
        } else {
          varest <- temp$varest
        }

        # Add estimates to the stderr_P data frame

        ind <- prop_names %in% names(varest)
        indx <- (1:nlev_ivar)[ind]
        stderr_P[isubpop, indx] <- stderr_P[isubpop, indx] +
          ((popsize_hat[i] / sum_popsize_hat)^2) * varest

        # End the subsection for individual strata
      }

      # Begin the subsection for all strata combined

      # Add estimates to the data frames for results

      names_sdest <- names(stderr_P)[stderr_P[isubpop, ] > 0]
      ind <- prop_names %in% names_sdest
      indx <- (1:nlev_ivar)[ind]
      stderr_P[isubpop, indx] <- sqrt(stderr_P[isubpop, indx])
      lbound <- unlist(pmax(prop[ind] - mult * stderr_P[isubpop, indx], 0))
      ubound <- unlist(pmin(prop[ind] + mult * stderr_P[isubpop, indx], 1))
      temp <- paste(rep(lev_itype[isubpop], length(names_sdest)), names_sdest,
        sep = ":"
      )
      ind <- rownames(confval_P) %in% temp
      confval_P[ind, ] <- cbind(lbound, ubound)

      # End the subsection for all strata combined

      # Branch for an unstratified sample
    } else {

      # Calculate the standard error estimates

      if (cluster_ind) {
        temp <- catvar_prop(
          factor(catvar[tst]), wgt2[tst], xcoord[tst],
          ycoord[tst], prop, prop_names, stratum_ind, NULL, cluster_ind,
          clusterID[tst], wgt1[tst], xcoord1[tst], ycoord1[tst], warn_ind,
          warn_df, warn_vec
        )
      } else {
        temp <- catvar_prop(factor(catvar[tst]), wgt[tst], xcoord[tst],
          ycoord[tst], prop, prop_names, stratum_ind, NULL, cluster_ind,
          warn_ind = warn_ind, warn_df = warn_df, warn_vec = warn_vec
        )
      }
      warn_ind <- temp$warn_ind
      warn_df <- temp$warn_df
      if (any(temp$varest < 0)) {
        temp$vartype <- "SRS"
        warn_ind <- TRUE
        act <- "The simple random sampling variance estimator for an infinite population was used.\n"
        warn <- paste0("The local mean variance estimator produced one or more  negative variance estimates, the \nsimple random sampling variance estimator for an infinite population was used to \ncalculate variance of the category proportion estimates.\n")
        warn_df <-
          rbind(
            warn_df,
            data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            )
          )
      }
      if (temp$vartype == "SRS") {
        rslt_svy <- svymean(make.formula(ivar),
          design = subset(design, tst),
          na.rm = TRUE
        )
        sdest <- SE(rslt_svy)
        names(sdest) <- prop_names
      } else {
        sdest <- sqrt(temp$varest)
      }

      # Calculate confidence bounds

      names_sdest <- names(sdest)
      ind <- prop_names %in% names_sdest
      lbound <- pmax(prop[ind] - mult * sdest, 0)
      ubound <- pmin(prop[ind] + mult * sdest, 1)

      # Add estimates to the data frames for results

      indx <- (1:nlev_ivar)[ind]
      stderr_P[isubpop, indx] <- sdest
      temp <- paste(rep(lev_itype[isubpop], length(names_sdest)), names_sdest,
        sep = ":"
      )
      ind <- rownames(confval_P) %in% temp
      confval_P[ind, ] <- cbind(lbound, ubound)
    }
  }

  # Return results

  list(
    stderr_P = stderr_P,
    confval_P = confval_P,
    warn_ind = warn_ind,
    warn_df = warn_df
  )
}

################################################################################
# Function: cdf_localmean_total (not exported)
# Programmer: Tom Kincaid
# Date: July 9, 2020
# Revised: April 27, 2021 to check whether the local mean variance estimator
#          produced negative estimates and to use the SRS variance estimator
#          when that situation occurs
# Revised: June 8, 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
#
#' Local Mean Variance Estimates of the Estimated CDF using the Total Scale
#'
#' This function organizes input and output for calculation of the local mean
#' variance estimator for the estimated CDF using the total scale.
#'
#' @param itype Character value that identifies a factor variable in the design
#'   argument containing subpopulation (domain) values.
#'
#' @param lev_itype Character vector that provides levels of the subpopulation
#'   variable.
#'
#' @param nlev_itype Numeric value that provides the number of levels of the
#'   subpopulation variable.
#'
#' @param ivar Character value that identifies a factor variable in the design
#'   argument containing categorical response values.
#'
#' @param design Object of class \code{survey.design} that specifies a complex
#'   survey design.
#'
#' @param design_names Character vector that provides names of survey design
#'   variables in the design argument.
#'
#' @param cdfval Vector of the set of values at which the CDF is estimated.
#'
#' @param ncdfval Numeric value containing length of the set of values at which
#'   the CDF is estimated.
#'
#' @param cdfest_U Object that provides CDF estimates on the total scale for the
#'   continuous response variable.
#'
#' @param mult Numeric value that provides the Normal distribution confidence
#'   bound multiplier.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @return A list containing the following objects:
#'   \itemize{
#'     \item{\code{stderr_U}}{data frame containing standard error estimates}
#'     \item{\code{confval_U}}{data frame containing confidence bound estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

cdf_localmean_total <- function(itype, lev_itype, nlev_itype, ivar, design,
                                design_names, cdfval, ncdfval, cdfest_U,
                                mult, warn_ind, warn_df) {

  # Assign a value to the function name variable

  fname <- "cdf_localmean_total"

  # For variables that exist in the design$variables data frame, assign survey
  # design variables

  dframe <- design$variables
  for (i in names(design_names)) {
    if (is.null(design_names[[i]])) {
      eval(parse(text = paste0(i, " <- NULL")))
    } else {
      eval(parse(text = paste0(i, " <- dframe[, \"", design_names[[i]], "\"]")))
    }
  }

  # Assign values to the continuous response variable vector, contvar

  contvar <- dframe[, ivar]

  # Assign a value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Assign values to weight variables

  if (cluster_ind) {
    wgt1 <- dframe$wgt1
    wgt2 <- dframe$wgt2
  } else {
    wgt <- dframe$wgt
  }

  # Create the output data frames for standard error estimates and confidence
  # bound estimates

  stderr_U <- data.frame(array(0, c(nlev_itype, ncdfval)))
  rownames(stderr_U) <- lev_itype
  confval_U <- data.frame(array(0, c(nlev_itype * ncdfval, 2)))
  dimnames(confval_U) <- list(
    paste(rep(lev_itype, ncdfval),
      rep(1:ncdfval, rep(nlev_itype, ncdfval)),
      sep = ":"
    ),
    c("LCB", "UCB")
  )

  # Loop through all subpopulations

  for (isubpop in 1:nlev_itype) {
    tst <- !is.na(dframe[, ivar]) &
      (dframe[, itype] %in% lev_itype[isubpop])

    # If post-stratification or calibration was applied to the design object, then
    # calculate the sum of weights for the subpopulation

    if ("postStrata" %in% names(design)) {
      totalwgt <- sum(weights(design)[tst])
    }

    # Assign values to the vector of CDF estimates, size

    size <- unlist(cdfest_U[isubpop, ])

    # Assign values to the warn_vec vector

    warn_vec <- c(itype, lev_itype[isubpop], ivar)

    # Assign a value to the indicator variable for a stratified sample

    stratum_ind <- !is.null(stratumID)

    # For a stratified design, determine whether the subpopulation contains a single
    # stratum

    if (stratum_ind) {
      stratum <- factor(stratumID[tst])
      stratum_levels <- levels(stratum)
      nstrata <- length(stratum_levels)
      if (nstrata == 1) {
        stratum_ind <- FALSE
      }
    }

    # Branch for a stratified sample

    if (stratum_ind) {

      # Begin the subsection for individual strata

      for (i in 1:nstrata) {

        # Calculate variance estimates

        stratum_i <- tst & stratumID == stratum_levels[i]
        if (cluster_ind) {
          temp <- cdfvar_total(
            contvar[stratum_i], wgt2[stratum_i],
            xcoord[stratum_i], ycoord[stratum_i], cdfval, stratum_ind,
            stratum_levels[i], cluster_ind, clusterID[stratum_i],
            wgt1[stratum_i], xcoord1[stratum_i], ycoord1[stratum_i], warn_ind,
            warn_df, warn_vec
          )
        } else {
          temp <- cdfvar_total(contvar[stratum_i], wgt[stratum_i],
            xcoord[stratum_i], ycoord[stratum_i], cdfval, stratum_ind,
            stratum_levels[i], cluster_ind,
            warn_ind = warn_ind,
            warn_df = warn_df, warn_vec = warn_vec
          )
        }
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
        if (any(temp$varest < 0)) {
          temp$vartype <- "SRS"
          warn_ind <- TRUE
          act <- "The simple random sampling variance estimator for an infinite population was used.\n"
          warn <- paste0("The local mean variance estimator produced one or more  negative variance estimates in \nstratum \"", stratum_levels[i], "\", the simple random sampling variance estimator for an infinite \npopulation was used to calculate variance of the CDF total estimates.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname),
            subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], I(stratum_levels[i]),
            warning = I(warn), action = I(act)
          ))
        }
        if (temp$vartype == "SRS") {
          rslt_svy <- lapply(cdfval, function(x) {
            svytotal(make.formula(paste0("I(", ivar, " <= ", x, ")")),
              design = subset(design, tst), na.rm = TRUE
            )
          })
          varest <- sapply(rslt_svy, function(x) SE(x)[2])^2
        } else {
          varest <- temp$varest
        }

        # Add estimates to the stderr_U data frame

        stderr_U[isubpop, ] <- stderr_U[isubpop, ] + varest

        # End the subsection for individual strata
      }

      # Begin the subsection for all strata combined

      # Add estimates to the data frames for results

      if ("postStrata" %in% names(design)) {
        tst <- size == totalwgt
        stderr_U[isubpop, tst] <- 0
      }
      stderr_U[isubpop, ] <- sqrt(stderr_U[isubpop, ])
      lbound <- unlist(pmax(size - mult * stderr_U[isubpop, ], 0))
      if ("postStrata" %in% names(design)) {
        ubound <- unlist(pmin(size + mult * stderr_U[isubpop, ], totalwgt))
      } else {
        ubound <- unlist(size + mult * stderr_U[isubpop, ])
      }
      temp <- paste(lev_itype[isubpop], 1:ncdfval, sep = ":")
      ind <- rownames(confval_U) %in% temp
      confval_U[ind, ] <- cbind(lbound, ubound)

      # End the subsection for all strata combined

      # Branch for an unstratified sample
    } else {

      # Calculate the standard error estimates

      if (cluster_ind) {
        temp <- cdfvar_total(
          contvar[tst], wgt2[tst], xcoord[tst], ycoord[tst],
          cdfval, stratum_ind, NULL, cluster_ind, clusterID[tst], wgt1[tst],
          xcoord1[tst], ycoord1[tst], warn_ind, warn_df, warn_vec
        )
      } else {
        temp <- cdfvar_total(contvar[tst], wgt[tst], xcoord[tst], ycoord[tst],
          cdfval, stratum_ind, NULL, cluster_ind,
          warn_ind = warn_ind,
          warn_df = warn_df, warn_vec = warn_vec
        )
      }
      warn_ind <- temp$warn_ind
      warn_df <- temp$warn_df
      if (any(temp$varest < 0)) {
        temp$vartype <- "SRS"
        warn_ind <- TRUE
        act <- "The simple random sampling variance estimator for an infinite population was used.\n"
        warn <- paste0("The local mean variance estimator produced one or more  negative variance estimates, the \nsimple random sampling variance estimator for an infinite population was used to \ncalculate variance of the CDF total estimates.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2],
          indicator = warn_vec[3], stratum = NA,
          warning = I(warn), action = I(act)
        ))
      }
      if (temp$vartype == "SRS") {
        rslt_svy <- lapply(cdfval, function(x) {
          svytotal(make.formula(paste0("I(", ivar, " <= ", x, ")")),
            design = subset(design, tst), na.rm = TRUE
          )
        })
        sdest <- sapply(rslt_svy, function(x) SE(x)[2])
      } else {
        sdest <- sqrt(temp$varest)
      }

      # Add estimates to the data frames for results

      if ("postStrata" %in% names(design)) {
        tst <- size == totalwgt
        sdest[tst] <- 0
      }
      stderr_U[isubpop, ] <- sdest
      lbound <- pmax(size - mult * sdest, 0)
      if ("postStrata" %in% names(design)) {
        ubound <- pmin(size + mult * sdest, totalwgt)
      } else {
        ubound <- size + mult * sdest
      }
      temp <- paste(lev_itype[isubpop], 1:ncdfval, sep = ":")
      ind <- rownames(confval_U) %in% temp
      confval_U[ind, ] <- cbind(lbound, ubound)
    }
  }

  # Return results

  list(
    stderr_U = stderr_U,
    confval_U = confval_U,
    warn_ind = warn_ind,
    warn_df = warn_df
  )
}

###############################################################################
# Function: summary functions (exported)
# Programmers: Michael Dumelle
# Date: January 22, 2021
#' Summarize sampling frames, design sites, and analysis data.
#'
#' @description
#' \code{summary()} summarizes sampling frames, design sites, and analysis data. The right-hand of the
#' formula specifies the variables (or factors) to
#' summarize by. If the left-hand side of the formula is empty, the
#' summary will be of the distributions of the right-hand side variables. If the left-hand side
#' of the formula contains a variable, the summary will be of the left-hand size variable
#' for each level of each right-hand side variable. Equivalent to \code{sp_summary()}; both
#' are currently maintained for backwards compatibility.
#'
#' @param object An object to summarize. When summarizing sampling frames,
#' an \code{sf}
#' object given the appropriate class using \code{sp_frame}.
#' When summarizing design sites, an object created by \code{grts()} or
#' \code{irs()} (which has class \code{sp_design}). When summarizing analysis data,
#' a data frame or an \code{sf} object given the appropriate class using \code{sp_frame}.
#'
#' @param formula A formula. One-sided formulas are used to summarize the
#' distribution of numeric or categorical variables. For one-sided formulas,
#' variable names are placed to the right of \code{~} (a right-hand side variable).
#' Two sided formulas are
#' used to summarize the distribution of a left-hand side variable
#' for each level of each right-hand side categorical variable in the formula.
#' Note that only for two-sided formulas are numeric right-hand side variables
#' coerced to a categorical variables. If an intercept
#' is included as a right-hand side variable (whether the formula is one-sided or
#' two-sided), the total will also be summarized. When summarizing sampling frames
#' or analysis data, the default formula is \code{~ 1}. When summarizing design sites,
#' \code{siteuse} should be used in the formula, and the default formula is
#' \code{~ siteuse}.
#'
#' @param onlyshow A string indicating the single level of the single right-hand side
#' variable for which a summary is requested. This argument is only used when
#' a single right-hand side variable is provided.
#'
#' @param siteuse A character vector indicating the design sites
#' for which summaries are requested in \code{object}. Defaults to computing summaries for
#' each non-\code{NULL} \code{sites_*} list in \code{object}.
#'
#' @param ... Additional arguments to pass to \code{sp_summary()}. If the left-hand
#' side of the formula is empty, the appropriate generic arguments are passed
#' to \code{summary.data.frame}. If the left-hand side of the formula is provided,
#' the appropriate generic arguments are passed to \code{summary.default}.
#'
#' @return If the left-hand side of the formula is empty, a named list
#' containing summaries of the count distribution for each right-hand side
#' variable is returned. If the left-hand side of the formula contains a
#' variable, a named list containing five number
#' summaries (numeric left-hand side) or tables (categorical or factor left
#' hand side) is returned for each right-hand side variable.
#'
#' @name summary
#' @method summary sp_frame
#'
#' @author Michael Dumelle \email{Dumelle.Michael@@epa.gov}
#'
#' @export
#' 
#'
#' @examples
#' \dontrun{
#' data("NE_Lakes")
#' summary(NE_Lakes, ELEV ~ 1)
#' summary(NE_Lakes, ~ ELEV_CAT * AREA_CAT)
#' sample <- grts(NE_Lakes, 100)
#' summary(sample, ~ ELEV_CAT * AREA_CAT)
#' }
summary.sp_frame <- function(object, formula = ~1, onlyshow = NULL, ...) {
  object <- sp_unframe(object)
  val <- sp_summary(object, formula, onlyshow, ...)
  new_val <- structure(val, class = c("summary.sp_frame", setdiff(class(val), "sp_summary.sp_frame")))
  new_val
}

#' @name summary
#' @method summary sp_design
#' @export
summary.sp_design <- function(object, formula = ~siteuse, siteuse = NULL, onlyshow = NULL, ...) {
  val <- sp_summary(object, formula, siteuse, onlyshow, ...)
  new_val <- structure(val, class = c("summary.sp_design", setdiff(class(val), "sp_summary.sp_design")))
  new_val
}

###############################################################################
# Function: cdf_nresp (not exported)
# Programmer: Tom Kincaid
# Date: June 16, 2020
#'
#' Internal Function: Calculate Number of Response Values Less Than a Set of Values
#'
#' This function calculates the number of response values less than or equal to
#' each of the set of values at which the cumulative distribution function (CDF)
#' is estimated.
#'
#' @param z Vector of the response values.
#'
#' @param val Vector of the set of values at which the CDF is estimated.
#'
#' @return Vector containing the number of response values for each CDF estimation
#'   value.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @noRd
###############################################################################

cdf_nresp <- function(z, val) {

  # Calculate the number of response values for each CDF estimation value

  z <- z[!is.na(z)]
  m <- length(val)
  nresp <- numeric(m)
  for (i in 1:m) {
    nresp[i] <- sum(ifelse(z <= val[i], 1, 0))
  }

  # Return the number of response values for each CDF estimation value

  nresp
}

###############################################################################
# Function: errorprnt (exported)
# Programmer: Tom Kincaid
# Date: June 24, 2020
#
#' Print errors from analysis functions
#'
#' This function prints the error messages vector in the analysis functions.
#'
#' @param error_vec Data frame that contains error messages.  The default is
#'   \code{"error_vec"}, which is the name given to the error messages vector created
#'   by functions in the spsurvey package.
#'
#' @return Printed errors.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @export
###############################################################################

errorprnt <- function(error_vec = get("error_vec", envir = .GlobalEnv)) {
  m <- 1:length(error_vec)
  for (i in m) {
    message(paste0("Error Message ", i, ":\n"))
    message(paste(error_vec[i], "\n"))
  }

  invisible(NULL)
}

################################################################################
# Function: cdf_plot (exported)
# Programmer Tom Kincaid
# Date: May 5, 2021
# Revised: August 31, 2021 to improve documentation
#'
#' Plot a cumulative distribution function (CDF)
#'
#' This function creates a CDF plot.  Input data for the plots is provided by a
#' data frame with the same structure as the "CDF" output from  \code{cont_analysis}.
#' Confidence limits for the CDF also are plotted.
#'
#' @param cdfest Data frame with the same structure as the "CDF" output from
#'   \code{cont_analysis}.
#'
#' @param var If \code{cdfest} has multiple variables in the "Indicator" column,
#'   then \code{var} is the single variable to be plotted. The default is
#'   \code{NULL}, which assumes that only one variable is in the "Indicator"
#'   column of \code{cdfest}.
#'
#' @param subpop If \code{cdfest} has multiple variables in the "Type" column,
#'   then \code{subpop} is the single variable to be plotted. The default is
#'   \code{NULL}, which assumes that only one variable is in the "Type"
#'   column of \code{cdfest}.
#'
#' @param subpop_level If \code{cdfest} has multiple levels of \code{subpop}
#'   in the "Subpopulation" column,
#'   then \code{subpop_level} is the single level to be plotted. The default is
#'   \code{NULL}, which assumes that only one level is in the "Subpopulation"
#'   column of \code{cdfest}.
#'
#' @param units_cdf Indicator for the label utilized for the left side y-axis
#'   and the values used for the left side y-axis tick marks, where "Percent"
#'   means the label and values are in terms of percent of the population, and
#'   "Units" means the label and values are in terms of units (count, length,
#'   or area) of the population.  The default is "Percent".
#'
#' @param type_cdf Character string consisting of the value "Continuous" or
#'   "Ordinal" that controls the type of CDF plot.  The default is "Continuous".
#'
#' @param log Character string consisting of the value "" or "x" that
#'   controls whether the x axis uses the original scale ("") or the base 10
#'   logarithmic scale ("x").  The default is "".
#'
#' @param xlab Character string providing the x-axis label.  If this argument
#'   equals NULL, then the indicator name is used as the label.  The default is
#'   NULL.
#'
#' @param ylab Character string providing the left side y-axis label.  If
#'   argument units_cdf equals "Units", a value should be provided for this
#'   argument.  Otherwise, the label will be "Percent".  The default is
#'   "Percent".
#'
#' @param ylab_r Character string providing the label for the right side y-axis
#'   (and, hence, determining the values used for the right side y-axis tick
#'   marks), where NULL means a right side y-axis is not created.  If this
#'   argument equals "Same", the right side y-axis will have the same label and
#'   tick mark values as the left side y-axis.  If this argument equals a
#'   character string other than "Same", the right side y-axis label will be the
#'   value provided for argument ylab_r, and the right side y-axis tick mark
#'   values will be determined by the choice not utilized for argument
#'   units_cdf, which means that the default value of argument units_cdf (i.e.,
#'   "Percent") will result in the right side y-axis tick mark values being
#'   expressed  in terms of units of the population (i.e., count, length, or
#'   area).  The default is NULL.
#'
#' @param main Character string providing the plot title.  The default is
#'   NULL.
#'
#' @param legloc Indicator for location of the plot legend, where "BR" means
#'   bottom right, "BL" means bottom left, "TR" means top right, "TL" means
#'   top left, and NULL means no legend.  The default is NULL.
#'
#' @param confcut Numeric value that controls plotting confidence limits at
#'   the CDF extremes.  Confidence limits for CDF values (percent scale) less
#'   than confcut or greater than 100 minus confcut are not plotted.  A value of
#'   zero means confidence limits are plotted for the complete range of the CDF.
#'   The default is 0.
#'
#' @param conflev Numeric value of the confidence level used for confidence
#'   limits. The default is 95.
#'
#' @param cex.main Expansion factor for the plot title.  The default is 1.2.
#'
#' @param cex.legend Expansion factor for the legend title. The default is 1.
#'
#' @param  ... Additional arguments passed to the \code{plot.default} function
#'   (aside from those already used and \code{ylim}).
#'
#' @return A plot of a variable's CDF estimates associated confidence limits.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @seealso
#'   \describe{
#'   \item{\code{\link{cont_cdfplot}}}{for creating a PDF file containing CDF
#'     plots}
#'   \item{\code{\link{cont_cdftest}}}{for CDF hypothesis testing}
#'   }
#'
#' @keywords survey
#'
#' @examples
#' \dontrun{
#' dframe <- data.frame(
#'   siteID = paste0("Site", 1:100),
#'   wgt = runif(100, 10, 100),
#'   xcoord = runif(100),
#'   ycoord = runif(100),
#'   stratum = rep(c("Stratum1", "Stratum2"), 50),
#'   ContVar = rnorm(100, 10, 1),
#'   All_Sites = rep("All Sites", 100),
#'   Resource_Class = rep(c("Good", "Poor"), c(55, 45))
#' )
#' myvars <- c("ContVar")
#' mysubpops <- c("All_Sites", "Resource_Class")
#' mypopsize <- data.frame(
#'   Resource_Class = c("Good", "Poor"),
#'   Total = c(4000, 1500)
#' )
#' myanalysis <- cont_analysis(dframe,
#'   vars = myvars, subpops = mysubpops,
#'   siteID = "siteID", weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
#'   stratumID = "stratum", popsize = mypopsize
#' )
#' keep <- with(myanalysis$CDF, Type == "Resource_Class" &
#'   Subpopulation == "Good")
#' par(mfrow = c(2, 1))
#' cdf_plot(myanalysis$CDF[keep, ],
#'   xlab = "ContVar",
#'   ylab = "Percent of Stream Length", ylab_r = "Stream Length (km)",
#'   main = "Estimates for Resource Class: Good"
#' )
#' cdf_plot(myanalysis$CDF[keep, ],
#'   xlab = "ContVar",
#'   ylab = "Percent of Stream Length", ylab_r = "Same",
#'   main = "Estimates for Resource Class: Good"
#' )
#' }
#' @export
################################################################################

cdf_plot <- function(cdfest, var = NULL, subpop = NULL, subpop_level = NULL,
                     units_cdf = "Percent", type_cdf = "Continuous", log = "",
                     xlab = NULL, ylab = NULL, ylab_r = NULL, main = NULL, legloc = NULL,
                     confcut = 0, conflev = 95, cex.main = 1.2, cex.legend = 1, ...) {

  # Set graphical parameter values

  op <- par(mgp = c(1.7, 0.6, 0), mar = c(3, 3, 2, 4) + 0.1)

  # Subset the data if required
  if (!is.null(var)) {
    cdfest <- cdfest[cdfest$Indicator == var, ]
  }

  if (!is.null(subpop)) {
    cdfest <- cdfest[cdfest$Type == subpop, ]
  }
  if (!is.null(subpop_level)) {
    cdfest <- cdfest[cdfest$Subpopulation == subpop_level, ]
  }

  # Create the data frame of values to be plotted and set the y-axis limits
  # The data frame structure follows: column 1: x-axis values
  #                                   column 2: CDF estimates for left y-axis
  #                                   column 3: lower confidence limit values
  #                                   column 4: upper confidence limit values
  #                                   column 5: CDF estimates for right y-axis

  if (units_cdf == "Percent") {
    cdfdata <- cdfest[, c(4, 6, 9, 10, 11)]
  } else if (units_cdf == "Units") {
    cdfdata <- cdfest[, c(4, 11, 14, 15, 6)]
  } else {
    stop(paste("\nThe choice of units for the CDF must be either \"Percent\" or \"Units\". The value \nsupplied for argument units_cdf was: \"", units_cdf, "\".\n", sep = ""))
  }

  # Restrict confidence limits to lie between confcut and 100-confcut percent

  pctval <- c(confcut, 100 - confcut)
  tvalue <- cdfest[, 6] >= pctval[1] & cdfest[, 6] <= pctval[2]
  x <- interp_cdf(pctval, cdfest[, 6], cdfdata[, 1])
  ylow <- interp_cdf(pctval, cdfest[, 6], cdfdata[, 3])
  yhi <- interp_cdf(pctval, cdfest[, 6], cdfdata[, 4])

  # Set the left side y-axis limits

  if (units_cdf == "Percent") {
    ylimit <- c(0, 100)
  } else if (units_cdf == "Units") {
    ylimit <- pretty(c(min(c(cdfdata[, 2], ylow)), max(c(cdfdata[, 2], yhi))))
    ylimit <- ylimit[c(1, length(ylimit))]
  }

  # Set default value for xlab
  if (is.null(xlab)) {
    if (!is.null(var)) {
      xlab <- var
    } else if ("Indicator" %in% names(cdfest)) {
      xlab <- unique(cdfest[["Indicator"]])
    } else {
      xlab <- NULL
    }
  }

  # Set default value for ylab
  if (is.null(ylab)) {
    if (units_cdf == "Percent") {
      ylab <- "Percent"
    } else if (units_cdf == "Units") {
      ylab <- "Units"
    }
  }

  # Plot the CDF for a continuous indicator

  if (type_cdf == "Continuous") {
    plot(cdfdata[, 1], cdfdata[, 2],
      type = "l", ylim = ylimit, xlab = xlab, ylab = ylab,
      log = log, ...
    )

    # Plot confidence limits

    value <- c(x[1], cdfdata[, 1][tvalue], x[2])
    lower <- c(ylow[1], cdfdata[, 3][tvalue], ylow[2])
    upper <- c(yhi[1], cdfdata[, 4][tvalue], yhi[2])
    lines(value, lower, lty = 3, lwd = 1.5)
    lines(value, upper, lty = 3, lwd = 1.5)

    # Plot the CDF for an ordinal (count) indicator
  } else if (type_cdf == "Ordinal") {
    x <- rep(cdfdata[, 1], each = 2)[-1]
    y <- rep(cdfdata[, 2], each = 2)
    tmp <- cbind(
      matrix(c(x, x[length(x)]), ncol = 2, byrow = TRUE),
      rep(NA, nrow(cdfdata))
    )
    x <- as.vector(t(tmp))
    tmp <- cbind(matrix(y, ncol = 2, byrow = TRUE), rep(NA, nrow(cdfdata)))
    y <- as.vector(t(tmp))
    plot(x, y, type = "l", ylim = ylimit, xlab = xlab, ylab = ylab, ...)

    # Plot confidence limits

    len <- length(cdfdata[, 1][tvalue])
    if (len > 1) {
      value <- rep(cdfdata[, 1][tvalue], each = 2)[-1]
      tmp <- cbind(
        matrix(c(value, value[length(value)]), ncol = 2, byrow = TRUE),
        rep(NA, len)
      )
      value <- as.vector(t(tmp))
      len <- length(cdfdata[, 4][tvalue])
      if (len > 1) {
        lower <- rep(cdfdata[, 3][tvalue], each = 2)
        tmp <- cbind(matrix(lower, ncol = 2, byrow = TRUE), rep(NA, len))
        lower <- as.vector(t(tmp))
        upper <- rep(cdfdata[, 4][tvalue], each = 2)
        tmp <- cbind(matrix(upper, ncol = 2, byrow = TRUE), rep(NA, len))
        upper <- as.vector(t(tmp))
        lines(value, lower, lty = 3, lwd = 1.5)
        lines(value, upper, lty = 3, lwd = 1.5)
      }
    }
  } else {
    stop(paste("\nThe type of CDF must be either \"Continuous\" or \"Ordinal\". The value supplied \nfor argument type_cdf was: \"", type_cdf, "\".\n", sep = ""))
  }

  # Create the plot title

  title(main, line = 1, cex.main = cex.main)

  # Create the plot legend

  if (!is.null(legloc)) {
    rx <- range(par("usr")[1:2], cdfdata[, 1])
    ry <- range(par("usr")[3:4], cdfdata[, 2])
    if (legloc == "BR") {
      xjust <- 1
      yjust <- 0
      legx <- rx[2]
      legy <- ry[1]
    } else if (legloc == "BL") {
      xjust <- 0
      yjust <- 0
      legx <- rx[1]
      legy <- ry[1]
    } else if (legloc == "TR") {
      xjust <- 1
      yjust <- 1
      legx <- rx[2]
      legy <- ry[2]
    } else if (legloc == "TL") {
      xjust <- 0
      yjust <- 1
      legx <- rx[1]
      legy <- ry[2]
    }
    legend(
      x = legx, y = legy, xjust = xjust, yjust = yjust,
      legend = c("CDF Estimate", paste(conflev, "% Confidence Limits", sep = "")),
      lty = c(1, 3), lwd = c(1, 1.5), bty = "n", cex = cex.legend
    )
  }


  # If requested, create the right side y-axis label and the values printed at
  # the right side y-axis tick marks

  if (!is.null(ylab_r)) {
    yl.lab <- seq(par("yaxp")[1], par("yaxp")[2], len = par("yaxp")[3] + 1)
    if (ylab_r == "Same") {
      axis(side = 4, at = yl.lab, labels = yl.lab)
      mtext(ylab, side = 4, line = 2, cex = par("cex"))
    } else {
      yr.lab <- interp_axis(yl.lab, cdfdata[, 2], cdfdata[, 5])
      axis(side = 4, at = yl.lab, labels = as.character(round(yr.lab)))
      mtext(ylab_r, side = 4, line = 2, cex = par("cex"))
    }
  }

  # Reset graphical parameter values

  par(op)

  invisible(NULL)
}

context("cdf_plot")

# find system info
on_solaris <- Sys.info()[["sysname"]] == "SunOS"
if (on_solaris) {
  test_that("on solaris", {
    expect_true(on_solaris)
  })
} else {

  #################################################
  ########### cdf_plot
  #################################################
  
  test_that("cdf_plot works", {
    bmmi <- cont_analysis(
      NLA_PNW,
      vars = "BMMI",
      weight = "WEIGHT",
      siteID = "SITE_ID"
    )
    expect_error(cdf_plot(bmmi$CDF), NA)
  })
  
  test_that("cdf_plot works subpop", {
    bmmi <- cont_analysis(
      NLA_PNW,
      vars = "BMMI",
      subpop = "PHOS_COND",
      weight = "WEIGHT",
      siteID = "SITE_ID"
    )
    expect_error(cdf_plot(bmmi$CDF, subpop = "PHOS_COND", subpop_level = "Good"), NA)
  })
  
  test_that("cdf_plot works variables", {
    bmmi <- cont_analysis(
      NLA_PNW,
      vars = c("BMMI", "WEIGHT"),
      weight = "WEIGHT",
      siteID = "SITE_ID"
    )
    expect_error(cdf_plot(bmmi$CDF, var = "BMMI"), NA)
  })
  
  test_that("cdf_plot works variables subpop", {
    bmmi <- cont_analysis(
      NLA_PNW,
      vars = c("BMMI", "WEIGHT"),
      subpop = "PHOS_COND",
      weight = "WEIGHT",
      siteID = "SITE_ID"
    )
    expect_error(cdf_plot(bmmi$CDF, var = "BMMI", subpop = "PHOS_COND", subpop_level = "Good"), NA)
  })
  
}

################################################################################
# Function: relrisk_var (not exported)
# Programmer: Tom Kincaid
# Date: June 24, 2020
# Revised: April 28, 2021 to use the SRS estimator when the local mean estimator
#          fails to produce a valid estimate
# Revised: May 3 2021 to correct an error that occurs when warning messages for
#          unstratified samples are added to the warn_df data frame
# Revised: June 8 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
#
#' Variance-Covariance Estimate for the Relative Risk Estimator
#'
#' This function calculates the variance-covariance estimate for the cell and
#' marginal totals used to calculate the relative risk estimate.  Either the
#' simple random sampling (SRS) variance estimator or the local mean variance
#' estimator is calculated, which is subject to user control.  The SRS variance
#' estimator uses the independent random sample approximation to calculate joint
#' inclusion probabilities.  The function can  accomodate single-stage and
#' two-stage samples.
#'
#' @param response Vector of the categorical response variable.
#'
#' @param stressor  Vector of the categorical stressor variable.
#'
#' @param response_levels Vector of category values (levels) for the
#'   categorical response variable, where the first level is used for
#'   calculating the relative risk estimate.
#'
#' @param stressor_levels Vector of category values (levels) for the
#'   categorical stressor variable, where the first level is used for
#'   calculating the numerator of the relative risk estimate and the second
#'   level is used for calculating the denominator of the estimate.
#'
#' @param wgt Vector of the final adjusted weight (reciprocal of the sample
#'   inclusion probability) for each site, which is either the weight for a
#'   single-stage sample or the stage two weight for a two-stage sample.
#'
#' @param x Vector of x-coordinate for location for each site, which is either
#'   the x- coordinate for a single-stage sample or the stage two x-coordinate
#'   for a two-stage sample.
#'
#' @param y Vector of y-coordinate for location for each site, which is either
#'   the y- coordinate for a single-stage sample or the stage two y-coordinate
#'   for a two-stage sample.
#'
#' @param stratum_ind Logical value that indicates whether the sample is
#'   stratified, where \code{TRUE} = a stratified sample and \code{FALSE} = not
#'   a stratified sample.
#'
#' @param stratum_level The stratum level.
#'
#' @param cluster_ind Logical value that indicates whether the sample is a two-
#'   stage sample, where \code{TRUE} = a two-stage sample and \code{FALSE} = not
#'   a two-stage sample.
#'
#' @param cluster Vector of the stage one sampling unit (primary sampling unit
#'   or cluster) code for each site.
#'
#' @param wgt1 Vector of the final adjusted stage one weight for each site.
#'
#' @param x1 Vector of the stage one x-coordinate for location for each site.
#'
#' @param y1 Vector of the stage one y-coordinate for location for each site.
#'
#' @param vartype The choice of variance estimator, where \code{"Local"} = local
#'   mean estimator and \code{"SRS"} = SRS estimator.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated, where \code{TRUE} = warning messages were generated and
#'   \code{FALSE} = warning messages were not generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @param warn_vec Vector that contains names of the population type, the
#'   subpopulation, and an indicator.
#'
#' @return A list containing the following objects:
#'   \itemize{
#'     \item{\code{varest}}{vector containing variance estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

relrisk_var <- function(response, stressor, response_levels, stressor_levels,
                        wgt, x, y, stratum_ind, stratum_level, cluster_ind,
                        cluster, wgt1, x1, y1, vartype, warn_ind, warn_df,
                        warn_vec) {

  # Assign the function name

  fname <- "relrisk_var"

  # Branch to handle two-stage and single-stage samples

  if (cluster_ind) {

    # Begin the section for a two-stage sample

    # Calculate additional required values

    cluster <- factor(cluster)
    cluster_levels <- levels(cluster)
    ncluster <- length(cluster_levels)
    response_lst <- split(response, cluster)
    stressor_lst <- split(stressor, cluster)
    if (vartype == "Local") {
      x2_lst <- split(x, cluster)
      y2_lst <- split(y, cluster)
      x1_u <- as.vector(tapply(x1, cluster, unique))
      y1_u <- as.vector(tapply(y1, cluster, unique))
    }
    wgt1_lst <- split(wgt, cluster)
    wgt1_u <- as.vector(tapply(wgt1, cluster, unique))
    var_ind <- sapply(split(cluster, cluster), length) > 1

    # For each stage one sampling unit and each required contingency table cell
    # or margin,  calculate an estimate of the total of the stage two indicator
    # variables and the variance of the total

    total2est <- matrix(0, ncluster, 4)
    var2est <- matrix(0, ncluster, 16)
    for (i in 1:ncluster) {

      # Calculate the number of response values

      nresp <- length(response_lst[[i]])

      # Create indicator variables for required contingency table cells and
      # margins

      Ind1 <- (response_lst[[i]] == response_levels[1]) * (stressor_lst[[i]] ==
        stressor_levels[1])
      Ind2 <- (stressor_lst[[i]] == stressor_levels[1])
      Ind3 <- (response_lst[[i]] == response_levels[1]) * (stressor_lst[[i]] ==
        stressor_levels[2])
      Ind4 <- (stressor_lst[[i]] == stressor_levels[2])

      # Calculate the matrix of weighted indicator variables

      rm <- cbind(Ind1, Ind2, Ind3, Ind4) * wgt1_lst[[i]]

      # Calculate the total estimate for each indicator variable

      total2est[i, ] <- apply(rm, 2, sum)

      # Adjust the variance-covariance estimator for small sample size

      SRSind <- FALSE
      if (vartype == "Local" && nresp < 4) {
        warn_ind <- TRUE
        act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("There are less than four response values for stage one sampling unit \"", cluster_levels[i], "\"\nin stratum \"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the total of the indicator variables.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("There are less than four response values for stage one sampling unit \"", cluster_levels[i], "\", \nthe simple random sampling covariance estimator for an infinite population was used \nto calculate covariance of the total of the indicator variables.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        vartype <- "SRS"
        SRSind <- TRUE
      }

      # Calculate the variance/covariance estimate for the weighted indicator
      # variables

      if (var_ind[i]) {
        if (vartype == "Local") {
          weight_lst <- localmean_weight(
            x2_lst[[i]], y2_lst[[i]],
            1 / wgt1_lst[[i]]
          )
          if (is.null(weight_lst)) {
            warn_ind <- TRUE
            act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
            if (stratum_ind) {
              warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stage one \nsampling unit \"", cluster_levels[i], "\" in stratum \"", stratum_level, "\", the simple random sampling \ncovariance estimator for an infinite population was used to calculate covariance of \nthe total of the indicator variables.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = stratum_level,
                warning = I(warn), action = I(act)
              ))
            } else {
              warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stage one \nsampling unit \"", cluster_levels[i], "\", the simple random sampling covariance estimator for an \ninfinite population was used to calculate covariance of the total of the indicator variables.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            }
            var2est[i, ] <- as.vector(nresp * var(rm))
          } else {
            temp <- localmean_cov(rm, weight_lst)
            var2est[i, ] <- as.vector(temp)
            if (any(diag(temp) < 0)) {
              warn_ind <- TRUE
              act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
              if (stratum_ind) {
                warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates for \nstage one sampling unit \"", cluster_levels[i], "\" in stratum \"", stratum_level, "\", the simple random \nsampling covariance estimator for an infinite population was used to calculate \ncovariance of the total of the indicator variables.\n")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1],
                  subpop = warn_vec[2], indicator = warn_vec[3],
                  stratum = stratum_level, warning = I(warn), action = I(act)
                ))
              } else {
                warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates \nfor stage one sampling unit \"", cluster_levels[i], "\", the simple random sampling covariance \nestimator for an infinite population was used to calculate covariance of the total of \nthe indicator variables.\n")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1],
                  subpop = warn_vec[2], indicator = warn_vec[3], stratum = NA,
                  warning = I(warn), action = I(act)
                ))
              }
              var2est[i, ] <- as.vector(nresp * var(rm))
            }
          }
        } else {
          var2est[i, ] <- as.vector(nresp * var(rm))
          if (SRSind) {
            vartype <- "Local"
          }
        }
      }
    }

    # Adjust the variance-covariance estimator for small sample size

    if (vartype == "Local" && ncluster < 4) {
      warn_ind <- TRUE
      act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
      if (stratum_ind) {
        warn <- paste0("There are less than four stage one sampling units in stratum ", stratum_level, ", the simple \nrandom sampling covariance estimator for an infinite population was used to calculate \ncovariance of the relative risk estimate.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
          stratum = I(stratum_level), warning = I(warn), action = I(act)
        ))
      } else {
        warn <- paste0("There are less than four stage one sampling units, the simple random sampling covariance \nestimator for an infinite population was used to calculate covariance of the relative risk estimate.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
          stratum = NA, warning = I(warn), action = I(act)
        ))
      }
      vartype <- "SRS"
    }

    # Calculate the variance-covariance estimate

    if (vartype == "Local") {
      weight_lst <- localmean_weight(x1_u, y1_u, 1 / wgt1_u)
      if (is.null(weight_lst)) {
        warn_ind <- TRUE
        act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stratum \n\"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the relative risk estimate.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("The local mean covariance estimator cannot calculate valid estimates, the simple random \nsampling covariance estimator for an infinite population was used to calculate covariance \nof the relative risk estimate.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA,
            warning = I(warn), action = I(act)
          ))
        }
        varest <- ncluster * var(total2est * matrix(rep(wgt1_u, 4),
          nrow = ncluster
        )) + matrix(apply(var2est * matrix(rep(wgt1_u, 16),
          nrow = ncluster
        ), 2, sum), nrow = 4)
      } else {
        varest <- localmean_cov(total2est * matrix(rep(wgt1_u, 4),
          nrow = ncluster
        ), weight_lst) + matrix(apply(var2est *
          matrix(rep(wgt1_u, 16), nrow = ncluster), 2, sum), nrow = 4)
        temp <- diag(varest)
        if (any(temp < 0)) {
          warn_ind <- TRUE
          act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
          if (stratum_ind) {
            warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates for \nstratum \"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the relative risk estimate.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = stratum_level,
              warning = I(warn), action = I(act)
            ))
          } else {
            warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates, \nthe simple random sampling covariance estimator for an infinite population was used to \ncalculate covariance of the relative risk estimate.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            ))
          }
          varest <- (ncluster * var(total2est * matrix(rep(wgt1_u, 4),
            nrow = ncluster
          )) + matrix(apply(var2est * matrix(rep(wgt1_u, 16),
            nrow = ncluster
          ), 2, sum), nrow = 4))
        }
      }
    } else {
      varest <- ncluster * var(total2est * matrix(rep(wgt1_u, 4),
        nrow = ncluster
      )) + matrix(apply(var2est * matrix(rep(wgt1_u, 16),
        nrow = ncluster
      ), 2, sum), nrow = 4)
    }

    # End of section for a two-stage sample
  } else {

    # Begin the section for a single-stage sample

    # Calculate the number of response values

    nresp <- length(response)

    # Create indicator variables for required cells and margins

    Ind1 <- (response == response_levels[1]) * (stressor == stressor_levels[1])
    Ind2 <- (stressor == stressor_levels[1])
    Ind3 <- (response == response_levels[1]) * (stressor == stressor_levels[2])
    Ind4 <- (stressor == stressor_levels[2])

    # Calculate the matrix of weighted indicator variables

    rm <- cbind(Ind1, Ind2, Ind3, Ind4) * wgt

    # Adjust the variance-covariance estimator for small sample size

    if (vartype == "Local" && nresp < 4) {
      warn_ind <- TRUE
      act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
      if (stratum_ind) {
        warn <- paste0("There are less than four response values in stratum ", stratum_level, ", the simple random \nsampling covariance estimator for an infinite population was used to calculate covariance \nof the relative risk estimate.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2],
          indicator = warn_vec[3], stratum = I(stratum_level), warning = I(warn),
          action = I(act)
        ))
      } else {
        warn <- paste0("\nThere are less than four response values, the simple random sampling covariance estimator \nfor an infinite population was used to calculate covariance of the relative risk estimate.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2],
          indicator = warn_vec[3], stratum = NA, warning = I(warn),
          action = I(act)
        ))
      }
      vartype <- "SRS"
    }

    # Calculate the variance-covariance estimate for the cell and marginal
    # totals

    if (vartype == "Local") {
      weight_lst <- localmean_weight(x = x, y = y, prb = 1 / wgt)
      if (is.null(weight_lst)) {
        warn_ind <- TRUE
        act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stratum \n\"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the relative risk estimate.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("The local mean covariance estimator cannot calculate valid estimates, the simple random \nsampling covariance estimator for an infinite population was used to calculate covariance \nof the relative risk estimate.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        varest <- nresp * var(rm)
      } else {
        varest <- localmean_cov(rm, weight_lst)
        temp <- diag(varest)
        if (any(temp < 0)) {
          warn_ind <- TRUE
          act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
          if (stratum_ind) {
            warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates for \nstratum \"", stratum_level, "\", the simple random sampling covariance estimator for an \ninfinite population was used to calculate covariance of the relative risk estimate.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1],
              subpop = warn_vec[2], indicator = warn_vec[3],
              stratum = stratum_level, warning = I(warn), action = I(act)
            ))
          } else {
            warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates, \nthe simple random sampling covariance estimator for an infinite population was used to \ncalculate covariance of the relative risk estimate.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1],
              subpop = warn_vec[2], indicator = warn_vec[3], stratum = NA,
              warning = I(warn), action = I(act)
            ))
          }
          varest <- nresp * var(rm)
        }
      }
    } else {
      varest <- nresp * var(rm)
    }

    # End of section for a single-stage sample
  }

  # Return the variance-covariance estimate, the warning message indicator, and
  # the warn_df data frame

  list(varest = varest, warn_ind = warn_ind, warn_df = warn_df)
}

################################################################################
# Function: change_est (not exported)
# Programmer: Tom Kincaid
# Date: July 29, 2020
# Revised: April 19, 2021 to ensure that repeated visit sites are correctly
#          determined for a subpopulation
# Revised: April 27, 2021 to ensure that the warnings indicator (warn_ind) and
#          the warnings data frame (warn_df) are reassigned after function calls
# Revised: June 11, 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
# Revised: June 14, 2021 to use the new function named mean_est for calculation
#          of mean estimates
# Revised: July 23, 2021 to terminate analysis of response variables that
#          contain only missing values in one of the surveys
# Revised: September 9, 2021 to remove argument vartype from the call list for
#          function percentile_est
# Revised: October 12, 2021 to correct an error that occurs assigning results
#          to the output object when calculating estimates using the median
#          for the case where there are no repeated visit sites
#
#' Estimate Change between Two Surveys
#'
#' This function estimates change between two probability surveys.  The function
#' can accommodate both categorical and continuous response variables. For a
#' categorical response variable, change is estimated by the difference in
#' category estimates for the two surveys, where a category estimate is the
#' estimated proportion of values in a category.  Note that a separate change
#' estimate is calculated for each category of a categorical response variable.
#' For a continuous response variable, change can be estimated for the mean, the
#' median, or for both the mean and median. For a continuous response variable
#' using the mean, change is estimated by the difference in estimated mean
#' values for the two surveys.  For change estimates using the median, the first
#' step is to calculate an estimate of the median for the first survey. The
#' estimated median from the first survey is then used to define two categories:
#' (1) values that are less than or equal to the estimated median and (2) values
#' that are greater than the estimated median.  Once the categories are defined,
#' change analysis for the median is identical to change analysis for a
#' categorical variable, i.e., change is estimated by the difference in category
#' estimates for the two surveys. In addition to change estimates, the standard
#' error of the change estimates and confidence bounds are calculated.  Variance
#' estimates are calculated using either the local mean variance estimator or
#' the simple random sampling (SRS) variance estimator.  The choice of variance
#' estimator is subject to user control. The local mean variance estimator
#' requires the x-coordinate and y-coordinate of each site.  The SRS variance
#' estimator uses the independent random sample approximation to calculate joint
#' inclusion probabilities.  Confidence bounds are calculated using a Normal
#' distribution multiplier.  The function can accommodate a stratified sample.
#' For a stratified sample, separate estimates and standard errors are
#' calculated for each stratum, which are used to produce estimates and standard
#' errors for all strata combined. Strata that contain a single value are
#' removed.  For a stratified sample, when either the size of the resource or
#' the sum of the size-weights of the resource is provided for each stratum,
#' those values are used as stratum weights for calculating the estimates and
#' standard errors for all strata combined.  For a stratified sample when
#' neither the size of the resource nor the sum of the size-weights of the
#' resource is provided for each stratum, estimated values are used as stratum
#' weights for calculating the estimates and standard errors for all strata
#' combined.  The function can accommodate single-stage and two-stage samples
#' for both stratified and unstratified sampling designs.  It is assumed that
#' both surveys employ the same type of survey design.  Finite population and
#' continuous population correction factors can be utilized in variance
#' estimation.  The function checks for compatibility of input values and
#' removes missing values.
#'
#' @param resp_ind A character value that indicates the type of response
#'   variable, where \code{"cat"} indicates a categorical variable and
#'   \code{"cont"} indicates a continuous variable.
#'
#' @param survey_names Character vector of length two that provides the survey
#'   names contained in the survey ID variable in the \code{dframe} data frame.
#'   The two values in the vector identify the first survey and second survey,
#'   respectively.
#'
#' @param changesum List containing estimates, which is composed of three data
#'   frames: \code{catsum}, \code{contsum_mean}, and \code{contsum_median}.
#'
#' @param dframe Data frame containing survey design variables, response
#'   variables, and subpopulation (domain) variables for both surveys.
#'
#' @param survey_1 Logical vector that identifies survey one sites in the
#'   \code{dframe} data frame.
#'
#' @param survey_2 Logical vector that identifies survey two sites in the
#'   \code{dframe} data frame.
#'
#' @param itype Character value that identifies the factor variable containing
#'   subpopulation (domain) values.
#'
#' @param isubpop Character value that identifies a level of the subpopulation
#'   variable.
#'
#' @param ivar Character value that identifies the response variable.
#'
#' @param lev_ivar Character vector that provides levels of a categorical
#'   response variable.
#'
#' @param nlev_ivar Numeric value that provides the number of levels of a
#'   categorical response variable.
#'
#' @param design_1 Object of class \code{survey.design} that specifies the
#'   complex survey design for survey one.
#'
#' @param design_2 Object of class \code{survey.design} that specifies the
#'   complex survey design for survey two
#'
#' @param design_names Character vector that provides names of survey design
#'   variables in the \code{design} argument.
#'
#' @param repeat_1 Logical vector that identifies repeat visit sites for survey
#'   one.
#'
#' @param repeat_2 Logical vector that identifies repeat visit sites for survey
#'   two.
#'
#' @param siteID Character value providing name of the site ID variable in the
#'   \code{dframe} data frame.
#'
#' @param revisitwgt Logical value that indicates whether each repeat visit site
#'   has the same survey design weight in the two surveys, where \code{TRUE} =
#'   the weight for each repeat visit site is the same and \code{FALSE} = the
#'   weight for each repeat visit site is not the same.  When this argument is
#'   \code{FALSE}, all of the repeat visit sites are assigned equal weights when
#'   calculating the covariance component of the change estimate standard error.
#'   The default is \code{FALSE}.
#'
#' @param test Character string or character vector providing the location
#'   measure(s) to use for change estimation for continuous variables.  The
#'   choices are \code{"mean"}, \code{"median"}, or \code{c("mean", "median")}.
#'
#' @param var_nondetect Character value that identifies the name of a logical
#'   variable in the \code{dframe} data frame specifying the presence of not
#'   detected (nondetect) values for a continuous response variable.
#'
#' @param vartype The choice of variance estimator, where \code{"Local"} = local
#'   mean estimator and \code{"SRS"} = SRS estimator.
#'
#' @param conf Numeric value for the confidence level.
#'
#' @param mult Numeric value that provides the Normal distribution confidence
#'   bound multiplier.
#'
#' @param warn_ind  Logical value that indicates whether warning messages were
#'   generated, where \code{TRUE} = warning messages were generated and
#'   \code{FALSE} = warning messages were not generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @param warn_vec Vector that contains names of the population type, the
#'   subpopulation, and an indicator.
#'
#' @return A list composed of the following objects:
#'   \itemize{
#'     \item{\code{results}}{if resp_ind equals \code{"cat"}, a data frame named
#'       catsum containing change estimates for categories; if \code{resp_ind}
#'       equals \code{"cont"}, two data frames named contsum_mean and
#'       contsum_median containing mean and median change estimates,
#'       respectively, as specified by the argument named test.}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

change_est <- function(resp_ind, survey_names, changesum, dframe, survey_1,
                       survey_2, itype, isubpop, ivar, lev_ivar, nlev_ivar,
                       design_1, design_2, design_names, repeat_1, repeat_2,
                       siteID, revisitwgt, test, var_nondetect, vartype, conf,
                       mult, warn_ind, warn_df, warn_vec) {

  # Assign a value to the function name variable

  fname <- "change_est"

  # Create a subset of the dframe data frame for each survey

  dframe_1 <- subset(dframe, survey_1)
  dframe_2 <- subset(dframe, survey_2)

  # Create logical vectors for subsetting the data

  subpop_1 <- dframe_1[, itype] %in% isubpop
  subpop_2 <- dframe_2[, itype] %in% isubpop

  # Subset the dframe_1 and dframe_2 objects

  dframe_1 <- subset(dframe_1, subpop_1)
  dframe_2 <- subset(dframe_2, subpop_2)

  # Determine whether the response variable contains only missing values for
  # one of the surveys

  if (all(is.na(dframe_1[, ivar])) | all(is.na(dframe_2[, ivar]))) {
    warn_ind <- TRUE
    warn <- paste0("For subpopulation \"", isubpop, "\" of population type \"", itype, "\", indicator \"", ivar, "\" \ncontains only missing values for one of the surveys.\n")
    act <- "A change estimate was not calculated.\n"
    warn_df <- rbind(warn_df, data.frame(
      func = I(fname), subpoptype = I(itype), subpop = I(isubpop),
      indicator = I(ivar), stratum = NA, warning = I(warn), action = I(act)
    ))
  } else {

    # Subset the design_1 and design_2 objects

    design_1 <- subset(design_1, subpop_1)
    design_2 <- subset(design_2, subpop_2)

    # If the surveys include repeat visit sites, ensure that the subpopulation
    # inludes the same number of repeat visit sites in each survey

    repeat_1 <- repeat_1[subpop_1]
    repeat_2 <- repeat_2[subpop_2]
    if (sum(repeat_1) != sum(repeat_2)) {
      ind_1 <- repeat_1 & !(dframe_1[, siteID] %in% dframe_2[, siteID])
      ind_2 <- repeat_2 & !(dframe_2[, siteID] %in% dframe_1[, siteID])
      if (sum(ind_1) > 0) {
        repeat_1[ind_1] <- FALSE
        warn_ind <- TRUE
        temp_str <- vecprint(dframe_1[ind_1, siteID])
        warn <- paste("The following repeat visit site IDs for subpopulation ", isubpop, "\nof population type ", itype, " for indicator ", ivar, "\nin survey one did not have analogous site IDs present in survey two:\n", temp_str, sep = "")
        act <- "The listed repeated visit sites were not used for covariance estimation.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = I(itype),
          subpop = I(isubpop), indicator = I(ivar), stratum = NA, warning = I(warn),
          action = I(act)
        ))
      }
      if (sum(ind_2) > 0) {
        repeat_2[ind_2] <- FALSE
        warn_ind <- TRUE
        temp_str <- vecprint(dframe_2[ind_2, siteID])
        warn <- paste("The following repeated visit site IDs for subpopulation ", isubpop, "\nof population type ", itype, " for indicator ", ivar, "\nin survey two did not have analogous site IDs present in survey one:\n", temp_str, sep = "")
        act <- "The listed repeated visit sites were not used for covariance estimation.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = I(itype), subpop = I(isubpop),
          indicator = I(ivar), stratum = NA, warning = I(warn), action = I(act)
        ))
      }
    }

    #
    # Begin the section for a categorical variable
    #

    if (resp_ind == "cat") {

      # Calculate estimates for all sites from survey one

      dframe_1[, itype] <- droplevels(dframe_1[, itype])
      temp <- category_est(
        NULL, dframe_1, itype, isubpop, 1, ivar, lev_ivar,
        nlev_ivar, design_1, design_names, vartype, conf, mult, warn_ind,
        warn_df
      )
      temp_1 <- droplevels(subset(temp$catsum, Category != "Total"))
      warn_ind <- temp$warn_ind
      warn_df <- temp$warn_df
      tw_1 <- sum(weights(design_1))

      # Calculate estimates for all sites from survey two

      dframe_2[, itype] <- droplevels(dframe_2[, itype])
      temp <- category_est(
        NULL, dframe_2, itype, isubpop, 1, ivar, lev_ivar,
        nlev_ivar, design_2, design_names, vartype, conf, mult, warn_ind,
        warn_df
      )
      temp_2 <- droplevels(subset(temp$catsum, Category != "Total"))
      warn_ind <- temp$warn_ind
      warn_df <- temp$warn_df
      tw_2 <- sum(weights(design_2))

      # Merge results for the two surveys

      results <- merge(temp_1, temp_2,
        by = "Category", suffix = c("_1", "_2"),
        all = TRUE, sort = FALSE
      )

      # Calculate the change estimates

      results$DiffEst.P <- (results$Estimate.P_2 - results$Estimate.P_1) / 100
      results$DiffEst.U <- results$Estimate.U_2 - results$Estimate.U_1

      # Express the standard error estimates for the two surveys on the
      # proportion scale

      results$StdError.P_1 <- results$StdError.P_1 / 100
      results$StdError.P_2 <- results$StdError.P_2 / 100

      # Calculate standard error of the change estimates for surveys with no
      # repeat visit sites

      if (sum(repeat_1) == 0) {
        results$StdError.P <- sqrt(results$StdError.P_1^2 +
          results$StdError.P_2^2)
        results$StdError.U <- sqrt(results$StdError.U_1^2 +
          results$StdError.U_2^2)
        results$LCB.P <- 100 * pmax(
          results$DiffEst.P - mult * results$StdError.P,
          -1
        )
        results$UCB.P <- 100 * pmin(
          results$DiffEst.P + mult * results$StdError.P,
          1
        )
        results$DiffEst.P <- 100 * results$DiffEst.P
        results$MarginofError.P <- 100 * (mult * results$StdError.P)
        results$MarginofError.P_1 <- 100 * (mult * results$StdError.P_1)
        results$MarginofError.P_2 <- 100 * (mult * results$StdError.P_2)
        results$StdError.P <- 100 * results$StdError.P
        results$StdError.P_1 <- 100 * results$StdError.P_1
        results$StdError.P_2 <- 100 * results$StdError.P_2
        results$MarginofError.U <- mult * results$StdError.U
        results$MarginofError.U_1 <- mult * results$StdError.U_1
        results$MarginofError.U_2 <- mult * results$StdError.U_2
        if ("postStrata" %in% names(design_1)) {
          results$LCB.U <- pmax(
            results$DiffEst.U - mult * results$StdError.U,
            -tw_1
          )
          results$UCB.U <- pmin(
            results$DiffEst.U + mult * results$StdError.U,
            tw_2
          )
        } else {
          results$LCB.U <- results$DiffEst.U - mult * results$StdError.U
          results$UCB.U <- results$DiffEst.U + mult * results$StdError.U
        }

        # Calculate standard error of the change estimates for surveys with
        # repeat visit sites
      } else {

        # Subset the dframe_1 and dframe_2 objects to retain repeat visit sites

        dframe_1 <- subset(dframe_1, repeat_1)
        dframe_2 <- subset(dframe_2, repeat_2)

        # Subset the design_1 and design_2 objects to retain repeat visit sites

        design_1 <- subset(design_1, repeat_1)
        design_2 <- subset(design_2, repeat_2)

        # Assign values for the categorical variables

        catvar_1 <- dframe_1[, ivar]
        catvar_2 <- dframe_2[, ivar]

        # Assign values for survey design variables using the survey one design
        # object

        if ("postStrata" %in% names(design_1)) {
          tempdf <- design_1$variables[subpop_1, ][repeat_1, ]
        } else {
          tempdf <- design_1$variables
        }
        for (i in names(design_names)) {
          if (is.null(design_names[[i]])) {
            eval(parse(text = paste0(i, " <- NULL")))
          } else {
            eval(parse(text = paste0(
              i, " <- tempdf[, \"", design_names[[i]],
              "\"]"
            )))
          }
        }

        # Assign a value to the indicator variable for a two-stage sample

        cluster_ind <- !is.null(clusterID)

        # Assign values to weight variables

        if (revisitwgt) {
          if (cluster_ind) {
            wgt1 <- tempdf$wgt1
            wgt2 <- tempdf$wgt2
          } else {
            wgt <- tempdf$wgt
          }
        } else {
          if (cluster_ind) {
            wgt1 <- rep(1, length(catvar_1))
            wgt2 <- rep(1, length(catvar_1))
          } else {
            wgt <- rep(1, length(catvar_1))
          }
        }

        # Assign a logical value to the indicator variable for a stratified
        # sample

        stratum_ind <- !is.null(stratumID)

        # If the sample is stratified, convert stratum to a factor, determine
        # stratum levels, and calculate number of strata

        if (stratum_ind) {
          stratum <- factor(stratumID)
          stratum_levels <- levels(stratum)
          nstrata <- length(stratum_levels)
        }

        # Remove missing values

        indx <- 1:sum(repeat_1)
        indx[is.na(catvar_1) | is.na(catvar_2)] <- 0
        catvar_1 <- catvar_1[indx]
        catvar_2 <- catvar_2[indx]
        if (stratum_ind) {
          if (cluster_ind) {
            wgt2 <- wgt2[indx]
            xcoord <- xcoord[indx]
            ycoord <- ycoord[indx]
            stratum <- stratum[indx]
            clusterID <- clusterID[indx]
            wgt1 <- wgt1[indx]
            xcoord1 <- xcoord1[indx]
            ycoord1 <- ycoord1[indx]
          } else {
            wgt <- wgt[indx]
            xcoord <- xcoord[indx]
            ycoord <- ycoord[indx]
            stratum <- stratum[indx]
          }
        } else {
          if (cluster_ind) {
            wgt2 <- wgt2[indx]
            xcoord <- xcoord[indx]
            ycoord <- ycoord[indx]
            clusterID <- clusterID[indx]
            wgt1 <- wgt1[indx]
            xcoord1 <- xcoord1[indx]
            ycoord1 <- ycoord1[indx]
          } else {
            wgt <- wgt[indx]
            xcoord <- xcoord[indx]
            ycoord <- ycoord[indx]
          }
        }

        # For a stratified sample, remove strata that contain a single site

        if (stratum_ind) {
          ind <- FALSE
          for (i in 1:nstrata) {
            tst <- stratum == stratum_levels[i]
            if (sum(tst) == 1) {
              warn_ind <- TRUE
              warn <- paste0("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
              act <- "Stratum was not used for standard error estimation.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = NA,
                subpop = NA, indicator = NA, stratum = NA, warning = I(warn),
                action = I(act)
              ))
              dframe <- dframe[!tst, ]
              ind <- TRUE
              catvar_1 <- catvar_1[!tst]
              catvar_2 <- catvar_2[!tst]
              if (vartype == "Local") {
                xcoord <- xcoord[!tst]
                ycoord <- ycoord[!tst]
              }
              stratum <- stratum[!tst]
              if (cluster_ind) {
                clusterID <- clusterID[!tst]
                wgt1 <- wgt1[!tst]
                wgt2 <- wgt2[!tst]
                if (vartype == "Local") {
                  xcoord1 <- xcoord1[!tst]
                  ycoord1 <- ycoord1[!tst]
                }
              } else {
                wgt <- wgt[!tst]
              }
              ind <- TRUE
            }
          }
          if (ind) {
            stratum <- factor(stratum)
            stratum_levels <- levels(stratum)
            nstrata <- length(stratum_levels)
          }
        }

        # For a stratified sample, check whether the number of strata is one

        if (stratum_ind) {
          if (nstrata == 1) {
            warn_ind <- TRUE
            warn <- "Only a single stratum was available for the analysis.\n"
            act <- "An unstratified data analysis was used.\n"
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            ))
            stratum_ind <- FALSE
          }
        }

        # Assign levels of the categorical variable

        catvar_levels <- levels(as.factor(results$Category))
        nlevels <- length(catvar_levels)

        # Calculate population size values

        if (stratum_ind) {
          if (cluster_ind) {
            popsize_hat <- tapply(wgt1 * wgt2, stratum, sum)
            sum_popsize_hat <- sum(wgt1 * wgt2)
          } else {
            popsize_hat <- tapply(wgt, stratum, sum)
            sum_popsize_hat <- sum(wgt)
          }
        } else {
          if (cluster_ind) {
            popsize_hat <- sum(wgt1 * wgt2)
          } else {
            popsize_hat <- sum(wgt)
          }
        }

        # Branch to handle stratified and unstratified data

        if (stratum_ind) {

          # Begin the section for stratified data

          # Create the vectors of covariance or correlation estimates for all
          # strata combined

          rslt_P <- rep(NA, nlevels)
          rslt_U <- rep(NA, nlevels)

          # Check whether the vectors of categorical variable values for revisit
          # sites are empty or contain a single value

          if (length(catvar_1) <= 1) {
            warn_ind <- TRUE
            act <- "Covariance among the revisited sites was not included in calculation of \nthe standard error estimate.\n"
            warn <- paste("The number of nonmissing repeat visit sites was less than two in one of the \nsurveys.\n", sep = "")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname),
              subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
              stratum = NA, warning = I(warn), action = I(act)
            ))

            # Begin section for nonempty vectors of categorical variable values
            # for revisit sites
          } else {

            # Begin the loop for individual strata

            for (i in 1:nstrata) {

              # Check whether the vectors of categorical variable values for
              # revisit sites are empty or contain a single value for a stratum

              stratum_i <- stratum == stratum_levels[i]
              if ((sum(!is.na(catvar_1[stratum_i])) <= 1) |
                (sum(!is.na(catvar_2[stratum_i])) <= 1)) {
                warn_ind <- TRUE
                act <- "Due to insufficient number of sites, the stratum was not included in \ncalculation of covariance among the revisited sites.\n"
                warn <- paste("The number of nonmissing repeat visit sites in one of the surveys was less \nthan two for stratum \"", stratum_levels[i], "\".\n", sep = "")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname),
                  subpoptype = warn_vec[1], subpop = warn_vec[2],
                  indicator = warn_vec[3], stratum = I(stratum_levels[i]),
                  warning = I(warn), action = I(act)
                ))

                # Begin section for nonempty vectors of categorical variable
                # values for revisit sites for a stratum
              } else {

                # Calculate proportion estimates

                z1 <- factor(catvar_1[stratum_i], levels = catvar_levels)
                z2 <- factor(catvar_2[stratum_i], levels = catvar_levels)
                m <- length(catvar_levels)
                prop1 <- as.vector(svymean(make.formula(ivar),
                  design = subset(design_1, stratum_i), na.rm = TRUE
                ))
                prop2 <- as.vector(svymean(make.formula(ivar),
                  design = subset(design_2, stratum_i), na.rm = TRUE
                ))

                # Calculate covariance or correlation estimates

                if (cluster_ind) {
                  temp <- changevar_prop(
                    catvar_levels, z1, z2, wgt2[stratum_i],
                    xcoord[stratum_i], ycoord[stratum_i], revisitwgt, prop1,
                    prop2, stratum_ind, stratum_levels[i], cluster_ind,
                    clusterID[stratum_i], wgt1[stratum_i], xcoord1[stratum_i],
                    ycoord1[stratum_i], vartype, warn_ind, warn_df, warn_vec
                  )
                } else {
                  temp <- changevar_prop(catvar_levels, z1, z2, wgt[stratum_i],
                    xcoord[stratum_i], ycoord[stratum_i], revisitwgt, prop1,
                    prop2, stratum_ind, stratum_levels[i], cluster_ind,
                    vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                    warn_vec = warn_vec
                  )
                }
                correst <- temp$rslt
                warn_ind <- temp$warn_ind
                warn_df <- temp$warn_df

                # Add estimates to the vector for all strata combined

                rslt_P[!is.na(correst)] <- rslt_P[!is.na(correst)] +
                  (popsize_hat[i] / sum_popsize_hat) * correst[!is.na(correst)]

                # Estimate the size of each category

                size1 <- popsize_hat[i] * prop1
                size2 <- popsize_hat[i] * prop2

                # Calculate covariance or correlation estimates

                if (cluster_ind) {
                  temp <- changevar_total(
                    catvar_levels, z1, z2, wgt2[stratum_i],
                    xcoord[stratum_i], ycoord[stratum_i], revisitwgt, size1,
                    size2, stratum_ind, stratum_levels[i], cluster_ind,
                    clusterID[stratum_i], wgt1[stratum_i], xcoord1[stratum_i],
                    ycoord1[stratum_i], vartype, warn_ind, warn_df, warn_vec
                  )
                } else {
                  temp <- changevar_total(catvar_levels, z1, z2, wgt[stratum_i],
                    xcoord[stratum_i], ycoord[stratum_i], revisitwgt, size1,
                    size2, stratum_ind, stratum_levels[i], cluster_ind,
                    vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                    warn_vec = warn_vec
                  )
                }
                correst <- temp$rslt
                warn_ind <- temp$warn_ind
                warn_df <- temp$warn_df

                # Add estimates to the vector for all strata combined

                rslt_U[!is.na(correst)] <- rslt_U[!is.na(correst)] +
                  correst[!is.na(correst)]

                # End the section for nonempty vectors of categorical variable
                # values for revisit sites for a stratum
              }

              # End the loop for individual strata
            }

            # End the section for nonempty vectors of categorical variable values
            # for revisit sites
          }

          # End the section for stratified data
        } else {

          # Begin the section for unstratified data

          # Check whether the vectors of categorical variable values for revisit
          # sites are empty or contain a single value

          if (length(catvar_1) <= 1) {
            rslt_P <- rep(NA, nlevels)
            rslt_U <- rep(NA, nlevels)
            warn_ind <- TRUE
            act <- "Covariance among the revisited sites was not included in calculation of \nthe standard error estimate.\n"
            warn <- paste("The number of nonmissing repeat visit sites was less than two in one of the \nsurveys.\n", sep = "")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname),
              subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
              stratum = NA, warning = I(warn), action = I(act)
            ))

            # Begin section for nonempty vectors of categorical variable values
            # for revisit sites
          } else {

            # Calculate proportion estimates

            z1 <- factor(catvar_1, levels = catvar_levels)
            z2 <- factor(catvar_2, levels = catvar_levels)
            prop1 <- as.vector(svymean(make.formula(ivar),
              design = design_1, na.rm = TRUE
            ))
            prop2 <- as.vector(svymean(make.formula(ivar),
              design = design_2, na.rm = TRUE
            ))

            # Calculate covariance or correlation estimates

            if (cluster_ind) {
              temp <- changevar_prop(
                catvar_levels, z1, z2, wgt2, xcoord, ycoord,
                revisitwgt, prop1, prop2, stratum_ind, NULL, cluster_ind,
                clusterID, wgt1, xcoord1, ycoord1, vartype, warn_ind, warn_df,
                warn_vec
              )
            } else {
              temp <- changevar_prop(catvar_levels, z1, z2, wgt, xcoord, ycoord,
                revisitwgt, prop1, prop2, stratum_ind, NULL, cluster_ind,
                vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                warn_vec = warn_vec
              )
            }
            rslt_P <- temp$rslt
            warn_ind <- temp$warn_ind
            warn_df <- temp$warn_df

            # Estimate the size of each category
            #
            size1 <- popsize_hat * prop1
            size2 <- popsize_hat * prop2

            # Calculate covariance or correlation estimates

            if (cluster_ind) {
              temp <- changevar_total(
                catvar_levels, z1, z2, wgt2, xcoord, ycoord,
                revisitwgt, size1, size2, stratum_ind, NULL, cluster_ind,
                clusterID, wgt1, xcoord1, ycoord1, vartype, warn_ind, warn_df,
                warn_vec
              )
            } else {
              temp <- changevar_total(catvar_levels, z1, z2, wgt, xcoord, ycoord,
                revisitwgt, size1, size2, stratum_ind, NULL, cluster_ind,
                vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                warn_vec = warn_vec
              )
            }
            rslt_U <- temp$rslt
            warn_ind <- temp$warn_ind
            warn_df <- temp$warn_df

            # End the section for nonempty vectors of categorical variable values
            # for revisit sites
          }

          # End the section for unstratified data
        }

        # Calculate standard errors

        results$StdError.P <- rep(NA, nlevels)
        results$StdError.U <- rep(NA, nlevels)
        ind <- is.na(rslt_P)
        results$StdError.P[ind] <- sqrt(results$StdError.P_1[ind]^2 +
          results$StdError.P_2[ind]^2)
        results$StdError.U[ind] <- sqrt(results$StdError.U_1[ind]^2 +
          results$StdError.U_2[ind]^2)
        if (any(!ind)) {
          tw_1r <- sum(weights(design_1))
          tw_2r <- sum(weights(design_2))
          if (revisitwgt) {
            temp <- results$StdError.P_1^2 + results$StdError.P_2^2 -
              ((2 * tw_1r * tw_2r) / (tw_1 * tw_2)) * rslt_P
            ind <- !is.na(rslt_P) & temp <= 0
            results$StdError.P[ind] <- sqrt(results$StdError.P_1[ind]^2 +
              results$StdError.P_2[ind]^2)
            ind <- !is.na(rslt_P) & temp > 0
            results$StdError.P[ind] <- sqrt(temp[ind])
            temp <- results$StdError.U_1^2 + results$StdError.U_2^2 - 2 * rslt_U
            ind <- !is.na(rslt_U) & temp <= 0
            results$StdError.U[ind] <- sqrt(results$StdError.U_1[ind]^2 +
              results$StdError.U_2[ind]^2)
            ind <- !is.na(rslt_U) & temp > 0
            results$StdError.U[ind] <- sqrt(temp[ind])
          } else {
            se_1_p <- rep(NA, nlevels)
            se_1_u <- rep(NA, nlevels)
            temp <- category_est(
              NULL, dframe_1, itype, isubpop, 1, ivar, lev_ivar, nlev_ivar,
              design_1, design_names, vartype, conf, mult, warn_ind, warn_df
            )
            warn_ind <- temp$warn_ind
            warn_df <- temp$warn_df
            temp$results <- droplevels(subset(temp$catsum, Category != "Total"))
            ind <- match(temp$results$Category, catvar_levels, nomatch = 0)
            se_1_p[ind] <- temp$results$StdError.P / 100
            se_1_u[ind] <- temp$results$StdError.U
            se_2_p <- rep(NA, nlevels)
            se_2_u <- rep(NA, nlevels)
            temp <- category_est(
              NULL, dframe_2, itype, isubpop, 1, ivar, lev_ivar, nlev_ivar,
              design_2, design_names, vartype, conf, mult, warn_ind, warn_df
            )
            warn_ind <- temp$warn_ind
            warn_df <- temp$warn_df
            temp$results <- droplevels(subset(temp$catsum, Category != "Total"))
            ind <- match(temp$results$Category, catvar_levels, nomatch = 0)
            se_2_p[ind] <- temp$results$StdError.P / 100
            se_2_u[ind] <- temp$results$StdError.U
            covest <- rslt_P * se_1_p * se_2_p
            temp <- results$StdError.P_1^2 + results$StdError.P_2^2 -
              ((2 * tw_1r * tw_2r) / (tw_1 * tw_2)) * covest
            ind <- !is.na(rslt_P) & temp <= 0
            results$StdError.P[ind] <- sqrt(results$StdError.P_1[ind]^2 +
              results$StdError.P_2[ind]^2)
            ind <- !is.na(rslt_P) & temp > 0
            results$StdError.P[ind] <- sqrt(temp[ind])
            covest <- rslt_U * se_1_u * se_2_u
            temp <- results$StdError.U_1^2 + results$StdError.U_2^2 - 2 * covest
            ind <- !is.na(rslt_U) & temp <= 0
            results$StdError.U[ind] <- sqrt(results$StdError.U_1[ind]^2 +
              results$StdError.U_2[ind]^2)
            ind <- !is.na(rslt_U) & temp > 0
            results$StdError.U[ind] <- sqrt(temp[ind])
          }
        }

        # Calculate margins of error and confidence bounds

        results$LCB.P <- 100 * pmax(results$DiffEst.P -
          mult * results$StdError.P, -1)
        results$UCB.P <- 100 * pmin(results$DiffEst.P +
          mult * results$StdError.P, 1)
        results$DiffEst.P <- 100 * results$DiffEst.P
        results$MarginofError.P <- 100 * (mult * results$StdError.P)
        results$MarginofError.P_1 <- 100 * (mult * results$StdError.P_1)
        results$MarginofError.P_2 <- 100 * (mult * results$StdError.P_2)
        results$StdError.P <- 100 * results$StdError.P
        results$StdError.P_1 <- 100 * results$StdError.P_1
        results$StdError.P_2 <- 100 * results$StdError.P_2
        results$MarginofError.U <- mult * results$StdError.U
        results$MarginofError.U_1 <- mult * results$StdError.U_1
        results$MarginofError.U_2 <- mult * results$StdError.U_2
        if ("postStrata" %in% names(design_1)) {
          results$LCB.U <- pmax(
            results$DiffEst.U - mult * results$StdError.U, -tw_1
          )
          results$UCB.U <- pmin(
            results$DiffEst.U + mult * results$StdError.U, tw_2
          )
        } else {
          results$LCB.U <- results$DiffEst.U - mult * results$StdError.U
          results$UCB.U <- results$DiffEst.U + mult * results$StdError.U
        }

        # End the section for surveys with repeat visit sites
      }

      # Add estimates to the catsum data frame

      changesum$catsum <- rbind(changesum$catsum, cbind(
        Survey_1 = survey_names[1],
        Survey_2 = survey_names[2],
        subset(results, select = c(
          2:4, 1, 30, 32, 36, 34:35, 31, 33, 37:39, 5:15,
          19:29
        ))
      ))

      #
      # End the section for a categorical variable
      #
    } else if (resp_ind == "cont") {

      #
      # Begin the section for a continuous variable
      #

      #
      # Begin the section for a continuous variable using the mean
      #

      if ("mean" %in% test) {

        # As necessary, store original values for the dframe_1 and dframe_2 data
        # frames and the design_1 and design_2 objects

        if (any(c("total", "median") %in% test)) {
          dframe_1_org <- dframe_1
          dframe_2_org <- dframe_2
          design_1_org <- design_1
          design_2_org <- design_2
        }

        # Calculate estimate for all sites from survey one
        temp <- mean_est(
          NULL, dframe_1, itype, isubpop, 1, ivar,
          design_1, design_names, var_nondetect, vartype, conf, mult, warn_ind,
          warn_df
        )
        meanest_1 <- temp$meansum
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
        tw_1 <- sum(weights(design_1))

        # Calculate estimate for all sites from survey two

        temp <- mean_est(
          NULL, dframe_2, itype, isubpop, 1, ivar,
          design_2, design_names, var_nondetect, vartype, conf, mult, warn_ind,
          warn_df
        )
        meanest_2 <- temp$meansum
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
        tw_2 <- sum(weights(design_2))

        # Merge results for the two surveys

        results <- merge(meanest_1, meanest_2,
          suffix = c("_1", "_2"),
          by = c("Type", "Subpopulation", "Indicator")
        )

        # Calculate the change estimate

        results$DiffEst <- results$Estimate_2 - results$Estimate_1

        # Calculate the confidence bound multiplier

        mult <- qnorm(0.5 + (conf / 100) / 2)

        #
        # Calculate standard error of the change estimate
        #

        # Section for surveys with no repeat visit sites

        if (sum(repeat_1) == 0) {
          results$StdError <- sqrt(results$StdError_1^2 + results$StdError_2^2)
          results$MarginofError <- mult * results$StdError
          results$LCB <- results$DiffEst - mult * results$StdError
          results$UCB <- results$DiffEst + mult * results$StdError


          # Section for surveys with repeat visit sites
        } else {

          # Subset the dframe_1 and dframe_2 objects to retain repeat visit sites

          dframe_1 <- subset(dframe_1, repeat_1)
          dframe_2 <- subset(dframe_2, repeat_2)

          # Subset the design_1 and design_2 objects to retain repeat visit sites

          design_1 <- subset(design_1, repeat_1)
          design_2 <- subset(design_2, repeat_2)

          # Assign values for the continuous variables

          contvar_1 <- dframe_1[, ivar]
          contvar_2 <- dframe_2[, ivar]

          # Assign values for survey design variables using the survey one design
          # object

          tempdf <- design_1$variables
          for (i in names(design_names)) {
            if (is.null(design_names[[i]])) {
              eval(parse(text = paste0(i, " <- NULL")))
            } else {
              eval(parse(text = paste0(
                i, " <- tempdf[, \"", design_names[[i]],
                "\"]"
              )))
            }
          }

          # Assign a value to the indicator variable for a two-stage sample

          cluster_ind <- !is.null(clusterID)

          # Assign values to weight variables

          if (revisitwgt) {
            if (cluster_ind) {
              wgt1 <- tempdf$wgt1
              wgt2 <- tempdf$wgt2
            } else {
              wgt <- tempdf$wgt
            }
          } else {
            if (cluster_ind) {
              wgt1 <- rep(1, length(contvar_1))
              wgt2 <- rep(1, length(contvar_1))
            } else {
              wgt <- rep(1, length(contvar_1))
            }
          }

          # Assign a logical value to the indicator variable for a stratified
          # sample

          stratum_ind <- !is.null(stratumID)

          # If the sample is stratified, convert stratum to a factor, determine
          # stratum levels, and calculate number of strata

          if (stratum_ind) {
            stratum <- factor(stratumID)
            stratum_levels <- levels(stratum)
            nstrata <- length(stratum_levels)
          }

          # Remove missing values

          indx <- 1:sum(repeat_1)
          indx[is.na(contvar_1) | is.na(contvar_2)] <- 0
          contvar_1 <- contvar_1[indx]
          contvar_2 <- contvar_2[indx]
          if (stratum_ind) {
            if (cluster_ind) {
              wgt2 <- wgt2[indx]
              xcoord <- xcoord[indx]
              ycoord <- ycoord[indx]
              stratum <- stratum[indx]
              clusterID <- clusterID[indx]
              wgt1 <- wgt1[indx]
              xcoord1 <- xcoord1[indx]
              ycoord1 <- ycoord1[indx]
            } else {
              wgt <- wgt[indx]
              xcoord <- xcoord[indx]
              ycoord <- ycoord[indx]
              stratum <- stratum[indx]
            }
          } else {
            if (cluster_ind) {
              wgt2 <- wgt2[indx]
              xcoord <- xcoord[indx]
              ycoord <- ycoord[indx]
              clusterID <- clusterID[indx]
              wgt1 <- wgt1[indx]
              xcoord1 <- xcoord1[indx]
              ycoord1 <- ycoord1[indx]
            } else {
              wgt <- wgt[indx]
              xcoord <- xcoord[indx]
              ycoord <- ycoord[indx]
            }
          }

          # For a stratified sample, remove strata that contain a single site

          if (stratum_ind) {
            ind <- FALSE
            for (i in 1:nstrata) {
              tst <- stratum == stratum_levels[i]
              if (sum(tst) == 1) {
                warn_ind <- TRUE
                warn <- paste0("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
                act <- "Stratum was not used for standard error estimation.\n"
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = NA,
                  subpop = NA, indicator = NA, stratum = NA, warning = I(warn),
                  action = I(act)
                ))
                dframe <- dframe[!tst, ]
                ind <- TRUE
                contvar_1 <- contvar_1[!tst]
                contvar_2 <- contvar_2[!tst]
                if (vartype == "Local") {
                  xcoord <- xcoord[!tst]
                  ycoord <- ycoord[!tst]
                }
                stratum <- stratum[!tst]
                if (cluster_ind) {
                  clusterID <- clusterID[!tst]
                  wgt1 <- wgt1[!tst]
                  wgt2 <- wgt2[!tst]
                  if (vartype == "Local") {
                    xcoord1 <- xcoord1[!tst]
                    ycoord1 <- ycoord1[!tst]
                  }
                } else {
                  wgt <- wgt[!tst]
                }
                ind <- TRUE
              }
            }
            if (ind) {
              stratum <- factor(stratum)
              stratum_levels <- levels(stratum)
              nstrata <- length(stratum_levels)
            }
          }

          # For a stratified sample, check whether the number of strata is one

          if (stratum_ind) {
            if (nstrata == 1) {
              warn_ind <- TRUE
              warn <- "Only a single stratum was available for the analysis.\n"
              act <- "An unstratified data analysis was used.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
              stratum_ind <- FALSE
            }
          }

          # Calculate population size values

          if (stratum_ind) {
            if (cluster_ind) {
              popsize_hat <- tapply(wgt1 * wgt2, stratum, sum)
              sum_popsize_hat <- sum(wgt1 * wgt2)
            } else {
              popsize_hat <- tapply(wgt, stratum, sum)
              sum_popsize_hat <- sum(wgt)
            }
          } else {
            if (cluster_ind) {
              popsize_hat <- sum(wgt1 * wgt2)
            } else {
              popsize_hat <- sum(wgt)
            }
          }

          # Branch to handle stratified and unstratified data

          if (stratum_ind) {

            # Begin the section for stratified data

            # Create the object for covariance or correlation estimates for all
            # strata combined

            rslt <- NA

            # Check whether the vectors of continuous variable values for revisit
            # sites are empty or contain a single value

            if (length(contvar_1) <= 1) {
              warn_ind <- TRUE
              act <- "Covariance among the revisited sites was not included in calculation of \nthe standard error estimate.\n"
              warn <- paste("The number of nonmissing repeat visit sites was less than two in one of the \nsurveys.\n", sep = "")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
                stratum = NA, warning = I(warn), action = I(act)
              ))

              # Begin section for nonempty vectors of continuous variable values
              # for revisit sites
            } else {

              # Begin the loop for individual strata

              for (i in 1:nstrata) {

                # Check whether the vectors of continuous variable values for
                # revisit sites are empty or contain a single value for a stratum

                stratum_i <- stratum == stratum_levels[i]
                if (length(contvar_1[stratum_i]) <= 1) {
                  warn_ind <- TRUE
                  act <- "Due to insufficient number of sites, the stratum was not included in \ncalculation of covariance among the revisited sites.\n"
                  warn <- paste("The number of nonmissing repeat visit sites  in one of the surveys was less \nthan two for stratum \"", stratum_levels[i], "\".\n", sep = "")
                  warn_df <- rbind(warn_df, data.frame(
                    func = I(fname),
                    subpoptype = warn_vec[1], subpop = warn_vec[2],
                    indicator = warn_vec[3], stratum = I(stratum_levels[i]),
                    warning = I(warn), action = I(act)
                  ))

                  # Begin section for nonempty vectors of continuous variables
                  # values for revisit sites for a stratum
                } else {

                  # Calculate mean estimates

                  stratum_i <- stratum == stratum_levels[i]
                  z1 <- contvar_1[stratum_i]
                  z2 <- contvar_2[stratum_i]
                  mean1 <- as.vector(svymean(make.formula(ivar),
                    design = subset(design_1, stratum_i), na.rm = TRUE
                  ))
                  mean2 <- as.vector(svymean(make.formula(ivar),
                    design = subset(design_2, stratum_i), na.rm = TRUE
                  ))

                  # Calculate covariance or correlation estimates

                  if (cluster_ind) {
                    temp <- changevar_mean(
                      z1, z2, wgt2[stratum_i],
                      xcoord[stratum_i], ycoord[stratum_i], revisitwgt, mean1,
                      mean2, stratum_ind, stratum_levels[i], cluster_ind,
                      clusterID[stratum_i], wgt1[stratum_i], xcoord1[stratum_i],
                      ycoord1[stratum_i], vartype, warn_ind, warn_df, warn_vec
                    )
                  } else {
                    temp <- changevar_mean(z1, z2, wgt[stratum_i],
                      xcoord[stratum_i], ycoord[stratum_i], revisitwgt, mean1,
                      mean2, stratum_ind, stratum_levels[i], cluster_ind,
                      vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                      warn_vec = warn_vec
                    )
                  }
                  correst <- temp$rslt
                  warn_ind <- temp$warn_ind
                  warn_df <- temp$warn_df

                  # Add estimates to the object for all strata combined

                  rslt[!is.na(correst)] <- rslt[!is.na(correst)] +
                    (popsize_hat[i] / sum_popsize_hat) * correst[!is.na(correst)]

                  # End the section for nonempty vectors of continuous variable
                  # values for revisit sites for a stratum
                }

                # End the loop for individual strata
              }

              # End the section for nonempty vectors of continuous variable
              # values for revisit sites
            }

            # End the section for stratified data
          } else {

            # Begin the section for unstratified data

            # Check whether the vectors of continuous variable values for revisit
            # sites are empty or contain a single value

            if (length(contvar_1) <= 1) {
              rslt <- NA
              warn_ind <- TRUE
              act <- "Covariance among the revisited sites was not included in calculation of \nthe standard error estimate.\n"
              warn <- paste("The number of nonmissing repeat visit sites was less than two in one of the \nsurveys.\n", sep = "")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))

              # Begin section for nonempty vectors of continuous variable values
              # for revisit sites
            } else {

              # Calculate mean estimates

              z1 <- contvar_1
              z2 <- contvar_2
              mean1 <- as.vector(svymean(make.formula(ivar),
                design = design_1,
                na.rm = TRUE
              ))
              mean2 <- as.vector(svymean(make.formula(ivar),
                design = design_2,
                na.rm = TRUE
              ))

              # Calculate covariance or correlation estimates

              if (cluster_ind) {
                temp <- changevar_mean(
                  z1, z2, wgt2, xcoord, ycoord, revisitwgt,
                  mean1, mean2, stratum_ind, NULL, cluster_ind, clusterID, wgt1,
                  xcoord1, ycoord1, vartype, warn_ind, warn_df, warn_vec
                )
              } else {
                temp <- changevar_mean(z1, z2, wgt, xcoord, ycoord, revisitwgt,
                  mean1, mean2, stratum_ind, NULL, cluster_ind,
                  vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                  warn_vec = warn_vec
                )
              }
              rslt <- temp$rslt
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df

              # End the section for nonempty vectors of continuous variable
              # values for revisit sites
            }

            # End the section for unstratified data
          }

          # Calculate standard errors

          if (is.na(rslt)) {
            results$StdError <- sqrt(results$StdError_1^2 + results$StdError_2^2)
          } else {
            tw_1r <- sum(weights(design_1))
            tw_2r <- sum(weights(design_2))
            if (revisitwgt) {
              temp <- results$StdError_1^2 + results$StdError_2^2 -
                ((2 * tw_1r * tw_2r) / (tw_1 * tw_2)) * rslt
              if (is.na(temp)) {
                temp <- 0
              }
              if (temp <= 0) {
                results$StdError <- sqrt(results$StdError_1^2 +
                  results$StdError_2^2)
              } else {
                results$StdError <- sqrt(temp)
              }
            } else {
              temp <- mean_est(
                NULL, dframe_1, itype, isubpop, 1, ivar,
                design_1, design_names, var_nondetect, vartype, conf, mult,
                warn_ind, warn_df
              )
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df
              se_1 <- temp$meansum$StdError
              temp <- mean_est(
                NULL, dframe_2, itype, isubpop, 1, ivar,
                design_2, design_names, var_nondetect, vartype, conf, mult,
                warn_ind, warn_df
              )
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df
              se_2 <- temp$meansum$StdError
              covest <- rslt * se_1 * se_2
              temp <- results$StdError_1^2 + results$StdError_2^2 -
                ((2 * tw_1r * tw_2r) / (tw_1 * tw_2)) * covest
              if (is.na(temp)) {
                temp <- 0
              }
              if (temp <= 0) {
                results$StdError <- sqrt(results$StdError_1^2 +
                  results$StdError_2^2)
              } else {
                results$StdError <- sqrt(temp)
              }
            }
          }

          # Calculate margins of error and confidence bounds for the change
          # estimate

          results$MarginofError <- mult * results$StdError
          results$LCB <- results$DiffEst - mult * results$StdError
          results$UCB <- results$DiffEst + mult * results$StdError

          # End the section for surveys with repeat visit sites
        }

        # Add estimates to the contsum_mean data frame

        changesum$contsum_mean <- rbind(changesum$contsum_mean, cbind(
          Survey_1 = survey_names[1],
          Survey_2 = survey_names[2],
          subset(results, select = c(1:3, 16:20, 4:9, 10:15))
        ))

        #
        # End the section for a continuous variable using the mean
        #
      }
      
      if ("total" %in% test) {
        
        # As necessary, store original values for the dframe_1 and dframe_2 data
        # frames and the design_1 and design_2 objects
        
        if (any(c("mean") %in% test)) {
          dframe_1 <- dframe_1_org
          dframe_2 <- dframe_2_org
          design_1 <- design_1_org
          design_2 <- design_2_org
        }
        
        if (any(c("median") %in% test)) {
          dframe_1_org <- dframe_1
          dframe_2_org <- dframe_2
          design_1_org <- design_1
          design_2_org <- design_2
        }
        
        # Calculate estimate for all sites from survey one
        temp <- total_est(
          NULL, dframe_1, itype, isubpop, 1, ivar,
          design_1, design_names, var_nondetect, vartype, conf, mult, warn_ind,
          warn_df
        )
        totalest_1 <- temp$totalsum
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
        tw_1 <- sum(weights(design_1))
        
        # Calculate estimate for all sites from survey two
        
        temp <- total_est(
          NULL, dframe_2, itype, isubpop, 1, ivar,
          design_2, design_names, var_nondetect, vartype, conf, mult, warn_ind,
          warn_df
        )
        totalest_2 <- temp$totalsum
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
        tw_2 <- sum(weights(design_2))
        
        # Merge results for the two surveys
        
        results <- merge(totalest_1, totalest_2,
                         suffix = c("_1", "_2"),
                         by = c("Type", "Subpopulation", "Indicator")
        )
        
        # Calculate the change estimate
        
        results$DiffEst <- results$Estimate_2 - results$Estimate_1
        
        # Calculate the confidence bound multiplier
        
        mult <- qnorm(0.5 + (conf / 100) / 2)
        
        #
        # Calculate standard error of the change estimate
        #
        
        # Section for surveys with no repeat visit sites
        
        if (sum(repeat_1) == 0) {
          results$StdError <- sqrt(results$StdError_1^2 + results$StdError_2^2)
          results$MarginofError <- mult * results$StdError
          results$LCB <- results$DiffEst - mult * results$StdError
          results$UCB <- results$DiffEst + mult * results$StdError
          
          
          # Section for surveys with repeat visit sites
        } else {
          
          # Subset the dframe_1 and dframe_2 objects to retain repeat visit sites
          
          dframe_1 <- subset(dframe_1, repeat_1)
          dframe_2 <- subset(dframe_2, repeat_2)
          
          # Subset the design_1 and design_2 objects to retain repeat visit sites
          design_1 <- subset(design_1, repeat_1)
          design_2 <- subset(design_2, repeat_2)
          
          # Assign values for the continuous variables
          
          contvar_1 <- dframe_1[, ivar]
          contvar_2 <- dframe_2[, ivar]
          
          # Assign values for survey design variables using the survey one design
          # object
          
          tempdf <- design_1$variables
          for (i in names(design_names)) {
            if (is.null(design_names[[i]])) {
              eval(parse(text = paste0(i, " <- NULL")))
            } else {
              eval(parse(text = paste0(
                i, " <- tempdf[, \"", design_names[[i]],
                "\"]"
              )))
            }
          }
          
          # Assign a value to the indicator variable for a two-stage sample
          
          cluster_ind <- !is.null(clusterID)
          
          # Assign values to weight variables
          
          if (revisitwgt) {
            if (cluster_ind) {
              wgt1 <- tempdf$wgt1
              wgt2 <- tempdf$wgt2
            } else {
              wgt <- tempdf$wgt
            }
          } else {
            if (cluster_ind) {
              wgt1 <- rep(1, length(contvar_1))
              wgt2 <- rep(1, length(contvar_1))
            } else {
              wgt <- rep(1, length(contvar_1))
            }
          }
          
          # Assign a logical value to the indicator variable for a stratified
          # sample
          
          stratum_ind <- !is.null(stratumID)
          
          # If the sample is stratified, convert stratum to a factor, determine
          # stratum levels, and calculate number of strata
          
          if (stratum_ind) {
            stratum <- factor(stratumID)
            stratum_levels <- levels(stratum)
            nstrata <- length(stratum_levels)
          }
          
          # Remove missing values
          
          indx <- 1:sum(repeat_1)
          indx[is.na(contvar_1) | is.na(contvar_2)] <- 0
          contvar_1 <- contvar_1[indx]
          contvar_2 <- contvar_2[indx]
          if (stratum_ind) {
            if (cluster_ind) {
              wgt2 <- wgt2[indx]
              xcoord <- xcoord[indx]
              ycoord <- ycoord[indx]
              stratum <- stratum[indx]
              clusterID <- clusterID[indx]
              wgt1 <- wgt1[indx]
              xcoord1 <- xcoord1[indx]
              ycoord1 <- ycoord1[indx]
            } else {
              wgt <- wgt[indx]
              xcoord <- xcoord[indx]
              ycoord <- ycoord[indx]
              stratum <- stratum[indx]
            }
          } else {
            if (cluster_ind) {
              wgt2 <- wgt2[indx]
              xcoord <- xcoord[indx]
              ycoord <- ycoord[indx]
              clusterID <- clusterID[indx]
              wgt1 <- wgt1[indx]
              xcoord1 <- xcoord1[indx]
              ycoord1 <- ycoord1[indx]
            } else {
              wgt <- wgt[indx]
              xcoord <- xcoord[indx]
              ycoord <- ycoord[indx]
            }
          }
          
          # For a stratified sample, remove strata that contain a single site
          
          if (stratum_ind) {
            ind <- FALSE
            for (i in 1:nstrata) {
              tst <- stratum == stratum_levels[i]
              if (sum(tst) == 1) {
                warn_ind <- TRUE
                warn <- paste0("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
                act <- "Stratum was not used for standard error estimation.\n"
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = NA,
                  subpop = NA, indicator = NA, stratum = NA, warning = I(warn),
                  action = I(act)
                ))
                dframe <- dframe[!tst, ]
                ind <- TRUE
                contvar_1 <- contvar_1[!tst]
                contvar_2 <- contvar_2[!tst]
                if (vartype == "Local") {
                  xcoord <- xcoord[!tst]
                  ycoord <- ycoord[!tst]
                }
                stratum <- stratum[!tst]
                if (cluster_ind) {
                  clusterID <- clusterID[!tst]
                  wgt1 <- wgt1[!tst]
                  wgt2 <- wgt2[!tst]
                  if (vartype == "Local") {
                    xcoord1 <- xcoord1[!tst]
                    ycoord1 <- ycoord1[!tst]
                  }
                } else {
                  wgt <- wgt[!tst]
                }
                ind <- TRUE
              }
            }
            if (ind) {
              stratum <- factor(stratum)
              stratum_levels <- levels(stratum)
              nstrata <- length(stratum_levels)
            }
          }
          
          # For a stratified sample, check whether the number of strata is one
          
          if (stratum_ind) {
            if (nstrata == 1) {
              warn_ind <- TRUE
              warn <- "Only a single stratum was available for the analysis.\n"
              act <- "An unstratified data analysis was used.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
              stratum_ind <- FALSE
            }
          }
          
          # Calculate population size values
          
          if (stratum_ind) {
            if (cluster_ind) {
              popsize_hat <- tapply(wgt1 * wgt2, stratum, sum)
              sum_popsize_hat <- sum(wgt1 * wgt2)
            } else {
              popsize_hat <- tapply(wgt, stratum, sum)
              sum_popsize_hat <- sum(wgt)
            }
          } else {
            if (cluster_ind) {
              popsize_hat <- sum(wgt1 * wgt2)
            } else {
              popsize_hat <- sum(wgt)
            }
          }
          
          # Branch to handle stratified and unstratified data
          
          if (stratum_ind) {
            
            # Begin the section for stratified data
            
            # Create the object for covariance or correlation estimates for all
            # strata combined
            
            rslt <- NA
            
            # Check whether the vectors of continuous variable values for revisit
            # sites are empty or contain a single value
            
            if (length(contvar_1) <= 1) {
              warn_ind <- TRUE
              act <- "Covariance among the revisited sites was not included in calculation of \nthe standard error estimate.\n"
              warn <- paste("The number of nonmissing repeat visit sites was less than two in one of the \nsurveys.\n", sep = "")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
                stratum = NA, warning = I(warn), action = I(act)
              ))
              
              # Begin section for nonempty vectors of continuous variable values
              # for revisit sites
            } else {
              
              # Begin the loop for individual strata
              
              for (i in 1:nstrata) {
                
                # Check whether the vectors of continuous variable values for
                # revisit sites are empty or contain a single value for a stratum
                
                stratum_i <- stratum == stratum_levels[i]
                if (length(contvar_1[stratum_i]) <= 1) {
                  warn_ind <- TRUE
                  act <- "Due to insufficient number of sites, the stratum was not included in \ncalculation of covariance among the revisited sites.\n"
                  warn <- paste("The number of nonmissing repeat visit sites  in one of the surveys was less \nthan two for stratum \"", stratum_levels[i], "\".\n", sep = "")
                  warn_df <- rbind(warn_df, data.frame(
                    func = I(fname),
                    subpoptype = warn_vec[1], subpop = warn_vec[2],
                    indicator = warn_vec[3], stratum = I(stratum_levels[i]),
                    warning = I(warn), action = I(act)
                  ))
                  
                  # Begin section for nonempty vectors of continuous variables
                  # values for revisit sites for a stratum
                } else {
                  
                  # Calculate mean estimates
                  
                  stratum_i <- stratum == stratum_levels[i]
                  z1 <- contvar_1[stratum_i]
                  z2 <- contvar_2[stratum_i]
                  total1 <- as.vector(svytotal(make.formula(ivar),
                                             design = subset(design_1, stratum_i), na.rm = TRUE
                  ))
                  total2 <- as.vector(svytotal(make.formula(ivar),
                                             design = subset(design_2, stratum_i), na.rm = TRUE
                  ))
                  
                  # for local neighborhood variance
                  mean1 <- as.vector(svymean(make.formula(ivar),
                                               design = subset(design_1, stratum_i), na.rm = TRUE
                  ))
                  mean2 <- as.vector(svymean(make.formula(ivar),
                                               design = subset(design_2, stratum_i), na.rm = TRUE
                  ))
                  
                  
                  # Calculate covariance or correlation estimates
                  
                  if (cluster_ind) {
                    temp <- changevar_mean(
                      z1, z2, wgt2[stratum_i],
                      xcoord[stratum_i], ycoord[stratum_i], revisitwgt, mean1,
                      mean2, stratum_ind, stratum_levels[i], cluster_ind,
                      clusterID[stratum_i], wgt1[stratum_i], xcoord1[stratum_i],
                      ycoord1[stratum_i], vartype, warn_ind, warn_df, warn_vec
                    )
                  } else {
                    temp <- changevar_mean(z1, z2, wgt[stratum_i],
                                           xcoord[stratum_i], ycoord[stratum_i], revisitwgt, mean1,
                                           mean2, stratum_ind, stratum_levels[i], cluster_ind,
                                           vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                                           warn_vec = warn_vec
                    )
                  }
                  correst <- temp$rslt
                  warn_ind <- temp$warn_ind
                  warn_df <- temp$warn_df
                  
                  # Add estimates to the object for all strata combined
                  
                  rslt[!is.na(correst)] <- rslt[!is.na(correst)] +
                    (popsize_hat[i] / sum_popsize_hat) * correst[!is.na(correst)]
                  
                  # End the section for nonempty vectors of continuous variable
                  # values for revisit sites for a stratum
                }
                
                # End the loop for individual strata
              }
              
              # End the section for nonempty vectors of continuous variable
              # values for revisit sites
            }
            
            # End the section for stratified data
          } else {
            
            # Begin the section for unstratified data
            
            # Check whether the vectors of continuous variable values for revisit
            # sites are empty or contain a single value
            
            if (length(contvar_1) <= 1) {
              rslt <- NA
              warn_ind <- TRUE
              act <- "Covariance among the revisited sites was not included in calculation of \nthe standard error estimate.\n"
              warn <- paste("The number of nonmissing repeat visit sites was less than two in one of the \nsurveys.\n", sep = "")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
              
              # Begin section for nonempty vectors of continuous variable values
              # for revisit sites
            } else {
              
              # Calculate mean and total estimates
              
              z1 <- contvar_1
              z2 <- contvar_2
              total1 <- as.vector(svytotal(make.formula(ivar),
                                         design = design_1,
                                         na.rm = TRUE
              ))
              total2 <- as.vector(svytotal(make.formula(ivar),
                                         design = design_2,
                                         na.rm = TRUE
              ))
              
              mean1 <- as.vector(svymean(make.formula(ivar),
                                           design = design_1,
                                           na.rm = TRUE
              ))
              mean2 <- as.vector(svymean(make.formula(ivar),
                                           design = design_2,
                                           na.rm = TRUE
              ))
              
              # Calculate covariance or correlation estimates
              
              if (cluster_ind) {
                temp <- changevar_mean(
                  z1, z2, wgt2, xcoord, ycoord, revisitwgt,
                  mean1, mean2, stratum_ind, NULL, cluster_ind, clusterID, wgt1,
                  xcoord1, ycoord1, vartype, warn_ind, warn_df, warn_vec
                )
              } else {
                temp <- changevar_mean(z1, z2, wgt, xcoord, ycoord, revisitwgt,
                                        mean1, mean2, stratum_ind, NULL, cluster_ind,
                                       vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                                       warn_vec = warn_vec
                )
              }
              rslt <- temp$rslt
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df
              
              # End the section for nonempty vectors of continuous variable
              # values for revisit sites
            }
            
            # End the section for unstratified data
          }
          
          # Calculate standard errors
          
          if (is.na(rslt)) {
            results$StdError <- sqrt(results$StdError_1^2 + results$StdError_2^2)
          } else {
            tw_1r <- sum(weights(design_1))
            tw_2r <- sum(weights(design_2))
            if (revisitwgt) {
              temp <- results$StdError_1^2 + results$StdError_2^2 -
                ((2 * tw_1r * tw_2r) / (tw_1 * tw_2)) * rslt
              if (is.na(temp)) {
                temp <- 0
              }
              if (temp <= 0) {
                results$StdError <- sqrt(results$StdError_1^2 +
                                           results$StdError_2^2)
              } else {
                results$StdError <- sqrt(temp)
              }
            } else {
              temp <- total_est(
                NULL, dframe_1, itype, isubpop, 1, ivar,
                design_1, design_names, var_nondetect, vartype, conf, mult,
                warn_ind, warn_df
              )
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df
              se_1 <- temp$totalsum$StdError
              temp <- total_est(
                NULL, dframe_2, itype, isubpop, 1, ivar,
                design_2, design_names, var_nondetect, vartype, conf, mult,
                warn_ind, warn_df
              )
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df
              se_2 <- temp$totalsum$StdError
              covest <- rslt * se_1 * se_2
              temp <- results$StdError_1^2 + results$StdError_2^2 -
                ((2 * tw_1r * tw_2r) / (tw_1 * tw_2)) * covest
              if (is.na(temp)) {
                temp <- 0
              }
              if (temp <= 0) {
                results$StdError <- sqrt(results$StdError_1^2 +
                                           results$StdError_2^2)
              } else {
                results$StdError <- sqrt(temp)
              }
            }
          }
          
          # Calculate margins of error and confidence bounds for the change
          # estimate
          
          results$MarginofError <- mult * results$StdError
          results$LCB <- results$DiffEst - mult * results$StdError
          results$UCB <- results$DiffEst + mult * results$StdError
          
          # End the section for surveys with repeat visit sites
        }
        
        # Add estimates to the contsum_mean data frame
        
        changesum$contsum_total <- rbind(changesum$contsum_total, cbind(
          Survey_1 = survey_names[1],
          Survey_2 = survey_names[2],
          subset(results, select = c(1:3, 16:20, 4:9, 10:15))
        ))
        
        #
        # End the section for a continuous variable using the total
        #
      }

      if ("median" %in% test) {

        #
        # Begin the section for a continuous variable using the median
        #

        # As necessary, restore original values for the dframe_1 and dframe_2
        # data frames and the design_1 and design_2 objects

        if (any(c("mean", "total") %in% test)) {
          dframe_1 <- dframe_1_org
          dframe_2 <- dframe_2_org
          design_1 <- design_1_org
          design_2 <- design_2_org
        }

        # Calculate estimate for all sites from survey one

        pctest_1 <- percentile_est(NULL, dframe_1, itype, isubpop, 1, ivar,
          design_1, design_names, var_nondetect, conf, mult,
          pctval = c(50),
          warn_ind, warn_df
        )
        warn_ind <- pctest_1$warn_ind
        warn_df <- pctest_1$warn_df

        # Calculate estimate for all sites from survey two

        pctest_2 <- percentile_est(NULL, dframe_2, itype, isubpop, 1, ivar,
          design_2, design_names, var_nondetect, conf, mult,
          pctval = c(50),
          warn_ind, warn_df
        )
        warn_ind <- pctest_2$warn_ind
        warn_df <- pctest_2$warn_df

        # Determine whether either of the surveys contains a single observation

        if (is.na(pctest_1$pctsum$Estimate) | is.na(pctest_2$pctsum$Estimate)) {
          warn_ind <- TRUE
          warn <- paste0("For subpopulation \"", isubpop, "\" of population type \"", itype, "\", indicator \"", ivar, "\" \ncontains a single value for at least one of the surveys.\n")
          act <- "A change estimate for the median was not calculated.\n"
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = I(itype), subpop = I(isubpop),
            indicator = I(ivar), stratum = NA, warning = I(warn),
            action = I(act)
          ))
        } else {

          # Create the categorical response variables

          p_1 <- factor(ifelse(dframe_1[, ivar] <= pctest_1$pctsum$Estimate,
            "Less_Than_Median", "Greater_Than_Median"
          ))
          dframe_1$medcat <- p_1
          if ("postStrata" %in% names(design_1)) {
            temp <- factor(ifelse(design_1$variables[, ivar] <=
              pctest_1$pctsum$Estimate, "Less_Than_Median",
            "Greater_Than_Median"
            ))
            design_1 <- update(design_1, medcat = temp)
          } else {
            design_1 <- update(design_1, medcat = p_1)
          }
          p_2 <- factor(ifelse(dframe_2[, ivar] <= pctest_1$pctsum$Estimate,
            "Less_Than_Median", "Greater_Than_Median"
          ))
          dframe_2$medcat <- p_2
          if ("postStrata" %in% names(design_1)) {
            temp <- factor(ifelse(design_2$variables[, ivar] <=
              pctest_1$pctsum$Estimate, "Less_Than_Median",
            "Greater_Than_Median"
            ))
            design_2 <- update(design_2, medcat = temp)
          } else {
            design_2 <- update(design_2, medcat = p_2)
          }
          # save indicator name to replace later after medcat used for analysis
          ivar_name <- ivar
          ivar <- "medcat"
          lev_ivar <- levels(p_1)
          nlev_ivar <- 2

          # Calculate estimates for all sites from survey one

          dframe_1[, itype] <- droplevels(dframe_1[, itype])
          temp <- category_est(
            NULL, dframe_1, itype, isubpop, 1, ivar, lev_ivar, nlev_ivar,
            design_1, design_names, vartype, conf, mult, warn_ind, warn_df
          )
          temp_1 <- droplevels(subset(temp$catsum, Category != "Total"))
          warn_ind <- temp$warn_ind
          warn_df <- temp$warn_df
          tw_1 <- sum(weights(design_1))

          # Calculate estimates for all sites from survey two

          dframe_2[, itype] <- droplevels(dframe_2[, itype])
          temp <- category_est(
            NULL, dframe_2, itype, isubpop, 1, ivar, lev_ivar, nlev_ivar,
            design_2, design_names, vartype, conf, mult, warn_ind, warn_df
          )
          temp_2 <- droplevels(subset(temp$catsum, Category != "Total"))
          warn_ind <- temp$warn_ind
          warn_df <- temp$warn_df
          tw_2 <- sum(weights(design_2))

          # Merge results for the two surveys

          results <- merge(temp_1, temp_2,
            by = "Category", suffix = c("_1", "_2"),
            all = TRUE, sort = FALSE
          )

          # Calculate the change estimates

          results$DiffEst.P <- (results$Estimate.P_2 - results$Estimate.P_1) / 100
          results$DiffEst.U <- results$Estimate.U_2 - results$Estimate.U_1

          # Express the standard error estimates for the two surveys on the
          # proportion scale

          results$StdError.P_1 <- results$StdError.P_1 / 100
          results$StdError.P_2 <- results$StdError.P_2 / 100

          # Calculate confidence bound multiplier

          mult <- qnorm(0.5 + (conf / 100) / 2)

          # Calculate standard error of the change estimates for surveys with no
          # repeat visit sites

          if (sum(repeat_1) == 0) {
            results$StdError.P <- sqrt(results$StdError.P_1^2 +
              results$StdError.P_2^2)
            results$StdError.U <- sqrt(results$StdError.U_1^2 +
              results$StdError.U_2^2)
            results$LCB.P <- 100 * pmax(
              results$DiffEst.P - mult * results$StdError.P, -1
            )
            results$UCB.P <- 100 * pmin(
              results$DiffEst.P + mult * results$StdError.P, 1
            )
            results$DiffEst.P <- 100 * results$DiffEst.P
            results$MarginofError.P <- 100 * (mult * results$StdError.P)
            results$MarginofError.P_1 <- 100 * (mult * results$StdError.P_1)
            results$MarginofError.P_2 <- 100 * (mult * results$StdError.P_2)
            results$StdError.P <- 100 * results$StdError.P
            results$StdError.P_1 <- 100 * results$StdError.P_1
            results$StdError.P_2 <- 100 * results$StdError.P_2
            results$MarginofError.U <- mult * results$StdError.U
            results$MarginofError.U_1 <- mult * results$StdError.U_1
            results$MarginofError.U_2 <- mult * results$StdError.U_2
            if ("postStrata" %in% names(design_1)) {
              results$LCB.U <- pmax(
                results$DiffEst.U - mult * results$StdError.U, -tw_1
              )
              results$UCB.U <- pmin(
                results$DiffEst.U + mult * results$StdError.U, tw_2
              )
            } else {
              results$LCB.U <- results$DiffEst.U - mult * results$StdError.U
              results$UCB.U <- results$DiffEst.U + mult * results$StdError.U
            }

            # Calculate standard error of the change estimates for surveys with
            # repeat visit sites
          } else {

            # Subset the dframe_1 and design_2 objects to retain repeat visit
            # sites

            dframe_1 <- subset(dframe_1, repeat_1)
            dframe_2 <- subset(dframe_2, repeat_2)

            # Subset the design_1 and design_2 objects to retain repeat visit
            # sites

            design_1 <- subset(design_1, repeat_1)
            design_2 <- subset(design_2, repeat_2)

            # Assign values for the categorical variables

            catvar_1 <- dframe_1[, ivar]
            catvar_2 <- dframe_2[, ivar]

            # Assign values for survey design variables using the survey one
            # design object

            tempdf <- design_1$variables
            for (i in names(design_names)) {
              if (is.null(design_names[[i]])) {
                eval(parse(text = paste0(i, " <- NULL")))
              } else {
                eval(parse(text = paste0(
                  i, " <- tempdf[, \"", design_names[[i]],
                  "\"]"
                )))
              }
            }

            # Assign a value to the indicator variable for a two-stage sample

            cluster_ind <- !is.null(clusterID)

            # Assign values to weight variables

            if (revisitwgt) {
              if (cluster_ind) {
                wgt1 <- tempdf$wgt1
                wgt2 <- tempdf$wgt2
              } else {
                wgt <- tempdf$wgt
              }
            } else {
              if (cluster_ind) {
                wgt1 <- rep(1, length(catvar_1))
                wgt2 <- rep(1, length(catvar_1))
              } else {
                wgt <- rep(1, length(catvar_1))
              }
            }

            # Assign a logical value to the indicator variable for a stratified
            # sample

            stratum_ind <- !is.null(stratumID)

            # If the sample is stratified, convert stratum to a factor,
            # determine stratum levels, and calculate number of strata

            if (stratum_ind) {
              stratum <- factor(stratumID)
              stratum_levels <- levels(stratum)
              nstrata <- length(stratum_levels)
            }

            # Remove missing values

            indx <- 1:sum(repeat_1)
            indx[is.na(catvar_1) | is.na(catvar_2)] <- 0
            catvar_1 <- catvar_1[indx]
            catvar_2 <- catvar_2[indx]
            if (stratum_ind) {
              if (cluster_ind) {
                wgt2 <- wgt2[indx]
                xcoord <- xcoord[indx]
                ycoord <- ycoord[indx]
                stratum <- stratum[indx]
                clusterID <- clusterID[indx]
                wgt1 <- wgt1[indx]
                xcoord1 <- xcoord1[indx]
                ycoord1 <- ycoord1[indx]
              } else {
                wgt <- wgt[indx]
                xcoord <- xcoord[indx]
                ycoord <- ycoord[indx]
                stratum <- stratum[indx]
              }
            } else {
              if (cluster_ind) {
                wgt2 <- wgt2[indx]
                xcoord <- xcoord[indx]
                ycoord <- ycoord[indx]
                clusterID <- clusterID[indx]
                wgt1 <- wgt1[indx]
                xcoord1 <- xcoord1[indx]
                ycoord1 <- ycoord1[indx]
              } else {
                wgt <- wgt[indx]
                xcoord <- xcoord[indx]
                ycoord <- ycoord[indx]
              }
            }

            # For a stratified sample, remove strata that contain a single site

            if (stratum_ind) {
              ind <- FALSE
              for (i in 1:nstrata) {
                tst <- stratum == stratum_levels[i]
                if (sum(tst) == 1) {
                  warn_ind <- TRUE
                  warn <- paste0("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
                  act <- "Stratum was not used for standard error estimation.\n"
                  warn_df <- rbind(warn_df, data.frame(
                    func = I(fname), subpoptype = NA,
                    subpop = NA, indicator = NA, stratum = NA, warning = I(warn),
                    action = I(act)
                  ))
                  dframe <- dframe[!tst, ]
                  ind <- TRUE
                  catvar_1 <- catvar_1[!tst]
                  catvar_2 <- catvar_2[!tst]
                  if (vartype == "Local") {
                    xcoord <- xcoord[!tst]
                    ycoord <- ycoord[!tst]
                  }
                  stratum <- stratum[!tst]
                  if (cluster_ind) {
                    clusterID <- clusterID[!tst]
                    wgt1 <- wgt1[!tst]
                    wgt2 <- wgt2[!tst]
                    if (vartype == "Local") {
                      xcoord1 <- xcoord1[!tst]
                      ycoord1 <- ycoord1[!tst]
                    }
                  } else {
                    wgt <- wgt[!tst]
                  }
                  ind <- TRUE
                }
              }
              if (ind) {
                stratum <- factor(stratum)
                stratum_levels <- levels(stratum)
                nstrata <- length(stratum_levels)
              }
            }

            # For a stratified sample, check whether the number of strata is one

            if (stratum_ind) {
              if (nstrata == 1) {
                warn_ind <- TRUE
                warn <- "Only a single stratum was available for the analysis.\n"
                act <- "An unstratified data analysis was used.\n"
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1],
                  subpop = warn_vec[2], indicator = warn_vec[3], stratum = NA,
                  warning = I(warn), action = I(act)
                ))
                stratum_ind <- FALSE
              }
            }

            # Assign levels of the categorical variables

            catvar_levels <- results$Category
            nlevels <- length(catvar_levels)

            # Calculate population size values

            if (stratum_ind) {
              if (cluster_ind) {
                popsize_hat <- tapply(wgt1 * wgt2, stratum, sum)
                sum_popsize_hat <- sum(wgt1 * wgt2)
              } else {
                popsize_hat <- tapply(wgt, stratum, sum)
                sum_popsize_hat <- sum(wgt)
              }
            } else {
              if (cluster_ind) {
                popsize_hat <- sum(wgt1 * wgt2)
              } else {
                popsize_hat <- sum(wgt)
              }
            }

            # Branch to handle stratified and unstratified data

            if (stratum_ind) {

              # Begin the section for stratified data

              # Create the vector of covariance or correlation estimates for all
              # strata combined

              rslt_P <- rep(NA, nlevels)
              rslt_U <- rep(NA, nlevels)

              # Check whether the vectors of categorical variable values for
              # revisit sites are empty or contain a single value

              if (length(catvar_1) <= 1) {
                warn_ind <- TRUE
                act <- "Covariance among the revisited sites was not included in calculation of \nthe standard error estimate.\n"
                warn <- paste("The number of nonmissing repeat visit sites was less than two in one of the \nsurveys.\n", sep = "")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1],
                  subpop = warn_vec[2], indicator = warn_vec[3], stratum = NA,
                  warning = I(warn), action = I(act)
                ))

                # Begin section for nonempty vectors of categorical variable
                #  values for revisit sites
              } else {

                # Begin the loop for individual strata

                for (i in 1:nstrata) {

                  # Check whether the vectors of categorical variable values for
                  # revisit sites are empty or contain a single value for a
                  #  stratum

                  stratum_i <- stratum == stratum_levels[i]
                  if ((sum(!is.na(catvar_1[stratum_i])) <= 1) |
                    (sum(!is.na(catvar_2[stratum_i])) <= 1)) {
                    warn_ind <- TRUE
                    act <- "Due to insufficient number of sites, the stratum was not included in \ncalculation of covariance among the revisited sites.\n"
                    warn <- paste("The number of nonmissing repeat visit sites  in one of the surveys was less \nthan two for stratum \"", stratum_levels[i], "\".\n", sep = "")
                    warn_df <- rbind(warn_df, data.frame(
                      func = I(fname),
                      subpoptype = warn_vec[1], subpop = warn_vec[2],
                      indicator = warn_vec[3], stratum = I(stratum_levels[i]),
                      warning = I(warn), action = I(act)
                    ))

                    # Begin section for nonempty vectors of categorical variable
                    # values for revisit sites for a stratum
                  } else {

                    # Calculate proportion estimates

                    z1 <- factor(catvar_1[stratum_i], levels = catvar_levels)
                    z2 <- factor(catvar_2[stratum_i], levels = catvar_levels)
                    m <- length(catvar_levels)
                    prop1 <- as.vector(svymean(make.formula(ivar),
                      design = subset(design_1, stratum_i), na.rm = TRUE
                    ))
                    prop2 <- as.vector(svymean(make.formula(ivar),
                      design = subset(design_2, stratum_i), na.rm = TRUE
                    ))

                    # Calculate covariance or correlation estimates

                    if (cluster_ind) {
                      temp <- changevar_prop(
                        catvar_levels, z1, z2, wgt2[stratum_i],
                        xcoord[stratum_i], ycoord[stratum_i], revisitwgt, prop1,
                        prop2, stratum_ind, stratum_levels[i], cluster_ind,
                        clusterID[stratum_i], wgt1[stratum_i],
                        xcoord1[stratum_i], ycoord1[stratum_i], vartype,
                        warn_ind, warn_df, warn_vec
                      )
                    } else {
                      temp <- changevar_prop(catvar_levels, z1, z2,
                        wgt[stratum_i],
                        xcoord[stratum_i], ycoord[stratum_i], revisitwgt, prop1,
                        prop2, stratum_ind, stratum_levels[i], cluster_ind,
                        vartype = vartype, warn_ind = warn_ind,
                        warn_df = warn_df, warn_vec = warn_vec
                      )
                    }
                    correst <- temp$rslt
                    warn_ind <- temp$warn_ind
                    warn_df <- temp$warn_df

                    # Add estimates to the vector for all strata combined

                    rslt_P[!is.na(correst)] <- rslt_P[!is.na(correst)] +
                      (popsize_hat[i] / sum_popsize_hat) * correst[!is.na(correst)]

                    # Estimate the size of each category

                    size1 <- popsize_hat[i] * prop1
                    size2 <- popsize_hat[i] * prop2

                    # Calculate covariance or correlation estimates

                    if (cluster_ind) {
                      temp <- changevar_total(
                        catvar_levels, z1, z2, wgt2[stratum_i],
                        xcoord[stratum_i], ycoord[stratum_i], revisitwgt, size1,
                        size2, stratum_ind, stratum_levels[i], cluster_ind,
                        clusterID[stratum_i], wgt1[stratum_i],
                        xcoord1[stratum_i], ycoord1[stratum_i], vartype,
                        warn_ind, warn_df, warn_vec
                      )
                    } else {
                      temp <- changevar_total(catvar_levels, z1, z2,
                        wgt[stratum_i], xcoord[stratum_i], ycoord[stratum_i],
                        revisitwgt, size1, size2, stratum_ind, stratum_levels[i],
                        cluster_ind,
                        vartype = vartype, warn_ind = warn_ind,
                        warn_df = warn_df, warn_vec = warn_vec
                      )
                    }
                    correst <- temp$rslt
                    warn_ind <- temp$warn_ind
                    warn_df <- temp$warn_df

                    # Add estimates to the vector for all strata combined

                    rslt_U[!is.na(correst)] <- rslt_U[!is.na(correst)] +
                      correst[!is.na(correst)]

                    # End the section for nonempty vectors of categorical
                    # variable values for revisit sites for a stratum
                  }

                  # End the loop for individual strata
                }

                # End the section for nonempty vectors of categorical variable
                # values for revisit sites
              }

              # End the section for stratified data
            } else {

              # Begin the section for unstratified data

              # Check whether the vectors of categorical variable values for
              # revisit sites are empty or contain a single value

              if (length(catvar_1) <= 1) {
                rslt_P <- rep(NA, nlevels)
                rslt_U <- rep(NA, nlevels)
                warn_ind <- TRUE
                act <- "Covariance among the revisited sites was not included in calculation of \nthe standard error estimate.\n"
                warn <- paste("The number of nonmissing repeat visit sites was less than two in one of the \nsurveys.\n", sep = "")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1],
                  subpop = warn_vec[2], indicator = warn_vec[3],
                  stratum = NA, warning = I(warn), action = I(act)
                ))

                # Begin section for nonempty vectors of categorical variable
                #  values for revisit sites
              } else {

                # Calculate proportion estimates

                z1 <- factor(catvar_1, levels = catvar_levels)
                z2 <- factor(catvar_2, levels = catvar_levels)
                prop1 <- as.vector(svymean(make.formula(ivar),
                  design = design_1, na.rm = TRUE
                ))
                prop2 <- as.vector(svymean(make.formula(ivar),
                  design = design_2, na.rm = TRUE
                ))

                # Calculate covariance or correlation estimates

                if (cluster_ind) {
                  temp <- changevar_prop(
                    catvar_levels, z1, z2, wgt2, xcoord, ycoord, revisitwgt,
                    prop1, prop2, stratum_ind, NULL, cluster_ind, clusterID,
                    wgt1, xcoord1, ycoord1, vartype, warn_ind, warn_df, warn_vec
                  )
                } else {
                  temp <- changevar_prop(catvar_levels, z1, z2, wgt, xcoord,
                    ycoord, revisitwgt, prop1, prop2, stratum_ind, NULL,
                    cluster_ind,
                    vartype = vartype, warn_ind = warn_ind,
                    warn_df = warn_df, warn_vec = warn_vec
                  )
                }
                rslt_P <- temp$rslt
                warn_ind <- temp$warn_ind
                warn_df <- temp$warn_df

                # Estimate the size of each category

                size1 <- popsize_hat * prop1
                size2 <- popsize_hat * prop2

                # Calculate covariance or correlation estimates

                if (cluster_ind) {
                  temp <- changevar_total(
                    catvar_levels, z1, z2, wgt2, xcoord, ycoord,
                    revisitwgt, size1, size2, stratum_ind, NULL, cluster_ind,
                    clusterID, wgt1, xcoord1, ycoord1, vartype, warn_ind,
                    warn_df, warn_vec
                  )
                } else {
                  temp <- changevar_total(
                    catvar_levels, z1, z2, wgt, xcoord, ycoord, revisitwgt,
                    size1, size2, stratum_ind, NULL, cluster_ind,
                    vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                    warn_vec = warn_vec
                  )
                }
                rslt_U <- temp$rslt
                warn_ind <- temp$warn_ind
                warn_df <- temp$warn_df

                # End the section for nonempty vectors of categorical variable
                # values for revisit sites
              }

              # End the section for unstratified data
            }

            # Calculate standard errors

            results$StdError.P <- rep(NA, nlevels)
            results$StdError.U <- rep(NA, nlevels)
            ind <- is.na(rslt_P)
            results$StdError.P[ind] <- sqrt(results$StdError.P_1[ind]^2 +
              results$StdError.P_2[ind]^2)
            results$StdError.U[ind] <- sqrt(results$StdError.U_1[ind]^2 +
              results$StdError.U_2[ind]^2)
            if (any(!ind)) {
              tw_1r <- sum(weights(design_1))
              tw_2r <- sum(weights(design_2))
              if (revisitwgt) {
                temp <- results$StdError.P_1^2 + results$StdError.P_2^2 -
                  ((2 * tw_1r * tw_2r) / (tw_1 * tw_2)) * rslt_P
                ind <- !is.na(rslt_P) & temp <= 0
                results$StdError.P[ind] <- sqrt(results$StdError.P_1[ind]^2 +
                  results$StdError.P_2[ind]^2)
                ind <- !is.na(rslt_P) & temp > 0
                results$StdError.P[ind] <- sqrt(temp[ind])
                temp <- results$StdError.U_1^2 + results$StdError.U_2^2 - 2 *
                  rslt_U
                ind <- !is.na(rslt_U) & temp <= 0
                results$StdError.U[ind] <- sqrt(results$StdError.U_1[ind]^2 +
                  results$StdError.U_2[ind]^2)
                ind <- !is.na(rslt_U) & temp > 0
                results$StdError.U[ind] <- sqrt(temp[ind])
              } else {
                se_1_p <- rep(NA, nlevels)
                se_1_u <- rep(NA, nlevels)
                temp <- category_est(
                  NULL, dframe_1, itype, isubpop, 1, ivar, lev_ivar, nlev_ivar,
                  design_1, design_names, vartype, conf, mult, warn_ind, warn_df
                )
                warn_ind <- temp$warn_ind
                warn_df <- temp$warn_df
                temp$results <- droplevels(subset(
                  temp$catsum,
                  Category != "Total"
                ))
                ind <- match(temp$results$Category, catvar_levels, nomatch = 0)
                se_1_p[ind] <- temp$results$StdError.P / 100
                se_1_u[ind] <- temp$results$StdError.U
                se_2_p <- rep(NA, nlevels)
                se_2_u <- rep(NA, nlevels)
                temp <- category_est(
                  NULL, dframe_2, itype, isubpop, 1, ivar, lev_ivar, nlev_ivar,
                  design_2, design_names, vartype, conf, mult, warn_ind, warn_df
                )
                warn_ind <- temp$warn_ind
                warn_df <- temp$warn_df
                temp$results <- droplevels(subset(
                  temp$catsum,
                  Category != "Total"
                ))
                ind <- match(temp$results$Category, catvar_levels, nomatch = 0)
                se_2_p[ind] <- temp$results$StdError.P / 100
                se_2_u[ind] <- temp$results$StdError.U
                covest <- rslt_P * se_1_p * se_2_p
                temp <- results$StdError.P_1^2 + results$StdError.P_2^2 -
                  ((2 * tw_1r * tw_2r) / (tw_1 * tw_2)) * covest
                ind <- !is.na(rslt_P) & temp <= 0
                results$StdError.P[ind] <- sqrt(results$StdError.P_1[ind]^2 +
                  results$StdError.P_2[ind]^2)
                ind <- !is.na(rslt_P) & temp > 0
                results$StdError.P[ind] <- sqrt(temp[ind])
                covest <- rslt_U * se_1_u * se_2_u
                temp <- results$StdError.U_1^2 + results$StdError.U_2^2 - 2 *
                  covest
                ind <- !is.na(rslt_U) & temp <= 0
                results$StdError.U[ind] <- sqrt(results$StdError.U_1[ind]^2 +
                  results$StdError.U_2[ind]^2)
                ind <- !is.na(rslt_U) & temp > 0
                results$StdError.U[ind] <- sqrt(temp[ind])
              }
            }

            # Calculate margins of error and confidence bounds

            results$LCB.P <- 100 * pmax(results$DiffEst.P -
              mult * results$StdError.P, -1)
            results$UCB.P <- 100 * pmin(results$DiffEst.P +
              mult * results$StdError.P, 1)
            results$DiffEst.P <- 100 * results$DiffEst.P
            results$MarginofError.P <- 100 * (mult * results$StdError.P)
            results$MarginofError.P_1 <- 100 * (mult * results$StdError.P_1)
            results$MarginofError.P_2 <- 100 * (mult * results$StdError.P_2)
            results$StdError.P <- 100 * results$StdError.P
            results$StdError.P_1 <- 100 * results$StdError.P_1
            results$StdError.P_2 <- 100 * results$StdError.P_2
            results$MarginofError.U <- mult * results$StdError.U
            results$MarginofError.U_1 <- mult * results$StdError.U_1
            results$MarginofError.U_2 <- mult * results$StdError.U_2
            if ("postStrata" %in% names(design_1)) {
              results$LCB.U <- pmax(
                results$DiffEst.U - mult * results$StdError.U, -tw_1
              )
              results$UCB.U <- pmin(
                results$DiffEst.U + mult * results$StdError.U, tw_2
              )
            } else {
              results$LCB.U <- results$DiffEst.U - mult * results$StdError.U
              results$UCB.U <- results$DiffEst.U + mult * results$StdError.U
            }

            # End the section for surveys with repeat visit sites
          }

          # Add estimates to the contsum_median data frame

          changesum$contsum_median <- rbind(changesum$contsum_median, cbind(
            Survey_1 = survey_names[1],
            Survey_2 = survey_names[2],
            subset(results, select = c(
              2:4, 1, 30, 32, 36, 34:35, 31, 33, 37:39,
              5:15, 19:29
            ))
          ))
        }
        
        #replace medcat name with original variable name
        changesum$contsum_median$Indicator_1 <- ivar_name

        #
        # End the section for a continuous variable using the median
        #
      }

      # Print an error message for an unrecognized type of test

      if (!any(c("mean", "total", "median") %in% test)) {
        stop(paste0("\nThe value provided for argument test, \"", test,
          "\", is not a valid value",
          sep = ""
        ))
      }
    } else {

      # Print an error message for an unrecognized type of response variable

      stop(paste("\nThe value provided for argument resp_ind, ", resp_ind,
        ", is not a valid value",
        sep = ""
      ))
    }
  }

  # Return the changesum object, the warn_ind logical value, and the warn_df
  # data frame

  list(changesum = changesum, warn_ind = warn_ind, warn_df = warn_df)
}

###############################################################################
# Function: grtspts_ipleg (not exported)
# Programmers: Tony Olsen
# Date: January 22, 2021
#'
#' Adjust the initial inclusion probabities for point sample to ensure inclusion of
#' legacy elements that must be in the sample.
#'
#' @param ip Initial inclusion probability for the sample ignoring which elements in the
#'   sample frame must be included in the sample as legacy sites
#'
#' @param legacy A logical vector that identifies whether an element of the sample
#'   frame is a legacy site (\code{TRUE}) or not (\code{FALSE}) that must be included in the sample.
#'
#' @return A numeric vector of inclusion probabilities with all values being greater
#'   than \code{0} and less than or equal \code{1}. Values equal to \code{1} will be selected with certainty.
#'   Values equal to \code{0} will not be selected. Legacy sites will have an inclusion
#'   probability equal to \code{1}.
#'
#' @author Tony Olsen \email{olsen.tony@@epa.gov}
#'
#' @keywords survey, inclusion probability
#'
#' @examples
#' a <- runif(30)
#' tmp <- grtspts_ipleg(ip = 10 * a / sum(a), legacy = c(
#'   TRUE, rep(FALSE, 5), TRUE, TRUE,
#'   rep(FALSE, 22)
#' ))
#' tmp
#' @noRd
###############################################################################
grtspts_ipleg <- function(ip, legacy) {
  # determine sample size
  n <- sum(ip)
  # set legacy site inclusion probabilities equal to 1
  ip[legacy == TRUE] <- 1

  # determine number of elements with inclusion probability >= 1
  elem_nge1 <- ip >= 1
  nge1 <- sum(elem_nge1)

  # adjust non legacy ip to sum to n - nleg
  tst <- 0
  while (nge1 != tst) {
    tmp <- ip[!elem_nge1]
    ip[!elem_nge1] <- (n - nge1) * tmp / sum(tmp)
    tst <- nge1
    element_ge1 <- ip >= 1
    nge1 <- sum(element_ge1)
  }
  invisible(ip)
}

###############################################################################
# Function: svychisq_localmean (not exported)
# Programmers: Thomas Lumley
#              Tom Kincaid
# Date: January 7, 2014
# Revised: October 23, 2020 to accomodate use of the local mean variance
#          estimator
# Revised: October 30, 2020 to correctly process the column variable  when it
#          includes missing (NA) values
#'
#' Design-Based Contingency Table Tests
#'
#' This function implements design-based contingency table tests.  It is a
#' modified version of the \code{svychisq} function in the survey package.  The
#' modification allows use of the local mean variance algorithms for calculating
#' variance of contingency table means and totals.
#'
#' @param formula Two-sided model formula specifying margins for the contingency
#'   table.
#'
#' @param design Survey design object created by the \code{svydesign} function.
#'
#' @param statistic Character value specifying the test statistic.  See Details.
#'
#' @param vartype  Character value identifying the choice for variance
#'   estimator, where \code{"Local"} = local mean estimator and \code{"SRS"} = simple random
#'   sampling estimator.  The default is \code{"Local"}.
#'
#' @param var_totals Vector containing variance of totals for the contingency
#'   table, which is required for calculating the Wald statistics when vartype
#'   equals \code{"Local"}.  The default is \code{NULL}.
#'
#' @param var_means Vector containing variance of means for contingency table
#'   cells, which is required for calculating the Rao-Scott statistics when
#'   vartype equals \code{"Local"}.  The default is \code{NULL}.
#'
#' @section Details:
#' \code{svychisq_localmean} computes first and second-order Rao-Scott corrections to
#' the Pearson chi-squared test and two Wald-type tests.  For calculating the
#' variance of contingency table means and totals, the function can use either
#' the local mean variance estimator or the simple random sampling variance
#' estimator.
#'
#' The default test (statistic equals \code{"F"}) is the Rao-Scott second-order
#' correction. The p-value is computed using a Satterthwaite approximation to
#' the distribution with denominator degrees of freedom as recommended by Thomas
#' and Rao (1990).
#'
#' The test specified by statistic equals \code{"Chisq"} adjusts the Pearson
#' chi-squared statistic by a design effect estimate and then compares that
#' value to the chi-squared distribution it would have under simple random
#' sampling.
#'
#' The test specified by statistic equals \code{"Wald"} is the procedure proposed by
#' Koch et al (1975) and used by the SUDAAN software package. It is a Wald test
#' based on the differences between the observed cells counts and those expected
#' under independence.
#'
#' The test specified by statistic equals \code{"adjWald"} reduces the Wald statistic
#' when the number of sites is small compared to the number of degrees of
#' freedom of the test. Thomas and Rao (1990) compared The two Wald tests and
#' found the adjustment benefical.
#'
#' The test specified by statistic equals \code{"lincom"} replaces the numerator of the
#' Rao-Scott F with the exact asymptotic distribution, which is a linear
#' combination of chi-squared variables (see \code{pchisqsum}).  The CompQuadForm
#' package is required when statistic equals \code{"lincom"} is specified.
#'
#' The test specified by statistic equals \code{"saddlepoint"} uses a saddlepoint
#' approximation to the  exact asymptotic distribution mentioned previously.
#' The CompQuadForm package is not required when statistic equals \code{"lincom"} is
#' specified. The saddlepoint approximation is especially useful when the
#' p-value is very small (e.g., large-scale multiple testing problems).
#'
#' @return List belonging to class \code{htest}.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @examples
#' n <- 100
#' resp <- rnorm(n, 10, 1)
#' wgt <- runif(n, 10, 100)
#' xcoord <- runif(n)
#' ycoord <- runif(n)
#' sample1 <- data.frame(
#'   samp = "sample1", z = resp, wgt = wgt, x = xcoord,
#'   y = ycoord
#' )
#' sample2 <- data.frame(
#'   samp = "sample2", z = resp + 0.5, wgt = wgt, x = xcoord,
#'   y = ycoord
#' )
#' bounds <- sort(c(sample1$z, sample2$z))[floor(seq((2 * n) / 3, (2 * n),
#'   length = 3
#' ))]
#' dframe <- rbind(sample1, sample2)
#' dframe$id <- paste0("Site", 1:(2 * n))
#' dframe$catvar <- cut(dframe$z, c(-Inf, bounds))
#' design <- survey::svydesign(id = ~id, weights = ~wgt, data = dframe)
#' svychisq_localmean(~ samp + catvar, design,
#'   statistic = "adjWald",
#'   vartype = "SRS"
#' )
#' @noRd
###############################################################################

svychisq_localmean <- function(formula, design, statistic = c(
                                 "F", "Chisq",
                                 "Wald", "adjWald", "lincom", "saddlepoint"
                               ), vartype = "Local",
                               var_totals = NULL, var_means = NULL) {
  statistic <- match.arg(statistic)

  rows <- formula[[2]][[2]]
  cols <- formula[[2]][[3]]
  rowvar <- unique(design$variables[, as.character(rows)])
  rowvar <- rowvar[!is.na(rowvar)]
  colvar <- unique(design$variables[, as.character(cols)])
  colvar <- colvar[!is.na(colvar)]
  nr <- length(rowvar)
  nc <- length(colvar)
  mf1 <- expand.grid(rows = 1:nr, cols = 1:nc)

  fsat <- eval(bquote(~ interaction(factor(.(rows)), factor(.(cols))) - 1))
  mm <- model.matrix(fsat, model.frame(fsat, design$variables,
    na.action = na.pass
  ))
  N <- nrow(mm)
  nu <- length(unique(design$cluster[, 1])) - length(unique(design$strata[, 1]))

  pearson <- suppressWarnings(chisq.test(svytable(formula, design, Ntotal = N),
    correct = FALSE
  ))

  if (statistic %in% c("Wald", "adjWald")) {
    frow <- eval(bquote(~ factor(.(rows)) - 1))
    fcol <- eval(bquote(~ factor(.(cols)) - 1))
    mr <- model.matrix(frow, model.frame(frow, design$variables,
      na.action = na.pass
    ))
    mc <- model.matrix(fcol, model.frame(fcol, design$variables,
      na.action = na.pass
    ))
    one <- rep(1, NROW(mc))
    cells <- svytotal(~ mm + mr + mc + one, design, na.rm = TRUE)

    Jcb <- cbind(
      diag(nr * nc),
      -outer(mf1$rows, 1:nr, "==") * rep(cells[(nr * nc) + nr + 1:nc] /
        cells[(nr * nc) + nr + nc + 1], each = nr),
      -outer(mf1$cols, 1:nc, "==") * cells[(nr * nc) + 1:nr] /
        cells[(nr * nc) + nr + nc + 1],
      as.vector(outer(cells[(nr * nc) + 1:nr], cells[(nr * nc + nr) + 1:nc]) /
        cells[(nr * nc) + nr + nc + 1]^2)
    )

    Y <- cells[1:(nc * nr)] - as.vector(outer(
      cells[(nr * nc) + 1:nr],
      cells[(nr * nc + nr) + 1:nc]
    )) / cells[(nr * nc) + nr + nc + 1]
    if (vartype == "Local") {
      V <- Jcb %*% var_totals %*% t(Jcb)
    } else {
      V <- Jcb %*% attr(cells, "var") %*% t(Jcb)
    }
    use <- as.vector(matrix(1:(nr * nc), nrow = nr, ncol = nc)[-1, -1])
    waldstat <- Y[use] %*% solve(V[use, use], Y[use])

    if (statistic == "Wald") {
      waldstat <- waldstat / ((nc - 1) * (nr - 1))
      numdf <- (nc - 1) * (nr - 1)
      denomdf <- nu
    } else {
      numdf <- (nr - 1) * (nc - 1)
      denomdf <- (nu - numdf + 1)
      waldstat <- waldstat * denomdf / (numdf * nu)
    }

    pearson$statistic <- waldstat
    pearson$parameter <- c(ndf = numdf, ddf = denomdf)
    pearson$p.value <- pf(pearson$statistic, numdf, denomdf, lower.tail = FALSE)
    attr(pearson$statistic, "names") <- "F"
    pearson$data.name <- deparse(sys.call(-1))
    pearson$method <- "Design-based Wald test of association"
  } else {
    X1 <- model.matrix(~ factor(rows) + factor(cols), mf1)
    X12 <- model.matrix(~ factor(rows) * factor(cols), mf1)
    Cmat <- qr.resid(qr(X1), X12[, -(1:(nr + nc - 1)), drop = FALSE])
    mean2 <- svymean(mm, design, na.rm = TRUE)
    Dmat <- diag(mean2)
    iDmat <- diag(ifelse(mean2 == 0, 0, 1 / mean2))
    Vsrs <- (Dmat - outer(mean2, mean2)) / N
    if (vartype == "Local") {
      V <- var_means
    } else {
      V <- attr(mean2, "var")
    }
    denom <- t(Cmat) %*% (iDmat / N) %*% Cmat
    numr <- t(Cmat) %*% iDmat %*% V %*% iDmat %*% Cmat
    Delta <- solve(denom, numr)
    d0 <- sum(diag(Delta))^2 / (sum(diag(Delta %*% Delta)))

    if (match.arg(statistic) == "F") {
      pearson$statistic <- pearson$statistic / sum(diag(Delta))
      pearson$p.value <- pf(pearson$statistic, d0, d0 * nu, lower.tail = FALSE)
      attr(pearson$statistic, "names") <- "F"
      pearson$parameter <- c(ndf = d0, ddf = d0 * nu)
      pearson$method <- "Pearson's X^2: Rao & Scott adjustment"
    } else if (match.arg(statistic) == "lincom") {
      pearson$p.value <- pFsum(pearson$statistic, rep(1, ncol(Delta)),
        eigen(Delta, only.values = TRUE)$values,
        lower.tail = FALSE,
        method = "integration", ddf = d0 * nu
      )
      pearson$parameter <- NULL
      pearson$method <- "Pearson's X^2: asymptotic exact distribution"
    } else if (match.arg(statistic) == "saddlepoint") {
      pearson$p.value <- pFsum(pearson$statistic, rep(1, ncol(Delta)),
        eigen(Delta, only.values = TRUE)$values,
        lower.tail = FALSE,
        method = "saddlepoint", ddf = d0 * nu
      )
      pearson$parameter <- NULL
      pearson$method <- "Pearson's X^2: saddlepoint approximation"
    } else {
      pearson$p.value <- pchisq(pearson$statistic / mean(diag(Delta)),
        df = NCOL(Delta), lower.tail = FALSE
      )
      pearson$parameter <- c(df = NCOL(Delta))
      pearson$method <- "Pearson's X^2: Rao & Scott adjustment"
    }
    pearson$data.name <- deparse(sys.call(-1))
  }

  pearson
}

###############################################################################
# Function: warnprnt (exported)
# Programmer: Tom Kincaid
# Date: June 18, 2020
#
#' Print grts(), irs()), and analysis function warnings
#'
#' This function prints the warnings messages from the \code{grts()}, \code{irs()},
#'   and analysis functions.
#'
#' @param warn_df Data frame that contains warning messages.  The default is
#'   \code{"warn_df"}, which is the name given to the warnings data frame created by
#'   functions in the spsurvey package.
#'
#' @param m Vector of indices for warning messages that are to be printed. The
#'   default is a vector containing the integers from 1 through the number of
#'   rows in \code{warn_df}, which will print all warning messages in the data frame.
#'
#' @return Printed warnings.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @export
###############################################################################

warnprnt <- function(warn_df = get("warn_df", envir = .GlobalEnv),
                     m = 1:nrow(warn_df)) {

  # old front code in warndsgn
  # for (i in m) {
  #   cat(paste(
  #     "Message", i, ": Stratum:", warn_df$stratum[i],
  #     "Function: ", warn_df$func[i], "\n"
  #   ))
  #   cat(paste("     ", warn_df$warning[i], "\n"))
  # }

  for (i in m) {
    if ("Warning" %in% names(warn_df)) {
      message(paste0("Warning Message ", i, ":\n"))
      message(paste("Stratum:", warn_df$stratum[i], "\n"))
      message(paste("Warning:", warn_df$Warning[i], "\n"))
    } else {
      message(paste0("Warning Message ", i, ":\n"))
      # message(paste("Function:", warn_df$func[i], "\n")) # removed function name output
      if (sum(!is.null(warn_df$subpoptype[i]), is.na(warn_df$subpoptype[i]))) {
        message(paste("Population Type:", warn_df$subpoptype[i], "\n"))
      }
      if (sum(!is.null(warn_df$subpop[i]), is.na(warn_df$subpop[i]))) {
        message(paste("Subpopulation:", warn_df$subpop[i], "\n"))
      }
      if (sum(!is.null(warn_df$indicator[i]), is.na(warn_df$indicator[i]))) {
        message(paste("Indicator:", warn_df$indicator[i], "\n"))
      }
      if (sum(!is.null(warn_df$stratum[i]), is.na(warn_df$stratum[i]))) {
        message(paste("Stratum:", warn_df$stratum[i], "\n"))
      }
      message(paste("Warning:", warn_df$warning[i]))
      if (sum(!is.null(warn_df$action[i]), is.na(warn_df$action[i]))) {
        message(paste("Action:", warn_df$action[i], "\n"))
      }
    }
  }

  invisible(NULL)
}

###############################################################################
# Function: irs (exported)
# Programmers: Tony Olsen, Tom Kincaid
# Date: January 22, 2022
#' Select an independent random sample (IRS)
#'
#' Select a sample that is not spatially balanced from a point (finite), linear / linestring (infinite),
#' or areal / polygon (infinite) sampling frame using the Independent Random Sampling (IRS) algorithm.
#' The IRS algorithm accommodates unstratified and
#' stratified sampling designs and allows for equal inclusion probabilities, unequal
#' inclusion probabilities according to a categorical variable, and inclusion
#' probabilities proportional to a positive auxiliary variable. Several additional
#' sampling options are included, such as including legacy (historical) sites,
#' requiring a minimum distance between sites, and selecting replacement sites.
#'
#' @inherit grts params details return author
#'
#' @seealso
#'   \describe{
#'     \item{\code{\link{grts}}}{ to select a sample that is spatially balanced}
#'  }
#'
#' @examples
#' \dontrun{
#' samp <- irs(NE_Lakes, n_base = 100)
#' print(samp)
#' strata_n <- c(low = 25, high = 30)
#' samp_strat <- irs(NE_Lakes, n_base = strata_n, stratum_var = "ELEV_CAT")
#' print(samp_strat)
#' samp_over <- irs(NE_Lakes, n_base = 30, n_over = 5)
#' print(samp_over)
#' }
#' @export
###############################################################################
irs <- function(sframe, n_base, stratum_var = NULL, seltype = NULL, caty_var = NULL,
                caty_n = NULL, aux_var = NULL, legacy_var = NULL,
                legacy_sites = NULL, legacy_stratum_var = NULL,
                legacy_caty_var = NULL, legacy_aux_var = NULL, mindis = NULL,
                maxtry = 10, n_over = NULL, n_near = NULL, wgt_units = NULL,
                pt_density = NULL, DesignID = "Site", SiteBegin = 1, sep = "-", projcrs_check = TRUE) {

  if (inherits(sframe, c("tbl_df", "tbl"))) { # identify if tibble class elements are present
    class(sframe) <- setdiff(class(sframe), c("tbl_df", "tbl"))
    # remove tibble class for rownames warning
  }

  if (!is.null(legacy_sites) & inherits(legacy_sites, c("tbl_df", "tbl"))) { # identify if tibble class elements are present
    class(legacy_sites) <- setdiff(class(legacy_sites), c("tbl_df", "tbl"))
    # remove tibble class for rownames warning
  }

  geom_col_name <- attr(sframe, "sf_column")
  st_geometry(sframe) <- "geometry"
  if (!is.null(legacy_sites)) {
    st_geometry(legacy_sites) <- "geometry"
  }

  # save initial variable specifications for the design list later
  initial_stratum_var <- stratum_var
  initial_caty_var <- caty_var
  initial_aux_var <- aux_var

  # Create warning indicator and data frame to collect all potential issues during
  # sample selection
  warn_ind <- FALSE
  warn_df <- data.frame(stratum = "Stratum", func = "Calling Function", warn = "Message")

  # Ensure that the geometry types for sframe are consistent
  temp <- st_geometry_type(sframe)
  tst <- all(temp %in% c("POINT", "MULTIPOINT")) |
    all(temp %in% c("LINESTRING", "MULTILINESTRING")) |
    all(temp %in% c("POLYGON", "MULTIPOLYGON"))
  if (!tst) {
    stop(paste("\nThe geometry types for the survey frame object passed to function irs: \n\"",
      unique(st_geometry_type(sframe)), "\" are not consistent.",
      sep = ""
    ))
  }

  # Drop m and z values to ensure no issues with grts functionality with sf object
  if (!is.null(st_m_range(sframe)) | !is.null(st_z_range(sframe))) {
    # warn_ind <- TRUE
    # warn_df$warn <- "\nThe survey frame object passed to function grts contains m or z values - they are being dropped to ensure functionality in grts."
    sframe <- st_zm(sframe)
  }

  # Determine type of sample frame: point, line, polygon
  if (all(temp %in% c("POINT", "MULTIPOINT"))) sf_type <- "sf_point"
  if (all(temp %in% c("LINESTRING", "MULTILINESTRING"))) sf_type <- "sf_linear"
  if (all(temp %in% c("POLYGON", "MULTIPOLYGON"))) sf_type <- "sf_area"

  if (all(is.null(legacy_sites), is.null(legacy_var))) {
    legacy_option <- FALSE
  } else {
    legacy_option <- TRUE
  }

  if (is.null(stratum_var)) {
    stratum <- NULL
  } else {
    stratum <- names(n_base)
  }

  # set default seltype if not provided (based on specification of other variables)
  if (is.null(seltype)) {
    if (is.null(caty_var) & is.null(aux_var)) {
      seltype <- "equal"
    } else if (!is.null(caty_var)) {
      seltype <- "unequal"
    } else {
      seltype <- "proportional"
    }
  }

  # check input. If errors, dsgn_check will stop grtspts and report errors.
  dsgn_check(
    sframe = sframe, sf_type = sf_type, legacy_sites = legacy_sites,
    legacy_option = legacy_option, stratum = stratum, seltype = seltype,
    n_base = n_base, caty_n = caty_n, n_over = n_over, n_near = n_near,
    stratum_var = stratum_var, caty_var = caty_var, aux_var = aux_var,
    legacy_stratum_var = legacy_stratum_var, legacy_caty_var = legacy_caty_var,
    legacy_aux_var = legacy_aux_var,
    legacy_var = legacy_var, mindis = mindis, DesignID = DesignID,
    SiteBegin = SiteBegin, maxtry = maxtry, projcrs_check = projcrs_check
  )

  # preserve original sframe names
  sframe_names <- names(sframe)

  # preserve original legacy_sites names if needed
  if (!is.null(legacy_sites)) {
    legacy_sites_names <- names(legacy_sites)
  }

  ## Create variables in sample frame if needed.
  # Create unique sample frame ID values
  sframe$id <- 1:nrow(sframe)

  # Assign stratum variable or create it if design not stratified and variable not provided.
  if (is.null(stratum_var)) {
    stratum_var <- "stratum"
    sframe$stratum <- "None"
    stratum <- c("None") # names of all strata
  } else {
    # ensure class for stratum variable is character and assign to stratum
    sframe$stratum <- as.character(sframe[[stratum_var]])
  }

  # set caty, aux and legacy variables in sample frame if needed
  if (!is.null(caty_var)) sframe$caty <- as.character(sframe[[caty_var]])
  if (!is.null(aux_var)) sframe$aux <- sframe[[aux_var]]
  if (!is.null(legacy_var)) sframe$legacy <- sframe[[legacy_var]]

  # set stratum, caty, aux and legacy variables in legacy_sites if needed
  # add idpts to legacy_sites
  if (legacy_option == TRUE & (sf_type != "sf_point" | ((sf_type == "sf_point") & !is.null(legacy_sites)))) {
    legacy_names <- names(legacy_sites)
    legacy_sites$idpts <- 1:nrow(legacy_sites)
    if (stratum[1] == "None") {
      legacy_sites$stratum <- "None"
    } else {
      if (is.null(legacy_stratum_var)) {
        legacy_stratum_var <- stratum_var
      }
      legacy_sites$stratum <- as.character(legacy_sites[[legacy_stratum_var]])
    }
    if (!is.null(caty_var)) {
      if (is.null(legacy_caty_var)) {
        legacy_caty_var <- caty_var
      }
      legacy_sites$caty <- as.character(legacy_sites[[legacy_caty_var]])
    }
    if (!is.null(aux_var)) {
      if (is.null(legacy_aux_var)) {
        legacy_aux_var <- aux_var
      }
      legacy_sites$aux <- legacy_sites[[legacy_aux_var]]
    }
    if (is.null(legacy_var)) {
      legacy_sites$legacy <- TRUE
      legacy_var <- "legacy"
    } else {
      legacy_sites$legacy <- legacy_sites[[legacy_var]]
    }
  }

  # save initial variable specifications for the design list later
  initial_legacy_stratum_var <- legacy_stratum_var
  initial_legacy_caty_var <- legacy_caty_var
  initial_legacy_aux_var <- legacy_aux_var

  ## Create a dsgn list object
  # variable assignments to dsgn list object
  dsgn <- list(
    stratum_var = stratum_var, caty_var = caty_var, aux_var = aux_var,
    legacy_option = legacy_option, legacy_var = legacy_var, stratum = stratum,
    wgt_units = wgt_units, seltype = NULL, n_base = NULL, caty_n = NULL,
    n_over = NULL, n_near = NULL, mindis = mindis
  )

  # seltype
  if (length(seltype) == length(stratum)) {
    dsgn$seltype <- seltype
    names(dsgn$seltype) <- stratum
  } else {
    tmp <- sapply(stratum, function(x, seltype) {
      x <- seltype
    }, seltype)
    names(tmp) <- stratum
    dsgn$seltype <- tmp
  }

  # n_base
  if (length(n_base) == length(stratum)) {
    dsgn$n_base <- n_base
    names(dsgn$n_base) <- stratum
  } else {
    tmp <- sapply(stratum, function(x, n_base) {
      x <- n_base
    }, n_base)
    names(tmp) <- stratum
    dsgn$n_base <- tmp
  }

  # caty_n
  if (is.list(caty_n)) {
    dsgn$caty_n <- caty_n
  } else {
    tmp <- lapply(stratum, function(x, caty_n) {
      x <- caty_n
    }, caty_n)
    names(tmp) <- stratum
    dsgn$caty_n <- tmp
  }

  # n_over
  if (!is.null(n_over)) {
    if (is.list(n_over)) {
      n_over <- lapply(n_over, function(x) if (all(x == 0)) NULL else x)
      dsgn$n_over <- n_over
    } else if (!is.null(names(n_over)) && all(sort(stratum) == sort(names(n_over)))) {
      tmp <- lapply(stratum, function(x, n_over) {
        x <- n_over[[x]]
      }, n_over)
      names(tmp) <- stratum
      dsgn$n_over <- tmp
    } else {
      tmp <- lapply(stratum, function(x, n_over) {
        x <- n_over
      }, n_over)
      names(tmp) <- stratum
      dsgn$n_over <- tmp
    }
  }

  # n_near
  if (!is.null(n_near)) {
    if (is.list(n_near)) {
      n_near <- lapply(n_near, function(x) if (all(x == 0)) NULL else x)
      dsgn$n_near <- n_near
    } else if (!is.null(names(n_near)) && all(sort(stratum) == sort(names(n_near)))) {
      tmp <- lapply(stratum, function(x, n_near) {
        x <- n_near[[x]]
      }, n_near)
      names(tmp) <- stratum
      dsgn$n_near <- tmp
    } else {
      tmp <- lapply(stratum, function(x, n_near) {
        x <- n_near
      }, n_near)
      names(tmp) <- stratum
      dsgn$n_near <- tmp
    }
  }

  # mindis
  if (!is.null(mindis)) {
    if (is.list(mindis)) {
      mindis <- lapply(mindis, function(x) if (all(x == 0)) NULL else x)
      dsgn$mindis <- mindis
    } else {
      tmp <- lapply(stratum, function(x, mindis) {
        x <- mindis
      }, mindis)
      names(tmp) <- stratum
      dsgn$mindis <- tmp
    }
  }

  # legacy_option
  if (legacy_option == TRUE) {
    tmp <- sapply(stratum, function(x, legacy_option) {
      x <- legacy_option
    }, legacy_option)
    names(tmp) <- stratum
    dsgn$legacy_option <- tmp
  }

  ## select sites for each stratum
  rslts <- lapply(dsgn$stratum, irs_stratum,
    dsgn = dsgn, sframe = sframe, sf_type = sf_type, wgt_units = wgt_units,
    pt_density = pt_density, legacy_option = legacy_option,
    legacy_sites = legacy_sites, maxtry = maxtry,
    warn_ind = warn_ind, warn_df = warn_df
  )
  names(rslts) <- stratum


  # combine across strata
  sites_legacy <- NULL
  sites_base <- NULL
  sites_over <- NULL
  sites_near <- NULL
  warn_ind <- FALSE
  warn_df <- NULL
  for (i in 1:length(rslts)) {
    sites_legacy <- rbind(sites_legacy, rslts[[i]]$sites_legacy)
    sites_base <- rbind(sites_base, rslts[[i]]$sites_base)
    sites_over <- rbind(sites_over, rslts[[i]]$sites_over)
    sites_near <- rbind(sites_near, rslts[[i]]$sites_near)
    if (rslts[[i]]$warn_ind) {
      warn_ind <- TRUE
      warn_df <- rbind(warn_df, rslts[[i]]$warn_df)
    }
  }

  # spelling change to avoid clash with analysis warnings
  if (!is.null(warn_df) && "warning" %in% names(warn_df)) {
    names(warn_df)[which(names(warn_df) == "warning")] <- "Warning"
  }

  # Create a siteID for all sites
  ntot <- NROW(sites_legacy) + NROW(sites_base) + NROW(sites_over) + NROW(sites_near)
  siteID <- gsub(" ", "0", paste0(DesignID, sep, format(SiteBegin - 1 + 1:ntot, sep = "")))
  nlast <- 0

  # Create siteID for legacy sites if present using DesignID and SiteBegin
  if (!is.null(sites_legacy)) {
    row.names(sites_legacy) <- 1:nrow(sites_legacy)
    sites_legacy$siteID <- siteID[1:nrow(sites_legacy)]
    nlast <- nrow(sites_legacy)
    # set siteuse and replsite
    sites_legacy$siteuse <- "Legacy"
    sites_legacy$replsite <- "None"
  }

  # Create siteID for base sites using DesignID and SiteBegin
  if (!is.null(sites_base)) {
    row.names(sites_base) <- 1:nrow(sites_base)
    sites_base$siteID <- siteID[(nlast + 1):(nlast + nrow(sites_base))]
    nlast <- nlast + nrow(sites_base)
    # set siteuse and replsite for base sites
    sites_base$siteuse <- "Base"
    sites_base$replsite <- "None"
  }

  # create siteID for n_over sites if any
  if (!is.null(n_over)) {
    row.names(sites_over) <- 1:nrow(sites_over)
    sites_over$siteID <- siteID[(nlast + 1):(nlast + nrow(sites_over))]
    nlast <- nlast + nrow(sites_over)
    # set siteuse and replsite for n_over sites
    sites_over$siteuse <- "Over"
    sites_over$replsite <- "Next"
  }

  # if n_near sample sites, assign base ids to the replacement sites. then add siteIDs
  if (!is.null(n_near)) {
    tst <- match(paste(sites_near$stratum, sites_near$replsite, sep = "_"),
      paste(sites_legacy$stratum, sites_legacy$idpts, sep = "_"),
      nomatch = 0
    )
    sites_near$replsite[tst > 0] <- sites_legacy$siteID[tst]

    tst <- match(paste(sites_near$stratum, sites_near$replsite, sep = "_"),
      paste(sites_base$stratum, sites_base$idpts, sep = "_"),
      nomatch = 0
    )
    sites_near$replsite[tst > 0] <- sites_base$siteID[tst]
    tst <- match(paste(sites_near$stratum, sites_near$replsite, sep = "_"),
      paste(sites_over$stratum, sites_over$idpts, sep = "_"),
      nomatch = 0
    )
    sites_near$replsite[tst > 0] <- sites_over$siteID[tst]

    # sort by id so that sites_near in same order as sites in sites_base and sites_over
    sites_near <- sites_near[order(sites_near$replsite, sites_near$siteuse), ]
    row.names(sites_near) <- 1:nrow(sites_near)
    # assign siteIDs
    sites_near$siteID <- siteID[(nlast + 1):(nlast + nrow(sites_near))]
  }

  # Add lat/lon in WGS84 or X/Y coordinates
  if (is.na(st_crs(sites_base)) | st_is_longlat(sites_base)) {
    if (!is.null(sites_legacy)) {
      sites_legacy$X <- st_coordinates(sites_legacy)[, "X"]
      sites_legacy$Y <- st_coordinates(sites_legacy)[, "Y"]
    }

    if (!is.null(sites_base)) {
      sites_base$X <- st_coordinates(sites_base)[, "X"]
      sites_base$Y <- st_coordinates(sites_base)[, "Y"]
    }

    if (!is.null(sites_over)) {
      sites_over$X <- st_coordinates(sites_over)[, "X"]
      sites_over$Y <- st_coordinates(sites_over)[, "Y"]
    }

    if (!is.null(sites_near)) {
      sites_near$X <- st_coordinates(sites_near)[, "X"]
      sites_near$Y <- st_coordinates(sites_near)[, "Y"]
    }

    # reorder sf object variables by first specifying design names excluding unique
    # feature ID id and idpts as they are internal
    dsgn_names <- c(
      "siteID", "siteuse", "replsite", "X", "Y",
      "stratum", "wgt", "ip", "caty", "aux"
    )
  } else {
    if (!is.null(sites_legacy)) {
      sites_legacy$lon_WGS84 <- st_coordinates(st_transform(sites_legacy, crs = 4326))[, "X"]
      sites_legacy$lat_WGS84 <- st_coordinates(st_transform(sites_legacy, crs = 4326))[, "Y"]
    }

    if (!is.null(sites_base)) {
      sites_base$lon_WGS84 <- st_coordinates(st_transform(sites_base, crs = 4326))[, "X"]
      sites_base$lat_WGS84 <- st_coordinates(st_transform(sites_base, crs = 4326))[, "Y"]
    }

    if (!is.null(sites_over)) {
      sites_over$lon_WGS84 <- st_coordinates(st_transform(sites_over, crs = 4326))[, "X"]
      sites_over$lat_WGS84 <- st_coordinates(st_transform(sites_over, crs = 4326))[, "Y"]
    }

    if (!is.null(sites_near)) {
      sites_near$lon_WGS84 <- st_coordinates(st_transform(sites_near, crs = 4326))[, "X"]
      sites_near$lat_WGS84 <- st_coordinates(st_transform(sites_near, crs = 4326))[, "Y"]
    }

    # reorder sf object variables by first specifying design names excluding unique
    # feature ID id and idpts as they are internal
    dsgn_names <- c(
      "siteID", "siteuse", "replsite", "lon_WGS84", "lat_WGS84",
      "stratum", "wgt", "ip", "caty", "aux"
    )
  }

  dsgn_names_extra <- c(dsgn_names, "xcoord", "ycoord", "idpts")

  if (geom_col_name != "geometry") {
    # sframe prefix if necessary
    if (geom_col_name %in% dsgn_names_extra) {
      geom_col_name <- paste("sframe", geom_col_name, sep = "_")
    }

    # restore original column names
    if (!is.null(sites_legacy)) {
      st_geometry(sites_legacy) <- geom_col_name
      legacy_sites_names[legacy_sites_names == "geometry"] <- geom_col_name
      st_geometry(legacy_sites) <- geom_col_name
    }

    if (!is.null(sites_base)) {
      sframe_names[sframe_names == "geometry"] <- geom_col_name
      st_geometry(sites_base) <- geom_col_name
    }

    if (!is.null(sites_over)) {
      st_geometry(sites_over) <- geom_col_name
    }

    if (!is.null(sites_near)) {
      st_geometry(sites_near) <- geom_col_name
    }
  }

  # sites_legacy
  if (!is.null(sites_legacy)) {
    if (sf_type != "sf_point") {
      add_names <- dsgn_names[dsgn_names %in% names(sites_legacy)]
      legacy_sites_names_good <- legacy_sites_names[!legacy_sites_names %in% dsgn_names_extra]
      if (all(legacy_sites_names %in% legacy_sites_names_good)) {
        sites_legacy <- subset(sites_legacy, select = c(add_names, legacy_sites_names))
      } else {
        legacy_sites_names_bad <- legacy_sites_names[legacy_sites_names %in% dsgn_names_extra]
        legacy_sites_temp <- legacy_sites[, legacy_sites_names_bad, drop = FALSE]
        temp_geometry_col <- which(names(legacy_sites_temp) == attr(legacy_sites_temp, "sf_column"))
        legacy_sites_geometry_col <- which(names(legacy_sites) == attr(legacy_sites, "sf_column"))
        names(legacy_sites_temp)[-temp_geometry_col] <- paste("legacy_sites", names(legacy_sites_temp)[-temp_geometry_col], sep = "_")
        sites_legacy <- st_join(sites_legacy, legacy_sites_temp, join = st_nearest_feature)
        sites_legacy <- subset(sites_legacy, select = c(add_names, legacy_sites_names_good[-legacy_sites_geometry_col], names(legacy_sites_temp)[-temp_geometry_col]))
        for (i in names(sites_legacy)) {
          if (i %in% c("legacy_sites_xcoord", "legacy_sites_ycoord", "legacy_sites_idpts")) {
            names(sites_legacy)[which(names(sites_legacy) == i)] <- substring(i, first = 14)
          }
        }
      }
    }
    if (sf_type == "sf_point") {
      add_names <- dsgn_names[dsgn_names %in% names(sites_legacy)]
      sframe_names_good <- sframe_names[!sframe_names %in% dsgn_names_extra]
      if (all(sframe_names %in% sframe_names_good)) {
        sites_legacy <- subset(sites_legacy, select = c(add_names, sframe_names))
      } else {
        sframe_names_bad <- sframe_names[sframe_names %in% dsgn_names_extra]
        sframe_temp <- sframe[, sframe_names_bad, drop = FALSE]
        temp_geometry_col <- which(names(sframe_temp) == attr(sframe_temp, "sf_column"))
        sframe_geometry_col <- which(names(sframe) == attr(sframe, "sf_column"))
        names(sframe_temp)[-temp_geometry_col] <- paste("sframe", names(sframe_temp)[-temp_geometry_col], sep = "_")
        sites_legacy <- st_join(sites_legacy, sframe_temp, join = st_nearest_feature)
        sites_legacy <- subset(sites_legacy,
          select = c(add_names, sframe_names_good[-sframe_geometry_col], names(sframe_temp)[-temp_geometry_col])
        )
        for (i in names(sites_legacy)) {
          if (i %in% c("sframe_xcoord", "sframe_ycoord", "sframe_idpts")) {
            names(sites_legacy)[which(names(sites_legacy) == i)] <- substring(i, first = 8)
          }
        }
      }
    }
  }

  # sites_base
  # check what design variables are present in sf objects and add if missing
  if (!is.null(sites_base)) {
    add_names <- dsgn_names[dsgn_names %in% names(sites_base)]
    sframe_names_good <- sframe_names[!sframe_names %in% dsgn_names_extra]
    if (all(sframe_names %in% sframe_names_good)) {
      sites_base <- subset(sites_base, select = c(add_names, sframe_names))
    } else {
      sframe_names_bad <- sframe_names[sframe_names %in% dsgn_names_extra]
      sframe_temp <- sframe[, sframe_names_bad, drop = FALSE]
      temp_geometry_col <- which(names(sframe_temp) == attr(sframe_temp, "sf_column"))
      sframe_geometry_col <- which(names(sframe) == attr(sframe, "sf_column"))
      names(sframe_temp)[-temp_geometry_col] <- paste("sframe", names(sframe_temp)[-temp_geometry_col], sep = "_")
      sites_base <- st_join(sites_base, sframe_temp, join = st_nearest_feature)
      sites_base <- subset(sites_base, select = c(add_names, sframe_names_good[-sframe_geometry_col], names(sframe_temp)[-temp_geometry_col]))
      for (i in names(sites_base)) {
        if (i %in% c("sframe_xcoord", "sframe_ycoord", "sframe_idpts")) {
          names(sites_base)[which(names(sites_base) == i)] <- substring(i, first = 8)
        }
      }
    }
  }

  # sites_over
  if (!is.null(sites_over)) {
    add_names <- dsgn_names[dsgn_names %in% names(sites_over)]
    sframe_names_good <- sframe_names[!sframe_names %in% dsgn_names_extra]
    if (all(sframe_names %in% sframe_names_good)) {
      sites_over <- subset(sites_over, select = c(add_names, sframe_names))
    } else {
      sframe_names_bad <- sframe_names[sframe_names %in% dsgn_names_extra]
      sframe_temp <- sframe[, sframe_names_bad, drop = FALSE]
      temp_geometry_col <- which(names(sframe_temp) == attr(sframe_temp, "sf_column"))
      sframe_geometry_col <- which(names(sframe) == attr(sframe, "sf_column"))
      names(sframe_temp)[-temp_geometry_col] <- paste("sframe", names(sframe_temp)[-temp_geometry_col], sep = "_")
      sites_over <- st_join(sites_over, sframe_temp, join = st_nearest_feature)
      sites_over <- subset(sites_over, select = c(add_names, sframe_names_good[-sframe_geometry_col], names(sframe_temp)[-temp_geometry_col]))
      for (i in names(sites_over)) {
        if (i %in% c("sframe_xcoord", "sframe_ycoord", "sframe_idpts")) {
          names(sites_over)[which(names(sites_over) == i)] <- substring(i, first = 8)
        }
      }
    }
  }

  # sites_near
  if (!is.null(sites_near)) {
    add_names <- dsgn_names[dsgn_names %in% names(sites_near)]
    sframe_names_good <- sframe_names[!sframe_names %in% dsgn_names_extra]
    if (all(sframe_names %in% sframe_names_good)) {
      sites_near <- subset(sites_near, select = c(add_names, sframe_names))
    } else {
      sframe_names_bad <- sframe_names[sframe_names %in% dsgn_names_extra]
      sframe_temp <- sframe[, sframe_names_bad, drop = FALSE]
      temp_geometry_col <- which(names(sframe_temp) == attr(sframe_temp, "sf_column"))
      sframe_geometry_col <- which(names(sframe) == attr(sframe, "sf_column"))
      names(sframe_temp)[-temp_geometry_col] <- paste("sframe", names(sframe_temp)[-temp_geometry_col], sep = "_")
      sites_near <- st_join(sites_near, sframe_temp, join = st_nearest_feature)
      sites_near <- subset(sites_near, select = c(add_names, sframe_names_good[-sframe_geometry_col], names(sframe_temp)[-temp_geometry_col]))
      for (i in names(sites_near)) {
        if (i %in% c("sframe_xcoord", "sframe_ycoord", "sframe_idpts")) {
          names(sites_near)[which(names(sites_near) == i)] <- substring(i, first = 8)
        }
      }
    }
  }

  # add function call to dsgn list
  # dsgn <- c(list(Call = match.call()), dsgn)
  dsgn <- list(
    call = match.call(), stratum_var = initial_stratum_var, stratum = dsgn$stratum, n_base = dsgn$n_base,
    seltype = dsgn$seltype, caty_var = initial_caty_var, caty_n = dsgn$caty_n, aux_var = initial_aux_var, legacy = dsgn$legacy_option,
    mindis = dsgn$mindis, n_over = dsgn$n_over, n_near = dsgn$n_near
  )

  if (any(dsgn$legacy)) {
    dsgn <- c(dsgn, list(legacy_stratum_var = initial_legacy_stratum_var, legacy_caty_var = initial_legacy_caty_var,
                         legacy_aux_var = initial_legacy_aux_var))
    dsgn <- dsgn[c("call", "stratum_var", "stratum", "n_base", "seltype", "caty_var",
                   "caty_n", "aux_var", "legacy", "legacy_stratum_var", "legacy_caty_var", "legacy_aux_var",
                   "mindis", "n_over", "n_near")]
  }

  # create output list
  sites <- list(
    sites_legacy = sites_legacy, sites_base = sites_base,
    sites_over = sites_over, sites_near = sites_near,
    design = dsgn
  )

  # As necessary, output a message indicating that warning messages were generated
  # during execution of the program

  if (warn_ind) {
    warn_df <<- warn_df
    if (nrow(warn_df) == 1) {
      message("During execution of the program, a warning message was generated. The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
    } else {
      message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
    }
  }

  # constructor for design class
  sites <- structure(sites, class = "sp_design")

  # return the survey design sf object
  invisible(sites)
}

context("irs")

# find system info
on_solaris <- Sys.info()[["sysname"]] == "SunOS"
if (on_solaris) {
  test_that("on solaris", {
    expect_true(on_solaris)
  })
} else {
  
  
  # set reproducible seed (as there are random components here)
  set.seed(5)
  
  test_local <- FALSE # FALSE for CRAN
  
  #################################################
  ########### NE_LAKES DATA TESTS
  #################################################
  
  #--------------------------------------
  #-------- Class Inheritance
  #--------------------------------------
  
  test_that("algorithm executes", {
    n_base <- 50
    irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "equal")
    # class inheritance
    expect_s3_class(irs_output, "sp_design")
  })
  
  #--------------------------------------
  #-------- Work with sp_frame
  #--------------------------------------
  
  test_that("algorithm executes", {
    n_base <- 50
    irs_output <- irs(sp_frame(NE_Lakes), n_base = n_base, seltype = "equal")
    # class inheritance
    expect_s3_class(irs_output, "sp_design")
  })
  
  if (test_local) {
    #--------------------------------------
    #-------- Regular
    #--------------------------------------
    
    # number of irs columns added
    col_irs_add <- 9
    
    # number of NE_Lakes columns
    col_data <- NCOL(NE_Lakes)
    
    # number of irs columns plus NE_Lakes columns
    col_out <- col_irs_add + col_data
    
    # unstratified, equal probability
    test_that("algorithm executes", {
      n_base <- 50
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "equal")
      # see if function ran without error
      expect_true(exists("irs_output"))
      # no legacy sites
      expect_equal(NROW(irs_output$sites_legacy), 0)
      # base sample size of 50
      expect_equal(NROW(irs_output$sites_base), n_base)
      # no rho replacement sites
      expect_equal(NROW(irs_output$sites_over), 0)
      # no nn replacement sites
      expect_equal(NROW(irs_output$sites_near), 0)
      # no legacy sites
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      # base sample size columns should equal extra columns plus original columns
      expect_equal(NCOL(irs_output$sites_base), col_out)
      # no rho replacement sites
      expect_equal(NCOL(irs_output$sites_over), 1)
      # no nn replacement sites
      expect_equal(NCOL(irs_output$sites_near), 1)
      # class inheritance
      expect_s3_class(irs_output, "sp_design")
    })
    
    # stratified, equal probability
    test_that("algorithm executes", {
      n_base <- c(low = 20, high = 30)
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "equal", stratum_var = "ELEV_CAT")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "low", , drop = FALSE]),
        n_base[["low"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "high", , drop = FALSE]),
        n_base[["high"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # unstratified, unequal probability
    test_that("algorithm executes", {
      n_base <- 50
      caty_n <- c(small = 24, large = 26)
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "unequal", caty_var = "AREA_CAT", caty_n = caty_n)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratified, unequal probability
    test_that("algorithm executes", {
      n_base <- c(low = 20, high = 30)
      caty_n <- list(low = c(small = 10, large = 10), high = c(small = 10, large = 20))
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "unequal", stratum_var = "ELEV_CAT",
                        caty_var = "AREA_CAT", caty_n = caty_n
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "low", , drop = FALSE]),
        n_base[["low"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "high", , drop = FALSE]),
        n_base[["high"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratified, unequal probability (with repeated caty_n)
    test_that("algorithm executes", {
      n_base <- c(low = 25, high = 25)
      caty_n <- c(small = 12.5, large = 12.5)
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "unequal", stratum_var = "ELEV_CAT",
                        caty_var = "AREA_CAT", caty_n = caty_n
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "low", , drop = FALSE]),
        n_base[["low"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "high", , drop = FALSE]),
        n_base[["high"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratified, unequal probability (with different caty_n)
    test_that("algorithm executes", {
      n_base <- c(low = 25, high = 25)
      caty_n <- list(low = c(small = 10, large = 15), high = c(small = 12, large = 13))
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "unequal", stratum_var = "ELEV_CAT",
                        caty_var = "AREA_CAT", caty_n = caty_n
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "low", , drop = FALSE]),
        n_base[["low"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "high", , drop = FALSE]),
        n_base[["high"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # unstratified, proportional (to size) probability
    test_that("algorithm executes", {
      n_base <- 50
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "proportional", aux_var = "AREA")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out + 1)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratified, proportional probability
    test_that("algorithm executes", {
      n_base <- c(low = 20, high = 30)
      irs_output <- irs(NE_Lakes, n_base = n_base, stratum_var = "ELEV_CAT", aux_var = "AREA")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "low", , drop = FALSE]),
        n_base[["low"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "high", , drop = FALSE]),
        n_base[["high"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out + 1)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    test_that("A warning (in message form) is produced", {
      n_base <- c(low = 20, high = 30)
      expect_message(expect_error(irs(NE_Lakes, n_base = n_base, stratum_var = "XYZ")))
      
    })
    
    #--------------------------------------
    #-------- Legacy
    #--------------------------------------
    
    # legacy sites, unstratified, equal probability
    test_that("algorithm executes", {
      n_base <- 50
      n_legacy <- NROW(NE_Lakes_Legacy)
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "equal", legacy_sites = NE_Lakes_Legacy)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), n_legacy)
      expect_equal(NROW(irs_output$sites_base), n_base - n_legacy)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), col_out)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # legacy sites, stratified, equal probability
    test_that("algorithm executes", {
      n_base <- c(low = 20, high = 30)
      n_legacy <- NROW(NE_Lakes_Legacy)
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "equal",
                        stratum_var = "ELEV_CAT", legacy_sites = NE_Lakes_Legacy,
                        legacy_stratum_var = "ELEV_CAT"
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), n_legacy)
      n_legacy_low <- sum(irs_output$sites_legacy$stratum == "low")
      n_legacy_high <- sum(irs_output$sites_legacy$stratum == "high")
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "low", , drop = FALSE]),
        n_base[["low"]] - n_legacy_low
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "high", , drop = FALSE]),
        n_base[["high"]] - n_legacy_high
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base) - n_legacy)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), col_out)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # legacy sites, unequal probability
    test_that("algorithm executes", {
      n_base <- 50
      caty_n <- c(small = 24, large = 26)
      n_legacy <- NROW(NE_Lakes_Legacy)
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "unequal",
                        caty_var = "AREA_CAT", caty_n = caty_n, legacy_sites = NE_Lakes_Legacy,
                        legacy_caty_var = "AREA_CAT"
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), n_legacy)
      expect_equal(NROW(irs_output$sites_base), n_base - n_legacy)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), col_out)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # legacy sites, proportional probability
    test_that("algorithm executes", {
      n_base <- 50
      n_legacy <- NROW(NE_Lakes_Legacy)
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "proportional",
                        aux_var = "AREA", legacy_sites = NE_Lakes_Legacy,
                        legacy_aux_var = "AREA"
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), n_legacy)
      expect_equal(NROW(irs_output$sites_base), n_base - n_legacy)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), col_out + 1)
      expect_equal(NCOL(irs_output$sites_base), col_out + 1)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # legacy sites, unstratified, equal probability -- old method
    test_that("algorithm executes", {
      n_base <- 50
      n_legacy <- NROW(NE_Lakes_Legacy)
      NE_Lakes$LEGACY <- NA
      NE_Lakes_Legacy$LEGACY <- paste0("LEGACY-SITES-", 1:5)
      NE_Lakes_bind <- rbind(NE_Lakes_Legacy, NE_Lakes)
      irs_output <- irs(NE_Lakes_bind, n_base = n_base, seltype = "equal", legacy_var = "LEGACY")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), n_legacy)
      expect_equal(NROW(irs_output$sites_base), n_base - n_legacy)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), col_out + 1) # as legacy variable added
      expect_equal(NCOL(irs_output$sites_base), col_out + 1) # as legacy variable added
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    #--------------------------------------
    #-------- Minimum Distance
    #--------------------------------------
    
    # minimum distance, unstratified, equal probability
    test_that("algorithm executes", {
      library(sf)
      n_base <- 50
      mindis <- 1600
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "equal", mindis = mindis)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
      dist_mx <- as.vector(st_distance(irs_output$sites_base))
      expect_true(min(dist_mx[dist_mx > 0]) > mindis)
    })
    
    #--------------------------------------
    #-------- RHO replacement
    #--------------------------------------
    
    # rho replacement sites, unstratified, equal probability
    test_that("algorithm executes", {
      n_base <- 50
      n_over <- 5
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "equal", n_over = n_over)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), n_over)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), col_out)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # rho replacement sites, stratified, equal probability
    test_that("algorithm executes", {
      n_base <- c(low = 20, high = 30)
      n_over <- list(low = 2, high = 3)
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "equal",
                        stratum_var = "ELEV_CAT", n_over = n_over
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "low", , drop = FALSE]),
        n_base[["low"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "high", , drop = FALSE]),
        n_base[["high"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(
        NROW(irs_output$sites_over[irs_output$sites_over$stratum == "low", , drop = FALSE]),
        n_over[["low"]]
      )
      expect_equal(
        NROW(irs_output$sites_over[irs_output$sites_over$stratum == "high", , drop = FALSE]),
        n_over[["high"]]
      )
      expect_equal(NROW(irs_output$sites_over), sum(unlist(n_over)))
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), col_out)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # rho replacement sites, unstratified, unequal probability
    test_that("algorithm executes", {
      n_base <- 50
      caty_n <- c(small = 24, large = 26)
      n_over <- 10
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "unequal",
                        caty_var = "AREA_CAT", caty_n = caty_n, n_over = n_over
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), n_over)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), col_out)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # rho replacement sites, unstratified, proportional probability
    test_that("algorithm executes", {
      n_base <- 50
      caty_n <- c(small = 24, large = 26)
      n_over <- 10
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "proportional",
                        aux_var = "AREA", n_over = n_over
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), n_over)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out + 1)
      expect_equal(NCOL(irs_output$sites_over), col_out + 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratification and n_over
    test_that("algorithm executes", {
      n_base <- c(low = 5, high = 6)
      stratum_var <- "ELEV_CAT"
      caty_n <- list(low = c(small = 2, large = 3), high = c(small = 3, large = 3))
      caty_var <- "AREA_CAT"
      n_over <- c(low = 4, high = 3)
      irs_output <- irs(NE_Lakes, n_base, stratum_var, caty_n = caty_n, caty_var = caty_var, n_over = n_over)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), sum(n_over))
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), col_out)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    #--------------------------------------
    #-------- NN replacement
    #--------------------------------------
    
    # nn replacement sites, unstratified, equal probability
    test_that("algorithm executes", {
      n_base <- 50
      n_near <- 2
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "equal", n_near = n_near)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), n_base * n_near)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), col_out)
    })
    
    # nn replacement sites, stratified, equal probability
    test_that("algorithm executes", {
      n_base <- c(low = 20, high = 30)
      n_near <- 2
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "equal",
                        stratum_var = "ELEV_CAT", n_near = n_near
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "low", , drop = FALSE]),
        n_base[["low"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "high", , drop = FALSE]),
        n_base[["high"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), n_near * sum(n_base))
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), col_out)
    })
    
    # nn replacement sites, unstratified, unequal probability
    test_that("algorithm executes", {
      n_base <- 50
      caty_n <- c(small = 24, large = 26)
      n_near <- 2
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "unequal",
                        caty_var = "AREA_CAT", caty_n = caty_n, n_near = n_near
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), n_base * n_near)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), col_out)
    })
    
    # nn replacement sites, unstratified, proportional probability
    test_that("algorithm executes", {
      n_base <- 50
      caty_n <- c(small = 24, large = 26)
      n_near <- 2
      irs_output <- irs(NE_Lakes,
                        n_base = n_base, seltype = "proportional",
                        aux_var = "AREA", n_near = n_near
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), n_base * n_near)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out + 1)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), col_out + 1)
    })
    
    # stratification and n_near
    test_that("algorithm executes", {
      n_base <- c(low = 5, high = 6)
      stratum_var <- "ELEV_CAT"
      n_near <- c(low = 2, high = 1)
      irs_output <- irs(NE_Lakes, n_base, stratum_var, n_near = n_near)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), sum(n_base * n_near))
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), col_out)
    })
    
    #--------------------------------------
    #-------- RHO and NN replacement
    #--------------------------------------
    
    # both replacement sites, unstratified
    test_that("algorithm executes", {
      n_base <- 50
      n_over <- 5
      n_near <- 2
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "equal", n_over = n_over, n_near = n_near)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), n_over)
      expect_equal(NROW(irs_output$sites_near), (n_base + n_over) * n_near)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), col_out)
      expect_equal(NCOL(irs_output$sites_near), col_out)
    })
    
    #--------------------------------------
    #-------- Bad name replacement
    #--------------------------------------
    
    test_that("algorithm executes", {
      n_legacy <- NROW(NE_Lakes_Legacy)
      n_base <- 50
      n_over <- 5
      n_near <- 2
      NE_Lakes$siteID <- seq_len(nrow(NE_Lakes))
      irs_output <- irs(NE_Lakes, n_base = n_base, seltype = "equal", legacy_sites = NE_Lakes_Legacy, n_over = n_over, n_near = n_near)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), n_legacy)
      expect_equal(NROW(irs_output$sites_base), n_base - n_legacy)
      expect_equal(NROW(irs_output$sites_over), n_over)
      # used to be n_base - n_legacy + n_over but made legacy sites have nn sites
      expect_equal(NROW(irs_output$sites_near), (n_base + n_over) * n_near)
      expect_equal(NCOL(irs_output$sites_legacy), col_out + 1)
      expect_equal(NCOL(irs_output$sites_base), col_out + 1)
      expect_equal(NCOL(irs_output$sites_over), col_out + 1)
      expect_equal(NCOL(irs_output$sites_near), col_out + 1)
    })
    
    #--------------------------------------
    #-------- Projected CRS
    #--------------------------------------
    
    # unstratified, equal probability
    test_that("algorithm executes", {
      n_base <- 50
      irs_output <- irs(st_transform(NE_Lakes, 4326), n_base = n_base, seltype = "equal", projcrs_check = FALSE)
      # see if function ran without error
      expect_true(exists("irs_output"))
      # no legacy sites
      expect_equal(NROW(irs_output$sites_legacy), 0)
      # base sample size of 50
      expect_equal(NROW(irs_output$sites_base), n_base)
      # no rho replacement sites
      expect_equal(NROW(irs_output$sites_over), 0)
      # no nn replacement sites
      expect_equal(NROW(irs_output$sites_near), 0)
      # no legacy sites
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      # base sample size columns should equal extra columns plus original columns
      expect_equal(NCOL(irs_output$sites_base), col_out)
      # no rho replacement sites
      expect_equal(NCOL(irs_output$sites_over), 1)
      # no nn replacement sites
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    #################################################
    ########### Illinois_River DATA TESTS
    #################################################
    
    # number of irs columns added
    col_irs_add <- 9
    
    # number of Illinois_River columns
    col_data <- NCOL(Illinois_River)
    
    # number of irs columns plus Illinois_River columns
    col_out <- col_irs_add + col_data
    
    #--------------------------------------
    #-------- Regular
    #--------------------------------------
    
    # unstratified, equal probability
    test_that("algorithm executes", {
      n_base <- 50
      irs_output <- irs(Illinois_River, n_base = n_base, seltype = "equal")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # unstratified, large sample size
    test_that("algorithm executes", {
      n_base <- 500
      irs_output <- irs(Illinois_River, n_base = n_base, seltype = "equal")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # unstratified, large sample size, replacement sites
    test_that("algorithm executes", {
      n_base <- 50
      n_over <- 200
      irs_output <- irs(Illinois_River, n_base = n_base, n_over = n_over, seltype = "equal")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), n_over)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), col_out)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratified, equal probability
    test_that("algorithm executes", {
      n_base <- c(Oklahoma = 20, Arkansas = 30)
      irs_output <- irs(Illinois_River, n_base = n_base, seltype = "equal", stratum_var = "STATE_NAME")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "Oklahoma", , drop = FALSE]),
        n_base[["Oklahoma"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "Arkansas", , drop = FALSE]),
        n_base[["Arkansas"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratified, equal probability
    test_that("algorithm executes", {
      n_base <- c(Oklahoma = 200, Arkansas = 300)
      irs_output <- irs(Illinois_River, n_base = n_base, seltype = "equal", stratum_var = "STATE_NAME")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "Oklahoma", , drop = FALSE]),
        n_base[["Oklahoma"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "Arkansas", , drop = FALSE]),
        n_base[["Arkansas"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratified, equal probability
    test_that("algorithm executes", {
      n_base <- c(Oklahoma = 20, Arkansas = 30)
      n_over <- list(Oklahoma = 200, Arkansas = 300)
      irs_output <- irs(Illinois_River, n_base = n_base, seltype = "equal", stratum_var = "STATE_NAME", n_over = n_over)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "Oklahoma", , drop = FALSE]),
        n_base[["Oklahoma"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "Arkansas", , drop = FALSE]),
        n_base[["Arkansas"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), sum(unlist(n_over)))
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), col_out)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    #--------------------------------------
    #-------- Legacy
    #--------------------------------------
    
    # legacy sites, unstratified, equal probability
    test_that("algorithm executes", {
      n_base <- 50
      n_legacy <- nrow(Illinois_River_Legacy)
      irs_output <- irs(Illinois_River, n_base = n_base, seltype = "equal", legacy_sites = Illinois_River_Legacy)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), n_legacy)
      expect_equal(NROW(irs_output$sites_base), n_base - n_legacy)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), col_out)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # legacy sites, stratified, equal probability
    test_that("algorithm executes", {
      n_base <- c(Oklahoma = 20, Arkansas = 30)
      n_legacy <- nrow(Illinois_River_Legacy)
      irs_output <- irs(Illinois_River,
                        n_base = n_base, seltype = "equal",
                        stratum_var = "STATE_NAME", legacy_sites = Illinois_River_Legacy,
                        legacy_stratum_var = "STATE_NAME"
      )
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), n_legacy)
      n_legacy_Oklahoma <- sum(irs_output$sites_legacy$stratum == "Oklahoma")
      n_legacy_Arkansas <- sum(irs_output$sites_legacy$stratum == "Arkansas")
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "Oklahoma", , drop = FALSE]),
        n_base[["Oklahoma"]] - n_legacy_Oklahoma
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "Arkansas", , drop = FALSE]),
        n_base[["Arkansas"]] - n_legacy_Arkansas
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base) - n_legacy)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), col_out)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    #################################################
    ########### Lake_Ontario DATA TESTS
    #################################################
    
    # number of irs columns added
    col_irs_add <- 9
    
    # number of Lake_Ontario columns
    col_data <- NCOL(Lake_Ontario)
    
    # number of irs columns plus Lake_Ontario columns
    col_out <- col_irs_add + col_data
    
    #--------------------------------------
    #-------- Regular
    #--------------------------------------
    
    # unstratified, equal probability
    test_that("algorithm executes", {
      n_base <- 50
      irs_output <- irs(Lake_Ontario, n_base = n_base, seltype = "equal")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # unstratified, large sample size
    test_that("algorithm executes", {
      n_base <- 500
      irs_output <- irs(Lake_Ontario, n_base = n_base, seltype = "equal")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # unstratified, large sample size, replacement sites
    test_that("algorithm executes", {
      n_base <- 50
      n_over <- 200
      irs_output <- irs(Lake_Ontario, n_base = n_base, n_over = n_over, seltype = "equal")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(NROW(irs_output$sites_base), n_base)
      expect_equal(NROW(irs_output$sites_over), n_over)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), col_out)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratified, equal probability
    test_that("algorithm executes", {
      n_base <- c(CAN = 20, USA = 30)
      irs_output <- irs(Lake_Ontario, n_base = n_base, seltype = "equal", stratum_var = "COUNTRY")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "CAN", , drop = FALSE]),
        n_base[["CAN"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "USA", , drop = FALSE]),
        n_base[["USA"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # stratified, large sample size
    test_that("algorithm executes", {
      n_base <- c(CAN = 200, USA = 300)
      irs_output <- irs(Lake_Ontario, n_base = n_base, seltype = "equal", stratum_var = "COUNTRY")
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "CAN", , drop = FALSE]),
        n_base[["CAN"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "USA", , drop = FALSE]),
        n_base[["USA"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), 0)
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), 1)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
    
    # replacement sites
    test_that("algorithm executes", {
      n_base <- c(CAN = 200, USA = 300)
      n_over <- list(CAN = 100, USA = 100)
      irs_output <- irs(Lake_Ontario, n_base = n_base, seltype = "equal", stratum_var = "COUNTRY", n_over = n_over)
      expect_true(exists("irs_output"))
      expect_equal(NROW(irs_output$sites_legacy), 0)
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "CAN", , drop = FALSE]),
        n_base[["CAN"]]
      )
      expect_equal(
        NROW(irs_output$sites_base[irs_output$sites_base$stratum == "USA", , drop = FALSE]),
        n_base[["USA"]]
      )
      expect_equal(NROW(irs_output$sites_base), sum(n_base))
      expect_equal(NROW(irs_output$sites_over), sum(unlist(n_over)))
      expect_equal(NROW(irs_output$sites_near), 0)
      expect_equal(NCOL(irs_output$sites_legacy), 1)
      expect_equal(NCOL(irs_output$sites_base), col_out)
      expect_equal(NCOL(irs_output$sites_over), col_out)
      expect_equal(NCOL(irs_output$sites_near), 1)
    })
  }
}

context("print")

# find system info
on_solaris <- Sys.info()[["sysname"]] == "SunOS"
if (on_solaris) {
  test_that("on solaris", {
    expect_true(on_solaris)
  })
} else {
  # sample frame
  expect_output(print(sp_summary(NE_Lakes, ~ ELEV_CAT)))
  expect_output(print(sp_summary(NE_Lakes, AREA_CAT ~ ELEV_CAT)))
  expect_output(print(summary(sp_frame(NE_Lakes), ~ ELEV_CAT)))
  expect_output(print(summary(sp_frame(NE_Lakes), AREA_CAT ~ ELEV_CAT)))
  
  # design frame
  expect_output(print(sp_summary(NLA_PNW, ~ BMMI_COND)))
  expect_output(print(sp_summary(NLA_PNW, PHOS_COND ~ BMMI_COND)))
  expect_output(print(summary(sp_frame(NLA_PNW), ~ BMMI_COND)))
  expect_output(print(summary(sp_frame(NLA_PNW), PHOS_COND ~ BMMI_COND)))
  
  # grts output
  grts_output <- grts(NE_Lakes, 50)
  expect_output(print(sp_summary(grts_output, ~ ELEV_CAT)))
  expect_output(print(sp_summary(grts_output, AREA_CAT ~ ELEV_CAT)))
  expect_output(print(summary(grts_output, ~ ELEV_CAT)))
  expect_output(print(summary(grts_output, AREA_CAT ~ ELEV_CAT)))  
  expect_output(print(grts_output))
  
  # irs output
  irs_output <- irs(NE_Lakes, 50)
  expect_output(print(sp_summary(irs_output, ~ ELEV_CAT)))
  expect_output(print(sp_summary(irs_output, AREA_CAT ~ ELEV_CAT)))
  expect_output(print(summary(irs_output, ~ ELEV_CAT)))
  expect_output(print(summary(irs_output, AREA_CAT ~ ELEV_CAT)))  
  expect_output(print(irs_output))
}

###############################################################################
# Function: stopprnt (exported)
# Programmer: Tony Olsen
# Date: January 22, 2021
#
#' Print grts() and irs() errors.
#'
#' This function prints the error messages vector in the \code{grts}
#' and \code{irs} functions.
#'
#' @param stop_df Data frame that contains stop messages.  The default is
#'   \code{stop_df}, which is the name given to the stop data frame created by
#'   functions in the spsurvey package.
#'
#' @param m Vector of indices for stop messages that are to be printed. The
#'   default is a vector containing the integers from 1 through the number of
#'   rows in \code{stop_df}, which will print all stop messages in the data frame.
#'
#' @return Printed errors
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#' @export
###############################################################################

stopprnt <- function(stop_df = get("stop_df", envir = .GlobalEnv),
                     m = 1:nrow(stop_df)) {
  message(paste("Input      ", "Error Message\n"))

  for (i in m) {
    message(paste(stop_df[i, 1], ": ", stop_df[i, 2], "\n"))
  }

  invisible(NULL)
}

###############################################################################
# Function: ash1_wgt.R (exported)
# Programmer: Tony Olsen
# 	Based on original script by Susan Hornsby
# Date: February 1, 2005
# Last Revised: January 22, 2021
#
#' Compute the average shifted histogram (ASH) for one-dimensional weighted data
#'
#' Calculate the average shifted histogram estimate of a density based on one-dimensional data
#'   from a survey design with weights.
#'
#' @param x Vector used to estimate the density. \code{NA} values are allowed.
#'
#' @param wgt Vector of weights for each observation from a
#'   probability sample. The default assigns equal weights (equal probability).
#'
#' @param m Number of empty bins to add to the ends when the range is not
#'   completely specified.  The default is \code{5}.
#'
#' @param nbin Number of bins for density estimation.  The default is \code{50}.
#'
#' @param ab Optional range for support associated with the density. Both
#'   values may be equal to \code{NA}.  If equal to \code{NA}, then corresponding limit will
#'   be based on \code{nicerange()}. The default is \code{NULL}.
#'
#' @param support Type of support.  If equal to \code{"Continuous"}, then data are
#'   from a continuous distribution.  If equal to \code{"Ordinal"}, then data are from
#'   a discrete distribution defined for integers only.  The default is
#'   \code{"Continuous"}.
#'
#' @return List containing the ASH density estimate.  List consists of
#'   \describe{
#'     \item{\code{tcen}}{ x-coordinate for center of bin}
#'     \item{\code{f}}{ y-coordinate for density estimate height}
#'   }
#'
#'
#' @author Tony Olsen \email{Olsen.tony@@epa.gov}
#'
#' @references
#'   Scott, D. W. (1985). "Averaged shifted histograms: effective nonparametric
#'   density estimators in several dimensions." \emph{The Annals of Statistics} 13(3):
#'   1024-1040.
#'
#'
#' @examples
#' x <- rnorm(100, 10, sqrt(10))
#' wgt <- runif(100, 10, 100)
#' rslt <- ash1_wgt(x, wgt)
#' plot(rslt)
#' @export
ash1_wgt <- function(x, wgt = rep(1, length(x)), m = 5, nbin = 50, ab = NULL,
                     support = "Continuous") {

  # Bin the possibly weighted data
  v <- bin1_wgt(x, wgt, nbin, ab, support = support)

  # Set delta based on range and number of bins
  delta <- attr(v, "delta")
  h <- m * delta

  # Set up vectors for estimation
  nbin <- attr(v, "nbin")
  a <- attr(v, "ab")[1]
  b <- attr(v, "ab")[2]

  # Add m-1 empty bins on ends when no ab boundary specified
  adj <- 0
  if (is.null(ab)) {
    v <- c(rep(0, m - 1), v, rep(0, m - 1))
    nbin <- nbin + 2 * (m - 1)
    adj <- m - 1
  } else if (is.na(ab[1])) {
    v <- c(rep(0, m - 1), v)
    nbin <- nbin + (m - 1)
    adj <- m - 1
  } else if (is.na(ab[2])) {
    v <- c(v, rep(0, m - 1))
    nbin <- nbin
  }

  # Compute lower limit, center, and upper limit of bins
  tlow <- a - adj * delta + ((1:nbin) - 1.0) * delta
  tcen <- a - adj * delta + ((1:nbin) - 0.5) * delta
  tup <- a - adj * delta + (1:nbin) * delta

  # Compute density height
  f <- rep(0, nbin)
  for (i in 1:nbin) {
    mlow <- max(1, i - m + 1)
    mhi <- min(nbin, i + m - 1)
    for (k in mlow:mhi) {
      if (tup[k] >= a & tlow[k] <= b) {
        f[i] <- f[i] + v[k] * wgt_lim(k - i, m, mlow = mlow - i, mhi = mhi - i)
      }
    }
  }

  # Adjust height so density area equals 1
  f <- f / (delta * sum(f))

  # Construct output
  ash <- list(x = tcen, y = f)
  attr(ash, "delta") <- delta
  attr(ash, "m") <- m
  attr(ash, "h") <- h
  attr(ash, "support") <- support

  # Return the result
  return(ash)
}


#
# BIN algorithm for unequal-probability sample,
#

bin1_wgt <- function(x, wgt = rep(1, length(x)), nbin = 50,
                     ab = nicerange(x), support = "Continuous") {

  # Remove any missing data
  x <- x[!is.na(x)]
  wgt <- wgt[!is.na(x)]
  n <- length(x)

  # Check that nbin is positive
  if (nbin <= 0) {
    stop("\nNumber of bin intervals nonpositive")
  }

  # Check for ab range
  tmp <- nicerange(x)
  if (is.null(ab)) {
    ab <- tmp
  } else {
    if (is.na(ab[1])) ab[1] <- tmp[1]
    if (is.na(ab[2])) ab[2] <- tmp[2]
    if (ab[1] >= ab[2]) {
      stop("\nInterval vector has negative orientation")
    }
  }

  # Determine delta
  # Continuous data case
  if (support == "Continuous") {
    delta <- (ab[2] - ab[1]) / nbin
  }
  if (support == "Ordinal") {
    delta <- 1
    ab[1] <- floor(ab[1]) - 0.5
    ab[2] <- ceiling(ab[2]) + 0.5
    nbin <- ab[2] - ab[1] + 1
  }

  # Sum weighted data in bins
  v <- rep(0, nbin)
  for (k in 1:nbin) {
    for (i in 1:n) {
      v[k] <- ifelse(((ab[1] + (k - 1) * delta) <= x[i]) & (x[i] < (ab[1] + k * delta)),
        v[k] + wgt[i], v[k]
      )
    }
  }
  attr(v, "nbin") <- nbin
  attr(v, "ab") <- ab
  attr(v, "delta") <- delta
  attr(v, "support") <- support

  # Return the result
  return(v)
}


#
# Define weight function
#
wgt_lim <- function(i, m, mlow = (1 - m), mhi = (m - 1)) {
  K <- function(t) {
    (15 / 16) * (1 - t^2)^2
  }
  I <- mlow:mhi
  w <- m * K(i / m) / sum(K(I / m))
  return(w)
}


#
# Find nice range for binning
#

nicerange <- function(x, beta = 0.1) {
  ab <- range(x)
  del <- ((ab[2] - ab[1]) * beta) / 2
  return(c(ab + c(-del, del)))
}

###############################################################################
# Function: localmean_cov (not exported)
# Programmer: Tom Kincaid
# Date: March 10, 2020
#
#' Internal Function: Variance-Covariance Matrix Based on Local Mean Estimator
#'
#' This function calculates the variance-covariance matrix using the local mean
#' estimator.
#'
#' @param zmat Matrix of weighted response values or weighted residual values
#'   for the sample points.
#'
#' @param weight_1st List from the local mean weight function containing two
#'   elements: a matrix named \code{ij} composed of the index values of neighboring
#'   points and a vector named \code{gwt} composed of weights.
#'
#' @return The local mean estimator of the variance-covariance matrix.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @export
###############################################################################

localmean_cov <- function(zmat, weight_1st) {

  # Calculate additional required values

  temp <- dim(zmat)
  m <- temp[2]

  # Initialize the results matrix

  lmvar <- array(0, c(m, m))

  # Begin loops for variance/covariance calculations

  for (k in 1:m) {
    for (l in k:m) {
      z1 <- zmat[, k]
      z2 <- zmat[, l]

      # Calculate local means

      zb1 <- sapply(split(z1[weight_1st$ij[, 2]] * weight_1st$gwt, weight_1st$ij[, 1]), sum)
      zb2 <- sapply(split(z2[weight_1st$ij[, 2]] * weight_1st$gwt, weight_1st$ij[, 1]), sum)

      # Calculate the variance or covariance estimate

      lmvar[k, l] <- sum(weight_1st$gwt * (z1[weight_1st$ij[, 2]] - zb1[weight_1st$ij[, 1]]) * (z2[weight_1st$ij[, 2]] - zb2[weight_1st$ij[, 1]]))
    }

    # Assign estimates that already have been calculated

    if (k > 1) {
      lmvar[k, 1:(k - 1)] <- lmvar[1:(k - 1), k]
    }
  }

  # Return the variance/covariance estimate

  lmvar
}

###############################################################################
# Function: uniqueID (not exported)
# Programmer: Tom Kincaid
# Date: February 23, 2004
# Last Revised: June 27, 2005
#
#' Internal Function: Create Unique IDs for a Survey Design
#'
#' This function creates unique site IDs by appending a unique number to each
#' occurrence of a site ID.  It is intended for survey designs that have repeat
#' visits to sites.
#'
#' @param siteID  Vector of site IDs.
#'
#' @return Vector of unique site IDs.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @noRd
###############################################################################

uniqueID <- function(siteID) {

  # If siteID is a factor, convert the input vector to character data

  factor.ind <- FALSE
  if (is.factor(siteID)) {
    siteID <- as.character(siteID)
    factor.ind <- TRUE
  }

  # Assign new site IDs

  for (id in unique(siteID)) {
    temp <- siteID == id
    if (sum(temp) > 1) {
      siteID[temp] <- paste(siteID[temp], 1:sum(temp), sep = ".")
    }
  }

  # If siteID is a factor, convert the result to a factor

  if (factor.ind) {
    siteID <- as.factor(siteID)
  }

  # Return the vector of site IDs

  siteID
}

################################################################################
# Function: input_check (not exported)
# Programmer: Tom Kincaid
# Date: October 9, 2020
# Revised: November 5, 2020 to correct an error when checking size-weights for
#          two-stage samples
# Revised: December 17, 2020 to check input values for the Horvitz-Thompson and
#          Yates-Grundy options for variance estimator
# Revised: June 4, 2021 to implement checking the revised approach used for
#          specification of the finite population correction factor and to
#          ensure that continuous variables have syntactically valid names
# Revised: July 14, 2021 to ensure that character variables are processed as
#          factors
#
#' Check Input Values for Analytical Functions
#'
#' This function checks input values for errors, consistency, and compatibility
#' with analytical functions.
#'
#' @param dframe Data frame containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param design_names Vector composed of character values that identify the
#'   names of the following survey design variables in the \code{dframe} data
#'   frame: \code{siteID}, \code{weight}, \code{xcoord}, \code{ycoord},
#'   \code{stratumID}, \code{clusterID}, \code{weight1}, \code{xcoord1},
#'   \code{ycoord1}, \code{sweight}, \code{sweight1}, \code{fpcsize},
#'   \code{Ncluster}, \code{stage1size}.
#'
#' @param vars_cat Vector composed of character values that identify the names
#'   of categorical response variables in the \code{dframe} data frame.
#'
#' @param vars_cont Vector composed of character values that identify the names
#'   of continuous response variables in the \code{dframe} data frame.
#'
#' @param vars_stressor Vector composed of character values that identify the
#'   names of stressor variables in the \code{dframe} data frame.
#'
#' @param vars_nondetect Vector composed of character values that identify the
#'   names of logical variables in the \code{dframe} data frame specifying the
#'   presence of not detected (nondetect) values for response variables.
#'
#' @param subpops Vector composed of character values that identify the names of
#'   subpopulation variables in the \code{dframe} data frame.
#'
#' @param sizeweight Logical value that indicates whether size weights should be
#'   used during estimation.
#'
#' @param fpc Object that specifies values required for calculation of the
#'   finite population correction factor used during variance estimation.
#'
#' @param popsize Object providing known size of the resource.
#'
#' @param vartype Character value providing choice of the variance estimator,
#'   where \code{"Local"} = the local mean estimator, \code{"SRS"} = the simple
#'   random sampling estimator, \code{"HT"} = the Horvitz-Thompson estimator,
#'   and \code{"YG"} = the Yates-Grundy estimator.
#'
#' @param jointprob Character value providing choice of joint inclusion
#'   probability approximation for use with Horvitz-Thompson and Yates-Grundy
#'   variance estimators, where \code{"overton"} indicates the Overton
#'   approximation, \code{"hr"} indicates the Hartley_Rao approximation, and
#'   \code{"brewer"} equals the Brewer approximation.
#'
#' @param conf Numeric value providing the confidence level.
#'
#' @param cdfval Vector of the set of values at which the CDF is estimated.
#'
#' @param pctval Vector of the set of values at which percentiles are estimated.
#'
#' @param error_ind Logical value that indicates whether error messages were
#'   generated.
#'
#' @param error_vec Vector for storing error messages.
#'
#' @return A list consisting of \code{dframe}, \code{vars_cat},
#'   \code{vars_cont}, \code{vars_stressor}, \code{subpops}, \code{popsize},
#'   \code{vartype}, \code{jointprob}, \code{error_ind}, and \code{error_vec}.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @noRd
################################################################################

input_check <- function(dframe, design_names, vars_cat, vars_cont,
                        vars_stressor, vars_nondetect, subpops, sizeweight,
                        fpc, popsize, vartype, jointprob, conf, cdfval = NULL,
                        pctval = NULL, error_ind, error_vec) {

  # Ensure that character variables are processed as factors

  # options(stringsAsFactors = TRUE)
  ## set factors to character and then reassign as factor so default factor ordering is imposed
  fac_index <- vapply(dframe, is.factor, logical(1))
  dframe[fac_index] <- vapply(dframe[fac_index], as.character, character(nrow(dframe)))
  dframe <- as.data.frame(unclass(dframe), stringsAsFactors = TRUE)

  # For variables that exist in the dframe data frame, assign survey design
  # variables and check those variables for missing values

  temp <- c("surveyID", "siteID", "weight")
  design_names <- design_names[!(names(design_names) %in% c(
    "fpcsize",
    "Ncluster", "stage1size"
  ))]
  for (i in names(design_names)) {
    if (is.null(design_names[[i]])) {
      eval(parse(text = paste0(i, " <- NULL")))
    } else {
      if (!(design_names[[i]] %in% names(dframe))) {
        if (!(i %in% temp)) {
          error_ind <- TRUE
          msg <- paste0("The name provided for the ", i, " argument, \"", design_names[[i]], "\", does not occur among \nthe names for the dframe data frame.\n")
          error_vec <- c(error_vec, msg)
        }
        eval(parse(text = paste0(i, " <- NULL")))
      } else {
        eval(parse(text = paste0(i, " <- dframe[, \"", design_names[[i]], "\"]")))
        if (any(is.na(dframe[, design_names[[i]]]))) {
          error_ind <- TRUE
          msg <- paste0("Missing values are not allowed for the ", design_names[[i]], " variable in the dframe data \nframe.\n")
          error_vec <- c(error_vec, msg)
          eval(parse(text = paste0(i, " <- NULL")))
        }
      }
    }
  }

  # If the sample is stratified, code stratumID as a factor, determine stratum
  # levels, and calculate number of strata

  if (!is.null(stratumID)) {
    stratumID <- factor(stratumID)
    stratum_levels <- levels(stratumID)
    nstrata <- length(stratum_levels)
  }

  # Check the response variables arguments to ensure that the arguments are
  # vectors composed of character values and that the elements of the vectors
  # are included among the names in the dframe data frame.  For categorical
  # response variables, ensure that the variables referenced in the argument are
  # coded as factors.  For continuous response variables, ensure that the
  # variables referenced in the argument are coded as numeric variables and that
  # variable names are syntactically valid

  if (!is.null(vars_cat)) {
    if (!is.vector(vars_cat)) {
      error_ind <- TRUE
      msg <- "The vars_cat argument must be a vector.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (is.list(vars_cat)) {
        vars_cat <- unlist(vars_cat)
      }
      if (!all(is.character(vars_cat))) {
        vars_cat <- sapply(vars_cat, as.character)
      }
      tst <- !(vars_cat %in% names(dframe))
      if (any(tst)) {
        error_ind <- TRUE
        temp_str <- vecprint(vars_cat[tst])
        msg <- paste0("The following categorical response variable names do not occur among the \nnames for the dframe data frame:\n", temp_str)
        error_vec <- c(error_vec, msg)
      } else {
        for (i in vars_cat) {
          if (!is.factor(dframe[, i])) {
            dframe[, i] <- factor(dframe[, i])
          }
        }
      }
    }
  }

  if (!is.null(vars_cont)) {
    if (!is.vector(vars_cont)) {
      error_ind <- TRUE
      msg <- "The vars_cont argument must be a vector.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (is.list(vars_cont)) {
        vars_cont <- unlist(vars_cont)
      }
      tst <- !(vars_cont %in% names(dframe))
      if (any(tst)) {
        error_ind <- TRUE
        temp_str <- vecprint(vars_cont[tst])
        msg <- paste0("The following continuous response variable names do not occur among the \nnames for the dframe data frame:\n", temp_str)
        error_vec <- c(error_vec, msg)
      } else {
        for (i in vars_cont) {
          if (!is.numeric(dframe[, i])) {
            dframe[, i] <- as.numeric(dframe[, i])
          }
        }
        indx <- match(vars_cont, names(dframe))
        vars_cont <- make.names(vars_cont)
        names(dframe)[indx] <- vars_cont
      }
    }
  }

  # Check the stressor variables argument to ensure that the argument is a
  # vector composed of character values,that elements of the vector are included
  # among the names in the dframe data frame, and that the variables referenced
  # in the vector are coded as factors.

  if (!is.null(vars_stressor)) {
    if (!is.vector(vars_stressor)) {
      error_ind <- TRUE
      msg <- "The vars_stressor argument must be a vector.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (is.list(vars_stressor)) {
        vars_stressor <- unlist(vars_stressor)
      }
      if (!all(is.character(vars_stressor))) {
        vars_stressor <- sapply(vars_stressor, as.character)
      }
      tst <- !(vars_stressor %in% names(dframe))
      if (any(tst)) {
        error_ind <- TRUE
        temp_str <- vecprint(vars_stressor[tst])
        msg <- paste0("The following stressor[ variable names do not occur among the names for the \ndframe data frame:\n", temp_str)
        error_vec <- c(error_vec, msg)
      } else {
        for (i in vars_stressor) {
          if (!is.factor(dframe[, i])) {
            dframe[, i] <- factor(dframe[, i])
          }
        }
      }
    }
  }

  # Check the vars_nondetect logical variables argument to ensure that the
  # argument is a vector composed of character values, that elements of the
  # vector are included among the names in the dframe data frame, and that the
  # variables referenced in the vector are coded as logical variables.

  if (!is.null(vars_nondetect)) {
    if (!is.vector(vars_nondetect)) {
      error_ind <- TRUE
      msg <- "The vars_nondetect argument must be a vector.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (is.list(vars_nondetect)) {
        vars_nondetect <- unlist(vars_nondetect)
      }
      if (!all(is.character(vars_nondetect))) {
        vars_nondetect <- sapply(vars_nondetect, as.character)
      }
      tst <- !(vars_nondetect %in% names(dframe))
      if (any(tst)) {
        error_ind <- TRUE
        temp_str <- vecprint(vars_nondetect[tst])
        msg <- paste0("The following response variable names do not occur among the names for the \ndframe data frame:\n", temp_str)
        error_vec <- c(error_vec, msg)
      } else {
        for (i in vars_nondetect) {
          if (!is.logical(dframe[, i])) {
            dframe[, i] <- as.logical(dframe[, i])
          }
        }
      }
    }
  }

  # Check the subpopulation (domain) names argument to ensure that the argument
  # is a vector composed of character values, that the elements of the vector
  # are included among the names in the dframe data frame, and that all response
  # variables referenced in the vector are coded as factors

  if (is.list(subpops)) {
    subpops <- unlist(subpops)
  }
  if (!is.vector(subpops)) {
    subpops <- as.vector(subpops)
  }
  if (!all(is.character(subpops))) {
    subpops <- as.character(subpops)
  }
  tst <- !(subpops %in% names(dframe))
  if (any(tst)) {
    error_ind <- TRUE
    temp_str <- vecprint(subpops[tst])
    msg <- paste0("The following subpopulation (domain) names do not occur among the names for the \ndframe data frame:\n", temp_str)
    error_vec <- c(error_vec, msg)
  } else {
    for (i in subpops) {
      if (!is.factor(dframe[, i])) {
        dframe[, i] <- factor(dframe[, i])
      }
    }
  }

  # Check weight arguments

  if (!is.null(clusterID)) {
    if (!is.null(weight)) {
      if (min(weight, na.rm = TRUE) <= 0) {
        error_ind <- TRUE
        msg <- "Stage two weights must be positive.\n"
        error_vec <- c(error_vec, msg)
      }
    }
    if (sizeweight) {
      if (is.null(sweight)) {
        error_ind <- TRUE
        msg <- "Argument sweight was not supplied.\n"
        error_vec <- c(error_vec, msg)
      } else {
        if (min(sweight, na.rm = TRUE) <= 0) {
          error_ind <- TRUE
          msg <- "Stage two size-weights must be positive.\n"
          error_vec <- c(error_vec, msg)
        }
      }
    }
    if (is.null(weight1)) {
      error_ind <- TRUE
      msg <- "Argument weight1 was not supplied.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (min(weight1, na.rm = TRUE) <= 0) {
        error_ind <- TRUE
        msg <- "Stage one weights must be positive.\n"
        error_vec <- c(error_vec, msg)
      }
      if (!is.null(stratumID)) {
        temp_weight1 <- split(weight1, stratumID)
        for (i in 1:nstrata) {
          tst <- stratumID == stratum_levels[i]
          temp_clusterID <- clusterID[tst]
          if (any(sapply(
            tapply(temp_weight1[[i]], temp_clusterID, unique),
            length
          ) > 1)) {
            error_ind <- TRUE
            msg <- paste0("\nThe stage one weight must be constant for all stage two sampling units within \neach stage one sampling unit of stratum ", stratum_levels[i], ".\n")
            error_vec <- c(error_vec, msg)
          }
        }
      } else {
        if (any(sapply(tapply(weight1, clusterID, unique), length) > 1)) {
          error_ind <- TRUE
          msg <- "The stage one weight must be constant for all stage two sampling units within \neach stage one sampling unit.\n"
          error_vec <- c(error_vec, msg)
        }
      }
    }
    if (sizeweight) {
      if (is.null(sweight1)) {
        error_ind <- TRUE
        msg <- "Argument sweight1 was not supplied.\n"
        error_vec <- c(error_vec, msg)
      } else {
        if (min(sweight1, na.rm = TRUE) <= 0) {
          error_ind <- TRUE
          msg <- "Stage one size-weights must be positive.\n"
          error_vec <- c(error_vec, msg)
        }
        if (!is.null(stratumID)) {
          temp.sweight1 <- split(sweight1, stratumID)
          for (i in 1:nstrata) {
            tst <- stratumID == stratum_levels[i]
            temp_clusterID <- clusterID[tst]
            if (any(sapply(
              tapply(temp.sweight1[[i]], temp_clusterID, unique),
              length
            ) > 1)) {
              error_ind <- TRUE
              msg <- paste0("\nThe stage one size-weight must be constant for all stage two sampling units \nwithin each stage one sampling unit of stratum ", stratum_levels[i], ".\n")
              error_vec <- c(error_vec, msg)
            }
          }
        } else {
          if (any(sapply(tapply(sweight1, clusterID, unique), length) > 1)) {
            error_ind <- TRUE
            msg <- "The stage one size-weight must be constant for all stage two sampling units \nwithin each stage one sampling unit.\n"
            error_vec <- c(error_vec, msg)
          }
        }
      }
    }
  } else {
    if (!is.null(weight)) {
      if (min(weight, na.rm = TRUE) <= 0) {
        error_ind <- TRUE
        msg <- "Weights must be positive.\n"
        error_vec <- c(error_vec, msg)
      }
    }
    if (sizeweight) {
      if (is.null(sweight)) {
        error_ind <- TRUE
        msg <- "Argument sweight was not supplied.\n"
        error_vec <- c(error_vec, msg)
      } else {
        if (min(sweight, na.rm = TRUE) <= 0) {
          error_ind <- TRUE
          msg <- "Size-weights must be positive.\n"
          error_vec <- c(error_vec, msg)
        }
      }
    }
  }

  # Check coordinate arguments

  if (vartype == "Local") {
    if (!is.null(clusterID)) {
      if (is.null(xcoord) || is.null(ycoord)) {
        error_ind <- TRUE
        msg <- "Stage two x-coordinates and y-coordinates for location are required for the \nlocal mean variance estimator.\n"
        error_vec <- c(error_vec, msg)
      }
      if (is.null(xcoord1) || is.null(ycoord1)) {
        error_ind <- TRUE
        msg <- "Stage one x-coordinates and y-coordinates for location are required for the \nlocal mean variance estimator.\n"
        error_vec <- c(error_vec, msg)
      } else {
        if (!is.null(stratumID)) {
          temp_xcoord1 <- split(xcoord1, stratumID)
          temp_ycoord1 <- split(ycoord1, stratumID)
          for (i in 1:nstrata) {
            tst <- stratumID == stratum_levels[i]
            temp_clusterID <- clusterID[tst]
            if (any(sapply(
              tapply(temp_xcoord1[[i]], temp_clusterID, unique),
              length
            ) > 1)) {
              error_ind <- TRUE
              msg <- paste0("\nThe stage one x-coordinate must be constant for all stage two sampling units within \neach stage one sampling unit of stratum ", stratum_levels[i], ".\n")
              error_vec <- c(error_vec, msg)
            }
            if (any(sapply(
              tapply(temp_ycoord1[[i]], temp_clusterID, unique),
              length
            ) > 1)) {
              error_ind <- TRUE
              msg <- paste0("\nThe stage one y-coordinate must be constant for all stage two sampling units within \neach stage one sampling unit of stratum ", stratum_levels[i], ".\n")
              error_vec <- c(error_vec, msg)
            }
          }
        } else {
          if (any(sapply(tapply(xcoord1, clusterID, unique), length) > 1)) {
            error_ind <- TRUE
            msg <- "The stage one x-coordinate must be constant for all stage two sampling units within \neach stage one sampling unit.\n"
            error_vec <- c(error_vec, msg)
          }
          if (any(sapply(tapply(ycoord1, clusterID, unique), length) > 1)) {
            error_ind <- TRUE
            msg <- "The stage one y-coordinate must be constant for all stage two sampling units within \neach stage one sampling unit.\n"
            error_vec <- c(error_vec, msg)
          }
        }
      }
    } else {
      if (is.null(xcoord) || is.null(ycoord)) {
        error_ind <- TRUE
        msg <- "x-coordinates and y-coordinates for location are required for the local mean \nvariance estimator.\n"
        error_vec <- c(error_vec, msg)
      }
    }
  }

  # Check the known size of the resource argument

  if (!is.null(popsize)) {
    cls <- class(popsize)[1]
    if (cls %in% c("data.frame", "table", "xtabs")) {
      if (cls == "data.frame") {
        pnames <- names(popsize)[-ncol(popsize)]
        tst <- !(pnames %in% names(dframe))
        if (any(tst)) {
          error_ind <- TRUE
          temp_str <- vecprint(pnames[tst])
          msg <- paste0("\nThe following post-stratification variables do not occur among the names for \nthe dframe data frame:\n", temp_str)
          error_vec <- c(error_vec, msg)
        } else {
          for (i in length(pnames)) {
            temp <- unique(as.character(popsize[, pnames[i]]))
            tst <- !(temp %in% unique(as.character(dframe[, pnames[i]])))
            if (any(tst)) {
              error_ind <- TRUE
              temp_str <- vecprint(temp[tst])
              msg <- paste0("\nThe following values for the \"", pnames[i], "\" post-stratification variable in the \npopsize data frame do not occur among the unique values for that variable in \nthe dframe data frame:\n", temp_str)
              error_vec <- c(error_vec, msg)
            }
          }
        }
      } else {
        pnames <- names(dimnames(popsize))
        if (is.null(pnames)) {
          error_ind <- TRUE
          msg <- paste0("\nThe popsize argument must have named dimnames when the class for popsize is: \n\"", cls, "\"")
          error_vec <- c(error_vec, msg)
        }
        tst <- !(pnames %in% names(dframe))
        if (any(tst)) {
          error_ind <- TRUE
          temp_str <- vecprint(pnames[tst])
          msg <- paste0("\nThe following post-stratification variables do not occur among the names for \nthe dframe data frame:\n", temp_str)
          error_vec <- c(error_vec, msg)
        } else {
          for (i in length(dim(popsize))) {
            temp <- dimnames(popsize)[[i]]
            tst <- !(temp %in% unique(as.character(dframe[, pnames[i]])))
            if (any(tst)) {
              error_ind <- TRUE
              temp_str <- vecprint(temp[tst])
              msg <- paste0("\nThe following values for the \"", pnames[i], "\" post-stratification variable in the \npopsize object do not occur among the unique values for that variable in \nthe dframe data frame:\n", temp_str)
              error_vec <- c(error_vec, msg)
            }
          }
        }
      }
    } else if (cls == "list") {
      cnames <- names(popsize)
      tst <- !(cnames %in% names(dframe))
      if (any(tst)) {
        error_ind <- TRUE
        temp_str <- vecprint(cnames[tst])
        msg <- paste0("\nThe following calibration variables do not occur among the names for the dframe \ndata frame:\n", temp_str)
        error_vec <- c(error_vec, msg)
      }
      for (i in length(popsize)) {
        temp <- names(popsize[[i]])
        tst <- !(temp %in% unique(as.character(dframe[, cnames[i]])))
        if (any(tst)) {
          error_ind <- TRUE
          temp_str <- vecprint(temp[tst])
          msg <- paste0("\nThe following values for the \"", cnames[i], "\" calibration variable in the popsize \nlist do not occur among the unique values for that variable in the dframe \ndata frame:\n", temp_str)
          error_vec <- c(error_vec, msg)
        }
      }
    } else {
      error_ind <- TRUE
      msg <- paste0("\nThe class for the popsize argument, \"", cls, "\", is not a valid class.  The class \nmust be either \"data frame\", \"table\", \"xtabs\", or \"list\".")
      error_vec <- c(error_vec, msg)
    }
  }

  # Check the finite population correction factor argument and, as necessary,
  # add variables to the dframe data frame

  if (!is.null(fpc)) {
    if (!is.null(stratumID)) {
      if (!is.null(clusterID)) {
        cluster_levels <- tapply(
          clusterID, stratumID,
          function(x) levels(factor(x))
        )
        ncluster <- sapply(cluster_levels, length)
        indx <- match(names(fpc), names(ncluster), nomatch = 0)
        if (!is.list(fpc)) {
          error_ind <- TRUE
          msg <- "For a stratified, two-stage survey design, the fpc argument must be a list.\n"
          error_vec <- c(error_vec, msg)
        } else if (length(fpc) != nstrata) {
          error_ind <- TRUE
          msg <- "For a stratified, two-stage survey design, the fpc argument must be the same length \nas the number of strata.\n"
          error_vec <- c(error_vec, msg)
        } else if (is.null(names(fpc))) {
          error_ind <- TRUE
          msg <- "For a stratified, single-stage survey design, the fpc argument must be a named list.\n"
          error_vec <- c(error_vec, msg)
        } else if (!all(stratum_levels %in% names(fpc))) {
          error_ind <- TRUE
          msg <- "For a stratified, single-stage survey design, names for the fpc argument must match \ncatgories for the stratum ID variable in the dframe data frame.\n"
          error_vec <- c(error_vec, msg)
        } else if (any(sapply(fpc, function(x) !is.vector(x))) ||
          any(sapply(fpc, length) != (ncluster[indx] + 1)) ||
          any(sapply(fpc, function(x) !is.numeric(x)))) {
          error_ind <- TRUE
          msg <- "For a stratified, two-stage survey design, the fpc argument, for each stratum, must be \na numeric vector with length equal to one plus the number of clusters in the sample for \nthe stratum.\n"
          error_vec <- c(error_vec, msg)
        } else {
          if (any(sapply(fpc, function(x) any(x <= 0)))) {
            error_ind <- TRUE
            msg <- "For a stratified, two-stage survey design, the fpc argument, for each stratum, must contain positive values.\n"
            error_vec <- c(error_vec, msg)
          } else {
            dframe$Ncluster <- numeric(nrow(dframe))
            dframe$stage1size <- numeric(nrow(dframe))
            for (i in stratum_levels) {
              temp <- fpc[[i]]
              tst <- stratumID == i
              k <- 1
              dframe$Ncluster[tst] <- temp[k]
              for (j in levels(factor(clusterID[tst]))) {
                tst <- stratumID == i & clusterID == j
                k <- k + 1
                dframe$stage1size[tst] <- temp[k]
              }
            }
          }
        }
      } else {
        if (!is.list(fpc)) {
          error_ind <- TRUE
          msg <- "For a stratified, single-stage survey design, the fpc argument must be a list.\n"
          error_vec <- c(error_vec, msg)
        } else if (length(fpc) != nstrata) {
          error_ind <- TRUE
          msg <- "For a stratified, single-stage survey design, the fpc argument must be the same length \nas the number of strata.\n"
          error_vec <- c(error_vec, msg)
        } else if (is.null(names(fpc))) {
          error_ind <- TRUE
          msg <- "For a stratified, single-stage survey design, the fpc argument must be a named list.\n"
          error_vec <- c(error_vec, msg)
        } else if (!all(stratum_levels %in% names(fpc))) {
          error_ind <- TRUE
          msg <- "For a stratified, single-stage survey design, names for the fpc argument must match \ncatgories for the stratum ID variable in the dframe data frame.\n"
          error_vec <- c(error_vec, msg)
        } else if (any(sapply(fpc, function(x) !is.vector(x))) ||
          any(sapply(fpc, length) > 1) ||
          any(sapply(fpc, function(x) !is.numeric(x)))) {
          error_ind <- TRUE
          msg <- "For a stratified, single-stage survey design, the fpc argument must contain a single \nnumeric value for each strata.\n"
          error_vec <- c(error_vec, msg)
        } else {
          if (any(fpc <= 0)) {
            error_ind <- TRUE
            msg <- "For a stratified, single-stage survey design, the fpc argument must contain a positive \nvalue for each stratum.\n"
            error_vec <- c(error_vec, msg)
          } else {
            dframe$fpcsize <- numeric(nrow(dframe))
            for (i in stratum_levels) {
              tst <- stratumID == i
              dframe$fpcsize[tst] <- fpc[[i]]
            }
          }
        }
      }
    } else {
      if (!is.null(clusterID)) {
        cluster_levels <- levels(factor(clusterID))
        ncluster <- length(cluster_levels)
        if (!is.vector(fpc)) {
          error_ind <- TRUE
          msg <- "For an unstratified, two-stage survey design, the fpc argument must be a vector\n"
          error_vec <- c(error_vec, msg)
        } else if (length(fpc) != (ncluster + 1)) {
          error_ind <- TRUE
          msg <- "For an unstratified, two-stage survey design, the fpc argument must be the same length \nas the number of clusters plus one\n"
          error_vec <- c(error_vec, msg)
        } else if (is.null(names(fpc))) {
          error_ind <- TRUE
          msg <- "For a stratified, single-stage survey design, the fpc argument must be a named vector.\n"
        } else if (!all(cluster_levels %in% names(fpc)[-1])) {
          error_ind <- TRUE
          msg <- "For a stratified, single-stage survey design, names for the fpc argument must include \nall of the catgories for the cluster ID variable in the dframe data frame.\n"
          error_vec <- c(error_vec, msg)
        } else if (any(sapply(fpc, length) > 1) ||
          any(sapply(fpc, function(x) !is.numeric(x)))) {
          error_ind <- TRUE
          msg <- "For unstratified, two-stage survey design, the fpc argument must contain a single \nnumeric value for each item in the vector\n"
          error_vec <- c(error_vec, msg)
        } else {
          if (any(fpc <= 0)) {
            error_ind <- TRUE
            msg <- "For an unstratified, two-stage survey design, the fpc argument must contain a positive \nvaluec.\n"
            error_vec <- c(error_vec, msg)
          } else {
            dframe$Ncluster <- fpc[1]
            dframe$stage1size <- numeric(nrow(dframe))
            for (i in cluster_levels) {
              tst <- clusterID == i
              dframe$stage1size[tst] <- fpc[[i]]
            }
          }
        }
      } else {
        if (!is.vector(fpc) || length(fpc) != 1 || !is.numeric(fpc)) {
          error_ind <- TRUE
          msg <- "For an unstratified, single-stage survey design, the fpc argument  must contain a single \nnumeric value.\n"
          error_vec <- c(error_vec, msg)
        } else {
          if (fpc <= 0) {
            error_ind <- TRUE
            msg <- "For an unstratified, single-stage survey design the fpc argument must contain a single \npositive value.\n"
            error_vec <- c(error_vec, msg)
          }
          dframe$fpcsize <- fpc
        }
      }
    }
  }

  # Check the vartype argument

  if (!(vartype %in% c("Local", "local", "SRS", "srs", "HT", "ht", "YG", "yg"))) {
    error_ind <- TRUE
    msg <- paste0("\nThe value provided for argument vartype must equal either \"Local\",  \"SRS\", \"HT\", or \"YG\".  \nThe value provided was: \"", vartype, "\".")
    error_vec <- c(error_vec, msg)
  }
  if (vartype == "local") {
    vartype <- "Local"
  }
  if (vartype == "srs") {
    vartype <- "SRS"
  }
  if (vartype == "ht") {
    vartype <- "HT"
  }
  if (vartype == "yg") {
    vartype <- "YG"
  }
  if (vartype %in% c("HT", "YG")) {
    if (jointprob == "Overton") {
      jointprob <- "overton"
    }
    if (jointprob == "HR") {
      jointprob <- "hr"
    }
    if (jointprob == "Brewer") {
      jointprob <- "brewer"
    }
    if (!(jointprob %in% c("overton", "hr", "brewer"))) {
      error_ind <- TRUE
      msg <- paste0("\nThe value provided for argument jointprob must equal either \"overton\",  \"hr\", or \"brewer\".  \nThe value provided was: \"", jointprob, "\".")
      error_vec <- c(error_vec, msg)
    }
    if (jointprob %in% c("overton", "hr") && !is.null(clusterID)) {
      error_ind <- TRUE
      msg <- paste0("\nA two-stage survey design is not allowed when the jointprob argument equals \"", jointprob, "\".")
      error_vec <- c(error_vec, msg)
    }
    if (jointprob == "brewer" && vartype != "HT") {
      error_ind <- TRUE
      msg <- paste("\nThe vartype argument must equal \"HT\" when the jointprob argument equals \"brewer\".")
      error_vec <- c(error_vec, msg)
    }
  }

  # Check the confidence level argument

  if (!is.numeric(conf)) {
    error_ind <- TRUE
    msg <- "The confidence level must be a numeric value.\n"
    error_vec <- c(error_vec, msg)
  }

  # Check the CDF values argument

  if (!is.null(cdfval) && !is.numeric(cdfval)) {
    error_ind <- TRUE
    msg <- "The set of value at which the CDF is estimated must be numeric values.\n"
    error_vec <- c(error_vec, msg)
  }


  # Check the percentile values argument

  if (!is.null(pctval) && !is.numeric(pctval)) {
    error_ind <- TRUE
    msg <- "The set of value at which percentiles are estimated must be numeric values.\n"
    error_vec <- c(error_vec, msg)
  }

  # Return the results list

  list(
    dframe = dframe, vars_cat = vars_cat, vars_cont = vars_cont,
    vars_stressor = vars_stressor, vars_nondetect = vars_nondetect,
    subpops = subpops, popsize = popsize, vartype = vartype,
    jointprob = jointprob, error_ind = error_ind, error_vec = error_vec
  )
}

################################################################################
# Function: cont_analysis (exported)
# Programmer: Tom Kincaid
# Date: July 23, 2020
# Revised: August 14, 2020 to allow use of an sf object as the input data
#          argument (dframe)
# Revised: December 15, 2020 to allow use of the Horvitz-Thompson and
#          Yates-Grundy variance estimators and to use a new function named
#          survey_design to create the survey design object
# Revised: March 2, 2021 to revise the process for creating unique site ID
#          values
# Revised: April 7, 2021 to ensure that the dframe argument does not contain
#          zero rows
# Revised: April 29, 2021 to ensure that the dframe argument only belongs to
#          class "data.frame"
# Revised: May 4, 2021 to avoid warning messages being generated during creation
#          of help files
# Revised: May 6, 2021 to ensure that sf objects do not belong to class tbl_df
# Revised: June 8, 2021 to simplify specification of the values required for
#          calculation of the finite population correction factor and to
#          eliminate use of the finite population correction factor with the
#          local mean variance estimator
# Revised: June 11, 2021 to allow user control over calculation of CDF
#          estimates, percentile estimates and mean estimates and to use a new
#          function named mean_est for calculation of mean estimates
# Revised: September 9, 2021 to remove argument vartype from the call list for
#          function percentile_est, to remove argument vars_nondetect since
#          currently it is not used, and to revise the documentation for
#          argument popsize
# Revised: October 25 to include the option to calculate total estimates
#
#' Continuous variable analysis
#'
#' This function organizes input and output for the analysis of continuous
#' variables. The analysis data, \code{dframe}, can be either a data frame or a
#' simple features (\code{sf}) object.  If an \code{sf} object is used,
#' coordinates are extracted from the geometry column in the object, arguments
#' \code{xcoord} and \code{ycoord} are assigned values \code{"xcoord"} and
#' \code{"ycoord"}, respectively, and the geometry column is dropped from the
#' object.
#'
#' @inherit cat_analysis params
#'
#' @param pctval  Vector of the set of values at which percentiles are
#'   estimated.  The default set is: \code{c(5, 10, 25, 50, 75, 90, 95)}.
#'
#' @param statistics Character vector specifying desired estimates, where
#'   \code{"CDF"} specifies CDF estimates, \code{"Pct"} specifies percentile
#'   estimates, \code{"Mean"} specifies mean estimates, and "Total" specifies
#'   total estimates.  Any combination of the four choices may be provided by
#'   the user.  The default value is \code{c("CDF", "Pct", "Mean", "Total")}.
#'
#' @return The analysis results. A list composed of one, two, three, or four
#'   data frames that contain population estimates for all combinations of
#'   subpopulations, categories within each subpopulation, and response
#'   variables, where the number of data frames is determined by argument
#'   \code{statistics}.  The possible data frames in the output list are:
#'   \describe{
#'     \item{\code{CDF}}{: a data frame containing CDF estimates}
#'     \item{\code{Pct}}{: data frame containing percentile estimates}
#'     \item{\code{Mean}}{: a data frame containing mean estimates}
#'     \item{\code{Total}}{: a data frame containing total estimates}
#'   }
#'   
#'   The \code{CDF} data frame contains the following variables:
#'   \describe{
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{Value}{value of response variable}
#'     \item{nResp}{sample size at or below \code{Value}}
#'     \item{Estimate.P}{CDF proportion estimate (in \%)}
#'     \item{StdError.P}{standard error of CDF proportion estimate}
#'     \item{MarginofError.P}{margin of error of CDF proportion estimate}
#'     \item{LCBxxPct.P}{xx\% (default 95\%) lower confidence bound of CDF proportion estimate}
#'     \item{UCBxxPct.P}{xx\% (default 95\%) upper confidence bound of CDF proportion estimate}
#'     \item{Estimate.U}{CDF total estimate}
#'     \item{StdError.U}{standard error of CDF total estimate}
#'     \item{MarginofError.U}{margin of error of CDF total estimate}
#'     \item{LCBxxPct.U}{xx\% (default 95\%) lower confidence bound of CDF total estimate}
#'     \item{UCBxxPct.U}{xx\% (default 95\%) upper confidence bound of CDF total estimate}
#'   }
#'   
#'   The \code{Pct} data frame contains the following variables:
#'   \describe{
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{Statistic}{value of percentile}
#'     \item{nResp}{sample size at or below \code{Value}}
#'     \item{Estimate}{percentile estimate}
#'     \item{StdError}{standard error of percentile estimate}
#'     \item{MarginofError}{margin of error of percentile estimate}
#'     \item{LCBxxPct}{xx\% (default 95\%) lower confidence bound of percentile estimate}
#'     \item{UCBxxPct}{xx\% (default 95\%) upper confidence bound of percentile estimate}
#'   }
#'   
#'   The \code{Mean} data frame contains the following variables:
#'   \describe{
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{nResp}{sample size at or below \code{Value}}
#'     \item{Estimate}{mean estimate}
#'     \item{StdError}{standard error of mean estimate}
#'     \item{MarginofError}{margin of error of mean estimate}
#'     \item{LCBxxPct}{xx\% (default 95\%) lower confidence bound of mean estimate}
#'     \item{UCBxxPct}{xx\% (default 95\%) upper confidence bound of mean estimate}
#'   }
#'   
#'   The \code{Total} data frame contains the following variables:
#'   \describe{
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{nResp}{sample size at or below \code{Value}}
#'     \item{Estimate}{total estimate}
#'     \item{StdError}{standard error of total estimate}
#'     \item{MarginofError}{margin of error of total estimate}
#'     \item{LCBxxPct}{xx\% (default 95\%) lower confidence bound of total estimate}
#'     \item{UCBxxPct}{xx\% (default 95\%) upper confidence bound of total estimate}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey univar
#'
#' @seealso
#'   \describe{
#'   \item{\code{\link{cat_analysis}}}{ for categorical variable analysis}
#'   }
#'
#' @examples
#' dframe <- data.frame(
#'   siteID = paste0("Site", 1:100),
#'   wgt = runif(100, 10, 100),
#'   xcoord = runif(100),
#'   ycoord = runif(100),
#'   stratum = rep(c("Stratum1", "Stratum2"), 50),
#'   ContVar = rnorm(100, 10, 1),
#'   All_Sites = rep("All Sites", 100),
#'   Resource_Class = rep(c("Good", "Poor"), c(55, 45))
#' )
#' myvars <- c("ContVar")
#' mysubpops <- c("All_Sites", "Resource_Class")
#' mypopsize <- data.frame(
#'   Resource_Class = c("Good", "Poor"),
#'   Total = c(4000, 1500)
#' )
#' cont_analysis(dframe,
#'   vars = myvars, subpops = mysubpops, siteID = "siteID",
#'   weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
#'   stratumID = "stratum", popsize = mypopsize, statistics = "Mean"
#' )
#' @export
################################################################################

cont_analysis <- function(dframe, vars, subpops = NULL, siteID = NULL,
                          weight = "weight", xcoord = NULL, ycoord = NULL,
                          stratumID = NULL, clusterID = NULL, weight1 = NULL,
                          xcoord1 = NULL, ycoord1 = NULL, sizeweight = FALSE,
                          sweight = NULL, sweight1 = NULL, fpc = NULL,
                          popsize = NULL, vartype = "Local",
                          jointprob = "overton", conf = 95,
                          pctval = c(5, 10, 25, 50, 75, 90, 95),
                          statistics = c("CDF", "Pct", "Mean", "Total"),
                          All_Sites = FALSE) {

  # Assign NULL to vars_nondetect

  vars_nondetect <- NULL

  # Create a vector for error messages

  error_ind <- FALSE
  error_vec <- NULL

  # Create a data frame for warning messages

  warn_ind <- FALSE
  warn_df <- NULL
  fname <- "cont_analysis"

  # Ensure that the dframe argument was provided

  if (missing(dframe) | is.null(dframe)) {
    stop("\nThe dframe argument must be provided.\n")
  }

  # If the dframe argument is an sf object, extract coordinates from the geometry
  # column, assign values "xcoord" and "ycoord" to arguments xcoord and ycoord,
  # respectively, and drop the geometry column from the object

  if ("sf" %in% class(dframe)) {
    temp <- st_coordinates(dframe)
    xcoord <- "xcoord"
    dframe$xcoord <- temp[, "X"]
    ycoord <- "ycoord"
    dframe$ycoord <- temp[, "Y"]
    dframe <- st_set_geometry(dframe, NULL)
  }

  # If the dframe argument is a tibble or does not belong to class
  # "data.frame", coerce the argument to class "data.frame"

  if ("tbl_df" %in% class(dframe) | !("data.frame" %in% class(dframe))) {
    dframe <- as.data.frame(dframe)
  }

  # Ensure that the dframe argument does not contain zero rows

  if (nrow(dframe) == 0) {
    stop("\nThe dframe argument contains zero rows.\n")
  }

  # Ensure that unused levels are dropped from factor variables in the dframe
  # data frame

  dframe <- droplevels(dframe)

  # If no siteID is provided, set one that assumes each row is a unique site

  if (is.null(siteID)) {
    siteID <- "siteID"
    dframe$siteID <- paste("site", seq_len(nrow(dframe)), sep = "-")
  }

  # Ensure that the dframe data frame contains the site ID variable

  if (!(siteID %in% names(dframe))) {
    ind <- FALSE
    error_ind <- TRUE
    msg <- paste0("The name provided for the siteID argument, \"", siteID, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  } else {
    ind <- TRUE
  }

  # Check site IDs for repeat values and, as necessary, create unique site IDs and
  # output a warning message

  if (ind) {
    IDs <- dframe[, siteID]
    temp <- sapply(split(IDs, IDs), length)
    if (any(temp > 1)) {
      warn_ind <- TRUE
      temp_str <- vecprint(names(temp)[temp > 1])
      warn <- paste("The following site ID values occur more than once among the values that were \ninput to the function:\n", temp_str)
      act <- "Unique site ID values were created.\n"
      warn_df <- rbind(warn_df, data.frame(
        func = I(fname), subpoptype = NA,
        subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
      ))
      dframe[, siteID] <- uniqueID(dframe[, siteID])
    }
  }

  # Ensure that the dframe data frame contains the survey weight variable

  if (!(weight %in% names(dframe))) {
    error_ind <- TRUE
    msg <- paste0("The name provided for the weight argument, \"", weight, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  }

  # Assign names to the variables required for calculation of the finite
  # population correction factor

  if (is.null(fpc)) {
    fpcfactor_ind <- FALSE
    fpcsize <- NULL
    Ncluster <- NULL
    stage1size <- NULL
  } else {
    fpcfactor_ind <- TRUE
    if (is.null(clusterID)) {
      fpcsize <- "fpcsize"
      Ncluster <- NULL
      stage1size <- NULL
    } else {
      fpcsize <- NULL
      Ncluster <- "Ncluster"
      stage1size <- "stage1size"
    }
  }

  # Create a list containing names of survey design variables

  design_names <- list(
    siteID = siteID,
    weight = weight,
    xcoord = xcoord,
    ycoord = ycoord,
    stratumID = stratumID,
    clusterID = clusterID,
    weight1 = weight1,
    xcoord1 = xcoord1,
    ycoord1 = ycoord1,
    sweight = sweight,
    sweight1 = sweight1,
    fpcsize = fpcsize,
    Ncluster = Ncluster,
    stage1size = stage1size
  )

  # Ensure that a value was provided for the vars (response variable names)
  # argument

  if (is.null(vars)) {
    error_ind <- TRUE
    msg <- "A value must be provided for the vars (response variable names) argument.\n"
    error_vec <- c(error_vec, msg)
  }

  # If a value was not provided for the subpops (subpopulation names) argument,
  # assign the value "All_Sites" to the subpops argument and create a factor
  # named "All_Sites" in the dframe data frame that takes the value "All Sites"

  if (is.null(subpops)) {
    subpops <- "All_Sites"
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # If the user wants information for all sites together in addition to the
  # subpops, add the value "All_Sites" to the subpops argument and create a
  # factor named "All_Sites" in the dframe data frame that takes the value
  # "All Sites"

  if (!is.null(subpops) && All_Sites) {
    subpops <- c(subpops, "All_Sites")
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # Check input arguments

  temp <- input_check(dframe, design_names, NULL, vars, NULL, vars_nondetect,
    subpops, sizeweight, fpc, popsize, vartype, jointprob, conf,
    pctval = pctval, error_ind = error_ind, error_vec = error_vec
  )
  dframe <- temp$dframe
  vars <- temp$vars_cont
  vars_nondetect <- temp$vars_nondetect
  subpops <- temp$subpops
  popsize <- temp$popsize
  vartype <- temp$vartype
  jointprob <- temp$jointprob
  error_ind <- temp$error_ind
  error_vec <- temp$error_vec

  # Check argument statistics

  if (!is.vector(statistics)) {
    error_ind <- TRUE
    msg <- "Argument statistics must be a vector\n"
    error_vec <- c(error_vec, msg)
  } else if (!is.character(statistics)) {
    error_ind <- TRUE
    msg <- "Argument statistics must contain character values.\n"
    error_vec <- c(error_vec, msg)
  } else {
    tst <- statistics %in% c("CDF", "Pct", "Mean", "Total")
    if (any(!tst)) {
      error_ind <- TRUE
      msg <- "Argument statistics must contain only the values: 'CDF', 'Pct', 'Mean, and 'Total'.\n"
      error_vec <- c(error_vec, msg)
    }
  }

  # As necessary, output a message indicating that error messages were generated
  # during execution of the program

  if (error_ind) {
    error_vec <<- error_vec
    if (length(error_vec) == 1) {
      message("During execution of the program, an error message was generated.  The error \nmessage is stored in a vector named 'error_vec'.  Enter the following command \nto view the error message: errorprnt()\n")
    } else {
      message(paste("During execution of the program,", length(error_vec), "error messages were generated.  The error \nmessages are stored in a vector named 'error_vec'.  Enter the following \ncommand to view the error messages: errorprnt()\n"))
    }

    if (warn_ind) {
      warn_df <<- warn_df
      if (nrow(warn_df) == 1) {
        message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
      } else {
        message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
      }
    }
    stop("See the preceding message(s).")
  }

  # Assign a logical value to the indicator variable for a stratified sample

  stratum_ind <- !is.null(stratumID)

  # For a stratified sample, remove strata that contain a single site

  if (stratum_ind) {
    dframe[, stratumID] <- factor(dframe[, stratumID])
    stratum_levels <- levels(dframe[, stratumID])
    nstrata <- length(stratum_levels)
    ind <- FALSE
    for (i in 1:nstrata) {
      tst <- dframe[, stratumID] == stratum_levels[i]
      if (sum(tst) == 1) {
        warn_ind <- TRUE
        warn <- paste0("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
        act <- "Stratum was removed from the analysis.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = NA,
          subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
        ))
        dframe <- dframe[!tst, ]
        ind <- TRUE
      }
    }
    if (ind) {
      dframe[, stratumID] <- factor(dframe[, stratumID])
      stratum_levels <- levels(dframe[, stratumID])
      nstrata <- length(stratum_levels)
      dframe <- droplevels(dframe)
    }
  }

  # Assign a logical value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Create the survey design object

  design <- survey_design(
    dframe, siteID, weight, stratum_ind, stratumID, cluster_ind, clusterID,
    weight1, sizeweight, sweight, sweight1, fpcfactor_ind, fpcsize, Ncluster,
    stage1size, vartype, jointprob
  )

  # If popsize is not equal to NULL, then call either the postStratify or
  # calibrate function, as appropriate

  if (!is.null(popsize)) {
    if (all(class(popsize) %in% c("data.frame", "table", "xtabs"))) {
      if ("data.frame" %in% class(popsize)) {
        pnames <- names(popsize)[-ncol(popsize)]
      } else {
        pnames <- names(dimnames(popsize))
      }
      design <- postStratify(design, make.formula(pnames), popsize)
    } else {
      cnames <- cal_names(make.formula(names(popsize)), design)
      pop_totals <- numeric(length(cnames))
      names(pop_totals) <- cnames
      pop_totals[1] <- sum(popsize[[1]])
      k <- 2
      for (i in names(popsize)) {
        temp <- popsize[[i]]
        for (j in 2:length(temp)) {
          pop_totals[k] <- temp[j]
          k <- k + 1
        }
      }
      design <- calibrate(design, make.formula(names(popsize)), pop_totals)
    }
  }

  # If popsize is not equal to NULL and vartype equals "Local", then assign
  # adjusted weights to the appropriate weight variable(s) in the
  # design$variables data frame

  if (!is.null(popsize) && vartype == "Local") {
    if (cluster_ind) {
      design$variables$wgt2 <- weights(design) / design$variables$wgt1
    } else {
      design$variables$wgt <- weights(design)
    }
  }

  # Create the contsum (results) list

  contsum <- list(
    CDF = NULL,
    Pct = NULL,
    Mean = NULL,
    Total = NULL
  )

  # Assign the confidence bound multiplier

  mult <- qnorm(0.5 + (conf / 100) / 2)

  # Loop through all subpopulations (domains)

  for (itype in subpops) {
    lev_itype <- levels(dframe[, itype])
    nlev_itype <- length(lev_itype)

    # Loop through all response variables (vars)

    for (ivar in vars) {
      indx <- match(ivar, vars)

      # Calculate CDF estimates

      if ("CDF" %in% statistics) {
        temp <- cdf_est(
          contsum$CDF, dframe, itype, lev_itype, nlev_itype, ivar, design,
          design_names, vars_nondetect[indx], vartype, conf, mult, warn_ind,
          warn_df
        )
        contsum$CDF <- temp$cdfsum
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      }

      # Calculate percentile estimates

      if ("Pct" %in% statistics) {
        temp <- percentile_est(
          contsum$Pct, dframe, itype, lev_itype, nlev_itype, ivar, design,
          design_names, vars_nondetect[indx], conf, mult, pctval, warn_ind,
          warn_df
        )
        contsum$Pct <- temp$pctsum
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      }

      # Calculate mean estimates

      if ("Mean" %in% statistics) {
        temp <- mean_est(
          contsum$Mean, dframe, itype, lev_itype, nlev_itype, ivar, design,
          design_names, vars_nondetect[indx], vartype, conf, mult, warn_ind,
          warn_df
        )
        contsum$Mean <- temp$meansum
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      }

      # Calculate total estimates

      if ("Total" %in% statistics) {
        temp <- total_est(
          contsum$Total, dframe, itype, lev_itype, nlev_itype, ivar, design,
          design_names, vars_nondetect[indx], vartype, conf, mult, warn_ind,
          warn_df
        )
        contsum$Total <- temp$totalsum
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      }

      # End of the loop for response variables
    }

    # End of the loop for subpopulations
  }

  # As necessary, output a message indicating that warning messages were
  # generated during execution of the program

  if (warn_ind) {
    warn_df <<- warn_df
    if (nrow(warn_df) == 1) {
      message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
    } else {
      message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
    }
  }

  # Assign dimension names to the contsum data frames


  if (!is.null(contsum$CDF)) {
    dimnames(contsum$CDF) <- list(1:nrow(contsum$CDF), c(
      "Type", "Subpopulation", "Indicator", "Value", "nResp", "Estimate.P",
      "StdError.P", "MarginofError.P", paste0("LCB", conf, "Pct.P"),
      paste0("UCB", conf, "Pct.P"), "Estimate.U", "StdError.U", "MarginofError.U",
      paste0("LCB", conf, "Pct.U"), paste0("UCB", conf, "Pct.U")
    ))
    class(contsum$CDF) <- c("sp_CDF", class(contsum$CDF))
  }

  if (!is.null(contsum$Pct)) {
    dimnames(contsum$Pct) <- list(1:nrow(contsum$Pct), c(
      "Type", "Subpopulation", "Indicator", "Statistic", "nResp", "Estimate",
      "StdError", "MarginofError", paste0("LCB", conf, "Pct"),
      paste0("UCB", conf, "Pct")
    ))
  }

  if (!is.null(contsum$Mean)) {
    dimnames(contsum$Mean) <- list(1:nrow(contsum$Mean), c(
      "Type", "Subpopulation", "Indicator", "nResp", "Estimate", "StdError",
      "MarginofError", paste0("LCB", conf, "Pct"), paste0("UCB", conf, "Pct")
    ))
  }

  if (!is.null(contsum$Total)) {
    dimnames(contsum$Total) <- list(1:nrow(contsum$Total), c(
      "Type", "Subpopulation", "Indicator", "nResp", "Estimate", "StdError",
      "MarginofError", paste0("LCB", conf, "Pct"), paste0("UCB", conf, "Pct")
    ))
  }

  # Return the contsum object

  contsum
}

context("change analysis")

#
# Section for data input
#

# Read the data frame containing survey design and analysis variables
load(system.file("extdata", "NLA_IN.rda", package = "spsurvey"))

# Create a population size data frame
popsize <- data.frame(
  LAKE_ORGN = c("MAN_MADE", "NATURAL"),
  Total = c(6000, 14000)
)

# Create finite population correction factor objects
fpc1 <- 20000
fpc2a <- list(
  Urban = 5000,
  "Non-Urban" = 15000
)
fpc2b <- list(
  MAN_MADE = 6000,
  NATURAL = 14000
)
fpc3 <- c(
  Ncluster = 200,
  clusterID_1 = 100,
  clusterID_2 = 100,
  clusterID_3 = 100,
  clusterID_4 = 100
)
fpc4a <- list(
  Urban = c(
    Ncluster = 75,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  "Non-Urban" = c(
    Ncluster = 125,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)
fpc4b <- list(
  NATURAL = c(
    Ncluster = 130,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  MAN_MADE = c(
    Ncluster = 70,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)
#
# Section for the change data analysis function
#

# Subset the input data frame
dframe <- droplevels(subset(NLA_IN, YEAR != 2017))

# Assign response variable names to the vars_cat and vars_cont vectors
vars_cat <- c("BENT_MMI_COND_2017")
vars_cont <- c("ContVar")

# Assign subpopulation variable names to the subpops vector
subpops <- c("All_Sites")

# Create a population size data frame
popsize <- data.frame(
  All_Sites = c("Indiana Lakes"),
  Total = c(20000)
)

# Perform tests
Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD"
)

test_that("Change: Unstratified single-stage analysis", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", popsize = popsize
)

test_that("Change: with known population sizes", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", fpc = fpc1
)

test_that("Change: with finite population correction factor", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "LAKE_ORGN"
)

test_that("Change: Stratified single-stage analysis", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "LAKE_ORGN", fpc = fpc2b
)

test_that("Change: with finite population correction factor", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", vartype = "SRS"
)

test_that("Change: Unstratified two-stage analysis", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", popsize = popsize,
  vartype = "SRS"
)

test_that("Change: with known population sizes", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", fpc = fpc3, vartype = "SRS"
)

test_that("Change: with finite population correction factor", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "LAKE_ORGN", clusterID = "clusterID",
  weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
  vartype = "SRS"
)

test_that("Change: Stratified two-stage analysis", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "LAKE_ORGN", clusterID = "clusterID",
  weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
  popsize = popsize, vartype = "SRS"
)

test_that("Change: with known population sizes", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

Change_Estimates <- change_analysis(
  dframe = dframe, vars_cat = vars_cat,
  vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
  surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "LAKE_ORGN", clusterID = "clusterID",
  weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
  fpc = fpc4b, vartype = "SRS"
)

test_that("Change: with finite population correction factor", {
  expect_true(exists("Change_Estimates"))
  expect_equal(attributes(Change_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_mean)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_total)$class, "data.frame")
  expect_equal(attributes(Change_Estimates$contsum_median)$class, "data.frame")
  expect_equal(nrow(Change_Estimates$catsum), 2)
  expect_equal(nrow(Change_Estimates$contsum_mean), 1)
  expect_equal(nrow(Change_Estimates$contsum_total), 1)
  expect_equal(nrow(Change_Estimates$contsum_median), 2)
})

test_that("A warning (in message form) is produced", {
  expect_message(expect_error(change_analysis(
    dframe = dframe, vars_cat = vars_cat,
    vars_cont = vars_cont, test = c("mean", "total", "median"), subpops = subpops,
    surveyID = "YEAR", siteID = "UNIQUE_ID", weight = "XYZ", xcoord = "XCOORD",
    ycoord = "YCOORD", stratumID = "LAKE_ORGN", clusterID = "clusterID",
    weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
    fpc = fpc4b, vartype = "SRS"
  )))
})

test_that("Change for totals matches cont_analysis()", {
  set.seed(1)
  dframe <- data.frame(
     surveyID = rep(c("Survey 1", "Survey 2"), c(100, 100)),
     siteID = paste0("Site", 1:200),
     wgt = runif(200, 10, 100),
     xcoord = runif(200),
     ycoord = runif(200),
     ContVar = rnorm(200)
   )
   myvars <- c("ContVar")
   change <- change_analysis(dframe,
     vars_cont = myvars, 
     surveyID = "surveyID", siteID = "siteID", weight = "wgt",
     xcoord = "xcoord", ycoord = "ycoord", test = "total"
   )$contsum_total
   cont1 <- cont_analysis(dframe[dframe$surveyID == "Survey 1", ], 
                          vars = myvars, 
                          siteID = "siteID", xcoord = "xcoord", ycoord = "ycoord",
                          weight = "wgt", statistics = "Total")$Total
   cont2 <- cont_analysis(dframe[dframe$surveyID == "Survey 2", ], 
                          vars = myvars, 
                          siteID = "siteID", xcoord = "xcoord", ycoord = "ycoord",
                          weight = "wgt", statistics = "Total")$Total
   expect_equal(change$DiffEst, cont2$Estimate - cont1$Estimate)
   expect_equal(change$StdError, sqrt(cont2$StdError^2 + cont1$StdError^2))
})

###############################################################################
# Function:   power.dsgn (exported)
# Programmer: Tony Olsen
# Date: March 14, 2019
#'
#' Power calculation for multiple panel designs
#'
#' Calculates the power for trend detection for one or more variables, for one
#' or more panel designs, for one or more linear trends, and for one or more
#' significance levels.  The panel designs create a covariance model where the
#' model includes variance components for units, periods, the interaction of
#' units and periods, and the residual (or index) variance.
#'
#' @param ind_names  Vector of indicator names
#'
#' @param ind_values  Vector of indicator mean values
#'
#' @param unit_var  Vector of variance component estimates for unit variability
#'   for the indicators
#'
#' @param period_var  Vector of variance component estimates for period
#'   variability for the indicators
#'
#' @param unitperiod_var  Vector of variance component estimates for unit by
#'   period interaction variability for the indicators
#'
#' @param index_var  Vector of variance component estimates for index (residual)
#'   error for the indicators
#'
#' @param unit_rho  Correlation across units. Default is \code{1}.
#'
#' @param period_rho  Correlation across periods. Default is \code{0}.
#'
#' @param paneldsgn  A list of panel designs each as a matrix.  Each element of
#'   the list is a matrix with \code{dimnames} (dimensions: number of panels (rows) by
#'   number of periods (columns)) containing the number of units visited for
#'   each combination of panel and period.  Dimnames for columns must be
#'   able to be coerced into an integer (e.g., 2016).  All designs must span the same
#'   number of periods.  Typically, the panel designs are the output of the
#'   function \code{revisit_dsgn}.
#'
#' @param nrepeats  Either \code{NULL} or a list of matrices the same length as
#'   \code{paneldsgn} specifying the number of revisits made to units in a panel in the
#'   same period for each design.  Specifying \code{NULL} indicates that number of
#'   revisits to units is the same for all panels and for all periods and for
#'   all panel designs. The default is \code{NULL}, a single visit. Names must match
#'   list names in \code{paneldsgn}.
#'
#' @param trend_type  Trend type is either \code{"mean"} where trend is applied as
#'   percent trend in the indicator mean or \code{"percent"} where the trend is applied
#'   as percent trend in the proportion (percent) of the distribution that is
#'   below or above a fixed value. Default is \code{trend_type="mean"}
#'
#' @param ind_pct  When \code{trend_type} is equal to \code{"percent"}, a vector of the
#'   values of the indicator fixed value that defines the percent.  Default is
#'   NULL
#'
#' @param ind_tail  When trend_type is equal to \code{"percent"}, a character vector
#'   with values of either \code{"lower"} or \code{"upper"} for each indicator.  \code{"lower"}
#'   states that the percent is associated with the lower tail of the
#'   distribution and \code{"upper"} states that the percent is associated with the
#'   upper tail of the distribution. Default is \code{NULL}.
#'
#' @param trend  Single value or vector of assumed percent change from
#'   initial value in the indicator for each period. Assumes the trend is
#'   expressed as percent per period. Note that the trend may be either positive
#'   or negative. The default is \code{2}.
#'
#' @param alpha  Single value or vector of significance level for linear
#'   trend test, alpha, Type I error, level.  The default is \code{0.05}.
#'
#' @details Calculates the power for detecting a change in the mean for
#'   different panel design structures. The model incorporates unit, period,
#'   unit by period, and index variance components as well as correlation across
#'   units and across periods.  See references for methods.
#
#' @references
#'   Urquhart, N. S., W. S. Overton, et al. (1993) Comparing sampling designs
#'   for monitoring ecological status and trends: impact of temporal patterns.
#'   In: \emph{Statistics for the Environment.} V. Barnett and K. F. Turkman.
#'   John Wiley & Sons, New York, pp. 71-86.
#'
#'   Urquhart, N. S. and T. M. Kincaid (1999). Designs for detecting trends
#'   from repeated surveys of ecological resources. \emph{Journal of
#'   Agricultural, Biological, and Environmental Statistics}, \bold{4(4)},
#'   404-414.
#'
#'   Urquhart, N. S. (2012). The role of monitoring design in detecting trend in
#'   long-term ecological monitoring studies. In: \emph{Design and Analysis of
#'   Long-term Ecological Monitoring Studies.} R. A. Gitzen, J. J. Millspaugh,
#'   A. B. Cooper, and D. S. Licht (eds.). Cambridge University Press, New York,
#'   pp. 151-173.
#
#' @return A list with components \code{trend_type}, \code{ind_pct}, \code{ind_tail}, trend values
#'   across periods, periods (all periods included in one or more panel
#'   designs), significance levels, a five-dimensional array of power
#'   calculations (dimensions: panel, design names, periods, indicator names,
#'   trend names, \code{alpha_names}), an array of indicator mean values for each trend
#'   and the function call.
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#' @keywords survey
#'
#' @seealso
#'   \describe{
#'     \item{\code{\link{ppd_plot}}}{ to plot power curves for
#'       panel designs}
#'   }
#'
#' @examples
#' # Power for rotating panel with sample size 60
#' power_dsgn("Variable_Name",
#'   ind_values = 43, unit_var = 280, period_var = 4,
#'   unitperiod_var = 40, index_var = 90, unit_rho = 1, period_rho = 0,
#'   paneldsgn = list(NoR60 = revisit_dsgn(20,
#'     panels = list(NoR60 = list(
#'       n = 60, pnl_dsgn = c(1, NA),
#'       pnl_n = NA, start_option = "None"
#'     )), begin = 1
#'   )),
#'   nrepeats = NULL, trend_type = "mean", trend = 1.0, alpha = 0.05
#' )
#' @export
###############################################################################
power_dsgn <- function(ind_names, ind_values, unit_var, period_var,
                       unitperiod_var, index_var, unit_rho = 1, period_rho = 0, paneldsgn,
                       nrepeats = NULL, trend_type = "mean", ind_pct = NULL, ind_tail = NULL,
                       trend = 2, alpha = 0.05) {

  # number of indicators for power comparison
  n_ind <- length(ind_names)

  # number of trend change values for power comparison
  n_trend <- length(trend)
  trend_names <- paste("Trend_", round(trend, 1), "%", sep = "")

  # check that trend_type has correct possible values
  if (!(trend_type %in% c("mean", "percent"))) stop("\ntrend_type value not 'mean' or 'percent' ")

  # number of significance levels for power comparison
  n_alpha <- length(alpha)
  alpha_names <- paste("alpha_", alpha, sep = "")

  # number of designs for power comparison
  n_dsgn <- length(paneldsgn)

  # Create nrepeats revisit design if necessary
  if (is.null(nrepeats)) {
    nrepeats <- vector("list", n_dsgn)
    for (i in names(paneldsgn)) {
      nrepeats[[i]] <- paneldsgn[[i]]
      nrepeats[[i]][nrepeats[[i]] > 0] <- 1
    }
  }

  # number of periods for power comparison
  # First set up periods to calculate trend for each panel design based on periods when units are sampled
  period_power <- vector("list", n_dsgn)
  names(period_power) <- names(paneldsgn)
  period_values <- numeric(length = 0)
  for (i in names(paneldsgn)) {
    period_power[[i]] <- ifelse(apply(paneldsgn[[i]], 2, max) > 0, TRUE, FALSE)
    period_power[[i]] <- as.numeric(dimnames(paneldsgn[[i]])[[2]][period_power[[i]]])
    period_values <- c(period_values, period_power[[i]])
  }
  # find minimum and maximum period from designs that are monitored
  period_min <- min(period_values)
  period_max <- max(period_values)
  periods <- seq(period_min, by = 1, to = period_max)
  n_periods <- length(periods)

  # When trend_type = "percent", check if for lower or upper tail
  if (!is.null(ind_tail)) {
    if (any(!(ind_tail %in% c("lower", "upper")))) stop("\nValues must be lower or upper")
    lower.tail <- ind_tail == "lower"
  }

  # set up output array for calculated power for each indicator, panel design, periods, trend, and alpha
  pout <- array(NA, c(n_dsgn, n_periods, n_ind, n_trend, n_alpha))
  dimnames(pout) <- list(names(paneldsgn), periods, ind_names, trend_names, alpha_names)

  # set up change in mean for output and use in power calculations
  trend_value <- array(NA, c(n_trend, n_periods, n_ind))
  dimnames(trend_value) <- list(trend_names, as.character(periods), ind_names)

  # set up trend for power calculation based on percent per period and indicator value so
  # that trend is in units indicator units of change per period.

  for (ind in 1:length(ind_names)) {
    for (k in 1:length(trend_names)) {
      # calculate trend value
      if (trend_type == "mean") {
        trend_delta <- ind_values[ind] * trend[k] / 100
        trend_value[k, , ind] <- ind_values[ind] + trend_delta * seq(0, to = n_periods - 1, by = 1)
      }
      if (trend_type == "percent") {
        # find cut point value for "pct" which is assumed to have a normal distribution
        ind_sd <- sqrt(unit_var[ind] + index_var[ind])
        ind_cut <- qnorm(ind_pct[ind] / 100, ind_values[ind], ind_sd, lower.tail = lower.tail)
        # function to search for shift in mean required to change the %Good by trend % change
        mean_change <- function(x, ind_pct, ind_cut, ind_sd, lower.tail = lower.tail) {
          abs(ind_pct / 100 - pnorm(ind_cut, x, ind_sd, lower.tail = lower.tail))
        }
        # find change in mean required to achieve change in percent
        for (i in 1:n_periods) {
          tmp <- optimize(mean_change,
            lower = ind_values[ind] - 3.1 * ind_sd,
            upper = ind_values[ind] + 3.1 * ind_sd,
            ind_pct = ind_pct[ind] + trend[k] * (i - 1),
            ind_cut = ind_cut, ind_sd = ind_sd,
            lower.tail = lower.tail
          )
          trend_value[k, i, ind] <- tmp$minimum
        }
      }

      # power for individual panal designs
      for (j in names(paneldsgn)) {
        nperiod <- ncol(paneldsgn[[j]])
        npanel <- nrow(paneldsgn[[j]])



        # Assume increasing number of periods monitored to compute power
        for (i_period in 1:length(period_power[[j]])) {
          if (i_period == 1) {
            # set up initial period variables for x matrix
            period_xmat <- rep(1, npanel)
            if (trend_type == "percent") {
              period_y <- rep(trend_value[k, 1, ind], npanel)
            }
          }
          if (i_period != 1) {
            # create the covariance matrix for jth design and ith end period
            tmp <- matrix(paneldsgn[[j]][, c(as.character(period_power[[j]][1:i_period]))], nrow = npanel)
            tmp2 <- matrix(nrepeats[[j]][, c(as.character(period_power[[j]][1:i_period]))], nrow = npanel)
            panel_cov <- cov_panel_dsgn(tmp, tmp2,
              unit_var = unit_var[ind], period_var = period_var[ind],
              unitperiod_var = unitperiod_var[ind], index_var = index_var[ind],
              unit_rho = unit_rho, period_rho = period_rho
            )

            # compute the power at ith monitoring time
            # define linear trend xmat for the panel
            period_diff <- period_power[[j]][i_period] - period_power[[j]][1]
            period_xmat <- c(period_xmat, rep(period_diff + 1, npanel))
            if (trend_type == "percent") {
              period_y <- c(period_y, rep(trend_value[k, i_period, ind], npanel))
            }
            xmat <- cbind(rep(1, npanel * i_period), period_xmat)

            # remove time periods with no sample units
            indx <- as.vector(tmp) > 0
            xmat <- xmat[indx, ]
            if (trend_type == "percent") {
              y <- period_y[indx]
            }

            # calculate inverse of covariance matrix and calculate trend covariance matrix
            phi_inv <- solve(panel_cov$cov[indx, indx])
            bhat_cov <- solve(t(xmat) %*% phi_inv %*% xmat)
            if (trend_type == "percent") {
              bhat <- bhat_cov %*% t(xmat) %*% phi_inv %*% y
            }

            # calculate trend slope standard error and power
            indexse <- sqrt(bhat_cov[2, 2])
            ifelse(trend_type == "mean", bhat.std <- trend_delta / indexse,
              bhat.std <- bhat[2] / indexse
            )
          } else {
            bhat.std <- 0
          }

          # calculate power for all alpha
          for (m in 1:length(alpha_names)) {
            pout[
              j, as.character(period_power[[j]][i_period]), ind_names[ind],
              trend_names[k], alpha_names[m]
            ] <-
              (pnorm(qnorm(alpha[m] / 2) - bhat.std)) +
              (1 - pnorm(qnorm(1 - (alpha[m] / 2)) - bhat.std))
          }
        }
      }
    }
  }

  # Fill in NAs for time poriods when no sampling occurs by repeating power from
  # last time period sampled, that is, has power calculated
  pwr_fill <- function(jnk) {
    keep <- jnk[1]
    for (i in 2:length(jnk)) {
      ifelse(is.na(jnk[i]), jnk[i] <- keep, keep <- jnk[i])
    }
    return(jnk)
  }
  if (n_trend > 1) {
    for (i in names(paneldsgn)) {
      for (j in 1:length(ind_names)) {
        for (k in 1:length(alpha_names)) {
          pout[i, , j, , k] <- apply(pout[i, , j, , k], 2, pwr_fill)
        }
      }
    }
  }


  # output list
  dsgn_pwr <- vector("list", 11)
  names(dsgn_pwr) <- c(
    "design", "period", "indicator", "trend", "alpha",
    "trend_type", "ind_pct", "ind_tail", "trend_change", "dsgn_power", "call"
  )
  dsgn_pwr$design <- names(paneldsgn)
  dsgn_pwr$period <- periods
  dsgn_pwr$indicator <- ind_names
  dsgn_pwr$trend <- trend
  dsgn_pwr$alpha <- alpha
  dsgn_pwr$trend_type <- trend_type
  dsgn_pwr$ind_pct <- ind_pct
  dsgn_pwr$ind_tail <- ind_tail
  dsgn_pwr$trend_change <- trend_value
  dsgn_pwr$dsgn_power <- pout
  dsgn_pwr$call <- sys.call()

  class(dsgn_pwr) <- "powerpaneldesign"

  return(dsgn_pwr)
}

###############################################################################
# Function: insideAreaGridCell (not exported)
# Programmer: Tom Kincaid
# Date: July 8, 2019
#
#' Calculate Clipped Feature Areas in a Set of Grid Cells
#'
#' For each grid cell, this function calculates the clipped area of each polygon
#' feature contained in the cell.
#'
#' @param sfobject The \code{sf} polygon object.
#'
#' @param rdx.u Vector of cell IDs.
#'
#' @param xc Vector of x-coordinates for the grid cells.
#'
#' @param yc Vector of y-coordinates for the grid cells.
#'
#' @param dx The x-axis grid cell dimension.
#'
#' @param dy The y-axis grid cell dimension.
#'
#' @return Data frame containing the following variables: \code{cellID}, \code{featureArea},
#'   and \code{featureID}.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
###############################################################################

insideAreaGridCell <- function(sfobject, rdx.u, xc, yc, dx, dy) {

  # Create a data frame for results

  cell.df <- data.frame()

  # Calculate clipped feature areas for each cell

  for (i in 1:length(xc)) {
    temp <- rbind(
      c(xc[i] - dx, yc[i] - dy), c(xc[i], yc[i] - dy),
      c(xc[i], yc[i]), c(xc[i] - dx, yc[i]),
      c(xc[i] - dx, yc[i] - dy)
    )
    sfcell <- st_sf(st_sfc(st_polygon(list(temp)), crs = st_crs(sfobject)))
    tempsf <- st_intersection(sfobject, sfcell)
    cell.df <- rbind(cell.df, cbind(
      cellID = rdx.u[i],
      featureArea = as.numeric(st_area(tempsf)),
      featureID = tempsf$id
    ))
  }

  # Return the data frame containing results

  return(cell.df)
}

#' Print summaries of sampling frames, design sites, and analysis data.
#'
#' @name print
#' @method print summary.sp_frame
#'
#' @description \code{print()} prints summaries of sampling frames, design sites,
#'   and analysis data.
#' 
#' @param x An object output from \code{grts()}, \code{irs()}, \code{summary()}, or \code{sp_summary()}.
#' @param ... Additional print and summary arguments
#'
#' @details When \code{x} is output from \code{grts()} or \code{irs()}, \code{print()} returns
#'   a summary of site counts for each sites object (\code{sites_legacy}, \code{sites_base},
#'   \code{sites_near}, and \code{sites_over}). These site counts are tabled by strata and 
#'   unequal probability levels if applicable. When \code{x} is output from \code{summary()}
#'   or \code{sp_summary()}, relevant summaries by variables specified in a formula
#'   are returned.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' summary(sp_frame(NE_Lakes), AREA_CAT ~ ELEV_CAT)
#' samp <- grts(NE_Lakes, n_base = c(small = 15, large = 20), stratum_var = "AREA_CAT", n_over = 20)
#' samp
#' }
#' @noRd
print.summary.sp_frame <- function(x, ...) {
  
  if (inherits(x, "table")) {
    print.table(x, ...)
  }
  
  if (inherits(x, "list")) {
    for (i in seq_along(x)) {
      cat(names(x)[i], ": ", sep = "")
      cat("\n")
      print(x[[i]], ...)
      cat("\n")
    }
  }
}
#' @export
#' @noRd
print.sp_summary.sp_frame <- print.summary.sp_frame

#' @name print
#' @method print summary.sp_design
#' @export
#' @noRd
print.summary.sp_design <- function(x, ...) {
  print.summary.sp_frame(x, ...)
}

#' @export
#' @noRd
print.sp_summary.sp_design <- print.summary.sp_design

#' @name print
#' @method print sp_design
#' @export
#' @noRd
print.sp_design <- function(x, ...) {

  if (!is.null(x$design$stratum_var) & !is.null(x$design$caty_var)) {
    object <- summary(x, formula = siteuse ~ stratum * caty, ...)
  } else if (is.null(x$design$stratum_var) & !is.null(x$design$caty_var)) {
    object <- summary(x, formula = siteuse ~ caty, ...)
  } else if (!is.null(x$design$stratum_var) & is.null(x$design$caty_var)) {
    object <- summary(x, formula = siteuse ~ stratum, ...)
  } else {
    object <- summary(x, ...)
  }
  # if (is.null(x$stratum_var)) {
  #   object <- object[-which(names(object) == "stratum")]
  # }
  cat("Summary of Site Counts: ")
  cat("\n")
  cat("\n")
  print(object)
}

###############################################################################
# Function: grtspts_mindis (not exported)
# Programmer:  Tony Olsen
# Date: September 28, 2020
#
#' Select a grts sample with minimum distance between sites.
#'
#' @param mindis Minimum distance required between sites in sample.
#'
#' @param sframe The \code{sf} object containing variables: \code{id} and \code{ip}.
#'
#' @param samplesize The sample size required for the stratum.
#'
#' @param stratum Character string that identifies stratum membership for each element
#'   in the frame. Cannot be \code{NULL}.
#'
#' @param maxtry Number of maximum attempts to ensure minimum distance between sites.
#'   Default is \code{10}.
#'
#' @param legacy_option Logical variable where if equal to \code{TRUE}, legacy sites are
#'   to be incorporated into survey design.
#'
#' @param legacy_var Character value for name of column for legacy site variable.
#'   Default is \code{NULL}.
#'
#' @param warn_ind  A logical value where \code{TRUE} indicates a warning message.
#'   Used for internal collection of messages only.
#'
#' @param warn_df A data frame containing messages warning of potential issues.
#'   Used for internal collection of messages only.
#'
#'
#' @return sites A list of \code{sf} object of sample points, an \code{sf} object of over sample
#'   points if any, warning indicator and warning messages \code{data.frame}.
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
###############################################################################

grtspts_mindis <- function(mindis, sframe, samplesize, stratum, maxtry = 10,
                           legacy_option = NULL, legacy_var = NULL,
                           warn_ind = NULL, warn_df = NULL) {

  # select initial set of sites
  sites <- sframe[get_address(sframe$xcoord, sframe$ycoord, rand = TRUE), ]
  s <- UPpivotal(sites$ip)
  sites_base <- sites[round(s) == 1, ]

  # calculate distance between sites
  site_dist <- st_distance(sites_base)
  class(site_dist) <- "numeric"
  nr <- nrow(sites_base)

  # find sites less than mindis and set to FALSE otherwise set to TRUE
  keep <- apply(site_dist, 1, function(x) {
    ifelse(any(x[x > 0] < mindis), FALSE, TRUE)
  })

  # if any legacy sites keep those sites
  if (legacy_option == TRUE) {
    keep[sites_base$legacy] <- TRUE
  }

  # see if any sites are less than mindis and check until none or max tries
  ntry <- 1
  while (any(!keep)) {

    # identify sites that will be treated as legacy probability sites in sample frame
    sframe$probdis <- FALSE
    sframe$probdis[sframe$idpts %in% sites_base$idpts[keep]] <- TRUE

    # if any true legacy sites add them to sites to be kept
    if (legacy_option == TRUE) {
      sframe$probdis[sframe$legacy] <- TRUE
    }

    # Adjust initial inclusion probabilities to account for current mindis sites
    # and any legacy sites
    sframe$ip <- grtspts_ipleg(sframe$ip_init, sframe$probdis)

    # select new sites that include legacy sites
    sites <- sframe[get_address(sframe$xcoord, sframe$ycoord, rand = TRUE), ]
    s <- UPpivotal(sites$ip)
    sites_base <- sites[round(s) == 1, ]

    # calculate distance between sites
    site_dist <- st_distance(sites_base)
    class(site_dist) <- "numeric"
    nr <- nrow(sites_base)

    # identify sites less than mindis
    keep <- apply(site_dist, 1, function(x) {
      ifelse(any(x[x > 0] < mindis), FALSE, TRUE)
    })

    # Change to TRUE if any legacy sites
    if (legacy_option == TRUE) {
      keep[sites_base$legacy] <- TRUE
    }

    # check if maxtry reached. If so write out warning message
    if (ntry >= maxtry) {
      keep <- rep(TRUE, nr)
      warn <- paste0("Minimum distance between sites not attained after ", maxtry, " attempts.")
      if (warn_ind) { # this code has a bug but is never called (the rbind fails because warn_df has a column warn and the bind has a column warning)
        warn_df <- rbind(warn_df, data.frame(
          stratum = stratum, func = I("grtspts_mindis"),
          warning = warn
        ))
      } else {
        warn_ind <- TRUE
        warn_df <- data.frame(stratum = stratum, func = I("grtspts_mindis"), warning = warn)
      }
    } else {
      ntry <- ntry + 1
    }
  } # end of ntry loop

  # drop internal variables
  tmp <- names(sites_base)
  sites_base <- subset(sites_base, select = tmp[!(tmp %in% c("probdis", "geometry"))])

  # Put sites in reverse hierarchical order
  sites_base <- rho(sites_base)
  sites_base$siteuse <- NA
  sites_base$replsite <- NA

  sites <- list(
    sites = sites_base,
    warn_ind = warn_ind, warn_df = warn_df
  )

  invisible(sites)
}

###############################################################################
# Function: dsgn_check (not exported)
# Programmer: Tony Olsen
# Date: January 22, 2021
#'
#' Check the input associated with the survey design list object and the sample frame
#'
#' @param sframe Sample frame as an \code{sf} object.
#'
#' @param sf_type Geometry type for sample frame: point, linear, or area.
#'
#' @param legacy_sites An \code{sf} object of any legacy sites to be included in the survey
#'   design.
#'
#' @param legacy_option Logical variable where if equal to \code{TRUE}, legacy sites are
#'   to be incorporated into survey design.
#'
#' @param stratum Single character value or character vector that identifies the strata for
#'   the design.  Default is \code{NULL} which means no stratification and all elements in the sample
#'   frame are assumed to be included.
#'
#' @param seltype Single character value or character vector that identifies the type of
#'   random selection, which must be one of following: \code{"equal"} for equal probability selection,
#'   \code{"unequal"} for unequal probability selection by the categories specified in \code{caty_n} or
#'   \code{"proportional"} for unequal probability selection proportional to the auxiliary variable
#'   \code{aux_var}. If single character, then seltype applies to all strata. If vector, then each
#'   stratum may have different selection type. Default is single character value of \code{"equal"}.
#'
#' @param n_base The sample size required. If single stratum, then single numeric value.
#'   If sample is stratified, then numeric vector with same length as \code{"stratum"} and sample sizes
#'   required in same order as strata in \code{"stratum"}. Must be specified.
#'
#' @param caty_n If design is not stratified and seltype is \code{"unequal"}, a named character vector
#'   with the expected sample size for each category specified in variable \code{caty_var}. If the design
#'   is stratified, then either a named character vector with the expected sample size for each
#'   category for all strata or if the expected sample size for each category may differ, then
#'   a list of named character vectors with the expected sample size for each category in the
#'   stratum. The list must be in same order as the \code{"stratum"} variable. For each stratum,
#'   the sum of \code{caty_n} values must equal \code{n_base} for that stratum. Default is \code{NULL}.
#'
#' @param n_over If seltype is \code{"equal"} and is not stratified, a numeric value specifying the
#'   over sample size requested. If seltype is \code{"equal"} and is stratified either a numeric value
#'   specifying the over sample size that will be applied to each stratum or a numeric vector
#'   specifying the over sample size for each stratum listed in same order as \code{"strata"}.
#'   If seltype is \code{"unequal"} and is not stratified, a named character vector with the over sample size
#'   for each category where names are the same as \code{"caty_n"}. If seltype is \code{"unequal"} and is
#'   stratified,  either a numeric vector specifying the over sample size for each category in
#'   \code{"caty_n"} that will be applied to each stratum or a list of named numeric vectors with
#'   the over sample size for each \code{"caty_n"} category for each stratum. List must be in same order
#'   as the \code{"stratum"} variable order. Default is \code{NULL}.
#'
#' @param n_near Numeric value specifying the number of nearby points to select as
#'   possible replacement sites if a site cannot be sampled. Default is \code{NULL}. If specified,
#'   must be integer from \code{1} to \code{10}.
#'
#' @param stratum_var Character string containing the name of the column from
#'   \code{sframe} that identifies stratum membership for each element in the frame.
#'   If stratum equals \code{NULL}, the design is unstratified and all elements in sample frame
#'   are eligible to be selected in the sample. The default is \code{NULL}.
#'
#' @param caty_var Character string containing the name of the column from
#'   \code{sframe} that identifies the unequal probability category for each element
#'   in the frame.  Default is \code{NULL}.
#'
#' @param aux_var Character string that is the name of the column from \code{sframe} that
#'   identifies the auxiliary variable value for each element in the sample frame
#'   that will be used to calculate inclusion probabilities when the survey design
#'   specifies that the selection type (seltype) is \code{"proportional"}. Default is \code{NULL}.
#'
#' @param legacy_var Character value for name of column for legacy site variable.
#'   Default is \code{NULL}.
#'
#' @param mindis Numeric value for the minimum distance required between elements
#'   in the sample. If design is stratified, then mindis applies only within each stratum.
#'   Units must be the same units as in \code{sf} geometry. Default is \code{NULL}.
#'
#' @param DesignID Name for the design, which is used to create a site ID for
#'   each site.  Default is \code{"Site"}.
#'
#' @param SiteBegin Number to use for first site in the design.  Default is \code{1}.
#'
#' @param maxtry Number of maximum attempts to ensure minimum distance (mindis) between sites.
#'   Default is \code{10}.
#'
#'  @param projcrs_check A check for whether the coordinates are projected. If \code{TRUE},
#'    an error is returned if coordinates are not projected. If \code{FALSE}, the
#'    check is not performed and the raw coordinates in the geometry column of
#'    \code{sframe} (and \code{legacy_sites} if provided) are used.
#'
#'
#' @return Nothing is returned. If errors are found they are collected and written out.
#'   One or more errors will cause the call to \code{grts()} to stop.
#'
#'
#' @author Tony Olsen email{olsen.tony@@epa.gov}
#'
#' @noRd
###############################################################################
dsgn_check <- function(sframe, sf_type, legacy_sites, legacy_option, stratum, seltype, n_base, caty_n,
                       n_over, n_near, stratum_var, caty_var, aux_var,
                       legacy_stratum_var, legacy_caty_var, legacy_aux_var,
                       legacy_var, mindis,
                       DesignID, SiteBegin, maxtry, projcrs_check) {

  # Create a data frame for stop messages
  stop_ind <- FALSE
  stop_df <- NULL

  # check that coordinates are NA or geographic # | st_is_longlat(sframe))
  if (projcrs_check & is.na(st_crs(sframe))) {
    stop_ind <- TRUE
    stop_mess <- "The coordinate reference system (crs) for sframe is NA. The coordinate reference system for sframe should instead use projected coordinates. For more information on geographic and projected coordinates, see spsurvey's \"Start Here\" vignette by running vignette(\"start-here\", \"spsurvey\"). To override the check for projected coordinates, set projcrs_check = FALSE."
    stop_df <- rbind(stop_df, data.frame(func = I("sframe"), I(stop_mess)))
  }

  if (projcrs_check & st_is_longlat(sframe)) {
    stop_ind <- TRUE
    stop_mess <- "The coordinate reference system (crs) for sframe is geographic. The coordinate reference system for sframe should instead use projected coordinates. For more information on geographic and projected coordinates, see spsurvey's \"Start Here\" vignette by running vignette(\"start-here\", \"spsurvey\"). To override the check for projected coordinates, set projcrs_check = FALSE."
    stop_df <- rbind(stop_df, data.frame(func = I("sframe"), I(stop_mess)))
  }

  # check that legacy and sframe coordinates match
  if (!is.null(legacy_sites)) {
    if (sum(is.na(st_crs(sframe)), is.na(st_crs(legacy_sites))) == 1) {
      stop_ind <- TRUE
      stop_mess <- "sframe and legacy_sites must have the same crs. If crs should be ignored completely, run st_crs(sframe) <- NA and st_crs(legacy_sites) <- NA"
      stop_df <- rbind(stop_df, data.frame(func = I("sframe"), I(stop_mess)))
    } else if (st_crs(sframe) != st_crs(legacy_sites)) {
      stop_ind <- TRUE
      stop_mess <- "sframe and legacy_sites must have the same crs. If crs should be ignored completely, run st_crs(sframe) <- NA and st_crs(legacy_sites) <- NA"
      stop_df <- rbind(stop_df, data.frame(func = I("sframe"), I(stop_mess)))
    }
  }

  # check that sframe has required variables for stratum, caty, aux and legacy
  # If stratum_var is provided, does the attribute exist in sframe
  if (!is.null(stratum_var)) {
    if (match(stratum_var, names(sframe), nomatch = 0) == 0) {
      stop_ind <- TRUE
      stop_mess <- "The value provided for stratum variable does not exist as a variable in sframe."
      stop_df <- rbind(stop_df, data.frame(func = I("stratum_var"), I(stop_mess)))
    }
  }

  # If caty_var is provided, does the attribute exist in sframe
  if (!is.null(caty_var)) {
    if (match(caty_var, names(sframe), nomatch = 0) == 0) {
      stop_ind <- TRUE
      stop_mess <- "The value provided for unequal probability category variable does not exist as a variable in sframe."
      stop_df <- rbind(stop_df, data.frame(func = I("caty_var"), I(stop_mess)))
    }
  }

  # If aux_var is provided, does the attribute exist in sframe
  if (!is.null(aux_var)) {
    if (match(aux_var, names(sframe), nomatch = 0) == 0) {
      stop_ind <- TRUE
      stop_mess <- "The value provided for the auxillary variable for proportional sampling does not exist as a variable in sframe."
      stop_df <- rbind(stop_df, data.frame(func = I("aux_var"), I(stop_mess)))
    }
    # ensure class for aux variable is numeric
    if (!is.numeric(sframe[[aux_var]])) {
      stop_ind <- TRUE
      stop_mess <- "The auxillary variable in sample frame for proportional sampling must be numeric."
      stop_df <- rbind(stop_df, data.frame(func = I("aux_var"), I(stop_mess)))
    } else {
      # check that values are > 0.
      if (any(sframe[[aux_var]] <= 0)) {
        stop_ind <- TRUE
        stop_mess <- "The auxillary variable for proportional sampling must have all values greater than zero"
        stop_df <- rbind(stop_df, data.frame(func = I("aux_var"), I(stop_mess)))
      }
    }
  }

  # If legacy_var is provided, does the attribute exist in sframe
  if (sf_type == "sf_point" & !is.null(legacy_var)) {
    if (match(legacy_var, names(sframe), nomatch = 0) == 0) {
      stop_ind <- TRUE
      stop_mess <- "The value provided for the variable identifying legacy sites does not exist as a variable in sframe."
      stop_df <- rbind(stop_df, data.frame(func = I("legacy_var"), I(stop_mess)))
    }
  }

  ### Check legacy_sites sf object if present
  if (sf_type %in% c("sf_point", "sf_linear", "sf_area") & !is.null(legacy_sites)) {
    # check that legacy_sites has required variables for stratum, caty, aux and legacy
    # If stratum_var is provided, does the attribute exist
    if (!is.null(stratum_var) & is.null(legacy_stratum_var)) {
      if (match(stratum_var, names(legacy_sites), nomatch = 0) == 0) {
        stop_ind <- TRUE
        stop_mess <- "The value provided for stratum variable does not exist as a variable in legacy_sites."
        stop_df <- rbind(stop_df, data.frame(func = I("stratum_var"), I(stop_mess)))
      }
    }
    # If caty_var is provided, does the attribute exist
    if (!is.null(caty_var) & is.null(legacy_caty_var)) {
      if (match(caty_var, names(legacy_sites), nomatch = 0) == 0) {
        stop_ind <- TRUE
        stop_mess <- "The value provided for caty variable does not exist as a variable in legacy_sites."
        stop_df <- rbind(stop_df, data.frame(func = I("stratum_var"), I(stop_mess)))
      }
    }
    # If aux_var is provided, does the attribute exist
    if (!is.null(aux_var) & is.null(legacy_aux_var)) {
      if (match(aux_var, names(legacy_sites), nomatch = 0) == 0) {
        stop_ind <- TRUE
        stop_mess <- "The value provided for aux variable does not exist as a variable in legacy_sites."
        stop_df <- rbind(stop_df, data.frame(func = I("stratum_var"), I(stop_mess)))
      }
    }
    # If legacy_var is provided, does the attribute exist
    if (!is.null(legacy_var)) {
      if (match(legacy_var, names(legacy_sites), nomatch = 0) == 0) {
        stop_ind <- TRUE
        stop_mess <- "The value provided for legacy variable does not exist as a variable in legacy_sites."
        stop_df <- rbind(stop_df, data.frame(func = I("stratum_var"), I(stop_mess)))
      }
    }
  }


  ##### Check design components to ensure they provide what is required.

  # check if stratum is provided and values are in sframe
  if (!is.null(stratum)) {
    if (is.null(stratum_var)) {
      stop_ind <- TRUE
      stop_mess <- "Design is stratified and no 'stratum_var' is provided."
      stop_df <- rbind(stop_df, data.frame(func = I("stratum_var"), I(stop_mess)))
    } else {
      if (any(stratum %in% unique(sframe[[stratum_var]]) == FALSE)) {
        stop_ind <- TRUE
        stop_mess <- paste0("Not all stratum values are in sample frame.")
        stop_df <- rbind(stop_df, data.frame(func = I("stratum"), I(stop_mess)))
      }
    }
  }

  # check seltype
  if (any(seltype %in% c("equal", "unequal", "proportional") == FALSE)) {
    stop_ind <- TRUE
    stop_mess <- paste0("seltype must be 'equal', 'unequal' or 'proportional'.")
    stop_df <- rbind(stop_df, data.frame(func = I("seltype"), I(stop_mess)))
  }

  # check seltype when caty_var and aux_var provided
  if (all(seltype %in% "unequal") | all(seltype %in% "proportional")) {
    if (!is.null(caty_var) & !is.null(aux_var)) {
      stop_ind <- TRUE
      stop_mess <- paste0("aux_var and caty_var cannot both be specified when all elements of seltype are the same.")
      stop_df <- rbind(stop_df, data.frame(func = I("seltype mismatch"), I(stop_mess)))
    }
  }

  # check caty_var and caty_n are provided together
  if ((is.null(caty_var) & !is.null(caty_n)) | (!is.null(caty_var) & is.null(caty_n))) {
    stop_ind <- TRUE
    stop_mess <- paste0("caty_n and caty_var must be provided together.")
    stop_df <- rbind(stop_df, data.frame(func = I("caty_var and caty_n"), I(stop_mess)))
  }

  # check n_base length and stratum_var are provided together
  if (length(n_base) > 1 & is.null(stratum_var)) {
    stop_ind <- TRUE
    stop_mess <- paste0("if the length of n_base is larger than 1 then stratification is assumed and stratum_var must be provided.")
    stop_df <- rbind(stop_df, data.frame(func = I("n_base and stratum_var"), I(stop_mess)))
  }

  # check names of caty_n when it is a list
  if (is.list(caty_n) & is.null(names(caty_n))) {
    stop_ind <- TRUE
    stop_mess <- paste0("caty_n must be a named list (and these names must match the strata)")
    stop_df <- rbind(stop_df, data.frame(func = I("caty_n names"), I(stop_mess)))
  }


  # check n_base
  if (any(n_base <= 0)) {
    stop_ind <- TRUE
    stop_mess <- paste0("Sample size must be integers greater than 0.")
    stop_df <- rbind(stop_df, data.frame(func = I("n_base"), I(stop_mess)))
  }

  # check total sample size
  if (sf_type == "sf_point") {
    if (length(stratum) > 1) {
      if (any(sapply(stratum, function(x) n_base[x] > NROW(sframe[sframe[[stratum_var]] == x, , drop = FALSE])))) {
        stop_ind <- TRUE
        stop_mess <- paste0("Each stratum must have a sample size no larger than the number of rows in 'sframe' representing that stratum")
        stop_df <- rbind(stop_df, data.frame(func = I("n_base"), I(stop_mess)))
      }
    } else {
      if (n_base > NROW(sframe)) {
        stop_ind <- TRUE
        stop_mess <- paste0("Sample size must be no larger than the number of rows in 'sframe'")
        stop_df <- rbind(stop_df, data.frame(func = I("n_base"), I(stop_mess)))
      }
    }
  }

  # check caty_n
  if (!is.null(caty_n)) {
    if (!is.list(caty_n)) {
      if (any(names(caty_n) %in% unique(sframe[[caty_var]]) == FALSE)) {
        stop_ind <- TRUE
        stop_mess <- paste0("Not all caty_n values are in sample frame.")
        stop_df <- rbind(stop_df, data.frame(func = I("caty_n"), I(stop_mess)))
      }
      if (any(unique(sframe[[caty_var]]) %in% names(caty_n) == FALSE)) {
        stop_ind <- TRUE
        stop_mess <- paste0("Not all caty_var values are in caty_n names.")
        stop_df <- rbind(stop_df, data.frame(func = I("caty_n"), I(stop_mess)))
      }
      tst <- function(x, caty_n) {
        x != sum(caty_n)
      }
      if (any(sapply(n_base, tst, caty_n))) {
        stop_ind <- TRUE
        stop_mess <- paste0("Sum of caty_n values do not equal n_base.")
        stop_df <- rbind(stop_df, data.frame(func = I("caty_n"), I(stop_mess)))
      }
    }
    if (is.list(caty_n)) {
      if (any(names(caty_n) %in% stratum == FALSE)) {
        stop_ind <- TRUE
        stop_mess <- paste0("Names for caty_n list are not values in 'stratum' variable.")
        stop_df <- rbind(stop_df, data.frame(func = I("caty_n"), I(stop_mess)))
        stop_mess <- paste0("For each stratum make sure caty_n values in 'caty_var' variable.")
        stop_df <- rbind(stop_df, data.frame(func = I("caty_n"), I(stop_mess)))
      }

      if (any(stratum %in% names(caty_n) == FALSE)) {
        stop_ind <- TRUE
        stop_mess <- paste0("Values in 'stratum' variable are not names in caty_n list.")
        stop_df <- rbind(stop_df, data.frame(func = I("caty_n"), I(stop_mess)))
        stop_mess <- paste0("For each stratum make sure all 'caty_var' variable values are in caty_n.")
        stop_df <- rbind(stop_df, data.frame(func = I("caty_n"), I(stop_mess)))
      }

      if (any(sapply(stratum, function(x) sum(caty_n[[x]]) != n_base[[x]]))) {
        stop_ind <- TRUE
        stop_mess <- paste0("The sum of the 'caty_n' values in each strata must match the value in n_base that corresponds to the respective strata")
        stop_df <- rbind(stop_df, data.frame(func = I("caty_n"), I(stop_mess)))
      }
    }
  }

  # check n_over
  if (!is.null(n_over)) {
    if (is.null(stratum) | length(stratum) == 1) {
      if (any(seltype %in% c("equal", "proportional"))) {
        if (n_over < 0) {
          stop_ind <- TRUE
          stop_mess <- paste0("n_over value must be zero or positive.")
          stop_df <- rbind(stop_df, data.frame(func = I("n_over"), I(stop_mess)))
        }
      }
      if (any(seltype == "unequal")) {
        if (!is.null(caty_n)) {
          if (!is.list(n_over)) {
            if (any(n_over < 0)) {
              stop_ind <- TRUE
              stop_mess <- paste0("n_over values must be zero or positive.")
              stop_df <- rbind(stop_df, data.frame(func = I("n_over"), I(stop_mess)))
            }
          }
        }
      }
    }
    if (length(stratum) > 1) {
      if (any(seltype %in% c("equal", "proportional", "unequal"))) {
        if (!is.list(n_over)) {
          if (any(n_over < 0)) {
            stop_ind <- TRUE
            stop_mess <- paste0("n_over values must be zero or positive.")
            stop_df <- rbind(stop_df, data.frame(func = I("n_over"), I(stop_mess)))
          }

          # fixes a bug downstream in checking sizes of n_base + n_over
          if (length(n_over) == 1) {
            n_over <- rep(n_over, length(stratum))
          }
          if (is.null(names(n_over)) || all(names(n_over) %in% stratum)) {
            names(n_over) <- stratum
          }
          n_over <- as.list(n_over)
        }
        if (is.list(n_over)) {
          if (any(names(n_over) %in% stratum == FALSE)) {
            stop_ind <- TRUE
            stop_mess <- paste0("Names for n_over list are not values in 'stratum' variable.")
            stop_df <- rbind(stop_df, data.frame(func = I("n_over"), I(stop_mess)))
            stop_mess <- paste0("For each stratum make sure n_over values are non-negative.")
            stop_df <- rbind(stop_df, data.frame(func = I("n_over"), I(stop_mess)))
          }
        }
      }
    }
  }

  # check total sample size for n_over
  if (sf_type == "sf_point") {
    if (!is.null(n_over)) {
      if (length(stratum) > 1) {
        if (is.list(n_over)) {
          if (any(sapply(stratum, function(x) (n_base[[x]] + ifelse(is.null(n_over[[x]]), 0, sum(n_over[[x]]))) > NROW(sframe[sframe[[stratum_var]] == x, , drop = FALSE])))) {
            stop_ind <- TRUE
            stop_mess <- paste0("For each stratum, the sum of the base sites and 'Over' replacement sites must be no larger than the number of rows in 'sframe' representing that stratum.")
            stop_df <- rbind(stop_df, data.frame(func = I("n_base + n_over"), I(stop_mess)))
          }
        } else {
           if (any(sapply(stratum, function(x) (n_base[[x]] + sum(n_over[[x]])) > NROW(sframe[sframe[[stratum_var]] == x, , drop = FALSE])))) {
            stop_ind <- TRUE
            stop_mess <- paste0("For each stratum, the sum of the base sites and 'Over' replacement sites must be no larger than the number of rows in 'sframe' representing that stratum.")
            stop_df <- rbind(stop_df, data.frame(func = I("n_base + n_over"), I(stop_mess)))
          }
        }
      } else {
        if ((n_base + sum(n_over)) > NROW(sframe)) {
          stop_ind <- TRUE
          stop_mess <- paste0("The sum of the base sites and 'Over' replacement sites must be no larger than the number of rows in 'sframe'.")
          stop_df <- rbind(stop_df, data.frame(func = I("n_base + n_over"), I(stop_mess)))
        }
      }
    }
  }

  # check n_near
  if (!is.null(n_near)) {
    if (!(all(unlist(n_near) %in% 1:10))) {
      stop_ind <- TRUE
      stop_mess <- paste0("values of n_near must be from 1 to 10.\n")
      stop_df <- rbind(stop_df, data.frame(func = I("n_near"), I(stop_mess)))
    }
  }

  # find system info
  on_solaris <- Sys.info()[["sysname"]] == "SunOS"
  if (on_solaris) {
    stop_ind <- TRUE
    stop_mess <- paste0("grts() and irs() are not supported on Solaris.")
    stop_df <- rbind(stop_df, data.frame(func = I("Solaris"), I(stop_mess)))
  }

  ### If any issues, write out stop_df and then stop
  if (stop_ind) {
    names(stop_df) <- c("Design Input", "Error Message")
    stop_df <<- stop_df
    message("During the check of the input to grtspts, one or more errors were identified.\n")
    message("Enter the following command to view all input error messages: stopprnt()\n")
    message("To view a subset of the errors (e.g., errors 1 and 5) enter stopprnt(m=c(1,5))\n\n")
    opt <- options(show.error.messages = FALSE)
    on.exit(options(opt))
    stop()
  }
}

###############################################################################
# Function: grtspts_ip (not exported)
# Programmers: Tony Olsen, Tom Kincaid
# Date: January 22, 2021
#'
#' Calculate inclusion probabities for point sample frame for equal, unequal based
#' on categories or proportional to a variable. The inclusion probabilities are
#' calculated assuming a single stratum.
#'
#' @param type Character string with values of \code{"equal"}, \code{"unequal"} or \code{"proportional"}. Default
#'   is \code{"equal"}.
#'
#' @param n_base Sample size for the stratum. If type is \code{"equal"} or \code{"proportional"}, then \code{n_base}
#'   is numeric value for the total sample size for the stratum. If type is \code{"unequal"},
#'   then \code{n_base} is a named vector where names are the unequal probability categories with the
#'   expected sample size for each category. Names in \code{n_base} must be a subset of the values in \code{caty}.
#'   The sum of the category sample sizes is the total sample size for the stratum.
#'
#' @param Nstratum Number of elements in the stratum. Each element will be assigned an
#'   inclusion probabibility. Only required when type is \code{"equal"}. Default is \code{NULL}.
#'
#' @param caty If type is \code{"unequal"}, a character variable that identifies the category
#'   for each element of the sample frame that may be used for an unequal probability
#'   design. All categories in \code{n_base} must be present in \code{pt_caty} but additional categories
#'   are allowed but will not be included in the survey design for the statum. Default is \code{NULL}.
#'
#' @param aux If type is proportional, a numeric vector that will be used to select
#'   points proportional to the variable. Default is NULL.
#'
#' @param warn_ind  A logical value where TRUE indicates a warning message.
#'   Used for internal collection of messages only.
#'
#' @param warn_df A data frame containing messages warning of potential issues.
#'   Used for internal collection of messages only.
#'
#' @return A numeric vector of inclusion probabilities with all values being greater
#'   than 0 and less than or equal 1. Values equal to 1 will be selected with certainty.
#'   Values equal to 0 will not be selected.
#'
#' @author Tony Olsen \email{olsen.tony@@epa.gov}
#'
#' @keywords survey, inclusion probability
#'
#' @examples
#' tmp <- grtspts_ip(type = "equal", n_base = 10, Nstratum = 25)
#' tmp
#' tmp <- grtspts_ip(
#'   type = "unequal", n_base = c(small = 5, large = 5),
#'   caty = c(rep("small", 15), rep("large", 10))
#' )
#' tmp
#' @noRd
###############################################################################
grtspts_ip <- function(type = "equal", n_base, Nstratum = NULL, caty = NULL,
                       aux = NULL, warn_ind = NULL, warn_df = NULL) {

  # equal inclusion probabilities
  if (type == "equal") {
    ip <- rep(sum(n_base, na.rm = TRUE) / Nstratum, Nstratum)
  }
  # unequal inclusion probabilities
  if (type == "unequal") {
    gsum <- table(caty)
    catmatch <- match(names(n_base), names(gsum), nomatch = 0)
    piden <- n_base / gsum[catmatch]
    if (any(piden > 1)) {
      stop(paste0("The number of expected samples in levels ", paste0(names(piden[piden > 1]), collapse = " and "), " from the caty_n variable exceeds the number of allowed samples in sframe (if stratification was not used) or in at least one stratum (if stratification was used). If sframe has POINT or MULTIPOINT geometry, this means that the number of expected samples specified by these levels exceeds the number of observations in sframe. Consider 1) incorporating the caty_n variable into stratification or 2) reduce the number of expected samples for these levels in caty_n. Then rerun grts(). If sframe has LINESTRING, MULTILINESTRING, POLYGON, or MULTIPOLYGON geometry, this means that the number of expected samples specified by these levels exceeds the number of observations used in the approximation specified by the pt_density argument. Consider 1) incorporating the caty_n variable into stratification, 2) reduce the number of expected samples for these levels in caty_n, or 3) increase the pt_density argument. Then rerun grts()."), call. = FALSE)
    }
    ip <- rep(NA, length(caty))
    for (i in names(n_base)) {
      ip[caty == i] <- piden[i]
    }
  }
  # proportional inclusion probabilities
  if (type == "proportional") {
    ip <- aux
    # check for "0" and negative values. if negative set to "0".
    nnull <- sum(aux == 0)
    nneg <- sum(aux < 0)
    if (nnull > 0) {
      warn <- "Proportional vector has zero values and their inclusion probabilities are set to 0."
      if (is.null(warn_ind)) {
        warn_df <- data.frame(stratum = NA, func = I("grtspts_ip"), warning = warn)
        warn_ind <- TRUE
      } else {
        if (warn_ind) {
          warn_df <- rbind(warn_df, data.frame(
            stratum = NA, func = I("grtspts_ip"),
            warning = warn
          ))
        } else {
          warn_df <- data.frame(stratum = NA, func = I("grtspts_ip"), warning = warn)
          warn_ind <- TRUE
        }
      }
    }

    if (nneg > 0) {
      ip[ip < 0] <- 0
      warn <- paste0("Proportional vector has ", nneg, " negative value(s) and
              their inclusion probabilities are set to 0.")
      if (warn_ind) {
        warn_df <- data.frame(stratum = NA, func = I("grtspts_ip"), warning = warn)
      } else {
        warn_df <- rbind(warn_df, data.frame(
          stratum = NA, func = I("grtspts_ip"),
          warning = warn
        ))
        warn_ind <- TRUE
      }
    }
    # initialize inclusion probabilities
    ip <- n_base * ip / sum(ip)

    # identify and subset elements that have ip > 0
    element_gt0 <- ip > 0
    ip_gt0 <- ip[element_gt0]
    # identify elements greater than or equal to 1 and count them.
    element_gt1 <- ip_gt0 >= 1
    ngt1 <- sum(element_gt1)
    # if ngt1 is greater than 0, then set them to 1 and adjust remaining elements
    # to have sample size n - ngt1. Adjustment may cause other elements to become
    # greater than 1 so repeat until all are less than or equal to 1.
    if (ngt1 > 0) {
      tst <- 0
      while (ngt1 != tst) {
        tmp <- ip_gt0[!element_gt1]
        ip_gt0[!element_gt1] <- (n_base - ngt1) * tmp / sum(tmp)
        ip_gt0[element_gt1] <- 1
        tst <- ngt1
        element_gt1 <- ip_gt0 >= 1
        ngt1 <- sum(element_gt1)
      }
      # replace non zero inclusion probabilities with new ones
      ip[element_gt0] <- ip_gt0
    }
    ip
  }

  # return list with vector of inclusion probabilities and warning indicator and messages
  ip <- list(ip = ip, warn_ind = warn_ind, warn_df = warn_df)

  invisible(ip)
}

###############################################################################
# Function: grts (exported)
# Programmers: Tony Olsen, Tom Kincaid
# Date: January 22, 2021
#' Select a generalized random tessellation stratified (GRTS) sample
#'
#' Select a spatially balanced sample from a point (finite), linear / linestring (infinite),
#' or areal / polygon (infinite) sampling frame using the Generalized Random Tessellation
#' Stratified (GRTS) algorithm. The GRTS algorithm accommodates unstratified and
#' stratified sampling designs and allows for equal inclusion probabilities, unequal
#' inclusion probabilities according to a categorical variable, and inclusion
#' probabilities proportional to a positive auxiliary variable. Several additional
#' sampling options are included, such as including legacy (historical) sites,
#' requiring a minimum distance between sites, and selecting replacement sites.
#' For technical details, see Stevens and Olsen (2004).
#'
#'
#' @param sframe A sampling frame as an \code{sf} object. The coordinate
#'   system for \code{sframe} must projected (not geographic). If m or z values
#'   are in \code{sframe}'s geometry, they are silently dropped (i.e., only x-coordinates
#'   and y-coordinates are preserved).
#'
#' @param n_base The base sample size required. If the sampling design is unstratified,
#'   this is a single numeric value. If the sampling design is stratified, this is a named
#'   vector or list whose names represent each stratum and whose values represent each
#'   stratum's sample size. These names must match the values of the stratification
#'   variable represented by \code{stratum_var}. Legacy sites are considered part
#'   of the base sample, so the value for \code{n_base} should be equal to the number
#'   of legacy sites plus the number of desired non-legacy sites.
#'
#' @param stratum_var A character string containing the name of the column from
#'   \code{sframe} that identifies stratum membership for each element in \code{sframe}.
#'   If stratum equals \code{NULL}, the sampling design is unstratified and all elements in \code{sframe}
#'   are eligible to be selected in the sample. The default is \code{NULL}.
#'
#' @param seltype A character string or vector indicating the inclusion probability type,
#'   which must be one of following: \code{"equal"} for equal inclusion probabilities;
#'   \code{"unequal"} for unequal inclusion probabilities according to a categorical
#'   variable specified by \code{caty_var}; and \code{"proportional"} for inclusion
#'   probabilities proportional to a positive auxiliary variable specified by
#'   \code{aux_var}. If the sampling design is unstratified, \code{seltype} is a single
#'   character vector. If the sampling design is stratified, \code{seltype} is a named vector
#'   whose names represent each stratum and whose values represent each stratum's
#'   inclusion probability type. \code{seltype}'s default value tries to match the
#'   intended inclusion probability type: If \code{caty_var} and \code{aux_var} are
#'   not specified, \code{seltype} is \code{"equal"}; if \code{caty_var} is specified,
#'   \code{seltype} is \code{"unequal"}; and if \code{aux_var} is specified, \code{seltype}
#'   is \code{"proportional"}.
#'
#' @param caty_var A character string containing the name of the column from
#'   \code{sframe} that represents the unequal probability variable.
#'
#' @param caty_n A character vector indicating the expected sample size for each
#'   level of \code{caty_var}, the unequal probability variable. If the sampling design
#'   is unstratified, \code{caty_n} is a named vector whose names represent each
#'   level of \code{caty_var} and whose values represent each level's expected
#'   sample size. The sum of \code{caty_n} must equal \code{n_base}. If the sampling design
#'   is stratified and the expected sample sizes are the same among strata, \code{caty_n} is
#'   a named vector whose names represent represent each
#'   level of \code{caty_var} and whose values represent each level's expected
#'   sample size -- these expected sample sizes are applied to all strata. The sum of
#'   \code{caty_n} must equal each stratum's value in \code{n_base}.
#'   If the sampling design is stratified and the expected sample sizes differ among strata,
#'   \code{caty_n} is a list where each element  is named as a stratum in \code{n_base}.
#'   Each stratum's list element is a named vector whose
#'   names represent each level of \code{caty_var} and whose values represent each
#'   level's expected sample size (within the stratum). The sum of the values in each stratum's
#'   list element must equal that stratum's value in \code{n_base}.
#'
#' @param aux_var A character string containing the name of the column from
#'   \code{sframe} that represents the proportional (to size) inclusion probability
#'   variable (auxiliary variable). This auxiliary variable must be positive, and the resulting
#'   inclusion probabilities are proportional to the values of the auxiliary variable.
#'   Larger values of the auxiliary variable result in higher inclusion probabilities.
#'
#' @param legacy_sites An sf object with a \code{POINT} or \code{MULTIPOINT}
#'   geometry representing the legacy sites. spsurvey assumes that
#'   the legacy sites were selected from a previous sampling design that
#'   incorporated randomness into site selection and that the legacy sites
#'   are elements of the current sampling frame. If \code{sframe} has a
#'   \code{POINT} or \code{MULTIPOINT} geometry, the observations in \code{legacy_sites}
#'   should not also be in \code{sframe} (i.e., duplicates are not removed). Thus, \code{sframe}
#'   and \code{legacy_sites} together compose the current sampling frame. If m or z values
#'   are in \code{legacy_sites}' geometry, they are silently dropped (i.e., only x-coordinates
#'   and y-coordinates are preserved).
#'
#' @param legacy_stratum_var A character string containing the name of the column from
#'   \code{legacy_sites} that identifies stratum membership for each element of \code{legacy_sites}.
#'   This argument is required when the sampling design is stratified and its levels
#'   must be contained in the levels of the \code{stratum_var} variable. The default value of \code{legacy_stratum_var}
#'   is \code{stratum_var}, so \code{legacy_stratum_var} need only be specified explicitly when
#'   the name of the stratification variable in \code{legacy_sites} differs from \code{stratum_var}.
#'
#' @param legacy_caty_var A character string containing the name of the column from
#'   \code{legacy_sites} that identifies the unequal probability variable for each element of \code{legacy_sites}.
#'   This argument is required when the sampling design uses unequal selection probabilities and its categories
#'   must be contained in the levels of the \code{caty_var} variable. The default value of \code{legacy_caty_var}
#'   is \code{caty_var}, so \code{legacy_caty_var} need only be specified explicitly when
#'   the name of the unequal probability variable in \code{legacy_sites} differs from \code{caty_var}.
#'
#' @param legacy_aux_var A character string containing the name of the column from
#'   \code{legacy_sites} that identifies the proportional probability variable for each element of \code{legacy_sites}.
#'   This argument is required when the sampling design uses proportional selection probabilities and the values of the
#'   \code{legacy_aux_var} variable must be positive. The
#'   default value of \code{legacy_aux_var} is \code{aux_var}, so \code{legacy_aux_var} need only be specified explicitly
#'   when the name of the proportional probability variable in \code{legacy_sites} differs from \code{aux_var}.
#'
#' @param legacy_var This argument can be used instead of \code{legacy_sites}
#'   when \code{sframe} is a \code{POINT} or \code{MULTIPOINT}
#'   geometry (i.e. a finite sampling frame),
#'   When \code{legacy_var} is used, it is a character string containing the name of the column
#'   from \code{sframe} that represents whether each site is a legacy site. For
#'   legacy sites, the values of the \code{legacy_var} must contain character strings that
#'   act as a legacy site identifier. For non-legacy sites, the values of the
#'   \code{legacy_var} column must be \code{NA}. Using this approach,
#'   \code{legacy_stratum_var}, \code{legacy_caty_var}, and \code{legacy_aux_var}
#'   are not required and should not be used (because \code{legacy_var} represents a column
#'   in \code{sframe}). \code{spsurvey} assumes that the legacy sites were selected from
#'   a previous sampling design that incorporated randomness into site selection
#'   and that the legacy sites are elements of the current sampling frame.
#'
#' @param mindis A numeric value indicating the desired minimum distance between sampled
#'   sites. If the sampling design is stratified and \code{mindis} is an numeric value, the minimum
#'   distance is applied to all strata. If the sampling design is stratified and different minimum distances
#'   are desired among strata, then \code{mindis}
#'   is a list whose names match the names of \code{n_base} and whose and values
#'   are the minimum distance for the corresponding stratum.  If a minimum distance is not desired
#'   for a particular stratum, then the corresponding value in \code{mindis} should be \code{0} or
#'    \code{NULL} (which is equivalent to \code{0}).
#'   The units of \code{mindis} must represent the units in \code{sframe}. A warning is returned if the
#'   minimum distance could not be reached after \code{maxtry} attempts. If legacy sites are used, the minimum distance
#'   requirement (and subsequent warning if \code{maxtry} attempts are reached) is enforced for all base sites
#'   that are not legacy sites (i.e., the minimum distance is enforced for these sites
#'   by comparing distances against all base sites (legacy and non-legacy)).
#'
#' @param maxtry The number of maximum attempts to apply the minimum distance algorithm to obtain
#'   the desired minimum distance between sites. Each iteration takes roughly as long as the
#'   standard GRTS algorithm. Successive iterations will always contain at least as many
#'   sites satisfying the minimum distance requirement as the previous iteration. The algorithm stops
#'   when the minimum distance requirement is met or there are \code{maxtry} iterations.
#'   The default number of maximum iterations is \code{10}.
#'
#' @param n_over The number of reverse hierarchically ordered (rho) replacement sites.
#'   If the sampling design is unstratified, then
#'   \code{n_over} is an integer specifying the number of rho replacement sites desired.
#'  If the sampling design is stratified,
#'   then \code{n_over} is a vector (or list) whose names match the names of \code{n_base} and
#'   whose values indicate the number of rho replacement sites for each stratum.
#'   If replacement sites are not desired for a particular stratum, then the corresponding
#'   value in \code{n_over} should be \code{0} or \code{NULL} (which is equivalent to \code{0}).
#'   If the sampling design is stratified but the number of \code{n_over} sites is the same in each
#'   stratum, \code{n_over} can be a vector which is used for each stratum.
#'   If \code{n_over} is an unnamed, length-one vector, it's value is recycled
#'   and used for each stratum. Note that if the
#'   sampling design has unequal selection probabilities (\code{seltype = "unequal"}), then \code{n_over} sites
#'   are given the same proportion of \code{caty_n} values as \code{n_base}.
#'
#'
#' @param n_near The number of nearest neighbor (nn) replacement sites.
#'   If the sampling design is unstratified, \code{n_near} is integer from \code{1}
#'   to \code{10} specifying the number of
#'   nn replacement sites to be selected for each base site. If the sampling design
#'   is stratified but the same number of nn replacement sites is desired
#'   for each stratum,  \code{n_near} is integer from \code{1}
#'   to \code{10} specifying the number of
#'   nn replacement sites to be selected for each base site. If the sampling design is
#'   unstratified and a different number of nn replacement sites is
#'   desired for each stratum, \code{n_near} is a vector (or list) whose names represent strata and whose
#'   values is integer from \code{1}
#'   to \code{10} specifying the number of
#'   nn replacement sites to be selected for each base site in the stratum. If replacement sites
#'   are not desired for a particular stratum, then the corresponding value in \code{n_over}
#'   should be \code{0} or \code{NULL} (which is equivalent to \code{0}). For
#'   infinite sampling frames, the distance between a site and its nn
#'   depends on \code{pt_density}. The larger \code{pt_density}, the closer the nn neighbors.
#'
#' @param wgt_units The units used to compute the design weights. These
#'   units must be standard units as defined by the \code{set_units()} function in
#'   the units package. The default units match the units of the sf object.
#'
#' @param pt_density A positive integer controlling the density of the GRTS approximation
#'   for infinite sampling frames. The GRTS approximation for infinite sample
#'   frames vastly improves computational efficiency by generating many finite points and
#'   selecting a sample from the points. \code{pt_density} represents the density
#'   of finite points per unit to use in the approximation. More specifically,
#'   for each stratum, the number of points used in the approximation equals
#'   \code{pt_density * (n_base + n_over)}. A larger value of \code{pt_density}
#'   means a closer approximation to the infinite sampling frame but less
#'   computational efficiency. The default value of \code{pt_density} is \code{10}. Note that
#'   when used with \code{caty_n}, the unequal inclusion probabilities generated from
#'   this approach are also approximations.
#'
#' @param DesignID A character string indicating the naming structure for each
#'   site's identifier selected in the sample, which is matched with \code{SiteBegin} and
#'   included as a variable in the
#'   sf object in the function's output. Default is "Site".
#'
#' @param SiteBegin A character string indicating the first number to use to match
#'   with \code{DesignID} while creating each site's identifier selected in the sample.
#'   Successive sites are given successive integers. The default starting number
#'   is \code{1} and the number of digits is equal to number of digits in
#'   \code{nbase + nover}.
#'   For example, if \code{nbase} is 50 and \code{nover} is 0, then the default
#'   site identifiers are \code{Site-01} to \code{Site-50}
#'
#' @param sep A character string that acts as a separator between
#'  \code{DesignID} and \code{SiteBegin}. The default is \code{"-"}.
#'
#' @param projcrs_check A check for whether the coordinates are projected. If \code{TRUE},
#'    an error is returned if coordinates are not projected  (i.e., they are geographic or NA). If \code{FALSE}, the
#'    check is not performed, which means that the crs in \code{sframe} (and \code{legacy_sites} if provided) can be projected, geographic, or NA.
#'
#' @details \code{n_base} is the number of sites used to calculate
#'   the design weights, which is typically the number of sites used in an analysis. When a panel sampling design is implemented, \code{n_base} is typically the
#'   number of sites in all panels that will be sampled in the same temporal period --
#'   \code{n_base} is not the total number of sites in all panels. The sum of \code{n_base} and
#'   \code{n_over} is equal to the total number of sites to be visited for all panels plus
#'   any replacement sites that may be required.
#'
#' @return The sampling design sites and additional information about the sampling design. More specifically, it is, a list with five elements:
#'   \itemize{
#'     \item \code{sites_legacy} An sf object containing legacy sites. This is
#'       \code{NULL} if legacy sites were not included in the sample.
#'     \item \code{sites_base} An sf object containing the base sites. This is \code{NULL}
#'     if \code{n_base} equals the number of legacy sites.
#'     \item \code{sites_over} An sf object containing the reverse hierarchically
#'       ordered replacement sites. This is \code{NULL} if no reverse hierarchically
#'       ordered replacement sites were included in the sample.
#'     \item \code{sites_near} An sf object containing the nearest neighbor
#'       replacement sites. This is \code{NULL} if no nearest neighbor replacement
#'       sites were included in the sample.
#'     \item \code{design} A list documenting the specifications of this sampling design.
#'       This can be checked to verify your sampling design ran as intended.
#'       \itemize{
#'         \item \code{call} The original function call.
#'         \item \code{stratum_var} The name of the stratification variable in \code{sframe}.
#'           This equals \code{NULL} if no stratification is used.
#'         \item \code{stratum} The unique strata. This equals \code{"None"} if
#'           the sampling design is unstratified.
#'         \item \code{n_base} The base sample size per stratum.
#'         \item \code{seltype} The selection type per stratum.
#'         \item \code{caty_var} The name of the unequal probability variable in \code{sframe}.
#'           This equals \code{NULL} if no unequal probability variable is used.
#'         \item \code{caty_n} The expected sample sizes for each level of the
#'           unequal probability grouping variable per stratum. This equals
#'           \code{NULL} when \code{seltype} is not \code{"unequal"}.
#'         \item \code{aux_var} The name of the proportional probability (auxiliary) variable in \code{sframe}.
#'           This equals \code{NULL} if no proportional probability variable is used.
#'         \item \code{legacy} A logical variable indicating whether legacy sites
#'           were included in the sample.
#'         \item \code{legacy_stratum_var} The name of the stratification variable in \code{legacy_sites}.
#'           Omitted if legacy sites are not used. This equals \code{NULL} if legacy sites were used but
#'           no stratification variable is used.
#'         \item \code{legacy_caty_var} The name of the unequal probability variable in \code{legacy_sites}.
#'           Omitted if legacy sites are not used. This equals \code{NULL} if legacy sites were used but
#'           no unequal probability variable is used.
#'         \item \code{legacy_aux_var} The name of the proportional probability (auxiliary)
#'           variable in \code{legacy_sites}.
#'           Omitted if legacy sites are not used. This equals \code{NULL} if legacy sites
#'           were used but no proportional probability variable is used.
#'         \item \code{mindis} The minimum distance requirement desired. This
#'           is \code{NULL} when no minimum distance requirement was applied.
#'         \item \code{n_over} The reverse hierarchically ordered replacement
#'           site sample sizes per stratum. If \code{seltype} is \code{unequal},
#'           this represents the expected sample sizes. This is \code{NULL}
#'           when no reverse hierarchically ordered replacement sites were selected.
#'         \item \code{n_near} The number of nearest neighbor replacement sites
#'           desired. This is \code{NULL} when no nearest neighbor replacement
#'           sites were selected.
#'       }
#'   }
#'   When non-\code{NULL}, the \code{sites_legacy}, \code{sites_base},
#'   \code{sites_over}, and \code{sites_near} objects contain the original columns
#'   in \code{sframe} and include a few additional columns. These additional columns
#'   are
#'   \itemize{
#'     \item \code{siteID} A site identifier (as named using the \code{DesignID}
#'       and \code{SiteBegin} arguments to \code{grts()}).
#'     \item \code{siteuse} Whether the site is a legacy site (\code{Legacy}), base
#'       site (\code{Base}), reverse hierarchically ordered replacement site
#'       (\code{Over}), or nearest neighbor replacement site (\code{Near}).
#'     \item \code{replsite} The replacement site ordering. \code{replsite} is
#'       \code{None} if the site is not a replacement site, \code{Next} if it is
#'       the next reverse hierarchically ordered replacement site to use, or
#'       \code{Near_}, where the word following \code{_} indicates the ordering of sites closest to
#'       the originally sampled site.
#'     \item \code{lon_WGS84} Longitude coordinates using the WGS84 coordinate
#'       system (EPSG:4326). Only given if coordinates are projected.
#'     \item \code{lat_WGS84} Latitude coordinates using the WGS84 coordinate
#'       system (EPSG:4326). Only given if coordinates are projected.
#'     \item \code{X} Longitude coordinates using the provided coordinate
#'       system. Only given if coordinates are not projected (i.e., they are geographic or NA).
#'     \item \code{Y} Latitude coordinates using the provided coordinate
#'       system. Only given if coordinates are not projected (i.e., they are geographic or NA).
#'     \item \code{stratum} A stratum indicator. \code{stratum} is \code{None}
#'       if the sampling design was unstratified. If the sampling design was \code{stratified},
#'       \code{stratum} indicates the stratum.
#'     \item \code{wgt} The design weight.
#'     \item \code{ip} The site's original inclusion probability (the reciprocal)
#'       of (\code{wgt}).
#'     \item \code{caty} An unequal probability grouping indicator. \code{caty}
#'       is \code{None} if the sampling design did not use unequal inclusion probabilities.
#'       If the sampling design did use unequal inclusion probabilities, \code{caty}
#'       indicates the unequal probability level.
#'     \item \code{aux} The auxiliary proportional probability variable. This
#'       column is only returned if \code{seltype} was \code{proportional} in the
#'       original sampling design.
#'   }
#'   If any columns in \code{sframe} contain these names, those columns
#'   from \code{sframe} will be automatically prefixed with \code{sframe_}
#'   in the \code{sites} object. When output is printed, a summary of site counts by
#'   the levels in \code{stratum_var} and \code{caty_var} is shown.
#'
#' @author Tony Olsen \email{olsen.tony@@epa.gov}
#'
#' @seealso
#'   \describe{
#'     \item{\code{\link{irs}}}{ to select a sample that is not spatially balanced}
#'  }
#'
#' @references
#' Stevens Jr., Don L. and Olsen, Anthony R. (2004). Spatially balanced sampling
#' of natural resources. \emph{Journal of the American Statistical Association}, 99(465), 262-278.
#'
#' @examples
#' \dontrun{
#' samp <- grts(NE_Lakes, n_base = 100)
#' print(samp)
#' strata_n <- c(low = 25, high = 30)
#' samp_strat <- grts(NE_Lakes, n_base = strata_n, stratum_var = "ELEV_CAT")
#' print(samp_strat)
#' samp_over <- grts(NE_Lakes, n_base = 30, n_over = 5)
#' print(samp_over)
#' }
#' @export
################################################################################
grts <- function(sframe, n_base, stratum_var = NULL, seltype = NULL, caty_var = NULL,
                 caty_n = NULL, aux_var = NULL, legacy_var = NULL,
                 legacy_sites = NULL, legacy_stratum_var = NULL,
                 legacy_caty_var = NULL, legacy_aux_var = NULL, mindis = NULL,
                 maxtry = 10, n_over = NULL, n_near = NULL, wgt_units = NULL,
                 pt_density = NULL, DesignID = "Site", SiteBegin = 1, sep = "-", projcrs_check = TRUE) {

  if (inherits(sframe, c("tbl_df", "tbl"))) { # identify if tibble class elements are present
    class(sframe) <- setdiff(class(sframe), c("tbl_df", "tbl"))
    # remove tibble class for rownames warning
  }

  if (!is.null(legacy_sites) & inherits(legacy_sites, c("tbl_df", "tbl"))) { # identify if tibble class elements are present
    class(legacy_sites) <- setdiff(class(legacy_sites), c("tbl_df", "tbl"))
    # remove tibble class for rownames warning
  }

  geom_col_name <- attr(sframe, "sf_column")
  st_geometry(sframe) <- "geometry"
  if (!is.null(legacy_sites)) {
    st_geometry(legacy_sites) <- "geometry"
  }

  # save initial variable specifications for the design list later
  initial_stratum_var <- stratum_var
  initial_caty_var <- caty_var
  initial_aux_var <- aux_var

  # Create warning indicator and data frame to collect all potential issues during
  # sample selection
  warn_ind <- FALSE
  warn_df <- data.frame(stratum = "Stratum", func = "Calling Function", warn = "Message")

  # Ensure that the geometry types for sframe are consistent

  temp <- st_geometry_type(sframe)
  tst <- all(temp %in% c("POINT", "MULTIPOINT")) |
    all(temp %in% c("LINESTRING", "MULTILINESTRING")) |
    all(temp %in% c("POLYGON", "MULTIPOLYGON"))
  if (!tst) {
    stop(paste("\nThe geometry types for the survey frame object passed to function grts: \n\"",
      unique(st_geometry_type(sframe)), "\" are not consistent.",
      sep = ""
    ))
  }

  # Drop m and z values to ensure no issues with grts functionality with sf object
  if (!is.null(st_m_range(sframe)) | !is.null(st_z_range(sframe))) {
    # warn_ind <- TRUE
    # warn_df$warn <- "\nThe survey frame object passed to function grts contains m or z values - they are being dropped to ensure functionality in grts."
    sframe <- st_zm(sframe)
  }

  # Determine type of sampling frame: point, line, polygon
  if (all(temp %in% c("POINT", "MULTIPOINT"))) sf_type <- "sf_point"
  if (all(temp %in% c("LINESTRING", "MULTILINESTRING"))) sf_type <- "sf_linear"
  if (all(temp %in% c("POLYGON", "MULTIPOLYGON"))) sf_type <- "sf_area"

  if (all(is.null(legacy_sites), is.null(legacy_var))) {
    legacy_option <- FALSE
  } else {
    legacy_option <- TRUE
  }

  if (is.null(stratum_var)) {
    stratum <- NULL
  } else {
    stratum <- names(n_base)
  }

  # set default seltype if not provided (based on specification of other variables)
  if (is.null(seltype)) {
    if (is.null(caty_var) & is.null(aux_var)) {
      seltype <- "equal"
    } else if (!is.null(caty_var)) {
      seltype <- "unequal"
    } else {
      seltype <- "proportional"
    }
  }


  # check input. If errors, dsgn_check will stop grtspts and report errors.
  dsgn_check(
    sframe = sframe, sf_type = sf_type, legacy_sites = legacy_sites,
    legacy_option = legacy_option, stratum = stratum, seltype = seltype,
    n_base = n_base, caty_n = caty_n, n_over = n_over, n_near = n_near,
    stratum_var = stratum_var, caty_var = caty_var, aux_var = aux_var,
    legacy_stratum_var = legacy_stratum_var, legacy_caty_var = legacy_caty_var,
    legacy_aux_var = legacy_aux_var,
    legacy_var = legacy_var, mindis = mindis, DesignID = DesignID,
    SiteBegin = SiteBegin, maxtry = maxtry, projcrs_check = projcrs_check
  )

  # preserve original sframe names
  sframe_names <- names(sframe)

  # preserve original legacy_sites names if needed
  if (!is.null(legacy_sites)) {
    legacy_sites_names <- names(legacy_sites)
  }

  ## Create variables in sampling frame if needed.
  # Create unique sampling frame ID values
  sframe$id <- 1:nrow(sframe)

  # Assign stratum variable or create it if design not stratified and variable not provided.
  if (is.null(stratum_var)) {
    stratum_var <- "stratum"
    sframe$stratum <- "None"
    stratum <- c("None") # names of all strata
  } else {
    # ensure class for stratum variable is character and assign to stratum
    sframe$stratum <- as.character(sframe[[stratum_var]])
  }

  # set caty, aux and legacy variables in sampling frame if needed
  if (!is.null(caty_var)) sframe$caty <- as.character(sframe[[caty_var]])
  if (!is.null(aux_var)) sframe$aux <- sframe[[aux_var]]
  if (!is.null(legacy_var)) sframe$legacy <- sframe[[legacy_var]]

  # set stratum, caty, aux and legacy variables in legacy_sites if needed
  # add idpts to legacy_sites
  if (legacy_option == TRUE & (sf_type != "sf_point" | ((sf_type == "sf_point") & !is.null(legacy_sites)))) {
    legacy_names <- names(legacy_sites)
    legacy_sites$idpts <- 1:nrow(legacy_sites)
    if (stratum[1] == "None") {
      legacy_sites$stratum <- "None"
    } else {
      if (is.null(legacy_stratum_var)) {
        legacy_stratum_var <- stratum_var
      }
      legacy_sites$stratum <- as.character(legacy_sites[[legacy_stratum_var]])
    }
    if (!is.null(caty_var)) {
      if (is.null(legacy_caty_var)) {
        legacy_caty_var <- caty_var
      }
      legacy_sites$caty <- as.character(legacy_sites[[legacy_caty_var]])
    }
    if (!is.null(aux_var)) {
      if (is.null(legacy_aux_var)) {
        legacy_aux_var <- aux_var
      }
      legacy_sites$aux <- legacy_sites[[legacy_aux_var]]
    }
    if (is.null(legacy_var)) {
      legacy_sites$legacy <- TRUE
      legacy_var <- "legacy"
    } else {
      legacy_sites$legacy <- legacy_sites[[legacy_var]]
    }
  }

  # save initial variable specifications for the design list later
  initial_legacy_stratum_var <- legacy_stratum_var
  initial_legacy_caty_var <- legacy_caty_var
  initial_legacy_aux_var <- legacy_aux_var

  ## Create a dsgn list object
  # variable assignments to dsgn list object
  dsgn <- list(
    stratum_var = stratum_var, caty_var = caty_var, aux_var = aux_var,
    legacy_option = legacy_option, legacy_var = legacy_var, stratum = stratum,
    wgt_units = wgt_units, seltype = NULL, n_base = NULL, caty_n = NULL,
    n_over = NULL, n_near = NULL, mindis = mindis
  )

  # seltype
  if (length(seltype) == length(stratum)) {
    dsgn$seltype <- seltype
    names(dsgn$seltype) <- stratum
  } else {
    tmp <- sapply(stratum, function(x, seltype) {
      x <- seltype
    }, seltype)
    names(tmp) <- stratum
    dsgn$seltype <- tmp
  }

  # n_base
  if (length(n_base) == length(stratum)) {
    dsgn$n_base <- n_base
    names(dsgn$n_base) <- stratum
  } else {
    tmp <- sapply(stratum, function(x, n_base) {
      x <- n_base
    }, n_base)
    names(tmp) <- stratum
    dsgn$n_base <- tmp
  }

  # caty_n
  if (is.list(caty_n)) {
    dsgn$caty_n <- caty_n
  } else {
    tmp <- lapply(stratum, function(x, caty_n) {
      x <- caty_n
    }, caty_n)
    names(tmp) <- stratum
    dsgn$caty_n <- tmp
  }

  # n_over
  if (!is.null(n_over)) {
    if (is.list(n_over)) {
      n_over <- lapply(n_over, function(x) if (all(x == 0)) NULL else x)
      dsgn$n_over <- n_over
    } else if (!is.null(names(n_over)) && all(sort(stratum) == sort(names(n_over)))) {
      tmp <- lapply(stratum, function(x, n_over) {
        x <- n_over[[x]]
      }, n_over)
      names(tmp) <- stratum
      dsgn$n_over <- tmp
    } else {
      tmp <- lapply(stratum, function(x, n_over) {
        x <- n_over
      }, n_over)
      names(tmp) <- stratum
      dsgn$n_over <- tmp
    }
  }

  # n_near
  if (!is.null(n_near)) {
    if (is.list(n_near)) {
      n_near <- lapply(n_near, function(x) if (all(x == 0)) NULL else x)
      dsgn$n_near <- n_near
    } else if (!is.null(names(n_near)) && all(sort(stratum) == sort(names(n_near)))) {
      tmp <- lapply(stratum, function(x, n_near) {
        x <- n_near[[x]]
      }, n_near)
      names(tmp) <- stratum
      dsgn$n_near <- tmp
    } else {
      tmp <- lapply(stratum, function(x, n_near) {
        x <- n_near
      }, n_near)
      names(tmp) <- stratum
      dsgn$n_near <- tmp
    }
  }

  # mindis
  if (!is.null(mindis)) {
    if (is.list(mindis)) {
      mindis <- lapply(mindis, function(x) if (all(x == 0)) NULL else x)
      dsgn$mindis <- mindis
    } else {
      tmp <- lapply(stratum, function(x, mindis) {
        x <- mindis
      }, mindis)
      names(tmp) <- stratum
      dsgn$mindis <- tmp
    }
  }

  # legacy_option
  if (legacy_option == TRUE) {
    tmp <- sapply(stratum, function(x, legacy_option) {
      x <- legacy_option
    }, legacy_option)
    names(tmp) <- stratum
    dsgn$legacy_option <- tmp
  }

  ## select sites for each stratum
  rslts <- lapply(dsgn$stratum, grts_stratum,
    dsgn = dsgn, sframe = sframe, sf_type = sf_type, wgt_units = wgt_units,
    pt_density = pt_density, legacy_option = legacy_option,
    legacy_sites = legacy_sites, maxtry = maxtry, warn_ind = warn_ind, warn_df = warn_df
  )
  names(rslts) <- stratum


  # combine across strata
  sites_legacy <- NULL
  sites_base <- NULL
  sites_over <- NULL
  sites_near <- NULL
  warn_ind <- FALSE
  warn_df <- NULL
  for (i in 1:length(rslts)) {
    sites_legacy <- rbind(sites_legacy, rslts[[i]]$sites_legacy)
    sites_base <- rbind(sites_base, rslts[[i]]$sites_base)
    sites_over <- rbind(sites_over, rslts[[i]]$sites_over)
    sites_near <- rbind(sites_near, rslts[[i]]$sites_near)
    if (rslts[[i]]$warn_ind) {
      warn_ind <- TRUE
      warn_df <- rbind(warn_df, rslts[[i]]$warn_df)
    }
  }

  # spelling change to avoid clash with analysis warnings
  if (!is.null(warn_df) && "warning" %in% names(warn_df)) {
    names(warn_df)[which(names(warn_df) == "warning")] <- "Warning"
  }


  # Create a siteID for all sites
  ntot <- NROW(sites_legacy) + NROW(sites_base) + NROW(sites_over) + NROW(sites_near)
  siteID <- gsub(" ", "0", paste0(DesignID, sep, format(SiteBegin - 1 + 1:ntot, sep = "")))
  nlast <- 0

  # Create siteID for legacy sites if present using DesignID and SiteBegin
  if (!is.null(sites_legacy)) {
    row.names(sites_legacy) <- 1:nrow(sites_legacy)
    sites_legacy$siteID <- siteID[1:nrow(sites_legacy)]
    nlast <- nrow(sites_legacy)
    # set siteuse and replsite
    sites_legacy$siteuse <- "Legacy"
    sites_legacy$replsite <- "None"
  }

  # Create siteID for base sites using DesignID and SiteBegin
  if (!is.null(sites_base)) {
    row.names(sites_base) <- 1:nrow(sites_base)
    sites_base$siteID <- siteID[(nlast + 1):(nlast + nrow(sites_base))]
    nlast <- nlast + nrow(sites_base)
    # set siteuse and replsite for base sites
    sites_base$siteuse <- "Base"
    sites_base$replsite <- "None"
  }

  # create siteID for n_over sites if any
  if (!is.null(n_over)) {
    row.names(sites_over) <- 1:nrow(sites_over)
    sites_over$siteID <- siteID[(nlast + 1):(nlast + nrow(sites_over))]
    nlast <- nlast + nrow(sites_over)
    # set siteuse and replsite for n_over sites
    sites_over$siteuse <- "Over"
    sites_over$replsite <- "Next"
  }

  # if n_near sample sites, assign base ids to the replacement sites. then add siteIDs
  if (!is.null(n_near)) {
    tst <- match(paste(sites_near$stratum, sites_near$replsite, sep = "_"),
      paste(sites_legacy$stratum, sites_legacy$idpts, sep = "_"),
      nomatch = 0
    )
    sites_near$replsite[tst > 0] <- sites_legacy$siteID[tst]

    tst <- match(paste(sites_near$stratum, sites_near$replsite, sep = "_"),
      paste(sites_base$stratum, sites_base$idpts, sep = "_"),
      nomatch = 0
    )
    sites_near$replsite[tst > 0] <- sites_base$siteID[tst]
    tst <- match(paste(sites_near$stratum, sites_near$replsite, sep = "_"),
      paste(sites_over$stratum, sites_over$idpts, sep = "_"),
      nomatch = 0
    )
    sites_near$replsite[tst > 0] <- sites_over$siteID[tst]

    # sort by id so that sites_near in same order as sites in sites_base and sites_over
    sites_near <- sites_near[order(sites_near$replsite, sites_near$siteuse), ]
    row.names(sites_near) <- 1:nrow(sites_near)
    # assign siteIDs
    sites_near$siteID <- siteID[(nlast + 1):(nlast + nrow(sites_near))]
  }

  # Add lat/lon in WGS84 or X/Y coordinates
  if ((!is.null(sites_base) && is.na(st_crs(sites_base))) | (!is.null(sites_base) && st_is_longlat(st_crs(sites_base))) | (!is.null(sites_legacy) && is.na(st_crs(sites_legacy))) | (!is.null(sites_legacy) && st_is_longlat(st_crs(sites_legacy)))) {
    if (!is.null(sites_legacy)) {
      sites_legacy$X <- st_coordinates(sites_legacy)[, "X"]
      sites_legacy$Y <- st_coordinates(sites_legacy)[, "Y"]
    }

    if (!is.null(sites_base)) {
      sites_base$X <- st_coordinates(sites_base)[, "X"]
      sites_base$Y <- st_coordinates(sites_base)[, "Y"]
    }

    if (!is.null(sites_over)) {
      sites_over$X <- st_coordinates(sites_over)[, "X"]
      sites_over$Y <- st_coordinates(sites_over)[, "Y"]
    }

    if (!is.null(sites_near)) {
      sites_near$X <- st_coordinates(sites_near)[, "X"]
      sites_near$Y <- st_coordinates(sites_near)[, "Y"]
    }

    # reorder sf object variables by first specifying design names excluding unique
    # feature ID id and idpts as they are internal
    dsgn_names <- c(
      "siteID", "siteuse", "replsite", "X", "Y",
      "stratum", "wgt", "ip", "caty", "aux"
    )
  } else {
    if (!is.null(sites_legacy)) {
      sites_legacy$lon_WGS84 <- st_coordinates(st_transform(sites_legacy, crs = 4326))[, "X"]
      sites_legacy$lat_WGS84 <- st_coordinates(st_transform(sites_legacy, crs = 4326))[, "Y"]
    }

    if (!is.null(sites_base)) {
      sites_base$lon_WGS84 <- st_coordinates(st_transform(sites_base, crs = 4326))[, "X"]
      sites_base$lat_WGS84 <- st_coordinates(st_transform(sites_base, crs = 4326))[, "Y"]
    }

    if (!is.null(sites_over)) {
      sites_over$lon_WGS84 <- st_coordinates(st_transform(sites_over, crs = 4326))[, "X"]
      sites_over$lat_WGS84 <- st_coordinates(st_transform(sites_over, crs = 4326))[, "Y"]
    }

    if (!is.null(sites_near)) {
      sites_near$lon_WGS84 <- st_coordinates(st_transform(sites_near, crs = 4326))[, "X"]
      sites_near$lat_WGS84 <- st_coordinates(st_transform(sites_near, crs = 4326))[, "Y"]
    }

    # reorder sf object variables by first specifying design names excluding unique
    # feature ID id and idpts as they are internal
    dsgn_names <- c(
      "siteID", "siteuse", "replsite", "lon_WGS84", "lat_WGS84",
      "stratum", "wgt", "ip", "caty", "aux"
    )
  }

  dsgn_names_extra <- c(dsgn_names, "xcoord", "ycoord", "idpts")

  if (geom_col_name != "geometry") {
    # sframe prefix if necessary
    if (geom_col_name %in% dsgn_names_extra) {
      geom_col_name <- paste("sframe", geom_col_name, sep = "_")
    }

    # restore original column names
    if (!is.null(sites_legacy)) {
      st_geometry(sites_legacy) <- geom_col_name
      legacy_sites_names[legacy_sites_names == "geometry"] <- geom_col_name
      st_geometry(legacy_sites) <- geom_col_name
    }

    if (!is.null(sites_base)) {
      sframe_names[sframe_names == "geometry"] <- geom_col_name
      st_geometry(sites_base) <- geom_col_name
    }

    if (!is.null(sites_over)) {
      st_geometry(sites_over) <- geom_col_name
    }

    if (!is.null(sites_near)) {
      st_geometry(sites_near) <- geom_col_name
    }
  }

  # sites_legacy
  if (!is.null(sites_legacy)) {
    if (sf_type != "sf_point") {
      add_names <- dsgn_names[dsgn_names %in% names(sites_legacy)]
      legacy_sites_names_good <- legacy_sites_names[!legacy_sites_names %in% dsgn_names_extra]
      if (all(legacy_sites_names %in% legacy_sites_names_good)) {
        sites_legacy <- subset(sites_legacy, select = c(add_names, legacy_sites_names))
      } else {
        legacy_sites_names_bad <- legacy_sites_names[legacy_sites_names %in% dsgn_names_extra]
        legacy_sites_temp <- legacy_sites[, legacy_sites_names_bad, drop = FALSE]
        temp_geometry_col <- which(names(legacy_sites_temp) == attr(legacy_sites_temp, "sf_column"))
        legacy_sites_geometry_col <- which(names(legacy_sites) == attr(legacy_sites, "sf_column"))
        names(legacy_sites_temp)[-temp_geometry_col] <- paste("legacy_sites", names(legacy_sites_temp)[-temp_geometry_col], sep = "_")
        sites_legacy <- st_join(sites_legacy, legacy_sites_temp, join = st_nearest_feature)
        sites_legacy <- subset(sites_legacy, select = c(add_names, legacy_sites_names_good[-legacy_sites_geometry_col], names(legacy_sites_temp)[-temp_geometry_col]))
        for (i in names(sites_legacy)) {
          if (i %in% c("legacy_sites_xcoord", "legacy_sites_ycoord", "legacy_sites_idpts")) {
            names(sites_legacy)[which(names(sites_legacy) == i)] <- substring(i, first = 14)
          }
        }
      }
    }
    if (sf_type == "sf_point") {
      add_names <- dsgn_names[dsgn_names %in% names(sites_legacy)]
      sframe_names_good <- sframe_names[!sframe_names %in% dsgn_names_extra]
      if (all(sframe_names %in% sframe_names_good)) {
        sites_legacy <- subset(sites_legacy, select = c(add_names, sframe_names))
      } else {
        sframe_names_bad <- sframe_names[sframe_names %in% dsgn_names_extra]
        sframe_temp <- sframe[, sframe_names_bad, drop = FALSE]
        temp_geometry_col <- which(names(sframe_temp) == attr(sframe_temp, "sf_column"))
        sframe_geometry_col <- which(names(sframe) == attr(sframe, "sf_column"))
        names(sframe_temp)[-temp_geometry_col] <- paste("sframe", names(sframe_temp)[-temp_geometry_col], sep = "_")
        sites_legacy <- st_join(sites_legacy, sframe_temp, join = st_nearest_feature)
        sites_legacy <- subset(sites_legacy,
          select = c(add_names, sframe_names_good[-sframe_geometry_col], names(sframe_temp)[-temp_geometry_col])
        )
        for (i in names(sites_legacy)) {
          if (i %in% c("sframe_xcoord", "sframe_ycoord", "sframe_idpts")) {
            names(sites_legacy)[which(names(sites_legacy) == i)] <- substring(i, first = 8)
          }
        }
      }
    }
  }

  # sites_base
  # check what design variables are present in sf objects and add if missing
  if (!is.null(sites_base)) {
    add_names <- dsgn_names[dsgn_names %in% names(sites_base)]
    sframe_names_good <- sframe_names[!sframe_names %in% dsgn_names_extra]
    if (all(sframe_names %in% sframe_names_good)) {
      sites_base <- subset(sites_base, select = c(add_names, sframe_names))
    } else {
      sframe_names_bad <- sframe_names[sframe_names %in% dsgn_names_extra]
      sframe_temp <- sframe[, sframe_names_bad, drop = FALSE]
      temp_geometry_col <- which(names(sframe_temp) == attr(sframe_temp, "sf_column"))
      sframe_geometry_col <- which(names(sframe) == attr(sframe, "sf_column"))
      names(sframe_temp)[-temp_geometry_col] <- paste("sframe", names(sframe_temp)[-temp_geometry_col], sep = "_")
      sites_base <- st_join(sites_base, sframe_temp, join = st_nearest_feature)
      sites_base <- subset(sites_base, select = c(add_names, sframe_names_good[-sframe_geometry_col], names(sframe_temp)[-temp_geometry_col]))
      for (i in names(sites_base)) {
        if (i %in% c("sframe_xcoord", "sframe_ycoord", "sframe_idpts")) {
          names(sites_base)[which(names(sites_base) == i)] <- substring(i, first = 8)
        }
      }
    }
  }

  # sites_over
  if (!is.null(sites_over)) {
    add_names <- dsgn_names[dsgn_names %in% names(sites_over)]
    sframe_names_good <- sframe_names[!sframe_names %in% dsgn_names_extra]
    if (all(sframe_names %in% sframe_names_good)) {
      sites_over <- subset(sites_over, select = c(add_names, sframe_names))
    } else {
      sframe_names_bad <- sframe_names[sframe_names %in% dsgn_names_extra]
      sframe_temp <- sframe[, sframe_names_bad, drop = FALSE]
      temp_geometry_col <- which(names(sframe_temp) == attr(sframe_temp, "sf_column"))
      sframe_geometry_col <- which(names(sframe) == attr(sframe, "sf_column"))
      names(sframe_temp)[-temp_geometry_col] <- paste("sframe", names(sframe_temp)[-temp_geometry_col], sep = "_")
      sites_over <- st_join(sites_over, sframe_temp, join = st_nearest_feature)
      sites_over <- subset(sites_over, select = c(add_names, sframe_names_good[-sframe_geometry_col], names(sframe_temp)[-temp_geometry_col]))
      for (i in names(sites_over)) {
        if (i %in% c("sframe_xcoord", "sframe_ycoord", "sframe_idpts")) {
          names(sites_over)[which(names(sites_over) == i)] <- substring(i, first = 8)
        }
      }
    }
  }

  # sites_near
  if (!is.null(sites_near)) {
    add_names <- dsgn_names[dsgn_names %in% names(sites_near)]
    sframe_names_good <- sframe_names[!sframe_names %in% dsgn_names_extra]
    if (all(sframe_names %in% sframe_names_good)) {
      sites_near <- subset(sites_near, select = c(add_names, sframe_names))
    } else {
      sframe_names_bad <- sframe_names[sframe_names %in% dsgn_names_extra]
      sframe_temp <- sframe[, sframe_names_bad, drop = FALSE]
      temp_geometry_col <- which(names(sframe_temp) == attr(sframe_temp, "sf_column"))
      sframe_geometry_col <- which(names(sframe) == attr(sframe, "sf_column"))
      names(sframe_temp)[-temp_geometry_col] <- paste("sframe", names(sframe_temp)[-temp_geometry_col], sep = "_")
      sites_near <- st_join(sites_near, sframe_temp, join = st_nearest_feature)
      sites_near <- subset(sites_near, select = c(add_names, sframe_names_good[-sframe_geometry_col], names(sframe_temp)[-temp_geometry_col]))
      for (i in names(sites_near)) {
        if (i %in% c("sframe_xcoord", "sframe_ycoord", "sframe_idpts")) {
          names(sites_near)[which(names(sites_near) == i)] <- substring(i, first = 8)
        }
      }
    }
  }

  # add function call to dsgn list
  # dsgn <- c(list(Call = match.call()), dsgn)
  dsgn <- list(
    call = match.call(), stratum_var = initial_stratum_var, stratum = dsgn$stratum, n_base = dsgn$n_base,
    seltype = dsgn$seltype, caty_var = initial_caty_var, caty_n = dsgn$caty_n, aux_var = initial_aux_var, legacy = dsgn$legacy_option,
    mindis = dsgn$mindis, n_over = dsgn$n_over, n_near = dsgn$n_near
  )

  if (any(dsgn$legacy)) {
    dsgn <- c(dsgn, list(legacy_stratum_var = initial_legacy_stratum_var, legacy_caty_var = initial_legacy_caty_var,
                         legacy_aux_var = initial_legacy_aux_var))
    dsgn <- dsgn[c("call", "stratum_var", "stratum", "n_base", "seltype", "caty_var",
                   "caty_n", "aux_var", "legacy", "legacy_stratum_var", "legacy_caty_var", "legacy_aux_var",
                   "mindis", "n_over", "n_near")]
  }

  # create output list
  sites <- list(
    sites_legacy = sites_legacy, sites_base = sites_base,
    sites_over = sites_over, sites_near = sites_near,
    design = dsgn
  )

  # As necessary, output a message indicating that warning messages were generated
  # during execution of the program

  if (warn_ind) {
    warn_df <<- warn_df
    if (nrow(warn_df) == 1) {
      message("During execution of the program, a warning message was generated. The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
    } else {
      message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
    }
  }

  # constructor for design class
  sites <- structure(sites, class = "sp_design")

  # return the survey design sf object
  invisible(sites)
}

context("sp_rbind")

# find system info
on_solaris <- Sys.info()[["sysname"]] == "SunOS"
if (on_solaris) {
  test_that("on solaris", {
    expect_true(on_solaris)
  })
} else {

  # set reproducible seed (as there are random components here)
  set.seed(5)

  test_that("sp_rbind works base", {
    n_base <- 50
    eqprob <- grts(NE_Lakes, n_base)
    eqprob_spr <- sp_rbind(eqprob)
    expect_equal(sum(eqprob_spr$siteuse == "Base"), n_base)
    expect_equal(NROW(eqprob_spr), n_base)
  })

  test_that("sp_rbind works legacy", {
    n_base <- 50
    n_legacy <- nrow(NE_Lakes_Legacy)
    eqprob <- grts(NE_Lakes, n_base = n_base, legacy_sites = NE_Lakes_Legacy)
    eqprob_spr <- sp_rbind(eqprob)
    expect_equal(sum(eqprob_spr$siteuse == "Legacy"), n_legacy)
    expect_equal(sum(eqprob_spr$siteuse == "Base"), n_base - n_legacy)
    expect_equal(NROW(eqprob_spr), n_base)
  })

  test_that("sp_rbind works over", {
    n_base <- 50
    n_over <- 5
    eqprob <- grts(NE_Lakes, n_base = n_base, n_over = n_over)
    eqprob_spr <- sp_rbind(eqprob)
    expect_equal(sum(eqprob_spr$siteuse == "Base"), n_base)
    expect_equal(sum(eqprob_spr$siteuse == "Over"), n_over)
    expect_equal(NROW(eqprob_spr), n_base + n_over)
  })

  test_that("sp_rbind works near", {
    n_base <- 50
    n_near <- 5
    eqprob <- grts(NE_Lakes, n_base = n_base, n_near = n_near)
    eqprob_spr <- sp_rbind(eqprob)
    expect_equal(sum(eqprob_spr$siteuse == "Base"), n_base)
    expect_equal(sum(substring(eqprob_spr$siteuse, 1, 4) == "Near"), n_base * n_near)
    expect_equal(NROW(eqprob_spr), n_base + n_base * n_near)
  })

  test_that("sp_rbind works with siteuse", {
    n_base <- 50
    n_near <- 5
    eqprob <- grts(NE_Lakes, n_base = n_base, n_near = n_near)
    eqprob_spr <- sp_rbind(eqprob, siteuse = "Base")
    expect_equal(sum(eqprob_spr$siteuse == "Base"), n_base)
    expect_equal(NROW(eqprob_spr), n_base)
  })
}

context("sp_balance")

# find system info
on_solaris <- Sys.info()[["sysname"]] == "SunOS"
if (on_solaris) {
  test_that("on solaris", {
    expect_true(on_solaris)
  })
} else {

  # set reproducible seed (as there are random components here)
  set.seed(5)

  test_that("sp_balance works unstrat", {
    n_base <- 50
    eqprob <- grts(NE_Lakes, n_base = n_base)
    spb_eqprob <- sp_balance(eqprob$sites_base, NE_Lakes)
    expect_equal(NROW(spb_eqprob), 1)
    expect_equal(NCOL(spb_eqprob), 3)
    spb_eqprob <- sp_balance(eqprob$sites_base, NE_Lakes,
      metrics = c("pielou", "simpsons", "rmse", "mse", "mae", "medae", "chisq")
    )
    expect_equal(NROW(spb_eqprob), 7)
    expect_equal(NCOL(spb_eqprob), 3)
  })

  test_that("sp_balance works unstrat custom ip", {
    n_base <- 50
    NE_Lakes$ip <- n_base / nrow(NE_Lakes) + rnorm(NROW(NE_Lakes), sd = 0.01)
    NE_Lakes$ip <- pmin(NE_Lakes$ip, 1)
    NE_Lakes$ip <- pmax(NE_Lakes$ip, 0)
    eqprob <- grts(NE_Lakes, n_base = n_base)
    spb_eqprob <- sp_balance(eqprob$sites_base, NE_Lakes, ip = "ip")
    expect_equal(NROW(spb_eqprob), 1)
    expect_equal(NCOL(spb_eqprob), 3)
    spb_eqprob <- sp_balance(eqprob$sites_base, NE_Lakes,
      metrics = c("pielou", "simpsons", "rmse", "mse", "mae", "medae", "chisq"),
      ip = "ip"
    )
    expect_equal(NROW(spb_eqprob), 7)
    expect_equal(NCOL(spb_eqprob), 3)
  })
}

context("sp_plot")

# find system info
on_solaris <- Sys.info()[["sysname"]] == "SunOS"
if (on_solaris) {
  test_that("on solaris", {
    expect_true(on_solaris)
  })
} else {


  # set reproducible seed (as there are random components here)
  set.seed(5)

  #--------------------------------------
  #-------- one sided formula
  #--------------------------------------

  # one sided formulas
  test_that("one sided formulas work", {
    expect_error(sp_plot(NE_Lakes, formula = ~1), NA)
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT), NA)
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, fix_bbox = FALSE), NA)
    expect_error(sp_plot(NE_Lakes, formula = ~ ELEV_CAT:AREA_CAT), NA)
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV), NA)
  })

  # changing graphical parameters
  test_that("one sided formulas work", {
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, pch = 19), NA)
    var_args <- list(ELEV_CAT = list(main = "maintest"))
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, var_args = var_args), NA)
    varlevel_args <- list(ELEV_CAT = list(levels = c("low", "high"), cex = c(1, NA)))
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, varlevel_args = varlevel_args), NA)
    varlevel_args <- list(ELEV_CAT = list(levels = c("low", "high"), cex = c(1, 2)))
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, varlevel_args = varlevel_args), NA)
    expect_error(sp_plot(NE_Lakes,
      formula = ~ELEV_CAT,
      varlevel_args = varlevel_args, var_args = var_args, pch = 19
    ), NA)
  })


  #--------------------------------------
  #-------- two sided formula
  #--------------------------------------

  # two sided formulas
  test_that("two sided formulas work", {
    expect_error(sp_plot(NE_Lakes, formula = ELEV ~ 1), NA)
    expect_error(sp_plot(NE_Lakes, formula = ELEV ~ AREA_CAT, onlyshow = "small"), NA)
  })

  # changing graphical parameters
  test_that("two sided formulas work", {
    var_args <- list(AREA_CAT = list(ELEV_CAT = list(levels = c("low", "high"), pch = c(1, 19))))
    expect_error(sp_plot(NE_Lakes, formula = ELEV_CAT ~ AREA_CAT, var_args = var_args, onlyshow = "large"), NA)
  })


  #################################################
  ########### sp_design
  #################################################

  n_base <- 50
  eqprob <- grts(NE_Lakes, n_base = n_base)
  eqprob_legacy <- grts(NE_Lakes, n_base = n_base, legacy_sites = NE_Lakes_Legacy)
  n_over <- 5
  eqprob_rho <- grts(NE_Lakes, n_base = n_base, n_over = n_over)
  n_near <- 1
  eqprob_nn <- grts(NE_Lakes, n_base = n_base, n_near = n_near)
  eqprob_both <- grts(NE_Lakes, n_base = n_base, n_over = n_over, n_near = n_near)
  n_base_strat <- c(low = 25, high = 25)
  eqprob_strat <- grts(NE_Lakes, n_base = n_base_strat, stratum_var = "ELEV_CAT")

  #--------------------------------------
  #-------- without sframe
  #--------------------------------------

  # test sp_plot works
  test_that("sp_plot works", {
    expect_error(sp_plot(eqprob, formula = ~siteuse), NA)
    expect_error(sp_plot(eqprob_legacy, formula = ~siteuse), NA)
    expect_error(sp_plot(eqprob_rho, formula = ~siteuse), NA)
    expect_error(sp_plot(eqprob_nn, formula = ~siteuse), NA)
    expect_error(sp_plot(eqprob_nn, formula = ~siteuse, siteuse = c("Base")), NA)
    expect_error(sp_plot(eqprob_strat, formula = siteuse ~ stratum, onlyshow = "low"), NA)
  })

  # graphical paraemters
  test_that("sp_plot works", {
    var_args <- list(siteuse = list(main = "maintest"))
    varlevel_args <- list(siteuse = list(levels = c("Base", "Over"), pch = c(1, 19)))
    expect_error(sp_plot(eqprob_rho,
      formula = ~siteuse,
      varlevel_args = varlevel_args, var_args = var_args, cex = 0.5
    ), NA)
    var_args <- list(ELEV_CAT = list(siteuse = list(levels = c("Base", "Over"), pch = c(1, 19))))
    expect_error(sp_plot(eqprob_rho,
      formula = siteuse ~ ELEV_CAT,
      var_args = var_args, onlyshow = "low", cex = 0.5
    ), NA)
  })

  #--------------------------------------
  #-------- with sframe
  #--------------------------------------

  # test sp_plot works
  test_that("sp_plot works", {
    expect_error(sp_plot(eqprob, NE_Lakes, formula = ~siteuse), NA)
    expect_error(sp_plot(eqprob_legacy, NE_Lakes, formula = ~siteuse), NA)
    expect_error(sp_plot(eqprob_rho, NE_Lakes, formula = ~siteuse), NA)
    expect_error(sp_plot(eqprob_nn, NE_Lakes, formula = ~siteuse), NA)
    expect_error(sp_plot(eqprob_nn, NE_Lakes, formula = ~siteuse, siteuse = c("Base")), NA)
    expect_error(sp_plot(eqprob_strat, NE_Lakes, formula = siteuse ~ ELEV_CAT, onlyshow = "low"), NA)
  })

  # graphical parameters
  test_that("sp_plot works", {
    var_args <- list(siteuse = list(main = "maintest"))
    varlevel_args <- list(siteuse = list(levels = c("Base", "Over"), pch = c(1, 19)))
    expect_error(sp_plot(eqprob_rho, NE_Lakes,
      formula = ~siteuse,
      varlevel_args = varlevel_args, var_args = var_args, cex = 0.5
    ), NA)
    var_args <- list(ELEV_CAT = list(siteuse = list(levels = c("Base"), pch = c(19))))
    expect_error(sp_plot(eqprob_rho, NE_Lakes,
      formula = siteuse ~ ELEV_CAT,
      var_args = var_args, onlyshow = "low", siteuse = c("Base"), cex = 1
    ), NA)
  })

  #--------------------------------------
  #-------- one sided formula
  #--------------------------------------

  # one sided formulas
  test_that("one sided formulas work", {
    expect_error(sp_plot(NE_Lakes, formula = ~1), NA)
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT), NA)
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, fix_bbox = FALSE), NA)
    expect_error(sp_plot(NE_Lakes, formula = ~ ELEV_CAT:AREA_CAT), NA)
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV), NA)
  })

  # changing graphical parameters
  test_that("one sided formulas work", {
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, pch = 19), NA)
    var_args <- list(ELEV_CAT = list(main = "maintest"))
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, var_args = var_args), NA)
    varlevel_args <- list(ELEV_CAT = list(levels = c("low", "high"), cex = c(1, NA)))
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, varlevel_args = varlevel_args), NA)
    varlevel_args <- list(ELEV_CAT = list(levels = c("low", "high"), cex = c(1, 2)))
    expect_error(sp_plot(NE_Lakes, formula = ~ELEV_CAT, varlevel_args = varlevel_args), NA)
    expect_error(sp_plot(NE_Lakes,
      formula = ~ELEV_CAT,
      varlevel_args = varlevel_args, var_args = var_args, pch = 19
    ), NA)
  })


  #--------------------------------------
  #-------- two sided formula
  #--------------------------------------

  # two sided formulas
  test_that("two sided formulas work", {
    expect_error(sp_plot(NE_Lakes, formula = ELEV ~ 1), NA)
    expect_error(sp_plot(NE_Lakes, formula = ELEV ~ AREA_CAT, onlyshow = "small"), NA)
  })

  # changing graphical parameters
  test_that("two sided formulas work", {
    var_args <- list(AREA_CAT = list(ELEV_CAT = list(levels = c("low", "high"), pch = c(1, 19))))
    expect_error(sp_plot(NE_Lakes, formula = ELEV_CAT ~ AREA_CAT, var_args = var_args, onlyshow = "large"), NA)
  })
}

context("sp_summary")

# find system info
on_solaris <- Sys.info()[["sysname"]] == "SunOS"
if (on_solaris) {
  test_that("on solaris", {
    expect_true(on_solaris)
  })
} else {


  # set reproducible seed (as there are random components here)
  set.seed(5)

  #--------------------------------------
  #-------- one sided formula
  #--------------------------------------

  # intercept only formula
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~1)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 1)
    expect_equal(length(output[, 1]), 1)
  })

  # single categorical variable
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~ELEV_CAT)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), length(unique(NE_Lakes$ELEV_CAT)))
  })

  # single categorical variable removing intercept
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~ ELEV_CAT - 1)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 1)
    expect_equal(length(output[, 1]), length(unique(NE_Lakes$ELEV_CAT)))
  })

  # two categorical variables
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~ ELEV_CAT + AREA_CAT)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 3)
    expect_equal(length(output[, 2]), length(unique(NE_Lakes$ELEV_CAT)))
    expect_equal(length(output[, 3]), length(unique(NE_Lakes$AREA_CAT)))
  })

  # interaction between two categorical variables
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~ ELEV_CAT:AREA_CAT)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), length(unique(NE_Lakes$ELEV_CAT)) * length(unique(NE_Lakes$AREA_CAT)))
  })

  # onlyshow for interaction between two categorical variables
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~ ELEV_CAT:AREA_CAT, onlyshow = "low:small")
    expect_true(exists("output"))
    expect_equal(NCOL(output), 1)
    expect_equal(length(output[, 1]), 1)
  })

  # single categorical variable
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~ELEV)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 6) # 5 number summary plus mean
  })

  # * interaction operator works
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~ ELEV_CAT * AREA_CAT)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 4)
  })

  # . interaction operator works
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~.)
    expect_true(exists("output"))
    expect_equal(NCOL(output), NCOL(NE_Lakes) + 1) # our summary adds an intercept
  })

  # maxsum works operator works
  test_that("one sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = ~AREA_CAT, maxsum = 1)
    expect_true(exists("output"))
    expect_equal(length(output[, 2]), 1)

    output <- sp_summary(NE_Lakes, formula = ~AREA_CAT)
    expect_true(exists("output"))
    expect_equal(length(output[, 2]), 2)
  })

  #--------------------------------------
  #-------- two sided formula
  #--------------------------------------

  # numeric left hand side variable
  test_that("two sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = AREA ~ ELEV_CAT)
    expect_true(exists("output"))
    expect_equal(NROW(output), 2)
    expect_equal(NCOL(output[[1]]), 6)
    expect_equal(NROW(output[[1]]), 1)
    expect_equal(NCOL(output[[2]]), 6)
    expect_equal(NROW(output[[2]]), length(unique(NE_Lakes$ELEV_CAT)))
  })

  # numeric right hand side variable
  test_that("two sided formulas work", {
    output <- sp_summary(NE_Lakes, formula = AREA_CAT ~ ELEV_CAT)
    expect_true(exists("output"))
    expect_equal(NROW(output), 2)
    expect_equal(NCOL(output[[1]]), length(unique(NE_Lakes$AREA_CAT)))
    expect_equal(NROW(output[[1]]), 1)
    expect_equal(NCOL(output[[2]]), length(unique(NE_Lakes$AREA_CAT)))
    expect_equal(NROW(output[[2]]), length(unique(NE_Lakes$ELEV_CAT)))
  })

  #################################################
  ########### sp_design
  #################################################

  n_base <- 50
  eqprob <- grts(NE_Lakes, n_base = n_base)
  eqprob_legacy <- grts(NE_Lakes, n_base = n_base, legacy_sites = NE_Lakes_Legacy)
  n_over <- 5
  eqprob_rho <- grts(NE_Lakes, n_base = n_base, n_over = n_over)
  n_near <- 1
  eqprob_nn <- grts(NE_Lakes, n_base = n_base, n_near = n_near)
  eqprob_both <- grts(NE_Lakes, n_base = n_base, n_over = n_over, n_near = n_near)
  n_base_strat <- c(low = 25, high = 25)
  eqprob_strat <- grts(NE_Lakes, n_base = n_base_strat, stratum_var = "ELEV_CAT")

  #--------------------------------------
  #-------- one sided formula
  #--------------------------------------

  # one sided formula
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob, formula = ~siteuse)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 1)
  })

  # one sided formula with additional variable
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob, formula = ~ siteuse + ELEV_CAT)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 3)
    expect_equal(sum(!is.na(output[, 2])), 1)
  })

  # use with legacy variable
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_legacy, formula = ~siteuse)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 2)
  })

  # siteuse variable being set
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_legacy, formula = ~siteuse, siteuse = "Base")
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 1)
  })

  # use with rho replacement
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_rho, formula = ~siteuse)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 2)
  })

  # siteuse variable being set
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_rho, formula = ~siteuse, siteuse = "Base")
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 1)
  })

  # use with nn replacement
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_nn, formula = ~siteuse)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 2)
  })

  # siteuse variable being set
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_nn, formula = ~siteuse, siteuse = "Base")
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 1)
  })

  # use with both replacement
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_both, formula = ~siteuse)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 3)
  })

  # siteuse variable being set
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_both, formula = ~siteuse, siteuse = "Base")
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 1)
  })

  # siteuse variable being set
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_both, formula = ~siteuse, siteuse = c("Base", "Over"))
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 2)
  })

  # siteuse variable being set
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_both, formula = ~siteuse, siteuse = c("Base", "Near"))
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 2)
  })

  # siteuse variable being set
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_both, formula = ~siteuse, siteuse = c("Over", "Near"))
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 2)
  })

  # with a stratified design
  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_strat, formula = ~siteuse)
    expect_true(exists("output"))
    expect_equal(NCOL(output), 2)
    expect_equal(length(output[, 2]), 1)
  })

  #--------------------------------------
  #-------- two sided formula
  #--------------------------------------

  test_that("one sided formulas work", {
    output <- sp_summary(eqprob_strat, formula = siteuse ~ stratum)
    expect_true(exists("output"))
    expect_equal(NROW(output), 2)
    expect_equal(NCOL(output[[1]]), 1)
    expect_equal(NROW(output[[1]]), 1)
    expect_equal(NCOL(output[[2]]), 1)
    expect_equal(NROW(output[[2]]), length(unique(eqprob_strat$sites_base$stratum)))
  })
}

################################################################################
# Function: mean_est (not exported)
# Programmer: Tom Kincaid
# Date: June 11, 2021
#
#' Mean Estimates for Probability Survey Data
#'
#' This function calculates mean estimates using the svymean function in the
#' survey package.  Upper and lower confidence bounds also are estimated.
#'
#' @param meansum Data frame containing estimates.
#'
#' @param dframe Data frame containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param itype Character value that identifies a factor variable in the design
#'   argument containing subpopulation (domain) values.
#'
#' @param lev_itype Character vector that provides levels of the subpopulation
#'   variable.
#'
#' @param nlev_itype Numeric value that provides the number of levels of the
#'   subpopulation variable.
#'
#' @param ivar Character value that identifies the response variable.
#'
#' @param design Object of class \code{survey.design} that specifies a complex
#'   survey design.
#'
#' @param design_names Character vector that provides names of survey design
#'   variables in the \code{design} argument.
#'
#' @param var_nondetect Character value that identifies the name of a logical
#'   variable in the \code{dframe} data frame specifying the presence of not
#'   detected (nondetect) values for the response variable.
#'
#' @param vartype Character value providing the choice of the variance
#'   estimator, where "Local" = the local mean estimator, \code{"SRS"} = the
#'   simple random sampling estimator, \code{"HT"} = the Horvitz-Thompson
#'   estimator, and \code{"YG"} = the Yates-Grundy estimator.  The default value
#'   is \code{"Local"}.
#'
#'
#' @param conf Numeric value for the confidence level.
#'
#' @param mult Numeric value that provides the Normal distribution confidence
#'   bound multiplier.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @return A list composed of the following objects:
#'   \describe{
#'     \item{\code{meansum}}{data frame containing the percentile estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#' @section Other Functions Required:
#'   \describe{
#'     \item{\code{\link{SE}}}{extracts standard errors from a survey design
#'       object}
#'     \item{\code{\link{confint}}}{computes confidence intervals for a survey
#'       design object}
#'     \item{\code{mean_localmean}}{organizes input and output for
#'       calculation of the local mean variance estimator for the estimated
#'       mean}
#'     \item{\code{\link{svyby}}}{Compute survey statistics on subsets of a
#'       survey defined by factors}
#'     \item{\code{\link{svymean}}}{calculates the mean for a complex survey
#'       design}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @seealso
#'   \code{\link{confint}}
#'   \code{\link{SE}}
#'   \code{\link{svyby}}
#'   \code{\link{svymean}}
#'
#' @keywords survey univar
#'
#' @noRd
################################################################################

mean_est <- function(meansum, dframe, itype, lev_itype, nlev_itype, ivar,
                     design, design_names, var_nondetect, vartype, conf, mult,
                     warn_ind, warn_df) {

  # Assign a value to the function name variable

  fname <- "mean_est"

  #
  # Calculate mean estimates
  #

  if (is.null(var_nondetect)) {

    # Calculate the mean estimate, standard error estimate, and confidence
    # bound estimates for each combination of subpopulation and response
    # variable for the case where nondetects are not present

    tst <- !is.na(dframe[, itype])
    if (nlev_itype == 1) {
      nresp <- sum(!is.na(dframe[tst, ivar]))
      if (nresp == 1) {
        warn_ind <- TRUE
        act <- "Variance of the mean estimate was not calculated.\n"
        warn <- paste0("Variance of the mean estimate was not calculated for subpopulation type \"", itype, "\" \nsince the number of non-missing response values equals one.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = itype, subpop = NA, indicator = ivar,
          stratum = NA, warning = I(warn), action = I(act)
        ))
        temp <- mean(dframe[tst, ivar], na.rm = TRUE)
        meanest <- temp
        nresp <- 1
        stderr <- 0
        lbound <- temp
        ubound <- temp
      } else {
        rslt <- svymean(
          make.formula(ivar),
          design = subset(design, tst), na.rm = TRUE
        )
        meanest <- rslt
        nresp <- sum(!is.na(dframe[, ivar]))
        if (vartype == "Local") {
          temp <- mean_localmean(
            itype, lev_itype, nlev_itype, c(1), ivar, design, design_names,
            meanest[1], mult, warn_ind, warn_df
          )
          stderr <- temp$stderr
          lbound <- unlist(temp$confval[1])
          ubound <- unlist(temp$confval[2])
          warn_ind <- temp$warn_ind
          warn_df <- temp$warn_df
        } else {
          stderr <- SE(rslt)
          temp <- confint(rslt, level = conf / 100)
          lbound <- temp[1]
          ubound <- temp[2]
        }
      }
    } else {
      meanest <- rep(NA, nlev_itype)
      nresp <- rep(NA, nlev_itype)
      stderr <- rep(NA, nlev_itype)
      lbound <- rep(NA, nlev_itype)
      ubound <- rep(NA, nlev_itype)
      nval <- tapply(dframe[tst, ivar], dframe[tst, itype], function(x) {
        sum(!is.na(x))
      })
      subpop_ind <- nval > 1
      if (any(!subpop_ind)) {
        levs <- (1:nlev_itype)[!subpop_ind]
        for (i in levs) {
          tst_mean <- tst & dframe[, itype] %in% lev_itype[i]
          temp <- mean(dframe[tst_mean, ivar], na.rm = TRUE)
          meanest[i] <- temp
          nresp[i] <- 1
          stderr[i] <- 0
          lbound[i] <- temp
          ubound[i] <- temp
        }
      }
      if (any(subpop_ind)) {
        tst <- tst & dframe[, itype] %in% lev_itype[subpop_ind]
        levs <- (1:nlev_itype)[subpop_ind]
        rslt <- svyby(
          make.formula(ivar), make.formula(itype),
          design = subset(design, tst),
          svymean, na.rm = TRUE
        )
        meanest[levs] <- rslt[, 2]
        temp <- tapply(dframe[, ivar], dframe[, itype], function(x) {
          sum(!is.na(x))
        })
        nresp[levs] <- temp[levs]
        if (vartype == "Local") {
          temp <- mean_localmean(
            itype, lev_itype, nlev_itype, levs, ivar, design, design_names,
            meanest, mult, warn_ind, warn_df
          )
          stderr[levs] <- temp$stderr[levs]
          lbound[levs] <- unlist(temp$confval[levs, 1])
          ubound[levs] <- unlist(temp$confval[levs, 2])
          warn_ind <- temp$warn_ind
          warn_df <- temp$warn_df
        } else {
          stderr[levs] <- SE(rslt)
          temp <- confint(rslt, level = conf / 100)
          lbound[levs] <- temp[, 1]
          ubound[levs] <- temp[, 2]
        }
      }
    }
  } else {

    # To be implemented
  }

  # Assign identifiers and estimates to the meansum data frame

  if (is.null(var_nondetect)) {
    if (nlev_itype == 1) {
      meansum <- rbind(meansum, data.frame(
        Type = itype,
        Subpopulation = lev_itype,
        Indicator = ivar,
        nResp = nresp,
        Estimate = meanest[1],
        StdError = stderr[1],
        MarginofError = mult * stderr[1],
        LCB = lbound,
        UCB = ubound
      ))
    } else {
      for (i in 1:nlev_itype) {
        meansum <- rbind(meansum, data.frame(
          Type = itype,
          Subpopulation = lev_itype[i],
          Indicator = ivar,
          nResp = nresp[i],
          Estimate = meanest[i],
          StdError = stderr[i],
          MarginofError = mult * stderr[i],
          LCB = unlist(lbound[i]),
          UCB = unlist(ubound[i])
        ))
      }
    }
  } else {

    # To be implemented
  }

  # Return the meansum data frame, the warn_ind logical value, and the warn_df
  # data frame

  list(meansum = meansum, warn_ind = warn_ind, warn_df = warn_df)
}

################################################################################
# Function: cont_cdfplot (exported)
# Programmer Tom Kincaid
# Date: May 5, 2021
# Revised: May 6, 2021 to correct an error when specifying argument type_cdf for
#          function cdf_plot
# Revised: August 31, 2021 to improve documentation
#
#' Create a PDF file containing cumulative distribution functions (CDF) plots
#'
#' This function creates a PDF file containing CDF plots.  Input data for the
#' plots is provided by a data frame with the same structure as the "CDF"
#' output from  \code{cont_analysis}.  Plots are produced for every combination of \code{Type} of
#' population, \code{Subpopulation} within \code{Type}, and \code{Indicator} (every combination
#' of subpopulations, subpopulation levels, and variables).
#'
#' @param pdffile Name of the PDF file.  The default is "cdf2x2.pdf".
#'
#' @param cdfest Data frame with the same structure as the "CDF"
#' output from  \code{cont_analysis}.
#'
#' @param units_cdf Indicator for the label utilized for the left side y-axis
#'   and the values used for the left side y-axis tick marks, where "Percent"
#'   means the label and values are in terms of percent of the population, and
#'   "Units" means the label and values are in terms of units (count, length,
#'   or area) of the population.  The default is "Percent".
#'
#' @param ind_type Character vector consisting of the values "Continuous" or
#'   "Ordinal" that controls the type of CDF plot for each indicator.  The
#'   default is "Continuous" for every indicator.
#'
#' @param log Character vector consisting of the values "" or "x" that
#'   controls whether the x axis uses the original scale ("") or the base 10
#'   logarithmic scale ("x") for each indicator.  The default is "" for every
#'   indicator.
#'
#' @param xlab Character vector consisting of the x-axis label for each
#'   indicator. If this argument equals NULL, then indicator names are used as
#'   the labels. The default is NULL.
#'
#' @param ylab Character string providing the left side y-axis label.  If
#'   argument units_cdf equals "Units", a value should be provided for this
#'   argument.  Otherwise, the label will be "Percent".  The default is
#'   "Percent".
#'
#' @param ylab_r Character string providing the label for the right side y-axis
#'   (and, hence, determining the values used for the right side y-axis tick
#'   marks), where NULL means a right side y-axis is not created.  If this
#'   argument equals "Same", the right side y-axis will have the same label and
#'   tick mark values as the left side y-axis.  If this argument equals a
#'   character string other than "Same", the right side y-axis label will be the
#'   value provided for argument ylab_r, and the right side y-axis tick mark
#'   values will be determined by the choice not utilized for argument
#'   units_cdf, which means that the default value of argument units_cdf (i.e.,
#'   "Percent") will result in the right side y-axis tick mark values being
#'   expressed  in terms of units of the population (i.e., count, length, or
#'   area).  The default is NULL.
#'
#' @param legloc Indicator for location of the plot legend, where "BR" means
#'   bottom right, "BL" means bottom left, "TR" means top right, "TL" means
#'   top left, and NULL means no legend.  The default is NULL.
#'
#' @param cdf_page  Number of CDF plots on each page, which must be chosen from
#'   the values: 1, 2, 4, or 6.  The default is 4.
#'
#' @param width Width of the graphic region in inches.  The default is 10.
#'
#' @param height Height of the graphic region in inches.  The default is 8.
#'
#' @param confcut Numeric value that controls plotting confidence limits at
#'   the CDF extremes.  Confidence limits for CDF values (percent scale) less
#'   than confcut or greater than 100 minus confcut are not plotted.  A value of
#'   zero means confidence limits are plotted for the complete range of the CDF.
#'   The default is 0.
#'
#' @param cex.main Expansion factor for the plot title.  The default is 1.2.
#'
#' @param cex.legend Expansion factor for the legend title. The default is 1.
#'
#' @param ... Additional arguments passed to the \code{cdf_plot} function.
#'
#' @return A PDF file containing the CDF plots.
#'
#' @seealso
#'   \describe{
#'   \item{\code{\link{cdf_plot}}}{for plotting a cumulative distribution
#'     function (CDF)}
#'   \item{\code{\link{cont_cdftest}}}{for CDF hypothesis testing}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey plot
#'
#' @examples
#' \dontrun{
#' dframe <- data.frame(
#'   siteID = paste0("Site", 1:100),
#'   wgt = runif(100, 10, 100),
#'   xcoord = runif(100),
#'   ycoord = runif(100),
#'   stratum = rep(c("Stratum1", "Stratum2"), 50),
#'   ContVar = rnorm(100, 10, 1),
#'   All_Sites = rep("All Sites", 100),
#'   Resource_Class = rep(c("Good", "Poor"), c(55, 45))
#' )
#' myvars <- c("ContVar")
#' mysubpops <- c("All_Sites", "Resource_Class")
#' mypopsize <- data.frame(
#'   Resource_Class = c("Good", "Poor"),
#'   Total = c(4000, 1500)
#' )
#' myanalysis <- cont_analysis(dframe,
#'   vars = myvars, subpops = mysubpops,
#'   siteID = "siteID", weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
#'   stratumID = "stratum", popsize = mypopsize
#' )
#' cont_cdfplot("myanalysis.pdf", myanalysis$CDF, ylab_r = "Stream Length (km)")
#' }
#'
#' @export
################################################################################

cont_cdfplot <- function(pdffile = "cdf2x2.pdf", cdfest, units_cdf = "Percent",
                         ind_type = rep("Continuous", nind), log = rep("", nind), xlab = NULL,
                         ylab = NULL, ylab_r = NULL, legloc = NULL, cdf_page = 4, width = 10,
                         height = 8, confcut = 0, cex.main = 1.2, cex.legend = 1, ...) {

  # Open the PDF file

  pdf(file = pdffile, width = width, height = height)

  # Set up the number of plots per page

  if (cdf_page == 1) {
    mf <- c(1, 1)
  } else if (cdf_page == 2) {
    mf <- c(2, 1)
  } else if (cdf_page == 4) {
    mf <- c(2, 2)
  } else if (cdf_page == 6) {
    mf <- c(3, 2)
  } else {
    mf <- c(2, 2)
  }

  # Set graphical parameter values

  op <- par(mfrow = mf, mgp = c(1.5, 0.7, 0))

  # Assign the vectors of population type names and indicator names

  typenames <- unique(cdfest$Type)
  indnames <- unique(cdfest$Indicator)
  nind <- length(indnames)

  # If not supplied, set up the x-axis labels

  if (is.null(xlab)) {
    xlab <- as.character(indnames)
    names(xlab) <- as.character(indnames)
  }

  # Obtain the confidence level

  conflev <- as.numeric(substr(names(cdfest)[9], 4, 5))

  # Create the plots

  for (itype in 1:length(typenames)) {
    tsttype <- cdfest$Type == typenames[itype]
    subnames <- unique(cdfest$Subpopulation[tsttype])
    for (jsub in 1:length(subnames)) {
      tstsub <- tsttype & cdfest$Subpopulation == subnames[jsub]
      temp <- match(unique(cdfest$Indicator[tstsub]), indnames)
      for (kin in temp) {
        tstind <- tstsub & cdfest$Indicator == indnames[kin]
        cdf_plot(
          cdfest[tstind, ],
          units_cdf = units_cdf, type_cdf = ind_type[kin],
          log = log[kin], xlab = xlab[indnames[kin]], ylab = ylab,
          ylab_r = ylab_r, main = paste0(
            typenames[itype], " - ",
            subnames[jsub], ": ", indnames[kin]
          ),
          legloc = legloc, confcut = confcut, conflev = conflev,
          cex.main = cex.main, cex.legend = cex.legend, ...
        )
      }
    }
  }


  # Reset graphical parameter values

  par(op)

  # Close the PDF file

  graphics.off()

  # Return an invisible NULL object

  invisible(NULL)
}

################################################################################
# Function: interp_axis (not exported)
# Programmer Tom Kincaid
# Date: May 5, 2021
#
#' Create Right-Side y-Axis Lables for a CDF Plot
#'
#' This function creates right side y-axis labels for a CDF plot.  It assumes
#' that arguments cdfest_l and cdfest_r are strictly increasing.  If argument
#' yl_lab is less than the first cdfest_l vlaue, then the funciton assumes 0 for
#' both cdfest_l and cdfest_r.
#'
#' @param yl_lab Vector of left side y-axis labels, which are the basis for
#'   interpolating cdfest_r values.
#'
#' @param cdfest_l Vector of CDF estimates corresponding to the left side
#'   y-axis.
#'
#' @param cdfest_r Vector of CDF estimates corresponding to the right side
#'   y-axis.
#'
#' @return A numeric vector consisting of the right side y-axis labels.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @noRd
################################################################################

interp_axis <- function(yl_lab, cdfest_l, cdfest_r) {
  nvec <- 1:length(cdfest_l)
  rslt <- numeric(0)
  for (j in 1:length(yl_lab)) {
    high <- ifelse(length(nvec[cdfest_l >= yl_lab[j]]) > 0,
      min(nvec[cdfest_l >= yl_lab[j]]), nvec[length(nvec)]
    )
    low <- ifelse(length(nvec[cdfest_l <= yl_lab[j]]) > 0,
      max(nvec[cdfest_l <= yl_lab[j]]), NA
    )
    if (is.na(low)) {
      ulow <- 0
      plow <- 0
      pdis <- (yl_lab[j] - plow) / (cdfest_l[high] - plow)
      rslt[j] <- ulow + pdis * (cdfest_r[high] - ulow)
    } else {
      if (high > low) {
        pdis <- (yl_lab[j] - cdfest_l[low]) / (cdfest_l[high] - cdfest_l[low])
        rslt[j] <- cdfest_r[low] + pdis * (cdfest_r[high] - cdfest_r[low])
      } else {
        rslt[j] <- cdfest_r[high]
      }
    }
  }

  return(rslt)
}

###############################################################################
# Function: sp_plot (exported)
# Programmers: Michael Dumelle
# Date: January 22, 2021
#' Plot sampling frames, design sites, and analysis data.
#'
#' This function plots sampling frames, design sites, and analysis data.
#' If the left-hand side of the formula is empty, plots
#' are of the distributions of the right-hand side variables. If the left-hand side
#' of the variable contains a variable, plots are of the left-hand size variable
#' for each level of each right-hand side variable.
#' This function is largely built on \code{plot.sf()}, and all spsurvey plotting
#' methods can supply additional arguments to \code{plot.sf()}. For more information on
#' plotting in \code{sf}, run \code{?sf::plot.sf()}. Equivalent to \code{spsurvey::plot()}; both
#' are currently maintained for backwards compatibility.
#'
#' @param object An object to plot. When plotting sampling frames or analysis data,
#' a data frame or \code{sf} object. When plotting design sites, an object created by \code{grts()} or
#' \code{irs()} (which has class \code{sp_design}).
#'
#' @param sframe The sampling frame (an \code{sf} object) to plot alongside design
#' sites. This argument is only used when \code{object} corresponds to the design sites.
#'
#' @param formula A formula. One-sided formulas are used to summarize the
#' distribution of numeric or categorical variables. For one-sided formulas,
#' variable names are placed to the right of \code{~} (a right-hand side variable).
#' Two sided formulas are
#' used to summarize the distribution of a left-hand side variable
#' for each level of each right-hand side categorical variable in the formula.
#' Note that only for two-sided formulas are numeric right-hand side variables
#' coerced to a categorical variables. If an intercept
#' is included as a right-hand side variable (whether the formula is one-sided or
#' two-sided), the total will also be summarized. When plotting sampling frames
#' or analysis data, the default formula is \code{~ 1}. When plotting design sites,
#' \code{siteuse} should be used in the formula, and the default formula is
#' \code{~ siteuse}.
#'
#' @param siteuse A character vector of site types to include when plotting design sites.
#' It can only take on values \code{"sframe"} (sampling frame),
#' \code{"Legacy"} (for legacy sites), \code{"Base"} (for base sites),
#' \code{"Over"} (for \code{n_over} replacement sites), and \code{"Near"}
#' (for \code{n_near} replacement sites). The order of sites represents the
#' layering in the plot (e.g. \code{siteuse = c("Base", "Legacy")} will plot
#' legacy sites on top of base sites. Defaults to all non-\code{NULL} elements
#' in \code{x} and \code{y} with plot order \code{"sframe"}, \code{"Legacy"},
#' \code{"Base"}, \code{"Over"}, \code{"Near"}.
#'
#' @param var_args A named list. The name of each list element corresponds to a
#' right-hand side variable in \code{formula}. Values in the list are composed of
#' graphical arguments that are to be passed to \strong{every} level of the
#' variable. To see all graphical arguments available, run \code{?plot.sf}.
#'
#' @param varlevel_args A named list. The name of each list element corresponds to a
#' right-hand side variable in \code{formula}. The first element in this list
#' should be \code{"levels"} and contain all levels of the particular right-hand side variable. Subsequent
#' names correspond to graphical arguments that are to be passed to
#' the specified levels (in order) of the right-hand side variable. Values for each
#' graphical argument must be specified for each level of the right-hand side variable,
#' but applicable sf defaults will be matched by inputting the value \code{NA}.
#' To see all graphical arguments available, run \code{?plot.sf}
#'
#' @param geom Should separate geometries for each level of the right-hand
#' side \code{formula} variables be plotted? Defaults to \code{FALSE}.
#'
#' @param onlyshow A string indicating the single level of the single right-hand side
#' variable for which a summary is requested. This argument is only used when
#' a single right-hand side variable is provided.
#'
#' @param fix_bbox Should the geometry bounding box be fixed across plots?
#' If a length-four vector with names "xmin", "ymin", "xmax", and "ymax" and values
#' indicating bounding box edges, the bounding box will be fixed as \code{fix_bbox}
#' across plots. If \code{TRUE}, the bounding box will be fixed across plots as the
#' bounding box of \code{object}. If \code{FALSE}, the bounding box will vary across
#' plots according to the unique geometry for each plot. Defaults to \code{TRUE}.
#'
#' @param xcoord Name of the x-coordinate (east-west) in \code{object} (only required if
#' \code{object} is not an \code{sf} object).
#'
#' @param ycoord Name of y (north-south)-coordinate in \code{object} (only required if
#' \code{object} is not an \code{sf} object).
#'
#' @param crs Projection code for \code{xcoord} and \code{ycoord} (only
#' required if \code{object} is not an \code{sf} object).
#'
#' @param ... Additional arguments to pass to \code{plot.sf()}.
#'
#' @author Michael Dumelle \email{Dumelle.Michael@@epa.gov}
#'
#' @name sp_plot
#'
#' @export
#'
#' @examples
#' \dontrun{
#' data("NE_Lakes")
#' sp_plot(NE_Lakes, formula = ~ELEV_CAT)
#' sample <- grts(NE_Lakes, 30)
#' sp_plot(sample, NE_Lakes)
#' data("NLA_PNW")
#' sp_plot(NLA_PNW, formula = ~BMMI)
#' }
sp_plot <- function(object, ...) {
  UseMethod("sp_plot", object)
}

#' @name sp_plot
#' @method sp_plot default
#' @export
sp_plot.default <- function(object, formula = ~1, xcoord, ycoord, crs,
                            var_args = NULL, varlevel_args = NULL,
                            geom = FALSE, onlyshow = NULL, fix_bbox = TRUE, ...) {

  # find system info
  on_solaris <- Sys.info()[["sysname"]] == "SunOS"
  if (on_solaris) {
    stop("sp_plot() is not supported on Solaris.")
  }

  # coerce to sf
  if (!inherits(object, "sf")) {
    object <- st_as_sf(object, coords = c(xcoord, ycoord), crs = crs)
  }

  # setting old graphical parameter value
  oldpar <- par(no.readonly = TRUE)
  # setting exit handler
  on.exit(par(ask = oldpar$ask), add = TRUE)

  # storing dotlist
  dot_list <- list(...)

  # fixing bbox
  if (!is.logical(fix_bbox)) {
    if (length(fix_bbox) == 4 && !is.null(names(fix_bbox)) && all(names(fix_bbox) %in% c("xmin", "ymin", "xmax", "ymax"))) {
      attr(st_geometry(object), "bbox") <- fix_bbox[c("xmin", "ymin", "xmax", "ymax")]
      fix_bbox <- TRUE # set as logical to be used later
    } else {
      stop("If fix_bbox is a vector, it must have length 4 with names \"xmin\", \"ymin\", \"xmax\", and \"ymax\" and values indicating bounding box edges.", call. = FALSE)
    }
  }
  if (fix_bbox) {
    dot_list$xlim <- st_bbox(object)[c(1, 3)]
    dot_list$ylim <- st_bbox(object)[c(2, 4)]
  }


  # making variable list
  formlist <- make_formlist(formula, onlyshow, object)
  varsf <- make_varsf(object, formlist)
  # varsf <- na.omit(varsf)

  # plot geometry or response for ~ 1
  if (length(formlist$varlabels) == 0 && formlist$intercept) {
    if (is.null(formlist$response)) {
      if (!("main" %in% names(dot_list))) {
        dot_list$main <- paste(expression("~"), " ", "1", sep = "")
      }
      sfplot <- do.call("plot", c(list(st_geometry(object)), dot_list))
    } else {
      if (!("main" %in% names(dot_list))) {
        dot_list$main <- paste(formlist$response, " ", expression("~"), " ", "1", sep = "")
      }
      sfplot <- do.call("plot", c(list(varsf[formlist$response]), dot_list))
    }
  } else {
    if (is.null(formlist$response)) {
      # get varlevel_args list
      if (!is.null(varlevel_args)) {
        varlevel_args_list <- make_varlevel_args_list(varsf, varlevel_args)
      } else {
        varlevel_args_list <- varlevel_args
      }

      if (is.null(onlyshow)) {
        if (geom) {
          # turning on ask if necessary
          if (get_varlevels(formlist, varsf) > 1) {
            par(ask = TRUE)
          }
          sfplot <- lapply(formlist$varlabels, function(a) {
            varsf_split <- split(varsf[a], varsf[[a]])
            names_varsf_split <- names(varsf_split)
            varlevel_args_split <- split(as.data.frame(varlevel_args_list[[a]], stringsAsFactors = FALSE), varsf[[a]])
            lapply(names_varsf_split, function(b) {
              list_args <- c(var_args[[a]], varlevel_args_split[[b]], dot_list)
              if (!("main" %in% names(list_args))) {
                list_args$main <- paste(formlist$response, " ", expression("~"), " ", a, " (", b, ")", sep = "")
              }
              if (any(is.na(unlist(list_args)))) {
                list_args <- match_sf_defaults(varsf_split[[b]], list_args)
              }
              do.call("plot", c(list(st_geometry(varsf_split[[b]])), list_args))
            })
          })
          names(sfplot) <- formlist$varlabels
        } else {
          # turning on ask if necessary
          if (length(formlist$varlabels) > 1) {
            par(ask = TRUE)
          }
          sfplot <- lapply(formlist$varlabels, function(a) {
            list_args <- c(var_args[[a]], varlevel_args_list[[a]], dot_list)
            if (!("main" %in% names(list_args))) {
              list_args$main <- paste(" ", expression("~"), " ", a, sep = "")
            }
            if (any(is.na(unlist(list_args)))) {
              list_args <- match_sf_defaults(varsf[a], list_args)
            }
            do.call("plot", c(list(varsf[a]), list_args))
          })
          names(sfplot) <- formlist$varlabels
        }
      } else {
        varsf_sub <- varsf[varsf[[formlist$varlabels]] == formlist$onlyshow, ]
        if (geom) {
          if (!("main" %in% names(dot_list))) {
            dot_list$main <- paste(" ", expression("~"), " ", formlist$varlabels, " (", formlist$onlyshow, ")", sep = "")
          }
          sfplot <- do.call("plot", c(list(st_geometry(varsf_sub[formlist$varlabels])), dot_list))
        } else {
          if (!("main" %in% names(dot_list))) {
            dot_list$main <- paste(" ", expression("~"), " ", formlist$varlabels, sep = "")
          }
          sfplot <- do.call("plot", c(list(varsf_sub[formlist$varlabels]), dot_list))
        }
      }
    } else {
      # get varlevel_args list
      if (!is.null(varlevel_args)) {
        varlevel_args_list <- make_varlevel_args_list(varsf, varlevel_args)
      } else {
        varlevel_args_list <- varlevel_args
      }

      if (is.null(onlyshow)) {
        if (get_varlevels(formlist, varsf) > 1) {
          par(ask = TRUE)
        }

        if (is.numeric(varsf[[formlist$response]])) {
          sfplot <- lapply(formlist$varlabels, function(a) {
            varsf_split <- split(varsf[, c(formlist$response, a)], varsf[[a]])
            names_varsf_split <- names(varsf_split)
            varlevel_args_split <- split(as.data.frame(varlevel_args_list[[a]], stringsAsFactors = FALSE), varsf[[a]])
            lapply(names_varsf_split, function(b) {
              list_args <- c(var_args[[a]], varlevel_args_split[[b]], dot_list)
              if (!("main" %in% names(list_args))) {
                list_args$main <- paste(formlist$response, " ", expression("~"), " ", a, " (", b, ")", sep = "")
              }
              if (any(is.na(unlist(list_args)))) {
                list_args <- match_sf_defaults(varsf_split[[b]], list_args)
              }
              sfplot <- do.call("plot", c(list(varsf_split[[b]][formlist$response]), list_args))
              sfplot <- list(sfplot)
              names(sfplot) <- b
              sfplot
            })
          })
          names(sfplot) <- formlist$varlabels
        } else {
          if (!is.null(var_args)) {
            var_args_list <- make_var_args_list(varsf, var_args)
          } else {
            var_args_list <- NULL
          }
          sfplot <- lapply(formlist$varlabels, function(a) {
            varsf_split <- split(varsf[, c(formlist$response, a)], varsf[[a]])
            names_varsf_split <- names(varsf_split)
            varlevel_args_split <- split(as.data.frame(varlevel_args_list[[a]], stringsAsFactors = FALSE), varsf[[a]])
            var_args_split <- split(as.data.frame(var_args_list[[a]][[formlist$response]], stringsAsFactors = FALSE), varsf[[a]])
            lapply(names_varsf_split, function(b) {
              list_args <- c(var_args_split[[b]], varlevel_args_split[[b]], dot_list)
              if (!("main" %in% names(list_args))) {
                list_args$main <- paste(formlist$response, " ", expression("~"), " ", a, " (", b, ")", sep = "")
              }
              if (any(is.na(unlist(list_args)))) {
                list_args <- match_sf_defaults(varsf_split[[b]], list_args)
              }
              sfplot <- do.call("plot", c(list(varsf_split[[b]][formlist$response]), list_args))
              sfplot <- list(sfplot)
              names(sfplot) <- b
              sfplot
            })
          })
          names(sfplot) <- formlist$varlabels
        }
      } else {
        varsf_sub <- varsf[varsf[[formlist$varlabels]] == formlist$onlyshow, ]
        if (!is.null(var_args)) {
          var_args_list <- make_var_args_list(varsf, var_args)
          var_args_split <- split(as.data.frame(var_args_list[[formlist$varlabels]][[formlist$response]],
            stringsAsFactors = FALSE
          ), varsf[[formlist$varlabels]])
          var_args_split <- var_args_split[[formlist$onlyshow]]
        } else {
          var_args_list <- NULL
          var_args_split <- NULL
        }
        if (!("main" %in% names(dot_list))) {
          dot_list$main <- paste(formlist$response, " ", expression("~"), " ", formlist$varlabels, " (", formlist$onlyshow, ")", sep = "")
        }
        sfplot <- do.call("plot", c(list(varsf_sub[formlist$response]), var_args_split, dot_list))
      }
    }
  }
  invisible(sfplot)
}

#' @name sp_plot
#' @method sp_plot sp_design
#' @export
sp_plot.sp_design <- function(object, sframe = NULL, formula = ~siteuse, siteuse = NULL,
                             var_args = NULL, varlevel_args = NULL, geom = FALSE, onlyshow = NULL,
                             fix_bbox = TRUE, ...) {

  # find system info
  on_solaris <- Sys.info()[["sysname"]] == "SunOS"
  if (on_solaris) {
    stop("sp_plot() is not supported on Solaris.")
  }

  if ((is.null(siteuse) & (!is.null(object$sites_near))) | "Near" %in% siteuse) {
    object$sites_near$siteuse <- "Near"
  }

  # set siteuse when NULL
  if (is.null(siteuse)) {
    if (is.null(sframe)) {
      siteuse_sframe <- NULL
    } else {
      siteuse_sframe <- "sframe"
    }

    if (is.null(object$sites_legacy)) {
      siteuse_legacy <- NULL
    } else {
      siteuse_legacy <- "Legacy"
    }

    if (is.null(object$sites_base)) {
      siteuse_base <- NULL
    } else {
      siteuse_base <- "Base"
    }

    if (is.null(object$sites_over)) {
      siteuse_over <- NULL
    } else {
      siteuse_over <- "Over"
    }

    if (is.null(object$sites_near)) {
      siteuse_near <- NULL
    } else {
      siteuse_near <- "Near"
    }
    siteuse <- c(siteuse_sframe, siteuse_legacy, siteuse_base, siteuse_over, siteuse_near)
  }

  # bind
  siteuse_spr <- siteuse[!(siteuse %in% "sframe")]
  object <- sp_rbind(object, siteuse = siteuse_spr)
  # make formlists
  formlist_object <- make_formlist(formula, onlyshow, object)
  # make sframe
  varsf_object <- make_varsf(object, formlist_object)

  if (!is.null(sframe) & "sframe" %in% siteuse) {
    sframe$siteuse <- "sframe"
    # make formlists
    formlist_sframe <- make_formlist(formula, onlyshow, sframe)
    # make sframe
    varsf_sframe <- make_varsf(sframe, formlist_sframe)
  } else {
    varsf_sframe <- NULL
  }

  new_varsf <- rbind(varsf_object, varsf_sframe)
  # set as factor
  new_varsf$siteuse <- factor(new_varsf$siteuse, levels = siteuse)

  # arrange by factor level
  ordered_varsf <- with(new_varsf, new_varsf[order(new_varsf$siteuse), , drop = FALSE])

  # make the plot
  sp_plot.default(object = ordered_varsf, formula = formula, var_args = var_args, varlevel_args = varlevel_args, geom = geom, onlyshow = onlyshow, fix_bbox = fix_bbox, ...)
}

###############################################################################
# Function: adjwgt (exported)
# Programmer: Tony Olsen
# Date: July 17, 2002
# Last Revised: February 25, 2004
#
#' Adjust survey design weights by categories
#'
#' @description Adjust initial survey design weights so that the
#' final weights sum to a desired frame size. Adjusted weights
#' proportionally scale the initial weights to sum to the desired frame size.
#' Separate adjustments are applied to each category specified in \code{wgtcat}.
#'
#' @param wgt Vector of initial weights for each site. These equal
#'   the reciprocal of the site's inclusion probability.
#'
#' @param wgtcat Vector containing each site's weight adjustment
#'   category name. The default is \code{NULL}, which assumes every
#'   site is in the same category.
#'
#' @param framesize Vector containing the known size of the frame
#'   for each category name in \code{wgtcat}. If \code{wgtcat} is provided,
#'   the names in \code{framesize} must match the names in \code{wgtcat}.
#'   If \code{wgtcat} is not provided, an unnamed scalar is given to
#'   \code{framesize}.
#'
#' @param sites Vector indicating site use; \code{TRUE} indicates the site
#'   should be included in the weight adjustment and \code{FALSE} indicates
#'   the site should not be included in the weight adjustment. The default is
#'   \code{NULL}, which assumes every site should be included.
#'
#' @return Vector of adjusted weights, where the adjusted weight is set
#'   to \code{0} for sites whose value in the sites argument was set to
#'   \code{FALSE}.
#'
#' @author Tony Olsen \email{olsen.tony@@epa.gov}
#'
#' @keywords survey misc
#'
#' @examples
#' wgt <- runif(50)
#' wgtcat <- rep(c("A", "B"), c(30, 20))
#' framesize <- c(A = 15, B = 10)
#' sites <- rep(rep(c(TRUE, FALSE), c(9, 1)), 5)
#' adjwgt(wgt, wgtcat, framesize, sites)
#' @export
adjwgt <- function(wgt, wgtcat = NULL, framesize, sites = NULL) {

  # set default wgtcat argument if omitted

  if (is.null(wgtcat)) {
    wgtcat <- rep("wgtgrp", length(wgt))
    names(framesize) <- "wgtgrp"
  }

  # set default sites argument if omitted

  if (is.null(sites)) {
    sites <- rep(TRUE, length(wgt))
  }

  # Sum initial weights by wgtcat for evaluated sites

  wgtsum <- tapply(wgt[sites], wgtcat[sites], sum)

  # Adjustment factor to be applied to weights for adjustment category

  adjfac <- framesize / wgtsum[match(names(framesize), names(wgtsum))]
  wtadj <- adjfac[match(wgtcat, names(adjfac))]
  adjwgt <- wgt * wtadj
  adjwgt[!sites] <- 0
  as.vector(adjwgt)
}

context("weight adjustment")

test_that("adjust weight works scalar", {
  wgt <- runif(50)
  framesize <- 100
  wgt_new <- adjwgt(wgt, framesize = framesize)
  wgtsums <- sum(wgt_new)
  expect_equal(as.vector(wgtsums), as.vector(framesize))
})

test_that("adjust weight works without sites", {
  wgt <- runif(50)
  wgtcat <- rep(c("A", "B"), c(30, 20))
  framesize <- c(A = 15, B = 10)
  wgt_new <- adjwgt(wgt, wgtcat, framesize)
  wgtsums <- tapply(wgt_new, wgtcat, sum)
  expect_equal(names(wgtsums), names(framesize))
  expect_equal(as.vector(wgtsums), as.vector(framesize))
})

test_that("adjust weight works with sites", {
  wgt <- runif(50)
  wgtcat <- rep(c("A", "B"), c(30, 20))
  framesize <- c(A = 15, B = 10)
  sites <- rep(rep(c(TRUE, FALSE), c(9, 1)), 5)
  wgt_new <- adjwgt(wgt, wgtcat, framesize, sites)
  wgtsums <- tapply(wgt_new, wgtcat, sum)
  expect_equal(names(wgtsums), names(framesize))
  expect_equal(as.vector(wgtsums), as.vector(framesize))
})

# contrast_analysis <- function(dframe, vars, siteID = NULL, weight = "weight", xcoord, ycoord, vartype = "Local",
#                               conf = 95, statistics = c("Mean", "Total")) {
#
#
#
#   if (is.null(siteID)) {
#     siteID <- "siteID"
#     dframe$siteID <- paste("site", seq_len(nrow(dframe)), sep = "-")
#   }
#
#   # create design object
#   stratum_ind <- FALSE
#   stratumID <- NULL
#   cluster_ind <- FALSE
#   clusterID <- NULL
#   weight1 <- NULL
#   sizeweight <- FALSE
#   sweight <- NULL
#   sweight1 <- NULL
#   fpcfactor_ind <- FALSE
#   fpcsize <- NULL
#   Ncluster <- NULL
#   stage1size <- NULL
#   jointprob <- NULL
#
#   design <- survey_design(
#     dframe, siteID, weight, stratum_ind, stratumID, cluster_ind, clusterID,
#     weight1, sizeweight, sweight, sweight1, fpcfactor_ind, fpcsize, Ncluster,
#     stage1size, vartype, jointprob
#   )
#
#   all_vars <- all.vars(vars)
#   # incorporate form when all_vars has only length one (function of single variable)?
#   # what about three variables? Function to assign matrix to localmean_cov?
#   vars_form <- paste(all_vars, collapse = " + ")
#
#   if (vartype == "Local") {
#     local_weights <- localmean_weight(x = dframe[[xcoord]], y = dframe[[ycoord]], prb = 1 / dframe[[weight]])
#   }
#
#   if ("Mean" %in% statistics) {
#     rslt <- svymean(make.formula(vars_form), design)
#     rslt_con <- svycontrast(rslt, vars)
#     if (vartype == "Local") {
#       tw <- sum(dframe[[weight]])
#       local_vars <- do.call(cbind, lapply(all_vars, function(x) get_local_vars(dframe, weight, x, rslt, "Mean")))
#       cov_mx <- localmean_cov(local_vars, local_weights) / tw^2
#       derivs <- lapply(all_vars, D, expr = vars)
#       grad <- rbind(get_grad(all_vars, derivs, rslt))
#       se_val <- sqrt(grad %*% cov_mx %*% t(grad))
#     }
#
#     if (vartype == "SRS") {
#       se_val <- SE(rslt_con)
#     }
#
#     mean_df <- data.frame(Estimate = rslt_con[1], StdError = se_val, LCB95 = rslt_con[1] - 1.96 * se_val, UCB95 = rslt_con[1] + 1.96 * se_val)
#   }
#
#
#   if ("Total" %in% statistics) {
#     rslt <- svytotal(make.formula(vars_form), design)
#     rslt_con <- svycontrast(rslt, vars)
#     if (vartype == "Local") {
#       tw <- sum(dframe[[weight]])
#       local_vars <- do.call(cbind, lapply(all_vars, function(x) get_local_vars(dframe, weight, x, rslt, "Total")))
#       cov_mx <- localmean_cov(local_vars, local_weights)
#       derivs <- lapply(all_vars, D, expr = vars)
#       grad <- rbind(get_grad(all_vars, derivs, rslt))
#       se_val <- sqrt(grad %*% cov_mx %*% t(grad))
#     }
#
#     if (vartype == "SRS") {
#       se_val <- SE(rslt_con)
#     }
#
#     tot_df <- data.frame(Estimate = rslt_con[1], StdError = se_val, LCB95 = rslt_con[1] - 1.96 * se_val, UCB95 = rslt_con[1] + 1.96 * se_val)
#
#   }
#
#   contr_out <- list()
#
#   if ("Mean" %in% statistics) {
#     contr_out$Mean <- mean_df
#   } else {
#     contr_out$Mean <- NULL
#   }
#
#   if ("Total" %in% statistics) {
#     contr_out$Total <- tot_df
#   } else {
#     contr_out$Total <- NULL
#   }
#
#   contr_out
#
# }
#
# get_grad <- function(all_vars, derivs, svyout) {
#
#   # hoping for no name conflict with .use suffix
#   grad.use.spsurvey <- rep(0, length(all_vars))
#   derivs.use.spsurvey <- derivs
#   svyout.use.spsurvey <- svyout
#
#   for (x in all_vars) {
#     assign(x, svyout.use.spsurvey[[x]])
#   }
#
#   for (i in seq_along(grad.use.spsurvey)) {
#     grad.use.spsurvey[i] <- eval(derivs.use.spsurvey[[i]])
#   }
#   grad.use.spsurvey
# }
#
# get_local_vars <- function(dframe, weight, var, rslt, statistic) {
#   if (statistic == "Mean") {
#     val <- (dframe[[var]] - rslt[[var]]) * dframe[[weight]]
#   }
#
#   if (statistic == "Total") {
#     val <- dframe[[var]] * dframe[[weight]]
#   }
#   val
# }

#' spsurvey: Spatial Sampling Design and Analysis
#'
#' spsurvey implements a design-based approach to statistical inference,
#'   with a focus on spatial data.
#'   Spatially balanced samples are selected using the
#'   Generalized Random Tessellation Stratified (GRTS) algorithm.
#'   The GRTS algorithm can be applied to finite resources (point geometries) and
#'   infinite resources (linear / linestring and areal / polygon geometries) and flexibly
#'   accommodates a diverse set of sampling design features, including
#'   stratification, unequal inclusion probabilities, proportional (to size)
#'   inclusion probabilities, legacy (historical) sites, a minimum distance between
#'   sites, and two options for replacement sites (reverse hierarchical order and
#'   nearest neighbor). Data are analyzed using a wide
#'   range of analysis functions that perform categorical variable analysis, continuous
#'   variable analysis, attributable risk analysis, risk difference analysis, relative
#'   risk analysis, change analysis, and trend analysis. spsurvey can also be used to
#'   summarize objects, visualize objects, select samples that are not spatially balanced,
#'   select panel samples, measure the amount of spatial balance in a sample,
#'   adjust design weights, and more.
#'   This R package has been reviewed in accordance
#'   with U.S. Environmental Protection Agency policy and approved for publication.
#'   Mention of trade names or commercial products does not constitute endorsement or
#'   recommendation for use.
#'
# Import all packages listed as Depends or Imports
#' @import sf
#' @import survey
#' @importFrom boot boot
#' @importFrom crossdes find.BIB
#' @importFrom deldir deldir tile.list
#' @importFrom graphics axis box legend lines mtext par plot points text title
#' @importFrom grDevices graphics.off pdf rainbow
#' @importFrom lme4 lmer VarCorr
#' @importFrom MASS ginv
#' @importFrom stats addmargins chisq.test confint dist dnorm extractAIC ftable
#'             lm model.frame model.matrix na.omit na.pass optimize pchisq pf
#'             pnorm qnorm rnorm runif terms update var weights
#' @importFrom units set_units
#' @importFrom sampling UPpivotal
#'
#'
#'
"_PACKAGE"

################################################################################
# Function: change_analysis (exported)
# Programmer: Tom Kincaid
# Date: July 29, 2020
# Revised: August 14, 2020 to allow use of an sf object as the input data
#          argument (dframe)
# Revised: December 15, 2020 to allow use of the Horvitz-Thompson and
#          Yates-Grundy variance estimators and to use a new function named
#          survey_design to create the survey design object
# Revised: January 28, 2021 to replace "warn.vec" with "warn_vec"
# Revised: March 2, 2021 to revise the process for creating unique site ID
#          values
# Revised: April 7, 2021 to ensure that the dframe argument does not contain
#          zero rows
# Revised: April 19, 2021 to ensure that categorical variables, continuopus
#          variables, and subpopulation variables do not contain only missing
#          values for either of the surveys
# Revised: April 29, 2021 to ensure that the dframe argument only belongs to
#          class "data.frame"
# Revised: May 4, 2021 to avoid warning messages being generated during creation
#          of help files
# Revised: May 6, 2021 to ensure that sf objects do not belong to class tbl_df
# Revised: June 8, 2021 to simplify specification of the values required for
#          calculation of the finite population correction factor and to
#          eliminate use of the finite population correction factor with the
#          local mean variance estimator
# Revised: June 14, 2021 to acommodate the revised aproach for calculation of
#          mean estimates
# Revised: July 23, 2021 to eliminate analysis of subpopulation categories that
#          contain only missing values in one of the surveys
# Revised: September 9, 2021 to remove argument vars_nondetect since currently
#          it is not used and to revise the documentation for argument popsize
## Revised: September 14, 2021 to correct an error that can occur when checking
##          that subpopulation variables contain only missing values

#' Change analysis
#'
#' This function organizes input and output for the estimation of change between two
#' samples (for categorical and continuous variables).  The analysis data,
#' \code{dframe}, can be either a data frame or a simple features (\code{sf}) object.  If an
#' \code{sf} object is used, coordinates are extracted from the geometry column in the
#' object, arguments \code{xcoord} and \code{ycoord} are assigned values
#' \code{"xcoord"} and \code{"ycoord"}, respectively, and the geometry column is
#' dropped from the object.
#'
#' @param dframe Data to be analyzed (analysis data). A data frame or
#'   \code{sf} object containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param vars_cat Vector composed of character values that identify the
#'   names of categorical response variables in \code{dframe}.  The
#'   default is \code{NULL}.
#'
#' @param vars_cont Vector composed of character values that identify the
#'   names of continuous response variables in \code{dframe}.  The
#'   default is \code{NULL}.
#'
#' @param test Character string or character vector providing the location
#'   measure(s) to use for change estimation for continuous variables.  The
#'   choices are \code{"mean"}, \code{"total"}, \code{"median"}, or some
#'   combination of the three options (e.g., \code{c("mean", "total")}).
#'   The default is \code{"mean"}.
#'
#' @param subpops Vector composed of character values that identify the
#'   names of subpopulation (domain) variables in \code{dframe}.
#'   If a value is not provided, the value \code{"All_Sites"} is assigned to the
#'   subpops argument and a factor variable named \code{"All_Sites"} that takes
#'   the value \code{"All Sites"} is added to \code{dframe}.  The
#'   default value is \code{NULL}.
#'
#' @param surveyID Character value providing name of the survey ID variable in
#'   \code{dframe}.  The default value is \code{"surveyID"}.
#'
#' @param survey_names Character vector of length two that provides the survey
#'   names contained in the \code{surveyID} variable in the \code{dframe} data
#'   frame.  The two values in the vector identify the first survey and second
#'   survey, respectively.  If a value is not provided, unique values of the
#'   \code{surveyID} variable are assigned to the \code{survey_names} argument.
#'   The default is \code{NULL}.
#'
#' @param siteID Character value providing name of the site ID variable in
#'   \code{dframe}.  For a two-stage sample, the site ID variable
#'   identifies stage two site IDs.  The default value is \code{"siteID"}. If a
#'   unique site is visited in both surveys, the corresponding \code{siteID}
#'   should be the same for both entries.
#'
#' @param weight Character value providing name of the design weight
#'   variable in \code{dframe}.  For a two-stage sample, the
#'   weight variable identifies stage two weights.  The default value is
#'   \code{"weight"}.
#'
#' @param revisitwgt Logical value that indicates whether each repeat visit
#'   site has the same design weight in the two surveys, where
#'   \code{TRUE} = the weight for each repeat visit site is the same and
#'   \code{FALSE} = the weight for each repeat visit site is not the same.  When
#'   this argument is \code{FALSE}, all of the repeat visit sites are assigned
#'   equal weights when calculating the covariance component of the change
#'   estimate standard error.  The default is \code{FALSE}.
#'
#' @param xcoord Character value providing name of the x-coordinate variable in
#'   \code{dframe}.  For a two-stage sample, the x-coordinate
#'   variable identifies stage two x-coordinates.  Note that x-coordinates are
#'   required for calculation of the local mean variance estimator.  If \code{dframe}
#'   is an \code{sf} object, this argument is not required (as the geometry column
#'   in \code{dframe} is used to find the x-coordinate). The default
#'   value is \code{NULL}.
#'
#' @param ycoord Character value providing name of the y-coordinate variable in
#'   \code{dframe}.  For a two-stage sample, the y-coordinate
#'   variable identifies stage two y-coordinates.  Note that y-coordinates are
#'   required for calculation of the local mean variance estimator.  If \code{dframe}
#'   is an \code{sf} object, this argument is not required (as the geometry column
#'   in \code{dframe} is used to find the y-coordinate). The default
#'   value is \code{NULL}.
#'
#' @param stratumID Character value providing name of the stratum ID variable in
#'   \code{dframe}.  The default value is \code{NULL}.
#'
#' @param clusterID Character value providing the name of the cluster
#'   (stage one) ID variable in \code{dframe}.  Note that cluster
#'   IDs are required for a two-stage sample.  The default value is \code{NULL}.
#'
#' @param weight1 Character value providing name of the stage one weight
#'   variable in \code{dframe}.  The default value is \code{NULL}.
#'
#' @param xcoord1 Character value providing the name of the stage one
#'   x-coordinate variable in \code{dframe}.  Note that x
#'   coordinates are required for calculation of the local mean variance
#'   estimator.  The default value is \code{NULL}.
#'
#' @param ycoord1 Character value providing the name of the stage one
#'   y-coordinate variable in \code{dframe}.  Note that
#'   y-coordinates are required for calculation of the local mean variance
#'   estimator.  The default value is \code{NULL}.
#'
#' @param sizeweight Logical value that indicates whether size weights should be
#'   used during estimation, where \code{TRUE} uses size weights and
#'   \code{FALSE} does not use size weights. To employ size weights for a
#'   single-stage sample, a value must be supplied for argument weight.  To
#'   employ size weights for a two-stage sample, values must be supplied for
#'   arguments \code{weight} and \code{weight1}. The default value is \code{FALSE}.
#'
#' @param sweight Character value providing the name of the size weight variable
#'   in \code{dframe}.  For a two-stage sample, the size weight
#'   variable identifies stage two size weights.  The default value is
#'   \code{NULL}.
#'
#' @param sweight1 Character value providing name of the stage one size weight
#'   variable in \code{dframe}.  The default value is \code{NULL}.
#'
#' @param fpc Object that specifies values required for calculation of the
#'   finite population correction factor used during variance estimation. The
#'   object must match the survey design in terms of stratification and whether
#'   the design is single-stage or two-stage.  For an unstratified design, the
#'   object is a vector.  The vector is composed of a single numeric value for a
#'   single-stage design.  For a two-stage unstratified design, the object is a
#'   named vector containing one more than the number of clusters in the sample,
#'   where the first item in the vector specifies the number of clusters in the
#'   population and each subsequent item specifies the number of stage two units
#'   for the cluster.  The name for the first item in the vector is arbitrary.
#'   Subsequent names in the vector identify clusters and must match the cluster
#'   IDs.  For a stratified design, the object is a named list of vectors, where
#'   names must match the strata IDs.  For each stratum, the format of the
#'   vector is identical to the format described for unstratified single-stage
#'   and two-stage designs.  Note that the finite population correction factor
#'   is not used with the local mean variance estimator.
#'
#'   Example fpc for a single-stage unstratified survey design:
#'
#'   \verb{fpc <- 15000}
#'
#'   Example fpc for a single-stage stratified survey design:
#'
#'   \verb{fpc <- list(
#'     Stratum_1 = 9000,
#'     Stratum_2 = 6000)
#'    }
#'
#'   Example fpc for a two-stage unstratified survey design:
#'
#'   \verb{fpc <- c(
#'     Ncluster = 150,
#'     Cluster_1 = 150,
#'     Cluster_2 = 75,
#'     Cluster_3 = 75,
#'     Cluster_4 = 125,
#'     Cluster_5 = 75)
#'   }
#'
#'   Example fpc for a two-stage stratified survey design:
#'
#'   \verb{fpc <- list(
#'     Stratum_1 = c(
#'       Ncluster_1 = 100,
#'       Cluster_1 = 125,
#'       Cluster_2 = 100,
#'       Cluster_3 = 100,
#'       Cluster_4 = 125,
#'       Cluster_5 = 50),
#'     Stratum_2 = c(
#'       Ncluster_2 = 50,
#'       Cluster_1 = 75,
#'       Cluster_2 = 150,
#'       Cluster_3 = 75,
#'       Cluster_4 = 75,
#'       Cluster_5 = 125))
#'   }
#'
#' @param popsize Object that provides values for the population argument of the
#'   \code{calibrate} or \code{postStratify} functions in the survey package. If
#'   a value is provided for popsize, then either the \code{calibrate} or
#'   \code{postStratify} function is used to modify the survey design object
#'   that is required by functions in the survey package.  Whether to use the
#'   \code{calibrate} or \code{postStratify} function is dictated by the format
#'   of popsize, which is discussed below.  Post-stratification adjusts the
#'   sampling and replicate weights so that the joint distribution of a set of
#'   post-stratifying variables matches the known population joint distribution.
#'   Calibration, generalized raking, or GREG estimators generalize
#'   post-stratification and raking by calibrating a sample to the marginal
#'   totals of variables in a linear regression model. For the \code{calibrate}
#'   function, the object is a named list, where the names identify factor
#'   variables in \code{dframe}.  Each element of the list is a
#'   named vector containing the population total for each level of the
#'   associated factor variable.  For the \code{postStratify} function, the
#'   object is either a data frame, table, or xtabs object that provides the
#'   population total for all combinations of selected factor variables in the
#'   \code{dframe} data frame.  If a data frame is used for \code{popsize}, the
#'   variable containing population totals must be the last variable in the data
#'   frame.  If a table is used for \code{popsize}, the table must have named
#'   \code{dimnames} where the names identify factor variables in the
#'   \code{dframe} data frame.  If the popsize argument is equal to \code{NULL},
#'   then neither calibration nor post-stratification is performed.  The default
#'   value is \code{NULL}.
#'
#'   Example popsize for calibration:
#'
#'   \verb{popsize <- list(
#'     Ecoregion = c(
#'       East = 750,
#'       Central = 500,
#'       West = 250),
#'     Type = c(
#'       Streams = 1150,
#'       Rivers = 350))
#'   }
#'
#'   Example popsize for post-stratification using a data frame:
#'
#'   \verb{popsize <- data.frame(
#'     Ecoregion = rep(c("East", "Central", "West"),
#'       rep(2, 3)),
#'     Type = rep(c("Streams", "Rivers"), 3),
#'     Total = c(575, 175, 400, 100, 175, 75))
#'   }
#'
#'   Example popsize for post-stratification using a table:
#'
#'   \verb{popsize <- with(MySurveyFrame,
#'     table(Ecoregion, Type))}
#'
#'   Example popsize for post-stratification using an xtabs object:
#'
#'   \verb{popsize <- xtabs(~Ecoregion + Type,
#'     data = MySurveyFrame)}
#'
#' @param vartype Character value providing the choice of the variance
#'   estimator, where \code{"Local"} indicates the local mean estimator and
#'   \code{"SRS"} indicates the simple random sampling estimator.  The default
#'   value is \code{"Local"}.
#'
#' @param jointprob Character value providing the choice of joint inclusion
#'   probability approximation for use with Horvitz-Thompson and Yates-Grundy
#'   variance estimators, where \code{"overton"} indicates the Overton
#'   approximation, \code{"hr"} indicates the Hartley-Rao approximation, and
#'   \code{"brewer"} equals the Brewer approximation.  The default value is
#'   \code{"overton"}.
#'
#' @param conf Numeric value providing the Gaussian-based confidence level.  The default value
#'   is \code{95}.
#'
#' @param All_Sites A logical variable used when \code{subpops} is not
#'   \code{NULL}. If \code{All_Sites} is \code{TRUE}, then alongside the
#'   subpopulation output, output for all sites (ignoring subpopulations) is
#'   returned for each variable in \code{vars}. If \code{All_Sites} is
#'   \code{FALSE}, then alongside the subpopulation output, output for all sites
#'   (ignoring subpopulations) is not returned for each variable in \code{vars}.
#'   The default is \code{FALSE}.
#'
#' @return List of change estimates composed of four items:
#'   (1) \code{catsum} contains change estimates for categorical variables,
#'   (2) \code{contsum_mean} contains estimates for continuous variables using
#'   the mean, (3) \code{contsum_total} contains estimates for continuous
#'   variables using the total, and (4) \code{contsum_median} contains estimates for continuous
#'   variables using the median.  The items in the list will contain \code{NULL}
#'   for estimates that were not calculated.  Each data frame includes estimates
#'   for all combinations of population Types, subpopulations within types,
#'   response variables, and categories within each response variable (for
#'   categorical variables and continuous variables using the median).  Change
#'   estimates are provided plus standard error estimates and confidence
#'   interval estimates.
#'   
#'   The \code{catsum} data frame contains the following variables:
#'   \describe{
#'     \item{Survey_1}{first survey name}
#'     \item{Survey_2}{second survey name}
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{Category}{category of response variable}
#'     \item{DiffEst.P}{proportion difference estimate (in \%; second survey - first survey)}
#'     \item{StdError.P}{standard error of proportion difference estimate}
#'     \item{MarginofError.P}{margin of error of proportion difference estimate}
#'     \item{LCBxxPct.P}{xx\% (default 95\%) lower confidence bound of proportion difference estimate}
#'     \item{UCBxxPct.P}{xx\% (default 95\%) upper confidence bound of proportion difference estimate}
#'     \item{Estimate.U}{total difference estimate (second survey - first survey)}
#'     \item{StdError.U}{standard error of total difference estimate}
#'     \item{MarginofError.U}{margin of error of total difference estimate}
#'     \item{LCBxxPct.U}{xx\% (default 95\%) lower confidence bound of total difference estimate}
#'     \item{UCBxxPct.U}{xx\% (default 95\%) upper confidence bound of total difference estimate}
#'     \item{nResp_1}{sample size in the first survey}
#'     \item{Estimate.P_1}{proportion estimate (in \%) from the first survey}
#'     \item{StdError.P_1}{standard error of proportion estimate from the first survey}
#'     \item{MarginofError.P_1}{margin of error of proportion estimate from the first survey}
#'     \item{LCBxxPct.P_1}{xx\% (default 95\%) lower confidence bound of proportion estimate from the first survey}
#'     \item{UCBxxPct.P_1}{xx\% (default 95\%) upper confidence bound of proportion estimate from the first survey}
#'     \item{nResp_2}{sample size in the second survey}
#'     \item{Estimate.U_1}{total estimate from the first survey}
#'     \item{StdError.U_1}{standard error of total estimate from the first survey}
#'     \item{MarginofError.U_1}{margin of error of total estimate from the first survey}
#'     \item{LCBxxPct.U_1}{xx\% (default 95\%) lower confidence bound of total estimate from the first survey}
#'     \item{UCBxxPct.U_1}{xx\% (default 95\%) upper confidence bound of total estimate from the first survey}
#'     \item{Estimate.P_2}{proportion estimate (in \%) from the second survey}
#'     \item{StdError.P_2}{standard error of proportion estimate from the second survey}
#'     \item{MarginofError.P_2}{margin of error of proportion estimate from the second survey}
#'     \item{LCBxxPct.P_2}{xx\% (default 95\%) lower confidence bound of proportion estimate from the second survey}
#'     \item{UCBxxPct.P_2}{xx\% (default 95\%) upper confidence bound of proportion estimate from the second survey}
#'     \item{Estimate.U_2}{total estimate from the second survey}
#'     \item{StdError.U_2}{standard error of total estimate from the second survey}
#'     \item{MarginofError.U_2}{margin of error of total estimate from the second survey}
#'     \item{LCBxxPct.U_2}{xx\% (default 95\%) lower confidence bound of total estimate from the second survey}
#'     \item{UCBxxPct.U_2}{xx\% (default 95\%) upper confidence bound of total estimate from the second survey}
#'   }
#'   
#'   The \code{contsum_mean} data frame contains the following variables:
#'   \describe{
#'     \item{Survey_1}{first survey name}
#'     \item{Survey_2}{second survey name}
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{Statistic}{value of percentile}
#'     \item{nResp}{sample size at or below \code{Value}}
#'     \item{DiffEst}{mean difference estimate}
#'     \item{StdError}{standard error of mean difference estimate}
#'     \item{MarginofError}{margin of error of mean difference estimate}
#'     \item{LCBxxPct}{xx\% (default 95\%) lower confidence bound of mean difference estimate}
#'     \item{UCBxxPct}{xx\% (default 95\%) upper confidence bound of mean difference estimate}
#'     \item{nResp_1}{sample size in the first survey}
#'     \item{Estimate_1}{mean estimate from the first survey}
#'     \item{StdError_1}{standard error of mean estimate from the first survey}
#'     \item{MarginofError_1}{margin of error of mean estimate from the first survey}
#'     \item{LCBxxPct_1}{xx\% (default 95\%) lower confidence bound of mean estimate from the first survey}
#'     \item{UCBxxPct_1}{xx\% (default 95\%) upper confidence bound of mean estimate from the first survey}
#'     \item{nResp_2}{sample size in the second survey}
#'     \item{Estimate_2}{mean estimate from the second survey}
#'     \item{StdError_2}{standard error of mean estimate from the second survey}
#'     \item{MarginofError_2}{margin of error of mean estimate from the second survey}
#'     \item{LCBxxPct_2}{xx\% (default 95\%) lower confidence bound of mean estimate from the second survey}
#'     \item{UCBxxPct_2}{xx\% (default 95\%) upper confidence bound of mean estimate from the second survey}
#'   }
#'   
#'   The \code{contsum_total} data frame contains the following variables:
#'   \describe{
#'     \item{Survey_1}{first survey name}
#'     \item{Survey_2}{second survey name}
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{Statistic}{value of percentile}
#'     \item{nResp}{sample size at or below \code{Value}}
#'     \item{DiffEst}{total difference estimate}
#'     \item{StdError}{standard error of total difference estimate}
#'     \item{MarginofError}{margin of error of total difference estimate}
#'     \item{LCBxxPct}{xx\% (default 95\%) lower confidence bound of total difference estimate}
#'     \item{UCBxxPct}{xx\% (default 95\%) upper confidence bound of total difference estimate}
#'     \item{nResp_1}{sample size in the first survey}
#'     \item{Estimate_1}{total estimate from the first survey}
#'     \item{StdError_1}{standard error of total estimate from the first survey}
#'     \item{MarginofError_1}{margin of error of total estimate from the first survey}
#'     \item{LCBxxPct_1}{xx\% (default 95\%) lower confidence bound of total estimate from the first survey}
#'     \item{UCBxxPct_1}{xx\% (default 95\%) upper confidence bound of total estimate from the first survey}
#'     \item{nResp_2}{sample size in the second survey}
#'     \item{Estimate_2}{total estimate from the second survey}
#'     \item{StdError_2}{standard error of total estimate from the second survey}
#'     \item{MarginofError_2}{margin of error of total estimate from the second survey}
#'     \item{LCBxxPct_2}{xx\% (default 95\%) lower confidence bound of total estimate from the second survey}
#'     \item{UCBxxPct_2}{xx\% (default 95\%) upper confidence bound of total estimate from the second survey}
#'   }
#'   
#'   The \code{contsum_median} data frame contains the following variables:
#'   \describe{
#'     \item{Survey_1}{first survey name}
#'     \item{Survey_2}{second survey name}
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{Category}{category of response variable}
#'     \item{DiffEst.P}{proportion above or below median difference estimate (in \%; second survey - first survey)}
#'     \item{StdError.P}{standard error of proportion above or below median difference estimate}
#'     \item{MarginofError.P}{margin of error of proportion above or below median difference estimate}
#'     \item{LCBxxPct.P}{xx\% (default 95\%) lower confidence bound of proportion above or below median difference estimate}
#'     \item{UCBxxPct.P}{xx\% (default 95\%) upper confidence bound of proportion above or below median difference estimate}
#'     \item{Estimate.U}{total above or below median difference estimate (second survey - first survey)}
#'     \item{StdError.U}{standard error of total above or below median difference estimate}
#'     \item{MarginofError.U}{margin of error of total above or below median difference estimate}
#'     \item{LCBxxPct.U}{xx\% (default 95\%) lower confidence bound of total above or below median difference estimate}
#'     \item{UCBxxPct.U}{xx\% (default 95\%) upper confidence bound of total above or below median difference estimate}
#'     \item{nResp_1}{sample size in the first survey}
#'     \item{Estimate.P_1}{proportion above or below median estimate (in \%) from the first survey}
#'     \item{StdError.P_1}{standard error of proportion above or below median estimate from the first survey}
#'     \item{MarginofError.P_1}{margin of error of proportion above or below median estimate from the first survey}
#'     \item{LCBxxPct.P_1}{xx\% (default 95\%) lower confidence bound of proportion above or below median estimate from the first survey}
#'     \item{UCBxxPct.P_1}{xx\% (default 95\%) upper confidence bound of proportion above or below median estimate from the first survey}
#'     \item{nResp_2}{sample size in the second survey}
#'     \item{Estimate.U_1}{total above or below median estimate from the first survey}
#'     \item{StdError.U_1}{standard error of total above or below median estimate from the first survey}
#'     \item{MarginofError.U_1}{margin of error of total above or below median estimate from the first survey}
#'     \item{LCBxxPct.U_1}{xx\% (default 95\%) lower confidence bound of total above or below median estimate from the first survey}
#'     \item{UCBxxPct.U_1}{xx\% (default 95\%) upper confidence bound of total above or below median estimate from the first survey}
#'     \item{Estimate.P_2}{proportion above or below median estimate (in \%) from the second survey}
#'     \item{StdError.P_2}{standard error of proportion above or below median estimate from the second survey}
#'     \item{MarginofError.P_2}{margin of error of proportion above or below median estimate from the second survey}
#'     \item{LCBxxPct.P_2}{xx\% (default 95\%) lower confidence bound of proportion above or below median estimate from the second survey}
#'     \item{UCBxxPct.P_2}{xx\% (default 95\%) upper confidence bound of proportion above or below median estimate from the second survey}
#'     \item{Estimate.U_2}{total above or below median estimate from the second survey}
#'     \item{StdError.U_2}{standard error of total above or below median estimate from the second survey}
#'     \item{MarginofError.U_2}{margin of error of total above or below median estimate from the second survey}
#'     \item{LCBxxPct.U_2}{xx\% (default 95\%) lower confidence bound of total above or below median estimate from the second survey}
#'     \item{UCBxxPct.U_2}{xx\% (default 95\%) upper confidence bound of total above or below median estimate from the second survey}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @seealso
#'   \describe{
#'   \item{\code{\link{trend_analysis}}}{ for trend analysis}
#'   }
#'
#' @examples
#' # Categorical variable example for three resource classes
#' dframe <- data.frame(
#'   surveyID = rep(c("Survey 1", "Survey 2"), c(100, 100)),
#'   siteID = paste0("Site", 1:200),
#'   wgt = runif(200, 10, 100),
#'   xcoord = runif(200),
#'   ycoord = runif(200),
#'   stratum = rep(rep(c("Stratum 1", "Stratum 2"), c(2, 2)), 50),
#'   CatVar = rep(c("North", "South"), 100),
#'   All_Sites = rep("All Sites", 200),
#'   Resource_Class = sample(c("Good", "Fair", "Poor"), 200, replace = TRUE)
#' )
#' myvars <- c("CatVar")
#' mysubpops <- c("All_Sites", "Resource_Class")
#' change_analysis(dframe,
#'   vars_cat = myvars, subpops = mysubpops,
#'   surveyID = "surveyID", siteID = "siteID", weight = "wgt",
#'   xcoord = "xcoord", ycoord = "ycoord", stratumID = "stratum"
#' )
#' @export
################################################################################

change_analysis <- function(dframe, vars_cat = NULL, vars_cont = NULL, test = "mean", subpops = NULL,
                            surveyID = "surveyID", survey_names = NULL, siteID = "siteID",
                            weight = "weight", revisitwgt = FALSE, xcoord = NULL, ycoord = NULL,
                            stratumID = NULL, clusterID = NULL, weight1 = NULL, xcoord1 = NULL,
                            ycoord1 = NULL, sizeweight = FALSE, sweight = NULL, sweight1 = NULL,
                            fpc = NULL, popsize = NULL, vartype = "Local", jointprob = "overton",
                            conf = 95, All_Sites = FALSE) {

  # Assign NULL to vars_nondetect

  vars_nondetect <- NULL

  # Create a vector for error messages

  error_ind <- FALSE
  error_vec <- NULL

  # Create a data frame for warning messages

  warn_ind <- FALSE
  warn_df <- NULL
  fname <- "change_analysis"

  # Ensure that the dframe argument was provided

  if (missing(dframe) | is.null(dframe)) {
    stop("\nThe dframe argument must be provided.\n")
  }

  # If the dframe argument is an sf object, extract coordinates from the geometry
  # column, assign values "xcoord" and "ycoord" to arguments xcoord and ycoord,
  # respectively, and drop the geometry column from the object

  if ("sf" %in% class(dframe)) {
    temp <- st_coordinates(dframe)
    xcoord <- "xcoord"
    dframe$xcoord <- temp[, "X"]
    ycoord <- "ycoord"
    dframe$ycoord <- temp[, "Y"]
    dframe <- st_set_geometry(dframe, NULL)
  }

  # If the dframe argument is a tibble or does not belong to class
  # "data.frame", coerce the argument to class "data.frame"

  if ("tbl_df" %in% class(dframe) | !("data.frame" %in% class(dframe))) {
    dframe <- as.data.frame(dframe)
  }

  # Ensure that the dframe argument does not contain zero rows

  if (nrow(dframe) == 0) {
    stop("\nThe dframe argument contains zero rows.\n")
  }

  # Ensure that unused levels are dropped from factor variables in the dframe
  # data frame

  dframe <- droplevels(dframe)

  # Ensure that the dframe data frame contains the survey ID variable

  ind1 <- FALSE
  if (!(surveyID %in% names(dframe))) {
    error_ind <- TRUE
    msg <- paste0("The name provided for the surveyID argument, \"", surveyID, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  } else {

    # Ensure that the survey names variable is a vector that has two unique values
    # and that those values match the categories used in the survey ID variable in
    # the dframe data frame

    if (is.null(survey_names)) {
      survey_names <- as.vector(unique(dframe[, surveyID]))
    }
    if (!is.vector(survey_names)) {
      error_ind <- TRUE
      msg <- "The survey_names argument must be a vector.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (is.list(survey_names)) {
        survey_names <- unlist(survey_names)
      }
      if (length(survey_names) != 2) {
        error_ind <- TRUE
        msg <- "The survey names variable should have two unique values.\n"
        error_vec <- c(error_vec, msg)
      } else {
        survey_names <- as.character(survey_names)
        temp <- unique(dframe[, surveyID])
        if (length(temp) != 2) {
          error_ind <- TRUE
          msg <- "The survey ID variable should have two categories.\n"
          error_vec <- c(error_vec, msg)
        } else {
          if (!all(survey_names %in% temp)) {
            error_ind <- TRUE
            msg <- "Values for the survey names variable do not match categories of the survey ID variable.\n"
            error_vec <- c(error_vec, msg)
          } else {
            dframe[, surveyID] <- factor(dframe[, surveyID], levels = survey_names)
            ind1 <- TRUE
          }
        }
      }
    }
  }

  # Create an indicator variable for survey one

  if (ind1) {
    survey_1 <- dframe[, surveyID] %in% survey_names[1]
  }

  # Create an indicator variable for survey two

  if (ind1) {
    survey_2 <- dframe[, surveyID] %in% survey_names[2]
  }

  # Ensure that the dframe data frame contains the site ID variable

  if (!(siteID %in% names(dframe))) {
    ind2 <- FALSE
    error_ind <- TRUE
    msg <- paste0("The name provided for the siteID argument, \"", siteID, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  } else {
    ind2 <- TRUE
  }

  # For each survey, check site IDs for repeat values and, as necessary, create
  # unique site IDs and output a warning message

  if (ind1 & ind2) {
    IDs <- dframe[, siteID]
    for (i in 1:2) {
      eval(parse(text = paste0("tst <- survey_", i)))
      temp <- sapply(split(IDs[tst], IDs[tst]), length)
      if (any(temp > 1)) {
        warn_ind <- TRUE
        temp.str <- vecprint(names(temp)[temp > 1])
        warn <- paste("The following site ID values occur more than once among the survey", i, "values \nthat were input to the function:\n", temp.str)
        act <- "Unique site ID values were created.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = NA,
          subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
        ))
        dframe[, siteID][tst] <- uniqueID(dframe[, siteID][tst])
      }
    }
  }

  # Determine whether the surveys include repeat visit sites and create indicator
  # variables for repeat visit sites in each survey

  repeat_1 <- logical(nrow(dframe))
  repeat_2 <- logical(nrow(dframe))
  if (ind1 & ind2) {
    repeat_1[survey_1] <- dframe[survey_1, siteID] %in% dframe[
      survey_2,
      siteID
    ]
    repeat_2[survey_2] <- dframe[survey_2, siteID] %in% dframe[
      survey_1,
      siteID
    ]
  }

  # Ensure that repeat visit sites for both surveys occur in the same order among
  # the rows of the dframe data frame

  if (ind1 & ind2) {
    if (any(repeat_1)) {
      indx <- match(dframe[repeat_1, siteID], dframe[repeat_2, siteID])
      dframe[repeat_2, ] <- dframe[repeat_2, ][indx, ]
    }
  }

  # Ensure that the dframe data frame contains the survey weight variable

  if (!(weight %in% names(dframe))) {
    error_ind <- TRUE
    msg <- paste0("The name provided for the weight argument, \"", weight, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  }

  # Assign names to the variables required for calculation of the finite
  # population correction factor

  if (is.null(fpc)) {
    fpcfactor_ind <- FALSE
    fpcsize <- NULL
    Ncluster <- NULL
    stage1size <- NULL
  } else {
    fpcfactor_ind <- TRUE
    if (is.null(clusterID)) {
      fpcsize <- "fpcsize"
      Ncluster <- NULL
      stage1size <- NULL
    } else {
      fpcsize <- NULL
      Ncluster <- "Ncluster"
      stage1size <- "stage1size"
    }
  }

  # Create a list containing names of survey design variables

  design_names <- list(
    siteID = siteID,
    weight = weight,
    xcoord = xcoord,
    ycoord = ycoord,
    stratumID = stratumID,
    clusterID = clusterID,
    weight1 = weight1,
    xcoord1 = xcoord1,
    ycoord1 = ycoord1,
    sweight = sweight,
    sweight1 = sweight1,
    fpcsize = fpcsize,
    Ncluster = Ncluster,
    stage1size = stage1size
  )

  # Ensure that a value was provided for at least one of the vars_cat (categorical
  # response variable names) or vars_cont (continuous response variable names)
  # arguments

  if (is.null(vars_cat) && is.null(vars_cont)) {
    error_ind <- TRUE
    msg <- "A value must be provided for at least one of the vars_cat (categorical \nresponse variable names) or vars_cont (continuous response variable names) \narguments.\n"
    error_vec <- c(error_vec, msg)
  }

  # For each categorical variable, check whether all values for either survey
  # are missing

  if (!is.null(vars_cat) && ind1 && ind2) {
    temp <- NULL
    for (v in vars_cat) {
      if (all(is.na(dframe[survey_1, v])) | all(is.na(dframe[survey_2, v]))) {
        temp <- c(temp, v)
      }
    }
    if (!is.null(temp)) {
      error_ind <- TRUE
      temp.str <- vecprint(temp)
      msg <- paste("For the following categorical variables, at least one of the surveys contains only \nmissing values:\n", temp.str)
      error_vec <- c(error_vec, msg)
    }
  }

  # For each continuous variable, check whether all values for either survey
  # are missing

  if (!is.null(vars_cont) && ind1 && ind2) {
    temp <- NULL
    for (v in vars_cont) {
      if (all(is.na(dframe[survey_1, v])) | all(is.na(dframe[survey_2, v]))) {
        temp <- c(temp, v)
      }
    }
    if (!is.null(temp)) {
      error_ind <- TRUE
      temp.str <- vecprint(temp)
      msg <- paste("For the following continuous variables, at least one of the surveys contains only \nmissing values:\n", temp.str)
      error_vec <- c(error_vec, msg)
    }
  }

  # If a value was not provided for the subpops (subpopulation names) argument,
  # assign the value "All_Sites" to the subpops argument and create a factor
  # named "All_Sites" in dframe that takes the value "All Sites"

  if (is.null(subpops)) {
    subpops <- "All_Sites"
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # If the user wants information for all sites together in addition to the
  # subpops, add the value "All_Sites" to the subpops argument and create a
  # factor named "All_Sites" in dframe that takes the value
  # "All Sites"

  if (!is.null(subpops) && All_Sites) {
    subpops <- c(subpops, "All_Sites")
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # For each subpopulation variable, check whether all values for either survey
  # are missing

  if (ind1 && ind2) {
    temp <- NULL
    for (v in subpops) {
      if (all(is.na(dframe[survey_1, v])) | all(is.na(dframe[survey_2, v]))) {
        temp <- c(temp, v)
      }
    }
    if (!is.null(temp)) {
      error_ind <- TRUE
      temp.str <- vecprint(temp)
      msg <- paste("For the following subpopulation variables, at least one of the surveys contains only \nmissing values:\n", temp.str)
      error_vec <- c(error_vec, msg)
    } else {

      # For each subpopulation variable, remove levels that contain only missing
      #  values for one of the surveys

      temp <- NULL
      for (v in subpops) {
        for (lv in levels(dframe[, v])) {
          tst1 <- dframe[survey_1, v] == lv
          tst2 <- dframe[survey_2, v] == lv
          if (all(is.na(dframe[survey_1, v][tst1])) |
            all(is.na(dframe[survey_2, v][tst2]))) {
            temp <- c(temp, lv)
          }
        }
        if (!is.null(temp)) {
          warn_ind <- TRUE
          temp.str <- vecprint(temp)
          warn <- paste0("For subpopulation variable \"", v, "\", one of the surveys contains only missing values \nfor the following categories:\n", temp.str)
          act <- "The subpopulation categories were eliminated from the analysis.\n"
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = v, subpop = NA, indicator = NA,
            stratum = NA, warning = I(warn), action = I(act)
          ))
          dframe[, v] <- as.character(dframe[, v])
          for (lv in temp) {
            tst <- dframe[, v] == lv
            dframe[tst, v] <- NA
          }
        }
      }
    }
  }

  # Check input arguments
  temp <- input_check(dframe, design_names, vars_cat, vars_cont, NULL, NULL,
    subpops, sizeweight, fpc, popsize, vartype, jointprob, conf,
    error_ind = error_ind, error_vec = error_vec
  )
  dframe <- temp$dframe
  vars_cat <- temp$vars_cat
  vars_cont <- temp$vars_cont
  subpops <- temp$subpops
  popsize <- temp$popsize
  vartype <- temp$vartype
  jointprob <- temp$jointprob
  error_ind <- temp$error_ind
  error_vec <- temp$error_vec

  # As necessary, output a message indicating that error messages were generated
  # during execution of the program

  if (error_ind) {
    error_vec <<- error_vec
    if (length(error_vec) == 1) {
      message("During execution of the program, an error message was generated.  The error \nmessage is stored in a vector named 'error_vec'.  Enter the following command \nto view the error message: errorprnt()\n")
    } else {
      message(paste("During execution of the program,", length(error_vec), "error messages were generated.  The error \nmessages are stored in a vector named 'error_vec'.  Enter the following \ncommand to view the error messages: errorprnt()\n"))
    }

    if (warn_ind) {
      warn_df <<- warn_df
      if (nrow(warn_df) == 1) {
        message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
      } else {
        message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
      }
    }
    stop("See the preceding message(s).")
  }

  # Assign a logical value to the indicator variable for a stratified sample

  stratum_ind <- !is.null(stratumID)

  # For a stratified sample, remove strata that contain a single site

  if (stratum_ind) {
    dframe[, stratumID] <- factor(dframe[, stratumID])
    stratum_levels <- levels(dframe[, stratumID])
    nstrata <- length(stratum_levels)
    ind <- FALSE
    for (i in 1:nstrata) {
      tst <- dframe[, stratumID] == stratum_levels[i]
      if (sum(tst) == 1) {
        warn_ind <- TRUE
        warn <- paste0("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
        act <- "Stratum was removed from the analysis.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = NA,
          subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
        ))
        dframe <- dframe[!tst, ]
        ind <- TRUE
      }
    }
    if (ind) {
      dframe[, stratumID] <- factor(dframe[, stratumID])
      stratum_levels <- levels(dframe[, stratumID])
      nstrata <- length(stratum_levels)
    }
  }

  # Assign a logical value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Create the survey design object for each survey
  ## give factors two levels if they only have one level
  for (x in vars_cat) {
    catvar_levels <- levels(dframe[[x]])
    if (length(catvar_levels) == 1) {
      levels(dframe[[x]]) <- c(catvar_levels, paste0("Not ", catvar_levels))
    }
  }
  design <- survey_design(
    dframe, siteID, weight, stratum_ind, stratumID, cluster_ind, clusterID,
    weight1, sizeweight, sweight, sweight1, fpcfactor_ind, fpcsize, Ncluster,
    stage1size, vartype, jointprob
  )
  design_1 <- subset(design, survey_1)
  design_2 <- subset(design, survey_2)

  # If popsize is not equal to NULL, then call either the postStratify or
  # calibrate function, as appropriate

  if (!is.null(popsize)) {
    if (all(class(popsize) %in% c("data.frame", "table", "xtabs"))) {
      if ("data.frame" %in% class(popsize)) {
        pnames <- names(popsize)[-ncol(popsize)]
      } else {
        pnames <- names(dimnames(popsize))
      }
      design_1 <- postStratify(design_1, make.formula(pnames), popsize)
      design_2 <- postStratify(design_2, make.formula(pnames), popsize)
    } else {
      cnames <- cal_names(make.formula(names(popsize)), design)
      pop_totals <- numeric(length(cnames))
      names(pop_totals) <- cnames
      pop_totals[1] <- sum(popsize[[1]])
      k <- 2
      for (i in names(popsize)) {
        temp <- popsize[[i]]
        for (j in 2:length(temp)) {
          pop_totals[k] <- temp[j]
          k <- k + 1
        }
      }
      design_1 <- calibrate(design_1, make.formula(names(popsize)), pop_totals)
      design_2 <- calibrate(design_2, make.formula(names(popsize)), pop_totals)
    }
  }

  # If popsize is not equal to NULL and vartype equals "Local", then assign
  # adjusted weights to the appropriate weight variable(s) in the design data
  # frames

  if (!is.null(popsize) && vartype == "Local") {
    if (cluster_ind) {
      design_1$variables$wgt2 <- weights(design_1) / design_1$variables$wgt1
      design_2$variables$wgt2 <- weights(design_2) / design_1$variables$wgt1
    } else {
      design_1$variables$wgt <- weights(design_1)
      design_2$variables$wgt <- weights(design_2)
    }
  }

  # Assign the confidence bound multiplier

  mult <- qnorm(0.5 + (conf / 100) / 2)

  # Create the output object

  changesum <- list(catsum = NULL, contsum_mean = NULL, contsum_total = NULL, contsum_median = NULL)

  #
  # Begin the section for categorical response variables
  #

  if (!is.null(vars_cat)) {

    # Loop through all subpopulations (domains)

    for (itype in subpops) {
      lev_itype <- levels(dframe[, itype])

      # Loop through all response variables

      for (ivar in vars_cat) {
        lev_ivar <- levels(dframe[, ivar])
        nlev_ivar <- length(lev_ivar)

        # Loop through all levels of a subpopulation

        for (isubpop in lev_itype) {

          # Calculate change estimates, standard error estimates, and confidence
          # bound estimates for each combination of subpopulation, response
          # variable, and level of the subpopulation

          temp <- change_est(
            resp_ind = "cat", survey_names, changesum,
            dframe, survey_1, survey_2, itype, isubpop, ivar, lev_ivar,
            nlev_ivar, design_1, design_2, design_names, repeat_1[survey_1],
            repeat_2[survey_2], siteID, revisitwgt, NULL, NULL, vartype, conf,
            mult, warn_ind, warn_df, warn_vec = c(itype, isubpop, ivar)
          )
          changesum <- temp$changesum
          warn_ind <- temp$warn_ind
          warn_df <- temp$warn_df

          # End of the loop for subpopulations
        }

        # End of the loop for response variables
      }

      # End of the loop for type of population
    }

    # End of the section for categorical response variables
  }

  #
  # Begin the section for continuous response variables
  #

  if (!is.null(vars_cont)) {

    # Loop through all subpopulations

    for (itype in subpops) {
      lev_itype <- levels(dframe[, itype])

      # Loop through all response variables

      for (ivar in vars_cont) {

        # Loop through all levels of a subpopulation

        for (isubpop in lev_itype) {

          # Calculate change estimates, standard error estimates, and confidence
          # bound estimates for each combination of subpopulation, response
          # variable, and level of the subpopulation

          indx <- match(ivar, vars_cont)
          temp <- change_est(
            resp_ind = "cont", survey_names, changesum,
            dframe, survey_1, survey_2, itype, isubpop, ivar, NULL, NULL,
            design_1, design_2, design_names, repeat_1[survey_1],
            repeat_2[survey_2], siteID, revisitwgt, test, vars_nondetect[indx],
            vartype, conf, mult, warn_ind, warn_df,
            warn_vec = c(itype, isubpop, ivar)
          )
          changesum <- temp$changesum
          warn_ind <- temp$warn_ind
          warn_df <- temp$warn_df
          changesum <- temp$changesum
          warn_ind <- temp$warn_ind
          warn_df <- temp$warn_df

          # End of the loop for subpopulations
        }

        # End of the loop for response variables
      }

      # End of the loop for type of population
    }

    # End of the section for continuous response variables
  }

  # As necessary, output a message indicating that warning messages were generated
  # during execution of the program

  if (warn_ind) {
    warn_df <<- warn_df
    if (nrow(warn_df) == 1) {
      message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
    } else {
      message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
    }
  }

  # As necessary, assign dimension names to the catsum, contsum_mean, and
  # contsum_median data frames

  if (!is.null(changesum$catsum)) {
    dimnames(changesum$catsum) <- list(1:nrow(changesum$catsum), c(
      "Survey_1",
      "Survey_2", "Type", "Subpopulation", "Indicator", "Category", "DiffEst.P",
      "StdError.P", "MarginofError.P", paste0("LCB", conf, "Pct.P"),
      paste0("UCB", conf, "Pct.P"), "DiffEst.U", "StdError.U",
      "MarginofError.U", paste0("LCB", conf, "Pct.U"),
      paste0("UCB", conf, "Pct.U"), "nResp_1", "Estimate.P_1", "StdError.P_1",
      "MarginofError.P_1", paste0("LCB", conf, "Pct.P_1"),
      paste0("UCB", conf, "Pct.P_1"), "Estimate.U_1", "StdError.U_1",
      "MarginofError.U_1", paste0("LCB", conf, "Pct.U_1"),
      paste0("UCB", conf, "Pct.U_1"), "nResp_2", "Estimate.P_2", "StdError.P_2",
      "MarginofError.P_2", paste0("LCB", conf, "Pct.P_2"),
      paste0("UCB", conf, "Pct.P_2"), "Estimate.U_2", "StdError.U_2",
      "MarginofError.U_2", paste0("LCB", conf, "Pct.U_2"),
      paste0("UCB", conf, "Pct.U_2")
    ))
  }

  if (!is.null(changesum$contsum_mean)) {
    dimnames(changesum$contsum_mean) <- list(
      1:nrow(changesum$contsum_mean),
      c(
        "Survey_1", "Survey_2", "Type", "Subpopulation", "Indicator",
        "DiffEst", "StdError", "MarginofError", paste0("LCB", conf, "Pct"),
        paste0("UCB", conf, "Pct"), "nResp_1", "Estimate_1", "StdError_1",
        "MarginofError_1", paste0("LCB", conf, "Pct_1"),
        paste0("UCB", conf, "Pct_1"), "nResp_2", "Estimate_2", "StdError_2",
        "MarginofError_2", paste0("LCB", conf, "Pct_2"),
        paste0("UCB", conf, "Pct_2")
      )
    )
  }
  
  if (!is.null(changesum$contsum_total)) {
    dimnames(changesum$contsum_total) <- list(
      1:nrow(changesum$contsum_total),
      c(
        "Survey_1", "Survey_2", "Type", "Subpopulation", "Indicator",
        "DiffEst", "StdError", "MarginofError", paste0("LCB", conf, "Pct"),
        paste0("UCB", conf, "Pct"), "nResp_1", "Estimate_1", "StdError_1",
        "MarginofError_1", paste0("LCB", conf, "Pct_1"),
        paste0("UCB", conf, "Pct_1"), "nResp_2", "Estimate_2", "StdError_2",
        "MarginofError_2", paste0("LCB", conf, "Pct_2"),
        paste0("UCB", conf, "Pct_2")
      )
    )
  }

  if (!is.null(changesum$contsum_median)) {
    dimnames(changesum$contsum_median) <- list(
      1:nrow(changesum$contsum_median),
      c(
        "Survey_1", "Survey_2", "Type", "Subpopulation", "Indicator",
        "Category", "DiffEst.P", "StdError.P", "MarginofError.P",
        paste0("LCB", conf, "Pct.P"), paste0("UCB", conf, "Pct.P"), "DiffEst.U",
        "StdError.U", "MarginofError.U", paste0("LCB", conf, "Pct.U"),
        paste0("UCB", conf, "Pct.U"), "nResp_1", "Estimate.P_1", "StdError.P_1",
        "MarginofError.P_1", paste0("LCB", conf, "Pct.P_1"),
        paste0("UCB", conf, "Pct.P_1"), "Estimate.U_1", "StdError.U_1",
        "MarginofError.U_1", paste0("LCB", conf, "Pct.U_1"),
        paste0("UCB", conf, "Pct.U_1"), "nResp_2", "Estimate.P_2",
        "StdError.P_2", "MarginofError.P_2", paste0("LCB", conf, "Pct.P_2"),
        paste0("UCB", conf, "Pct.P_2"), "Estimate.U_2", "StdError.U_2",
        "MarginofError.U_2", paste0("LCB", conf, "Pct.U_2"),
        paste0("UCB", conf, "Pct.U_2")
      )
    )
  }

  # Return the output object

  changesum
}

#' Combine rows from GRTS or IRS samples.
#'
#' This function row binds the \code{sites_legacy}, \code{sites_base},
#' \code{sites_over}, and \code{sites_near} objects from a GRTS or IRS sample
#' into a single \code{sf} object. This function is most useful when a single
#' \code{sf} object that contains all design sites is desired
#' (e.g. writing out a single shapefile using \code{sf::write_sf()}).
#'
#' @param object The design sites (output from \code{grts()} or \code{irs()}).
#'
#' @param siteuse A character vector of site types to return. Can contain
#' \code{"Legacy"} (for legacy sites), \code{"Base"} (for base sites),
#' \code{"Over"} (for \code{n_over} replacement sites), and \code{"Near"}
#' (for \code{n_near} replacement sites). The default is \code{NULL}, which
#' returns all non-\code{NULL} output from \code{object$sites_legacy},
#' \code{object$sites_base}, \code{object$sites_over}, and \code{object$sites_near}.
#'
#' @return A single \code{sf} object containing all requested design sites.
#'
#' @author Michael Dumelle \email{Dumelle.Michael@@epa.gov}
#'
#' @export
#'
#' @examples
#' \dontrun{
#' sample <- grts(NE_Lakes, 50, n_over = 10)
#' sample <- sp_rbind(sample)
#' write_sf(sample, "mypath/sample.shp")
#' }
sp_rbind <- function(object, siteuse = NULL) {
  if (!inherits(object, "sp_design")) {
    stop("object must be output from grts() or irs()")
  } else {
    if (is.null(siteuse)) {
      siteuse <- c("Legacy", "Base", "Over", "Near")
    }

    # legacy
    if ("Legacy" %in% siteuse) {
      new_legacy <- object$sites_legacy
    } else {
      new_legacy <- NULL
    }

    if ("Base" %in% siteuse) {
      new_base <- object$sites_base
    } else {
      new_base <- NULL
    }

    if ("Over" %in% siteuse) {
      new_over <- object$sites_over
    } else {
      new_over <- NULL
    }

    if ("Near" %in% siteuse) {
      new_near <- object$sites_near
    } else {
      new_near <- NULL
    }

    new_non_legacy <- rbind(new_base, new_over, new_near)

    if (is.null(new_non_legacy) & is.null(new_legacy)) {
      stop("siteuse cannot be empty")
    }

    if (is.null(new_legacy)) {
      return(new_non_legacy)
    } else {
      if (is.null(new_non_legacy)) {
        return(new_legacy)
      } else {
        new_legacy[setdiff(names(new_non_legacy), names(new_legacy))] <- NA
        new_non_legacy[setdiff(names(new_legacy), names(new_non_legacy))] <- NA
        return(rbind(new_legacy, new_non_legacy))
      }
    }
  }
}

################################################################################
# Function: cdftestvar_prop (not exported)
# Programmer: Tom Kincaid
# Date: October 23, 2020
# Revised: November 2, 2020 to correctly process the column variable when it
#          includes missing (NA) values
# Revised: April 28, 2021 to use the SRS estimator when the local mean estimator
#          fails to produce a valid estimate
# Revised: May 3 2021 to correct an error that occurs when warning messages for
#          unstratified samples are added to the warn_df data frame
# Revised: June 8 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
#
#' Local Mean Variance/Covariance Estimates of Estimated Population Proportions
#'
#' This function calculates the local mean estimate of the variance/covariance
#' matrix of the population proportions for a contingency table.  The function
#' can accomodate single-stage and two-stage samples.
#'
#' @param design Object of class \code{survey.design} that specifies a complex
#'   survey design.
#'
#' @param wgt Vector of the final adjusted weight (reciprocal of the sample
#'   inclusion probability) for each site, which is either the weight for a
#'   single-stage sample or the stage two weight for a two-stage sample.
#'
#' @param x Vector of x-coordinate for location for each site, which is either
#'   the x- coordinate for a single-stage sample or the stage two x-coordinate
#'   for a two-stage sample.
#'
#' @param y Vector of y-coordinate for location for each site, which is either
#'   the y- coordinate for a single-stage sample or the stage two y-coordinate
#'   for a two-stage sample.
#'
#' @param stratum_ind Logical value that indicates whether the sample is
#'   stratified, where \code{TRUE} = a stratified sample and \code{FALSE} = not
#'   a stratified sample.
#'
#' @param stratum_level Vector indicating the stratum level.
#'
#' @param cluster_ind Logical value that indicates whether the sample is a
#'   two- stage sample, where \code{TRUE} = a two-stage sample and
#'   \code{FALSE} = not a two-stage sample.
#'
#' @param clusterID Vector of the stage one sampling unit (primary sampling unit
#'   or cluster) code for each site.
#'
#' @param wgt1 Vector of the final adjusted stage one weight for each site.
#'
#' @param x1 Vector of the stage one x-coordinate for location for each site.
#'
#' @param y1 Vector of the stage one y-coordinate for location for each site.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated, where \code{TRUE} = warning messages were generated and
#'   \code{FALSE} = warning messages were not generated.
#'
#' @param warn_df A data frame for storing warning messages.
#'
#' @param warn_vec Character vector that contains a subpopulation name, the
#'   first subpopulation level, the second subpopulation level, and an
#'   indicator name.
#'
#' @return A list containing the following objects:
#'   \itemize{
#'     \item{\code{varest}}{matrix containing the variance/covariance estimates
#'       for the contingency table total estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

cdftestvar_prop <- function(design, wgt, x, y, stratum_ind, stratum_level,
                            cluster_ind, clusterID, wgt1, x1, y1, warn_ind,
                            warn_df, warn_vec) {

  # Assign the function name

  fname <- "cdftestvar_prop"

  # Assign the variance type

  vartype <- "Local"

  # Create the model matrix for the contingency table using all of the data

  frm_cells <- eval(bquote(~ interaction(
    factor(.(as.name("rowvar"))),
    factor(.(as.name("colvar")))
  ) - 1))
  mm_cells <- model.matrix(frm_cells, model.frame(frm_cells, design$variables,
    na.action = na.pass
  ))

  # Calculate contingency table means

  means <- svymean(mm_cells, design, na.rm = TRUE)
  names_means <- names(means)
  m <- length(names_means)

  #
  # Branch to handle two-stage and single-stage samples
  #

  if (cluster_ind) {

    # Begin the section for a two-stage sample

    # Calculate additional required values

    cluster <- factor(clusterID)
    cluster_levels <- levels(cluster)
    ncluster <- length(cluster_levels)
    z_lst <- split(design$variables$colvar, cluster)
    wgt2_lst <- split(wgt, cluster)
    wgt1_u <- as.vector(tapply(wgt1, cluster, unique))
    tw2 <- (sum(wgt1 * wgt))^2
    x2_lst <- split(x, cluster)
    y2_lst <- split(y, cluster)
    x1_u <- as.vector(tapply(x1, cluster, unique))
    y1_u <- as.vector(tapply(y1, cluster, unique))
    var_ind <- sapply(split(cluster, cluster), length) > 1

    # For each stage one sampling unit and each contingency table cell,
    # calculate an estimate of the total of the stage two sampling unit
    # residuals; then calculate the variance/covariance matrix of the totals

    total2est <- matrix(0, ncluster, m)
    var2est <- matrix(0, ncluster, m^2)
    for (i in 1:ncluster) {

      # Create the model matrix for the contingency table using using a single
      # cluster

      design_var <- subset(design$variables, cluster == cluster_levels[i] &
        !(is.na(rowvar) | is.na(colvar)))
      mm_cluster <- model.matrix(frm_cells, model.frame(frm_cells, design_var,
        na.action = na.pass
      ))

      # Calculate the weighted residuals matrix

      n <- length(z_lst[[i]])
      m_cl <- ncol(mm_cluster)
      tst <- names_means %in% colnames(mm_cluster)
      rm <- (mm_cluster - matrix(rep(means[tst], n), nrow = n, byrow = TRUE)) *
        matrix(rep(wgt2_lst[[i]], m_cl), nrow = n)

      # Calculate total estimates for the stage one sampling unit

      total2est[i, tst] <- apply(rm, 2, sum)

      # Adjust the variance/covariance estimator for small sample size

      if (var_ind[i] && n < 4) {
        warn_ind <- TRUE
        act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("There are less than four response values for stage one sampling unit \"", cluster_levels[i], "\"\nin stratum \"", stratum_level, "\", the simple random sampling covariance estimator for an \ninfinite population was used to calculate covariance of the total of the residuals.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("There are less than four response values for stage one sampling unit \"", cluster_levels[i], "\", \nthe simple random sampling covariance estimator for an infinite population was used \nto calculate covariance of the total of the residuals.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        vartype <- "SRS"
      }

      # Calculate variance/covariance estimates for the stage one sampling unit

      if (var_ind[i]) {
        tst <- rep(tst, m)
        if (vartype == "Local") {
          weight_lst <- localmean_weight(
            x2_lst[[i]], y2_lst[[i]],
            1 / wgt2_lst[[i]]
          )
          if (is.null(weight_lst)) {
            warn_ind <- TRUE
            act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
            if (stratum_ind) {
              warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stage one \nsampling unit \"", cluster_levels[i], "\" in stratum \"", stratum_level, "\", the simple random sampling \ncovariance estimator for an infinite population was used to calculate covariance of the \ntotal of the residuals.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = stratum_level,
                warning = I(warn), action = I(act)
              ))
            } else {
              warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stage one \nsampling unit \"", cluster_levels[i], "\", the simple random sampling covariance estimator for an \ninfinite population was used to calculate covariance of the total of the residuals.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            }
            var2est[i, tst] <- as.vector(n * var(rm))
          } else {
            temp <- localmean_cov(rm, weight_lst)
            var2est[i, ] <- as.vector(temp)
            if (any(diag(temp) < 0)) {
              warn_ind <- TRUE
              act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
              if (stratum_ind) {
                warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates for \nstage one sampling unit \"", cluster_levels[i], "\" in stratum \"", stratum_level, "\", the simple random \nsampling covariance estimator for an infinite population was used to calculate \ncovariance of the total of the residuals.\n")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1],
                  subpop = warn_vec[2], indicator = warn_vec[3],
                  stratum = stratum_level, warning = I(warn), action = I(act)
                ))
              } else {
                warn <- paste0("The local mean covariance estimator produced one or more  negative variance estimates \nfor stage one sampling unit \"", cluster_levels[i], "\", the simple random sampling covariance \nestimator for an infinite population was used to calculate covariance of the total of \nthe residuals.\n")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1],
                  subpop = warn_vec[2], indicator = warn_vec[3], stratum = NA,
                  warning = I(warn), action = I(act)
                ))
              }
              var2est[i, tst] <- as.vector(n * var(rm))
            }
          }
        } else {
          var2est[i, tst] <- as.vector(n * var(rm))
          vartype <- "Local"
        }
      }
    }

    # Adjust the variance/covariance estimator for small sample size

    if (ncluster < 4) {
      warn_ind <- TRUE
      act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
      if (stratum_ind) {
        warn <- paste0("There are less than four stage one sampling units in stratum \"", stratum_level, "\", the simple \nrandom sampling covariance estimator for an infinite population was used to calculate \ncovariance of the contingency table proportion estimates.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
          stratum = I(stratum_level), warning = I(warn), action = I(act)
        ))
      } else {
        warn <- paste0("There are less than four stage one sampling units, the simple random sampling covariance\nestimator for an infinite population was used to calculate covariance of the contingency \ntable proportion estimates.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
          stratum = NA, warning = I(warn), action = I(act)
        ))
      }
      vartype <- "SRS"
    }

    # Calculate the variance/covariance estimate

    if (vartype == "Local") {
      weight_lst <- localmean_weight(x1_u, y1_u, 1 / wgt1_u)
      if (is.null(weight_lst)) {
        warn_ind <- TRUE
        act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stratum \n\"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the contingency table proportion \nestimates.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("The local mean covariance estimator cannot calculate valid estimates, the simple random \nsampling covariance estimator for an infinite population was used to calculate \ncovariance of the contingency table proportion estimates.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        varest <- (ncluster * var(total2est * matrix(rep(wgt1_u, m_cl),
          nrow = ncluster
        )) + matrix(apply(var2est * matrix(rep(wgt1_u, m_cl^2),
          nrow = ncluster
        ), 2, sum), nrow = m_cl)) / tw2
      } else {
        varest <- (localmean_cov(total2est * matrix(rep(wgt1_u, m_cl),
          nrow = ncluster
        ), weight_lst) + matrix(apply(var2est *
          matrix(rep(wgt1_u, m_cl^2), nrow = ncluster), 2, sum), nrow = m_cl)) /
          tw2
        temp <- diag(varest)
        if (any(temp < 0)) {
          warn_ind <- TRUE
          act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
          if (stratum_ind) {
            warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates for \nstratum \"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the contingency table proportion \nestimates.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = stratum_level,
              warning = I(warn), action = I(act)
            ))
          } else {
            warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates, \nthe simple random sampling covariance estimator for an infinite population was used to \ncalculate covariance of the contingency table proportion estimates.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            ))
          }
          varest <- (ncluster * var(total2est * matrix(rep(wgt1_u, m_cl),
            nrow = ncluster
          )) + matrix(apply(var2est *
            matrix(rep(wgt1_u, m_cl^2), nrow = ncluster), 2, sum),
          nrow = m_cl
          )) / tw2
        }
      }
    } else {
      varest <- (ncluster * var(total2est * matrix(rep(wgt1_u, m_cl),
        nrow = ncluster
      )) + matrix(apply(var2est * matrix(rep(wgt1_u, m_cl^2),
        nrow = ncluster
      ), 2, sum), nrow = m_cl)) / tw2
    }
    colnames(varest) <- names_means

    # End of section for a two-stage sample
  } else {

    # Begin the section for a single-stage sample

    # Calculate additional required values

    tw2 <- (sum(wgt))^2

    # Calculate the weighted residuals matrix

    mm_cells <- subset(mm_cells, !(is.na(design$variables$rowvar) |
      is.na(design$variables$colvar)))
    n <- nrow(mm_cells)
    rm <- (mm_cells - matrix(rep(means, n), nrow = n, byrow = TRUE)) *
      matrix(rep(wgt, m), nrow = n)

    # Adjust the variance/covariance estimator for small sample size

    if (n < 4) {
      warn_ind <- TRUE
      act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
      if (stratum_ind) {
        warn <- paste0("There are less than four response values in stratum \"", stratum_level, "\", the simple \nrandom sampling covariance estimator for an infinite population was used to calculate \ncovariance of the contingency table proportion estimates.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
          indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
          action = I(act)
        ))
      } else {
        warn <- paste0("\nThere are less than four response values, the simple random sampling covariance \nestimator for an infinite population was used to calculate covariance of the contingency \ntable proportion estimates.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
          indicator = warn_vec[3], stratum = NA, warning = I(warn),
          action = I(act)
        ))
      }
      vartype <- "SRS"
    }

    # Calculate the variance/covariance estimate

    if (vartype == "Local") {
      weight_lst <- localmean_weight(x, y, 1 / wgt)
      if (is.null(weight_lst)) {
        warn_ind <- TRUE
        act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stratum \n\"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the contingency table proportion \nestimates.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("The local mean covariance estimator cannot calculate valid estimates, the simple random \nsampling covariance estimator for an infinite population was used to calculate \ncovariance of the contingency table proportion estimates.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        varest <- n * var(rm) / tw2
      } else {
        varest <- localmean_cov(rm, weight_lst) / tw2
        temp <- diag(varest)
        if (any(temp < 0)) {
          warn_ind <- TRUE
          act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
          if (stratum_ind) {
            warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates for \nstratum \"", stratum_level, "\", the simple random sampling covariance estimator for an \ninfinite population was used to calculate covariance of the contingency table proportion \nestimates.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = I(stratum_level),
              warning = I(warn), action = I(act)
            ))
          } else {
            warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates, \nthe simple random sampling covariance estimator for an infinite population was used to \ncalculate covariance of the contingency table proportion estimates.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            ))
          }
          varest <- n * var(rm) / tw2
        }
      }
    } else {
      varest <- n * var(rm) / tw2
    }
    colnames(varest) <- names_means

    # End of section for a single-stage sample
  }

  # Return the variance estimates, warning message indicator, and the warn_df
  # data frame

  list(varest = varest, warn_ind = warn_ind, warn_df = warn_df)
}

###############################################################################
# Function: sp_balance (exported)
# Programmer: Michael Dumelle
# Date: December 03, 2020
# Last Revised: December 03, 2020
#
#' Calculate spatial balance metrics
#'
#' This function measures the spatial balance (with respect to the
#' sampling frame) of design sites using
#' Voronoi polygons (Dirichlet tessellations).
#'
#' @param object An \code{sf} object containing some design sites.
#'
#' @param sframe The sampling frame as an \code{sf} object. The coordinate
#'   system for \code{sframe} must be one where distance for coordinates is meaningful.
#'
#' @param stratum_var The name of the stratum variable in \code{object}
#' and \code{sframe}. If \code{NULL} (the default), no strata is assumed.
#' If a single character vector is provided, it is assumed this is the
#' name of the stratum variable in \code{object} and \code{sframe}. If
#' a two-dimensional character vector is provided, one element must be
#' named "object" and corresponds to the name of the stratum variable
#' in \code{object}, while the other element must be named "sframe" and
#' corresponds to the name of the stratum variable in \code{sframe}.
#'
#' @param ip Inclusion probabilities associated with each row of \code{sframe}.
#' If these are not provided, an equal probability design is assumed (within
#' strata).
#'
#' @param metrics A character vector of spatial balance metrics:
#' \describe{
#'   \item{\code{pielou}}{ Pielou's Evenness Index (the default). This statistic
#'     can take on a value between zero and one.}
#'   \item{\code{simpsons}}{ Simpsons Evenness Index. This statistic
#'     can take on a value between zero and logarithm of the sample size.}
#'   \item{\code{rmse}}{ Root-Mean-Squared Error. This statistic
#'     can take on a value between zero and infinity.}
#'   \item{\code{mse}}{ Mean-Squared Error. This statistic
#'     can take on a value between zero and infinity.}
#'   \item{\code{mae}}{ Median-Absolute Error. This statistic
#'     can take on a value between zero and infinity.}
#'   \item{\code{medae}}{ Mean-Absolute Error. This statistic
#'     can take on a value between zero and infinity.}
#'   \item{\code{chisq}}{ Chi-Squared Loss. This statistic
#'     can take on a value between zero and infinity.}
#'  }
#'   All spatial balance metrics have a lower bound of zero, which indicates perfect
#'   spatial balance. As the metric value increases, the spatial balance decreases.
#'
#' @param extents Should the extent (total units) within each Voronoi polygon
#'   be returned? Defaults to \code{FALSE}.
#'
#' @return A data frame with columns providing the stratum (\code{stratum}),
#'   spatial balance metric (\code{metric}), and spatial balance (\code{value}).
#'
#' @importFrom stats median
#'
#' @author Michael Dumelle \email{Dumelle.Michael@@epa.gov}
#'
#' @export
#'
#' @examples
#' \dontrun{
#' sample <- grts(NE_Lakes, 30)
#' sp_balance(sample$sites_base, NE_Lakes)
#' strata_n <- c(low = 25, high = 30)
#' sample_strat <- grts(NE_Lakes, n_base = strata_n, stratum_var = "ELEV_CAT")
#' sp_balance(sample_strat$sites_base, NE_Lakes, stratum_var = "ELEV_CAT", metric = "rmse")
#' }
sp_balance <- function(object, sframe, stratum_var = NULL, ip = NULL, metrics = "pielou", extents = FALSE) {
  if (inherits(object, "spdesign")) {
    stop("object must be an sf object. If object is output from grts() or irs(), instead 1) use object$sites_legacy, object$sites_base, object$sites_over, or object$sites_near; or 2) use sp_rbind().")
  }

  # find system info
  on_solaris <- Sys.info()[["sysname"]] == "SunOS"
  if (on_solaris) {
    stop("sp_balance() is not supported on Solaris.")
  }

  if (is.null(stratum_var)) {
    object$stratum_var <- "None"
    sframe$stratum_var <- "None"
    stratum_var <- "stratum_var"
  }

  if (length(stratum_var) == 1) {
    stratum_var <- c(object = stratum_var, sframe = stratum_var)
  } else if (length(stratum_var) == 2) {
    if (!all(names(stratum_var) %in% c("object", "sframe"))) {
      stop("names of stratum_var must be object and sframe")
    }
  } else {
    stop("length of stratum_var must be zero, one, or two")
  }

  object_levels <- sort(unique(object[[stratum_var["object"]]]))
  sframe_levels <- sort(unique(sframe[[stratum_var["sframe"]]]))

  if (!all.equal(object_levels, sframe_levels)) {
    stop("stratum_var levels for object and sframe must be the same")
  }

  object_split <- split(object, object[[stratum_var["object"]]])
  sframe_split <- split(sframe, sframe[[stratum_var["sframe"]]])

  output <- lapply(object_levels, function(x) calculate_sp_balance(object_split[[x]], sframe_split[[x]], ip, metrics, extents))
  names(output) <- object_levels
  metrics_dfs <- lapply(object_levels, function(x) cbind(stratum = x, data.frame(metric = metrics, value = output[[x]]$values)))
  metrics <- do.call("rbind", metrics_dfs)
  metrics <- metrics[order(metrics$metric), , drop = FALSE]
  row.names(metrics) <- NULL
  if (extents) {
    extent_sf_split <- lapply(names(output), function(x) {
      object_split[[x]]$extent <- output[[x]]$extent
      object_split[[x]][c(stratum_var["object"], "extent")]
    })
    extents <- do.call("rbind", extent_sf_split)
    names(extents)[[which(names(extents) == stratum_var["object"])]] <- "stratum"
    return(list(metrics = metrics, extents = extents))
  } else {
    return(metrics)
  }
}

calculate_sp_balance <- function(object_split, sframe_split, ip, metrics, extents) {
  # need to calculate the density of each row in sframe_split
  if (all(st_geometry_type(sframe_split) %in% c("POINT", "MULTIPOINT"))) {
    sframe_split$dens <- 1
  } else if (all(st_geometry_type(sframe_split) %in% c("LINESTRING", "MULTILINESTRING"))) {
    sframe_split$dens <- as.numeric(st_length(sframe_split))
  } else if (all(st_geometry_type(sframe_split) %in% c("POLYGON", "MULTIPOLYGON"))) {
    sframe_split$dens <- as.numeric(st_area(sframe_split))
  }

  # these inclusion probabilities must be provided for the entire sampling frame
  if (is.null(ip)) {
    sframe_split$ip <- nrow(object_split) * (sframe_split$dens / sum(sframe_split$dens))
  } else {
    sframe_split$ip <- sframe_split[[ip]]
  }

  # finding the sframe bounding box, this needs to be reordered to
  # use in the Dirichlet Tesselation
  pop_bbox <- st_bbox(sframe_split)[c("xmin", "xmax", "ymin", "ymax")]

  # working with the object sites

  ## take the object sites coordinates
  samp_coords <- st_coordinates(object_split)
  ## name them X and Y
  colnames(samp_coords) <- c("X", "Y")
  ## isolate X
  samp_xcoord <- samp_coords[, "X"]
  ## isolate Y
  samp_ycoord <- samp_coords[, "Y"]
  ## recover the object size
  n <- nrow(object_split)

  # spatial balance with respect to the sframe
  tiles <- tile.list(deldir(x = samp_xcoord, y = samp_ycoord, rw = pop_bbox))
  ## using lapply instead of a loop
  sftess <- lapply(tiles, get_sftess)
  sftess <- st_sfc(sftess, crs = st_crs(sframe_split))
  sftess <- st_sf(poly = 1:n, geometry = sftess)
  # next part required because st_intersection won't include points if they are on the
  # exact boundary of a polygon (Voronoi polygon has this occur when a site is along the
  # bounding box)
  buffer_dist <- (pop_bbox["xmax"] - pop_bbox["xmin"]) * (pop_bbox["ymax"] - pop_bbox["ymin"]) / 1e12
  sftess <- suppressWarnings(st_intersection(st_make_valid(sframe_split), st_buffer(sftess, buffer_dist)))


  # scaling the inclusion probabilities by the amount of the geometry
  # in the respective tesselation -- polydens can be at most one -- if the
  # row of sframe is contained fully in the respective row of sframe
  # polydens can be less than one -- when the row of sframe is not contained
  # fully in the respective row of sframe
  # for points, sframe geometry must be point or multipoint
  if (all(st_geometry_type(sframe_split) %in% c("POINT", "MULTIPOINT"))) {
    ## storing a dummy variable to index counts by
    sftess$polydens <- 1
    sftess$adjip <- sftess$ip
  } else if (all(st_geometry_type(sframe_split) %in% c("LINESTRING", "MULTILINESTRING"))) {
    sftess$polydens <- as.numeric(st_length(sftess))
    sftess$adjip <- sftess$ip * sftess$polydens / sftess$dens
  } else if (all(st_geometry_type(sframe_split) %in% c("POLYGON", "MULTIPOLYGON"))) {
    sftess$polydens <- as.numeric(st_area(sftess))
    sftess$adjip <- sftess$ip * sftess$polydens / sftess$dens
  }

  ## summing over each polygon
  propextent <- with(sftess, tapply(adjip, poly, sum))
  ## setting NA's equal to zero
  propextent[is.na(propextent)] <- 0

  # making proportions and expected quantities
  proportions <- propextent / sum(propextent) # which equals n
  expected_proportions <- 1 / n


  # metrics
  values <- vapply(metrics, calculate_metric, double(1), proportions, expected_proportions)

  # returning the results when stored by the user
  output <- list(values = values)
  if (extents) {
    output$extent <- with(sftess, tapply(polydens, poly, sum))
  }
  output
}

get_sftess <- function(tile) {
  ## finding the number of points in the bounding polygon
  npol <- length(tile$x)

  ## creating and returning the appropriate polygon after binding coords
  st_polygon(list(cbind(
    c(tile$x[1], tile$x[npol:1]),
    c(tile$y[1], tile$y[npol:1])
  )))
}


calculate_metric <- function(metric, proportions, expected_proportions) {
  switch(metric,
    pielou = calculate_pielou(proportions, expected_proportions),
    simpsons = calculate_simpsons(proportions, expected_proportions),
    rmse = calculate_rmse(proportions, expected_proportions),
    mse = calculate_mse(proportions, expected_proportions),
    mae = calculate_mae(proportions, expected_proportions),
    medae = calculate_medae(proportions, expected_proportions),
    chisq = calculate_chisq(proportions, expected_proportions),
    stop("an invalid metric was provided")
  )
}

calculate_pielou <- function(proportions, expected_proportions) {
  pielou <- 1 + sum(proportions * log(proportions)) / log(1 / expected_proportions) # 1/E(p) = n$
  names(pielou) <- "pielou"
  pielou
}

calculate_simpsons <- function(proportions, expected_proportions) {
  simpsons <- sum(proportions^2) - expected_proportions
  names(simpsons) <- "simpsons"
  simpsons
}

calculate_rmse <- function(proportions, expected_proportions) {
  sqr_dev <- (proportions - expected_proportions)^2
  mse <- sum(sqr_dev) / length(sqr_dev)
  rmse <- sqrt(mse)
  names(rmse) <- "rmse"
  rmse
}

calculate_mse <- function(proportions, expected_proportions) {
  sqr_dev <- (proportions - expected_proportions)^2
  mse <- sum(sqr_dev) / length(sqr_dev)
  names(mse) <- "mse"
  mse
}

calculate_mae <- function(proportions, expected_proportions) {
  mae <- mean(abs(proportions - expected_proportions) / expected_proportions)
  names(mae) <- "mae"
  mae
}

calculate_medae <- function(proportions, expected_proportions) {
  medae <- median(abs(proportions - expected_proportions) / expected_proportions)
  names(medae) <- "medae"
  medae
}

calculate_chisq <- function(proportions, expected_proportions) {
  chisq <- sum((proportions - expected_proportions)^2 / expected_proportions)
  names(chisq) <- "chisq"
  chisq
}

# Helpers ---------------------------------------------------------------------

make_formlist <- function(formula, onlyshow, object, remove_geom = TRUE) {

  # find all terms from the formula
  varterms <- terms(formula, data = object)
  # find all variable names
  allvars <- all.vars(varterms)
  # find all right hand side names
  varlabels <- attr(varterms, "term.labels")
  # remove geometry if present
  # this was taken to be logical so it could 1) show geometry summaries and 2) remove a bug that
  # caused summaries to fail silently if no geometry column was in the data
  # but remain unaltered for plots
  if (remove_geom) {
    varlabels <- varlabels[varlabels != attr(object, "sf_column")]
  }
  # find if intercept exists in the formula
  if (attr(varterms, "intercept") == 1) {
    intercept <- TRUE
  } else {
    intercept <- FALSE
  }
  # find if response exists in the formula
  if (attr(varterms, "response") == 1) {
    response <- allvars[1]
  } else {
    response <- NULL
  }
  # make a vector of names
  varnames <- c(response, varlabels)
  # make a list where names are split if they interact
  varnames_split <- strsplit(varnames, ":")
  # giving the list names
  names(varnames_split) <- varnames
  # this will be used if onlyshow is removed from summary
  if (missing(onlyshow)) {
    onlyshow <- NULL
  }
  # storing the output list
  formlist <- list(
    varterms = varterms,
    allvars = allvars,
    varlabels = varlabels,
    intercept = intercept,
    response = response,
    varnames = varnames,
    varnames_split = varnames_split,
    onlyshow = onlyshow
  )
}

make_varsf <- function(object, formlist) {
  # can possibly deprecate this in the future by making use of
  # model.frame and extracting the main effects and using them to make the interactions
  # only real advantage will be creating variables for use mid formula with numeric variables



  if (formlist$intercept && is.null(formlist$response) && length(formlist$varlabels) == 0) {
    return(object)
  } else {
    # store geometry
    if ("sf" %in% class(object)) {
      object_geometry <- st_geometry(object)
      object_df <- st_drop_geometry(object)
    } else {
      object_df <- object
    }

    formlist <- lapply(
      formlist$varnames_split,
      function(x) {
        if (length(x) == 1 && (is.numeric(object_df[[x]]) || is.list(object_df[[x]]))) {
          return(object_df[, x, drop = FALSE]) # return numeric if provided
        } else if (length(x) > 1 && any(vapply(x, function(y) is.list(object_df[[y]]), logical(1)))) {
           stop("summarizing list-columns interacted with other variables not supported")
        }  else {
          return(interaction(object_df[, x, drop = FALSE], sep = ":")) # return factors
        }
      }
    )
    varsf <- as.data.frame(formlist, optional = TRUE) # without optional the : in name gets
    # converted to synctactic name with .
    if ("sf" %in% class(object)) {
      varsf <- st_as_sf(varsf, geometry = object_geometry)
    }
    return(varsf)
  }
}

get_varlevels <- function(formlist, varsf) {
  varsf_nogeom <- st_drop_geometry(varsf)
  levels <- lapply(formlist$varnames, function(x) {
    if (is.numeric(varsf[[x]]) || (x %in% formlist$response)) { # & !(x %in% formlist$varlabels))) {
      levels <- 0
    } else {
      levels <- length(na.omit(unique(varsf[[x]])))
    }
  })
  levels <- sum(unlist(levels))
}

make_varlevel_args_list <- function(varsf, varlevel_args) {
  varlevel_args_list <- lapply(names(varlevel_args), function(x) {
    vardf <- st_drop_geometry(varsf[x])
    vardf[[x]] <- as.character(vardf[[x]])
    colnames(vardf) <- "levels"
    vardf$index <- 1:nrow(vardf)
    varlevel_args_df <- as.data.frame(varlevel_args[[x]], stringsAsFactors = FALSE)
    varlevel_args_df <- merge(vardf, varlevel_args_df)
    varlevel_args_df <- varlevel_args_df[order(varlevel_args_df$index), , drop = FALSE]
    badcol <- which(colnames(varlevel_args_df) %in% c("levels", "index"))
    varlevel_args_df <- varlevel_args_df[, -badcol, drop = FALSE]
    varlevel_args_listval <- as.list(varlevel_args_df)
  })
  names(varlevel_args_list) <- names(varlevel_args)
  varlevel_args_list
}

make_var_args_list <- function(varsf, var_args) {
  var_args_list <- lapply(names(var_args), function(x) {
    var_args_listsub <- lapply(names(var_args[[x]]), function(y) {
      vardf <- st_drop_geometry(varsf[y])
      vardf[[y]] <- as.character(vardf[[y]])
      colnames(vardf) <- "levels"
      vardf$index <- 1:nrow(vardf)
      var_args_df <- as.data.frame(var_args[[x]][[y]], stringsAsFactors = FALSE)
      var_args_df <- merge(vardf, var_args_df)
      var_args_df <- var_args_df[order(var_args_df$index), , drop = FALSE]
      badcol <- which(colnames(var_args_df) %in% c("levels", "index"))
      var_args_df <- var_args_df[, -badcol, drop = FALSE]
      var_args_listsubval <- as.list(var_args_df)
    })
    names(var_args_listsub) <- names(var_args[[x]])
    var_args_listsub
  })
  names(var_args_list) <- names(var_args)
  var_args_list
}

check_rhs_cat <- function(varsf, formlist) {
  any_numeric <- vapply(varsf[[formlist$varlabels]], is.numeric(), logical(1))
  if (any_numeric) {
    stop("Right hand side of formula must only contain categorical variables")
  }
}

match_sf_defaults <- function(varsf, list_args) {
  sf_default_df <- data.frame(
    geometry = c("POINT", "MULTIPOINT", "LINESTRING", "MULTILINESTRING", "POLYGON", "MULTIPOLYGON"),
    pch = c(1, 1, 1, 1, NA, NA),
    cex = c(1, 1, NA, NA, 1, 1),
    col = c(1, 1, 1, 1, NA, NA),
    bg = c(0, 0, NA, NA, NA, NA),
    lwd = c(1, 1, 1, 1, 1, 1),
    lty = c(1, 1, 1, 1, 1, 1),
    type = c("p", "p", "l", "l", NA, NA),
    border = c(NA, NA, NA, NA, 1, 1),
    rule = c(NA, NA, NA, NA, "evenodd", "evenodd"),
    stringsAsFactors = FALSE
  )
  sf_defaults <- merge(data.frame(geometry = as.character(st_geometry_type(varsf))), sf_default_df, sort = FALSE)
  # sf_defaults <- lapply(st_geometry_type(varsf), get_sf_defaults)
  # sf_defaults <- do.call("rbind", sf_defaults)


  names_list_args <- names(list_args)
  list_args <- lapply(names_list_args, function(x) {
    if (x %in% names(sf_defaults) && any(is.na(list_args[[x]]))) {
      list_args[[x]] <- ifelse(is.na(list_args[[x]]), sf_defaults[[x]], list_args[[x]])
    } else {
      list_args[[x]] <- list_args[[x]]
    }
  })
  names(list_args) <- names_list_args
  list_args
}

# CRAN release questions
release_questions <- function() {
  c(
    "Have you changed version numbers in DESCRIPTION"
  )
}

# get_sf_defaults <-   function(geometry) {
#   if (geometry %in% c("POINT", "MULTIPOINT")) {
#     sf_defaults <- data.frame(pch = 1, cex = 1, col = 1, bg = 0, lwd = 1, lty = 1, type = "p", border = NA, rule = NA, stringsAsFactors = FALSE)
#   } else if (geometry %in% c("LINESTRING", "MULTILINESTRING")){
#     sf_defaults <- data.frame(pch = 1, cex = NA, col = 1, bg = 0, lwd = 1, lty = 1, type = "l", border = NA, rule = NA, stringsAsFactors = FALSE)
#   } else if (geometry %in% c("POLYGON", "MULTIPOLYGON")){
#     sf_defaults <- data.frame(pch = NA, cex = 1, col = NA, bg = NA, lwd = 1, lty = 1, type = NA, border = 1, rule = "evenodd", stringsAsFactors = FALSE)
#   } else {
#     stop("All x geometries must be POINT, MULTIPOINT, LINESTRING, MULTILINESTRING, POLYGON, or MULTIPOLYGON")
#   }
#   sf_defaults
# }

################################################################################
# Function: category_est (not exported)
# Programmer: Tom Kincaid
# Date: July 23, 2020
# Revised: February 9, 2021 to fix incorrect coding of "confval_P" as
#          "stderr_P".
# Revised: February 16, 2021 to correctly handle the case where the response
#          variable contains a single category.
# Revised: February 22, 2021 to fix incorrect coding of "rslt_T" as
#          "rslt_U".
# Revised: May 20 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
#
#' Category Proportion and Total Estimates for Probability Survey Data
#'
#' This function calculates estimates of the proportion (expressed as percent)
#' and total (size) for each of the categories of a categorical response
#' variable.  Upper and lower confidence bounds also are estimated.  Proportion
#' estimates are calculated using the Horvitz-Thompson ratio estimator, i.e.,
#' the ratio of two Horvitz-Thompson estimators.  The numerator of the ratio
#' estimates the total of the category.  The denominator of the ratio estimates
#' the total of the resource. Variance estimates for the proportion estimates
#' are calculated using either the local mean variance estimator or the simple
#' random sampling (SRS) variance estimator.  The choice of variance estimator
#' is subject to user control. The local mean variance estimator requires the
#' x-coordinate and the y-coordinate of each site.  Confidence bounds are
#' calculated using a Normal distribution multiplier. For a finite resource,
#' total is the number of units in the resource.  For an extensive resource,
#' total is the measure (extent) of the resource, i.e., length, area, or volume.
#' Total estimates are calculated using the Horvitz-Thompson estimator.
#' Variance estimates for the total estimates are calculated using either the
#' local mean variance estimator or the SRS variance estimator.
#'
#' @param catsum Data frame containing estimates.
#'
#' @param dframe Data frame containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param itype Character value that identifies a factor variable in the design
#'   argument containing subpopulation (domain) values.
#'
#' @param lev_itype Character vector that provides levels of the subpopulation
#'   variable.
#'
#' @param nlev_itype Numeric value that provides the number of levels of the
#'   subpopulation variable.
#'
#' @param ivar Character value that identifies a factor variable in the design
#'   argument containing categorical response values.
#'
#' @param lev_ivar Character vector that provides levels of the categorical
#'   response variable.
#'
#' @param nlev_ivar Numeric value that provides the number of levels of the
#'   categorical response variable.
#'
#' @param design Object of class survey.design that specifies a complex survey
#'   design.
#'
#' @param design_names Character vector that provides names of survey design
#'   variables in the design argument.
#'
#' @param vartype Character value providing the choice of the variance
#'   estimator, where "Local" = the local mean estimator, \code{"SRS"} is the
#'   simple random sampling estimator, \code{"HT"} is the Horvitz-Thompson
#'   estimator, and \code{"YG"} is the Yates-Grundy estimator.  The default value
#'   is \code{"Local"}.
#'
#' @param conf Numeric value for the confidence level.
#'
#' @param mult Numeric value that provides the Normal distribution confidence
#'   bound multiplier.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @return A list composed of the following objects:
#'   \itemize{
#'     \item{\code{catsum}}{data frame containing the category estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey univar
#'
#' @noRd
################################################################################

category_est <- function(catsum, dframe, itype, lev_itype, nlev_itype, ivar,
                         lev_ivar, nlev_ivar, design, design_names,
                         vartype, conf, mult, warn_ind, warn_df) {

  # Assign a value to the function name variable

  fname <- "category_est"

  # Calculate category proportion estimates, standard error estimates, and
  # confidence bound estimates for each combination of subpopulation and response
  # variable

  tst <- !is.na(dframe[, itype])
  if (nlev_itype == 1) {
    if (nlev_ivar <= 1) {
      temp <- ifelse(all(is.na(dframe[, ivar])), 0.0, 1.0)
      rslt_P <- data.frame(ivar = temp, Total = temp)
      dimnames(rslt_P)[1] <- lev_itype
      stderr_P <- data.frame(ivar = 0.0, Total = 0.0)
      temp <- data.frame(LCB = temp, UCB = temp)
      confval_P <- rbind(ivar = temp, Total = temp)
    } else {
      rslt_svy <- svymean(make.formula(ivar),
        design = subset(design, tst),
        na.rm = TRUE
      )
      rslt_P <- cbind(data.frame(t(as.data.frame(rslt_svy)[1])),
        Total = 1.0
      )
      dimnames(rslt_P)[1] <- lev_itype
      if (vartype == "Local") {
        temp <- cat_localmean_prop(
          itype, lev_itype, nlev_itype, ivar, lev_ivar, nlev_ivar, design,
          design_names, rslt_P, mult, warn_ind, warn_df
        )
        stderr_P <- temp$stderr_P
        confval_P <- temp$confval_P
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        stderr_P <- cbind(t(data.frame(SE(rslt_svy))), Total = 0.0)
        confval_P <- confint(rslt_svy, level = conf / 100)
        tempdf <- data.frame(LCB = 1.0, UCB = 1.0)
        dimnames(tempdf) <- list("Total", colnames(confval_P))
        confval_P <- rbind(confval_P, tempdf)
      }
    }
  } else {
    if (nlev_ivar == 1) {
      ivar_splt <- split(dframe[, ivar], dframe[, itype])
      temp <- ifelse(sapply(ivar_splt, function(x) all(is.na(x))), 0.0, 1.0)
      rslt_P <- data.frame(ivar = temp, Total = temp)
      dimnames(rslt_P)[[1]] <- lev_itype
      stderr_P <- data.frame(ivar = rep(0.0, nlev_itype), Total = rep(
        0.0,
        nlev_itype
      ))
      confval_P <- NULL
      for (i in 1:nlev_itype) {
        tempdf <- rbind(data.frame(temp[i], temp[i]), data.frame(
          temp[i],
          temp[i]
        ))
        rownames(tempdf) <- c(
          paste(lev_itype[i], ivar, sep = ":"),
          paste(lev_itype[i], "Total", sep = ":")
        )
        colnames(tempdf) <- c("LCB", "UCB")
        confval_P <- rbind(confval_P, tempdf)
      }
    } else {
      rslt_svy <- svyby(make.formula(ivar), make.formula(itype),
        design = subset(design, tst), svymean, na.rm = TRUE
      )
      rslt_P <- rslt_svy[, 2:(nlev_ivar + 1)]
      rslt_P <- cbind(rslt_P, Total = rep(1, nlev_itype))
      if (vartype == "Local") {
        temp <- cat_localmean_prop(
          itype, lev_itype, nlev_itype, ivar, lev_ivar, nlev_ivar, design,
          design_names, rslt_P, mult, warn_ind, warn_df
        )
        stderr_P <- temp$stderr_P
        confval_P <- temp$confval_P
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        stderr_P <- cbind(SE(rslt_svy), Total = rep(0, nlev_itype))
        tempdf <- confint(rslt_svy, level = conf / 100)
        confval_P <- NULL
        for (i in 1:nlev_itype) {
          confval_P <- rbind(confval_P, tempdf[seq(i,
            by = nlev_itype,
            length = nlev_ivar
          ), ])
          temp <- data.frame(1.0, 1.0)
          colnames(temp) <- colnames(tempdf)
          confval_P <- rbind(confval_P, Total = temp)
        }
      }
    }
  }

  # Calculate category total estimates, standard error estimates, and
  # confidence bound estimates for each combination of subpopulation and response
  # variable

  if ("postStrata" %in% names(design)) {
    zzz <- ifelse(is.na(design$variables[, ivar]), NA, 1)
  } else {
    zzz <- ifelse(is.na(dframe[, ivar]), NA, 1)
  }
  design <- update(design, zzz = zzz)
  if (nlev_itype == 1) {
    if (nlev_ivar <= 1) {
      rslt_svy <- svytotal(~zzz, design = subset(design, tst), na.rm = TRUE)
      rslt_U <- data.frame(ivar = rslt_svy[1], Total = rslt_svy[1])
      dimnames(rslt_U)[1] <- lev_itype
      if (vartype == "Local") {
        temp <- cat_localmean_total(
          itype, lev_itype, nlev_itype, ivar, lev_ivar, nlev_ivar, design,
          design_names, rslt_U, mult, warn_ind, warn_df
        )
        stderr_U <- temp$stderr_U
        confval_U <- temp$confval_U
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        temp <- SE(rslt_svy)
        stderr_U <- data.frame(ivar = as.vector(temp), Total = as.vector(temp))
        temp <- as.data.frame(confint(rslt_svy, level = conf / 100))
        confval_U <- rbind(ivar = temp, Total = temp)
      }
    } else {
      rslt_svy <- svytotal(make.formula(ivar),
        design = subset(design, tst),
        na.rm = TRUE
      )
      rslt_T <- svytotal(~zzz, design = subset(design, tst), na.rm = TRUE)
      rslt_U <- cbind(data.frame(t(as.data.frame(rslt_svy)[1])),
        Total = rslt_T[1]
      )
      dimnames(rslt_U)[1] <- lev_itype
      if (vartype == "Local") {
        temp <- cat_localmean_total(
          itype, lev_itype, nlev_itype, ivar, lev_ivar, nlev_ivar, design,
          design_names, rslt_U, mult, warn_ind, warn_df
        )
        stderr_U <- temp$stderr_U
        confval_U <- temp$confval_U
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        temp <- SE(rslt_T)
        stderr_U <- cbind(t(data.frame(SE(rslt_svy))), Total = as.vector(temp))
        temp <- as.data.frame(confint(rslt_T, level = conf / 100))
        confval_U <- rbind(confint(rslt_svy, level = conf / 100), Total = temp)
      }
    }
  } else {
    if (nlev_ivar == 1) {
      rslt_svy <- svyby(~zzz, make.formula(itype),
        design = subset(design, tst),
        svytotal, na.rm = TRUE
      )
      rslt_U <- data.frame(ivar = rslt_svy$zzz, Total = rslt_svy$zzz)
      dimnames(rslt_U)[[1]] <- lev_itype
      if (vartype == "Local") {
        temp <- cat_localmean_total(
          itype, lev_itype, nlev_itype, ivar, lev_ivar, nlev_ivar, design,
          design_names, rslt_U, mult, warn_ind, warn_df
        )
        stderr_U <- temp$stderr_U
        confval_U <- temp$confval_U
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        temp <- SE(rslt_svy)
        stderr_U <- data.frame(ivar = temp, Total = temp)
        temp <- as.data.frame(confint(rslt_svy, level = conf / 100))
        confval_U <- rbind(ivar = temp, Total = temp)
      }
    } else {
      rslt_svy <- svyby(make.formula(ivar), make.formula(itype),
        design = subset(design, tst), svytotal, na.rm = TRUE
      )
      rslt_T <- svyby(~zzz, make.formula(itype),
        design = subset(design, tst),
        svytotal, na.rm = TRUE
      )
      rslt_U <- rslt_svy[, 2:(nlev_ivar + 1)]
      temp <- rslt_T[2]
      names(temp) <- "Total"
      rslt_U <- cbind(rslt_U, Total = temp)
      if (vartype == "Local") {
        temp <- cat_localmean_total(
          itype, lev_itype, nlev_itype, ivar, lev_ivar, nlev_ivar, design,
          design_names, rslt_U, mult, warn_ind, warn_df
        )
        stderr_U <- temp$stderr_U
        confval_U <- temp$confval_U
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        temp <- SE(rslt_T)
        stderr_U <- cbind(SE(rslt_svy), Total = temp)
        temp <- as.data.frame(confint(rslt_T, level = conf / 100))
        tempdf <- confint(rslt_svy, level = conf / 100)
        confval_U <- NULL
        for (i in 1:nlev_itype) {
          confval_U <- rbind(confval_U, tempdf[seq(i,
            by = nlev_itype,
            length = nlev_ivar
          ), ], Total = temp[i, ])
        }
      }
    }
  }

  # Assign identifiers and estimates to the catsum data frame

  snames <- dimnames(rslt_P)[[1]]
  inames <- rep(ivar, nlev_ivar + 1)
  lev <- c(lev_ivar, "Total")
  cnames <- lev
  nresp <- NULL
  for (j in snames) {
    eval(parse(text = paste0("temp <- with(dframe, addmargins(table(", itype, " ,", ivar, ")))")))
    colnames(temp)[ncol(temp)] <- "Total"
    temp <- temp[-nrow(temp), ]
    for (k in lev) {
      if (nlev_itype == 1) {
        nresp <- c(nresp, temp[k])
      } else {
        nresp <- c(nresp, temp[j, k])
      }
    }
  }
  k <- 1
  for (i in 1:nrow(rslt_P)) {
    inc <- (i - 1) * (nlev_ivar + 1)
    for (j in 1:length(inames)) {
      catsum <- rbind(catsum, data.frame(
        Type = itype,
        Subpopulation = snames[i],
        Indicator = inames[j],
        Category = cnames[j],
        nResp = nresp[k],
        Estimate.P = 100 * rslt_P[i, j],
        StdError.P = 100 * stderr_P[i, j],
        MarginofError.P = 100 * (mult * stderr_P[i, j]),
        LCB.P = 100 * max(confval_P[inc + j, 1], 0),
        UCB.P = 100 * min(confval_P[inc + j, 2], 1),
        Estimate.U = rslt_U[i, j],
        StdError.U = stderr_U[i, j],
        MarginofError.U = mult * stderr_U[i, j],
        LCB.U = max(confval_U[inc + j, 1], 0),
        UCB.U = confval_U[inc + j, 2]
      ))
      k <- k + 1
    }
  }

  # Return the catsum data frame, the warn_ind logical value, and the warn_df
  # data frame

  list(catsum = catsum, warn_ind = warn_ind, warn_df = warn_df)
}

################################################################################
# Function: attrisk_analysis (exported)
# Programmer: Tom Kincaid
# Date: July 16, 2020
# Revised: August 14, 2020 to allow use of an sf object as the input data
#          argument (dframe)
# Revised: December 16, 2020 to use a new function named survey_design to create
#          the survey design object
# Revised: January 28, 2021 to replace "pcfactor.ind" with "pcfactor_ind"
# Revised: March 2, 2021 to revise the process for creating unique site ID
#          values
# Revised: April 7, 2021 to ensure that the dframe argument does not contain
#          zero rows
# Revised: April 29, 2021 to ensure that the dframe argument only belongs to
#          class "data.frame"
# Revised: May 4, 2021 to avoid warning messages being generated during creation
#          of help files
# Revised: May 6, 2021 to ensure that sf objects do not belong to class tbl_df
# Revised: June 8, 2021 to simplify specification of the values required for
#          calculation of the finite population correction factor and to
#          eliminate use of the finite population correction factor with the
#          local mean variance estimator
# Revised: June 11, 2021 to modify the approach for specification of arguments
#          response_levels and stressor_levels
# Revised: July 28, 2021 to improve documentation
# Revised: September 9, 2021 to revise the documentation for argument popsize
#
#' Attributable risk analysis
#'
#' This function organizes input and output for the analysis of attributable risk (for
#' categorical variables).  The analysis data,
#' \code{dframe}, can be either a data frame or a simple features (\code{sf}) object.  If an
#' \code{sf} object is used, coordinates are extracted from the geometry column in the
#' object, arguments \code{xcoord} and \code{ycoord} are assigned values
#' \code{"xcoord"} and \code{"ycoord"}, respectively, and the geometry column is
#' dropped from the object.
#'
#' @param dframe Data to be analyzed (analysis data). A data frame or
#'   \code{sf} object containing survey design
#'   variables, response variables, stressor variables, and subpopulation
#'   (domain) variables.
#'
#' @param vars_response Vector composed of character values that identify the
#'   names of response variables in \code{dframe}. Each response
#'   variable must have two category values (levels), where one level is
#'   associated with poor condition and the other level is associated with good
#'   condition.
#'
#' @param vars_stressor Vector composed of character values that identify the
#'   names of stressor variables in \code{dframe}. Each stressor
#'   variable must have two category values (levels), where one level is
#'   associated with poor condition and the other level is associated with good
#'   condition.
#'
#' @param response_levels List providing the category values (levels) for each
#'   element in the \code{vars_response} argument.  Each element in the list
#'   must contain two values, where the first value identifies poor condition,
#'   and the second value identifies good condition.  This argument must be
#'   named and must be the same length as argument \code{vars_response}.  Names
#'   for this argument must match the values in the \code{vars_response}
#'   argument. If this argument equals NULL, then a named list is created that
#'   contains the values \code{"Poor"} and \code{"Good"} for the first and
#'   second levels, respectively, of each element in the \code{vars_response}
#'   argument and that uses values in the \code{vars_response} argument as names
#'   for the list.  If \code{response_levels} is provided without names,
#'   then the names of \code{response_levels} are set to \code{vars_response}.
#'   The default value is NULL.
#'
#' @param stressor_levels List providing the category values (levels) for each
#'   element in the \code{vars_stressor} argument.  Each element in the list
#'   must contain two values, where the first value identifies poor condition,
#'   and the second value identifies good condition.  This argument must be
#'   named and must be the same length as argument \code{vars_stressor}.  Names
#'   for this argument must match the values in the \code{vars_stressor}
#'   argument. If this argument equals NULL, then a named list is created that
#'   contains the values \code{"Poor"} and \code{"Good"} for the first and
#'   second levels, respectively, of each element in the \code{vars_stressor}
#'   argument and that uses values in the \code{vars_stressor} argument as names
#'   for the list.  If \code{stressor_levels} is provided without names,
#'   then the names of \code{stressor_levels} are set to \code{vars_stressor}. 
#'   The default value is NULL.
#'
#' @param subpops Vector composed of character values that identify the
#'   names of subpopulation (domain) variables in \code{dframe}.
#'   If a value is not provided, the value \code{"All_Sites"} is assigned to the
#'   subpops argument and a factor variable named \code{"All_Sites"} that takes
#'   the value \code{"All Sites"} is added to \code{dframe}.  The
#'   default value is \code{NULL}.
#'
#' @param siteID Character value providing the name of the site ID variable in
#'   \code{dframe}.  For a two-stage sample, the site ID variable
#'   identifies stage two site IDs.  The default value is \code{NULL}, which
#'   assumes that each row in \code{dframe} represents a unique site.
#'
#' @param weight Character value providing the name of the design weight
#'   variable in \code{dframe}.  For a two-stage sample, the
#'   weight variable identifies stage two weights.  The default value is
#'   \code{"weight"}.
#'
#' @param xcoord Character value providing name of the x-coordinate variable in
#'   \code{dframe}.  For a two-stage sample, the x-coordinate
#'   variable identifies stage two x-coordinates.  Note that x-coordinates are
#'   required for calculation of the local mean variance estimator.  If \code{dframe}
#'   is an \code{sf} object, this argument is not required (as the geometry column
#'   in \code{dframe} is used to find the x-coordinate). The default
#'   value is \code{NULL}.
#'
#' @param ycoord Character value providing name of the y-coordinate variable in
#'   \code{dframe}.  For a two-stage sample, the y-coordinate
#'   variable identifies stage two y-coordinates.  Note that y-coordinates are
#'   required for calculation of the local mean variance estimator.  If \code{dframe}
#'   is an \code{sf} object, this argument is not required (as the geometry column
#'   in \code{dframe} is used to find the t-coordinate). The default
#'   value is \code{NULL}.
#'
#' @param stratumID Character value providing the name of the stratum ID
#'   variable in \code{dframe}.  The default value is
#'   \code{NULL}.
#'
#' @param clusterID Character value providing the name of the cluster
#'   (stage one) ID variable in \code{dframe}.  Note that cluster
#'   IDs are required for a two-stage sample.  The default value is \code{NULL}.
#'
#' @param weight1 Character value providing the name of the stage one weight
#'   variable in \code{dframe}.  The default value is
#'   \code{NULL}.
#'
#' @param xcoord1 Character value providing the name of the stage one
#'   x-coordinate variable in \code{dframe}.  Note that x
#'   coordinates are required for calculation of the local mean variance
#'   estimator.  The default value is \code{NULL}.
#'
#' @param ycoord1 Character value providing the name of the stage one
#'   y-coordinate variable in \code{dframe}.  Note that
#'   y-coordinates are required for calculation of the local mean variance
#'   estimator.  The default value is \code{NULL}.
#'
#' @param sizeweight Logical value that indicates whether size weights should be
#'   used during estimation, where \code{TRUE} uses size weights and
#'   \code{FALSE} does not use size weights. To employ size weights for a
#'   single-stage sample, a value must be supplied for argument weight.  To
#'   employ size weights for a two-stage sample, values must be supplied for
#'   arguments \code{weight} and \code{weight1}. The default value is
#'   \code{FALSE}.
#'
#' @param sweight Character value providing the name of the size weight variable
#'   in \code{dframe}.  For a two-stage sample, the size weight
#'   variable identifies stage two size weights.  The default value is
#'   \code{NULL}.
#'
#' @param sweight1 Character value providing the name of the stage one size
#' weight variable in \code{dframe}.  The default value is
#' \code{NULL}.
#'
#' @param fpc Object that specifies values required for calculation of the
#'   finite population correction factor used during variance estimation. The
#'   object must match the survey design in terms of stratification and whether
#'   the design is single-stage or two-stage.  For an unstratified design, the
#'   object is a vector.  The vector is composed of a single numeric value for a
#'   single-stage design.  For a two-stage unstratified design, the object is a
#'   named vector containing one more than the number of clusters in the sample,
#'   where the first item in the vector specifies the number of clusters in the
#'   population and each subsequent item specifies the number of stage two units
#'   for the cluster.  The name for the first item in the vector is arbitrary.
#'   Subsequent names in the vector identify clusters and must match the cluster
#'   IDs.  For a stratified design, the object is a named list of vectors, where
#'   names must match the strata IDs.  For each stratum, the format of the
#'   vector is identical to the format described for unstratified single-stage
#'   and two-stage designs.  Note that the finite population correction factor
#'   is not used with the local mean variance estimator.
#'
#'   Example fpc for a single-stage unstratified survey design:
#'
#'   \verb{fpc <- 15000}
#'
#'   Example fpc for a single-stage stratified survey design:
#'
#'   \verb{fpc <- list(
#'     Stratum_1 = 9000,
#'     Stratum_2 = 6000)
#'    }
#'
#'   Example fpc for a two-stage unstratified survey design:
#'
#'   \verb{fpc <- c(
#'     Ncluster = 150,
#'     Cluster_1 = 150,
#'     Cluster_2 = 75,
#'     Cluster_3 = 75,
#'     Cluster_4 = 125,
#'     Cluster_5 = 75)
#'   }
#'
#'   Example fpc for a two-stage stratified survey design:
#'
#'   \verb{fpc <- list(
#'     Stratum_1 = c(
#'       Ncluster_1 = 100,
#'       Cluster_1 = 125,
#'       Cluster_2 = 100,
#'       Cluster_3 = 100,
#'       Cluster_4 = 125,
#'       Cluster_5 = 50),
#'     Stratum_2 = c(
#'       Ncluster_2 = 50,
#'       Cluster_1 = 75,
#'       Cluster_2 = 150,
#'       Cluster_3 = 75,
#'       Cluster_4 = 75,
#'       Cluster_5 = 125))
#'   }
#'
#' @param popsize Object that provides values for the population argument of the
#'   \code{calibrate} or \code{postStratify} functions in the survey package. If
#'   a value is provided for popsize, then either the \code{calibrate} or
#'   \code{postStratify} function is used to modify the survey design object
#'   that is required by functions in the survey package.  Whether to use the
#'   \code{calibrate} or \code{postStratify} function is dictated by the format
#'   of popsize, which is discussed below.  Post-stratification adjusts the
#'   sampling and replicate weights so that the joint distribution of a set of
#'   post-stratifying variables matches the known population joint distribution.
#'   Calibration, generalized raking, or GREG estimators generalize
#'   post-stratification and raking by calibrating a sample to the marginal
#'   totals of variables in a linear regression model. For the \code{calibrate}
#'   function, the object is a named list, where the names identify factor
#'   variables in \code{dframe}.  Each element of the list is a
#'   named vector containing the population total for each level of the
#'   associated factor variable.  For the \code{postStratify} function, the
#'   object is either a data frame, table, or xtabs object that provides the
#'   population total for all combinations of selected factor variables in the
#'   \code{dframe} data frame.  If a data frame is used for \code{popsize}, the
#'   variable containing population totals must be the last variable in the data
#'   frame.  If a table is used for \code{popsize}, the table must have named
#'   \code{dimnames} where the names identify factor variables in the
#'   \code{dframe} data frame.  If the popsize argument is equal to \code{NULL},
#'   then neither calibration nor post-stratification is performed.  The default
#'   value is \code{NULL}.
#'
#'   Example popsize for calibration:
#'
#'   \verb{popsize <- list(
#'     Ecoregion = c(
#'       East = 750,
#'       Central = 500,
#'       West = 250),
#'     Type = c(
#'       Streams = 1150,
#'       Rivers = 350))
#'   }
#'
#'   Example popsize for post-stratification using a data frame:
#'
#'   \verb{popsize <- data.frame(
#'     Ecoregion = rep(c("East", "Central", "West"),
#'       rep(2, 3)),
#'     Type = rep(c("Streams", "Rivers"), 3),
#'     Total = c(575, 175, 400, 100, 175, 75))
#'   }
#'
#'   Example popsize for post-stratification using a table:
#'
#'   \verb{popsize <- with(MySurveyFrame,
#'     table(Ecoregion, Type))}
#'
#'   Example popsize for post-stratification using an xtabs object:
#'
#'   \verb{popsize <- xtabs(~Ecoregion + Type,
#'     data = MySurveyFrame)}
#'
#' @param vartype Character value providing the choice of the variance
#'   estimator, where \code{"Local"} indicates the local mean estimator and \code{"SRS"} indicates the
#'   simple random sampling estimator.  The default value is \code{"Local"}.
#'
#' @param conf Numeric value providing the Gaussian-based confidence level.  The default value
#'   is \code{95}.
#'
#' @param All_Sites A logical variable used when \code{subpops} is not
#'   \code{NULL}. If \code{All_Sites} is \code{TRUE}, then alongside the
#'   subpopulation output, output for all sites (ignoring subpopulations) is
#'   returned for each variable in \code{vars}. If \code{All_Sites} is
#'   \code{FALSE}, then alongside the subpopulation output, output for all sites
#'   (ignoring subpopulations) is not returned for each variable in \code{vars}.
#'   The default is \code{FALSE}.
#'
#' @section Details:
#' Attributable risk measures the proportional reduction in the extent of poor
#' condition of a response variable that presumably would result from
#' eliminating a stressor variable, where the response and stressor variables
#' are classified as either good (i.e., reference condition) or poor (i.e.,
#' different from reference condition).  Attributable risk is defined as one
#' minus the ratio of two probabilities.  The numerator of the ratio is the
#' conditional probability that the response variable is in poor condition given
#' that the stressor variable is in good condition.   The denominator of the
#' ratio is the probability that the response variable is in poor condition.
#' Attributable risk values close to zero indicate that removing the stressor
#' variable will have little or no impact on the probability that the response
#' variable is in poor condition.  Attributable risk values close to one
#' indicate that removing the stressor variable will result in extensive
#' reduction of the probability that the response variable is in poor condition.
#'
#' @return The analysis results. A data frame of population estimates for all combinations of
#'   subpopulations, categories within each subpopulation, response variables,
#'   and categories within each response variable.  Estimates are provided for
#'   proportion and size of the population plus standard error, margin of
#'   error, and confidence interval estimates. The data frame contains the following
#'   variables:
#'   \describe{
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Response}{response variable}
#'     \item{Stressor}{stressor variable}
#'     \item{nResp}{sample size}
#'     \item{Estimate}{attributable risk estimate}
#'     \item{StdError_log}{attributable risk standard error (on the log scale)}
#'     \item{MarginofError_log}{attributable risk margin of error (on the log scale)}
#'     \item{LCBxxPct}{xx\% (default 95\%) lower confidence bound}
#'     \item{UCBxxPct}{xx\% (default 95\%) upper confidence bound}
#'     \item{WeightTotal}{sum of design weights}
#'     \item{Count_RespPoor_StressPoor}{number of observations in the poor response and poor stressor group}
#'     \item{Count_RespPoor_StressGood}{number of observations in the poor response and good stressor group}
#'     \item{Count_RespGood_StressPoor}{number of observations in the good response and poor stressor group}
#'     \item{Count_RespGood_StressGood}{number of observations in the good response and good stressor group}
#'     \item{Prop_RespPoor_StressPoor}{weighted proportion of observations in the poor response and poor stressor group}
#'     \item{Prop_RespPoor_StressGood}{weighted proportion of observations in the poor response and good stressor group}
#'     \item{Prop_RespGood_StressPoor}{weighted proportion of observations in the good response and poor stressor group}
#'     \item{Prop_RespGood_StressGood}{weighted proportion of observations in the good response and good stressor group}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey univar
#'
#' @seealso
#'   \describe{
#'   \item{\code{\link{relrisk_analysis}}}{ for relative risk analysis}
#'   \item{\code{\link{diffrisk_analysis}}}{ for risk difference analysis}
#'   }
#'
#' @references
#'   Sickle, J. V., & Paulsen, S. G. (2008). Assessing the attributable risks,
#'    relative risks, and regional extents of aquatic stressors.
#'    \emph{Journal of the North American Benthological Society}, 27(4), 920-931.
#'
#' @examples
#' dframe <- data.frame(
#'   siteID = paste0("Site", 1:100),
#'   wgt = runif(100, 10, 100),
#'   xcoord = runif(100),
#'   ycoord = runif(100),
#'   stratum = rep(c("Stratum1", "Stratum2"), 50),
#'   RespVar1 = sample(c("Poor", "Good"), 100, replace = TRUE),
#'   RespVar2 = sample(c("Poor", "Good"), 100, replace = TRUE),
#'   StressVar = sample(c("Poor", "Good"), 100, replace = TRUE),
#'   All_Sites = rep("All Sites", 100),
#'   Resource_Class = rep(c("Agr", "Forest"), c(55, 45))
#' )
#' myresponse <- c("RespVar1", "RespVar2")
#' mystressor <- c("StressVar")
#' mysubpops <- c("All_Sites", "Resource_Class")
#' attrisk_analysis(dframe,
#'   vars_response = myresponse,
#'   vars_stressor = mystressor, subpops = mysubpops, siteID = "siteID",
#'   weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
#'   stratumID = "stratum"
#' )
#' @export
################################################################################

attrisk_analysis <- function(dframe, vars_response, vars_stressor, response_levels = NULL,
                             stressor_levels = NULL, subpops = NULL, siteID = NULL, weight = "weight",
                             xcoord = NULL, ycoord = NULL, stratumID = NULL, clusterID = NULL,
                             weight1 = NULL, xcoord1 = NULL, ycoord1 = NULL, sizeweight = FALSE,
                             sweight = NULL, sweight1 = NULL, fpc = NULL, popsize = NULL,
                             vartype = "Local", conf = 95, All_Sites = FALSE) {

  # Create a vector for error messages

  error_ind <- FALSE
  error_vec <- NULL

  # Create a data frame for warning messages

  warn_ind <- FALSE
  warn_df <- NULL
  fname <- "attrisk_analysis"

  # Ensure that the dframe argument was provided

  if (missing(dframe) | is.null(dframe)) {
    stop("\nThe dframe argument must be provided.\n")
  }

  # If the dframe argument is an sf object, extract coordinates from the
  # geometry column, assign values "xcoord" and "ycoord" to arguments xcoord
  # and ycoord, respectively, and drop the geometry column from the object

  if ("sf" %in% class(dframe)) {
    temp <- st_coordinates(dframe)
    xcoord <- "xcoord"
    dframe$xcoord <- temp[, "X"]
    ycoord <- "ycoord"
    dframe$ycoord <- temp[, "Y"]
    dframe <- st_set_geometry(dframe, NULL)
  }

  # If the dframe argument is a tibble or does not belong to class
  # "data.frame", coerce the argument to class "data.frame"

  if ("tbl_df" %in% class(dframe) | !("data.frame" %in% class(dframe))) {
    dframe <- as.data.frame(dframe)
  }

  # Ensure that the dframe argument does not contain zero rows

  if (nrow(dframe) == 0) {
    stop("\nThe dframe argument contains zero rows.\n")
  }

  # Ensure that unused levels are dropped from factor variables in the dframe
  # data frame

  dframe <- droplevels(dframe)

  # If no siteID is provided, set one that assumes each row is a unique site

  if (is.null(siteID)) {
    siteID <- "siteID"
    dframe$siteID <- paste("site", seq_len(nrow(dframe)), sep = "-")
  }

  # Ensure that the dframe data frame contains the site ID variable

  if (!(siteID %in% names(dframe))) {
    ind <- FALSE
    error_ind <- TRUE
    msg <- paste0("The name provided for the siteID argument, \"", siteID, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  } else {
    ind <- TRUE
  }

  # Check site IDs for repeat values and, as necessary, create unique site IDs
  # and output a warning message

  if (ind) {
    IDs <- dframe[, siteID]
    temp <- sapply(split(IDs, IDs), length)
    if (any(temp > 1)) {
      warn_ind <- TRUE
      temp_str <- vecprint(names(temp)[temp > 1])
      warn <- paste("The following site ID values occur more than once among the values that were \ninput to the function:\n", temp_str)
      act <- "Unique site ID values were created.\n"
      warn_df <- rbind(warn_df, data.frame(
        func = I(fname), subpoptype = NA,
        subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
      ))
      dframe[, siteID] <- uniqueID(dframe[, siteID])
    }
  }

  # Ensure that the dframe data frame contains the survey weight variable

  if (!(weight %in% names(dframe))) {
    error_ind <- TRUE
    msg <- paste0("The name provided for the weight argument, \"", weight, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  }

  # Assign names to the variables required for calculation of the finite
  # population correction factor

  if (is.null(fpc)) {
    fpcfactor_ind <- FALSE
    fpcsize <- NULL
    Ncluster <- NULL
    stage1size <- NULL
  } else {
    fpcfactor_ind <- TRUE
    if (is.null(clusterID)) {
      fpcsize <- "fpcsize"
      Ncluster <- NULL
      stage1size <- NULL
    } else {
      fpcsize <- NULL
      Ncluster <- "Ncluster"
      stage1size <- "stage1size"
    }
  }

  # Create a list containing names of survey design variables

  design_names <- list(
    siteID = siteID,
    weight = weight,
    xcoord = xcoord,
    ycoord = ycoord,
    stratumID = stratumID,
    clusterID = clusterID,
    weight1 = weight1,
    xcoord1 = xcoord1,
    ycoord1 = ycoord1,
    sweight = sweight,
    sweight1 = sweight1,
    fpcsize = fpcsize,
    Ncluster = Ncluster,
    stage1size = stage1size
  )

  # Ensure that a value was provided for the vars_response (response variable
  # names) argument

  if (missing(vars_response)) {
    error_ind <- TRUE
    msg <- "A value must be provided for the vars_response (response variable names) argument.\n"
    error_vec <- c(error_vec, msg)
  }

  # Ensure that a value was provided for the vars_stressor (stressor variable
  # names) argument

  if (missing(vars_stressor)) {
    error_ind <- TRUE
    msg <- "A value must be provided for the vars_stressor (stressor variable names) argument.\n"
    error_vec <- c(error_vec, msg)
  }

  # If a value was not provided for the subpops (subpopulation names) argument,
  # assign the value "All_Sites" to the subpops argument and create a factor
  # named "All_Sites" in the dframe data frame that takes the value "All Sites"

  if (is.null(subpops)) {
    subpops <- "All_Sites"
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # If the user wants information for all sites together in addition to the
  # subpops, add the value "All_Sites" to the subpops argument and create a
  # factor named "All_Sites" in the dframe data frame that takes the value
  # "All Sites"

  if (!is.null(subpops) && All_Sites) {
    subpops <- c(subpops, "All_Sites")
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # Check input arguments

  temp <- input_check(dframe, design_names, vars_response, NULL, vars_stressor,
    NULL, subpops, sizeweight, fpc, popsize, vartype, NULL, conf,
    error_ind = error_ind, error_vec = error_vec
  )
  dframe <- temp$dframe
  vars_response <- temp$vars_cat
  vars_stressor <- temp$vars_stressor
  subpops <- temp$subpops
  popsize <- temp$popsize
  vartype <- temp$vartype
  error_ind <- temp$error_ind
  error_vec <- temp$error_vec

  # Check arguments response_levels and stressor_levels

  if (is.null(response_levels)) {
    response_levels <- rep(list(c("Poor", "Good")), length(vars_response))
    names(response_levels) <- vars_response
  } else {
    if (!is.list(response_levels)) {
      error_ind <- TRUE
      msg <- "Argument response_levels must be a list.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (length(response_levels) != length(vars_response)) {
        error_ind <- TRUE
        msg <- "Argument response_levels must be the same length as argument vars_response.\n"
        error_vec <- c(error_vec, msg)
      }
      if (is.null(names(response_levels))) { # set default names if none provided
        names(response_levels) <- vars_response
      }
      if (any(sapply(response_levels, function(x) length(x) != 2))) {
        error_ind <- TRUE
        msg <- "Each element of argument response_levels must contain only two values.\n"
        error_vec <- c(error_vec, msg)
      }
      if (any(sapply(response_levels, function(x) !is.character(x)))) {
        error_ind <- TRUE
        msg <- "Each element of argument response_levels must contain character values.\n"
        error_vec <- c(error_vec, msg)
      }
      if (all(vars_response %in% names(dframe))) {
        tst <- logical(length(vars_response))
        for (i in 1:length(vars_response)) {
          if (!all(response_levels[[i]] %in% levels(dframe[, vars_response[i]]))) {
            tst[i] <- TRUE
          }
        }
        if (any(tst)) {
          temp_str <- vecprint(vars_response[tst])
          error_ind <- TRUE
          msg <- paste0("\nCategory names for the following response variables do not match category names \nin the response_levels argument:\n", temp_str)
          error_vec <- c(error_vec, msg)
        } else {
          if (!all(names(response_levels) %in% vars_response)) {
            error_ind <- TRUE
            msg <- "Names for the response_levels list do not match the values in the vars_response \nargument."
            error_vec <- c(error_vec, msg)
          } else {
            indx <- match(vars_response, names(response_levels))
            response_levels <- response_levels[indx]
          }
        }
      }
    }
  }

  if (is.null(stressor_levels)) {
    stressor_levels <- rep(list(c("Poor", "Good")), length(vars_stressor))
    names(stressor_levels) <- vars_stressor
  } else {
    if (!is.list(stressor_levels)) {
      error_ind <- TRUE
      msg <- "Argument stressor_levels must be a list.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (length(stressor_levels) != length(vars_stressor)) {
        error_ind <- TRUE
        msg <- "Argument stressor_levels must be the same length as argument vars_stressor.\n"
        error_vec <- c(error_vec, msg)
      }
      if (is.null(names(stressor_levels))) { # set default names if none provided
        names(stressor_levels) <- vars_stressor
      }
      if (any(sapply(stressor_levels, function(x) length(x) != 2))) {
        error_ind <- TRUE
        msg <- "Each element of argument stressor_levels must contain only two values.\n"
        error_vec <- c(error_vec, msg)
      }
      if (any(sapply(stressor_levels, function(x) !is.character(x)))) {
        error_ind <- TRUE
        msg <- "Each element of argument stressor_levels must contain character values.\n"
        error_vec <- c(error_vec, msg)
      }
      if (all(vars_stressor %in% names(dframe))) {
        tst <- logical(length(vars_stressor))
        for (i in 1:length(vars_stressor)) {
          if (!all(stressor_levels[[i]] %in% levels(dframe[, vars_stressor[i]]))) {
            tst[i] <- TRUE
          }
        }
        if (any(tst)) {
          temp_str <- vecprint(vars_stressor[tst])
          error_ind <- TRUE
          msg <- paste0("\nCategory names for the following stressor variables do not match category names \nin the stressor_levels argument:\n", temp_str)
          error_vec <- c(error_vec, msg)
        } else {
          if (!all(names(stressor_levels) %in% vars_stressor)) {
            error_ind <- TRUE
            msg <- "Names for the stressor_levels list do not match the values in the vars_stressor \nargument."
            error_vec <- c(error_vec, msg)
          } else {
            indx <- match(vars_stressor, names(stressor_levels))
            stressor_levels <- stressor_levels[indx]
          }
        }
      }
    }
  }

  # As necessary, output a message indicating that error messages were generated
  # during execution of the program

  if (error_ind) {
    error_vec <<- error_vec
    if (length(error_vec) == 1) {
      message("During execution of the program, an error message was generated.  The error \nmessage is stored in a vector named 'error_vec'.  Enter the following command \nto view the error message: errorprnt()\n")
    } else {
      message(paste("During execution of the program,", length(error_vec), "error messages were generated.  The error \nmessages are stored in a vector named 'error_vec'.  Enter the following \ncommand to view the error messages: errorprnt()\n"))
    }

    if (warn_ind) {
      warn_df <<- warn_df
      if (nrow(warn_df) == 1) {
        message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
      } else {
        message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
      }
    }
    stop("See the preceding message(s).")
  }

  # Assign a logical value to the indicator variable for a stratified sample

  stratum_ind <- !is.null(stratumID)

  # For a stratified sample, remove strata that contain a single site

  if (stratum_ind) {
    dframe[, stratumID] <- factor(dframe[, stratumID])
    stratum_levels <- levels(dframe[, stratumID])
    nstrata <- length(stratum_levels)
    ind <- FALSE
    for (i in 1:nstrata) {
      tst <- dframe[, stratumID] == stratum_levels[i]
      if (sum(tst) == 1) {
        warn_ind <- TRUE
        warn <- paste("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
        act <- "Stratum was removed from the analysis.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = NA,
          subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
        ))
        dframe <- dframe[!tst, ]
        ind <- TRUE
      }
    }
    if (ind) {
      dframe[, stratumID] <- factor(dframe[, stratumID])
      stratum_levels <- levels(dframe[, stratumID])
      nstrata <- length(stratum_levels)
    }
  }

  # Assign a logical value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Create the survey design object

  design <- survey_design(
    dframe, siteID, weight, stratum_ind, stratumID, cluster_ind, clusterID,
    weight1, sizeweight, sweight, sweight1, fpcfactor_ind, fpcsize, Ncluster,
    stage1size, vartype, NULL
  )

  # If popsize is not equal to NULL, then call either the postStratify or
  # calibrate function, as appropriate

  if (!is.null(popsize)) {
    if (all(class(popsize) %in% c("data.frame", "table", "xtabs"))) {
      if ("data.frame" %in% class(popsize)) {
        pnames <- names(popsize)[-ncol(popsize)]
      } else {
        pnames <- names(dimnames(popsize))
      }
      design <- postStratify(design, make.formula(pnames), popsize)
    } else {
      cnames <- cal_names(make.formula(names(popsize)), design)
      pop_totals <- numeric(length(cnames))
      names(pop_totals) <- cnames
      pop_totals[1] <- sum(popsize[[1]])
      k <- 2
      for (i in names(popsize)) {
        temp <- popsize[[i]]
        for (j in 2:length(temp)) {
          pop_totals[k] <- temp[j]
          k <- k + 1
        }
      }
      design <- calibrate(design, make.formula(names(popsize)), pop_totals)
    }
  }

  # If popsize is not equal to NULL and vartype equals "Local", then assign
  # adjusted weights to the appropriate weight variable(s) in the
  # design$variables data frame

  if (!is.null(popsize) && vartype == "Local") {
    if (cluster_ind) {
      design$variables$wgt2 <- weights(design) / design$variables$wgt1
    } else {
      design$variables$wgt <- weights(design)
    }
  }

  # For variables that exist in the design$variables data frame, assign survey
  # design variables

  dframe <- design$variables
  for (i in names(design_names)) {
    if (is.null(design_names[[i]])) {
      eval(parse(text = paste0(i, " <- NULL")))
    } else {
      eval(parse(text = paste0(i, " <- dframe[, \"", design_names[[i]], "\"]")))
    }
  }

  # Assign values to weight variables

  if (cluster_ind) {
    wgt1 <- dframe$wgt1
    wgt2 <- dframe$wgt2
  } else {
    wgt <- dframe$wgt
  }

  # Create the arsum (results) data frame

  arsum <- NULL

  # Assign the confidence bound multiplier

  mult <- qnorm(0.5 + (conf / 100) / 2)

  # Loop through all subpopulations (domains)

  for (itype in subpops) {
    lev_itype <- levels(dframe[, itype])
    nlev_itype <- length(lev_itype)

    # Loop through all response variables (vars_response)

    for (ivar_r in vars_response) {

      # Loop through all stressor variables (vars_stressor)

      for (ivar_s in vars_stressor) {

        # Loop through all levels of the subpopulation

        for (isubpop in lev_itype) {
          tst <- !is.na(dframe[, itype]) & dframe[, itype] == isubpop &
            !is.na(dframe[, ivar_r]) & !is.na(dframe[, ivar_s])

          # Compute sum of the weights

          if (cluster_ind) {
            popsize_hat <- sum(wgt1[tst] * wgt2[tst])
          } else {
            popsize_hat <- sum(wgt[tst])
          }

          # Assign response variable values

          response <- dframe[, ivar_r]
          nresp <- length(response)

          # Assign stressor variable values

          stressor <- dframe[, ivar_s]

          # Create the warn_vec object

          warn_vec <- c(itype, isubpop, paste(ivar_r, "and", ivar_s))

          #
          # Branch to handle stratified and unstratified data
          #

          if (stratum_ind) {

            #
            # Begin the section for stratified data
            #

            # Initialize variables for all strata combined

            wgt_total <- 0
            varest <- 0

            #
            # Begin the subsection for individual strata
            #

            for (i in 1:nstrata) {

              # Calculate required values

              stratum_i <- tst & stratumID == stratum_levels[i]
              response_st <- response[stratum_i]
              stressor_st <- stressor[stratum_i]
              if (cluster_ind) {
                wgt1_st <- wgt1[stratum_i]
                wgt2_st <- wgt2[stratum_i]
              } else {
                wgt_st <- wgt[stratum_i]
              }

              # Compute the 2x2 table of weight totals

              wgt_total_st <- svytable(make.formula(paste(
                ivar_r, "+",
                ivar_s
              )), design = subset(design, stratum_i))

              # Calculate the variance-covariance estimate for the cell and
              # marginal totals

              if (cluster_ind) {
                temp <- attrisk_var(
                  response[stratum_i], stressor[stratum_i],
                  response_levels[[ivar_r]], stressor_levels[[ivar_s]],
                  wgt2[stratum_i], xcoord[stratum_i], ycoord[stratum_i],
                  stratum_ind, stratum_levels[i], cluster_ind,
                  clusterID[stratum_i], wgt1[stratum_i], xcoord1[stratum_i],
                  ycoord1[stratum_i], vartype, warn_ind, warn_df, warn_vec
                )
              } else {
                temp <- attrisk_var(response[stratum_i], stressor[stratum_i],
                  response_levels[[ivar_r]], stressor_levels[[ivar_s]],
                  wgt[stratum_i], xcoord[stratum_i], ycoord[stratum_i],
                  stratum_ind, stratum_levels[i], cluster_ind,
                  vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                  warn_vec = warn_vec
                )
              }
              varest.st <- temp$varest
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df

              # Add estimates to the variables for all strata combined

              wgt_total <- wgt_total + wgt_total_st
              varest <- varest + varest.st

              #
              # End the subsection for individual strata
              #
            }

            # Add margins to the wgt_total table

            wgt_total <- addmargins(wgt_total)

            # Assign cell weight totals

            total1 <- wgt_total[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][1]
            ]
            total2 <- wgt_total[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][1]
            ]
            total3 <- wgt_total[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][2]
            ]
            total4 <- wgt_total[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][2]
            ]

            # Calculate the estimate of attributable risk for all strata
            # combined

            if (total2 == 0 || total4 == 0) {
              ar <- NA
              warn_ind <- TRUE
              temp <- ifelse(total2 == 0, stressor_levels[[ivar_s]][1], stressor_levels[[ivar_s]][2])
              warn <- paste("Since there are no observations for level \"", temp, "\" of the stressor \nvariable, the attributable risk estimate and its standard error cannot be \ncalculated for stratum \"", stratum_levels[i], "\".  Also, the stratum \nwas removed from the analysis.\n", sep = "")
              act <- paste("The attributable risk estimate and its standard error were not calculated for \nstratum \"", stratum_levels[i], "\".  Also, the stratum was removed from \nthe analysis.\n", sep = "")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if (total1 == 0 && total3 != 0) {
              ar <- 0
              warn_ind <- TRUE
              warn <- paste("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][1], "\" of the stressor \nvariable, the attributable risk estimate is zero and standard error of the attributable \nrisk estimate cannot be calculated for stratum \"", stratum_levels[i], "\".  \nAlso, the stratum was removed from the analysis.\n", sep = "")
              act <- paste("Standard error of the attributable risk estimate was not calculated for stratum \n\"", stratum_levels[i], "\".  Also, the stratum was removed from the \nanalysis.\n", sep = "")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if (total1 == 0 && total3 == 0) {
              ar <- NA
              warn_ind <- TRUE
              warn <- paste("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][1], "\" of the stressor \nvariable and for the cell defined by level \"", response_levels[[ivar_r]][1], "\" of the \nresponse variable and level \"", stressor_levels[[ivar_s]][2], "\" of the stressor variable, \nthe attributable risk estimate and its standard error cannot be calculated for \nstratum \"", stratum_levels[i], "\".  Also, the stratum was removed from \nthe analysis.\n", sep = "")
              act <- paste("The attributable risk estimate and its standard error were not calculated for \nstratum \"", stratum_levels[i], "\".  Also, the stratum was removed from \nthe analysis.\n", sep = "")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if (total3 == 0) {
              ar <- NA
              warn_ind <- TRUE
              warn <- paste("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][2], "\" of the stressor \nvariable, the attributable risk estimate and its standard error cannot be \ncalculated for stratum \"", stratum_levels[i], "\".  Also, the stratum \nwas removed from the analysis.\n", sep = "")
              act <- paste("The attributable risk estimate and its standard error were not calculated for \nstratum \"", stratum_levels[i], "\".  Also, the stratum was removed from \nthe analysis.\n", sep = "")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else {
              theta <- (popsize_hat * total3) / ((total1 + total3) * (total3 +
                total4))
              ar <- 1 - theta
            }

            # Calculate the standard error estimate of the log of attributable
            # risk for all strata combined

            if (total3 == 0 || (total1 + total3) == 0 || (total3 + total4) == 0) {
              arlog_se <- NA
            } else {
              pder <- numeric(4)
              pder[1] <- 1 / popsize_hat - 1 / (total1 + total3)
              pder[2] <- 1 / popsize_hat
              pder[3] <- 1 / popsize_hat + 1 / total3 - 1 / (total1 + total3) -
                1 / (total3 + total4)
              pder[4] <- 1 / popsize_hat - 1 / (total3 + total4)
              pder <- 1 / c(total1, -total2, -total3, total4)
              arlog_se <- sqrt(t(pder) %*% varest %*% pder)
            }

            #
            # End the section for stratified data
            #
          } else {

            #
            # Begin the section for unstratified data
            #

            # Check whether the vector of response values contains a single
            # element

            if (nresp == 1) {
              stop("\nEstimates cannot be calculated since the vector of response values contains a \nsingle element.")
            }

            # Compute the 2x2 table of weight totals

            wgt_total <- addmargins(svytable(
              make.formula(paste(ivar_r, "+", ivar_s)),
              design = subset(design, tst)
            ))

            # Assign cell weight totals

            total1 <- wgt_total[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][1]
            ]
            total2 <- wgt_total[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][1]
            ]
            total3 <- wgt_total[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][2]
            ]
            total4 <- wgt_total[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][2]
            ]

            # Calculate the estimate of attributable risk

            if (total3 == 0 & total1 != 0 & total4 != 0) {
              ar <- 0
              warn_ind <- TRUE
              warn <- paste("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][2], "\" of the stressor variable, \nthe attributable risk estimate is zero and standard error of the attributable risk \nestimate cannot be calculated.\n", sep = "")
              act <- "Standard error of the attributable risk estimate was not calculated.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if ((total1 + total3) == 0) {
              ar <- NA
              warn_ind <- TRUE
              warn <- paste("Since there are no observations for level \"", response_levels[[ivar_r]][1], "\" of the response \nvariable, the attributable risk estimate and its standard error cannot be \ncalculated.\n", sep = "")
              act <- "The attributable risk estimate and its standard error were not calculated.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if ((total3 + total4) == 0) {
              ar <- NA
              warn_ind <- TRUE
              warn <- paste("Since there are no observations for level \"", stressor_levels[[ivar_s]][2], "\" of the stressor \nvariable, the attributable risk estimate and its standard error cannot be \ncalculated.\n", sep = "")
              act <- "The attributable risk estimate and its standard error were not calculated.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else {
              theta <- (popsize_hat * total3) / ((total1 + total3) * (total3 +
                total4))
              ar <- 1 - theta
            }

            # Determine whether the standard error can be calculated

            if (total3 == 0 || (total1 + total3) == 0 || (total3 + total4) == 0) {
              arlog_se <- NA
            } else {

              # Calculate the variance-covariance estimate for the cell totals

              if (cluster_ind) {
                temp <- attrisk_var(
                  response[tst], stressor[tst],
                  response_levels[[ivar_r]], stressor_levels[[ivar_s]],
                  wgt2[tst], xcoord[tst], ycoord[tst], stratum_ind, NULL,
                  cluster_ind, clusterID[tst], wgt1[tst], xcoord1[tst],
                  ycoord1[tst], vartype, warn_ind, warn_df, warn_vec
                )
              } else {
                temp <- attrisk_var(response[tst], stressor[tst],
                  response_levels[[ivar_r]], stressor_levels[[ivar_s]],
                  wgt[tst], xcoord[tst], ycoord[tst], stratum_ind, NULL,
                  cluster_ind,
                  vartype = vartype, warn_ind = warn_ind,
                  warn_df = warn_df, warn_vec = warn_vec
                )
              }
              varest <- temp$varest
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df

              # Calculate the standard error estimate of the log of theta

              pder <- numeric(4)
              pder[1] <- 1 / popsize_hat - 1 / (total1 + total3)
              pder[2] <- 1 / popsize_hat
              pder[3] <- 1 / popsize_hat + 1 / total3 - 1 / (total1 + total3) -
                1 / (total3 + total4)
              pder[4] <- 1 / popsize_hat - 1 / (total3 + total4)
              arlog_se <- sqrt(t(pder) %*% varest %*% pder)
            }

            #
            # End section for unstratified data
            #
          }

          # Calculate confidence limits for the estimate of attributable risk

          if (is.na(arlog_se)) {
            cl <- NA
          } else {
            cl <- c(
              1 - exp(log(theta) + arlog_se * mult),
              1 - exp(log(theta) - arlog_se * mult)
            )
          }

          # Calculate the table of cell and margin counts

          cc <- addmargins(table(list(
            response = response[tst],
            stressor = stressor[tst]
          )))

          # Calculate the table of cell proportion estimates

          cp <- wgt_total / wgt_total["Sum", "Sum"]

          # Append results to the arsum data frame

          arsum <- rbind(arsum, data.frame(
            Type = itype,
            Subpopulation = isubpop,
            Response = ivar_r,
            Stressor = ivar_s,
            nResp = cc[3, 3],
            Estimate = ar,
            StdError_log = arlog_se,
            MarginofError_log = mult * arlog_se,
            LCB = cl[1],
            UCB = cl[2],
            WeightTotal = wgt_total["Sum", "Sum"],
            CellCounts_11 = cc[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][1]
            ],
            CellCounts_12 = cc[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][2]
            ],
            CellCounts_21 = cc[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][1]
            ],
            CellCounts_22 = cc[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][2]
            ],
            CellProportions_11 = cp[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][1]
            ],
            CellProportions_12 = cp[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][2]
            ],
            CellProportions_21 = cp[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][1]
            ],
            CellProportions_22 = cp[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][2]
            ]
          ))

          # End of the loop for levels of the subpopulation
        }

        # End of the loop for stressor variables
      }

      # End of the loop for response variables
    }

    # End of the loop for subpopulations
  }

  # As necessary, output a message indicating that warning messages were
  # generated during execution of the program

  if (warn_ind) {
    warn_df <<- warn_df
    if (nrow(warn_df) == 1) {
      message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
    } else {
      message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
    }
  }

  # Assign dimension names to the arsum data frame

  cell_lbl <- character(4)
  cell_lbl[1] <- "RespPoor_StressPoor"
  cell_lbl[2] <- "RespPoor_StressGood"
  cell_lbl[3] <- "RespGood_StressPoor"
  cell_lbl[4] <- "RespGood_StressGood"
  dimnames(arsum) <- list(1:nrow(arsum), c(
    "Type", "Subpopulation", "Response", "Stressor", "nResp", "Estimate",
    "StdError_log", "MarginofError_log", paste0("LCB", conf, "Pct"),
    paste0("UCB", conf, "Pct"), "WeightTotal", paste0("Count_", cell_lbl[1]),
    paste0("Count_", cell_lbl[2]), paste0("Count_", cell_lbl[3]),
    paste0("Count_", cell_lbl[4]), paste0("Prop_", cell_lbl[1]),
    paste0("Prop_", cell_lbl[2]), paste0("Prop_", cell_lbl[3]),
    paste0("Prop_", cell_lbl[4])
  ))

  # Return the arsum data frame

  arsum
}

###############################################################################
# Function: get.address (not exported)
# Programmer: Guillaume Chauvet
# Adapted by: Marc Weber
# Date: September 2, 2020
#
#' Get Hierarchical Address
#'
#' This function creates a tessellation of the two dimensional space, attributes
#' an address to each unit in the population and sorts the sampling frame in the
#' order given by the addresses. This function was used directly from Guillaume Chauvet and Ronan Le Gleut
#' as they describe in their paper Inference under pivotal sampling: Properties,
#' variance estimation, and application to tessellation for spatial sampling,
#' Scand J Statist. 2020;1–24.
#'
#' GET ADDRESS: function to attribute an address to each unit and to sort the
#' sampling frame. The parameter rand allows a randomization of the tessellation.
#' The output is the order in which the sampling frame should be sorted.
#'
#' BETWEEN: function to project the coordinates between two bounds low and up.
#'
#' @param x The x-coordinate value in the sample frame of points.
#'
#' @param y The y-coordinate value in the sample frame of points.
#'
#' @param rand Whether or not to randomize the addressing.  Default is \code{TRUE}
#'
#' @return A sorted sample frame.
#'
#' @author Marc Weber \email{Weber.Marc@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
###############################################################################


get_address <- function(x, y, rand = TRUE) {
  x <- trunc(between(x, 0, 2^31 - 1))
  y <- trunc(between(y, 0, 2^31 - 1))
  if (!rand) {
    a <- 0L
    b <- 1L
    c <- 2L
    d <- 3L
  } else {
    t <- sapply(1:32, function(i) sample(0:3))
    a <- rep(t[1, ], length(x))
    b <- rep(t[2, ], length(x))
    c <- rep(t[3, ], length(x))
    d <- rep(t[4, ], length(x))
  }
  x <- as.logical(rev(intToBits(x)))
  y <- as.logical(rev(intToBits(y)))
  t <- a * (!x & !y) + b * (!x & y) + c * (x & !y) + d * (x & y)
  l <- lapply(2:32L, function(i) {
    rev(t[seq(i, length(t), by = 32L)])
  })
  do.call(order, l)
}

between <- function(x, low = 0, up = 1) {
  minx <- min(x, na.rm = TRUE)
  maxx <- max(x, na.rm = TRUE)
  low + (up - low) * (x - minx) / (maxx - minx)
}

###############################################################################
# Function: globalVariables (exported)
# Programmers: Tom Kincaid
# Date: January 22, 2021
#' Define Global Variables
#'
#' @noRd
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    "siteID", "weight", "xcoord", "ycoord", "stratumID",
    "clusterID", "weight1", "xcoord1", "ycoord1", "sweight", "sweight1",
    "fpcsize", "Ncluster", "stage1size", "Category", "Statistic", "rowvar",
    "colvar", "regest", "df.residual", "pt", "qt"
  ))
}

################################################################################
# Function: total_localmean (not exported)
# Programmer: Tom Kincaid
# Date: October 25, 2021
#
#' Local total Variance Estimates of the Estimated Total
#'
#' This function organizes input and output for calculation of the local mean
#' variance estimator for the estimated total.
#'
#' @param itype Character value that identifies a factor variable in the design
#'   argument containing subpopulation (domain) values.
#'
#' @param lev_itype Character vector that provides levels of the subpopulation
#'   variable.
#'
#' @param nlev_itype Numeric value that provides the number of levels of the
#'   subpopulation variable.
#'
#' @param levs Numeric vector that is used to select subpopulation levels
#'
#' @param ivar Character value that identifies a factor variable in the design
#'   argument containing categorical response values.
#'
#' @param design Object of class \code{survey.design} that specifies a complex
#'   survey design.
#'
#' @param design_names Character vector that provides names of survey design
#'   variables in the \code{design} argument.
#'
#' @param totalest Vector that provides estimates of the total.
#'
#' @param mult Numeric value that provides the Normal distribution confidence
#'   bound multiplier.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @return A list containing the following objects:
#'   \describe{
#'     \item{\code{stderr}}{vector containing standard error estimates}
#'     \item{\code{confval}}{data frame containing confidence bound estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#' @section Other Functions Required:
#'   \describe{
#'     \item{\code{total_var}}{calculates variance estimate of the
#'       estimated total}
#'     \item{\code{\link{svytotal}}}{calculates the total for a complex survey
#'       design}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@epa.gov}
#'
#' @seealso
#'   \code{\link{svytotal}}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

total_localmean <- function(itype, lev_itype, nlev_itype, levs, ivar, design,
                            design_names, totalest, mult, warn_ind, warn_df) {

  # Assign a value to the function name variable

  fname <- "total_localmean"

  # For variables that exist in the design$variables data frame, assign survey
  # design variables

  dframe <- design$variables
  for (i in names(design_names)) {
    if (is.null(design_names[[i]])) {
      eval(parse(text = paste0(i, " <- NULL")))
    } else {
      eval(parse(text = paste0(i, " <- dframe[, \"", design_names[[i]], "\"]")))
    }
  }

  # Assign values to the continuous response variable vector, contvar

  contvar <- dframe[, ivar]

  # Assign a value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Assign values to weight variables

  if (cluster_ind) {
    wgt1 <- dframe$wgt1
    wgt2 <- dframe$wgt2
  } else {
    wgt <- dframe$wgt
  }

  # Create the output objects for standard error estimates and confidence bound
  # estimates

  nlevs <- length(levs)
  stderr <- numeric(nlev_itype)
  names(stderr) <- lev_itype
  confval <- data.frame(array(0, c(nlev_itype, 2)))
  dimnames(confval) <- list(lev_itype, c("LCB", "UCB"))

  # Loop through all subpopulations

  for (isubpop in levs) {
    tst <- !is.na(dframe[, ivar]) &
      (dframe[, itype] %in% lev_itype[isubpop])

    # Assign values to the warn_vec vector

    warn_vec <- c(itype, lev_itype[isubpop], ivar)

    # Assign a value to the indicator variable for a stratified sample

    stratum_ind <- !is.null(stratumID)

    # For a stratified design, determine whether the subpopulation contains a
    # single stratum

    if (stratum_ind) {
      stratum <- factor(stratumID[tst])
      stratum_levels <- levels(stratum)
      nstrata <- length(stratum_levels)
      if (nstrata == 1) {
        stratum_ind <- FALSE
      }
    }

    # Branch for a stratified sample

    if (stratum_ind) {

      # Calculate values required for weighting strata

      if (cluster_ind) {
        popsize_hat <- tapply(wgt1[tst] * wgt2[tst], stratum, sum)
        sum_popsize_hat <- sum(wgt1[tst] * wgt2[tst])
      } else {
        popsize_hat <- tapply(wgt[tst], stratum, sum)
        sum_popsize_hat <- sum(wgt[tst])
      }

      # Begin the subsection for individual strata

      for (i in 1:nstrata) {

        # Calculate total estimates for the stratum

        stratum_i <- tst & stratumID == stratum_levels[i]
        totalest_st <- svytotal(make.formula(ivar),
          design = subset(design, stratum_i),
          na.rm = TRUE
        )

        # Calculate variance estimates

        if (cluster_ind) {
          temp <- total_var(
            contvar[stratum_i], wgt2[stratum_i], xcoord[stratum_i],
            ycoord[stratum_i], stratum_ind, stratum_levels[i], cluster_ind,
            clusterID[stratum_i], wgt1[stratum_i], xcoord1[stratum_i],
            ycoord1[stratum_i], warn_ind, warn_df, warn_vec
          )
        } else {
          temp <- total_var(
            contvar[stratum_i], wgt[stratum_i], xcoord[stratum_i],
            ycoord[stratum_i], stratum_ind, stratum_levels[i], cluster_ind,
            warn_ind = warn_ind, warn_df = warn_df, warn_vec = warn_vec
          )
        }
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
        if (temp$varest < 0) {
          temp$vartype <- "SRS"
          warn_ind <- TRUE
          act <- "The simple random sampling variance estimator for an infinite population was used.\n"
          warn <- paste0("The local total variance estimator produced a  negative variance estimate in stratum \n\"", stratum_levels[i], "\", the simple random sampling variance estimator for an infinite \npopulation was used to calculate variance of the total estimate.\n")
          warn_df <-
            rbind(
              warn_df,
              data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = I(stratum_levels[i]),
                warning = I(warn), action = I(act)
              )
            )
        }
        if (temp$vartype == "SRS") {
          rslt_svy <- svytotal(make.formula(ivar),
            design = subset(design, tst),
            na.rm = TRUE
          )
          varest <- SE(rslt_svy)^2
        } else {
          varest <- temp$varest
        }

        # Add estimate to the stderr vector

        stderr[isubpop] <- stderr[isubpop] +
          ((popsize_hat[i] / sum_popsize_hat)^2) * varest

        # End the subsection for individual strata
      }

      # Begin the subsection for all strata combined

      # Add estimates to the data frames for results

      stderr[isubpop] <- sqrt(stderr[isubpop])
      lbound <- totalest[isubpop] - mult * stderr[isubpop]
      ubound <- totalest[isubpop] + mult * stderr[isubpop]
      confval[isubpop, ] <- c(lbound, ubound)

      # End the subsection for all strata combined

      # Branch for an unstratified sample
    } else {

      # Calculate the standard error estimates

      if (cluster_ind) {
        temp <- total_var(
          contvar[tst], wgt2[tst], xcoord[tst], ycoord[tst],
          stratum_ind, NULL, cluster_ind, clusterID[tst], wgt1[tst],
          xcoord1[tst], ycoord1[tst], warn_ind, warn_df, warn_vec
        )
      } else {
        temp <- total_var(contvar[tst], wgt[tst], xcoord[tst], ycoord[tst],
          stratum_ind, NULL, cluster_ind,
          warn_ind = warn_ind,
          warn_df = warn_df, warn_vec = warn_vec
        )
      }
      warn_ind <- temp$warn_ind
      warn_df <- temp$warn_df
      if (temp$varest < 0) {
        temp$vartype <- "SRS"
        warn_ind <- TRUE
        act <- "The simple random sampling variance estimator for an infinite population was used.\n"
        warn <- paste0("The local total variance estimator produced a  negative variance estimate, the simple \nrandom sampling variance estimator for an infinite population was used to calculate \nvariance of the total estimate.\n")
        warn_df <-
          rbind(
            warn_df,
            data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            )
          )
      }
      if (temp$vartype == "SRS") {
        rslt_svy <- svytotal(make.formula(ivar),
          design = subset(design, tst),
          na.rm = TRUE
        )
        sdest <- SE(rslt_svy)
      } else {
        sdest <- sqrt(temp$varest)
      }

      # Calculate confidence bounds

      lbound <- totalest[isubpop] - mult * sdest
      ubound <- totalest[isubpop] + mult * sdest

      # Add estimates to the output objects

      stderr[isubpop] <- sdest
      confval[isubpop, ] <- c(lbound, ubound)
    }
  }

  # Return results

  list(
    stderr = stderr,
    confval = confval,
    warn_ind = warn_ind,
    warn_df = warn_df
  )
}

################################################################################
# Function: cdftest_localmean_prop (not exported)
# Programmer: Tom Kincaid
# Date: October 22, 2020
# Revised: November 2, 2020 to correctly process the column variable when it
#          includes missing (NA) values
# Revised: May 27 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
#
#' Local Mean Variance/Covarince Estimates of Estimated Population Proportions
#'
#' This function organizes input and output for calculation of the local mean
#' variance/covariance estimator for estimated proportions for categorical data.
#'
#' @param design Object of class \code{survey.design} that specifies a complex
#'   survey design.
#'
#' @param design_names Character vector that provides names of survey design
#'   variables in the design argument.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @param warn_vec Character vector that contains a subpopulation name, the
#'   first subpopulation level, the second subpopulation level, and an indicator
#'   name.
#'
#' @return A list containing the following objects:
#'   \itemize{
#'     \item{\code{varest}}{matrix containing the variance/covariance estimates
#'       for the contingency table proportion estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

cdftest_localmean_prop <- function(design, design_names, warn_ind, warn_df,
                                   warn_vec) {

  # Assign a value to the function name variable

  fname <- "cdftest_localmean_prop"

  # For variables that exist in the design$variables data frame, assign survey
  # design variables

  dframe <- subset(design$variables, !(is.na(rowvar) | is.na(colvar)))
  for (i in names(design_names)) {
    if (is.null(design_names[[i]])) {
      eval(parse(text = paste0(i, " <- NULL")))
    } else {
      eval(parse(text = paste0(i, " <- dframe[, \"", design_names[[i]], "\"]")))
    }
  }

  # Assign a value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Assign values to weight variables

  if (cluster_ind) {
    wgt1 <- dframe$wgt1
    wgt2 <- dframe$wgt2
  } else {
    wgt <- dframe$wgt
  }

  # Assign a value to the indicator variable for a stratified sample

  stratum_ind <- !is.null(stratumID)

  # For a stratified design, determine whether the subpopulation contains a
  # single stratum

  if (stratum_ind) {
    stratum <- factor(stratumID)
    stratum_levels <- levels(stratum)
    nstrata <- length(stratum_levels)
    if (nstrata == 1) {
      stratum_ind <- FALSE
    }
  }

  # Branch for a stratified sample

  if (stratum_ind) {

    # Calculate values required for weighting strata

    if (cluster_ind) {
      popsize_hat <- tapply(wgt1 * wgt2, stratum, sum)
      sum_popsize_hat <- sum(wgt1 * wgt2)
    } else {
      popsize_hat <- tapply(wgt, stratum, sum)
      sum_popsize_hat <- sum(wgt)
    }

    # Create the varest matrix

    m <- with(dframe, length(levels(rowvar)) * length(levels(colvar)))
    varest <- matrix(0, m, m)
    temp <- paste(
      "interaction(factor(rowvar), factor(colvar))Subpopulation",
      1:2
    )
    colnames_varest <- paste(rep(temp, length(levels(dframe$colvar))),
      rep(levels(dframe$colvar), each = 2),
      sep = "."
    )
    colnames(varest) <- colnames_varest

    # Calculate variance estimates

    for (i in 1:nstrata) {
      temp <- design_names$stratumID
      tst <- design$variables[, temp] != stratum_levels[i]
      design_temp <- design
      design_temp$variables$rowvar[tst] <- NA
      stratum_i <- stratumID == stratum_levels[i]
      if (cluster_ind) {
        temp <- cdftestvar_prop(
          design_temp, wgt2[stratum_i], xcoord[stratum_i], ycoord[stratum_i],
          stratum_ind, stratum_levels[i], cluster_ind, clusterID[stratum_i],
          wgt1[stratum_i], xcoord1[stratum_i], ycoord1[stratum_i], warn_ind,
          warn_df, warn_vec
        )
      } else {
        temp <- cdftestvar_prop(
          design_temp, wgt[stratum_i], xcoord[stratum_i], ycoord[stratum_i],
          stratum_ind, stratum_levels[i], cluster_ind,
          warn_ind = warn_ind,
          warn_df = warn_df, warn_vec = warn_vec
        )
      }
      varest_st <- temp$varest
      warn_ind <- temp$warn_ind
      warn_df <- temp$warn_df

      # Add estimates to the varest matrix

      tst <- colnames_varest %in% colnames(varest_st)
      varest[tst, tst] <- varest[tst, tst] +
        ((popsize_hat[i] / sum_popsize_hat)^2) * varest_st

      # End the loop for strata
    }

    # Create the results list

    results <- list(
      varest = varest,
      warn_ind = warn_ind,
      warn_df = warn_df
    )

    # Branch for an unstratified sample
  } else {

    # Calculate the variance/covariance estimates

    if (cluster_ind) {
      results <- cdftestvar_prop(
        design, wgt2, xcoord, ycoord, stratum_ind, NULL, cluster_ind, clusterID,
        wgt1, xcoord1, ycoord1, warn_ind, warn_df, warn_vec
      )
    } else {
      results <- cdftestvar_prop(
        design, wgt, xcoord, ycoord, stratum_ind, NULL, cluster_ind,
        warn_ind = warn_ind, warn_df = warn_df, warn_vec = warn_vec
      )
    }
  }

  # Return results

  results
}

#' ###############################################################################
#' # Function: startup (not exported)
#' # Programmer: Tom Kincaid
#' # Date: July 17, 2002
#' # Last Revised: February 25, 2004
#' #
#' #' Successful startup message
#' #'
#' #' This is a function that displays a startup message.
#' #'
#' #' @param libname Data library name.
#' #' @param pkgname Package name.
#' #' @return A startup message
#' #' @noRd
#' .onAttach <- function(libname, pkgname) {
#'   packageStartupMessage(
#'     "spsurvey version 5.0.0 introduced significant changes to the inputs and outputs of many functions. Please review the updated materials, vignettes, and documentation by visiting \n https://cran.r-project.org/package=spsurvey"
#'   )
#' }

###############################################################################
# Function: vecprint (not exported)
# Programmer: Tom Kincaid
# Date: February 6, 2004
# Last Revised:  May 3, 2004
#
#' Internal Function: Create Vector to Print
#'
#' This function takes an input vector and outputs a character string with line
#' breaks inserted so that, whenever possible, no line in the string exceeds the
#' input value n_char, which is set to 78 characters by default.  The input
#' vector is coereced to mode character.  When an element of the input vector is
#' greater than n_char characters in length, then that element is inserted in
#' the output character string as an individual line.
#'
#' @param x Character vector.
#'
#' @param n_char The maximum number of characters per line.  The default is \code{78}.
#'
#' @return  Character string that is suitable for printing by the functions:
#'   \code{stop}, \code{warning}, or \code{cat}.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @examples
#' sites <- paste("Site Number", 1:50)
#' sites_str <- vecprint(sites)
#' cat(sites_str)
#'
#' temp <- c(1, 5, 21:25, 33:37)
#' sites_str <- vecprint(sites[temp])
#' warning(paste("\nThe following site ID values were removed from the
#'   analysis:\n", sites_str, sep = ""))
#' @noRd
###############################################################################

vecprint <- function(x, n_char = 78) {
  x <- as.character(x)
  n <- length(x)
  nc <- nchar(x)

  i <- 1
  j <- 1
  nc_sum <- 0
  x_str <- character()
  while (j <= n) {
    if ((nc_sum + nc[j]) <= n_char) {
      nc_sum <- nc_sum + nc[j] + 2
      j <- j + 1
    } else {
      if (i == j) {
        x_str <- paste(x_str, x[j], "\n")
      } else {
        j <- j - 1
        x_str <- paste(x_str, paste(x[i:j], collapse = ", "), "\n")
      }
      j <- j + 1
      i <- j
      nc_sum <- 0
    }
  }
  if (i < j) {
    j <- j - 1
    x_str <- paste(x_str, paste(x[i:j], collapse = ", "), "\n")
  }

  x_str
}

################################################################################
# Function: percentile_est (not exported)
# Programmer: Tom Kincaid
# Date: July 23, 2020
# Revised: May 20, 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
# Revised: June 11, 2021 to eliminate estimating means, which will be performed
#          by a new function named mean_est
# Revised: September 9, 2021 to remove argument vartype, which is not used
# Revised: September 17 2021 to set argument ties equal to "rounded" when
#          calling function oldsvyquantile in order to ensure backward
#          compatability with spsurvey prior to version 5.0.0
# Revised: October 12, 2021 to correct an error that occurs when assigning
#          results for the case where a subpopulation contain a single value
# Revised: October 25, 2021 to correct an error that occurs when assigning
#          results for the case where a domain contains a single subpopulation
#
#' Percentile Estimates for Probability Survey Data
#'
#' This function calculates percentile estimates using the
#' \code{oldsvyquantile()} function in the survey package (\code{svyquantile()}
#' on survey version pre 4.1-1).  Upper and lower confidence bounds also are
#' estimated.
#'
#' @param pctsum Data frame containing estimates.
#'
#' @param dframe Data frame containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param itype Character value that identifies a factor variable in the design
#'   argument containing subpopulation (domain) values.
#'
#' @param lev_itype Character vector that provides levels of the subpopulation
#'   variable.
#'
#' @param nlev_itype Numeric value that provides the number of levels of the
#'   subpopulation variable.
#'
#' @param ivar Character value that identifies the response variable.
#'
#' @param design Object of class \code{survey.design} that specifies a complex
#'   survey design.
#'
#' @param design_names Character vector that provides names of survey design
#'   variables in the \code{design} argument.
#'
#' @param var_nondetect Character value that identifies the name of a logical
#'   variable in the \code{dframe} data frame specifying the presence of not
#'   detected (nondetect) values for the response variable.
#'
#' @param conf Numeric value for the confidence level.
#'
#' @param mult Numeric value that provides the Normal distribution confidence
#'   bound multiplier.
#'
#' @param pctval  Vector of the set of values at which percentiles are
#'   estimated.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @return A list composed of the following objects:
#'   \itemize{
#'     \item{\code{pctsum}}{data frame containing the percentile estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey univar
#'
#' @noRd
################################################################################

percentile_est <- function(pctsum, dframe, itype, lev_itype, nlev_itype, ivar,
                           design, design_names, var_nondetect, conf, mult,
                           pctval, warn_ind, warn_df) {

  # Assign a value to the function name variable

  fname <- "percentile_est"

  # Assign the number of perentile values

  npctval <- length(pctval)

  #
  # Calculate percentile estimates
  #

  if (is.null(var_nondetect)) {

    # Calculate percentile estimates, standard error estimates, and confidence
    # bound estimates for each combination of subpopulation and response
    # variable for the case where nondetects are not present

    tst <- !is.na(dframe[, itype])
    if (nlev_itype == 1) {
      nresp <- sum(!is.na(dframe[tst, ivar]))
      if (nresp == 1) {
        warn_ind <- TRUE
        act <- "Percentiles were not calculated.\n"
        warn <- paste0("Percentile estimates were not calculated for subpopulation type \"", itype, "\" \nsince the number of non-missing response values equals one.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = itype, subpop = NA, indicator = ivar,
          stratum = NA, warning = I(warn), action = I(act)
        ))
        pctest <- rep(NA, npctval)
        nresp <- rep(NA, npctval)
        stderr <- rep(NA, npctval)
        lbound <- rep(NA, npctval)
        ubound <- rep(NA, npctval)
      } else {
        options(warn = -1)
        rslt_svy <- tryCatch(
          oldsvyquantile(make.formula(ivar),
          design = subset(design, tst), quantiles = pctval / 100,
          alpha = (100 - conf) / 100, ci = TRUE, na.rm = TRUE, ties = "rounded"),
          error = function(e) oldsvyquantile(make.formula(ivar),
          design = subset(design, tst), quantiles = pctval / 100,
          alpha = (100 - conf) / 100, ci = TRUE, na.rm = TRUE, ties = "rounded", method = "constant")
        )
        options(warn = 0)
        pctest <- rslt_svy$quantiles
        nresp <- cdf_nresp(dframe[, ivar], as.vector(pctest))
        stderr <- SE(rslt_svy)
        temp <- confint(rslt_svy)
        lbound <- temp[, 1]
        ubound <- temp[, 2]
      }
    } else {
      pctest <- array(NA, c(nlev_itype, npctval))
      nresp <- array(NA, c(nlev_itype, npctval))
      stderr <- array(NA, c(nlev_itype, npctval))
      lbound <- array(NA, c(nlev_itype, npctval))
      ubound <- array(NA, c(nlev_itype, npctval))
      nval <- tapply(dframe[tst, ivar], dframe[tst, itype], function(x) {
        sum(!is.na(x))
      })
      subpop_ind <- nval > 1
      if (any(!subpop_ind)) {
        temp_str <- vecprint(lev_itype[!subpop_ind])
        warn_ind <- TRUE
        act <- "Percentiles were not calculated.\n"
        warn <- paste0("Percentile estimates were not calculated for the following subpopulations of \nsubpopulation type \"", itype, "\" since the number of non-missing response values \nequals one for each subpopulation:\n", temp_str)
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = itype, subpop = NA, indicator = ivar, stratum = NA,
          warning = I(warn), action = I(act)
        ))
      }
      if (any(subpop_ind)) {
        tst <- tst & dframe[, itype] %in% lev_itype[subpop_ind]
        levs <- (1:nlev_itype)[subpop_ind]
        options(warn = -1)
        rslt_svy <- tryCatch(
          svyby(make.formula(ivar), make.formula(itype),
          design = subset(design, tst), oldsvyquantile,
          quantiles = pctval / 100, alpha = (100 - conf) / 100, ci = TRUE,
          na.rm = TRUE, ties = "rounded"),
          error = function(e) svyby(make.formula(ivar), make.formula(itype),
          design = subset(design, tst), oldsvyquantile,
          quantiles = pctval / 100, alpha = (100 - conf) / 100, ci = TRUE,
          na.rm = TRUE, ties = "rounded", method = "constant")
        )
        options(warn = 0)
        j <- 1
        for (i in levs) {
          pctest[i, ] <- unlist(rslt_svy[j, 2:(npctval + 1)])
          j <- j + 1
        }
        for (i in levs) {
          ind <- dframe[, itype] == lev_itype[i]
          nresp[i, ] <- cdf_nresp(dframe[ind, ivar], pctest[i, ])
        }
        rownames(nresp) <- lev_itype
        temp <- SE(rslt_svy)
        j <- 1
        for (i in levs) {
          stderr[i, ] <- unlist(temp[j, 1:npctval])
          j <- j + 1
        }
        temp <- confint(rslt_svy)
        j <- 1
        for (i in levs) {
          lbound[i, ] <- temp[seq(j, by = length(levs), length = npctval), 1]
          ubound[i, ] <- temp[seq(j, by = length(levs), length = npctval), 2]
          j <- j + 1
        }
        rownames(lbound) <- lev_itype
        rownames(ubound) <- lev_itype
      }
    }
  } else {

    # To be implemented
  }

  # Assign identifiers and estimates to the pctsum data frame

  if (is.null(var_nondetect)) {
    if (nlev_itype == 1) {
      pctsum <- rbind(pctsum, data.frame(
        Type = itype,
        Subpopulation = lev_itype,
        Indicator = ivar,
        Statistic = paste0(pctval, "Pct"),
        nResp = nresp,
        Estimate = pctest[1:npctval],
        StdError = stderr,
        MarginofError = mult * stderr,
        LCB = lbound,
        UCB = ubound
      ))
    } else {
      for (i in 1:nlev_itype) {
        pctsum <- rbind(pctsum, data.frame(
          Type = itype,
          Subpopulation = lev_itype[i],
          Indicator = ivar,
          Statistic = paste0(pctval, "Pct"),
          nResp = nresp[i, ],
          Estimate = pctest[i, ],
          StdError = stderr[i, ],
          MarginofError = mult * stderr[i, ],
          LCB = lbound[i, ],
          UCB = ubound[i, ]
        ))
      }
    }
  } else {

    # To be implemented
  }

  # Return the pctsum data frame, the warn_ind logical value, and the warn_df
  # data frame

  list(pctsum = pctsum, warn_ind = warn_ind, warn_df = warn_df)
}

###############################################################################
# Function: revisit_rand (exported)
# Programmer: Tony Olsen
# Date: March 15, 2019
#'
#' Create a revisit design with random assignment to panels and time periods
#'
#' Create a revisit design for a survey that specifies the panels and time
#' periods that will be sampled by random selection of panels and time periods.
#' Three options for random assignments are \code{"period"} where the number of time
#' periods to be sampled in a panel is fixed, \code{"panel"} where the number panels to
#' be sampled in a time period is fixed, and \code{"none"} where the number of
#' panel-period combinations is fixed.
#'
#' @param n_period  Number of time periods for the survey design. Typically,
#'   number of periods if sampling occurs once per period or number of months if
#'   sampling occurs once per month. (v, number of varieties (or treatments) in
#'   BIBD terms)
#'
#' @param n_pnl Number of panels
#'
#' @param rand_control  Character value must be \code{"none"}, \code{"panel"}, or \code{"period"}.
#'   Specifies whether the number of sample events will be fixed for each panel
#'   (\code{"panel"}), for each sample occasion (\code{"occasion"}), or for total panel-period
#'   combinations (\code{"none"}).  Default is \code{"panel"}.
#'
#' @param n_visit If \code{rand_control} is \code{"panel"}, this is the number of panels that
#'   will be sampled in each time period. If rand_control is \code{"period"}, this is
#'   the number of time periods to be sampled in each panel. If \code{rand_control} is
#'   \code{"none"}, this is the total number of panel-period combinations that will
#'   have units sampled in the revisit design.
#'
#' @param nsamp Number of samples in each panel.
#'
#' @param panel_name  Prefix for name of each panel
#'
#' @param begin  Numeric name of first sampling occasion, e.g. a specific period.
#'
#' @param skip  Number of sampling occasions to skip between planned sampling
#'   periods, e.g., sampling will occur only every 5 periods if \code{skip = 5}.
#'
#' @details  The revisit design for a survey is created by random selection of
#'   panels and time periods that will have sample events.  The number of sample
#'   occasions that will be visited by a panel is random.
#'
#' @return A two-dimensional array of sample sizes to be sampled for each panel
#'   and each time period.
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#' @seealso
#'   \describe{
#'     \item{\code{\link{revisit_bibd}}}{create a balanced incomplete block
#'       panel revisit design}
#'     \item{\code{\link{revisit_dsgn}}}{create a panel revisit design}
#'     \item{\code{\link{pd_summary}}}{ to summarize characteristics of a
#'       panel revisit design}
#'   }
#'
#' @keywords survey
#'
#' @examples
#' revisit_rand(
#'   n_period = 20, n_pnl = 10, rand_control = "none", n_visit = 50,
#'   nsamp = 20
#' )
#' revisit_rand(
#'   n_period = 20, n_pnl = 10, rand_control = "panel", n_visit = 5,
#'   nsamp = 10
#' )
#' revisit_rand(
#'   n_period = 20, n_pnl = 10, rand_control = "period",
#'   n_visit = 5, nsamp = 10
#' )
#' @export
###############################################################################

revisit_rand <- function(n_period, n_pnl, rand_control = "period", n_visit,
                         nsamp, panel_name = "Random", begin = 1, skip = 1) {
  if (!(rand_control %in% c("none", "panel", "period"))) {
    stop("\nRandom control not equal to none, panel or period")
  }

  if (rand_control == "panel") {
    pan_dsgn <- matrix(rep(c(rep(nsamp, n_visit), rep(0, n_period - n_visit)), n_pnl),
      ncol = n_period, byrow = TRUE
    )
    pan_dsgn <- t(apply(pan_dsgn, 1, function(x) {
      sample(x,
        size = length(x),
        replace = FALSE
      )
    }))
  }

  if (rand_control == "period") {
    pan_dsgn <- matrix(rep(c(rep(nsamp, n_visit), rep(0, n_pnl - n_visit)), n_period),
      ncol = n_period
    )
    pan_dsgn <- apply(pan_dsgn, 2, function(x) {
      sample(x,
        size = length(x),
        replace = FALSE
      )
    })
  }

  if (rand_control == "none") {
    pan_dsgn <- c(rep(nsamp, n_visit), rep(0, (n_pnl * n_period) - n_visit))
    pan_dsgn <- sample(pan_dsgn, size = length(pan_dsgn), replace = FALSE)
    pan_dsgn <- matrix(pan_dsgn, nrow = n_pnl)
  }

  # drop panels if no visits
  keep <- ifelse(apply(pan_dsgn, 1, sum) > 0, TRUE, FALSE)
  pan_dsgn <- pan_dsgn[keep, ]

  # assign dimnames
  if (nrow(pan_dsgn) < 10) {
    dimnames(pan_dsgn) <- list(
      c(paste(panel_name, 1:nrow(pan_dsgn), sep = "_")),
      seq(begin, by = skip, length.out = ncol(pan_dsgn))
    )
  } else {
    dimnames(pan_dsgn) <- list(
      c(
        paste(panel_name, 1:9, sep = "_0"),
        paste(panel_name, 10:nrow(pan_dsgn), sep = "_")
      ),
      seq(begin, by = skip, length.out = ncol(pan_dsgn))
    )
  }
  class(pan_dsgn) <- "paneldesign"
  return(pan_dsgn)
}

###############################################################################
# Function: rho (not exported)
# Programmers: Tony Olsen
# Date: September 9, 2020
#'
#' Place the sites selected by the UPpivotal method in reverse hierarchical order
#'
#' @param sites Sample object with sites selected by GRTS algorithm
#'
#' @return sites Sample object returned in reverse hierarchical order
#'
#' @author Tony Olsen \email{olsen.tony@@epa.gov}
#'
#' @keywords survey
#'
#'
#' @noRd
###############################################################################

rho <- function(sites) {

  # Construct sample hierarchical address
  np <- nrow(sites)
  nlev <- max(1, trunc(logb(np, 4)))
  ifelse(np == 4^nlev, nlev, nlev <- nlev + 1)
  ad <- matrix(0, 4^nlev, nlev)
  rv4 <- 0:3
  pwr4 <- 4.^(0.:(nlev - 1.))
  for (i in 1:nlev) {
    ad[, i] <- rep(rep(rv4, rep(pwr4[i], 4.)), pwr4[nlev] / pwr4[i])
  }
  rho4 <- as.vector(ad %*% matrix(rev(pwr4), nlev, 1))

  # Place sample in reverse hierarchical order
  sites <- sites[unique(floor(rho4 * np / 4^nlev)) + 1., ]
}

###############################################################################
# Function: cdf_prop (not exported)
# Programmer: Tom Kincaid
# Date: March 3, 2020
#
#' Estimate of Cumulative Distribution Function for a Proportion
#'
#' This function calculates an estimate of the cumulative distribution function
#' (CDF) for the proportion of a finite or an extensive resource.  The set of
#' values at which the CDF is estimated is supplied to the function.  The
#' Horvitz-Thompson ratio estimator, i.e., the ratio of two Horvitz-Thompson
#' estimators, is used to calculate the estimate.  The numerator of the ratio
#' estimates the total of the resource equal to or less than a specified value.
#' The denominator of the ratio estimates the size of the resource.  For a
#' finite resource size is the number of units in the resource.  For an
#' extensive resource size is the extent (measure) of the resource, i.e.,
#' length, area, or volume.  The function can accomodate single-stage and
#' two-stage samples.
#'
#' @param z Vector of the response value for each site.
#'
#' @param wgt Vector of the final adjusted weight (reciprocal of the sample
#'   inclusion probability) for each site, which is either the weight for a
#'   single- stage sample or the stage two weight for a two-stage sample.
#'
#' @param val Vector of the set of values at which the CDF is estimated.
#'
#' @param cluster_ind Logical value that indicates whether the sample is a
#'   two-stage sample, where \code{TRUE} = a two-stage sample and \code{FALSE} = not a
#'   two-stage sample.
#'
#' @param cluster Vector of the stage one sampling unit (primary sampling unit
#'   or cluster) code for each site.  The default value is \code{NULL}.
#'
#' @param wgt1 = Vector of the final adjusted stage one weight for each site.
#'   The default value is \code{NULL}.
#'
#' @return The CDF estimate.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
###############################################################################

cdf_prop <- function(z, wgt, val, cluster_ind, cluster = NULL, wgt1 = NULL) {

  # Calculate additional required values

  m <- length(val)
  if (cluster_ind) {
    cluster <- factor(cluster)
    ncluster <- length(levels(cluster))
    z_1st <- split(z, cluster)
    wgt2_1st <- split(wgt, cluster)
    wgt1_u <- as.vector(tapply(wgt1, cluster, unique))
  }

  # Calculate the cdf estimate

  cdf <- numeric(m)
  if (cluster_ind) {
    for (i in 1:m) {
      temp <- numeric(ncluster)
      for (j in 1:ncluster) {
        temp[j] <- sum(ifelse(z_1st[[j]] <= val[i], wgt2_1st[[j]], 0))
      }
      cdf[i] <- sum(wgt1_u * temp)
    }
  } else {
    for (i in 1:m) {
      cdf[i] <- sum(ifelse(z <= val[i], wgt, 0))
    }
  }

  if (cluster_ind) {
    cdf <- cdf / sum(wgt1 * wgt)
  } else {
    cdf <- cdf / sum(wgt)
  }

  # Return the estimate

  cdf
}

################################################################################
# Function: cont_cdftest (exported)
# Programmer Tom Kincaid
# Date: October 23, 2020
# Revised: December 15, 2020 to allow use of the Horvitz-Thompson and
#          Yates-Grundy variance estimators and to use a new function named
#          survey_design to create the survey design object
# Revised: January 28, 2021 to replace "warn.vec" with "warn_vec"
# Revised: February 23, 2021 to correct errors in the code for creating the
#          warnings data frame (warn_df)
# Revised: April 7, 2021 to ensure that the dframe argument does not contain
#          zero rows
# Revised: April 29, 2021 to ensure that the dframe argument only belongs to
#          class "data.frame"
# Revised: May 4, 2021 to avoid warning messages being generated during creation
#          of help files
# Revised: May 6, 2021 to ensure that sf objects do not belong to class tbl_df
# Revised: June 8, 2021 to simplify specification of the values required for
#          calculation of the finite population correction factor and to
#          eliminate use of the finite population correction factor with the
#          local mean variance estimator
# Revised: September 9, 2021 to revise the documentation for argument popsize
#
#' Cumulative distribution function (CDF) inference for a probability survey
#'
#' This function organizes input and output for conducting inference regarding
#' cumulative distribution functions (CDFs) generated by a probability survey.
#' For every response variable and every subpopulation (domain) variable,
#' differences between CDFs are tested for every pair of subpopulations within
#' the domain.  Data input to the function can be either a single survey or
#' multiple surveys (two or more).  If the data contain multiple surveys, then
#' the domain variables will reference those surveys and (potentially)
#' subpopulations within those surveys.  The inferential procedures divide the
#' CDFs into a discrete set of intervals (classes) and then utilize procedures
#' that have been developed for analysis of categorical data from probability
#' surveys.  Choices for inference are the Wald, adjusted Wald, Rao-Scott first
#' order corrected (mean eigenvalue corrected), and Rao-Scott second order
#' corrected (Satterthwaite corrected) test statistics. The default test
#' statistic is the adjusted Wald statistic.  The input data argument can be
#' either a data frame or a simple features (sf) object.  If an sf object is
#' used, coordinates are extracted from the geometry column in the object,
#' arguments xcoord and ycoord are assigned values "xcoord" and "ycoord",
#' respectively, and the geometry column is dropped from the object.
#'
#' @param dframe Data frame containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param vars Vector composed of character values that identify the
#'   names of response variables in the \code{dframe} data frame.
#'
#' @param subpops Vector composed of character values that identify the
#'   names of subpopulation (domain) variables in the \code{dframe} data frame.
#'   If a value is not provided, the value \code{"All_Sites"} is assigned to the
#'   subpops argument and a factor variable named \code{"All_Sites"} that takes
#'   the value \code{"All Sites"} is added to the \code{dframe} data frame.  The
#'   default value is \code{NULL}.
#'
#' @param surveyID Character value providing name of the survey ID variable in
#'   the \code{dframe} data frame.  If this argument equals \code{NULL}, then
#'   the dframe data frame contains data for a single survey.  The default value
#'   is \code{NULL}.
#'
#' @param siteID Character value providing name of the site ID variable in
#'   the \code{dframe} data frame.  For a two-stage sample, the site ID variable
#'   identifies stage two site IDs.  The default value is \code{"siteID"}.
#'
#' @param weight Character value providing name of the survey design weight
#'   variable in the \code{dframe} data frame.  For a two-stage sample, the
#'   weight variable identifies stage two weights.  The default value is
#'   \code{"weight"}.
#'
#' @param xcoord Character value providing name of the x-coordinate variable in
#'   the \code{dframe} data frame.  For a two-stage sample, the x-coordinate
#'   variable identifies stage two x-coordinates.  Note that x-coordinates are
#'   required for calculation of the local mean variance estimator.  The default
#'   value is \code{NULL}.
#'
#' @param ycoord Character value providing name of the y-coordinate variable in
#'   the \code{dframe} data frame.  For a two-stage sample, the y-coordinate
#'   variable identifies stage two y-coordinates.  Note that y-coordinates are
#'   required for calculation of the local mean variance estimator.  The default
#'   value is \code{NULL}.
#'
#' @param stratumID Character value providing name of the stratum ID variable in
#'   the \code{dframe} data frame.  The default value is \code{NULL}.
#'
#' @param clusterID Character value providing the name of the cluster
#'   (stage one) ID variable in the \code{dframe} data frame.  Note that cluster
#'   IDs are required for a two-stage sample.  The default value is \code{NULL}.
#'
#' @param weight1 Character value providing name of the stage one weight
#'   variable in the \code{dframe} data frame.  The default value is \code{NULL}.
#'
#' @param xcoord1 Character value providing the name of the stage one
#'   x-coordinate variable in the \code{dframe} data frame.  Note that x
#'   coordinates are required for calculation of the local mean variance
#'   estimator.  The default value is \code{NULL}.
#'
#' @param ycoord1 Character value providing the name of the stage one
#'   y-coordinate variable in the \code{dframe} data frame.  Note that
#'   y-coordinates are required for calculation of the local mean variance
#'   estimator.  The default value is \code{NULL}.
#'
#' @param sizeweight Logical value that indicates whether size weights should be
#'   used during estimation, where \code{TRUE} uses size weights and
#'   \code{FALSE} does not use size weights. To employ size weights for a
#'   single-stage sample, a value must be supplied for argument weight.  To
#'   employ size weights for a two-stage sample, values must be supplied for
#'   arguments \code{weight} and \code{weight1}. The default value is \code{FALSE}.
#'
#' @param sweight Character value providing the name of the size weight variable
#'   in the \code{dframe} data frame.  For a two-stage sample, the size weight
#'   variable identifies stage two size weights.  The default value is
#'   \code{NULL}.
#'
#' @param sweight1 Character value providing name of the stage one size weight
#'   variable in the \code{dframe} data frame.  The default value is \code{NULL}.
#'
#' @param sweight1 Character value providing name of the stage one size weight
#'   variable in the \code{dframe} data frame.  The default value is \code{NULL}.
#'
#' @param fpc Object that specifies values required for calculation of the
#'   finite population correction factor used during variance estimation. The
#'   object must match the survey design in terms of stratification and whether
#'   the design is single-stage or two-stage.  For an unstratified design, the
#'   object is a vector.  The vector is composed of a single numeric value for a
#'   single-stage design.  For a two-stage unstratified design, the object is a
#'   named vector containing one more than the number of clusters in the sample,
#'   where the first item in the vector specifies the number of clusters in the
#'   population and each subsequent item specifies the number of stage two units
#'   for the cluster.  The name for the first item in the vector is arbitrary.
#'   Subsequent names in the vector identify clusters and must match the cluster
#'   IDs.  For a stratified design, the object is a named list of vectors, where
#'   names must match the strata IDs.  For each stratum, the format of the
#'   vector is identical to the format described for unstratified single-stage
#'   and two-stage designs.  Note that the finite population correction factor
#'   is not used with the local mean variance estimator.
#'
#'   Example fpc for a single-stage unstratified survey design:
#'
#'   \verb{fpc <- 15000}
#'
#'   Example fpc for a single-stage stratified survey design:
#'
#'   \verb{fpc <- list(
#'     Stratum_1 = 9000,
#'     Stratum_2 = 6000)
#'    }
#'
#'   Example fpc for a two-stage unstratified survey design:
#'
#'   \verb{fpc <- c(
#'     Ncluster = 150,
#'     Cluster_1 = 150,
#'     Cluster_2 = 75,
#'     Cluster_3 = 75,
#'     Cluster_4 = 125,
#'     Cluster_5 = 75)
#'   }
#'
#'   Example fpc for a two-stage stratified survey design:
#'
#'   \verb{fpc <- list(
#'     Stratum_1 = c(
#'       Ncluster_1 = 100,
#'       Cluster_1 = 125,
#'       Cluster_2 = 100,
#'       Cluster_3 = 100,
#'       Cluster_4 = 125,
#'       Cluster_5 = 50),
#'     Stratum_2 = c(
#'       Ncluster_2 = 50,
#'       Cluster_1 = 75,
#'       Cluster_2 = 150,
#'       Cluster_3 = 75,
#'       Cluster_4 = 75,
#'       Cluster_5 = 125))
#'   }
#'
#' @param popsize Object that provides values for the population argument of the
#'   \code{calibrate} or \code{postStratify} functions in the survey package. If
#'   a value is provided for popsize, then either the \code{calibrate} or
#'   \code{postStratify} function is used to modify the survey design object
#'   that is required by functions in the survey package.  Whether to use the
#'   \code{calibrate} or \code{postStratify} function is dictated by the format
#'   of popsize, which is discussed below.  Post-stratification adjusts the
#'   sampling and replicate weights so that the joint distribution of a set of
#'   post-stratifying variables matches the known population joint distribution.
#'   Calibration, generalized raking, or GREG estimators generalize
#'   post-stratification and raking by calibrating a sample to the marginal
#'   totals of variables in a linear regression model. For the \code{calibrate}
#'   function, the object is a named list, where the names identify factor
#'   variables in the \code{dframe} data frame.  Each element of the list is a
#'   named vector containing the population total for each level of the
#'   associated factor variable.  For the \code{postStratify} function, the
#'   object is either a data frame, table, or xtabs object that provides the
#'   population total for all combinations of selected factor variables in the
#'   \code{dframe} data frame.  If a data frame is used for \code{popsize}, the
#'   variable containing population totals must be the last variable in the data
#'   frame.  If a table is used for \code{popsize}, the table must have named
#'   \code{dimnames} where the names identify factor variables in the
#'   \code{dframe} data frame.  If the popsize argument is equal to \code{NULL},
#'   then neither calibration nor post-stratification is performed.  The default
#'   value is \code{NULL}.
#'
#'   Example popsize for calibration:
#'
#'   \verb{popsize <- list(
#'     Ecoregion = c(
#'       East = 750,
#'       Central = 500,
#'       West = 250),
#'     Type = c(
#'       Streams = 1150,
#'       Rivers = 350))
#'   }
#'
#'   Example popsize for post-stratification using a data frame:
#'
#'   \verb{popsize <- data.frame(
#'     Ecoregion = rep(c("East", "Central", "West"),
#'       rep(2, 3)),
#'     Type = rep(c("Streams", "Rivers"), 3),
#'     Total = c(575, 175, 400, 100, 175, 75))
#'   }
#'
#'   Example popsize for post-stratification using a table:
#'
#'   \verb{popsize <- with(MySurveyFrame,
#'     table(Ecoregion, Type))}
#'
#'   Example popsize for post-stratification using an xtabs object:
#'
#'   \verb{popsize <- xtabs(~Ecoregion + Type,
#'     data = MySurveyFrame)}
#'
#' @param vartype Character value providing the choice of the variance
#'   estimator, where \code{"Local"} indicates the local mean estimator,
#'   \code{"SRS"} indicates the simple random sampling estimator, \code{"HT"}
#'   indicates the Horvitz-Thompson estimator, and \code{"YG"} indicates the
#'   Yates-Grundy estimator.  The default value is \code{"Local"}.
#'
#' @param jointprob Character value providing the choice of joint inclusion
#'   probability approximation for use with Horvitz-Thompson and Yates-Grundy
#'   variance estimators, where \code{"overton"} indicates the Overton
#'   approximation, \code{"hr"} indicates the Hartley-Rao approximation, and
#'   \code{"brewer"} equals the Brewer approximation.  The default value is
#'   \code{"overton"}.
#'
#' @param testname Name of the test statistic to be reported in the output
#'   data frame.  Choices for the name are: \code{"Wald"}, \code{"adjWald"},
#'   \code{"RaoScott_First"}, and \code{"RaoScott_Second"}, which correspond to
#'   the Wald statistic, adjusted Wald statistic, Rao-Scott first-order
#'   corrected statistic, and Rao-Scott second-order corrected statistic,
#'   respectively.  The default is \code{"adjWald"}.
#'
#' @param nclass Number of classes into which the CDFs will be divided
#'   (binned), which must equal at least \code{2}.  The default is \code{3}.
#'
#' @return Data frame of CDF test results for all pairs of subpopulations
#'   within each population type for every response variable.  The data frame
#'   includes the test statistic specified by argument \code{testname} plus its
#'   degrees of freedom and p-value.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @seealso
#'   \describe{
#'   \item{\code{\link{cdf_plot}}}{ for visualizing CDF plots}
#'   \item{\code{\link{cont_cdfplot}}}{ for making CDF plots output to pdfs}
#'   }
#'
#' @examples
#' n <- 200
#' mysiteID <- paste("Site", 1:n, sep = "")
#' dframe <- data.frame(
#'   siteID = mysiteID,
#'   wgt = runif(n, 10, 100),
#'   xcoord = runif(n),
#'   ycoord = runif(n),
#'   stratum = rep(c("Stratum1", "Stratum2"), n / 2),
#'   Resource_Class = sample(c("Agr", "Forest", "Urban"), n, replace = TRUE)
#' )
#' ContVar <- numeric(n)
#' tst <- dframe$Resource_Class == "Agr"
#' ContVar[tst] <- rnorm(sum(tst), 10, 1)
#' tst <- dframe$Resource_Class == "Forest"
#' ContVar[tst] <- rnorm(sum(tst), 10.1, 1)
#' tst <- dframe$Resource_Class == "Urban"
#' ContVar[tst] <- rnorm(sum(tst), 10.5, 1)
#' dframe$ContVar <- ContVar
#' myvars <- c("ContVar")
#' mysubpops <- c("Resource_Class")
#' mypopsize <- data.frame(
#'   Resource_Class = rep(c("Agr", "Forest", "Urban"), rep(2, 3)),
#'   stratum = rep(c("Stratum1", "Stratum2"), 3),
#'   Total = c(2500, 1500, 1000, 500, 600, 450)
#' )
#' cont_cdftest(dframe,
#'   vars = myvars, subpops = mysubpops, siteID = "siteID",
#'   weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
#'   stratumID = "stratum", popsize = mypopsize
#' )
#' @export
################################################################################

cont_cdftest <- function(dframe, vars, subpops = NULL, surveyID = NULL, siteID = "siteID",
                         weight = "weight", xcoord = NULL, ycoord = NULL, stratumID = NULL,
                         clusterID = NULL, weight1 = NULL, xcoord1 = NULL, ycoord1 = NULL,
                         sizeweight = FALSE, sweight = NULL, sweight1 = NULL, fpc = NULL,
                         popsize = NULL, vartype = "Local", jointprob = "overton",
                         testname = "adjWald", nclass = 3) {

  # Create a vector for error messages

  error_ind <- FALSE
  error_vec <- NULL

  # Create a data frame for warning messages

  warn_ind <- FALSE
  warn_df <- NULL
  fname <- "cont_cdftest"

  # Check for a valid test statistic name

  temp <- match(testname, c(
    "Wald", "adjWald", "RaoScott_First",
    "RaoScott_Second"
  ), nomatch = 0)
  if (temp == 0) {
    error_ind <- TRUE
    msg <- paste0("\nThe value provided for argument testname, \"", testname, "\", is not a valid test statistic name.\n")
    error_vec <- c(error_vec, msg)
  }

  # As necessary, reassign the test statistic name to match the values required
  # by the svychisq_localmean function

  if (testname == "RaoScott_First") {
    testname <- "Chisq"
  } else if (testname == "RaoScott_Second") {
    testname <- "F"
  }

  # Check for the minimum number of classes

  if (testname %in% c("Wald", "adjWald")) {
    if (nclass < 2) {
      error_ind <- TRUE
      msg <- paste("\nThe number of classes into which the CDFs will be divided (binned) must equal \nat least two.")
      error_vec <- c(error_vec, msg)
    }
  } else {
    if (nclass < 3) {
      error_ind <- TRUE
      msg <- paste("\nThe number of classes into which the CDFs will be divided (binned) must equal \nat least three")
      error_vec <- c(error_vec, msg)
    }
  }

  # Ensure that the dframe argument was provided

  if (missing(dframe) | is.null(dframe)) {
    stop("\nThe dframe argument must be provided.\n")
  }

  # If the dframe argument is an sf object, extract coordinates from the
  # geometry column, assign values "xcoord" and "ycoord" to arguments xcoord and
  # ycoord, respectively, and drop the geometry column from the object

  if ("sf" %in% class(dframe)) {
    temp <- st_coordinates(dframe)
    xcoord <- "xcoord"
    dframe$xcoord <- temp[, "X"]
    ycoord <- "ycoord"
    dframe$ycoord <- temp[, "Y"]
    dframe <- st_set_geometry(dframe, NULL)
  }

  # If the dframe argument is a tibble or does not belong to class
  # "data.frame", coerce the argument to class "data.frame"

  if ("tbl_df" %in% class(dframe) | !("data.frame" %in% class(dframe))) {
    dframe <- as.data.frame(dframe)
  }

  # Ensure that the dframe argument does not contain zero rows

  if (nrow(dframe) == 0) {
    stop("\nThe dframe argument contains zero rows.\n")
  }

  # Ensure that unused levels are dropped from factor variables in the dframe
  # data frame

  dframe <- droplevels(dframe)

  # As necessary, ensure that the dframe data frame contains the survey ID
  # variable

  if (!is.null(surveyID)) {
    if (!(surveyID %in% names(dframe))) {
      ind1 <- FALSE
      error_ind <- TRUE
      msg <- paste0("The name provided for the surveyID argument, \"", surveyID, "\", does not occur among \nthe names for the dframe data frame.\n")
      error_vec <- c(error_vec, msg)
    } else {
      survey_names <- as.vector(unique(dframe[, surveyID]))
      ind1 <- TRUE
    }
  }

  # Ensure that the dframe data frame contains the site ID variable

  if (!(siteID %in% names(dframe))) {
    ind2 <- FALSE
    error_ind <- TRUE
    msg <- paste0("The name provided for the siteID argument, \"", siteID, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  } else {
    ind2 <- TRUE
  }

  # Check site IDs for repeat values and, as necessary, create unique site IDs
  # and output a warning message

  if (is.null(surveyID)) {
    if (ind2) {
      IDs <- dframe[, siteID]
      temp <- sapply(split(IDs, IDs), length)
      if (any(temp > 1)) {
        warn_ind <- TRUE
        temp.str <- vecprint(names(temp)[temp > 1])
        warn <- paste("The following site ID values occur more than once among the values that were \ninput to the function:\n", temp.str)
        act <- "Unique site ID values were created.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = NA,
          subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
        ))
        dframe[, siteID] <- uniqueID(dframe[, siteID])
      }
    }
  } else {
    if (ind1 & ind2) {
      IDs <- dframe[, siteID]
      for (i in survey_names) {
        eval(parse(text = paste0("tst <- dframe[, surveyID] == \"", i, "\"")))
        temp <- sapply(split(IDs[tst], IDs[tst]), length)
        if (any(temp > 1)) {
          warn_ind <- TRUE
          temp.str <- vecprint(names(temp)[temp > 1])
          warn <- paste("The following site ID values occur more than once among survey", i, "values \nthat were input to the function:\n", temp.str)
          act <- "Unique site ID values were created.\n"
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = NA,
            subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
          ))
          dframe[, siteID][tst] <- uniqueID(dframe[, siteID][tst])
        }
      }
    }
  }

  # Ensure that the dframe data frame contains the survey weight variable

  if (!(weight %in% names(dframe))) {
    error_ind <- TRUE
    msg <- paste0("The name provided for the weight argument, \"", weight, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  }

  # As necessary, create a stratum variable that incorporates survey ID values

  if (!is.null(surveyID)) {
    if (!is.null(stratumID)) {
      dframe$stratumID <- paste(dframe[, surveyID], dframe[, stratumID],
        sep = "."
      )
      stratumID <- "stratumID"
    } else {
      stratumID <- surveyID
    }
  }

  # Assign names to the variables required for calculation of the finite
  # population correction factor

  if (is.null(fpc)) {
    fpcfactor_ind <- FALSE
    fpcsize <- NULL
    Ncluster <- NULL
    stage1size <- NULL
  } else {
    fpcfactor_ind <- TRUE
    if (is.null(clusterID)) {
      fpcsize <- "fpcsize"
      Ncluster <- NULL
      stage1size <- NULL
    } else {
      fpcsize <- NULL
      Ncluster <- "Ncluster"
      stage1size <- "stage1size"
    }
  }

  # Create a list containing names of survey design variables

  design_names <- list(
    siteID = siteID,
    weight = weight,
    xcoord = xcoord,
    ycoord = ycoord,
    stratumID = stratumID,
    clusterID = clusterID,
    weight1 = weight1,
    xcoord1 = xcoord1,
    ycoord1 = ycoord1,
    sweight = sweight,
    sweight1 = sweight1,
    fpcsize = fpcsize,
    Ncluster = Ncluster,
    stage1size = stage1size
  )

  # Ensure that a value was provided for the vars (response variable names)
  # argument

  if (is.null(vars)) {
    error_ind <- TRUE
    msg <- "A value must be provided for the vars (response variable names) argument.\n"
    error_vec <- c(error_vec, msg)
  }

  # If a value was not provided for the subpops (subpopulation names) argument,
  # assign the value "All_Sites" to the subpops argument and create a factor
  # named "All_Sites" in the dframe data frame that takes the value "All Sites"

  if (is.null(subpops)) {
    subpops <- "All_Sites"
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # Check input arguments
  temp <- input_check(dframe, design_names, NULL, vars, NULL, NULL, subpops,
    sizeweight, fpc, popsize, vartype, jointprob,
    conf = 95,
    error_ind = error_ind, error_vec = error_vec
  )
  dframe <- temp$dframe
  vars <- temp$vars_cont
  subpops <- temp$subpops
  popsize <- temp$popsize
  vartype <- temp$vartype
  jointprob <- temp$jointprob
  error_ind <- temp$error_ind
  error_vec <- temp$error_vec

  # As necessary, output a message indicating that error messages were generated
  # during execution of the program

  if (error_ind) {
    error_vec <<- error_vec
    if (length(error_vec) == 1) {
      message("During execution of the program, an error message was generated.  The error \nmessage is stored in a vector named 'error_vec'.  Enter the following command \nto view the error message: errorprnt()\n")
    } else {
      message(paste("During execution of the program,", length(error_vec), "error messages were generated.  The error \nmessages are stored in a vector named 'error_vec'.  Enter the following \ncommand to view the error messages: errorprnt()\n"))
    }

    if (warn_ind) {
      warn_df <<- warn_df
      if (nrow(warn_df) == 1) {
        message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
      } else {
        message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
      }
    }
    stop("See the preceding message(s).")
  }

  # Assign a logical value to the indicator variable for a stratified sample

  stratum_ind <- !is.null(stratumID)

  # For a stratified sample, remove strata that contain a single site

  if (stratum_ind) {
    dframe[, stratumID] <- factor(dframe[, stratumID])
    stratum_levels <- levels(dframe[, stratumID])
    nstrata <- length(stratum_levels)
    ind <- FALSE
    for (i in 1:nstrata) {
      tst <- dframe[, stratumID] == stratum_levels[i]
      if (sum(tst) == 1) {
        warn_ind <- TRUE
        warn <- paste0("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
        act <- "Stratum was removed from the analysis.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = NA,
          subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
        ))
        dframe <- dframe[!tst, ]
        ind <- TRUE
      }
    }
    if (ind) {
      dframe[, stratumID] <- factor(dframe[, stratumID])
      stratum_levels <- levels(dframe[, stratumID])
      nstrata <- length(stratum_levels)
      dframe <- droplevels(dframe)
    }
  }

  # Assign a logical value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Create the survey design object

  design <- survey_design(
    dframe, siteID, weight, stratum_ind, stratumID, cluster_ind, clusterID,
    weight1, sizeweight, sweight, sweight1, fpcfactor_ind, fpcsize, Ncluster,
    stage1size, vartype, jointprob
  )

  # If popsize is not equal to NULL, then call either the postStratify or
  # calibrate function, as appropriate

  if (!is.null(popsize)) {
    if (all(class(popsize) %in% c("data.frame", "table", "xtabs"))) {
      if ("data.frame" %in% class(popsize)) {
        pnames <- names(popsize)[-ncol(popsize)]
      } else {
        pnames <- names(dimnames(popsize))
      }
      design <- postStratify(design, make.formula(pnames), popsize)
    } else {
      cnames <- cal_names(make.formula(names(popsize)), design)
      pop_totals <- numeric(length(cnames))
      names(pop_totals) <- cnames
      pop_totals[1] <- sum(popsize[[1]])
      k <- 2
      for (i in names(popsize)) {
        temp <- popsize[[i]]
        for (j in 2:length(temp)) {
          pop_totals[k] <- temp[j]
          k <- k + 1
        }
      }
      design <- calibrate(design, make.formula(names(popsize)), pop_totals)
    }
  }

  # If popsize is not equal to NULL and vartype equals "Local", then assign
  # adjusted weights to the appropriate weight variable(s) in the
  # design$variables data frame

  if (!is.null(popsize) && vartype == "Local") {
    if (cluster_ind) {
      design$variables$wgt2 <- weights(design) / design$variables$wgt1
    } else {
      design$variables$wgt <- weights(design)
    }
  }

  # Create the contsum (results) data frame

  contsum <- NULL

  # Loop through all subpopulations (domains)

  for (itype in subpops) {

    # Identify levels of the subpopulation

    lev_itype <- levels(dframe[, itype])
    nlev_itype <- length(lev_itype)

    # Check whether the number of levels of the subpopulation is greater than one

    if (nlev_itype == 1) {
      warn_ind <- TRUE
      warn <- paste("Population type", itype, "contains a single subpopulation. \nNo CDF tests could be performed\n")
      act <- "None.\n"
      warn_df <- rbind(warn_df, data.frame(
        func = I(fname), subpoptype = I(itype), subpop = NA, indicator = NA,
        stratum = NA, warning = I(warn), action = I(act)
      ))
      next
    }

    # Loop through all response variables (vars)

    for (ivar in vars) {

      # Determine the set of upper bounds for the response variable

      temp <- dframe[!is.na(dframe[, ivar]), ivar]
      bounds <- sort(temp)[floor(seq(length(temp) / nclass, length(temp),
        length = nclass
      ))]
      design$variables$colvar <- cut(dframe[, ivar], c(-Inf, bounds))

      # Loop through all combinations of levels of the subpopulation

      # Begin the loop for the the first level of the combination

      for (isubpop1 in 1:(nlev_itype - 1)) {

        # Select sites in the first level

        subpop1_ind <- dframe[, itype] == lev_itype[isubpop1]

        # Determine whether the level is empty

        if (all(is.na(dframe[subpop1_ind, ivar]))) {
          warn_ind <- TRUE
          warn <- paste("Subpopulation", lev_itype[isubpop1], "of population type", itype, "for indicator", ivar, "\ncontains no data.\n")
          act <- "None.\n"
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = I(itype),
            subpop = I(lev_itype[isubpop1]), indicator = ivar, stratum = NA,
            warning = I(warn), action = I(act)
          ))
          next
        }

        # Determine whether the level contains a single value

        if (sum(!is.na(dframe[subpop1_ind, ivar])) == 1) {
          warn_ind <- TRUE
          warn <- paste("Subpopulation", lev_itype[isubpop1], "of population type", itype, "for indicator", ivar, "\ncontains a single value.  No analysis was performed.\n")
          act <- "None.\n"
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = I(itype),
            subpop = I(lev_itype[isubpop1]), indicator = ivar, stratum = NA,
            warning = I(warn), action = I(act)
          ))
          next
        }

        # Begin the loop for the second level of the combination

        for (isubpop2 in (isubpop1 + 1):nlev_itype) {

          # Select sites in the second level

          subpop2_ind <- dframe[, itype] == lev_itype[isubpop2]

          # Determine whether level is empty

          if (all(is.na(dframe[subpop2_ind, ivar]))) {
            warn_ind <- TRUE
            warn <- paste("Subpopulation", lev_itype[isubpop2], "of population type", itype, "for indicator", ivar, "\ncontains no data.\n")
            act <- "None.\n"
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = I(itype),
              subpop = I(lev_itype[isubpop2]), indicator = ivar, stratum = NA,
              warning = I(warn), action = I(act)
            ))
            next
          }

          # Determine whether the level contains a single value

          if (sum(!is.na(dframe[subpop2_ind, ivar])) == 1) {
            warn_ind <- TRUE
            warn <- paste("Subpopulation", lev_itype[isubpop2], "of population type", itype, "for indicator", ivar, "\ncontains a single value.  No analysis was performed.\n")
            act <- "None.\n"
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = I(itype),
              subpop = I(lev_itype[isubpop2]), indicator = ivar, stratum = NA,
              warning = I(warn), action = I(act)
            ))
            next
          }

          # Perform the CDF test for the response variable

          rowvar <- rep(NA, nrow(dframe))
          rowvar[subpop1_ind] <- "Subpopulation 1"
          rowvar[subpop2_ind] <- "Subpopulation 2"
          design$variables$rowvar <- factor(rowvar,
            levels = c("Subpopulation 1", "Subpopulation 2")
          )

          var_totals <- NULL
          var_means <- NULL
          if (vartype == "Local") {
            warn_vec <- c(itype, lev_itype[isubpop1], lev_itype[isubpop2], ivar)
            if (testname %in% c("Wald", "adjWald")) {
              temp <- cdftest_localmean_total(
                design, design_names, warn_ind, warn_df, warn_vec
              )
              var_totals <- temp$varest
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df
            } else {
              temp <- cdftest_localmean_prop(
                design, design_names, warn_ind, warn_df, warn_vec
              )
              var_means <- temp$varest
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df
            }
          }

          temp <- svychisq_localmean(
            ~ rowvar + colvar, design, testname, vartype, var_totals, var_means
          )

          # Assign estimates for the response variable to the contsum data frame

          contsum <- rbind(
            contsum,
            data.frame(
              Type = itype,
              Subpopulation_1 = lev_itype[isubpop1],
              Subpopulation_2 = lev_itype[isubpop2],
              Indicator = ivar,
              statistic = temp$statistic,
              Degrees_of_Freedom_1 = temp$parameter[1],
              Degrees_of_Freedom_2 = temp$parameter[2],
              p_Value = temp$p.value
            )
          )

          # End of the loops for combinations of levels of the subpopulation
        }
      }

      # End of the loop for response variables
    }

    # End of the loop for subpopulations
  }

  # Assign a value for the column in the output data frame that contains test
  # statistic values

  if (testname == "Wald") {
    names(contsum)[5] <- "Wald Statistic"
  } else if (testname == "adjWald") {
    names(contsum)[5] <- "Adjusted Wald Statistic"
  } else if (testname == "Chisq") {
    names(contsum)[5] <- "Rao-Scott First Order Statistic"
  } else {
    names(contsum)[5] <- "Rao-Scott Second Order Statistic"
  }

  # As necessary, modify dimension names

  row.names(contsum) <- 1:nrow(contsum)
  if (testname == "Chisq") {
    contsum <- contsum[, -7]
    names(contsum)[6] <- "Degrees_of_Freedom"
  }

  # As necessary, output a message indicating that warning messages were generated
  # during execution of the program

  if (warn_ind) {
    warn_df <<- warn_df
    if (nrow(warn_df) == 1) {
      message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
    } else {
      message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
    }
  }

  # Return the data frame

  contsum
}

context("CDF testing")

#
# Section for data input
#

# Read the data frame containing survey design and analysis variables
load(system.file("extdata", "NLA_IN.rda", package = "spsurvey"))

# Create a population size data frame
popsize <- data.frame(
  LAKE_ORGN = c("MAN_MADE", "NATURAL"),
  Total = c(6000, 14000)
)

# Create finite population correction factor objects
fpc1 <- 20000
fpc2a <- list(
  Urban = 5000,
  "Non-Urban" = 15000
)
fpc2b <- list(
  MAN_MADE = 6000,
  NATURAL = 14000
)
fpc3 <- c(
  Ncluster = 200,
  clusterID_1 = 100,
  clusterID_2 = 100,
  clusterID_3 = 100,
  clusterID_4 = 100
)
fpc4a <- list(
  Urban = c(
    Ncluster = 75,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  "Non-Urban" = c(
    Ncluster = 125,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)
fpc4b <- list(
  NATURAL = c(
    Ncluster = 130,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  MAN_MADE = c(
    Ncluster = 70,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)
#
# Section for the CDF inference function
#

# Assign response variable names to the vars vector
vars <- c("ContVar")

# Assign subpopulation variable names to the subpops vector
subpops <- c("LAKE_ORGN", "AG_ECO9")

# Perform tests
CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  testname = "adjWald"
)

test_that("CDF Inference: Unstratified single-stage analysis - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: Unstratified single-stage analysis - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  popsize = popsize, testname = "adjWald"
)

test_that("CDF Inference: with known population sizes - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   popsize = popsize, testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: with known population sizes - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  fpc = fpc1, testname = "adjWald"
)

test_that("CDF Inference: with finite population correction factor - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   fpc = fpc1, testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: with finite population correction factor - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "URBN_NLA17", testname = "adjWald"
)

test_that("CDF Inference: Stratified single-stage analysis - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   stratumID = "URBN_NLA17", testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: Stratified single-stage analysis - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "URBN_NLA17", popsize = popsize, testname = "adjWald"
)

test_that("CDF Inference: with known population sizes - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   stratumID = "URBN_NLA17", popsize = popsize, testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: with known population sizes - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "URBN_NLA17", fpc = fpc2a, testname = "adjWald"
)

test_that("CDF Inference: with finite population correction factor - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   stratumID = "URBN_NLA17", fpc = fpc2a,
#   testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: with finite population correction factor - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  clusterID = "clusterID", weight1 = "weight1", xcoord1 = "xcoord1",
  ycoord1 = "ycoord1", vartype = "SRS", testname = "adjWald"
)

test_that("CDF Inference: Unstratified two-stage analysis - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   clusterID = "clusterID", weight1 = "weight1", xcoord1 = "xcoord1",
#   ycoord1 = "ycoord1", vartype = "SRS", testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: Unstratified two-stage analysis - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  clusterID = "clusterID", weight1 = "weight1", xcoord1 = "xcoord1",
  ycoord1 = "ycoord1", popsize = popsize, vartype = "SRS", testname = "adjWald"
)

test_that("CDF Inference: with known population sizes - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   clusterID = "clusterID", weight1 = "weight1", xcoord1 = "xcoord1",
#   ycoord1 = "ycoord1", popsize = popsize, vartype = "SRS",
#   testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: with known population sizes - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  clusterID = "clusterID", weight1 = "weight1", xcoord1 = "xcoord1",
  ycoord1 = "ycoord1", fpc = fpc3, vartype = "SRS", testname = "adjWald"
)

test_that("CDF Inference: with finite population correction factor - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   clusterID = "clusterID", weight1 = "weight1", xcoord1 = "xcoord1",
#   ycoord1 = "ycoord1", fpc = fpc3, vartype = "SRS", testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: with finite population correction factor - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "URBN_NLA17", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", vartype = "SRS",
  testname = "adjWald"
)

test_that("CDF Inference: Stratified two-stage analysis - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   stratumID = "URBN_NLA17", clusterID = "clusterID", weight1 = "weight1",
#   xcoord1 = "xcoord1", ycoord1 = "ycoord1", vartype = "SRS",
#   testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: Stratified two-stage analysis - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "URBN_NLA17", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", popsize = popsize,
  vartype = "SRS", testname = "adjWald"
)

test_that("CDF Inference: with known population sizes - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   stratumID = "URBN_NLA17", clusterID = "clusterID", weight1 = "weight1",
#   xcoord1 = "xcoord1", ycoord1 = "ycoord1", popsize = popsize,
#   vartype = "SRS", testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: with known population sizes - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

CDF_Tests <- cont_cdftest(
  dframe = NLA_IN, vars = vars, subpops = subpops,
  siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "URBN_NLA17", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", fpc = fpc4a, vartype = "SRS",
  testname = "adjWald"
)

test_that("CDF Inference: with finite population correction factor - adjusted Wald", {
  expect_true(exists("CDF_Tests"))
  expect_equal(attributes(CDF_Tests)$class, "data.frame")
  expect_equal(nrow(CDF_Tests), 4)
})

# CDF_Tests <- cont_cdftest(
#   dframe = NLA_IN, vars = vars, subpops = subpops,
#   siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
#   stratumID = "URBN_NLA17", clusterID = "clusterID", weight1 = "weight1",
#   xcoord1 = "xcoord1", ycoord1 = "ycoord1", fpc = fpc4a, vartype = "SRS",
#   testname = "RaoScott_First"
# )
#
# test_that("CDF Inference: with finite population correction factor - RaoScott_First", {
#   expect_true(exists("CDF_Tests"))
#   expect_equal(attributes(CDF_Tests)$class, "data.frame")
#   expect_equal(nrow(CDF_Tests), 4)
# })

###############################################################################
# Function: ppd_plot (exported)
# Programmer: Tony Olsen
# Date: March 14, 2019
#
#' Plot power curves for panel designs
#'
#' Plot power curves and relative power curves for trend detection for set of
#' panel designs, time periods, indicators, significance levels and trend.  Trend
#' may be based on percent change per period in mean or percent change in
#' proportion of cumulative distribution function above or below a fixed cut
#' point. Types of plots are combinations of standard/relative, mean/percent,
#' period/change and design/indicator.  Input must be be of class
#' powerpaneldesign and is normally the output of function power_dsgn.
#'
#' @param object  List object of class \code{powerpaneldesign}. Object provides
#'   power calculated for a set of panel designs, set of indicators, set of
#'   trend values, and set of alpha values. Expect input as list as output from
#'   function \code{power_dsgn}.
#'
#' @param plot_type   Default is \code{"standard"} which plots standard power curve. If
#'   equal to \code{"relative"}, then plot power of one panel design compared to one or
#'   more other panel designs.
#'
#' @param trend_type Character value for trend in mean (\code{"mean"}) or or percent
#'   change in proportion (\code{"percent"}) of cumulative distribution function above
#'   or below a fixed cut point.  Default is \code{"mean"}.
#'
#' @param xaxis_type Character value equal to \code{"period"} or \code{"change"} which
#'   designates the type of x-axis for power plot where power is plotted on
#'   y-axis.  For \code{xaxis_type = "period"}, x-axis is periods in \code{dsgnpower}. If
#'   \code{xaxis_type = "change"}, then x-axis is percent per period with secondary
#'   x-axes for total percent per period and associated change in mean.
#'   Default is \code{"period"}.  Note that \code{xaxis_type} controls how the input for
#'   \code{"period"} and \code{"trend"} parameters is used.
#'
#' @param comp_type  Character value equal to \code{"design"} or \code{"indicator"} which
#'   designates the type of power curve comparison that will occur on a single
#'   plot.  If \code{comp_type = "design"}, then on a single plot of power curves all
#'   panel designs specified in \code{"dsgns"} are plotted for a single indicator,
#'   single trend value and single alpha.  If \code{comp_type = "indicator"}, then on a
#'   single plot of power curves all indicators specified in \code{"indicator"} are
#'   plotted for a single panel design, single trend value and single alpha.
#'   Default is \code{"design"}.
#'
#' @param dsgns  Vector of names of panel designs that are to be plotted.  Names
#'   must be all, or a subset of, names of designs in \code{dsgnpower}. Default is \code{NULL}
#'   which results in only the first panel design in \code{dsgnpower} being used.
#'
#' @param indicator  Vector of indicator names contained in \code{dsgnpower} that are
#'   to be plotted.  Indicator names must be all, or a subset of, indicator
#'   names in \code{dsgnpower}. Default is \code{NULL} which results in only the first
#'   indicator in \code{dsgnpower} being used.
#'
#' @param trend  \code{NULL}. A single value or vector of values contained in \code{dsgnpower}
#'   that will be plotted. Values must be all, or a subset of, trend values in
#'   \code{dsgnpower}. If \code{xaxis_type} is equal to \code{"period"}, then \code{NULL} results in maximum
#'   trend value being used and a single value or vector of values results in a
#'   separate plot for each value specified.  If \code{xaxis_type} is equal to
#'   \code{"change"}, then \code{NULL} results in all trend values in \code{dsgnpower} being plotted
#'   on x-axis and a vector of values results in all trend values in \code{dsgnpower}
#'   from minimum value to maximum value specified being plotted on x-axis.
#'
#' @param period \code{NULL}, a single value or vector of values contained in \code{dsgnpower}
#'   that will be plotted. Values must be all, or a subset of, period values in
#'   \code{dsgnpower}. If \code{xaxis_type} is equal to \code{"period"}, then \code{NULL} results in all
#'   time periods in \code{dsgnpower} being plotted on x-axis and a vector of values
#'   results in all period values in \code{dsgnpower} from minimum value to maximum
#'   value specified being plotted on x-axis. If \code{xaxis_type} is equal to
#'   \code{"change"}, then \code{NULL} results in all time periods in \code{dsgnpower} being plotted
#'   in separate plots and a vector of values results in time periods specified
#'   being plotted in separate plots.
#'
#' @param alpha A single value or vector of significance levels (as proportion,
#'   e.g. \code{0.05}) contained in \code{dsgnpower} to used for power plots. Specifying more
#'   than a single value results in multiple plots. Default is \code{NULL} which
#'   results in the minimum significance level in \code{dsgnpower} being used.
#'
#' @param ... Additional arguments (S3 consistency)
#'
#' @details By default the plot function produces a standard power curve at end
#'   of each time period on the x-axis with y-axis as power. When more than one
#'   panel design is in \code{dsgnpower}, the first panel design is used. When more than
#'   one indicator is in \code{dsgnpower}, the first indicator is used.  When more than
#'   one trend value is in \code{dsgnpower}, the maximum trend value is used. When more
#'   than one significance level, \code{alpha}, is in \code{dsgnpower}, the minimum
#'   significance level is used.
#'
#'   Control of the type of plot produced is governed by \code{plot_type}, \code{trend_type},
#'   \code{xaxis_type} and \code{comp_type}. The number of plots produced is governed by the
#'   number of panel designs (\code{dsgn}) specified, the number of indicators
#'   (\code{indicator}) specified, the number of time periods (\code{period}) specifies, the
#'   number of trend values (trend) specified and the number of significance
#'   levels (\code{alpha}) specified.
#'
#'   When the comparison type (\code{"comp_type"}) is equal to \code{"design"}, all power
#'   curves specified by dsgn are plotted on the same plot.  When \code{comp_type} is
#'   equal to \code{"indicator"}, all power curves specified by \code{"indicator"} are plotted
#'   on the same plot.  Typically, no more than 4-5 power curves should be
#'   plotted on same plot.
#'
#' @return  One or more power curve plots are created and plotted.  User must
#'   specify output graphical device if more than one plot is created.  See
#'   Devices for graphical output options.
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#'
#' @keywords survey
#'
#' @examples
#' \dontrun{
#' # Construct a rotating panel design with sample size of 60
#' R60N <- revisit_dsgn(20, panels = list(R60N = list(
#'   n = 60, pnl_dsgn = c(1, NA),
#'   pnl_n = NA, start_option = "None"
#' )), begin = 1)
#'
#' # Construct a fixed panel design with sample size of 60
#' F60 <- revisit_dsgn(20, panels = list(F60 = list(
#'   n = 60, pnl_dsgn = c(1, 0),
#'   pnl_n = NA, start_option = "None"
#' )), begin = 1)
#'
#' # Power for rotating panel with sample size 60
#' Power_tst <- power_dsgn("Variable_Name",
#'   ind_values = 43, unit_var = 280,
#'   period_var = 4, unitperiod_var = 40, index_var = 90,
#'   unit_rho = 1, period_rho = 0, paneldsgn = list(
#'     R60N = R60N, F60 = F60
#'   ), nrepeats = NULL,
#'   trend_type = "mean", trend = c(1.0, 2.0), alpha = 0.05
#' )
#' ppd_plot(Power_tst)
#' ppd_plot(Power_tst, dsgns = c("F60", "R60N"))
#' ppd_plot(Power_tst, dsgns = c("F60", "R60N"), trend = 1.0)
#' ppd_plot(Power_tst,
#'   plot_type = "relative", comp_type = "design",
#'   trend_type = "mean", trend = c(1, 2), dsgns = c("R60N", "F60"),
#'   indicator = "Variable_Name"
#' )
#' }
#' @export
###############################################################################
ppd_plot <- function(object, plot_type = "standard",
                     trend_type = "mean", xaxis_type = "period", comp_type = "design",
                     dsgns = NULL, indicator = NULL, trend = NULL, period = NULL, alpha = NULL, ...) {
  if (!inherits(object, "powerpaneldesign")) {
    stop("object must be output from spsurvey::power_dsgn()")
  }
  dsgnpower <- object
  # preserve current plot parameters
  oldpar <- par(mar = c(5.1, 4.1, 0.1, 0.1), oma = c(0, 0.1, 2.1, 0.1), xpd = TRUE)

  # extract names from dsgnpower
  dsgn_names <- dimnames(dsgnpower$dsgn_power)[[1]]
  period_names <- dimnames(dsgnpower$dsgn_power)[[2]]
  ind_names <- dimnames(dsgnpower$dsgn_power)[[3]]
  trend_names <- dimnames(dsgnpower$dsgn_power)[[4]]
  alpha_names <- dimnames(dsgnpower$dsgn_power)[[5]]

  # extract periods covered by panel designs
  period_values <- dsgnpower$period
  period_min <- min(period_values)
  period_max <- max(period_values)

  # check on plot_type to use
  if (!(plot_type %in% c("standard", "relative"))) {
    stop("\nplot_type is not standard or relative")
  }

  # check on trend_type
  if (!(trend_type %in% c("mean", "percent"))) {
    stop("\ntrend_type is not mean or percent")
  }

  # check on xaxis type
  if (!(xaxis_type %in% c("period", "change"))) {
    stop("\nType plot comparison must be period or change")
  }

  # check on comparison type for plot
  if (!(comp_type %in% c("design", "indicator"))) {
    stop("\nType plot comparison must be design or indicator.")
  }

  # check on designs to use
  if (!is.null(dsgns)) {
    if (any(!(dsgns %in% dsgn_names))) {
      stop("\nOne or more dsgn names not present in dsgnpower")
    }
    dsgn_plot <- dsgns
  } else {
    dsgn_plot <- dsgn_names[1]
  }

  # check on indicators to use
  if (!is.null(indicator)) {
    if (any(!(indicator %in% ind_names))) {
      stop("\nOne or more indicator names not present in dsgnpower")
    }
    ind_plot <- indicator
  } else {
    ind_plot <- ind_names[1]
  }

  # check on trend values to use
  if (!is.null(trend)) {
    # check to see that values are in dsgnpower
    if (!(all(round(trend, 1) %in% round(dsgnpower$trend, 1)))) {
      stop("\nOne or more trend values not present in dsgnpower")
    }
    if (xaxis_type == "period") {
      trend_value <- trend
    }
    if (xaxis_type == "change") {
      trend_value <- dsgnpower$trend[
        dsgnpower$trend >= min(trend) & dsgnpower$trend <= max(trend)
      ]
    }
  } else {
    # find maximum trend value to plot
    trend_value <- max(dsgnpower$trend)
  }
  trend_plot <- paste("Trend_", round(trend_value, 1), "%", sep = "")
  trend_names <- trend_plot

  # check on time periods to plot
  if (!is.null(period)) {
    if (any(!(period %in% period_values))) {
      stop("\nOne or more period values not present in dsgnpower")
    }
    if (xaxis_type == "period") {
      period_values <- dsgnpower$period[
        dsgnpower$period >= min(period) & dsgnpower$period <= max(period)
      ]
      period_plot <- period_values
      if (length(period) == 1) {
        period_values <- dsgnpower$period
      }
      period_min <- min(period_values)
      period_max <- max(period_values)
    }
    if (xaxis_type == "change") {
      period_plot <- period
    }
  } else {
    # extract periods covered by panel designs
    period_values <- dsgnpower$period
    period_min <- min(period_values)
    period_max <- max(period_values)
    period_plot <- period_max
  }

  # check on alphas to use
  if (!is.null(alpha)) {
    if (any(!(alpha %in% dsgnpower$alpha))) {
      stop("\nOne or more alpha values not present in dsgnpower")
    }
    alpha_value <- alpha
    alpha_plot <- paste("alpha_", alpha, sep = "")
  } else {
    # use minimum alpha value
    alpha_value <- min(dsgnpower$alpha)
    alpha_plot <- paste("alpha_", min(dsgnpower$alpha), sep = "")
  }

  n_dsgn <- length(dsgn_plot)
  n_period <- length(period_plot)
  n_ind <- length(ind_plot)
  n_trend <- length(trend_plot)
  n_alpha <- length(alpha_plot)

  ################################################################
  # Standard power plot section
  if (plot_type == "standard") {

    #########################
    # type of plot: type.comp: design
    if (comp_type == "design") {
      for (j in 1:n_alpha) {
        for (k in 1:n_ind) {
          if (xaxis_type == "period") {
            for (i in 1:n_trend) {
              # set xaxis values for period
              xset <- seq(period_min, by = 1, to = period_max)

              # set up initial plot area,  xaxis and yaxis
              plot(xset, seq(0, 1, length = length(xset)),
                ylim = c(0, 1), xlim = c(period_min, period_max),
                ylab = "", xlab = "", type = "n", axes = F
              )
              axis(side = 1, line = -0.75, at = xset, labels = xset, adj = 0.5, font = 3, cex = 1)
              axis(
                side = 2, line = 0, at = seq(0, 1, by = 0.2),
                labels = c("0.0", "0.2", "0.4", "0.6", "0.8", "1.0"),
                adj = 0.85, font = 3, cex = 1
              )
              mtext(outer = F, side = 1, line = 1.8, text = "End Period", cex = 1.5, font = 3)
              mtext(outer = F, side = 2, line = 2.5, text = "Power for Trend", cex = 1.5, font = 3)

              # legend for panel design power plotted
              # text identifying indicator, trend and alpha used for power
              text(period_min, 0.98, font = 3, cex = .8, adj = 0, paste0("Trend Type = ", trend_type))
              text(period_min, 0.94, font = 3, cex = .8, adj = 0, paste0("Trend = ", trend_value[i], "%"))
              text(period_min, 0.90, font = 3, cex = .8, adj = 0, paste0("alpha =", alpha_value[j]))
              text(period_min, 0.86, font = 3, cex = .8, adj = 0, paste0("Indicator = ", ind_plot[k]))
              legend(period_min, 0.82, dsgn_plot, lty = 1:n_dsgn, lwd = 2, seg.len = 4, bty = "n")

              # plot power curve for mth panel design
              ltype <- 1
              for (m in dsgn_plot[1:n_dsgn]) {
                pow <- dsgnpower$dsgn_power[
                  m, as.character(period_values),
                  ind_plot[k], trend_plot[i], alpha_plot[j]
                ]
                keep <- !is.na(pow)
                pow <- pow[keep]
                lines(period_values[keep], pow, lty = ltype, lwd = 2)
                ltype <- ltype + 1
              }
            }
          }
          #######
          if (xaxis_type == "change") {
            for (i in 1:n_period) {
              # set xaxis values for trend
              x <- trend_value
              xset <- trend_value
              xmin <- min(xset)
              xmax <- max(xset)
              xset_tot <- (period_plot[i] - period_min + 1) * xset
              xset_mean <- dsgnpower$trend.change[
                paste("Trend_", round(x, 1), "%", sep = ""),
                as.character(period_plot[i]), ind_plot[k]
              ]
              xlabel <- paste("Trend: %/period; Total % and Mean at Period ", period_plot[i], sep = "")
              # set up initial plot area,  xaxis and yaxis
              plot(xset, seq(0, 1, length = length(xset)),
                ylim = c(0, 1), xlim = c(xmin, xmax),
                ylab = "", xlab = "", type = "n", axes = F
              )
              axis(side = 1, line = -1, at = xset, labels = round(xset, 2), adj = 0.5, font = 3, cex = 1)
              axis(
                side = 1, line = 0, at = xset, tick = FALSE,
                labels = round(xset_tot, 1), adj = 0.5, font = 3, cex = 1
              )
              axis(
                side = 1, line = 1, at = xset, tick = FALSE,
                labels = round(xset_mean, 1), adj = 0.5, font = 3, cex = 1
              )
              axis(
                side = 2, line = 0, at = seq(0, 1, by = 0.2),
                labels = c("0.0", "0.2", "0.4", "0.6", "0.8", "1.0"),
                adj = 0.85, font = 3, cex = 1
              )
              mtext(outer = F, side = 1, line = 3.5, text = xlabel, cex = 1.5, font = 3)
              mtext(
                outer = F, side = 2, line = 2.5, text = "Power for Trend",
                cex = 1.5, font = 3, col = "Black", adj = 0.5
              )

              # text identifying indicator, trend and alpha used for power
              text(xmin, 0.98, font = 3, cex = .8, adj = 0, paste("Trend Type = ", trend_type, sep = ""))
              text(xmin, 0.94, font = 3, cex = .8, adj = 0, paste("End Period = ", period_plot[i], sep = ""))
              text(xmin, 0.90, font = 3, cex = .8, adj = 0, paste("alpha =", alpha_value[j]))
              text(xmin, 0.86, font = 3, cex = .8, adj = 0, paste("Indicator = ", ind_plot[k]))
              # legend for panel design power plotted
              legend(xmin, 0.82, dsgn_plot, lty = 1:n_dsgn, lwd = 2, seg.len = 4, bty = "n")

              # plot power curve for mth panel design
              ltype <- 1
              for (m in dsgn_plot[1:n_dsgn]) {
                pow <- dsgnpower$dsgn_power[
                  m, as.character(period_plot[i]),
                  ind_plot[k], trend_names, alpha_plot
                ]
                keep <- !is.na(pow)
                pow <- pow[keep]
                lines(x[keep], pow, lty = ltype, lwd = 2)
                ltype <- ltype + 1
              }
            }
          }
        }
      }
    }
    ####### end type.comp: design

    #######################################
    # type of plot: type.comp: indicator
    if (comp_type == "indicator") {
      for (j in 1:n_alpha) {
        for (k in 1:n_dsgn) {
          if (xaxis_type == "period") {
            for (i in 1:n_trend) {
              # set xaxis values for period
              xset <- seq(period_min, by = 1, to = period_max)

              # set up initial plot area,  xaxis and yaxis
              plot(xset, seq(0, 1, length = length(xset)),
                ylim = c(0, 1), xlim = c(period_min, period_max),
                ylab = "", xlab = "", type = "n", axes = F
              )
              axis(side = 1, line = -0.75, at = xset, labels = xset, adj = 0.5, font = 3, cex = 1)
              axis(
                side = 2, line = 0, at = seq(0, 1, by = 0.2),
                labels = c("0.0", "0.2", "0.4", "0.6", "0.8", "1.0"),
                adj = 0.85, font = 3, cex = 1
              )
              mtext(outer = F, side = 1, line = 1.8, text = "End period", cex = 1.5, font = 3)
              mtext(outer = F, side = 2, line = 2.5, text = "Power for Trend", cex = 1.5, font = 3)

              # legend for panel design power plotted
              # text identifying indicator, trend and alpha used for power
              text(period_min, 0.98, font = 3, cex = .8, adj = 0, paste("Trend Type = ", trend_type, sep = ""))
              text(period_min, 0.95, font = 3, cex = .8, adj = 0, paste("Trend = ", trend_value[i], sep = ""))
              text(period_min, 0.92, font = 3, cex = .8, adj = 0, paste("Alpha = ", alpha_value[j], sep = ""))
              text(period_min, 0.88, font = 3, cex = .8, adj = 0, paste("Design = ", dsgn_plot[k], sep = ""))
              legend(period_min, 0.85, ind_plot, lty = 1:n_ind, lwd = 2, seg.len = 4, bty = "n")

              # plot power curve for mth indicator
              ltype <- 1
              for (m in ind_plot[1:n_ind]) {
                pow <- dsgnpower$dsgn_power[
                  dsgn_plot[k], as.character(period_values),
                  m, trend_plot[i], alpha_plot[j]
                ]
                keep <- !is.na(pow)
                pow <- pow[keep]
                lines(period_values[keep], pow, lty = ltype, lwd = 2)
                ltype <- ltype + 1
              }
            }
          }
          #######
          if (xaxis_type == "change") {
            for (i in 1:n_period) {
              # set xaxis values for trend
              x <- trend_value
              xset <- trend_value
              xmin <- min(xset)
              xmax <- max(xset)
              xset_tot <- (period_plot[i] - period_min + 1) * xset
              if (n_ind == 1) {
                xset_mean <- dsgnpower$trend.change[
                  paste("Trend_", x, "%", sep = ""),
                  as.character(period_plot[i]), ind_plot[k]
                ]
                xlabel <- paste("Trend: %/period; Total % and Mean at Period ", period_plot[i], sep = "")
              } else {
                xlabel <- paste("Trend: %/period and Total % at Period ", period_plot[i], sep = "")
              }
              # set up initial plot area,  xaxis and yaxis
              plot(xset, seq(0, 1, length = length(xset)),
                ylim = c(0, 1), xlim = c(xmin, xmax),
                ylab = "", xlab = "", type = "n", axes = F
              )
              axis(side = 1, line = -1, at = xset, labels = round(xset, 2), adj = 0.5, font = 3, cex = 1)
              axis(
                side = 1, line = 0, at = xset, tick = FALSE,
                labels = round(xset_tot, 1), adj = 0.5, font = 3, cex = 1
              )
              if (n_ind == 1) {
                axis(
                  side = 1, line = 1, at = xset, tick = FALSE,
                  labels = round(xset_mean, 1), adj = 0.5, font = 3, cex = 1
                )
              }
              axis(
                side = 2, line = 0, at = seq(0, 1, by = 0.2),
                labels = c("0.0", "0.2", "0.4", "0.6", "0.8", "1.0"),
                adj = 0.85, font = 3, cex = 1
              )
              mtext(outer = FALSE, side = 1, line = 3.5, text = xlabel, cex = 1.5, font = 3)
              mtext(
                outer = FALSE, side = 2, line = 2.5, text = "Power for Trend",
                cex = 1.5, font = 3, col = c("Black"), adj = c(0.5)
              )

              # text identifying indicator, trend and alpha used for power
              text(xmin, 0.98, font = 3, cex = .8, adj = 0, paste("Trend Type = ", trend_type, sep = ""))
              text(xmin, 0.94, font = 3, cex = .8, adj = 0, paste("End Period = ", period_plot[i], sep = ""))
              text(xmin, 0.90, font = 3, cex = .8, adj = 0, paste("alpha =", alpha_value[j]))
              text(xmin, 0.86, font = 3, cex = .8, adj = 0, paste("Panel Design = ", dsgn_plot[k]))
              # legend for indicator power plotted
              legend(xmin, 0.82, ind_plot, lty = 1:n_ind, lwd = 2, seg.len = 4, bty = "n")

              # plot power curve for mth indicator
              ltype <- 1
              for (m in ind_plot[1:n_ind]) {
                pow <- dsgnpower$dsgn_power[
                  dsgn_plot[k], as.character(period_plot[i]),
                  m, trend_names, alpha_plot[j]
                ]
                keep <- !is.na(pow)
                pow <- pow[keep]
                lines(x[keep], pow, lty = ltype, lwd = 2)
                ltype <- ltype + 1
              }
            }
          }
        }
      }
    }
    ####### end comp_type: indicator
  }

  ##################################################
  # Relative power plot section
  if (plot_type == "relative") {
    # type of plot: type.comp: design
    if (comp_type == "design") {
      for (j in 1:n_alpha) {
        for (k in 1:n_ind) {
          if (xaxis_type == "period") {
            for (i in 1:n_trend) {
              # Determine y-axis minimum
              ymin <- -0.2
              # find panel_base power and plot power curve
              base_pow <- dsgnpower$dsgn_power[
                dsgn_plot[1], as.character(period_values),
                ind_plot[k], trend_plot[i], alpha_plot[j]
              ]
              for (m in dsgn_plot) {
                pow <- dsgnpower$dsgn_power[
                  m, as.character(period_values), ind_plot[k],
                  trend_plot[i], alpha_plot[j]
                ]
                ymin <- min(ymin, pow - base_pow, na.rm = TRUE)
              }
              ylow <- ceiling(-ymin * 10)
              ifelse(ylow %% 2 == 0, ylow <- -ylow / 10, ylow <- -ylow / 10 - 0.1)
              # set axix values for period
              xset <- seq(period_min, by = 1, to = period_max)
              # set up initial plot area,  xaxis and yaxis
              plot(xset, seq(0, 1, length = length(xset)),
                ylim = c(ylow, 1), xlim = c(period_min, period_max),
                ylab = "", xlab = "", type = "n", axes = F
              )
              axis(side = 1, line = -0.75, at = xset, labels = xset, adj = 0.5, font = 3, cex = 1)
              axis(
                side = 2, line = 0, at = seq(ylow, 1, by = 0.2),
                labels = round(seq(ylow, 1, by = 0.2), 1), adj = 0.85, font = 3, cex = 1
              )
              mtext(outer = F, side = 1, line = 1.8, text = "End Period", cex = 1.5, font = 3)
              mtext(
                outer = F, side = 2, line = 2.5, text = c("Relative Power", "Power for Trend"),
                cex = 1.5, font = 3, col = c("Black", "Blue"), adj = c(0.1, 0.8)
              )

              # text identifying indicator, trend and alpha used for power
              text(period_min, 0.98, font = 3, cex = .8, adj = 0, paste0("Trend Type = ", trend_type))
              text(period_min, 0.94, font = 3, cex = .8, adj = 0, paste0("Trend = ", trend_value[i]))
              text(period_min, 0.90, font = 3, cex = .8, adj = 0, paste0("alpha =", alpha_value[j]))
              text(period_min, 0.86, font = 3, cex = .8, adj = 0, paste0("Indicator = ", ind_plot[k]))
              # legend for panel design power plotted
              legend(period_min, 0.82, c(dsgn_plot[1], dsgn_plot),
                lty = c(1, 1:n_dsgn),
                col = c("blue", rep("black", length(dsgn_plot))), lwd = 2, seg.len = 5, bty = "n"
              )

              # find panel_base power and plot power curve
              base_pow <- dsgnpower$dsgn_power[
                dsgn_plot[1], as.character(period_values),
                ind_plot[k], trend_plot[i], alpha_plot[j]
              ]
              keep <- !is.na(base_pow)
              lines(period_values[keep], base_pow[keep], lty = 1, col = "blue", lwd = 2)

              ltype <- 1
              # plot relative power
              for (m in dsgn_plot) {
                pow <- dsgnpower$dsgn_power[
                  m, as.character(period_values), ind_plot[k],
                  trend_plot[i], alpha_plot[j]
                ]
                pow_diff <- pow - base_pow
                keep <- !is.na(pow_diff)
                lines(period_values[keep], pow_diff[keep], lty = ltype, lwd = 2)
                ltype <- ltype + 1
              }
            }
          }
          #######
          if (xaxis_type == "change") {
            for (i in 1:n_period) {
              # Determine y-axis minimum
              ymin <- -0.2
              # find panel_base power and plot power curve
              base_pow <- dsgnpower$dsgn_power[
                dsgn_plot[1], as.character(period_plot[i]),
                ind_plot[k], trend_names, alpha_plot[j]
              ]
              for (m in dsgn_plot) {
                pow <- dsgnpower$dsgn_power[
                  m, as.character(period_plot[i]),
                  ind_plot[k], trend_names, alpha_plot[j]
                ]
                ymin <- min(ymin, pow - base_pow, na.rm = TRUE)
              }
              ylow <- ceiling(-ymin * 10)
              ifelse(ylow %% 2 == 0, ylow <- -ylow / 10, ylow <- -ylow / 10 - 0.1)
              # set xaxis values for trend
              x <- trend_value
              xset <- trend_value
              xmin <- min(xset)
              xmax <- max(xset)
              xset_tot <- (period_plot[i] - period_min + 1) * xset
              xset_mean <- dsgnpower$trend.change[trend_names, as.character(period_plot[i]), ind_plot[k]]
              xlabel <- paste("Trend: %/Period; Total % and Mean at Period ", period_plot[i], sep = "")
              # set up initial plot area,  xaxis and yaxis
              plot(xset, seq(0, 1, length = length(xset)),
                ylim = c(ylow, 1), xlim = c(xmin, xmax),
                ylab = "", xlab = "", type = "n", axes = F
              )
              axis(side = 1, line = -1, at = xset, labels = round(xset, 2), adj = 0.5, font = 3, cex = 1)
              axis(
                side = 1, line = 0, at = xset, tick = FALSE,
                labels = round(xset_tot, 1), adj = 0.5, font = 3, cex = 1
              )
              axis(
                side = 1, line = 1, at = xset, tick = FALSE,
                labels = round(xset_mean, 1), adj = 0.5, font = 3, cex = 1
              )
              axis(
                side = 2, line = 0, at = seq(ylow, 1, by = 0.2),
                labels = round(seq(ylow, 1, by = 0.2), 1), adj = 0.85, font = 3, cex = 1
              )
              mtext(outer = F, side = 1, line = 3.2, text = xlabel, cex = 1.5, font = 3)
              mtext(
                outer = F, side = 2, line = 2.5, text = c("Relative Power", "Power for Trend"),
                cex = 1.5, font = 3, col = c("Black", "Blue"), adj = c(0.1, 0.8)
              )

              # text identifying indicator, trend and alpha used for power
              text(xmin, 0.98, font = 3, cex = .8, adj = 0, paste0("Trend Type = ", trend_type))
              text(xmin, 0.94, font = 3, cex = .8, adj = 0, paste0("End  Period = ", period_plot[i]))
              text(xmin, 0.90, font = 3, cex = .8, adj = 0, paste0("alpha =", alpha_value[j]))
              text(xmin, 0.86, font = 3, cex = .8, adj = 0, paste0("Indicator = ", ind_plot[k]))
              # legend for panel design power plotted
              legend(xmin, 0.82, c(dsgn_plot[1], dsgn_plot),
                lty = c(1, 1:n_dsgn),
                col = c("blue", rep("black", length(dsgn_plot))), lwd = 2, seg.len = 5, bty = "n"
              )

              # find panel_base power and plot power curve
              base_pow <- dsgnpower$dsgn_power[
                dsgn_plot[1], as.character(period_plot[i]),
                ind_plot[k], trend_names, alpha_plot[j]
              ]
              lines(x[!is.na(base_pow)], base_pow[!is.na(base_pow)], lty = 1, col = "blue", lwd = 2)
              # plot relative power
              ltype <- 1
              for (m in dsgn_plot) {
                pow <- dsgnpower$dsgn_power[
                  m, as.character(period_plot[i]),
                  ind_plot[k], trend_names, alpha_plot[j]
                ]
                pow_diff <- pow - base_pow
                lines(x[!is.na(pow_diff)], pow_diff[!is.na(pow_diff)], lty = ltype, lwd = 2)
                ltype <- ltype + 1
              }
            }
          }
        }
      }
    }
    ####### end comp_type: dsgn

    #######################################
    # type of plot: type.comp: indicator
    if (comp_type == "indicator") {
      for (j in 1:n_alpha) {
        for (k in 1:n_dsgn) {
          if (xaxis_type == "period") {
            for (i in 1:n_trend) {
              # Determine y-axis minimum
              ymin <- -0.2
              # find panel_base power and plot power curve
              base_pow <- dsgnpower$dsgn_power[
                dsgn_plot[k], as.character(period_values),
                ind_plot[1], trend_plot[i], alpha_plot[j]
              ]
              for (m in ind_plot) {
                pow <- dsgnpower$dsgn_power[
                  dsgn_plot[k], as.character(period_values),
                  m, trend_plot[i], alpha_plot[j]
                ]
                ymin <- min(ymin, pow - base_pow, na.rm = TRUE)
              }
              ylow <- ceiling(-ymin * 10)
              ifelse(ylow %% 2 == 0, ylow <- -ylow / 10, ylow <- -ylow / 10 - 0.1)
              # set axix values for period
              xset <- seq(period_min, by = 1, to = period_max)
              # set up initial plot area,  xaxis and yaxis
              plot(xset, seq(0, 1, length = length(xset)),
                ylim = c(ylow, 1), xlim = c(period_min, period_max),
                ylab = "", xlab = "", type = "n", axes = F
              )
              axis(side = 1, line = -0.75, at = xset, labels = xset, adj = 0.5, font = 3, cex = 1)
              axis(
                side = 2, line = 0, at = seq(ylow, 1, by = 0.2),
                labels = round(seq(ylow, 1, by = 0.2), 1), adj = 0.85, font = 3, cex = 1
              )
              mtext(outer = F, side = 1, line = 1.8, text = "End Period", cex = 1.5, font = 3)
              mtext(
                outer = F, side = 2, line = 2.5, text = c("Relative Power", "Power for Trend"),
                cex = 1.5, font = 3, col = c("Black", "Blue"), adj = c(0.1, 0.8)
              )

              # text identifying indicator, trend and alpha used for power
              text(period_min, 0.98, font = 3, cex = .8, adj = 0, paste0("Trend Type = ", trend_type))
              text(period_min, 0.94, font = 3, cex = .8, adj = 0, paste0("Trend = ", round(trend_value[i], 1), "%"))
              text(period_min, 0.90, font = 3, cex = .8, adj = 0, paste0("alpha =", alpha_value[j]))
              text(period_min, 0.86, font = 3, cex = .8, adj = 0, paste0("Panel Design = ", dsgn_plot[k]))
              # legend for panel design power plotted
              legend(period_min, 0.82, c(ind_plot[1], ind_plot),
                lty = c(1, 1:n_ind),
                col = c("blue", rep("black", length(ind_plot))), lwd = 2, seg.len = 5, bty = "n"
              )

              # find panel_base power and plot power curve
              base_pow <- dsgnpower$dsgn_power[
                dsgn_plot[k], as.character(period_values),
                ind_plot[1], trend_plot[i], alpha_plot[j]
              ]
              keep <- !is.na(base_pow)
              lines(period_values[keep], base_pow[keep], lty = 1, col = "blue", lwd = 2)

              ltype <- 1
              # plot relative power
              for (m in ind_plot) {
                pow <- dsgnpower$dsgn_power[
                  dsgn_plot[k], as.character(period_values),
                  m, trend_plot[i], alpha_plot[j]
                ]
                pow_diff <- pow - base_pow
                keep <- !is.na(pow_diff)
                lines(period_values[keep], pow_diff[keep], lty = ltype, lwd = 2)
                ltype <- ltype + 1
              }
            }
          }
          #######
          if (xaxis_type == "change") {
            for (i in 1:n_period) {
              # Determine y-axis minimum
              ymin <- -0.2
              # find panel_base power and plot power curve
              base_pow <- dsgnpower$dsgn_power[
                dsgn_plot[k], as.character(period_plot[i]),
                ind_plot[1], trend_names, alpha_plot[j]
              ]
              for (m in ind_plot) {
                pow <- dsgnpower$dsgn_power[
                  dsgn_plot[k], as.character(period_plot[i]),
                  m, trend_names, alpha_plot[j]
                ]
                ymin <- min(ymin, pow - base_pow, na.rm = TRUE)
              }
              ylow <- ceiling(-ymin * 10)
              ifelse(ylow %% 2 == 0, ylow <- -ylow / 10, ylow <- -ylow / 10 - 0.1)
              # set xaxis values for trend
              x <- trend_value
              xset <- trend_value
              xmin <- min(xset)
              xmax <- max(xset)
              xset_tot <- (period_plot[i] - period_min + 1) * xset
              xlabel <- paste("Trend: %/Period; Total % at Period ", period_plot[i], sep = "")
              # set up initial plot area,  xaxis and yaxis
              plot(xset, seq(0, 1, length = length(xset)),
                ylim = c(ylow, 1), xlim = c(xmin, xmax),
                ylab = "", xlab = "", type = "n", axes = F
              )
              axis(side = 1, line = -1, at = xset, labels = round(xset, 2), adj = 0.5, font = 3, cex = 1)
              axis(
                side = 1, line = 0, at = xset, tick = FALSE,
                labels = round(xset_tot, 1), adj = 0.5, font = 3, cex = 1
              )
              axis(
                side = 2, line = 0, at = seq(ylow, 1, by = 0.2),
                labels = round(seq(ylow, 1, by = 0.2), 1), adj = 0.85, font = 3, cex = 1
              )
              mtext(outer = F, side = 1, line = 2.2, text = xlabel, cex = 1.5, font = 3)
              mtext(
                outer = F, side = 2, line = 2.5, text = c("Relative Power", "Power for Trend"),
                cex = 1.5, font = 3, col = c("Black", "Blue"), adj = c(0.1, 0.8)
              )

              # text identifying indicator, trend and alpha used for power
              text(xmin, 0.98, font = 3, cex = .8, adj = 0, paste0("Trend Type = ", trend_type))
              text(xmin, 0.94, font = 3, cex = .8, adj = 0, paste0("End Period = ", period_plot[i]))
              text(xmin, 0.90, font = 3, cex = .8, adj = 0, paste0("alpha =", alpha_value[j]))
              text(xmin, 0.86, font = 3, cex = .8, adj = 0, paste0("Panel Design = ", dsgn_plot[k]))
              # legend for indicator power plotted
              legend(xmin, 0.82, c(ind_plot[1], ind_plot),
                lty = c(1, 1:n_ind),
                col = c("blue", rep("black", length(ind_plot))), lwd = 2, seg.len = 5, bty = "n"
              )

              # find panel_base power and plot power curve
              base_pow <- dsgnpower$dsgn_power[
                dsgn_plot[k], as.character(period_plot[i]),
                ind_plot[1], trend_names, alpha_plot[j]
              ]
              lines(x[!is.na(base_pow)], base_pow[!is.na(base_pow)], lty = 1, col = "blue", lwd = 2)
              # plot relative power
              ltype <- 1
              for (m in ind_plot) {
                pow <- dsgnpower$dsgn_power[
                  dsgn_plot[k], as.character(period_plot[i]),
                  m, trend_names, alpha_plot[j]
                ]
                pow_diff <- pow - base_pow
                lines(x[!is.na(pow_diff)], pow_diff[!is.na(pow_diff)], lty = ltype, lwd = 2)
                ltype <- ltype + 1
              }
            }
          }
        }
      }
    }
    ####### end comp_type: indicator
  }

  # end of relative power plot section

  # reset plot parameters
  par(oldpar)
}

################################################################################
# Function: survey_design (not exported)
# Programmer: Tom Kincaid
# Date: December 16, 2020
# Revised: May 14 2021 to rename the popcorrect argument to fpcfactor_ind
#
#' Create a Survey Design Object
#'
#' This function creates a survey design object.  The function organizes input
#' and then calls the svydesign function in the survey package to create an
#' object of class survey.design2.
#'
#' @param dframe Data frame containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param siteID Character value providing name of the site ID variable in the
#'   \code{dframe} data frame.  For a two-stage sample, the site ID variable
#'   identifies stage two site IDs.
#'
#' @param weight Character value providing name of the survey design weight
#'   variable in the \code{dframe} data frame.  For a two-stage sample, the
#'   weight variable identifies stage two weights.
#'
#' @param stratum_ind Logical value that indicates whether the sample is
#'   stratified, where \code{TRUE} = a stratified sample and \code{FALSE} = an
#'   unstratified sample.
#'
#' @param stratumID Character value providing name of the stratum ID variable in
#'   the \code{dframe} data frame.
#'
#' @param cluster_ind Logical value that indicates whether the sample contains
#'   two stages, where \code{TRUE} = a two-stage sample and \code{FALSE} = a
#'   single stage sample.
#'
#' @param clusterID Character value providing name of the cluster (stage one) ID
#'   variable in the \code{dframe} data frame.  Note that cluster IDs are
#'   required for a two-stage sample.
#'
#' @param weight1 Character value providing name of the stage one weight
#'   variable in the \code{dframe} data frame.
#'
#' @param sizeweight Logical value that indicates whether size weights should be
#'   used during estimation, where \code{TRUE} = use size weights and
#'   \code{FALSE} = do not use size weights.
#'
#' @param sweight Character value providing name of the size weight variable in
#'   the \code{dframe} data frame.  For a two-stage sample, the size weight
#'   variable identifies stage two size weights.
#'
#' @param sweight1 Character value providing name of the stage one size weight
#'   variable in the \code{dframe} data frame.
#'
#' @param fpcfactor_ind Logical value that indicates whether the finite
#'   population correction factor is used during variance estimation.
#'
#' @param fpcsize Character value providing name of the variable in the
#'   \code{dframe} data frame that identifies size of the resource, which is
#'   required for calculation of the finite population correction factor for a
#'   single-stage sample.
#'
#' @param Ncluster Character value providing name of the variable in the
#'   \code{dframe} data frame that identifies the number of clusters (stage one
#'   sampling units) in the resource, which is required for calculation of the
#'   finite population correction factor for a two-stage sample.  This argument
#'   is also required for a two-stage sample when the popsize argument is not
#'   equal to \code{NULL} and the \code{vartype} argument equals \code{"Local"}.
#'
#' @param stage1size Character value providing name of the variable in the
#'   \code{dframe} data frame that identifies cluster size, i.e. the number of
#'   the stage two sampling units in the resource for a cluster.  Note that
#'   cluster size is required for calculation of the finite population
#'   correction factor for a two-stage sample.
#'
#' @param vartype Character value providing choice of the variance estimator,
#'   where \code{"Local"} = the local mean estimator, \code{"SRS"} = the simple
#'   random sampling estimator, \code{"HT"} = the Horvitz-Thompson estimator,
#'   and \code{"YG"} = the Yates-Grundy estimator.
#'
#' @param jointprob Character value providing choice of joint inclusion
#'   probability approximation for use with Horvitz-Thompson and Yates-Grundy
#'   variance estimators, where \code{"overton"} indicates the Overton
#'   approximation, \code{"hr"} indicates the Hartley_Rao approximation, and
#'   \code{"brewer"} equals the Brewer approximation.
#'
#' @return An object of class \code{survey.design2}.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

survey_design <- function(dframe, siteID, weight, stratum_ind, stratumID,
                          cluster_ind, clusterID, weight1, sizeweight, sweight,
                          sweight1, fpcfactor_ind, fpcsize, Ncluster,
                          stage1size, vartype, jointprob) {
  if (vartype %in% c("Local", "SRS")) {
    if (stratum_ind) {
      if (cluster_ind) {
        if (sizeweight) {
          dframe$wgt1 <- dframe[, weight1] * dframe[, sweight1]
          dframe$wgt2 <- dframe[, weight] * dframe[, sweight]
          if (fpcfactor_ind) {
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              weights = ~ wgt1 + wgt2,
              strata = make.formula(stratumID),
              nest = TRUE,
              fpc = make.formula(paste(Ncluster, stage1size, sep = " + ")),
              data = dframe
            )
          } else {
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              weights = ~ wgt1 + wgt2,
              strata = make.formula(stratumID),
              nest = TRUE,
              data = dframe
            )
          }
        } else {
          dframe$wgt1 <- dframe[, weight1]
          dframe$wgt2 <- dframe[, weight]
          if (fpcfactor_ind) {
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              weights = ~ wgt1 + wgt2,
              strata = make.formula(stratumID),
              nest = TRUE,
              fpc = make.formula(paste(Ncluster, stage1size, sep = " + ")),
              data = dframe
            )
          } else {
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              weights = ~ wgt1 + wgt2,
              strata = make.formula(stratumID),
              nest = TRUE,
              data = dframe
            )
          }
        }
      } else {
        if (sizeweight) {
          dframe$wgt <- dframe[, weight] * dframe[, sweight]
          if (fpcfactor_ind) {
            design <- svydesign(
              ids = make.formula(siteID),
              weights = ~wgt,
              strata = make.formula(stratumID),
              fpc = make.formula(fpcsize),
              data = dframe
            )
          } else {
            design <- svydesign(
              ids = make.formula(siteID),
              weights = ~wgt,
              strata = make.formula(stratumID),
              data = dframe
            )
          }
        } else {
          dframe$wgt <- dframe[, weight]
          if (fpcfactor_ind) {
            design <- svydesign(
              ids = make.formula(siteID),
              weights = ~wgt,
              strata = make.formula(stratumID),
              fpc = make.formula(fpcsize),
              data = dframe
            )
          } else {
            design <- svydesign(
              ids = make.formula(siteID),
              weights = ~wgt,
              strata = make.formula(stratumID),
              data = dframe
            )
          }
        }
      }
    } else {
      if (cluster_ind) {
        if (sizeweight) {
          dframe$wgt1 <- dframe[, weight1] * dframe[, sweight1]
          dframe$wgt2 <- dframe[, weight] * dframe[, sweight]
          if (fpcfactor_ind) {
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              weights = ~ wgt1 + wgt2,
              fpc = make.formula(paste(Ncluster, stage1size, sep = " + ")),
              data = dframe
            )
          } else {
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              weights = ~ wgt1 + wgt2,
              data = dframe
            )
          }
        } else {
          dframe$wgt1 <- dframe[, weight1]
          dframe$wgt2 <- dframe[, weight]
          if (fpcfactor_ind) {
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              weights = ~ wgt1 + wgt2,
              fpc = make.formula(paste(Ncluster, stage1size, sep = " + ")),
              data = dframe
            )
          } else {
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              weights = ~ wgt1 + wgt2,
              data = dframe
            )
          }
        }
      } else {
        if (sizeweight) {
          dframe$wgt <- dframe[, weight] * dframe[, sweight]
          if (fpcfactor_ind) {
            design <- svydesign(
              ids = make.formula(siteID),
              weights = ~wgt,
              fpc = make.formula(fpcsize),
              data = dframe
            )
          } else {
            design <- svydesign(
              ids = make.formula(siteID),
              weights = ~wgt,
              data = dframe
            )
          }
        } else {
          dframe$wgt <- dframe[, weight]
          if (fpcfactor_ind) {
            design <- svydesign(
              ids = make.formula(siteID),
              weights = ~wgt,
              fpc = make.formula(fpcsize),
              data = dframe
            )
          } else {
            design <- svydesign(
              ids = make.formula(siteID),
              weights = ~wgt,
              data = dframe
            )
          }
        }
      }
    }
  } else {
    if (jointprob %in% c("overton", "hr")) {
      if (stratum_ind) {
        if (cluster_ind) {
          if (sizeweight) {
            dframe$prb1 <- 1 / (dframe[, weight1] * dframe[, sweight1])
            dframe$prb2 <- 1 / (dframe[, weight] * dframe[, sweight])
            if (jointprob == "overton") {
              design <- svydesign(
                ids = make.formula(paste(clusterID, siteID, sep = " + ")),
                probs = ~ prb1 + prb2,
                strata = make.formula(stratumID),
                nest = TRUE,
                variance = vartype,
                pps = jointprob,
                data = dframe
              )
            } else {
              design <- svydesign(
                ids = make.formula(paste(clusterID, siteID, sep = " + ")),
                probs = ~ prb1 + prb2,
                strata = make.formula(stratumID),
                nest = TRUE,
                variance = vartype,
                pps = HR(),
                data = dframe
              )
            }
          } else {
            dframe$prb1 <- 1 / dframe[, weight1]
            dframe$prb2 <- 1 / dframe[, weight]
            if (jointprob == "overton") {
              design <- svydesign(
                ids = make.formula(paste(clusterID, siteID, sep = " + ")),
                probs = ~ prb1 + prb2,
                strata = make.formula(stratumID),
                nest = TRUE,
                variance = vartype,
                pps = jointprob,
                data = dframe
              )
            } else {
              design <- svydesign(
                ids = make.formula(paste(clusterID, siteID, sep = " + ")),
                probs = ~ prb1 + prb2,
                strata = make.formula(stratumID),
                nest = TRUE,
                variance = vartype,
                pps = HR(),
                data = dframe
              )
            }
          }
        } else {
          if (sizeweight) {
            dframe$prb <- 1 / (dframe[, weight] * dframe[, sweight])
            if (jointprob == "overton") {
              design <- svydesign(
                ids = make.formula(siteID),
                probs = ~prb,
                strata = make.formula(stratumID),
                variance = vartype,
                pps = jointprob,
                data = dframe
              )
            } else {
              design <- svydesign(
                ids = make.formula(siteID),
                probs = ~prb,
                strata = make.formula(stratumID),
                variance = vartype,
                pps = HR(),
                data = dframe
              )
            }
          } else {
            dframe$prb <- 1 / dframe[, weight]
            if (jointprob == "overton") {
              design <- svydesign(
                ids = make.formula(siteID),
                probs = ~prb,
                strata = make.formula(stratumID),
                variance = vartype,
                pps = jointprob,
                data = dframe
              )
            } else {
              design <- svydesign(
                ids = make.formula(siteID),
                probs = ~prb,
                strata = make.formula(stratumID),
                variance = vartype,
                pps = HR(),
                data = dframe
              )
            }
          }
        }
      } else {
        if (cluster_ind) {
          if (sizeweight) {
            dframe$prb1 <- 1 / (dframe[, weight1] * dframe[, sweight1])
            dframe$prb2 <- 1 / (dframe[, weight] * dframe[, sweight])
            if (jointprob == "overton") {
              design <- svydesign(
                ids = make.formula(paste(clusterID, siteID, sep = " + ")),
                probs = ~ prb1 + prb2,
                variance = vartype,
                pps = jointprob,
                data = dframe
              )
            } else {
              design <- svydesign(
                ids = make.formula(paste(clusterID, siteID, sep = " + ")),
                probs = ~ prb1 + prb2,
                variance = vartype,
                pps = HR(),
                data = dframe
              )
            }
          } else {
            dframe$prb1 <- 1 / dframe[, weight1]
            dframe$prb2 <- 1 / dframe[, weight]
            if (jointprob == "overton") {
              design <- svydesign(
                ids = make.formula(paste(clusterID, siteID, sep = " + ")),
                probs = ~ prb1 + prb2,
                variance = vartype,
                pps = jointprob,
                data = dframe
              )
            } else {
              design <- svydesign(
                ids = make.formula(paste(clusterID, siteID, sep = " + ")),
                probs = ~ prb1 + prb2,
                variance = vartype,
                pps = HR(),
                data = dframe
              )
            }
          }
        } else {
          if (sizeweight) {
            dframe$prb <- 1 / (dframe[, weight] * dframe[, sweight])
            if (jointprob == "overton") {
              design <- svydesign(
                ids = make.formula(siteID),
                probs = ~prb,
                variance = vartype,
                pps = jointprob,
                data = dframe
              )
            } else {
              design <- svydesign(
                ids = make.formula(siteID),
                probs = ~prb,
                variance = vartype,
                pps = HR(),
                data = dframe
              )
            }
          } else {
            dframe$prb <- 1 / dframe[, weight]
            if (jointprob == "overton") {
              design <- svydesign(
                ids = make.formula(siteID),
                probs = ~prb,
                variance = vartype,
                pps = jointprob,
                data = dframe
              )
            } else {
              design <- svydesign(
                ids = make.formula(siteID),
                probs = ~prb,
                variance = vartype,
                pps = HR(),
                data = dframe
              )
            }
          }
        }
      }
    } else {
      if (stratum_ind) {
        if (cluster_ind) {
          if (sizeweight) {
            dframe$prb1 <- 1 / (dframe[, weight1] * dframe[, sweight1])
            dframe$prb2 <- 1 / (dframe[, weight] * dframe[, sweight])
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              probs = ~ prb1 + prb2,
              strata = make.formula(stratumID),
              nest = TRUE,
              variance = vartype,
              pps = jointprob,
              data = dframe
            )
          } else {
            dframe$prb1 <- 1 / dframe[, weight1]
            dframe$prb2 <- 1 / dframe[, weight]
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              probs = ~ prb1 + prb2,
              strata = make.formula(stratumID),
              nest = TRUE,
              variance = vartype,
              pps = jointprob,
              data = dframe
            )
          }
        } else {
          if (sizeweight) {
            dframe$prb <- 1 / (dframe[, weight] * dframe[, sweight])
            design <- svydesign(
              ids = make.formula(siteID),
              probs = ~prb,
              strata = make.formula(stratumID),
              variance = vartype,
              pps = jointprob,
              data = dframe
            )
          } else {
            dframe$prb <- 1 / dframe[, weight]
            design <- svydesign(
              ids = make.formula(siteID),
              probs = ~prb,
              strata = make.formula(stratumID),
              variance = vartype,
              pps = jointprob,
              data = dframe
            )
          }
        }
      } else {
        if (cluster_ind) {
          if (sizeweight) {
            dframe$prb1 <- 1 / (dframe[, weight1] * dframe[, sweight1])
            dframe$prb2 <- 1 / (dframe[, weight] * dframe[, sweight])
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              probs = ~ prb1 + prb2,
              variance = vartype,
              pps = jointprob,
              data = dframe
            )
          } else {
            dframe$prb1 <- 1 / dframe[, weight1]
            dframe$prb2 <- 1 / dframe[, weight]
            design <- svydesign(
              ids = make.formula(paste(clusterID, siteID, sep = " + ")),
              probs = ~ prb1 + prb2,
              variance = vartype,
              pps = jointprob,
              data = dframe
            )
          }
        } else {
          if (sizeweight) {
            dframe$prb <- 1 / (dframe[, weight] * dframe[, sweight])
            design <- svydesign(
              ids = make.formula(siteID),
              probs = ~prb,
              variance = vartype,
              pps = jointprob,
              data = dframe
            )
          } else {
            dframe$prb <- 1 / dframe[, weight]
            design <- svydesign(
              ids = make.formula(siteID),
              probs = ~prb,
              variance = vartype,
              pps = jointprob,
              data = dframe
            )
          }
        }
      }
    }
  }

  design
}

###############################################################################
# Function: cov_panel_dsgn (exported)
# Programmers: Tom Kincaid
#              Tony Olsen
# Date: March 14, 2019
#
#'
#' Create a covariance matrix for a panel design
#'
#' Covariance structure accounts for the panel design and the four variance
#' components: unit variation, period variation, unit by period interaction
#' variation and index (or residual) variation. The model incorporates unit,
#' period, unit by period, and index variance components. It also includes a
#' provision for unit correlation and period autocorrelation.
#'
#' @param paneldsgn  A matrix (dimensions: number of panels (rows) by number of
#'   periods (columns)) containing the number of units visited for each
#'   combination of panel and period. Default is matrix(50, 1, 10) which is a
#'   single panel of 50 units visited 10 times, typical time is a period.
#'
#' @param nrepeats   Either \code{NULL} or a list of matrices the same length as
#'   paneldsgn specifying the number of revisits made to units in a panel in the
#'   same period for each design.  Specifying \code{NULL} indicates that number of
#'   revisits to units is the same for all panels and for all periods and for
#'   all panel designs. The default is \code{NULL}, a single visit. Names must match
#'   list names in \code{paneldsgn}.
#'
#' @param unit_var   The variance component estimate for unit. The default is
#'   \code{NULL}.
#'
#' @param period_var   The variance component estimate for period The default is
#'   \code{NULL}.
#'
#' @param unitperiod_var The variance component estimate for unit by period
#'   interaction. The default is \code{NULL}.
#'
#' @param index_var  The variance component estimate for index error. The
#'   default is \code{NULL}.
#'
#' @param unit_rho   Unit correlation across periods. The default is \code{1}.
#'
#' @param period_rho   Period autocorrelation. The default is \code{0}.
#'
#' @details Covariance structure accounts for the panel design and the four
#'   variance components: unit variation, period variation, unit by period
#'   interaction variation and index (or residual) variation. Uses the model
#'   structure defined by Urquhart 2012.
#'
#'   If \code{nrepeats} is \code{NULL}, then no units sampled more than once in a specific
#'   panel, period combination) and then unit by period and index variances are
#'   added together or user may have only estimated unit, period and unit by
#'   period variance components so that index component is zero. It calculates
#'   the covariance matrix for the simple linear regression. The standard error
#'   for a linear trend coefficient is the square root of the variance.
#'
#' @references
#'   Urquhart, N. S., W. S. Overton, et al. (1993) Comparing sampling designs
#'   for monitoring ecological status and trends: impact of temporal patterns.
#'   In: \emph{Statistics for the Environment.} V. Barnett and K. F. Turkman.
#'   John Wiley & Sons, New York, pp. 71-86.
#'
#'   Urquhart, N. S. and T. M. Kincaid (1999). Designs for detecting trends
#'   from repeated surveys of ecological resources. \emph{Journal of
#'   Agricultural, Biological, and Environmental Statistics}, \bold{4(4)},
#'   404-414.
#'
#'   Urquhart, N. S. (2012). The role of monitoring design in detecting trend in
#'   long-term ecological monitoring studies. In: \emph{Design and Analysis of
#'   Long-term Ecological Monitoring Studies.} R. A. Gitzen, J. J. Millspaugh,
#'   A. B. Cooper, and D. S. Licht (eds.). Cambridge University Press, New York,
#'   pp. 151-173.
#
#' @return A list containing the covariance matrix (\code{cov}) for the panel design,
#'   the input panel design (\code{paneldsgn}), the input \code{nrepeats} design
#'   (\code{nrepeats.dsgn}) and the function call.
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#' @seealso
#'   \describe{
#'     \item{\code{\link{power_dsgn}}}{ for power calculations of multiple panel
#'       designs}
#'   }
#'
#' @keywords survey
#'
#' @export
###############################################################################

cov_panel_dsgn <- function(paneldsgn = matrix(50, 1, 10), nrepeats = 1,
                           unit_var = NULL, period_var = NULL, unitperiod_var = NULL, index_var = NULL,
                           unit_rho = 1, period_rho = 0) {
  paneldsgn <- as.matrix(paneldsgn)
  nperiod <- ncol(paneldsgn)
  npanel <- nrow(paneldsgn)

  if (any(is.null(unit_var), is.null(period_var), is.null(unitperiod_var), is.null(index_var))) {
    stop("Must provide four variance components. index_var may be zero if nrepeats=1")
  }

  # Create covariance matrix for period correlation
  rhomat <- matrix(0, nrow = nperiod, ncol = nperiod)
  if (nperiod > 1) {
    rhomat[1, ] <- 0:(nperiod - 1)
    for (i in 2:nperiod) {
      rhomat[i, ] <- c(rev(1:(i - 1)), 0:(nperiod - i))
    }
  }

  period_cov <- period_var * (period_rho^rhomat)

  # Create covariance matrix for unit correlation
  unit_cov <- unit_var * (unit_rho^rhomat)

  # Construct panel covariance term
  # nunits is number of units in a panel
  nunits <- apply(paneldsgn, 1, max)
  if (npanel > 1) {
    pan_cov <- kronecker(unit_cov, diag(ifelse(nunits > 0, 1 / nunits, 0)))
  } else {
    pan_cov <- unit_cov / nunits
  }

  # construct period covariance term
  yr_cov <- kronecker(period_cov, matrix(1, nrow = npanel, ncol = npanel))

  # Construct unit by period covariance term
  if (length(nunits) > 1) {
    sy_cov <- unitperiod_var * kronecker(diag(nperiod), diag(ifelse(nunits > 0, 1 / nunits, 0)))
  } else {
    sy_cov <- unitperiod_var * diag(nperiod) / nunits
  }

  # Construct index covariance term
  # Create nrepeats revisit design if necessary
  if (is.null(nrepeats)) {
    nrepeats <- vector("list", length(paneldsgn))
    for (i in names(paneldsgn)) {
      nrepeats[[i]] <- paneldsgn[[i]]
      nrepeats[[i]][nrepeats[[i]] > 0] <- 1
    }
  }
  if (npanel > 1) {
    index_cov <- index_var * kronecker(
      diag(nperiod),
      diag(ifelse(nunits > 0, 1 / (nunits * apply(nrepeats, 1, max)), 0))
    )
  } else {
    index_cov <- index_var * diag(nperiod) / nunits
  }

  # overall covariance matrix
  phi <- pan_cov + yr_cov + sy_cov + index_cov

  cov <- list(cov = phi, paneldsgn = paneldsgn, nrepeat_dsgn = nrepeats, call = sys.call())
  return(cov)
}

#' \code{sp_frame} objects
#' 
#' @description Turn sampling frames or analysis data into an \code{sp_frame} object
#'   or transform \code{sp_frame} objects back into their original object.
#'
#' @name sp_frame 
#'
#' @param frame A sampling frame or analysis data
#'
#' @return An \code{sp_frame} object.
#' 
#' @details The \code{sp_frame()} function assigns \code{frame} class \code{sp_frame}
#'   to be used by \code{summary()} and \code{plot()}. \code{sp_frame()} objects
#'   can sometimes clash with other sf and tidyverse generics, so \code{un_spframe()} removes
#'   class \code{sp_frame()}, leaving the original classes of \code{frame} intact.
#' 
#' @export
#'
#' @examples
#' NE_Lakes <- sp_frame(NE_Lakes)
#' class(NE_Lakes)
#' NE_Lakes <- sp_unframe(NE_Lakes)
#' class(NE_Lakes)
sp_frame <- function(frame) {
  new_sp_frame <- structure(frame, class = c("sp_frame", class(frame)))
}

#' @name sp_frame
#' @param sp_frame An \code{sp_frame} object.
#' @export
sp_unframe <- function(sp_frame) {
  new_sp_unframe <- structure(sp_frame, class = setdiff(class(sp_frame), "sp_frame"))
}

################################################################################
# Function: changevar_total (not exported)
# Programmer: Tom Kincaid
# Date: July 22, 2020
# Revised: April 28, 2021 to use the SRS estimator when the local mean estimator
#          fails to produce a valid estimate
# Revised: May 3 2021 to correct an error that occurs when warning messages for
#          unstratified samples are added to the warn_df data frame
# Revised: June 8 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
#
#' Covariance or Correlation Matrix Estimate of Change in Class Resource Sizes
#' between Two Surveys
#'
#' This function uses the repeat visit sites for two probability surveys to
#' calculate either covariance or correlation estimates of estimated change in
#' resource size in each of a set of categories.  Covariance estimates are
#' calculated when the resivit sites have the same survey design weight in both
#' surveys.  Correlation estimates are calculated when the revisit sites do not
#' have the same weight in both surveys, in which case the sites are assigned
#' equal weights.  The revisitwgt argument controls whether covariance or
#' correlation estimates are calculated.  Either the simple random sampling
#' (SRS) variance/covariance estimator or the local mean variance/covariance
#' estimator is calculated, which is subject to user control.  The simple random
#' sampling variance/covariance estimator uses the independent random sample
#' approximation to calculate joint inclusion probabilities.  The function can
#' accomodate single-stage and two-stage samples.
#'
#' @param catvar_levels Vector of the set of categorical response values.
#'
#' @param catvar1 Vector of the response value for each site for survey one.
#'
#' @param catvar2 Vector of the response value for each site for survey two.
#'
#' @param wgt Vector of the final adjusted weight (reciprocal of the sample
#'   inclusion probability) for each site, which is either the weight for a
#'   single-stage sample or the stage two weight for a two-stage sample.
#'
#' @param x Vector of x-coordinate for location for each site, which is either
#'   the x- coordinate for a single-stage sample or the stage two x-coordinate
#'   for a two-stage sample.
#'
#' @param y Vector of y-coordinate for location for each site, which is either
#'   the y- coordinate for a single-stage sample or the stage two y-coordinate
#'   for a two-stage sample.
#'
#' @param revisitwgt Logical value that indicates whether each repeat visit site
#'   has the same survey design weight in the two surveys, where \code{TRUE} =
#'   the weight for each repeat visit site is the same and \code{FALSE} = the
#'   weight for each repeat visit site is not the same.  When this argument is
#'   \code{FALSE}, all of the repeat visit sites are assigned equal weights when
#'   calculating the covariance component of the change estimate standard error.
#'
#' @param size1 The set of category size estimates for survey one.
#'
#' @param size2 The set of category size estimates for survey two.
#'
#' @param stratum_ind Logical value that indicates whether the sample is
#'   stratified, where \code{TRUE} = a stratified sample and \code{FALSE} = not
#'   a stratified sample.
#'
#' @param stratum_level The stratum level.
#'
#' @param cluster_ind  Logical value that indicates whether the sample is a two-
#'   stage sample, where \code{TRUE} = a two-stage sample and \code{FALSE} = not
#'   a two-stage sample.
#'
#' @param clusterID Vector of the stage one sampling unit (primary sampling unit
#'   or cluster) code for each site.
#'
#' @param wgt1 Vector of the final adjusted stage one weight for each site.
#'
#' @param x1 Vector of the stage one x-coordinate for location for each site.
#'
#' @param y1 Vector of the stage one y-coordinate for location for each site.
#'
#' @param vartype The choice of variance estimator, where \code{"Local"} = local
#'   mean estimator and \code{"SRS"} = SRS estimator.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated, where \code{TRUE} = warning messages were generated and
#'   \code{FALSE} = warning messages were not generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @param warn_vec Vector that contains names of the population type, the
#'   subpopulation, and an indicator.
#'
#' @return An object in list format composed of a vector named \code{rslt},
#'   which contains the covariance or correlation estimates, a logical variable
#'   named \code{warn_ind}, which is the indicator for warning messges, and a
#'   data frame named \code{warn_df}, which contains warning messages.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

changevar_total <- function(catvar_levels, catvar1, catvar2, wgt, x, y,
                            revisitwgt, size1, size2, stratum_ind,
                            stratum_level, cluster_ind, clusterID, wgt1, x1, y1,
                            vartype, warn_ind, warn_df, warn_vec) {

  # Assign the function name
  fname <- "changevar_total"

  #
  # Calculate covariance or correlation using the repeat visit sites
  #

  # Begin the section for a two-stage sample

  if (cluster_ind) {

    # Calculate additional required values
    m <- length(catvar_levels)
    cluster <- factor(clusterID)
    cluster_levels <- levels(cluster)
    ncluster <- length(cluster_levels)
    catvar1_lst <- split(catvar1, cluster)
    catvar2_lst <- split(catvar2, cluster)
    wgt2_lst <- split(wgt, cluster)
    wgt1_u <- as.vector(tapply(wgt1, cluster, unique))
    if (vartype == "Local") {
      x2_lst <- split(x, cluster)
      y2_lst <- split(y, cluster)
      x1_u <- as.vector(tapply(x1, cluster, unique))
      y1_u <- as.vector(tapply(y1, cluster, unique))
    }
    var_ind <- sapply(split(cluster, cluster), length) > 1

    # Loop through each category level

    rslt <- rep(NA, m)
    for (k in 1:m) {

      # Determine whether the categorical level is present in both surveys

      if (is.na(size1[k]) | is.na(size2[k])) {
        warn_ind <- TRUE
        act <- "Covariance among the repeat visit sites was not included in calculation of \nthe standard error estimate.\n"
        if (stratum_ind) {
          warn <- paste("Category level \"", catvar_levels[k], "\" in stratum \"", stratum_level, "\" \nwas not present among the repeat visit sites in one of the surveys.\n", sep = "")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname),
            subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = I(stratum_level), warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste("Category level \"", catvar_levels[k], "\" was not present among the repeat visit sites \nin one of the surveys.\n", sep = "")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname),
            subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        next
      }

      # For each stage one sampling unit, each category, and each survey,
      # calculate an estimate of the total of the stage two sampling unit
      # residuals; then calculate the variance/covariance matrix of the totals

      total2est <- matrix(0, ncluster, 2)
      var2est <- matrix(0, ncluster, 4)
      for (i in 1:ncluster) {

        # Calculate the weighted indicator matrix

        n <- length(catvar1_lst[[i]])
        z1 <- catvar1_lst[[i]] == catvar_levels[k]
        z2 <- catvar2_lst[[i]] == catvar_levels[k]
        im <- cbind(z1, z2) * matrix(rep(wgt2_lst[[i]], 2), nrow = n)

        # Calculate total estimates for the stage one sampling unit

        total2est[i, ] <- apply(im, 2, sum)

        # Adjust the variance/covariance estimator for small sample size

        SRSind <- FALSE
        if (vartype == "Local" && n < 4) {
          warn_ind <- TRUE
          act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
          if (stratum_ind) {
            warn <- paste0("There are less than four response values for stage one sampling unit \"", cluster_levels[i], "\"\nin stratum \"", stratum_level, "\", the simple random sampling covariance estimator for an \ninfinite population was used to calculate covariance of the total of the residuals.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = stratum_level,
              warning = I(warn), action = I(act)
            ))
          } else {
            warn <- paste0("There are less than four response values for stage one sampling unit \"", cluster_levels[i], "\", \nthe simple random sampling covariance estimator for an infinite population was used \nto calculate covariance of the total of the residuals.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            ))
          }
          vartype <- "SRS"
          SRSind <- TRUE
        }

        # Calculate variance/covariance estimates for the stage one sampling
        # unit

        if (var_ind[i]) {
          if (vartype == "Local") {
            weight_lst <- localmean_weight(
              x2_lst[[i]], y2_lst[[i]],
              1 / wgt2_lst[[i]]
            )
            if (is.null(weight_lst)) {
              warn_ind <- TRUE
              act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
              if (stratum_ind) {
                warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stage one \nsampling unit \"", cluster_levels[i], "\" in stratum \"", stratum_level, "\", the simple random sampling \ncovariance estimator for an infinite population was used to calculate covariance of the \ntotal of the residuals.\n")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                  indicator = warn_vec[3], stratum = stratum_level,
                  warning = I(warn), action = I(act)
                ))
              } else {
                warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stage one \nsampling unit \"", cluster_levels[i], "\", the simple random sampling covariance estimator for an \ninfinite population was used to calculate covariance of the total of the residuals.\n")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                  indicator = warn_vec[3], stratum = NA, warning = I(warn),
                  action = I(act)
                ))
              }
              var2est[i, ] <- as.vector(n * var(im))
            } else {
              temp <- localmean_cov(im, weight_lst)
              var2est[i, ] <- as.vector(temp)
              if (any(diag(temp) < 0)) {
                warn_ind <- TRUE
                act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
                if (stratum_ind) {
                  warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates for \nstage one sampling unit \"", cluster_levels[i], "\" in stratum \"", stratum_level, "\", the simple random \nsampling covariance estimator for an infinite population was used to calculate \ncovariance of the total of the residuals.\n")
                  warn_df <- rbind(warn_df, data.frame(
                    func = I(fname), subpoptype = warn_vec[1],
                    subpop = warn_vec[2], indicator = warn_vec[3],
                    stratum = stratum_level, warning = I(warn), action = I(act)
                  ))
                } else {
                  warn <- paste0("The local mean covariance estimator produced one or more  negative variance estimates \nfor stage one sampling unit \"", cluster_levels[i], "\", the simple random sampling covariance \nestimator for an infinite population was used to calculate covariance of the total of \nthe residuals.\n")
                  warn_df <- rbind(warn_df, data.frame(
                    func = I(fname), subpoptype = warn_vec[1],
                    subpop = warn_vec[2], indicator = warn_vec[3], stratum = NA,
                    warning = I(warn), action = I(act)
                  ))
                }
                var2est[i, ] <- as.vector(n * var(im))
              }
            }
          } else {
            var2est[i, ] <- as.vector(n * var(im))
            if (SRSind) {
              vartype <- "Local"
            }
          }
        }
      }

      # Adjust the variance/covariance estimator for small sample size

      if (vartype == "Local" && ncluster < 4) {
        warn_ind <- TRUE
        act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("There are less than four stage one sampling units in stratum \"", stratum_level, "\", the simple \nrandom sampling covariance estimator for an infinite population was used to calculate \ncovariance of the change estimate for totals.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = I(stratum_level),
            warning = I(warn), action = I(act)
          ))
        } else {
          warn <- paste0("There are less than four stage one sampling units, the simple random sampling covariance\nestimator for an infinite population was used to calculate covariance of the change \nestimate for totals.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        vartype <- "SRS"
      }

      # Calculate the covariance or correlation estimates

      if (vartype == "Local") {
        weight_lst <- localmean_weight(x1_u, y1_u, 1 / wgt1_u)
        if (is.null(weight_lst)) {
          warn_ind <- TRUE
          act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
          if (stratum_ind) {
            warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stratum \n\"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the change estimate for totals.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = stratum_level,
              warning = I(warn), action = I(act)
            ))
          } else {
            warn <- paste0("The local mean covariance estimator cannot calculate valid estimates, the simple random \nsampling covariance estimator for an infinite population was used to calculate \ncovariance of the change estimate for totals.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            ))
          }
          varest <- (ncluster * var(total2est * matrix(rep(wgt1_u, 2),
            nrow = ncluster
          )) + matrix(apply(var2est * matrix(rep(wgt1_u, 4),
            nrow = ncluster
          ), 2, sum), nrow = 2))
        } else {
          varest <- (localmean_cov(total2est * matrix(rep(wgt1_u, 2),
            nrow = ncluster
          ), weight_lst) + matrix(apply(var2est *
            matrix(rep(wgt1_u, 4), nrow = ncluster), 2, sum), nrow = 2))
          temp <- diag(varest)
          if (any(temp < 0)) {
            warn_ind <- TRUE
            act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
            if (stratum_ind) {
              warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates for \nstratum \"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the change estimate for totals.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1],
                subpop = warn_vec[2], indicator = warn_vec[3],
                stratum = stratum_level, warning = I(warn), action = I(act)
              ))
            } else {
              warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates, \nthe simple random sampling covariance estimator for an infinite population was used to \ncalculate covariance of the change estimate for totals.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1],
                subpop = warn_vec[2], indicator = warn_vec[3], stratum = NA,
                warning = I(warn), action = I(act)
              ))
            }
            varest <- (ncluster * var(total2est * matrix(rep(wgt1_u, 2),
              nrow = ncluster
            )) + matrix(apply(var2est * matrix(rep(wgt1_u, 4),
              nrow = ncluster
            ), 2, sum), nrow = 2))
          }
        }
      } else {
        varest <- (ncluster * var(total2est * matrix(rep(wgt1_u, 2),
          nrow = ncluster
        )) + matrix(apply(var2est * matrix(rep(wgt1_u, 4),
          nrow = ncluster
        ), 2, sum), nrow = 2))
      }
      if (revisitwgt) {
        rslt[k] <- varest[1, 2]
      } else {
        rslt[k] <- varest[1, 2] / sqrt(varest[1, 1] * varest[2, 2])
      }

      # End the loop for category levels
    }

    # End of section for a two-stage sample
  } else {

    # Begin the section for a single-stage sample

    # Calculate additional required values

    n <- length(catvar1)
    m <- length(catvar_levels)

    # Loop through each category level

    rslt <- rep(NA, m)
    for (i in 1:m) {

      # Determine whether the categorical level is present in both surveys

      if (is.na(size1[i]) | is.na(size2[i])) {
        warn_ind <- TRUE
        act <- "Covariance among the repeat visit sites was not included in calculation of \nthe standard error estimate.\n"
        if (stratum_ind) {
          warn <- paste("Category level \"", catvar_levels[i], "\" in stratum \"", stratum_level, "\" \nwas not present among the repeat visit sites in one of the surveys.\n", sep = "")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = I(stratum_level),
            warning = I(warn), action = I(act)
          ))
        } else {
          warn <- paste("Category level \"", catvar_levels[i], "\" was not present among the repeat visit sites \nin one of the surveys.\n", sep = "")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        next
      }

      # Calculate the weighted residuals matrix

      z1 <- catvar1 == catvar_levels[i]
      z2 <- catvar2 == catvar_levels[i]
      im <- cbind(z1, z2) * matrix(rep(wgt, 2), nrow = n)

      # Adjust the variance/covariance estimator for small sample size

      if (vartype == "Local" && n < 4) {
        warn_ind <- TRUE
        act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("There are less than four response values in stratum \"", stratum_level, "\", the simple random \nsampling covariance estimator for an infinite population was used to calculate \ncovariance of the change estimate for totals.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname),
            subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = I(stratum_level), warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("\nThere are less than four response values, the simple random sampling covariance \nestimator for an infinite population was used to calculate covariance of the change \nestimate for totals.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname),
            subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        vartype <- "SRS"
      }

      # Calculate covariance or correlation estimates

      if (vartype == "Local") {
        weight_lst <- localmean_weight(x, y, 1 / wgt)
        if (is.null(weight_lst)) {
          warn_ind <- TRUE
          act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
          if (stratum_ind) {
            warn <- paste0("The local mean covariance estimator cannot calculate valid estimates for stratum \n\"", stratum_level, "\", the simple random sampling covariance estimator for an infinite \npopulation was used to calculate covariance of the change estimate for totals.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = stratum_level,
              warning = I(warn), action = I(act)
            ))
          } else {
            warn <- paste0("The local mean covariance estimator cannot calculate valid estimates, the simple random \nsampling covariance estimator for an infinite population was used to calculate \ncovariance of the change estimate for totals.\n")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            ))
          }
          varest <- n * var(im)
        } else {
          varest <- localmean_cov(im, weight_lst)
          temp <- diag(varest)
          if (any(temp < 0)) {
            warn_ind <- TRUE
            act <- "The simple random sampling covariance estimator for an infinite population was used.\n"
            if (stratum_ind) {
              warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates for \nstratum \"", stratum_level, "\", the simple random sampling covariance estimator for an \ninfinite population was used to calculate covariance of the change estimate for totals.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1],
                subpop = warn_vec[2], indicator = warn_vec[3],
                stratum = stratum_level, warning = I(warn), action = I(act)
              ))
            } else {
              warn <- paste0("The local mean covariance estimator produced one or more negative variance estimates, \nthe simple random sampling covariance estimator for an infinite population was used to \ncalculate covariance of the change estimate for totals.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1],
                subpop = warn_vec[2], indicator = warn_vec[3], stratum = NA,
                warning = I(warn), action = I(act)
              ))
            }
            varest <- n * var(im)
          }
        }
      } else {
        varest <- n * var(im)
      }
      if (revisitwgt) {
        rslt[i] <- varest[1, 2]
      } else {
        if (varest[1, 1] == 0 | varest[2, 2] == 0 | any(is.na(varest))) {
          warn_ind <- TRUE
          act <- "Covariance among the repeat visit sites was not included in calculation of \nthe standard error estimate.\n"
          if (stratum_ind) {
            warn <- paste("The variance estimate for category level \"", catvar_levels[i], "\" \nin stratum \"", stratum_level, "\" was equal to zero for at least one of the surveys.\n", sep = "")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname),
              subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = I(stratum_level), warning = I(warn),
              action = I(act)
            ))
          } else {
            warn <- paste("The variance estimate for category level \"", catvar_levels[i], "\" was equal to zero \nfor at least one of the surveys.\n", sep = "")
            warn_df <- rbind(warn_df, data.frame(
              func = I(fname),
              subpoptype = warn_vec[1], subpop = warn_vec[2],
              indicator = warn_vec[3], stratum = NA, warning = I(warn),
              action = I(act)
            ))
          }
          next
        }
        rslt[i] <- varest[1, 2] / sqrt(varest[1, 1] * varest[2, 2])
      }

      # End the loop for category levels
    }

    # End of section for a single-stage sample
  }

  # Return the covariance or correlation estimates, the warning message
  # indicator, and the warn_df data frame

  list(rslt = rslt, warn_ind = warn_ind, warn_df = warn_df)
}

###############################################################################
# Function: revisit_bibd (exported)
# Programmer: Tony Olsen
# Date: March 14, 2019
#'
#' Create a balanced incomplete block panel revisit design
#'
#' Create a revisit design for panels in a survey that specifies the time
#' periods for the units of each panel to be sampled based on searching for a
#' D-optimal block design that is a member of the class of generalized Youden
#' designs.  The resulting design need not be a balanced incomplete block
#' design.  Based on algorithmic idea by Cook and Nachtsheim (1989) and
#' implemented by Robert Wheeler.
#'
#' @param n_period  Number of time periods for the survey design. Typically,
#'   number of periods if sampling occurs once per period or number of months if
#'   sampling occurs once per month. (v, number of varieties/treatments in BIBD
#'   terms)
#'
#' @param n_pnl Number of panels (b, number of blocks in BIBD terms)
#'
#' @param n_visit  Number of time periods to be visited in a panel (k, block
#'   size in BIBD terms)
#'
#' @param nsamp Number of samples in each panel.
#'
#' @param panel_name  Prefix for name of each panel
#'
#' @param begin  Numeric name of first sampling occasion, e.g. a specific
#'   period.
#'
#' @param skip  Number of sampling occasions to skip between planned sampling
#'   periods, e.g., sampling will occur only every 5 periods if \code{skip = 5}.
#'
#' @param iter  Maximum number of iterations in search for D-optimal
#'   Generalized Youden Design.
#'
#' @details  The function uses \code{find.BIB} function from crossdes package to
#' search for a D-optimal block design.  crossdes uses package AlgDesign
#' to search balanced incomplete block designs.
#'
#' @return A two-dimensional array of sample sizes to be sampled for each panel
#'   and each sampling occasion.
#'
#' @references
#'   Cook R. D.  and C. Nachtsheim. (1989). Computer-aided blocking of factorial
#'   and response-surface designs. \emph{Technometrics} \bold{31(3)}, 339-346.
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#' @seealso
#'   \describe{
#'     \item{\code{\link{revisit_dsgn}}}{to create a panel revisit design}
#'     \item{\code{\link{revisit_rand}}}{to create a panel revisit design with random
#'       assignment to panels and time periods}
#'     \item{\code{\link{pd_summary}}}{ to summarize characteristics of a
#'       panel revisit design}
#'   }
#'
#' @keywords survey
#'
#' @examples
#' # Balanced incomplete block design with 20 sample occasions, 20 panels,
#' # 3 visits to each unit, and 20 units in each panel.
#' revisit_bibd(n_period = 20, n_pnl = 20, n_visit = 3, nsamp = 20)
#' @export
###############################################################################

revisit_bibd <- function(n_period, n_pnl, n_visit, nsamp, panel_name = "BIB",
                         begin = 1, skip = 1, iter = 30) {

  # check input parameters for meeting requirements
  if (!(n_period > n_visit & n_pnl >= n_period & n_visit > 1)) {
    stop("\nInput parameters do not satisfy minimal conditions")
  }

  # use crossdes package functions to generate BIB design
  bib <- find.BIB(n_period, n_pnl, n_visit, iter = iter)
  pan_dsgn <- table(c(rep(1:n_pnl, rep(n_visit, n_pnl))), as.vector(t(bib)))
  pan_dsgn[pan_dsgn == 1] <- nsamp

  # assign dimnames
  if (nrow(pan_dsgn) < 10) {
    dimnames(pan_dsgn) <- list(
      c(paste(panel_name, 1:nrow(pan_dsgn), sep = "_")),
      seq(begin, by = skip, length.out = ncol(pan_dsgn))
    )
  } else {
    dimnames(pan_dsgn) <- list(
      c(
        paste(panel_name, 1:9, sep = "_0"),
        paste(panel_name, 10:nrow(pan_dsgn), sep = "_")
      ),
      seq(begin, by = skip, length.out = ncol(pan_dsgn))
    )
  }
  class(pan_dsgn) <- "paneldesign"
  return(pan_dsgn)
}

###############################################################################
# Function: sp_summary functions (exported)
# Programmers: Michael Dumelle
# Date: January 22, 2021
#' Summarize sampling frames, design sites, and analysis data.
#'
#' @description
#' \code{sp_summary()} summarizes sampling frames, design sites, and analysis data. The right-hand of the
#' formula specifies the variables (or factors) to
#' summarize by. If the left-hand side of the formula is empty, the
#' summary will be of the distributions of the right-hand side variables. If the left-hand side
#' of the formula contains a variable, the summary will be of the left-hand size variable
#' for each level of each right-hand side variable. Equivalent to \code{spsurvey::summary()}; both
#' are currently maintained for backwards compatibility.
#'
#' @param object An object to summarize. When summarizing sampling frames,
#' an \code{sf}
#' object. When summarizing design sites, an object created by \code{grts()} or
#' \code{irs()} (which has class \code{sp_design}). When summarizing analysis data,
#' a data frame or an \code{sf} object. 
#'
#' @param formula A formula. One-sided formulas are used to summarize the
#' distribution of numeric or categorical variables. For one-sided formulas,
#' variable names are placed to the right of \code{~} (a right-hand side variable).
#' Two sided formulas are
#' used to summarize the distribution of a left-hand side variable
#' for each level of each right-hand side categorical variable in the formula.
#' Note that only for two-sided formulas are numeric right-hand side variables
#' coerced to a categorical variables. If an intercept
#' is included as a right-hand side variable (whether the formula is one-sided or
#' two-sided), the total will also be summarized. When summarizing sampling frames
#' or analysis data, the default formula is \code{~ 1}. When summarizing design sites,
#' \code{siteuse} should be used in the formula, and the default formula is
#' \code{~ siteuse}.
#'
#' @param onlyshow A string indicating the single level of the single right-hand side
#' variable for which a summary is requested. This argument is only used when
#' a single right-hand side variable is provided.
#'
#' @param siteuse A character vector indicating the design sites
#' for which summaries are requested in \code{object}. Defaults to computing summaries for
#' each non-\code{NULL} \code{sites_*} list in \code{object}.
#'
#' @param ... Additional arguments to pass to \code{sp_summary()}. If the left-hand
#' side of the formula is empty, the appropriate generic arguments are passed
#' to \code{summary.data.frame}. If the left-hand side of the formula is provided,
#' the appropriate generic arguments are passed to \code{summary.default}.
#'
#' @return If the left-hand side of the formula is empty, a named list
#' containing summaries of the count distribution for each right-hand side
#' variable is returned. If the left-hand side of the formula contains a
#' variable, a named list containing five number
#' summaries (numeric left-hand side) or tables (categorical or factor left
#' hand side) is returned for each right-hand side variable.
#'
#' @name sp_summary
#'
#' @author Michael Dumelle \email{Dumelle.Michael@@epa.gov}
#'
#' @export
#'
#' @examples
#' \dontrun{
#' data("NE_Lakes")
#' sp_summary(NE_Lakes, ELEV ~ 1)
#' sp_summary(NE_Lakes, ~ ELEV_CAT * AREA_CAT)
#' sample <- grts(NE_Lakes, 100)
#' sp_summary(sample, ~ ELEV_CAT * AREA_CAT)
#' }
sp_summary <- function(object, ...) {
  UseMethod("sp_summary", object)
}

#' @name sp_summary
#' @method sp_summary default
#' @export
sp_summary.default <- function(object, formula = ~1, onlyshow = NULL, ...) {

  # find system info
  on_solaris <- Sys.info()[["sysname"]] == "SunOS"
  if (on_solaris) {
    stop("sp_summary() is not supported on Solaris.")
  }

  # store as data frame for geometry summaries
  # this removes default sf sticky behavior
  object <- data.frame(object)
  # making formlist (utils.R)
  formlist <- make_formlist(formula, onlyshow, object, remove_geom = FALSE)
  # making varsf (utils.R)
  varsf <- make_varsf(object, formlist)
  # accomodating an intercept
  if (formlist$intercept) {
    # adding "total" to varlabels
    formlist$varlabels <- c("total", formlist$varlabels)
    # making a factor called "total"
    varsf$total <- as.factor("total")
    # adding the factor to varsf
    varsf <- varsf[, c(formlist$response, formlist$varlabels), drop = FALSE]
  }
  # calling the appropriate summary based on formula (right-hand side vs two sided)
  if (is.null(formlist$response)) {
    output <- cat_summary(formlist, varsf, ...)
  } else {
    output <- cont_summary(formlist, varsf, ...)
  }
  new_output <- structure(output, class = c("sp_summary.sp_frame", class(output)))
  new_output
}

#' @name sp_summary
#' @method sp_summary sp_design
#' @export
sp_summary.sp_design <- function(object, formula = ~siteuse, siteuse = NULL, onlyshow = NULL, ...) {
  if ((is.null(siteuse) & (!is.null(object$sites_near))) | "Near" %in% siteuse) {
    object$sites_near$siteuse <- "Near"
  }

  # bind
  object <- sp_rbind(object, siteuse)

  if (is.null(siteuse)) {
    fac_levels <- c("Legacy", "Base", "Over", "Near")
    fac_levels_used <- fac_levels[fac_levels %in% unique(object$siteuse)]
    object$siteuse <- factor(object$siteuse, levels = fac_levels_used)
  } else {
    object$siteuse <- factor(object$siteuse, levels = siteuse)
  }

  output <- sp_summary.default(object, formula, onlyshow, ...)
  new_output <- structure(output, class = c("sp_summary.sp_design", setdiff(class(output), c("sp_summary.sp_frame", "summary.sp_frame"))))
  new_output
}

# Helpers -----------------------------------------------------------------

cat_summary <- function(formlist, varsf, ...) {
  dotlist <- list(...)
  if (!("maxsum" %in% names(dotlist))) {
    dotlist$maxsum <- 10
  }
  if ("sf" %in% class(varsf)) {
    varsf_nogeom <- st_drop_geometry(varsf)
  } else {
    varsf_nogeom <- varsf
  }

  if (!is.null(formlist$onlyshow)) {
    indexcol <- formlist$varlabels[length(formlist$varlabels)]
    varsf_nogeom <- varsf_nogeom[varsf_nogeom[[indexcol]] == formlist$onlyshow, indexcol, drop = FALSE]
    varsf_nogeom <- na.omit(varsf_nogeom)
    varsf_nogeom[[indexcol]] <- factor(varsf_nogeom[[indexcol]], levels = formlist$onlyshow)
  }
  output <- do.call("summary.data.frame", c(list(varsf_nogeom), dotlist))
}

cont_summary <- function(formlist, varsf, ...) {
  dotlist <- list(...)
  if ("sf" %in% class(varsf)) {
    varsf_nogeom <- st_drop_geometry(varsf)
  } else {
    varsf_nogeom <- varsf
  }
  if (!is.null(formlist$onlyshow)) {
    formlist$varlabels <- formlist$varlabels[length(formlist$varlabels)]
    varsf_nogeom <- varsf_nogeom[varsf_nogeom[[formlist$varlabels]] == formlist$onlyshow, c(formlist$response, formlist$varlabels), drop = FALSE]
    varsf_nogeom <- na.omit(varsf_nogeom)
    varsf_nogeom[[formlist$varlabels]] <- factor(varsf_nogeom[[formlist$varlabels]], levels = formlist$onlyshow)
  }
  output <- lapply(formlist$varlabels, function(x) {
    varlevels <- do.call(
      "tapply",
      c(
        list(
          X = varsf_nogeom[[formlist$response]],
          INDEX = varsf_nogeom[[x]],
          FUN = summary.default,
          simplify = FALSE
        ),
        dotlist
      )
    )
    do.call("rbind", varlevels)
  })
  names(output) <- paste(formlist$response, "by", formlist$varlabels, sep = " ")
  output
}

#' New England Lakes data
#'
#' An \code{sf} POINT object of 195 lakes in the Northeastern
#' United States.
#'
#' @format 195 rows and 5 variables:
#' \describe{
#'   \item{\code{AREA}}{ Lake area in hectares.}
#'   \item{\code{AREA_CAT}}{ Lake area categories based on a hectare cutoff.}
#'   \item{\code{ELEV}}{ Elevation in meters.}
#'   \item{\code{ELEV_CAT}}{ Elevation categories based on a meter cutoff.}
#'   \item{\code{geometry}}{ POINT geometry using the NAD83 / Conus Albers
#'     coordinate reference system (EPSG: 5070).}
#' }
"NE_Lakes"

#' New England Lakes legacy data
#'
#' An \code{sf} POINT object of 5 legacy sites for the NE Lakes data
#'
#' @format 5 rows and 5 variables:
#' \describe{
#'   \item{\code{AREA}}{ Lake area in hectares.}
#'   \item{\code{AREA_CAT}}{ Lake area categories based on a hectare cutoff.}
#'   \item{\code{ELEV}}{ Elevation in meters.}
#'   \item{\code{ELEV_CAT}}{ Elevation categories based on a meter cutoff.}
#'   \item{\code{geometry}}{ POINT geometry using the NAD83 / Conus Albers
#'     coordinate reference system (EPSG: 5070).}
#' }
"NE_Lakes_Legacy"

#' New England Lakes data (as a data frame)
#'
#' An data frame of 195 lakes in the Northeastern
#' United States.
#'
#' @format 195 rows and 6 variables:
#' \describe{
#'   \item{\code{AREA}}{ Lake area in hectares.}
#'   \item{\code{AREA_CAT}}{ Lake area categories based on a hectare cutoff.}
#'   \item{\code{ELEV}}{ Elevation in meters.}
#'   \item{\code{ELEV_CAT}}{ Elevation categories based on a meter cutoff.}
#'   \item{\code{XCOORD}}{ x-coordinate using the WGS 84 coordinate reference system
#'     (EPSG: 4326)}
#'   \item{\code{YCOORD}}{ y-coordinate using WGS 84 coordinate reference system
#'     (EPSG: 4326)}
#' }
"NE_Lakes_df"

#' Illinois River data
#'
#' An (\code{sf}) MULTILINESTRING object of 244 segments of the
#' Illinois River in Arkansas and Oklahoma.
#'
#' @format 244 rows and 2 variables:
#' \describe{
#'   \item{\code{STATE_NAME}}{ State name.}
#'   \item{\code{geometry}}{ MULTILINESTRING geometry using the NAD83 / Conus Albers
#'     coordinate reference system (EPSG: 5070).}
#' }
"Illinois_River"

#' Illinois River legacy data
#'
#' An (\code{sf}) POINT object of legacy sites for the Illinois
#' River data.
#'
#' @format 5 rows and 2 variables:
#' \describe{
#'   \item{\code{STATE_NAME}}{ State name.}
#'   \item{\code{geometry}}{ POINT geometry using the NAD83 / Conus Albers
#'     coordinate reference system (EPSG: 5070).}
#' }
"Illinois_River_Legacy"

#' Lake Ontario data
#'
#' An \code{sf} MULTIPOLYGON object of 187 polygons consisting
#' of shore segments in Lake Ontario.
#'
#' @format 187 rows and 5 variables:
#' \describe{
#'   \item{\code{COUNTRY}}{ Country.}
#'   \item{\code{RSRC_CLASS}}{ Bay class.}
#'   \item{\code{PSTL_CODE}}{ Postal code.}
#'   \item{\code{AREA_SQKM}}{ Area in square kilometers}
#'   \item{\code{geometry}}{ MULTIPOLYGON geometry using the NAD83 / Conus Albers
#'     coordinate reference system (EPSG: 5070).}
#' }
"Lake_Ontario"

#' NLA PNW data
#'
#' An \code{sf} POINT object of 96 lakes in the Pacific Northwest Region of the United
#' States during the year 2017, from a subset of the Environmental
#' Protection Agency's "National Lakes Assessment."
#'
#' @format 96 rows and 9 variables:
#' \describe{
#'   \item{\code{SITE_ID}}{ A unique lake identifier.}
#'   \item{\code{WEIGHT}}{ The sampling design weight.}
#'   \item{\code{URBAN}}{ Urban category.}
#'   \item{\code{STATE}}{ State name.}
#'   \item{\code{BMMI}}{ Benthic MMI value.}
#'   \item{\code{BMMI_COND}}{ Benthic MMI condition categories.}
#'   \item{\code{PHOS_COND}}{ Phosphorus condition categories.}
#'   \item{\code{NITR_COND}}{ Nitrogen condition categories.}
#'   \item{\code{geometry}}{ POINT geometry using the NAD83 / Conus Albers
#'     coordinate reference system (EPSG: 5070).}
#' }
"NLA_PNW"

#' NRSA EPA7 data
#'
#' An \code{sf} POINT object of 353 stream segments in the Central
#' United States during the years 2008 and 2013, from a subset of the Environmental
#' Protection Agency's "National Rivers and Streams Assessment."
#'
#' @format 353 rows and 10 variables:
#' \describe{
#'   \item{\code{SITE_ID}}{ A unique site identifier.}
#'   \item{\code{YEAR}}{ Year of design cycle.}
#'   \item{\code{WEIGHT}}{ Sampling design weights.}
#'   \item{\code{ECOREGION}}{ Ecoregion.}
#'   \item{\code{STATE}}{ State name.}
#'   \item{\code{BMMI}}{ Benthic MMI value.}
#'   \item{\code{BMMI_COND}}{ Benthic MMI categories.}
#'   \item{\code{PHOS_COND}}{ Phosphorus condition categories.}
#'   \item{\code{NITR_COND}}{ Nitrogen condition categories.}
#'   \item{\code{geometry}}{ POINT geometry using the NAD83 / Conus Albers
#'     coordinate reference system (EPSG: 5070).}
#' }
"NRSA_EPA7"

###############################################################################
# Function: grts_stratum (not exported)
# Programmers: Tony Olsen
# Date: January 22, 2021
#'
#' For a single stratum, select a spatially balanced sample using generalized random
#' tessellation stratified algorithm. For a point sample frame, the selection is a one-step
#' process. For linear and area sample frames, a two-step process is used. First a systematic
#' point sample is selected from either linear or area sample frame to create a dense point
#' sample. Then the grts algorithm is applied to the systematic point sample. The selection
#' in all cases is based on the survey design specification for the stratum.
#'
#' @param stratum Character value for the stratum name.
#'
#' @param dsgn List of components that specify the survey design. Includes all strata.
#'   See grts for contents of dsgn.
#'
#' @param sframe Sample frame as an sf object. If the design is stratified,
#'   unequal probability, proportional probability or has legacy sites, then sample frame
#'   must include variables that identify the stratum; category, auxiliary and legacy variables
#'   for unequal selection; or that identify elements that are legacy sites.
#'   The coordinate system for sframe must be one where distance for coordinates is meaningful.
#'
#' @param sf_type The sample frame geometry type: point, linear or area
#'
#' @param wgt_units The units for weights if different from units in the \code{sf} object. Default
#'   is \code{NULL}. Unit conversion uses \code{set_unit}s in \code{sf} package.
#'
#' @param pt_density For linear and area sample frame, the point density for the systematic
#'   sample. Must be in units of the sframe sf.object. Default is NULL.
#'
#' @param legacy_option Logical variable that when TRUE legacy sites are to be included
#'   in the survey design. Default is FALSE
#'
#' @param legacy_sites An sf object of legacy sites to be included in the survey design.
#'
#' @param maxtry Number of maximum attempts to ensure minimum distance between sites.
#'   Default is 10.
#'
#' @param warn_ind  A logical value where TRUE indicates a warning message.
#'   Used for internal collection of messages only.
#'
#' @param warn_df A data frame containing messages warning of potential issues.
#'   Used for internal collection of messages only.
#'
#' @return rslts A list consisting of an \code{sf} object for base sites, an \code{sf} object of
#'   \code{n_over} sites (\code{NULL} if none) an \code{sf} object of \code{n_near} sites (\code{NULL} if none) where
#'   the \code{sf} objects containing the sites selected that meet the survey design requirements
#'   and \code{warn_ind} - logical value for warning indicator and \code{warn_df} - a data frame
#'   for warning messages.
#'
#' @author Tony Olsen \email{olsen.tony@@epa.gov}
#'
#' @keywords survey
#'
#' @examples
#' \dontrun{
#' test_sample <- grts(stratum, dsgn = test_design, sframe = "test_sf", sf_type = "point")
#' }
#'
#' @noRd
###############################################################################

grts_stratum <- function(stratum, dsgn, sframe, sf_type, wgt_units = NULL, pt_density = NULL,
                         legacy_option = FALSE, legacy_sites = NULL, maxtry = 10,
                         warn_ind = FALSE, warn_df = NULL) {

  # Sample sizes required
  n_base <- dsgn[["n_base"]][[stratum]]
  n_over <- sum(dsgn[["n_over"]][[stratum]], na.rm = TRUE)
  if (is.null(n_over)) {
    n_over <- 0
  }
  n_near <- dsgn[["n_near"]][[stratum]]
  if (is.null(n_near)) {
    n_near <- 0
  }
  n_total <- n_base + n_over

  # set number of legacy sites to 0
  n_legacy <- 0

  # subset sframe to stratum
  sftmp <- sframe[sframe$stratum == stratum, , drop = FALSE]

  # find legacy site number for points if legacy_var provided
  if (legacy_option == TRUE & is.null(legacy_sites)) {
    n_legacy <- sum(!is.na(sftmp$legacy))
  }

  # subset legacy_sites to stratum if present for linear and area option
  if (legacy_option == TRUE & (sf_type != "sf_point" | ((sf_type == "sf_point") & !is.null(legacy_sites)))) {
    legtmp <- legacy_sites[legacy_sites$stratum == stratum, , drop = FALSE]
    n_legacy <- nrow(legtmp)
  }

  # sf_type equals point
  if (sf_type == "sf_point") {
    ip_step1 <- 1
    sftmp$xcoord <- st_coordinates(sftmp)[, "X"]
    sftmp$ycoord <- st_coordinates(sftmp)[, "Y"]
    sftmp$idpts <- 10000 + 1:nrow(sftmp)
  }

  # sf_type equals linear
  if (sf_type == "sf_linear") {
    # determine sample size from pt_density and total length of sample frame in stratum
    stratum_len <- sum(st_length(sftmp))
    if (!is.null(wgt_units)) {
      stratum_len <- set_units(stratum_len, wgt_units, mode = "standard")
    }
    # set default equal to 10 population sites per requested sample site
    if (is.null(pt_density)) {
      pt_density <- 10
    }
    n_size <- as.integer(ceiling(pmin(1e9, pt_density * (n_base + n_over))))
    sfpts <- st_sample(sftmp, size = n_size, type = "regular", exact = TRUE)
    sfpts <- st_as_sf(as.data.frame(sfpts), crs = st_crs(sftmp))
    # sfpts <- st_cast(sfpts, to = "POINT")
    # drop features with no points
    sfpts <- sfpts[!st_is_empty(sfpts), ]
    # join sites with linear features
    sfpts <- st_cast(sfpts, to = "POINT")
    sftmp <- st_join(sfpts, sftmp, join = st_nearest_feature)
    names(sftmp)[names(sftmp) == "sfpts"] <- "geometry"
    sftmp$xcoord <- st_coordinates(sftmp)[, "X"]
    sftmp$ycoord <- st_coordinates(sftmp)[, "Y"]
    sftmp$idpts <- 10000 + 1:nrow(sftmp)
    # calculate step 1 inclusion probability based on realized sample size
    ip_step1 <- nrow(sftmp) / stratum_len
  }


  # sf_type equals area
  if (sf_type == "sf_area") {
    # determine sample size from pt_density and total area of sample frame in stratum
    stratum_area <- sum(st_area(sftmp))
    if (!is.null(wgt_units)) {
      stratum_area <- set_units(stratum_area, wgt_units, mode = "standard")
    }
    # set default equal to 10 population sites per requested sample site
    if (is.null(pt_density)) {
      pt_density <- 10
    }
    n_size <- as.integer(ceiling(pmin(1e9, pt_density * (n_base + n_over))))
    sfpts <- st_sample(sftmp, size = n_size, type = "hexagonal", exact = TRUE)
    sfpts <- st_as_sf(as.data.frame(sfpts), crs = st_crs(sftmp))
    # sfpts <- st_cast(sfpts, to = "POINT")
    # drop features with no points
    sfpts <- sfpts[!st_is_empty(sfpts), ]
    sfpts <- st_cast(sfpts, to = "POINT")
    sftmp <- st_join(sfpts, sftmp)
    sftmp$xcoord <- st_coordinates(sftmp)[, "X"]
    sftmp$ycoord <- st_coordinates(sftmp)[, "Y"]
    sftmp$idpts <- 10000 + 1:nrow(sftmp)
    # calculate step 1 inclusion probability based on realized sample size
    ip_step1 <- nrow(sftmp) / stratum_area
  }

  # Determine number of elements in stratum
  Nstratum <- nrow(sftmp)

  # Determine if legacy sites are to be included for stratum design
  if (legacy_option == TRUE & n_legacy > 0 & (sf_type != "sf_point" | ((sf_type == "sf_point") & !is.null(legacy_sites)))) {
    tmp <- legtmp
    addtmp <- setdiff(names(tmp), names(sftmp))
    addleg <- setdiff(names(sftmp), names(tmp))
    sftmp[, addtmp] <- NA
    tmp[, addleg] <- NA
    sftmp <- rbind(tmp, sftmp)
    # Determine number of elements in stratum
    Nstratum <- nrow(sftmp)
  }

  # set legacy that is NA to FALSE
  if (legacy_option == TRUE & n_legacy > 0) {
    sftmp$legacy <- ifelse(is.na(sftmp$legacy), FALSE, TRUE)
    tmp <- sftmp[sftmp$legacy == TRUE, , drop = FALSE]
    n_legacy <- nrow(tmp)
  }

  if (legacy_option == TRUE & n_legacy == 0) {
    sftmp$legacy <- FALSE
  }

  # check that number of legacy sites is less than or equal number of base sites
  # stop if not
  if (n_legacy > n_base) {
    message("Number of legacy sites is greater than number of base sites in at least one\n")
    message("stratum. Please check that all strata have fewer legacy sites than base sites.\n")
    opt <- options(show.error.messages = FALSE)
    on.exit(options(opt))
    stop()
  }

  # Step 2 site selection if linear or area; otherwise Step 1 for points.
  # determine overall sample size required from dsgn for stratum
  # account for n_over sample option if present
  if (dsgn[["seltype"]][[stratum]] == "equal" | dsgn[["seltype"]][[stratum]] == "proportional") {
    n_caty <- n_total
  } else {
    if (n_over == 0) {
      n_caty <- dsgn[["caty_n"]][[stratum]]
    } else {
      base_prop <- dsgn[["caty_n"]][[stratum]] / sum(dsgn[["caty_n"]][[stratum]])
      n_caty <- dsgn[["caty_n"]][[stratum]] + dsgn[["n_over"]][[stratum]] * base_prop
    }
  }

  # If seltype is "equal" or "proportional", set caty to same as stratum
  if (dsgn[["seltype"]][[stratum]] == "equal" | dsgn[["seltype"]][[stratum]] == "proportional") {
    sftmp$caty <- "None"
  }

  # compute inclusion probabilities
  ip <- grtspts_ip(
    type = dsgn[["seltype"]][stratum], n_base = n_caty,
    Nstratum = Nstratum, caty = sftmp$caty, aux = sftmp$aux,
    warn_ind = warn_ind, warn_df = warn_df
  )

  # save initial inclusion probabilities
  sftmp$ip_init <- ip$ip
  sftmp$ip <- ip$ip
  # accumulate warning messages if any
  if (ip$warn_ind) {
    warn_ind <- ip$warn_ind
    warn_df <- ip$warn_df
    warn_df$stratum <- ifelse(is.na(warn_df$stratum), stratum, warn_df$stratum)
  }

  # If legacy sites, adjust inclusion probabilities to use
  # legacy inclusion probabilities
  if (legacy_option == TRUE & n_legacy > 0) {
    sftmp$ip <- grtspts_ipleg(sftmp$ip_init, sftmp$legacy == TRUE)
    # accumulate warning messages if any
    if (ip$warn_ind) {
      warn_ind <- ip$warn_ind
      warn_df <- ip$warn_df
      warn_df$stratum <- ifelse(is.na(warn_df$stratum), stratum, warn_df$stratum)
    }
  }

  # select sites if no minimum distance between sites
  if (is.null(dsgn[["mindis"]][[stratum]])) {
    sites <- sftmp[get_address(sftmp$xcoord, sftmp$ycoord, rand = TRUE), ]
    s <- UPpivotal(sites$ip)
    sites <- sites[round(s) == 1, ]
    sites <- rho(sites)
    sites$siteuse <- NA
    sites$replsite <- NA
    sites <- list(sites = sites, warn_ind = warn_ind, warn_df = warn_df)
  }
  # If minimum distance between sites, select sites
  if (!is.null(dsgn[["mindis"]][[stratum]])) {
    sites <- grtspts_mindis(dsgn[["mindis"]][[stratum]], sftmp,
      samplesize = n_total,
      stratum = stratum, maxtry = maxtry, legacy_option = legacy_option,
      legacy_var = dsgn[["legacy_var"]],
      warn_ind = warn_ind, warn_df = warn_df
    )
  }
  # check for warning messages
  warn_ind <- sites$warn_ind
  warn_df <- sites$warn_df
  if (warn_ind) {
    warn_df$stratum <- ifelse(is.na(warn_df$stratum), stratum, warn_df$stratum)
  }

  # adjust inclusion probabilities when over sample sites present
  sites[["sites"]]$ip_init <- sites[["sites"]]$ip_init * n_base / n_total

  # Select replacement sites if n_near not NULL when do not have legacy sites
  if (legacy_option == FALSE | n_legacy == 0) {
    if (!is.null(dsgn[["n_near"]][[stratum]])) {
      sites_near <- replace_near(dsgn[["n_near"]][[stratum]],
        sites = sites[["sites"]],
        sframe = sftmp
      )

      # Adjust inclusion probabilities for replacement sites if over sample sites present
      if (n_over != 0) {
        sites_near$ip_init <- sites_near$ip_init * n_base / n_total
      }
    }
  }

  # Select replacement sites if n_near not NULL when have legacy sites
  if (legacy_option == TRUE & n_legacy > 0) {
    if (!is.null(dsgn[["n_near"]][[stratum]])) {
      keep <- sites[["sites"]][sites[["sites"]]$legacy %in% c(TRUE, FALSE), "idpts", drop = TRUE]
      sites_near <- replace_near(dsgn[["n_near"]][[stratum]],
        sites = sites[["sites"]][sites[["sites"]]$legacy %in% c(TRUE, FALSE), ],
        sframe = subset(sftmp, !(sftmp$idpts %in% keep))
      )

      # Adjust inclusion probabilities for replacement sites if over sample sites present
      if (n_over != 0) {
        sites_near$ip_init <- sites_near$ip_init * n_base / n_total
      }
    }
  }

  # Assign original inclusion probabilities to sites, create weights and drop legacy ip variable
  sites[["sites"]]$ip <- sites[["sites"]]$ip_init * ip_step1
  sites[["sites"]]$wgt <- 1 / sites[["sites"]]$ip
  tmp <- names(sites[["sites"]])
  sites[["sites"]] <- subset(sites[["sites"]],
    select = tmp[!(tmp %in% c("ip_init", "geometry"))]
  )

  # Do same for sites_near if any
  if (is.null(dsgn[["n_near"]][[stratum]])) {
    sites_near <- NULL
  }
  if (!is.null(dsgn[["n_near"]][[stratum]])) {
    sites_near$ip <- sites_near$ip_init * ip_step1
    sites_near$wgt <- 1 / sites_near$ip
    tmp <- names(sites_near)
    sites_near <- subset(sites_near, select = tmp[!(tmp %in% c("ip_init", "geometry"))])
    sites_near[, setdiff(names(legacy_sites), names(sites_near))] <- NA
  }

  # Split sites to have separate sites_base, sites_legacy and sites_over
  # save legacy sites if any and reduce sites_base to non legacy sites
  sites_legacy <- NULL
  if (legacy_option == TRUE & n_legacy > 0) {
    sites_legacy <- sites[["sites"]][sites[["sites"]]$legacy == TRUE, ]
    sites[["sites"]] <- sites[["sites"]][sites[["sites"]]$legacy == FALSE, ]
    n_legacy <- nrow(sites_legacy)
  }



  # save base sites
  sites_base <- NULL
  if (n_base > n_legacy) {
    sites_base <- sites[["sites"]][1:(n_base - n_legacy), ]
    sites_base[, setdiff(names(legacy_sites), names(sites_base))] <- NA
  }

  # save n_over sample sites if any
  sites_over <- NULL
  if (n_over != 0) {
    sites_over <- sites[["sites"]][(n_base - n_legacy + 1):(n_total - n_legacy), ]
    sites_over$siteuse <- "Over"
    sites_over[, setdiff(names(legacy_sites), names(sites_over))] <- NA
  }

  # if no legacy sites match in strata then put in appropriate column
  if (legacy_option == TRUE & n_legacy == 0) {
    sites_base$legacy <- FALSE
  }


  # create list for output and return result
  rslts <- list(
    sites_base = sites_base, sites_legacy = sites_legacy,
    sites_over = sites_over, sites_near = sites_near,
    warn_ind = warn_ind, warn_df = warn_df
  )

  invisible(rslts)
}

################################################################################
# Function: cdf_est (not exported)
# Programmer: Tom Kincaid
# Date: July 23, 2020
# Revised: May 25 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
# Revised: September 15 2021 to correct errors that occur when using the
#          svymean and svytotal functions to estimate CDFs
#
#' Cumulative Distribution Function Estimate for Probability Survey Data
#'
#' This function calculates an estimate of the cumulative distribution function
#' (CDF) for the proportion (expressed as percent) and the total of a
#' quantitative response variable, where the response variable may be defined
#' for either a finite or an extensive resource.  In addition, the standard
#' error of the estimated CDF and confidence bounds are calculated.  For the
#' CDF of a proportion, the Horvitz-Thompson ratio estimator, i.e., the ratio
#' of two Horvitz-Thompson estimators, is used to calculate the CDF estimate.
#' For the CDF of a total, the classic ratio estimator is used to calculate the
#' CDF estimate, where that estimator is the product of the known value and the
#' Horvitz-Thompson ratio estimator.  Variance estimates for the estimated CDF
#' are calculated using either the local mean variance estimator or the simple
#' random sampling (SRS) variance estimator.  The choice of variance estimator
#' is subject to user control. The local mean variance estimator requires the
#' x-coordinate and the y-coordinate of each site.  The SRS variance estimator
#' uses the independent random sampling approximation to calculate joint
#' inclusion probabilities.  Confidence bounds are calculated using a Normal
#' distribution multiplier.  The function can accommodate a stratified sample.
#' For a stratified sample, separate estimates and standard errors are
#' calculated for each stratum, which are used to produce estimates and standard
#' errors for all strata combined. The function can accommodate single-stage and
#' two-stage samples for both stratified and unstratified sampling designs.  The
#' finite population correction factor can be utilized in variance estimation.
#'
#' @param cdfsum Data frame containing estimates.
#'
#' @param dframe Data frame containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param itype Character value that identifies a factor variable in the design
#'   argument containing subpopulation (domain) values.
#'
#' @param lev_itype Character vector that provides levels of the subpopulation
#'   variable.
#'
#' @param nlev_itype Numeric value that provides the number of levels of the
#'   subpopulation variable.
#'
#' @param ivar Character value that identifies a factor variable in the design
#'   argument containing categorical response values.
#'
#' @param design Object of class \code{survey.design} that specifies a complex
#'   survey design.
#'
#' @param design_names Character vector that provides names of survey design
#'   variables in the design argument.
#'
#' @param var_nondetect Character value that identifies the name of a logical
#'   variable in the \code{dframe} data frame specifying the presence of not
#'   detected (nondetect) values for the response variable.
#'
#' @param vartype Character value providing the choice of the variance
#'   estimator, where "Local" = the local mean estimator, \code{"SRS"} = the
#'   simple random sampling estimator, \code{"HT"} = the Horvitz-Thompson
#'   estimator, and \code{"YG"} = the Yates-Grundy estimator.  The default value
#'   is \code{"Local"}.
#'
#' @param conf Numeric value for the confidence level.
#'
#' @param mult Numeric value that provides the Normal distribution confidence
#'   bound multiplier.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated.
#'
#' @param warn_df Data frame for storing warning messages.
#'
#' @return A list composed of the following objects:
#'   \itemize{
#'     \item{\code{cdfsum}}{data frame containing the CDF estimate}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey univar
#'
#' @noRd
################################################################################

cdf_est <- function(cdfsum, dframe, itype, lev_itype, nlev_itype, ivar, design,
                    design_names, var_nondetect, vartype, conf, mult, warn_ind,
                    warn_df) {

  # Assign a value to the function name variable

  fname <- "cdf_est"

  #
  # Calculate CDF estimates
  #

  if (is.null(var_nondetect)) {

    # Using the proportion scale, calculate CDF estimates, standard error
    # estimates, and confidence bound estimates for each combination of
    # subpopulation and response variable for the case where nondetects are not
    # present
    # Note: A small value (1e-10) is added to each element in the cdfval object
    #       in order to force the svymean function to produce estimates that
    #       match the estimates produced by the svycdf function

    tst <- !is.na(dframe[, itype])
    cdfval <- sort(unique(dframe[!is.na(dframe[, ivar]), ivar])) + 1e-10
    ncdfval <- length(cdfval)
    if (nlev_itype == 1) {
      rslt_svy <- lapply(cdfval, function(x) {
        svymean(make.formula(paste0("I(", ivar, " <= ", x, ")")),
          design = subset(design, tst), na.rm = TRUE
        )
      })
      cdfest_P <- sapply(rslt_svy, function(x) x[2])
      cdfest_P <- as.data.frame(t(cdfest_P))
      nresp <- list(cdf_nresp(dframe[, ivar], cdfval))
      if (vartype == "Local") {
        temp <- cdf_localmean_prop(
          itype, lev_itype, nlev_itype, ivar, design, design_names, cdfval,
          ncdfval, cdfest_P, mult, warn_ind, warn_df
        )
        stderr_P <- temp$stderr_P
        confval_P <- temp$confval_P
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        stderr_P <- sapply(rslt_svy, function(x) SE(x)[2])
        stderr_P <- as.data.frame(t(stderr_P))
        temp <- sapply(rslt_svy, function(x) confint(x, level = conf / 100))
        confval_P <- t(temp[c(2, 4), ])
      }
    } else {
      rslt_svy <- lapply(cdfval, function(x) {
        svyby(make.formula(paste0("I(", ivar, " <= ", x, ")")),
          make.formula(itype),
          design = subset(design, tst), svymean,
          na.rm = TRUE
        )
      })
      cdfest_P <- sapply(rslt_svy, function(x) x[, 3])
      rownames(cdfest_P) <- lev_itype
      nresp <- tapply(dframe[, ivar], dframe[, itype], cdf_nresp, cdfval)
      if (vartype == "Local") {
        temp <- cdf_localmean_prop(
          itype, lev_itype, nlev_itype, ivar, design, design_names, cdfval,
          ncdfval, cdfest_P, mult, warn_ind, warn_df
        )
        stderr_P <- temp$stderr_P
        confval_P <- temp$confval_P
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        stderr_P <- sapply(rslt_svy, function(x) SE(x)[, 2])
        rownames(stderr_P) <- lev_itype
        temp <- lapply(rslt_svy, function(x) {
          confint(x, level = conf / 100)[
            (nlev_itype + 1):(2 * nlev_itype),
          ]
        })
        confval_P <- NULL
        for (i in 1:length(temp)) {
          confval_P <- rbind(confval_P, temp[[i]])
        }
      }
    }

    # Using the total scale, calculate CDF estimates, standard error estimates,
    # and confidence bound estimates for each combination of subpopulation and
    # response variable for the case where nondetects are not present

    if (nlev_itype == 1) {
      rslt_svy <- lapply(cdfval, function(x) {
        svytotal(make.formula(paste0("I(", ivar, " <= ", x, ")")),
          design = subset(design, tst), na.rm = TRUE
        )
      })
      confest_U <- sapply(rslt_svy, function(x) x[2])
      confest_U <- as.data.frame(t(confest_U))
      if (vartype == "Local") {
        temp <- cdf_localmean_total(
          itype, lev_itype, nlev_itype, ivar, design, design_names, cdfval,
          ncdfval, confest_U, mult, warn_ind, warn_df
        )
        stderr_U <- temp$stderr_U
        confval_U <- temp$confval_U
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        stderr_U <- sapply(rslt_svy, function(x) SE(x)[2])
        stderr_U <- as.data.frame(t(stderr_U))
        temp <- sapply(rslt_svy, function(x) confint(x, level = conf / 100))
        confval_U <- t(temp[c(2, 4), ])
      }
    } else {
      rslt_svy <- lapply(cdfval, function(x) {
        svyby(make.formula(paste0("I(", ivar, " <= ", x, ")")),
          make.formula(itype),
          design = subset(design, tst), svytotal,
          na.rm = TRUE
        )
      })
      confest_U <- sapply(rslt_svy, function(x) x[, 3])
      rownames(confest_U) <- lev_itype
      if (vartype == "Local") {
        temp <- cdf_localmean_total(
          itype, lev_itype, nlev_itype, ivar, design, design_names, cdfval,
          ncdfval, confest_U, mult, warn_ind, warn_df
        )
        stderr_U <- temp$stderr_U
        confval_U <- temp$confval_U
        warn_ind <- temp$warn_ind
        warn_df <- temp$warn_df
      } else {
        stderr_U <- sapply(rslt_svy, function(x) SE(x)[, 2])
        rownames(stderr_U) <- lev_itype
        temp <- lapply(rslt_svy, function(x) {
          confint(x, level = conf / 100)[
            (nlev_itype + 1):(2 * nlev_itype),
          ]
        })
        confval_U <- NULL
        for (i in 1:length(temp)) {
          confval_U <- rbind(confval_U, temp[[i]])
        }
      }
    }
  } else {

    # Calculate CDF estimates, standard error estimates, and confidence bound
    # estimates for each combination of subpopulation and response variable for
    # the case where nondetects are present

    maxval <- max(dframe[, ivar], na.rm = TRUE)
    flip <- maxval - dframe[, ivar]
    event <- !dframe[, var_nondetect]
    if (nlev_itype == 1) {
      rslt_svy <- svykm(Surv(flip, event) ~ 1, design = subset(design, tst))
    }
  }

  # Assign identifiers and estimates to the cdfsum data frame

  if (is.null(var_nondetect)) {
    for (i in 1:nlev_itype) {
      indx <- seq(i, by = nlev_itype, length = ncdfval)
      cdfsum <- rbind(cdfsum, data.frame(
        Type = itype,
        Subpopulation = lev_itype[i],
        Indicator = ivar,
        Value = cdfval,
        nResp = nresp[[i]],
        Estimate.P = 100 * unlist(cdfest_P[i, ]),
        StdError.P = 100 * unlist(stderr_P[i, ]),
        MarginofError.P = 100 * (mult * unlist(stderr_P[i, ])),
        LCB.P = 100 * pmax(confval_P[indx, 1], 0),
        UCB.P = 100 * pmin(confval_P[indx, 2], 1),
        Estimate.U = unlist(confest_U[i, ]),
        StdError.U = unlist(stderr_U[i, ]),
        MarginofError.U = mult * unlist(stderr_U[i, ]),
        LCB.U = pmax(confval_U[indx, 1], 0),
        UCB.U = confval_U[indx, 2]
      ))
    }
  } else {
    # To be implemented
  }


  # Return the cdfsum data frame, the warn_ind logical value, and the warn_df
  # data frame

  list(cdfsum = cdfsum, warn_ind = warn_ind, warn_df = warn_df)
}

###############################################################################
# Function: revisit_dsgn (exported)
# Programmer: Tony Olsen
# Date: March 14, 2019
#'
#' Create a panel revisit design
#'
#' Create a revisit design for panels in a survey that specifies the time
#' periods that members of each panel will be sampled. Three basic panel design
#' structures may be created: always revisit panel, serially alternating panels,
#' or rotating panels.
#'
#' @param n_period  Number of time periods for the panel design. For example,
#'   number of periods if sampling occurs once per period or number of months if
#'   sampling occurs once per month.
#'
#' @param panels List of lists where each list specifies a revisit panel
#'   structure. Each sublist consists of four components: \code{n} - sample size for
#'   each panel in the sublist, \code{pnl_dsgn} - a vector with an even number of
#'   elements specifying the panel revisit schedule in terms of the number of
#'   consecutive time periods sample units will be sampled, followed by number
#'   of consecutive time periods skipped, and then repeated as necessary. \code{pnl_n}
#'   - number of panels in the sublist, and \code{start_option} - option for starting
#'   the \code{revisit_dsgn} (\code{None}, \code{Partial_Begin}, or \code{Partial_End})
#'   which must be the same a \code{pnl_dsgn}. Three basic panel
#'   structures are possible: a) if \code{pnl_dsgn} ends in \code{0}, then the sample units
#'   are visited on all subsequent time periods, b) if \code{pnl_dsgn} ends in \code{NA}, then
#'   panel follows a rotating panel structure, and c) if \code{pnl_dsgn} ends in any
#'   number > \code{0}, then panel follows a serially alternating panel structure. See
#'   details for further information.
#'
#' @param begin  Numeric name of first sampling occasion, e.g. a specific
#'   period.
#'
#' @param skip  Number of time periods to skip between planned sampling
#'   periods, e.g., sampling will occur only every 5 periods if \code{skip = 5}.
#'
#' @details The function creates revisit designs using the concepts in McDonald
#'   (2003) to specify the revisit pattern across time periods for each panel.
#'   The panel revisit schedule is specified by a vector. Odd positions in
#'   vector specify the number of consecutive time periods when panel units are
#'   sampled. Even positions in vector specify the number of consecutive time
#'   periods when panel units are not sampled.
#'
#'   If last even position is a \code{"0"}, then a single panel follows an always
#'   revisit panel structure.  After satisfying the initial revisit schedule
#'   specified prior to the \code{"0"}, units in a panel are always visited for rest of
#'   the time periods. The simplest always revisit panel design is to revisit
#'   every sample unit on every time period, specified as \code{pnl_dsgn = c(1,0)} or
#'   using McDonald's notation [1-0].
#'
#'   If the last even position is \code{NA}, the panels follow a rotating panel
#'   structure. For example, \code{pnl_dsgn = c(1, NA)} designates that sample units in
#'   a panel will be visited once and then never again, [1-n] in McDonald's
#'   notation. \code{pnl_dsgn =c(1, 4, 1, NA)} designates that sample units in a panel
#'   will be visited once, then not sampled on next four time periods, then
#'   sampled again once at the next time period and then never sampled again,
#'   [1-4-1-n] in McDonald/s notation.
#'
#'   If the last even position is > \code{0}, the panels follow a serially alternating
#'   panel structure. For example, \code{pnl_dsgn = c(1, 4)} designates that sample
#'   units in a panel will be visited once, then not sampled during the next
#'   four time periods, then sampled once and not sampled for next four time
#'   periods, and that cycle repeated until end of the number of time periods,
#'   [1-4] in McDonald's notation. \code{pnl_dsgn = c(2, 3, 1, 4)} designates that the
#'   cycle has sample units in a panel being visited during two consecutive time
#'   periods, not sampled for three consecutive time periods, sampled for one time
#'   period and then not sampled on next four time periods, and the cycle is
#'   repeated until end of the number of time periods, [2-3-1-4] in McDonald's
#'   notation.
#'
#'   The number of panels in a single panel design is specified by \code{pnl_n}.  For
#'   an always revisit panel structure, a single panel is created and \code{pnl_n} is
#'   ignored. For a rotating panel structure, when \code{pnl_n = NA}, the number of
#'   panels is equal to n_period. Note that this should only be used when the
#'   rotating panel structure is the only panel design, i.e., no split panel
#'   design (see below for split panel details). If \code{pnl_n = m} is specified for a
#'   rotating panel design, then then number of panels will be \code{m}.  For example,
#'   \code{pnl_dsgn = c( 1, 4, 1, NA)} and and \code{pnl_n = 5} means that only 5 panels will
#'   be constructed and the last time period to be sampled will be time period
#'   10. In McDonald's notation the panel design structure is [(1-4-1-n)^5].  If
#'   the number of time periods, \code{n_period}, is 20 and no other panel design
#'   structure is specified, then the last 10 time periods will not be sampled.
#'   For serially alternating panels, when \code{pnl_n = NA}, the number of panels will
#'   be the sum of the elements in pan_dsgn (ignoring \code{NA}). If \code{pnl_n} is specified
#'   as \code{m}, then \code{m} panels will be created.  For example, \code{pnl_dsgn = c(1, 4, 1, 4)}
#'   and \code{pnl_n = 3}, [(1-4-1-4)^3] in McDonald's notation, will create first three
#'   panels of the 510 serially alternating panels specified by \code{pnl_dsgn}.
#'
#'   A serially alternating or rotating panel revisit design may not result in
#'   the same number of units being sampled during each time period,
#'   particularly during the initial start up period.  The default is to not
#'   specify a startup option (\code{"None"}).  Start up option \code{"Partial_Begin"}
#'   initiates the revisit design at the last time period scheduled for sampling
#'   in the first panel. For example, a [2-3-1-4] design starts at time period 6
#'   instead of time period 1 under the Partial_Begin option. For a serially
#'   alternating panel structure, start up option \code{"Partial_End"} initiates the
#'   revisit design at the time period that begins the second serially
#'   alternating pattern. For example, a [2-3-1-4] design starts at time period
#'   11 instead of time period 1. For a rotating panel structure design, use of
#'   Partial_End makes the assumption that the number of panels equals the
#'   number of time periods and adds units to the last "m" panels for time
#'   periods \code{1} to \code{"m"} as if number of time periods was extended by \code{"m"} where \code{"m"}
#'   is one less than then the sum of the panel design.  For example, a
#'   [1-4-1-4-1-n] design would result in \code{m = 10}.  Note that some designs with
#'   \code{pnl_n} not equal to the number of sample occasions can produce unexpected
#'   panel designs.  See examples.
#'
#'   Different types of panel structures can be combined, these are termed split
#'   panels by many authors, by specifying more than one list for the panels
#'   parameter.  The total number of panels is the sum of the number of panels
#'   in each of the panel structures specified by the split panel design.
#'
#' @return A two-dimensional array of sample sizes to be sampled at each
#'   combination of panel and time period.
#'
#' @references
#'   McDonald, T. (2003). Review of environmental monitoring methods: survey
#'   designs. \emph{Environmental Monitoring and Assessment} \bold{85},
#'   277-292.
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#' @seealso
#'   \describe{
#'     \item{\code{\link{revisit_bibd}}}{to create a balanced incomplete block
#'       panel revisit design}
#'     \item{\code{\link{revisit_rand}}}{to create a revisit design with random
#'       assignment to panels and time periods}
#'     \item{\code{\link{pd_summary}}}{ to summarize characteristics of a
#'       panel revisit design}
#'   }
#'
#' @keywords survey
#'
#' @examples
#' # One panel of  60 sample units sampled at every time period: [1-0]
#' revisit_dsgn(20, panels = list(
#'   Annual = list(
#'     n = 60, pnl_dsgn = c(1, 0), pnl.n = NA,
#'     start_option = "None"
#'   )
#' ), begin = 1)
#'
#' # Rotating panels of 60 units sampled once and never again: [1-n].  Number
#' # of panels equal n_period.
#' revisit_dsgn(20,
#'   panels = list(
#'     R60N = list(n = 60, pnl_dsgn = c(1, NA), pnl_n = NA, start_option = "None")
#'   ),
#'   begin = 1
#' )
#'
#' # Serially alternating panel with three visits to sample unit then skip
#' # next two time periods: [3-2]
#' revisit_dsgn(20, panels = list(
#'   SA60PE = list(
#'     n = 20, pnl_dsgn = c(3, 2), pnl_n = NA,
#'     start_option = "Partial_End"
#'   )
#' ), begin = 1)
#'
#' # Split panel of sample units combining above two panel designs: [1-0, 1-n]
#' revisit_dsgn(n_period = 20, begin = 2017, panels = list(
#'   Annual = list(
#'     n = 60, pnl_dsgn = c(1, 0), pnl.n = NA,
#'     start_option = "None"
#'   ),
#'   R60N = list(n = 60, pnl_dsgn = c(1, NA), pnl_n = NA, start_option = "None")
#' ))
#' @export
###############################################################################

revisit_dsgn <- function(n_period, panels, begin = 1, skip = 1) {
  if (!is.list(panels)) {
    stop("\npanels must be a list of lists")
  }

  # initialize final panel design array
  panels_dsgn <- NULL

  ## cycle through the number of panel structures
  for (nsplit in 1:length(names(panels))) {
    tmp_n <- panels[[nsplit]]$n
    tmp_dsgn <- panels[[nsplit]]$pnl_dsgn
    tmp_pnl_n <- panels[[nsplit]]$pnl_n
    tmp_start_option <- panels[[nsplit]]$start_option
    # determine if start up option required. If num_vis > 1 start up options okay
    num_vis <- sum(tmp_dsgn[seq(1, by = 2, length.out = length(tmp_dsgn) / 2)])

    # check that pnl_dsgn has even number of elements as required
    if (length(tmp_dsgn) %% 2 == 1) {
      stop("\nVector specifying pnl_dsgn must have even number of elements")
    }

    # check start options
    if (!(tmp_start_option %in% c("None", "Partial_Begin", "Partial_End"))) {
      stop("\nStart option for a panel design is not one of None, Partial_Begin, Partial_End")
    }

    # check that pnl_dsgn ends in NA, 0 or > 0 as required
    pnl_type <- tmp_dsgn[length(tmp_dsgn)]
    if (!(pnl_type == 0 | is.na(pnl_type) | pnl_type > 0)) {
      stop("\nVector specifying pnl_dsgn must end in NA, 0, or > 0")
    }

    # set up panel core visit, skip schedule that may or may not be repeated
    # v_len is number of occasions in a panel cycle
    v_len <- sum(tmp_dsgn, na.rm = TRUE)
    visit <- rep(tmp_n, tmp_dsgn[1])
    if (length(tmp_dsgn) > 1) {
      for (j in 2:length(tmp_dsgn)) {
        if (j %% 2 == 1) visit <- c(visit, rep(tmp_n, tmp_dsgn[j]))
        if (j %% 2 == 0 & !is.na(tmp_dsgn[j])) visit <- c(visit, rep(0, tmp_dsgn[j]))
      }
    }

    ####  Always revisit panels
    if (!is.na(pnl_type) & pnl_type == 0) {
      pan_dsgn <- array(c(visit, rep(tmp_n, n_period - length(visit))), c(1, n_period))
      # assign dimnames
      dimnames(pan_dsgn) <- list(names(panels)[nsplit], seq(begin, by = skip, length.out = n_period))
      # combine with other panels if any
      panels_dsgn <- rbind(panels_dsgn, pan_dsgn)
    }

    #### serially alternating panel type designS
    if (!is.na(pnl_type) & pnl_type > 0) {

      # determine number of panels
      n_panels <- sum(tmp_dsgn)
      n_cycle <- sum(tmp_dsgn)

      pan_dsgn <- rep(visit, ceiling(1 + n_period / v_len))
      for (i in 2:n_panels) {
        tmp <- c(rep(0, i - 1), rep(visit, ceiling(1 + n_period / v_len)))
        pan_dsgn <- rbind(pan_dsgn, tmp[1:(length(tmp) - i + 1)])
      }

      ## set up start options: None, Partial_Begin, Partial_End

      # start.option = None
      if (tmp_start_option == "None") {
        pan_dsgn <- pan_dsgn[, 1:n_period]
      }

      # set up start.option = Partial_Begin
      if (tmp_start_option == "Partial_Begin") {
        if (num_vis == 1) {
          pan_dsgn <- pan_dsgn[, 1:n_period]
        }
        if (num_vis > 1) {
          itmp <- sum(tmp_dsgn[1:(length(tmp_dsgn) - 1)])
          pan_dsgn <- pan_dsgn[, itmp:ncol(pan_dsgn)]
          pan_dsgn <- pan_dsgn[, 1:n_period]
        }
      }

      # set up start.option = Partial_End
      if (tmp_start_option == "Partial_End") {
        if (num_vis == 1) {
          pan_dsgn <- pan_dsgn[, 1:n_period]
        }
        if (num_vis > 1) {
          itmp <- n_cycle + 1
          pan_dsgn <- pan_dsgn[, itmp:ncol(pan_dsgn)]
          pan_dsgn <- pan_dsgn[, 1:n_period]
        }
      }

      # check to see if want to use all panels
      if (!is.na(tmp_pnl_n)) {
        pan_dsgn <- pan_dsgn[1:tmp_pnl_n, ]
      }

      # assign dimnames
      if (nrow(pan_dsgn) < 10) {
        dimnames(pan_dsgn) <- list(
          c(paste(names(panels)[nsplit], 1:nrow(pan_dsgn), sep = "_")),
          seq(begin, by = skip, length.out = ncol(pan_dsgn))
        )
      } else {
        dimnames(pan_dsgn) <- list(
          c(
            paste(names(panels)[nsplit], 1:9, sep = "_0"),
            paste(names(panels)[nsplit], 10:nrow(pan_dsgn), sep = "_")
          ),
          seq(begin, by = skip, length.out = ncol(pan_dsgn))
        )
      }
      # combine with other panels if any
      panels_dsgn <- rbind(panels_dsgn, pan_dsgn)
    }

    #### rotating panel type designs
    if (is.na(pnl_type)) {

      # determine cycle length based on number of sampling occasions
      # from first through last sample visit
      n_cycle <- sum(tmp_dsgn, na.rm = TRUE)
      n_panels <- n_period

      for (i in 1:(n_panels + n_cycle)) {
        if (i == 1) {
          pan_dsgn <- c(visit, rep(0, n_period + n_cycle))
        } else {
          pan_dsgn <- rbind(pan_dsgn, c(rep(0, i - 1), visit, rep(0, n_period + n_cycle - (i - 1))))
        }
      }

      ## set up start options: None, Partial_Begin, Partial_End

      # if num_vis equal to 1 then no start option required
      if (num_vis == 1) {
        # shorten columns to n_period and keep only panels with at least one sample occasion
        pan_dsgn <- pan_dsgn[, 1:n_period]
        keep <- ifelse(apply(pan_dsgn, 1, sum) > 0, TRUE, FALSE)
        pan_dsgn <- pan_dsgn[keep, ]
      }

      # start.option = None
      if (tmp_start_option == "None") {
        # shorten columns to n_period and keep only panels with at least one sample occasion
        pan_dsgn <- pan_dsgn[, 1:n_period]
        keep <- ifelse(apply(pan_dsgn, 1, sum) > 0, TRUE, FALSE)
        pan_dsgn <- pan_dsgn[keep, ]
      }
      # set up start.option = Partial_Begin
      if (tmp_start_option == "Partial_Begin" & num_vis > 1) {
        itmp <- sum(tmp_dsgn[1:(length(tmp_dsgn) - 1)])
        pan_dsgn <- pan_dsgn[, itmp:ncol(pan_dsgn)]

        # shorten columns to n_period and keep only panels with at least one sample occasion
        pan_dsgn <- pan_dsgn[, 1:n_period]
        keep <- ifelse(apply(pan_dsgn, 1, sum) > 0, TRUE, FALSE)
        pan_dsgn <- pan_dsgn[keep, ]
      }
      # set up start.option = Partial_End:
      if (tmp_start_option == "Partial_End" & num_vis > 1) {
        if (v_len > 1) {
          for (i in 1:(v_len - 1)) {
            pan_dsgn[n_panels - v_len + 1 + i, 1:i] <- visit[(v_len + 1 - i):v_len]
          }
        }
        # shorten columns to n_period and keep only panels with at least one sample occasion
        pan_dsgn <- pan_dsgn[, 1:n_period]
        keep <- ifelse(apply(pan_dsgn, 1, sum) > 0, TRUE, FALSE)
        pan_dsgn <- pan_dsgn[keep, ]
      }

      # check to see if want to use all panels
      if (!is.na(tmp_pnl_n)) {
        pan_dsgn <- pan_dsgn[1:tmp_pnl_n, ]
      }

      # assign dimnames
      if (nrow(pan_dsgn) < 10) {
        dimnames(pan_dsgn) <- list(
          c(paste(names(panels)[nsplit], 1:nrow(pan_dsgn), sep = "_")),
          seq(begin, by = skip, length.out = ncol(pan_dsgn))
        )
      } else {
        dimnames(pan_dsgn) <- list(
          c(
            paste(names(panels)[nsplit], 1:9, sep = "_0"),
            paste(names(panels)[nsplit], 10:nrow(pan_dsgn), sep = "_")
          ),
          seq(begin, by = skip, length.out = ncol(pan_dsgn))
        )
      }
      # combine with All Revisit and Serially Alternating panel designs if any
      panels_dsgn <- rbind(panels_dsgn, pan_dsgn)
    }
    # end of rotating panel structure
  }

  # return final revisit panel design structure
  class(panels_dsgn) <- "paneldesign"
  return(panels_dsgn)
  
}

################################################################################
# Function: interp_cdf (not exported)
# Programmer Tom Kincaid
# Date: May 5, 2021
#
#' Interpolate CDF Values at a Set of Percentiles
#'
#' This function interpolates CDF values at a set of percentiles.  The CDF
#' values can be CDF estimates, CDF confidence bound estimates, or values at
#' which the CDF is estimated (i.e., x-axis values).  It is assumed that
#' arguments cdfest_p and cdf_value are strictly increasing.
#'
#' @param pctval Vector of percentiles (expressed as percents) at which the
#'   CDF values are to be interpolated.
#'
#' @param cdfest_p Vector of CDF estimates in terms of proportions.
#'
#' @param cdf_value Vector of CDF values to be interpolated.
#'
#' @return A numeric vector consisting of the interpolated CDF values.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @noRd
################################################################################

interp_cdf <- function(pctval, cdfest_p, cdf_value) {
  nvec <- 1:length(cdfest_p)
  rslt <- numeric(0)
  for (j in 1:length(pctval)) {
    high <- ifelse(length(nvec[cdfest_p >= pctval[j]]) > 0,
      min(nvec[cdfest_p >= pctval[j]]), NA
    )
    low <- ifelse(length(nvec[cdfest_p <= pctval[j]]) > 0,
      max(nvec[cdfest_p <= pctval[j]]), NA
    )
    if (is.na(high)) {
      rslt[j] <- NA
    } else if (is.na(low)) {
      rslt[j] <- cdf_value[high]
    } else {
      if (high > low) {
        pdis <- (pctval[j] - cdfest_p[low]) / (cdfest_p[high] - cdfest_p[low])
        rslt[j] <- cdf_value[low] + pdis * (cdf_value[high] - cdf_value[low])
      } else {
        rslt[j] <- cdf_value[high]
      }
    }
  }

  return(rslt)
}

################################################################################
# Function: trend_analysis (exported)
# Programmer: Tom Kincaid
# Date: March 3, 2021
# Revised March 5, 2021 to add argument jointprob that is required for
#         additional variance estimators
# Revised: March 10, 2021 to assign zero to missing year estimates when fitting
#          the SLR and WLR models for categorical variables
# Revised: April 7, 2021 to ensure that the dframe argument does not contain
#          zero rows
# Revised: April 29, 2021 to ensure that the dframe argument only belongs to
#          class "data.frame"
# Revised: May 4, 2021 to avoid warning messages being generated during creation
#          of help files
# Revised: May 6, 2021 to ensure that sf objects do not belong to class tbl_df
# Revised: June 8, 2021 to simplify specification of the values required for
#          calculation of the finite population correction factor and to
#          eliminate use of the finite population correction factor with the
#          local mean variance estimator
# Revised: June 14, 2021 to use the new function named mean_est for calculation
#          of mean estimates
# Revised: July 28, 2021 to ensure that data is available for a sufficient
#          number of years to perform linear regression for categorical
#          variables
# Revised: August 9, 2021 to correct an error when appending values to the
#          contsum data frame
# Revised: September 9, 2021 to revise the documentation for argument popsize
# Revised: September 17, 2021 to allow the user to specify the formula for a
#          linear mixed-effects model and to allow use of generalized linear
#          mixed-effects models for categorical data
#
#' Trend analysis
#'
#' This function organizes input and output for estimation of trend across time
#' for a series of samples (for categorical and continuous variables). Trend is estimated using the
#' analytical procedure identified by the model arguments.  For categorical
#' variables, the choices for the \code{model_cat} argument are: (1) simple linear
#' regression, (2) weighted linear regression, and (3) generalized linear
#' mixed-effects model. For continuous variables, the choices for the
#' \code{model_cont} argument are: (1) simple linear regression, (2) weighted
#' linear regression, and (3)  linear mixed-effects model.  The analysis data,
#' \code{dframe}, can be either a data frame or a simple features (\code{sf}) object.  If an
#' \code{sf} object is used, coordinates are extracted from the geometry column in the
#' object, arguments \code{xcoord} and \code{ycoord} are assigned values
#' \code{"xcoord"} and \code{"ycoord"}, respectively, and the geometry column is
#' dropped from the object.
#'
#' @param dframe Data to be analyzed (analysis data). A data frame or
#'   \code{sf} object containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param vars_cat Vector composed of character values that identify the names
#'   of categorical response variables in \code{dframe}. If
#'   argument \code{model_cat} equals "GLMM", the categorical variables in the
#'   \code{dframe} data frame must be factors each of which has two levels,
#'   where the second level will be assumed to specify "success".  The default
#'   value is \code{NULL}.
#'
#' @param vars_cont Vector composed of character values that identify the
#'   names of continuous response variables in \code{dframe}.
#'   The default value is \code{NULL}.
#'
#' @param subpops Vector composed of character values that identify the
#'   names of subpopulation (domain) variables in \code{dframe}.
#'   If a value is not provided, the value \code{"All_Sites"} is assigned to the
#'   subpops argument and a factor variable named \code{"All_Sites"} that takes
#'   the value \code{"All Sites"} is added to \code{dframe}.  The
#'   default value is \code{NULL}.
#'
#' @param model_cat Character value identifying the analytical procedure used
#'   for trend estimation for categorical variables.  The choices are:
#'   \code{"SLR"} (simple linear regression), \code{"WLR"} (weighted linear
#'   regression), and \code{"GLMM"} (generalized linear mixed-effects model).
#'   The default value is \code{"SLR"}.
#'
#' @param cat_rhs Character value specifying the right hand side of the formula
#'   for a generalized linear mixed-effects model.  If a value is not provided,
#'   the argument is assigned a value that specifies the Piepho and Ogutu (2002)
#'   model.  The default value is \code{NULL}.
#'
#' @param model_cont Character value identifying the analytical procedure used
#'   for trend estimation for continuous variables.  The choices are:
#'   \code{"SLR"} (simple linear regression), \code{"WLR"} (weighted linear
#'   regression), and \code{"LMM"} (linear mixed-effects model).  The default
#'   value is \code{"LMM"}.
#'
#' @param cont_rhs Character value specifying the right hand side of the
#'   formula for a linear mixed-effects model.  If a value is not provided, the
#'   argument is assigned a value that specifies the Piepho and Ogutu (2002)
#'   model.  The default value is \code{NULL}.
#'
#' @param siteID Character value providing name of the site ID variable in
#'   \code{dframe}.  If repeat visit sites are present, the site
#'   ID value for each revisit site will be the same for each survey.  For a
#'   two-stage sample, the site ID variable identifies stage two site IDs.  The
#'   default value is \code{"siteID"}.
#'
#' @param yearID Character value providing name of the time period variable in
#'   \code{dframe}, which must be numeric and will be forced to
#'   numeric if it is not.  The default assumption is that the time period
#'   variable is years.  The default value is \code{"year"}.
#'
#' @param weight Character value providing name of the design weight
#'   variable in \code{dframe}.  For a two-stage sample, the
#'   weight variable identifies stage two weights.  The default value is
#'   \code{"weight"}.
#'
#' @param xcoord Character value providing name of the x-coordinate variable in
#'   \code{dframe}.  For a two-stage sample, the x-coordinate
#'   variable identifies stage two x-coordinates.  Note that x-coordinates are
#'   required  for calculation of the local mean variance estimator.  If \code{dframe}
#'   is an \code{sf} object, this argument is not required (as the geometry column
#'   in \code{dframe} is used to find the x-coordinate). The
#'   default value is \code{NULL}.
#'
#' @param ycoord Character value providing name of the y-coordinate variable in
#'   \code{dframe}.  For a two-stage sample, the y-coordinate
#'   variable identifies stage two y-coordinates.  Note that y-coordinates are
#'   required for calculation of the local mean variance estimator.  If \code{dframe}
#'   is an \code{sf} object, this argument is not required (as the geometry column
#'   in \code{dframe} is used to find the y-coordinate). The default
#'   value is \code{NULL}.
#'
#' @param stratumID Character value providing name of the stratum ID variable in
#'   \code{dframe}.  The default value is \code{NULL}.
#'
#' @param clusterID Character value providing name of the cluster (stage one) ID
#'   variable in \code{dframe}.  Note that cluster IDs are
#'   required for a two-stage sample.  The default value is \code{NULL}.
#'
#' @param weight1 Character value providing name of the stage one weight
#'   variable in \code{dframe}.  The default value is \code{NULL}.
#'
#' @param xcoord1 Character value providing name of the stage one x-coordinate
#'   variable in \code{dframe}.  Note that x-coordinates are
#'   required for calculation of the local mean variance estimator.  The default
#'   value is \code{NULL}.
#'
#' @param ycoord1 Character value providing name of the stage one y-coordinate
#'   variable in \code{dframe}.  Note that y-coordinates are
#'   required for calculation of the local mean variance estimator.  The default
#'   value is \code{NULL}.
#'
#' @param sizeweight Logical value that indicates whether size weights should be
#'   used during estimation, where \code{TRUE} = use size weights and
#'   \code{FALSE} = do not use size weights. To employ size weights for a
#'   single-stage sample, a value must be supplied for argument weight.  To
#'   employ size weights for a two-stage sample, values must be supplied for
#'   arguments \code{weight} and \code{weight1}. The default value is
#'   \code{FALSE}.
#'
#' @param sweight Character value providing name of the size weight variable in
#'   \code{dframe}.  For a two-stage sample, the size weight
#'   variable identifies stage two size weights.  The default value is
#'   \code{NULL}.
#'
#' @param sweight1 Character value providing name of the stage one size weight
#'   variable in \code{dframe}.  The default value is
#'   \code{NULL}.
#'
#' @param fpc Object that specifies values required for calculation of the
#'   finite population correction factor used during variance estimation. The
#'   object must match the survey design in terms of stratification and whether
#'   the design is single-stage or two-stage.  For an unstratified design, the
#'   object is a vector.  The vector is composed of a single numeric value for a
#'   single-stage design.  For a two-stage unstratified design, the object is a
#'   named vector containing one more than the number of clusters in the sample,
#'   where the first item in the vector specifies the number of clusters in the
#'   population and each subsequent item specifies the number of stage two units
#'   for the cluster.  The name for the first item in the vector is arbitrary.
#'   Subsequent names in the vector identify clusters and must match the cluster
#'   IDs.  For a stratified design, the object is a named list of vectors, where
#'   names must match the strata IDs.  For each stratum, the format of the
#'   vector is identical to the format described for unstratified single-stage
#'   and two-stage designs.  Note that the finite population correction factor
#'   is not used with the local mean variance estimator.
#'
#'   Example fpc for a single-stage unstratified survey design:
#'
#'   \verb{fpc <- 15000}
#'
#'   Example fpc for a single-stage stratified survey design:
#'
#'   \verb{fpc <- list(
#'     Stratum_1 = 9000,
#'     Stratum_2 = 6000)
#'    }
#'
#'   Example fpc for a two-stage unstratified survey design:
#'
#'   \verb{fpc <- c(
#'     Ncluster = 150,
#'     Cluster_1 = 150,
#'     Cluster_2 = 75,
#'     Cluster_3 = 75,
#'     Cluster_4 = 125,
#'     Cluster_5 = 75)
#'   }
#'
#'   Example fpc for a two-stage stratified survey design:
#'
#'   \verb{fpc <- list(
#'     Stratum_1 = c(
#'       Ncluster_1 = 100,
#'       Cluster_1 = 125,
#'       Cluster_2 = 100,
#'       Cluster_3 = 100,
#'       Cluster_4 = 125,
#'       Cluster_5 = 50),
#'     Stratum_2 = c(
#'       Ncluster_2 = 50,
#'       Cluster_1 = 75,
#'       Cluster_2 = 150,
#'       Cluster_3 = 75,
#'       Cluster_4 = 75,
#'       Cluster_5 = 125))
#'   }
#'
#' @param popsize Object that provides values for the population argument of the
#'   \code{calibrate} or \code{postStratify} functions in the survey package. If
#'   a value is provided for popsize, then either the \code{calibrate} or
#'   \code{postStratify} function is used to modify the survey design object
#'   that is required by functions in the survey package.  Whether to use the
#'   \code{calibrate} or \code{postStratify} function is dictated by the format
#'   of popsize, which is discussed below.  Post-stratification adjusts the
#'   sampling and replicate weights so that the joint distribution of a set of
#'   post-stratifying variables matches the known population joint distribution.
#'   Calibration, generalized raking, or GREG estimators generalize
#'   post-stratification and raking by calibrating a sample to the marginal
#'   totals of variables in a linear regression model. For the \code{calibrate}
#'   function, the object is a named list, where the names identify factor
#'   variables in \code{dframe}.  Each element of the list is a
#'   named vector containing the population total for each level of the
#'   associated factor variable.  For the \code{postStratify} function, the
#'   object is either a data frame, table, or xtabs object that provides the
#'   population total for all combinations of selected factor variables in the
#'   \code{dframe} data frame.  If a data frame is used for \code{popsize}, the
#'   variable containing population totals must be the last variable in the data
#'   frame.  If a table is used for \code{popsize}, the table must have named
#'   \code{dimnames} where the names identify factor variables in the
#'   \code{dframe} data frame.  If the popsize argument is equal to \code{NULL},
#'   then neither calibration nor post-stratification is performed.  The default
#'   value is \code{NULL}.
#'
#'   Example popsize for calibration:
#'
#'   \verb{popsize <- list(
#'     Ecoregion = c(
#'       East = 750,
#'       Central = 500,
#'       West = 250),
#'     Type = c(
#'       Streams = 1150,
#'       Rivers = 350))
#'   }
#'
#'   Example popsize for post-stratification using a data frame:
#'
#'   \verb{popsize <- data.frame(
#'     Ecoregion = rep(c("East", "Central", "West"),
#'       rep(2, 3)),
#'     Type = rep(c("Streams", "Rivers"), 3),
#'     Total = c(575, 175, 400, 100, 175, 75))
#'   }
#'
#'   Example popsize for post-stratification using a table:
#'
#'   \verb{popsize <- with(MySurveyFrame,
#'     table(Ecoregion, Type))}
#'
#'   Example popsize for post-stratification using an xtabs object:
#'
#'   \verb{popsize <- xtabs(~Ecoregion + Type,
#'     data = MySurveyFrame)}
#'
#' @param invprboot Logical value that indicates whether the inverse probability
#'   bootstrap procedure is used to calculate trend parameter estimates.  This
#'   bootstrap procedure is only available for the "LMM" option for continuous
#'   variables.  Inverse probability references the design weights, which
#'   are the inverse of the sample inclusion probabilities.  The default value
#'   is \code{TRUE}.
#'
#' @param nboot Numeric value for the number of bootstrap iterations.  The
#'   default is \code{1000}.
#'
#' @param vartype Character value providing choice of the variance estimator,
#'   where \code{"Local"} = the local mean estimator, \code{"SRS"} = the simple
#'   random sampling estimator, \code{"HT"} = the Horvitz-Thompson estimator,
#'   and \code{"YG"} = the Yates-Grundy estimator.  The default value is
#'   \code{"Local"}.
#'
#' @param jointprob Character value providing choice of joint inclusion
#'   probability approximation for use with Horvitz-Thompson and Yates-Grundy
#'   variance estimators, where \code{"overton"} indicates the Overton
#'   approximation, \code{"hr"} indicates the Hartley_Rao approximation, and
#'   \code{"brewer"} equals the Brewer approximation.  The default value is
#'   \code{"overton"}.
#'
#' @param conf Numeric value for the Gaussian-based confidence level.  The default is
#'   \code{95}.
#'
#' @param All_Sites A logical variable used when \code{subpops} is not
#'   \code{NULL}. If \code{All_Sites} is \code{TRUE}, then alongside the
#'   subpopulation output, output for all sites (ignoring subpopulations) is
#'   returned for each variable in \code{vars}. If \code{All_Sites} is
#'   \code{FALSE}, then alongside the subpopulation output, output for all sites
#'   (ignoring subpopulations) is not returned for each variable in \code{vars}.
#'   The default is \code{FALSE}.
#'
#' @section Details:
#' For the simple linear regression (SLR) model, a design-based estimate of the
#' category proportion (categorical variables) or the mean (continuous
#' variables) is calculated for each time period (year).  Four choices of
#' variance estimator are available for calculating variance of the design-based
#' estimates: (1) the local mean estimator, (2) the simple random sampling
#' estimator, (3) the Horvitz-Thompson estimator, and (4) the Yates-Grundy
#' estimator.  For the Horvitz-Thompson and Yates-Grundy estimators, there are
#' three choices for calculating joint inclusion probabilities: (1) the Overton
#' approximation, (2) the Hartley-Rao approximation, and (3) the Brewer
#' approximation.  The \code{lm} function in the stats package is used to fit a
#' linear model using a \code{formula} argument that specifies the proportion or
#' mean estimates as the response variable and years as the regressor variable.
#' For fitting the SLR model, the \code{yearID} variable from the \code{dframe}
#' argument is modified by subtracting the minimum value of years from all
#' values of the variable.  Parameter estimates are extracted from the object
#' returned by the \code{lm} function.  For the weighted linear regression (WLR)
#' model, the process is the same as the SLR model except that the inverse of
#' the variances of the proportion or mean estimates is used as the
#' \code{weights} argument in the call to the \code{lm} function.  For the LMM
#' option, the \code{lmer} function in the lme4 package is used to fit a linear
#' mixed-effects model for trend across years.  For both the GLMM and LMM
#' options, the default Piepho and Ogutu (PO) model includes fixed effects for
#' intercept and trend (slope) and random effects for intercept and trend for
#' individual sites, where the \code{siteID} variable from the \code{dframe}
#' argument identifies sites.  Correlation between the random effects for site
#' intercepts and site trends is included in the model. Finally, the PO model
#' contains random effects for year variance and residual variance. For the GLMM
#' and LMM options, arguments \code{cat_rhs} and \code{cont_rhs}, respectively,
#' can be used to specify the right hand side of the model formula. Internally,
#' a variable named \code{Wyear} is created that is useful for specifying the
#' \code{cat_rhs} and \code{cont_rhs} arguments.  The \code{Wyear} variable is
#' created by subtracting the minimum value of the \code{yearID} variable from
#' all values of the variable.  If argument \code{invprboot} is \code{FALSE},
#' parameter estimates are extracted from the object returned by the \code{lmer}
#' function. If argument \code{invprboot} is \code{TRUE}, the \code{boot}
#' function in the boot package is used to generate bootstrap replicates using a
#' function named \code{bootfcn} as the \code{statistic} argument passed to the
#' \code{boot} function.  For each bootstrap replicate, \code{bootfcn} calls the
#' \code{glmer} or \code{lmer} function, as appropriate, using the specified
#' model.  design weights identified by the \code{weight} argument for
#' the \code{trend_analysis} function are passed as the \code{weights} argument
#' for the \code{boot} function, which specifies importance weights.  Using the
#' design weights as the \code{weights} argument ensures that bootstrap
#' replicates are representative of the survey population.  Parameter estimates
#' are calculated using the object returned by the \code{boot} function.
#'
#' @return The analysis results. A list composed of two data frames containing trend estimates for all
#'   combinations of population Types, subpopulations within Types, and response
#'   variables.  For categorical variables, trend estimates are calculated for
#'   each category of the variable.  The two data frames in the output list are:
#'   \describe{
#'     \item{\code{catsum}}{data frame containing trend estimates for
#'       categorical variables}
#'     \item{\code{contsum}}{data frame containing trend
#'       estimates for continuous variables}
#'    }
#'
#'   For the SLR and WLR model options, the data frame contains the following
#'   variables:
#'   \describe{
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{Trend_Estimate}{trend estimate}
#'     \item{Trend_Std_Error}{trend standard error}
#'     \item{Trend_LCBxxPct}{trend xx\% (default 95\%) lower confidence bound}
#'     \item{Trend_UCBxxPct}{trend xx\% (default 95\%) upper confidence bound}
#'     \item{Trend_p_Value}{trend p-value}
#'     \item{Intercept_Estimate}{intercept estimate}
#'     \item{Intercept_Std_Error}{intercept standard error}
#'     \item{Intercept_LCBxxPct}{intercept xx\% (default 95\%) lower confidence
#'       bound}
#'     \item{Intercept_UCBxxPct}{intercept xx\% (default 95\%) upper confidence
#'       bound}
#'     \item{Intercept_p_Value}{intercept p-value}
#'     \item{R_Squared}{R-squared value}
#'     \item{Adj_R_Squared}{adjusted R-squared value}
#'   }
#'
#'   For the GLMM and LMM model options, contents of the data frames will vary
#'   depending on the model specified by arguments \code{cat_rhs} and
#'   \code{cont_rhs}.  For the default PO model, the data frame contains the
#'   following variables:
#'   \describe{
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Indicator}{response variable}
#'     \item{Trend_Estimate}{trend estimate}
#'     \item{Trend_Std_Error}{trend standard error}
#'     \item{Trend_LCBxxPct}{trend xx\% (default 95\%) lower confidence bound}
#'     \item{Trend_UCBxxPct}{trend xx\% (default 95\%) upper confidence bound}
#'     \item{Trend_p_Value}{trend p-value}
#'     \item{Intercept_Estimate}{intercept estimate}
#'     \item{Intercept_Std_Error}{intercept standard error}
#'     \item{Intercept_LCBxxPct}{intercept xx\% (default 95\%) lower confidence
#'       bound}
#'     \item{Intercept_UCBxxPct}{intercept xx\% (default 95\%) upper confidence
#'       bound}
#'     \item{Intercept_p_Value}{intercept p-value}
#'     \item{Var_SiteInt}{variance of the site intercepts}
#'     \item{Var_SiteTrend}{variance of the site trends}
#'     \item{Corr_SiteIntSlope}{correlation of site intercepts and site trends}
#'     \item{Var_Year}{year variance}
#'     \item{Var_Residual}{residual variance}
#'     \item{AIC}{generalized Akaike Information Criterion}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@epa.gov}
#'
#' @keywords survey
#'
#' @seealso
#'   \describe{
#'   \item{\code{\link{change_analysis}}}{for change analysis}
#'   }
#'
#' @examples
#' # Example using a categorical variable with three resource classes and a
#' # continuous variable
#' mydframe <- data.frame(
#'   siteID = rep(paste0("Site", 1:40), rep(5, 40)),
#'   yearID = rep(seq(2000, 2020, by = 5), 40),
#'   wgt = rep(runif(40, 10, 100), rep(5, 40)),
#'   xcoord = rep(runif(40), rep(5, 40)),
#'   ycoord = rep(runif(40), rep(5, 40)),
#'   All_Sites = rep("All Sites", 200),
#'   Region = sample(c("North", "South"), 200, replace = TRUE),
#'   Resource_Class = sample(c("Good", "Fair", "Poor"), 200, replace = TRUE),
#'   ContVar = rnorm(200, 10, 1)
#' )
#' myvars_cat <- c("Resource_Class")
#' myvars_cont <- c("ContVar")
#' mysubpops <- c("All_Sites", "Region")
#' trend_analysis(
#'   dframe = mydframe,
#'   vars_cat = myvars_cat,
#'   vars_cont = myvars_cont,
#'   subpops = mysubpops,
#'   model_cat = "WLR",
#'   model_cont = "SLR",
#'   siteID = "siteID",
#'   yearID = "yearID",
#'   weight = "wgt",
#'   xcoord = "xcoord",
#'   ycoord = "ycoord"
#' )
#' @export
################################################################################

trend_analysis <- function(dframe, vars_cat = NULL, vars_cont = NULL, subpops = NULL, model_cat = "SLR",
                           cat_rhs = NULL, model_cont = "LMM", cont_rhs = NULL, siteID = "siteID",
                           yearID = "year", weight = "weight", xcoord = NULL, ycoord = NULL,
                           stratumID = NULL, clusterID = NULL, weight1 = NULL, xcoord1 = NULL,
                           ycoord1 = NULL, sizeweight = FALSE, sweight = NULL, sweight1 = NULL,
                           fpc = NULL, popsize = NULL, invprboot = TRUE, nboot = 1000, vartype = "Local",
                           jointprob = "overton", conf = 95, All_Sites = FALSE) {

  # Create a vector for error messages

  error_ind <- FALSE
  error_vec <- NULL

  # Create a data frame for warning messages

  warn_ind <- FALSE
  warn_df <- NULL
  fname <- "trend_analysis"

  # Ensure that the dframe argument was provided

  if (missing(dframe)) {
    stop("\nThe dframe argument must be provided.\n")
  }

  # If the dframe argument is an sf object, extract coordinates from the
  # geometry column, assign values "xcoord" and "ycoord" to arguments xcoord and
  # ycoord, respectively, and drop the geometry column from the object

  if ("sf" %in% class(dframe)) {
    temp <- st_coordinates(dframe)
    xcoord <- "xcoord"
    dframe$xcoord <- temp[, "X"]
    ycoord <- "ycoord"
    dframe$ycoord <- temp[, "Y"]
    dframe <- st_set_geometry(dframe, NULL)
  }

  # If the dframe argument is a tibble or does not belong to class
  # "data.frame", coerce the argument to class "data.frame"

  if ("tbl_df" %in% class(dframe) | !("data.frame" %in% class(dframe))) {
    dframe <- as.data.frame(dframe)
  }

  # Ensure that the dframe argument does not contain zero rows

  if (nrow(dframe) == 0) {
    stop("\nThe dframe argument contains zero rows.\n")
  }

  # Ensure that unused levels are dropped from factor variables in the dframe
  # data frame

  dframe <- droplevels(dframe)

  # Ensure that the dframe data frame contains the site ID variable

  if (!(siteID %in% names(dframe))) {
    ind <- FALSE
    error_ind <- TRUE
    msg <- paste0("The name provided for the siteID argument, \"", siteID, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  } else {
    ind <- TRUE
  }

  # Create a variable containing the original values of the site ID variable in
  # the dframe data frame and a variable containing a unique set of values
  # created from the values of the site ID variable in the dframe data frame

  if (ind) {
    dframe$siteID_orginal <- dframe[, siteID]
    dframe$siteID_unique <- dframe[, siteID]
    temp <- sapply(split(dframe$siteID_unique, dframe$siteID_unique), length)
    if (any(temp > 1)) {
      dframe$siteID_unique <- uniqueID(dframe$siteID_unique)
    }
  }

  # Ensure that the dframe data frame contains the time period identifier
  # variable and ensure that the variable is numeric

  if (!(yearID %in% names(dframe))) {
    error_ind <- TRUE
    msg <- paste0("The name provided for the yearID argument, \"", yearID, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  } else {
    if (!is.numeric(dframe[, yearID])) {
      warn_ind <- TRUE
      warn <- paste0("The variable in the dframe data frame identified by argument yearID, \"", yearID, "\", was coerced \nto class numeric.\n")
      act <- "Variable coerced to class numeric\n"
      warn_df <- rbind(warn_df, data.frame(
        func = I(fname), subpoptype = NA,
        subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
      ))
      dframe[, yearID] <- as.numeric(dframe[, yearID])
    }
  }

  # Ensure that the dframe data frame contains the survey weight variable

  if (!(weight %in% names(dframe))) {
    error_ind <- TRUE
    msg <- paste0("The name provided for the weight argument, \"", weight, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  }

  # Assign names to the variables required for calculation of the finite
  # population correction factor

  if (is.null(fpc)) {
    fpcfactor_ind <- FALSE
    fpcsize <- NULL
    Ncluster <- NULL
    stage1size <- NULL
  } else {
    fpcfactor_ind <- TRUE
    if (is.null(clusterID)) {
      fpcsize <- "fpcsize"
      Ncluster <- NULL
      stage1size <- NULL
    } else {
      fpcsize <- NULL
      Ncluster <- "Ncluster"
      stage1size <- "stage1size"
    }
  }

  # Create a list containing names of survey design variables

  design_names <- list(
    siteID = siteID,
    weight = weight,
    xcoord = xcoord,
    ycoord = ycoord,
    stratumID = stratumID,
    clusterID = clusterID,
    weight1 = weight1,
    xcoord1 = xcoord1,
    ycoord1 = ycoord1,
    sweight = sweight,
    sweight1 = sweight1,
    fpcsize = fpcsize,
    Ncluster = Ncluster,
    stage1size = stage1size
  )

  # Ensure that a value was provided for at least one of the vars_cat
  # (categoroical response variable names) or vars_cont (continuous response
  # variable names) arguments

  if (missing(vars_cat) & missing(vars_cont)) {
    error_ind <- TRUE
    msg <- "A value must be provided for at least one of the vars_cat (categoroical response \nvariable names) or vars_cont (continuous response variable names) arguments.\n"
    error_vec <- c(error_vec, msg)
  }

  # If a value was not provided for the subpops (subpopulation names) argument,
  # assign the value "All_Sites" to the subpops argument and create a factor
  # named "All_Sites" in the dframe data frame that takes the value "All Sites"

  if (is.null(subpops)) {
    subpops <- "All_Sites"
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # If the user wants information for all sites together in addition to the
  # subpops, add the value "All_Sites" to the subpops argument and create a
  # factor named "All_Sites" in the dframe data frame that takes the value
  # "All Sites"

  if (!is.null(subpops) && All_Sites) {
    subpops <- c(subpops, "All_Sites")
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # Ensure that arguments model_cat and model_cont contain valid values

  if (!(model_cat %in% c("SLR", "WLR", "GLMM"))) {
    error_ind <- TRUE
    msg <- paste0("The value provided for argument model_cat, \"", model_cat, "\", is not a valid value.\n")
    error_vec <- c(error_vec, msg)
  }

  if (!(model_cont %in% c("SLR", "WLR", "LMM"))) {
    error_ind <- TRUE
    msg <- paste0("The value provided for argument model_cont, \"", model_cont, "\", is not a valid value.\n")
    error_vec <- c(error_vec, msg)
  }

  # As necessary, assign a value to the cat_rhs and cont_rhs arguments

  PO_ind_cat <- FALSE
  if (model_cat == "GLMM" & is.null(cat_rhs)) {
    PO_ind_cat <- TRUE
    cat_rhs <- paste0("Wyear + (1 + Wyear | ", siteID, ") + (1 | ", yearID, ")")
  }
  PO_ind_cont <- FALSE
  if (model_cont == "LMM" & is.null(cont_rhs)) {
    PO_ind_cont <- TRUE
    cont_rhs <- paste0(
      "Wyear + (1 + Wyear | ", siteID, ") + (1 | ", yearID,
      ")"
    )
  }

  # As necessary, to ensure that the input_check function does not produce an
  # invalid error message, assign "SRS" to argument vartype

  if (is.null(vars_cat) & model_cont == "LMM" |
    !is.null(vars_cat) & model_cat == "GLMM" & model_cont == "LMM") {
    vartype <- "SRS"
  }

  # Check input arguments

  temp <- input_check(dframe, design_names, vars_cat, vars_cont, NULL, NULL,
    subpops, sizeweight, fpc, popsize, vartype, jointprob, conf,
    error_ind = error_ind, error_vec = error_vec
  )
  dframe <- temp$dframe
  vars <- temp$vars_cat
  subpops <- temp$subpops
  popsize <- temp$popsize
  vartype <- temp$vartype
  jointprob <- temp$jointprob
  error_ind <- temp$error_ind
  error_vec <- temp$error_vec

  # As necessary, ensure that variables identified by the vars_cat argument
  # contain two levels

  if (!is.null(vars_cat) & model_cat == "GLMM") {
    tst <- logical(length(vars_cat))
    for (i in 1:length(vars_cat)) {
      tst[i] <- length(levels(dframe[, vars_cat[i]])) != 2
    }
    if (any(tst)) {
      error_ind <- TRUE
      temp_str <- vecprint(vars_cat[tst])
      msg <- paste0("For the \"GLMM\" option, categorical variables must be composed of only two levels.  The \nfollowing variables do not have only two levels:\n", temp_str)
      error_vec <- c(error_vec, msg)
    }
  }

  # As necessary, output a message indicating that error messages were generated
  # during execution of the program

  if (error_ind) {
    error_vec <<- error_vec
    if (length(error_vec) == 1) {
      message("During execution of the program, an error message was generated.  The error \nmessage is stored in a vector named 'error_vec'.  Enter the following command \nto view the error message: errorprnt()\n")
    } else {
      message(paste("During execution of the program,", length(error_vec), "error messages were generated.  The error \nmessages are stored in a vector named 'error_vec'.  Enter the following \ncommand to view the error messages: errorprnt()\n"))
    }

    if (warn_ind) {
      warn_df <<- warn_df
      if (nrow(warn_df) == 1) {
        message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
      } else {
        message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
      }
    }
    stop("See the preceding message(s).")
  }

  # Assign a logical value to the indicator variable for a stratified sample

  stratum_ind <- !is.null(stratumID)

  # For a stratified sample, remove strata that contain a single site

  if (stratum_ind) {
    dframe[, stratumID] <- factor(dframe[, stratumID])
    stratum_levels <- levels(dframe[, stratumID])
    nstrata <- length(stratum_levels)
    ind <- FALSE
    for (i in 1:nstrata) {
      tst <- dframe[, stratumID] == stratum_levels[i]
      if (sum(tst) == 1) {
        warn_ind <- TRUE
        warn <- paste0("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
        act <- "Stratum was removed from the analysis.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = NA,
          subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
        ))
        dframe <- dframe[!tst, ]
        ind <- TRUE
      }
    }
    if (ind) {
      dframe[, stratumID] <- factor(dframe[, stratumID])
      stratum_levels <- levels(dframe[, stratumID])
      nstrata <- length(stratum_levels)
    }
  }

  # Assign a logical value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Create the survey design object

  design <- survey_design(
    dframe, siteID, weight, stratum_ind, stratumID, cluster_ind, clusterID,
    weight1, sizeweight, sweight, sweight1, fpcfactor_ind, fpcsize, Ncluster,
    stage1size, vartype, jointprob
  )

  # If popsize is not equal to NULL, then call either the postStratify or
  # calibrate function, as appropriate

  if (!is.null(popsize)) {
    if (all(class(popsize) %in% c("data.frame", "table", "xtabs"))) {
      if ("data.frame" %in% class(popsize)) {
        pnames <- names(popsize)[-ncol(popsize)]
      } else {
        pnames <- names(dimnames(popsize))
      }
      design <- postStratify(design, make.formula(pnames), popsize)
    } else {
      cnames <- cal_names(make.formula(names(popsize)), design)
      pop.totals <- numeric(length(cnames))
      names(pop.totals) <- cnames
      pop.totals[1] <- sum(popsize[[1]])
      k <- 2
      for (i in names(popsize)) {
        temp <- popsize[[i]]
        for (j in 2:length(temp)) {
          pop.totals[k] <- temp[j]
          k <- k + 1
        }
      }
      design <- calibrate(design, make.formula(names(popsize)), pop.totals)
    }
  }

  # If popsize is not equal to NULL and vartype equals "Local", then assign
  # adjusted weights to the appropriate weight variable(s) in the
  # design$variables data frame

  if (!is.null(popsize) && vartype == "Local") {
    if (cluster_ind) {
      design$variables$wgt2 <- weights(design) / design$variables$wgt1
    } else {
      design$variables$wgt <- weights(design)
    }
  }

  # If invprboot equals TRUE, then assign the bootstrap weights

  if (invprboot == TRUE) {
    if (cluster_ind) {
      bootwgt <- dframe$wgt1 * dframe$wgt2
    } else {
      bootwgt <- dframe$wgt
    }
  }

  # Create a year variable for use in modelling

  Wyear <- "Wyear"
  dframe$Wyear <- dframe[, yearID]
  dframe$Wyear <- dframe$Wyear - min(dframe$Wyear)

  # Assign the confidence bound multiplier

  mult <- qnorm(0.5 + (conf / 100) / 2)

  # Create the output object

  trendsum <- list(catsum = NULL, contsum = NULL)

  #
  # Begin the section for categorical response variables
  #

  if (!is.null(vars_cat)) {

    # Assign values to the site ID variable in the dframe data frame

    if (model_cat %in% c("SLR", "WLR")) {
      dframe[, siteID] <- dframe$siteID_unique
    } else {
      dframe[, siteID] <- dframe$siteID_orginal
    }

    # Loop through all subpopulations (domains)

    for (itype in subpops) {
      lev_itype <- levels(dframe[, itype])
      nlev_itype <- length(lev_itype)

      # Loop through all  response variables

      for (ivar in vars_cat) {
        lev_ivar <- levels(dframe[, ivar])
        nlev_ivar <- length(lev_ivar)

        # Loop through all levels of a subpopulation

        for (isubpop in lev_itype) {

          # Section for the simple linear regression and weighted linear
          # regression models

          if (model_cat %in% c("SLR", "WLR")) {

            # Determine the set of years for this subpopulation

            subpop_ind <- dframe[, itype] %in% isubpop
            years <- sort(unique(dframe[subpop_ind, Wyear]))
            nyears <- length(years)

            # Create matrices to contain category estimates and variance
            # estimates for each time period

            catest <- matrix(NA, nlev_ivar, nyears)
            varest <- matrix(NA, nlev_ivar, nyears)

            # Loop through all time periods for this subpopulation

            for (iyear in 1:nyears) {

              # Select sites in a year for this subpopulation

              subpop_ind <- dframe[, itype] %in% isubpop &
                dframe$Wyear %in% years[iyear]

              # Determine whether the time period for this subpopulation is empty

              if (all(is.na(dframe[subpop_ind, ivar]))) {
                warn_ind <- TRUE
                warn <- paste0("Year ", years[iyear], " of Subpopulation \"", isubpop, "\" of population type \"", itype, "\" for indicator\n \"", ivar, "\" contains no data.\n")
                act <- "None.\n"
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = I(itype), subpop = I(isubpop),
                  indicator = I(ivar), stratum = NA, warning = I(warn), action = I(act)
                ))
                next
              }

              # Determine whether the subpopulation contains a single value

              tst <- !is.na(dframe[subpop_ind, ivar])
              if (sum(tst) == 1) {
                warn_ind <- TRUE
                warn <- paste0("Year ", years[iyear], " of Subpopulation \"", isubpop, "\" of population type \"", itype, "\" for indicator\n \"", ivar, "\" contains a single value.\n")
                act <- "None.\n"
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = I(itype), subpop = I(isubpop),
                  indicator = I(ivar), stratum = NA, warning = I(warn), action = I(act)
                ))
                next
              }

              # Estimate category proportions for the response variable

              tempdf <- subset(dframe, subpop_ind)
              templev <- levels(tempdf[, ivar])
              temp <- category_est(
                NULL, tempdf, itype, isubpop, 1, ivar, templev,
                length(templev), subset(design, subpop_ind), design_names,
                vartype, conf, mult, warn_ind, warn_df
              )
              temp_cat <- temp$catsum
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df

              # Assign the category estimate and variance estimate for each trend
              # category

              for (icat in 1:nlev_ivar) {
                if (lev_ivar[icat] %in% temp_cat$Category) {
                  tst <- temp_cat$Category == lev_ivar[icat]
                  catest[icat, iyear] <- temp_cat$Estimate.P[tst]
                  varest[icat, iyear] <- (temp_cat$StdError.P[tst])^2
                } else {
                  catest[icat, iyear] <- 0
                  varest[icat, iyear] <- 0
                }
              }

              # End of the loop for years
            }

            # Perform linear regression and assign results

            for (icat in 1:nlev_ivar) {
              tst <- !is.na(catest[icat, ])
              if (all(tst == FALSE)) {
                warn_ind <- TRUE
                warn <- paste0("Subpopulation \"", isubpop, "\" of population type \"", itype, "\" for indicator \"", ivar, "\" \ncontains no data for the \"", lev_ivar[icat], "\" category.\n")
                act <- "None.\n"
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = I(itype), subpop = I(isubpop),
                  indicator = I(ivar), stratum = NA, warning = I(warn), action = I(act)
                ))
                next
              } else if (sum(tst) < 3) {
                warn_ind <- TRUE
                warn <- paste0("Subpopulation \"", isubpop, "\" of population type \"", itype, "\" for indicator \"", ivar, "\" \ncontains less than three years of data for the \"", lev_ivar[icat], "\" category.\n")
                act <- "None.\n"
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = I(itype), subpop = I(isubpop),
                  indicator = I(ivar), stratum = NA, warning = I(warn), action = I(act)
                ))
                next
              }
              if (model_cat == "SLR") {
                regest <- lm(catest[icat, tst] ~ years[tst])
              } else {
                regest <- lm(catest[icat, tst] ~ years[tst],
                  weights = 1 / varest[icat, tst]
                )
              }
              coeff <- summary(regest)$coefficients
              cint <- confint(regest, level = conf / 100)
              trend <- coeff[, "Estimate"][2]
              t_stderror <- coeff[, "Std. Error"][2]
              t_lcb <- cint["years[tst]", ][1]
              t_ucb <- cint["years[tst]", ][2]
              t_pval <- coeff[, "Pr(>|t|)"][2]
              intercept <- coeff[, "Estimate"][1]
              i_stderror <- coeff[, "Std. Error"][1]
              i_lcb <- cint["(Intercept)", ][1]
              i_ucb <- cint["(Intercept)", ][2]
              i_pval <- coeff[, "Pr(>|t|)"][1]
              rsq <- summary(regest)$r.squared
              adjrsq <- summary(regest)$adj.r.squared

              # Assign estimates for the response variable to a data frame

              trendsum$catsum <- rbind(trendsum$catsum, data.frame(
                itype, isubpop, ivar, lev_ivar[icat], trend, t_stderror, t_lcb,
                t_ucb, t_pval, intercept, i_stderror, i_lcb, i_ucb, i_pval, rsq,
                adjrsq
              ))
            }

            # Section for a generalized linear mixed-effects model
          } else if (invprboot == TRUE) {

            # Fit the model

            if (stratum_ind) {
              eval(parse(text = paste0(
                "regest <- glmer(", ivar, " ~ ",
                cat_rhs, " + ", stratumID, ", data = dframe",
                ", family = binomial",
                ", control = glmerControl(check.nobs.vs.nRE = 'warning'",
                ", check.conv.singular = lme4::.makeCC(action = 'warning',",
                " tol = formals(lme4::isSingular)$tol)))"
              )))
            } else {
              eval(parse(text = paste0(
                "regest <- glmer(", ivar, " ~ ",
                cat_rhs, ", data = dframe, family = binomial",
                ", control = glmerControl(check.nobs.vs.nRE = 'warning'",
                ", check.conv.singular = lme4::.makeCC(action = 'warning',",
                " tol = formals(lme4::isSingular)$tol)))"
              )))
            }

            # Create objects that are required for assigning names to the output
            # data frame

            coeff <- summary(regest)$coefficients
            cint <- confint(regest,
              parm = "beta_", level = conf / 100,
              method = "Wald"
            )
            pval <- coeff[, "Pr(>|z|)"]
            fixed <- matrix(0, 5, nrow(coeff))
            fixed[1, ] <- coeff[, "Estimate"]
            fixed[2, ] <- coeff[, "Std. Error"]
            fixed[3, ] <- cint[, 1]
            fixed[4, ] <- cint[, 2]
            fixed[5, ] <- pval
            vcor <- as.data.frame(VarCorr(regest))
            varcomp <- vector("numeric", nrow(vcor) + sum(!is.na(vcor$vars)))
            for (i in 1:length(varcomp)) {
              if (is.na(vcor$var2[i])) {
                varcomp[i] <- vcor$vcov[i]
              } else {
                varcomp[i] <- vcor$sdcor[i]
              }
            }
            varcomp <- c(varcomp, 1.0)

            # Call the bootstrap function

            if (stratum_ind) {
              bootest <- boot(dframe, bootfcn, nboot,
                strata = dframe[, stratumID], weights = bootwgt,
                model = model_cat, ivar = ivar, rhs = cat_rhs, conf = conf
              )
            } else {
              bootest <- boot(dframe, bootfcn, nboot,
                weights = bootwgt,
                model = model_cat, ivar = ivar, rhs = cat_rhs, conf = conf
              )
            }

            # Calculate bootstrap estimates

            estimates <- apply(bootest$t, 2, mean)

            # Assign estimates for the response variable to a data frame

            trendsum$catsum <- rbind(trendsum$catsum, data.frame(t(c(
              itype, isubpop, ivar, estimates
            ))))
          } else {

            # Fit the model

            if (stratum_ind) {
              eval(parse(text = paste0(
                "regest <- glmer(", ivar, " ~ ",
                cat_rhs, " + ", stratumID, ", data = dframe",
                ", family = binomial",
                ", control = glmerControl(check.nobs.vs.nRE = 'warning'",
                ", check.conv.singular = lme4::.makeCC(action = 'warning',",
                " tol = formals(lme4::isSingular)$tol)))"
              )))
            } else {
              eval(parse(text = paste0(
                "regest <- glmer(", ivar, " ~ ",
                cat_rhs, ", data = dframe, family = binomial",
                ", control = glmerControl(check.nobs.vs.nRE = 'warning'",
                ", check.conv.singular = lme4::.makeCC(action = 'warning',",
                " tol = formals(lme4::isSingular)$tol)))"
              )))
            }

            # Calculate estimates

            coeff <- summary(regest)$coefficients
            cint <- confint(regest,
              parm = "beta_", level = conf / 100,
              method = "Wald"
            )
            pval <- coeff[, "Pr(>|z|)"]
            fixed <- matrix(0, 5, nrow(coeff))
            fixed[1, ] <- coeff[, "Estimate"]
            fixed[2, ] <- coeff[, "Std. Error"]
            fixed[3, ] <- cint[, 1]
            fixed[4, ] <- cint[, 2]
            fixed[5, ] <- pval
            vcor <- as.data.frame(VarCorr(regest))
            varcomp <- vector("numeric", nrow(vcor) + sum(!is.na(vcor$vars)))
            for (i in 1:length(varcomp)) {
              if (is.na(vcor$var2[i])) {
                varcomp[i] <- vcor$vcov[i]
              } else {
                varcomp[i] <- vcor$sdcor[i]
              }
            }
            varcomp <- c(varcomp, 1.0)
            AIC <- extractAIC(regest)[2]
            if (attr(terms(regest), "intercept") == 1) {
              if (ncol(fixed) > 2) {
                estimates <- c(
                  fixed[, 2],
                  as.vector(fixed[, c(1, 3:ncol(fixed))]), varcomp, AIC
                )
              } else {
                estimates <- c(fixed[, 2], fixed[, 1], varcomp, AIC)
              }
            } else {
              estimates <- c(as.vector(fixed), varcomp, AIC)
            }

            # Assign estimates for the response variable to a data frame

            trendsum$catsum <- rbind(trendsum$catsum, data.frame(t(c(
              itype, isubpop, ivar, estimates
            ))))
          }

          # End of the loop for levels of a subpopulation
        }

        # End of the loop for response variables
      }

      # End of the loop for subpopulations
    }

    # End of the section for categorical response variables
  }

  #
  # Begin the section for continuous response variables
  #

  if (!is.null(vars_cont)) {

    # Assign values to the site ID variable in the dframe data frame

    if (model_cont %in% c("SLR", "WLR")) {
      dframe[, siteID] <- dframe$siteID_unique
    } else {
      dframe[, siteID] <- dframe$siteID_orginal
    }

    # Loop through all subpopulations (domains)

    for (itype in subpops) {
      lev_itype <- levels(dframe[, itype])
      nlev_itype <- length(lev_itype)

      # Loop through all  response variables

      for (ivar in vars_cont) {

        # Loop through all levels of a subpopulation

        for (isubpop in lev_itype) {

          # Section for the simple linear regression and weighted linear
          # regression models

          if (model_cont %in% c("SLR", "WLR")) {

            # Determine the set of years for this subpopulation

            subpop_ind <- dframe[, itype] %in% isubpop
            years <- sort(unique(dframe[subpop_ind, Wyear]))
            nyears <- length(years)

            # Create vectors to contain mean estimates and variance estimates
            # for each time period

            contest <- rep(NA, nyears)
            varest <- rep(NA, nyears)

            # Loop through all time periods for this subpopulation

            for (iyear in 1:nyears) {

              # Select sites in a year for this subpopulation

              subpop_ind <- dframe[, itype] %in% isubpop &
                dframe$Wyear %in% years[iyear]

              # Determine whether the time period for this subpopulation is
              # empty

              if (all(is.na(dframe[subpop_ind, ivar]))) {
                warn_ind <- TRUE
                warn <- paste0("Year ", years[iyear], " of Subpopulation \"", isubpop, "\" of population type \"", itype, "\" for indicator\n \"", ivar, "\" contains no data.\n")
                act <- "None.\n"
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = I(itype), subpop = I(isubpop),
                  indicator = I(ivar), stratum = NA, warning = I(warn), action = I(act)
                ))
                next
              }

              # Determine whether the subpopulation contains a single value

              tst <- !is.na(dframe[subpop_ind, ivar])
              if (sum(tst) == 1) {
                warn_ind <- TRUE
                warn <- paste0("Year ", years[iyear], " of Subpopulation \"", isubpop, "\" of population type \"", itype, "\" for indicator\n \"", ivar, "\" contains a single value.\n")
                act <- "None.\n"
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = I(itype), subpop = I(isubpop),
                  indicator = I(ivar), stratum = NA, warning = I(warn), action = I(act)
                ))
                next
              }

              # Estimate the mean for the response variable

              temp <- mean_est(
                NULL, droplevels(subset(dframe, subpop_ind)),
                itype, isubpop, 1, ivar, subset(design, subpop_ind),
                design_names, NULL, vartype, conf, mult, warn_ind, warn_df
              )
              temp_cont <- temp$meansum
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df

              # Assign the mean estimate and variance estimate

              contest[iyear] <- temp_cont$Estimate
              varest[iyear] <- (temp_cont$StdError)^2

              # End of the loop for years
            }

            # Perform linear regression and assign results

            if (model_cont == "SLR") {
              regest <- lm(contest ~ years)
            } else {
              regest <- lm(contest ~ years, weights = 1 / varest)
            }
            coeff <- summary(regest)$coefficients
            cint <- confint(regest, level = conf / 100)
            trend <- coeff[, "Estimate"][2]
            t_stderror <- coeff[, "Std. Error"][2]
            t_lcb <- cint["years", ][1]
            t_ucb <- cint["years", ][2]
            t_pval <- coeff[, "Pr(>|t|)"][2]
            intercept <- coeff[, "Estimate"][1]
            i_stderror <- coeff[, "Std. Error"][1]
            i_lcb <- cint["(Intercept)", ][1]
            i_ucb <- cint["(Intercept)", ][2]
            i_pval <- coeff[, "Pr(>|t|)"][1]
            rsq <- summary(regest)$r.squared
            adjrsq <- summary(regest)$adj.r.squared

            # Assign estimates for the response variable to a data frame

            trendsum$contsum <- rbind(
              trendsum$contsum,
              data.frame(
                itype, isubpop, ivar, trend, t_stderror, t_lcb, t_ucb,
                t_pval, intercept, i_stderror, i_lcb, i_ucb, i_pval, rsq, adjrsq
              )
            )

            # Section for a linear mixed-effects model
          } else if (invprboot == TRUE) {

            # Fit the model

            if (stratum_ind) {
              eval(parse(text = paste0(
                "regest <- lmer(", ivar, " ~ ",
                cont_rhs, " + ", stratumID, ", data = dframe",
                ", control = lmerControl(check.nobs.vs.nRE = 'warning'",
                ", check.conv.singular = lme4::.makeCC(action = 'warning',",
                " tol = formals(lme4::isSingular)$tol)))"
              )))
            } else {
              eval(parse(text = paste0(
                "regest <- lmer(", ivar, " ~ ",
                cont_rhs, ", data = dframe",
                ", control = lmerControl(check.nobs.vs.nRE = 'warning'",
                ", check.conv.singular = lme4::.makeCC(action = 'warning',",
                " tol = formals(lme4::isSingular)$tol)))"
              )))
            }

            # Create objects that are required for assigning names to the output
            # data frame

            coeff <- summary(regest)$coefficients
            cint <- confint(regest,
              parm = "beta_", level = conf / 100,
              method = "Wald"
            )
            dfval <- df.residual(regest)
            tvalue <- coeff[, "t value"]
            pval <- 2 * (1 - pt(abs(tvalue), dfval))
            fixed <- matrix(0, 5, nrow(coeff))
            fixed[1, ] <- coeff[, "Estimate"]
            fixed[2, ] <- coeff[, "Std. Error"]
            fixed[3, ] <- cint[, 1]
            fixed[4, ] <- cint[, 2]
            fixed[5, ] <- pval
            vcor <- as.data.frame(VarCorr(regest))
            varcomp <- vector("numeric", nrow(vcor))
            for (i in 1:length(varcomp)) {
              if (is.na(vcor$var2[i])) {
                varcomp[i] <- vcor$vcov[i]
              } else {
                varcomp[i] <- vcor$sdcor[i]
              }
            }

            # Call the bootstrap function

            if (stratum_ind) {
              bootest <- boot(dframe, bootfcn, nboot,
                strata = dframe[, stratumID], weights = bootwgt,
                model = model_cont, ivar = ivar, rhs = cont_rhs, conf = conf
              )
            } else {
              bootest <- boot(dframe, bootfcn, nboot,
                weights = bootwgt,
                model = model_cont, ivar = ivar, rhs = cont_rhs, conf = conf
              )
            }

            # Calculate bootstrap estimates

            estimates <- apply(bootest$t, 2, mean)

            # Assign estimates for the response variable to a data frame

            trendsum$contsum <- rbind(
              trendsum$contsum,
              data.frame(t(c(
                itype, isubpop, ivar, estimates
              )))
            )
          } else {

            # Fit the model

            if (stratum_ind) {
              eval(parse(text = paste0(
                "regest <- lmer(", ivar, " ~ ",
                cont_rhs, " + ", stratumID, ", data = dframe",
                ", control = lmerControl(check.nobs.vs.nRE = 'warning'",
                ", check.conv.singular = lme4::.makeCC(action = 'warning',",
                " tol = formals(lme4::isSingular)$tol)))"
              )))
            } else {
              eval(parse(text = paste0(
                "regest <- lmer(", ivar, " ~ ",
                cont_rhs, ", data = dframe",
                ", control = lmerControl(check.nobs.vs.nRE = 'warning'",
                ", check.conv.singular = lme4::.makeCC(action = 'warning',",
                " tol = formals(lme4::isSingular)$tol)))"
              )))
            }

            # Calculate estimates

            coeff <- summary(regest)$coefficients
            cint <- confint(regest,
              parm = "beta_", level = conf / 100,
              method = "Wald"
            )
            dfval <- df.residual(regest)
            tvalue <- coeff[, "t value"]
            pval <- 2 * (1 - pt(abs(tvalue), dfval))
            fixed <- matrix(0, 5, nrow(coeff))
            fixed[1, ] <- coeff[, "Estimate"]
            fixed[2, ] <- coeff[, "Std. Error"]
            fixed[3, ] <- cint[, 1]
            fixed[4, ] <- cint[, 2]
            fixed[5, ] <- pval
            vcor <- as.data.frame(VarCorr(regest))
            varcomp <- vector("numeric", nrow(vcor))
            for (i in 1:length(varcomp)) {
              if (is.na(vcor$var2[i])) {
                varcomp[i] <- vcor$vcov[i]
              } else {
                varcomp[i] <- vcor$sdcor[i]
              }
            }
            AIC <- extractAIC(regest)[2]
            if (attr(terms(regest), "intercept") == 1) {
              if (ncol(fixed) > 2) {
                estimates <- c(
                  fixed[, 2],
                  as.vector(fixed[, c(1, 3:ncol(fixed))]), varcomp, AIC
                )
              } else {
                estimates <- c(fixed[, 2], fixed[, 1], varcomp, AIC)
              }
            } else {
              estimates <- c(as.vector(fixed), varcomp, AIC)
            }

            # Assign estimates for the response variable to a data frame

            trendsum$contsum <- rbind(
              trendsum$contsum,
              data.frame(t(c(
                itype, isubpop, ivar, estimates
              )))
            )
          }

          # End of the loop for levels of a subpopulation
        }

        # End of the loop for response variables
      }

      # End of the loop for subpopulations
    }

    # End of the section for continuous response variables
  }

  # Assign row names and column names to the output data frames

  if (!is.null(trendsum$catsum)) {
    nrows <- nrow(trendsum$catsum)
    if (model_cat %in% c("SLR", "WLR")) {
      dimnames(trendsum$catsum) <- list(1:nrows, c(
        "Type", "Subpopulation",
        "Indicator", "Category", "Trend_Estimate", "Trend_Std_Error",
        paste0("Trend_LCB", conf, "Pct"), paste0("Trend_UCB", conf, "Pct"),
        "Trend_p_Value", "Intercept_Estimate", "Intercept_Std_Error",
        paste0("Intercept_LCB", conf, "Pct"), paste0(
          "Intercept_UCB", conf,
          "Pct"
        ), "Intercept_p_Value", "R_Squared", "Adj_R_Squared"
      ))
    } else {
      if (PO_ind_cat == TRUE) {
        dimnames(trendsum$catsum) <- list(1:nrows, c(
          "Type", "Subpopulation",
          "Indicator", "Trend_Estimate", "Trend_Std_Error", paste0(
            "Trend_LCB",
            conf, "Pct"
          ), paste0("Trend_UCB", conf, "Pct"), "Trend_p_Value",
          "Intercept_Estimate", "Intercept_Std_Error", paste0(
            "Intercept_LCB",
            conf, "Pct"
          ), paste0("Intercept_UCB", conf, "Pct"),
          "Intercept_p_Value", "Var_SiteID_Int", "Var_SiteID_Trend",
          "Corr_SiteID_Int_Trend", "Var_YearID", "Var_Residual", "AIC"
        ))
      } else if (attr(terms(regest), "intercept") == 1) {
        fixed_names <- NULL
        if (ncol(fixed) > 2) {
          fnames <- rownames(coeff)
          for (i in 3:ncol(fixed)) {
            temp <- fnames[i]
            fixed_names <- c(
              fixed_names, paste0(temp, "_Estimate"),
              paste0(temp, "_Std_Error"), paste0(temp, "_LCB", conf, "Pct"),
              paste0(temp, "_UCB", conf, "Pct"), paste0(temp, "_p_Value")
            )
          }
        }
        varcomp_names <- NULL
        vnames <- vcor$grp
        for (i in 1:nrow(vcor)) {
          if (is.na(vcor$var2[i])) {
            if (vcor$var1[i] %in% "(Intercept)") {
              varcomp_names <- c(varcomp_names, paste("Var", vnames[i],
                "Int",
                sep = "_"
              ))
            } else {
              varcomp_names <- c(varcomp_names, paste("Var", vnames[i],
                vcor$var1[i],
                sep = "_"
              ))
            }
          } else {
            if (vcor$var1[i] %in% "(Intercept)") {
              varcomp_names <- c(
                varcomp_names,
                paste("Corr", vnames[i], "Int", vcor$var2[i], sep = "_")
              )
            } else {
              varcomp_names <- c(varcomp_names, paste("Corr", vnames[1],
                vcor$var1[i], vcor$var2[i],
                sep = "_"
              ))
            }
          }
        }
        varcomp_names <- gsub("Wyear", "Trend", varcomp_names)
        dimnames(trendsum$catsum) <- list(1:nrows, c(
          "Type", "Subpopulation",
          "Indicator", "Trend_Estimate", "Trend_Std_Error",
          paste0("Trend_LCB", conf, "Pct"),
          paste0("Trend_UCB", conf, "Pct"), "Trend_p_Value",
          "Intercept_Estimate", "Intercept_Std_Error",
          paste0("Intercept_LCB", conf, "Pct"),
          paste0("Intercept_UCB", conf, "Pct"), "Intercept_p_Value",
          fixed_names, varcomp_names, "Var_Residual", "AIC"
        ))
      } else {
        fixed_names <- NULL
        if (ncol(fixed) > 1) {
          fnames <- rownames(coeff)
          for (i in 2:ncol(fixed)) {
            temp <- fnames[i]
            fixed_names <- c(
              fixed_names, paste0(temp, "_Estimate"),
              paste0(temp, "_Std_Error"), paste0(temp, "_LCB", conf, "Pct"),
              paste0(temp, "_UCB", conf, "Pct"), paste0(temp, "_p_Value")
            )
          }
        }
        varcomp_names <- NULL
        vnames <- vcor$grp
        for (i in 1:nrow(vcor)) {
          if (is.na(vcor$var2[i])) {
            if (vcor$var1[i] %in% "(Intercept)") {
              varcomp_names <- c(varcomp_names, paste("Var", vnames[i],
                "Int",
                sep = "_"
              ))
            } else {
              varcomp_names <- c(varcomp_names, paste("Var", vnames[i],
                vcor$var1[i],
                sep = "_"
              ))
            }
          } else {
            if (vcor$var1[i] %in% "(Intercept)") {
              varcomp_names <- c(
                varcomp_names,
                paste("Corr", vnames[i], "Int", vcor$var2[i], sep = "_")
              )
            } else {
              varcomp_names <- c(varcomp_names, paste("Corr", vnames[1],
                vcor$var1[i], vcor$var2[i],
                sep = "_"
              ))
            }
          }
        }
        varcomp_names <- gsub("Wyear", "Trend", varcomp_names)
        dimnames(trendsum$catsum) <- list(1:nrows, c(
          "Type", "Subpopulation",
          "Indicator", "Trend_Estimate", "Trend_Std_Error",
          paste0("Trend_LCB", conf, "Pct"),
          paste0("Trend_UCB", conf, "Pct"), "Trend_p_Value",
          fixed_names, varcomp_names, "Var_Residual", "AIC"
        ))
      }
    }
  }

  if (!is.null(trendsum$contsum)) {
    nrows <- nrow(trendsum$contsum)
    if (model_cont %in% c("SLR", "WLR")) {
      dimnames(trendsum$contsum) <- list(1:nrows, c(
        "Type", "Subpopulation",
        "Indicator", "Trend_Estimate", "Trend_Std_Error", paste0(
          "Trend_LCB",
          conf, "Pct"
        ), paste0("Trend_UCB", conf, "Pct"), "Trend_p_Value",
        "Intercept_Estimate", "Intercept_Std_Error", paste0(
          "Intercept_LCB",
          conf, "Pct"
        ), paste0("Intercept_UCB", conf, "Pct"),
        "Intercept_p_Value", "R_Squared", "Adj_R_Squared"
      ))
    } else {
      if (PO_ind_cont == TRUE) {
        dimnames(trendsum$contsum) <- list(1:nrows, c(
          "Type", "Subpopulation",
          "Indicator", "Trend_Estimate", "Trend_Std_Error", paste0(
            "Trend_LCB",
            conf, "Pct"
          ), paste0("Trend_UCB", conf, "Pct"), "Trend_p_Value",
          "Intercept_Estimate", "Intercept_Std_Error", paste0(
            "Intercept_LCB",
            conf, "Pct"
          ), paste0("Intercept_UCB", conf, "Pct"),
          "Intercept_p_Value", "Var_SiteID_Int", "Var_SiteID_Trend",
          "Corr_SiteID_Int_Trend", "Var_YearID", "Var_Residual", "AIC"
        ))
      } else if (attr(terms(regest), "intercept") == 1) {
        fixed_names <- NULL
        if (ncol(fixed) > 2) {
          fnames <- rownames(coeff)
          for (i in 3:ncol(fixed)) {
            temp <- fnames[i]
            fixed_names <- c(
              fixed_names, paste0(temp, "_Estimate"),
              paste0(temp, "_Std_Error"), paste0(temp, "_LCB", conf, "Pct"),
              paste0(temp, "_UCB", conf, "Pct"), paste0(temp, "_p_Value")
            )
          }
        }
        varcomp_names <- NULL
        vnames <- vcor$grp
        for (i in 1:(nrow(vcor) - 1)) {
          if (is.na(vcor$var2[i])) {
            if (vcor$var1[i] %in% "(Intercept)") {
              varcomp_names <- c(varcomp_names, paste("Var", vnames[i],
                "Int",
                sep = "_"
              ))
            } else {
              varcomp_names <- c(varcomp_names, paste("Var", vnames[i],
                vcor$var1[i],
                sep = "_"
              ))
            }
          } else {
            if (vcor$var1[i] %in% "(Intercept)") {
              varcomp_names <- c(
                varcomp_names,
                paste("Corr", vnames[i], "Int", vcor$var2[i], sep = "_")
              )
            } else {
              varcomp_names <- c(varcomp_names, paste("Corr", vnames[1],
                vcor$var1[i], vcor$var2[i],
                sep = "_"
              ))
            }
          }
        }
        varcomp_names <- gsub("Wyear", "Trend", varcomp_names)
        dimnames(trendsum$contsum) <- list(1:nrows, c(
          "Type", "Subpopulation",
          "Indicator", "Trend_Estimate", "Trend_Std_Error",
          paste0("Trend_LCB", conf, "Pct"),
          paste0("Trend_UCB", conf, "Pct"), "Trend_p_Value",
          "Intercept_Estimate", "Intercept_Std_Error",
          paste0("Intercept_LCB", conf, "Pct"),
          paste0("Intercept_UCB", conf, "Pct"), "Intercept_p_Value",
          fixed_names, varcomp_names, "Var_Residual", "AIC"
        ))
      } else {
        fixed_names <- NULL
        if (ncol(fixed) > 1) {
          fnames <- rownames(coeff)
          for (i in 2:ncol(fixed)) {
            temp <- fnames[i]
            fixed_names <- c(
              fixed_names, paste0(temp, "_Estimate"),
              paste0(temp, "_Std_Error"), paste0(temp, "_LCB", conf, "Pct"),
              paste0(temp, "_UCB", conf, "Pct"), paste0(temp, "_p_Value")
            )
          }
        }
        varcomp_names <- NULL
        vnames <- vcor$grp
        for (i in 1:(nrow(vcor) - 1)) {
          if (is.na(vcor$var2[i])) {
            if (vcor$var1[i] %in% "(Intercept)") {
              varcomp_names <- c(varcomp_names, paste("Var", vnames[i],
                "Int",
                sep = "_"
              ))
            } else {
              varcomp_names <- c(varcomp_names, paste("Var", vnames[i],
                vcor$var1[i],
                sep = "_"
              ))
            }
          } else {
            if (vcor$var1[i] %in% "(Intercept)") {
              varcomp_names <- c(
                varcomp_names,
                paste("Corr", vnames[i], "Int", vcor$var2[i], sep = "_")
              )
            } else {
              varcomp_names <- c(varcomp_names, paste("Corr", vnames[1],
                vcor$var1[i], vcor$var2[i],
                sep = "_"
              ))
            }
          }
        }
        varcomp_names <- gsub("Wyear", "Trend", varcomp_names)
        dimnames(trendsum$contsum) <- list(1:nrows, c(
          "Type", "Subpopulation",
          "Indicator", "Trend_Estimate", "Trend_Std_Error",
          paste0("Trend_LCB", conf, "Pct"),
          paste0("Trend_UCB", conf, "Pct"), "Trend_p_Value",
          fixed_names, varcomp_names, "Var_Residual", "AIC"
        ))
      }
    }
  }

  # As necessary, output a message indicating that warning messages were
  # generated during execution of the program

  if (warn_ind) {
    warn_df <<- warn_df
    if (nrow(warn_df) == 1) {
      message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
    } else {
      message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
    }
  }

  # Return the output object

  trendsum
}

context("trend analysis")

#
# Section for data input
#

# Read the data frame containing survey design and analysis variables
load(system.file("extdata", "NLA_IN.rda", package = "spsurvey"))

# Create a population size data frame
popsize <- data.frame(
  LAKE_ORGN = c("MAN_MADE", "NATURAL"),
  Total = c(6000, 14000)
)

# Create finite population correction factor objects
fpc1 <- 20000
fpc2a <- list(
  Urban = 5000,
  "Non-Urban" = 15000
)
fpc2b <- list(
  MAN_MADE = 6000,
  NATURAL = 14000
)
fpc3 <- c(
  Ncluster = 200,
  clusterID_1 = 100,
  clusterID_2 = 100,
  clusterID_3 = 100,
  clusterID_4 = 100
)
fpc4a <- list(
  Urban = c(
    Ncluster = 75,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  "Non-Urban" = c(
    Ncluster = 125,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)
fpc4b <- list(
  NATURAL = c(
    Ncluster = 130,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  MAN_MADE = c(
    Ncluster = 70,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)

#
# Section for the trend data analysis function
#

# Create a year variable for trend estimation
NLA_IN$year <- NLA_IN$YEAR - 2007

# Assign response variable names to the vars_cat and vars_cont vectors
vars_cat <- c("BENT_MMI_COND_2017")
vars_cont <- c("ContVar")

# Assign subpopulation variable names to the subpops vector
subpops <- c("All_Sites", "LAKE_ORGN")

# Perform tests
Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD"
)

test_that("Trend: Unstratified single-stage analysis", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  popsize = popsize
)

test_that("Trend: with known population sizes", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  fpc = fpc1
)

test_that("Trend: with finite population correction factor", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "LAKE_ORGN"
)

test_that("Trend: Stratified single-stage analysis", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "LAKE_ORGN", fpc = fpc2b
)

test_that("Trend: with finite population correction factor", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  clusterID = "clusterID", weight1 = "weight1", xcoord1 = "xcoord1",
  ycoord1 = "ycoord1", vartype = "SRS"
)

test_that("Trend: Unstratified two-stage analysis", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  clusterID = "clusterID", weight1 = "weight1", xcoord1 = "xcoord1",
  ycoord1 = "ycoord1", popsize = popsize, vartype = "SRS"
)

test_that("Trend: with known population sizes", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  clusterID = "clusterID", weight1 = "weight1", xcoord1 = "xcoord1",
  ycoord1 = "ycoord1", fpc = fpc3, vartype = "SRS"
)

test_that("Trend: with finite population correction factor", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "LAKE_ORGN", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", vartype = "SRS"
)

test_that("Trend: Stratified two-stage analysis", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "LAKE_ORGN", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", popsize = popsize, vartype = "SRS"
)

test_that("Trend: with known population sizes", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

Trend_Estimates <- trend_analysis(
  dframe = NLA_IN, vars_cat = vars_cat,
  vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
  siteID = "UNIQUE_ID", weight = "WGT_TP", xcoord = "XCOORD", ycoord = "YCOORD",
  stratumID = "LAKE_ORGN", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", fpc = fpc4b, vartype = "SRS"
)

test_that("Trend: with finite population correction factor", {
  expect_true(exists("Trend_Estimates"))
  expect_equal(attributes(Trend_Estimates$catsum)$class, "data.frame")
  expect_equal(attributes(Trend_Estimates$contsum)$class, "data.frame")
  expect_equal(nrow(Trend_Estimates$catsum), 6)
  expect_equal(nrow(Trend_Estimates$contsum), 3)
})

test_that("A warning (in message form) is produced", {
  expect_message(expect_error(trend_analysis(
    dframe = NLA_IN, vars_cat = vars_cat,
    vars_cont = vars_cont, model_cont = "SLR", subpops = subpops,
    siteID = "UNIQUE_ID", weight = "XYZ", xcoord = "XCOORD", ycoord = "YCOORD",
    stratumID = "LAKE_ORGN", clusterID = "clusterID", weight1 = "weight1",
    xcoord1 = "xcoord1", ycoord1 = "ycoord1", fpc = fpc4b, vartype = "SRS"
  )))
})

###############################################################################
# Function: replace_near.R (not exported)
# Programmer:  Michael Dumelle and Tony Olsen
# Date: March 2, 2021
#
#' Find Nearest Neighbor Replacement Sites
#'
#' @param n_near number of nearby sites to be used as potential replacement(s)
#'    if a site cannot be sampled for any reason. Must be integer from 1 to 10.
#
#' Replacement sites generated via nearest neighbors
#'
#' Returns 1 to 10 nearest neighbor replacement sites for each site in the
#'   base sample
#'
#' @param n_near number of nearby sites to be used as potential replacement(s)
#'    if a site cannot be sampled for any reason. Must be integer from \code{1} to \code{10}.
#'    Default is \code{NULL}.
#'
#' @param sites sf point object of selected sites that require replacement sites
#'
#' @param sframe sf point object that was used as sample frame to select sites.
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#' @keywords survey design
#'
#' @noRd
################################################################################
replace_near <- function(n_near, sites, sframe) {

  # remove sampled sites from sframe
  sframe <- sframe[!(sframe$idpts %in% sites$idpts), , drop = FALSE]

  # calculate distance between points
  site_dist <- st_distance(sites, sframe)

  # set  possible levels for siteuse
  names_siteuse <- c(
    "Near-1st", "Near-2nd", "Near-3rd", "Near-4th", "Near-5th",
    "Near-6th", "Near-7th", "Near-8th", "Near-9th", "Near-10th"
  )

  # split
  site_dist_list <- split(site_dist, 1:nrow(site_dist))
  # apply
  sites_near <- mapply(function(x, y) {
    sites_tmp_ind <- sframe$idpts[order(x)][seq_len(n_near)]
    sites_tmp <- sframe[sframe$idpts %in% sites_tmp_ind, , drop = FALSE]
    sites_tmp$siteuse <- names_siteuse[seq_len(nrow(sites_tmp))] # covers cases where there are less than n_near sites available
    sites_tmp$replsite <- y
    sites_tmp
  },
  x = site_dist_list,
  y = sites$idpts,
  SIMPLIFY = FALSE
  )

  # combine
  sites_near <- do.call("rbind", sites_near)
}

###############################################################################
# Function: plot (exported)
# Programmers: Michael Dumelle
# Date: January 22, 2021
#' Plot sampling frames, design sites, and analysis data.
#'
#' This function plots sampling frames, design sites, and analysis data.
#' If the left-hand side of the formula is empty, plots
#' are of the distributions of the right-hand side variables. If the left-hand side
#' of the variable contains a variable, plots are of the left-hand size variable
#' for each level of each right-hand side variable.
#' This function is largely built on \code{plot.sf()}, and all spsurvey plotting
#' methods can supply additional arguments to \code{plot.sf()}. For more information on
#' plotting in \code{sf}, run \code{?sf::plot.sf()}. Equivalent to \code{sp_plot()}; both
#' are currently maintained for backwards compatibility.
#'
#' @param x An object to plot. When plotting sampling frames an \code{sf} object
#' given the appropriate class using \code{sp_frame}.
#'  When plotting design sites, an object created by \code{grts()} or
#' \code{irs()} (which has class \code{sp_design}). When plotting analysis data, a data frame
#' or an \code{sf} object given the appropriate class using \code{sp_frame}.
#'
#' @param sframe The sampling frame (an \code{sf} object) to plot alongside design
#' sites. This argument is only used when \code{object} corresponds to the design sites.
#'
#' @param formula A formula. One-sided formulas are used to summarize the
#' distribution of numeric or categorical variables. For one-sided formulas,
#' variable names are placed to the right of \code{~} (a right-hand side variable).
#' Two sided formulas are
#' used to summarize the distribution of a left-hand side variable
#' for each level of each right-hand side categorical variable in the formula.
#' Note that only for two-sided formulas are numeric right-hand side variables
#' coerced to a categorical variables. If an intercept
#' is included as a right-hand side variable (whether the formula is one-sided or
#' two-sided), the total will also be summarized. When plotting sampling frames
#' or analysis data, the default formula is \code{~ 1}. When plotting design sites,
#' \code{siteuse} should be used in the formula, and the default formula is
#' \code{~ siteuse}.
#'
#' @param siteuse A character vector of site types to include when plotting design sites.
#' It can only take on values \code{"sframe"} (sampling frame),
#' \code{"Legacy"} (for legacy sites), \code{"Base"} (for base sites),
#' \code{"Over"} (for \code{n_over} replacement sites), and \code{"Near"}
#' (for \code{n_near} replacement sites). The order of sites represents the
#' layering in the plot (e.g. \code{siteuse = c("Base", "Legacy")} will plot
#' legacy sites on top of base sites. Defaults to all non-\code{NULL} elements
#' in \code{x} and \code{y} with plot order \code{"sframe"}, \code{"Legacy"},
#' \code{"Base"}, \code{"Over"}, \code{"Near"}.
#'
#' @param var_args A named list. The name of each list element corresponds to a
#' right-hand side variable in \code{formula}. Values in the list are composed of
#' graphical arguments that are to be passed to \strong{every} level of the
#' variable. To see all graphical arguments available, run \code{?plot.sf}.
#'
#' @param varlevel_args A named list. The name of each list element corresponds to a
#' right-hand side variable in \code{formula}. The first element in this list
#' should be \code{"levels"} and contain all levels of the particular right-hand side variable. Subsequent
#' names correspond to graphical arguments that are to be passed to
#' the specified levels (in order) of the right-hand side variable. Values for each
#' graphical argument must be specified for each level of the right-hand side variable,
#' but applicable sf defaults will be matched by inputting the value \code{NA}.
#' To see all graphical arguments available, run \code{?plot.sf}
#'
#' @param geom Should separate geometries for each level of the right-hand
#' side \code{formula} variables be plotted? Defaults to \code{FALSE}.
#'
#' @param onlyshow A string indicating the single level of the single right-hand side
#' variable for which a summary is requested. This argument is only used when
#' a single right-hand side variable is provided.
#'
#' @param fix_bbox Should the geometry bounding box be fixed across plots?
#' If a length-four vector with names "xmin", "ymin", "xmax", and "ymax" and values
#' indicating bounding box edges, the bounding box will be fixed as \code{fix_bbox}
#' across plots. If \code{TRUE}, the bounding box will be fixed across plots as the
#' bounding box of \code{object}. If \code{FALSE}, the bounding box will vary across
#' plots according to the unique geometry for each plot. Defaults to \code{TRUE}.
#'
#' @param xcoord Name of the x-coordinate (east-west) in \code{object} (only required if
#' \code{object} is not an \code{sf} object).
#'
#' @param ycoord Name of y (north-south)-coordinate in \code{object} (only required if
#' \code{object} is not an \code{sf} object).
#'
#' @param crs Projection code for \code{xcoord} and \code{ycoord} (only
#' required if \code{object} is not an \code{sf} object).
#'
#' @param ... Additional arguments to pass to \code{plot.sf()}.
#'
#' @author Michael Dumelle \email{Dumelle.Michael@@epa.gov}
#'
#' @name plot
#' @method plot sp_frame
#'
#' @export
#'
#' @examples
#' \dontrun{
#' data("NE_Lakes")
#' NE_Lakes <- sp_frame(NE_Lakes)
#' plot(NE_Lakes, formula = ~ELEV_CAT)
#' sample <- grts(NE_Lakes, 30)
#' plot(sample, NE_Lakes)
#' }
plot.sp_frame <- function(x, formula = ~1, xcoord, ycoord, crs,
                          var_args = NULL, varlevel_args = NULL,
                          geom = FALSE, onlyshow = NULL, fix_bbox = TRUE, ...) {

  x <- sp_unframe(x)
  sp_plot(x, formula, xcoord, ycoord, crs,
          var_args, varlevel_args,
          geom, onlyshow, fix_bbox, ...)
  
}

#' @name plot
#' @method plot sp_design
#' @export
plot.sp_design <- function(x, sframe = NULL, formula = ~siteuse, siteuse = NULL,
                           var_args = NULL, varlevel_args = NULL, geom = FALSE, onlyshow = NULL,
                           fix_bbox = TRUE, ...) {
  
  sp_plot.sp_design(x, sframe, formula, siteuse,
                    var_args, varlevel_args, geom, onlyshow,
                    fix_bbox, ...)
}

################################################################################
# Function: plot.sp_CDF (exported)
# Programmer Tom Kincaid
# Date: May 5, 2021
# Revised: August 31, 2021 to improve documentation
#'
#' Plot a cumulative distribution function (CDF)
#'
#' This function creates a CDF plot.  Input data for the plots is provided by a
#' data frame from the "CDF" output given by  \code{cont_analysis}.
#' Confidence limits for the CDF also are plotted. Equivalent to \code{cdf_plot()}; 
#' both are currently maintained for backwards compatibility.
#' 
#' @name plot.sp_CDF
#' @method plot sp_CDF
#'
#' @param x Data frame from the "CDF" output given by
#'   \code{cont_analysis}.
#'
#' @param var If \code{cdfest} has multiple variables in the "Indicator" column,
#'   then \code{var} is the single variable to be plotted. The default is
#'   \code{NULL}, which assumes that only one variable is in the "Indicator"
#'   column of \code{cdfest}.
#'
#' @param subpop If \code{cdfest} has multiple variables in the "Type" column,
#'   then \code{subpop} is the single variable to be plotted. The default is
#'   \code{NULL}, which assumes that only one variable is in the "Type"
#'   column of \code{cdfest}.
#'
#' @param subpop_level If \code{cdfest} has multiple levels of \code{subpop}
#'   in the "Subpopulation" column,
#'   then \code{subpop_level} is the single level to be plotted. The default is
#'   \code{NULL}, which assumes that only one level is in the "Subpopulation"
#'   column of \code{cdfest}.
#'
#' @param units_cdf Indicator for the label utilized for the left side y-axis
#'   and the values used for the left side y-axis tick marks, where "Percent"
#'   means the label and values are in terms of percent of the population, and
#'   "Units" means the label and values are in terms of units (count, length,
#'   or area) of the population.  The default is "Percent".
#'
#' @param type_cdf Character string consisting of the value "Continuous" or
#'   "Ordinal" that controls the type of CDF plot.  The default is "Continuous".
#'
#' @param log Character string consisting of the value "" or "x" that
#'   controls whether the x axis uses the original scale ("") or the base 10
#'   logarithmic scale ("x").  The default is "".
#'
#' @param xlab Character string providing the x-axis label.  If this argument
#'   equals NULL, then the indicator name is used as the label.  The default is
#'   NULL.
#'
#' @param ylab Character string providing the left side y-axis label.  If
#'   argument units_cdf equals "Units", a value should be provided for this
#'   argument.  Otherwise, the label will be "Percent".  The default is
#'   "Percent".
#'
#' @param ylab_r Character string providing the label for the right side y-axis
#'   (and, hence, determining the values used for the right side y-axis tick
#'   marks), where NULL means a right side y-axis is not created.  If this
#'   argument equals "Same", the right side y-axis will have the same label and
#'   tick mark values as the left side y-axis.  If this argument equals a
#'   character string other than "Same", the right side y-axis label will be the
#'   value provided for argument ylab_r, and the right side y-axis tick mark
#'   values will be determined by the choice not utilized for argument
#'   units_cdf, which means that the default value of argument units_cdf (i.e.,
#'   "Percent") will result in the right side y-axis tick mark values being
#'   expressed  in terms of units of the population (i.e., count, length, or
#'   area).  The default is NULL.
#'
#' @param main Character string providing the plot title.  The default is
#'   NULL.
#'
#' @param legloc Indicator for location of the plot legend, where "BR" means
#'   bottom right, "BL" means bottom left, "TR" means top right, "TL" means
#'   top left, and NULL means no legend.  The default is NULL.
#'
#' @param confcut Numeric value that controls plotting confidence limits at
#'   the CDF extremes.  Confidence limits for CDF values (percent scale) less
#'   than confcut or greater than 100 minus confcut are not plotted.  A value of
#'   zero means confidence limits are plotted for the complete range of the CDF.
#'   The default is 0.
#'
#' @param conflev Numeric value of the confidence level used for confidence
#'   limits. The default is 95.
#'
#' @param cex.main Expansion factor for the plot title.  The default is 1.2.
#'
#' @param cex.legend Expansion factor for the legend title. The default is 1.
#'
#' @param  ... Additional arguments passed to the \code{plot.default} function
#'   (aside from those already used and \code{ylim}).
#'
#' @return A plot of a variable's CDF estimates associated confidence limits.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @seealso
#'   \describe{
#'   \item{\code{\link{cont_cdfplot}}}{for creating a PDF file containing CDF
#'     plots}
#'   \item{\code{\link{cont_cdftest}}}{for CDF hypothesis testing}
#'   }
#'
#' @keywords survey
#'
#' @examples
#' \dontrun{
#' dframe <- data.frame(
#'   siteID = paste0("Site", 1:100),
#'   wgt = runif(100, 10, 100),
#'   xcoord = runif(100),
#'   ycoord = runif(100),
#'   stratum = rep(c("Stratum1", "Stratum2"), 50),
#'   ContVar = rnorm(100, 10, 1),
#'   All_Sites = rep("All Sites", 100),
#'   Resource_Class = rep(c("Good", "Poor"), c(55, 45))
#' )
#' myvars <- c("ContVar")
#' mysubpops <- c("All_Sites", "Resource_Class")
#' mypopsize <- data.frame(
#'   Resource_Class = c("Good", "Poor"),
#'   Total = c(4000, 1500)
#' )
#' myanalysis <- cont_analysis(dframe,
#'   vars = myvars, subpops = mysubpops,
#'   siteID = "siteID", weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
#'   stratumID = "stratum", popsize = mypopsize
#' )
#' keep <- with(myanalysis$CDF, Type == "Resource_Class" &
#'   Subpopulation == "Good")
#' par(mfrow = c(2, 1))
#' plot(myanalysis$CDF[keep, ],
#'   xlab = "ContVar",
#'   ylab = "Percent of Stream Length", ylab_r = "Stream Length (km)",
#'   main = "Estimates for Resource Class: Good"
#' )
#' plot(myanalysis$CDF[keep, ],
#'   xlab = "ContVar",
#'   ylab = "Percent of Stream Length", ylab_r = "Same",
#'   main = "Estimates for Resource Class: Good"
#' )
#' }
#' @export
################################################################################
plot.sp_CDF <- function(x, var = NULL, subpop = NULL, subpop_level = NULL,
                        units_cdf = "Percent", type_cdf = "Continuous", log = "",
                        xlab = NULL, ylab = NULL, ylab_r = NULL, main = NULL, legloc = NULL,
                        confcut = 0, conflev = 95, cex.main = 1.2, cex.legend = 1, ...) {
  cdf_plot(x, var, subpop, subpop_level, units_cdf, type_cdf, log,
           xlab, ylab, ylab_r, main, legloc, confcut, conflev, cex.main, 
           cex.legend, ...)
}

###############################################################################
# Function: localmean_var (not exported)
# Programmers: Don Stevens and Tom Kincaid
# Date: October 17, 2000
#
#' Internal Function: Local Mean Variance Estimator
#'
#' This function calculates the local mean variance estimator.
#'
#' @param z Vector of weighted response values or weighted residual values for
#'  the sample points.
#'
#' @param weight_1st List from the local mean weight function containing two
#'  elements: a matrix named \code{ij} composed of the index values of neighboring
#'  points and a vector named \code{gwt} composed of weights.
#'
#' @return The local mean estimator of the variance.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @export
###############################################################################

localmean_var <- function(z, weight_1st) {

  # Calculate local means

  zb <- sapply(split(z[weight_1st$ij[, 2]] * weight_1st$gwt, weight_1st$ij[, 1]), sum)

  # Calculate the variance estimate

  lmvar <- sum(weight_1st$gwt * (z[weight_1st$ij[, 2]] - zb[weight_1st$ij[, 1]])^2)

  # Return the variance estimate

  lmvar
}

################################################################################
# Function: cdfvar_total (not exported)
# Programmer: Tom Kincaid
# Date: July 2, 2020
# Revised: April 28, 2021 to use the SRS estimator when the local mean estimator
#          fails to produce a valid estimate
# Revised: May 3, 2021 to correct an error that occurs when warning messages for
#          unstratified samples are added to the warn_df data frame
# Revised: June 8, 2021 to eliminate use of the finite population correction
#          factor with the local mean variance estimator
#
#' Variance Estimates of the Estimated CDF using the Total Scale
#'
#' This function calculates variance estimates of the estimated cumulative
#' distribution function (CDF) for the total
#'
#' @param z Vector of the response value for each site.
#'
#' @param wgt Vector of the final adjusted weight (inverse of the sample
#'   inclusion probability) for each site, which is either the weight for a
#'   single- stage sample or the stage two weight for a two-stage sample.
#'
#' @param x Vector of x-coordinate for location for each site, which is either
#'   the x-coordinate for a single-stage sample or the stage two x-coordinate
#'   for a two-stage sample.
#'
#' @param y Vector of y-coordinate for location for each site, which is either
#'   the y-coordinate for a single-stage sample or the stage two y-coordinate
#'   for a two-stage sample.
#'
#' @param val Vector of the set of values at which the CDF is estimated.
#'
#' @param stratum_ind  Logical value that indicates whether the sample is
#'   stratified, where \code{TRUE} = a stratified sample and \code{FALSE} = not
#'   a stratified sample.
#'
#' @param stratum_level The stratum level.
#'
#' @param cluster_ind Logical value that indicates whether the sample is a two-
#'   stage sample, where \code{TRUE} = a two-stage sample and \code{FALSE} = not
#'   a two-stage sample.
#'
#' @param clusterID Vector of the stage one sampling unit (primary sampling unit
#'   or cluster) code for each site.
#'
#' @param wgt1 Vector of the final adjusted stage one weight for each site.
#'
#' @param x1 Vector of the stage one x-coordinate for location for each site.
#'
#' @param y1 Vector of the stage one y-coordinate for location for each site.
#'
#' @param warn_ind Logical value that indicates whether warning messages were
#'   generated, where \code{TRUE} = warning messages were generated and
#'   \code{FALSE} = warning messages were not generated.
#'
#' @param warn_df A data frame for storing warning messages.
#'
#' @param warn_vec Vector that contains names of the subpopulation type, the
#'   subpopulation, and the response variable.
#'
#' @return A list containing the following objects:
#'   \itemize{
#'     \item{\code{vartype}}{character variable containing the type of variance
#'       estimator}
#'     \item{\code{varest}}{vector containing variance estimates}
#'     \item{\code{warn_ind}}{logical variable that indicates whether warning
#'       messages were generated}
#'     \item{\code{warn_df}}{data frame for storing warning messages}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey
#'
#' @noRd
################################################################################

cdfvar_total <- function(z, wgt, x, y, val, stratum_ind, stratum_level,
                         cluster_ind, clusterID, wgt1, x1, y1, warn_ind,
                         warn_df, warn_vec) {

  # Assign the function name

  fname <- "cdfvar_total"

  # Assign the variance type

  vartype <- "Local"

  #
  # Branch to handle two-stage and single-stage samples
  #

  if (cluster_ind) {

    # Begin the section for a two-stage sample

    # Calculate additional required values

    m <- length(val)
    cluster <- factor(clusterID)
    cluster_levels <- levels(cluster)
    ncluster <- length(cluster_levels)
    z_lst <- split(z, cluster)
    x2_lst <- split(x, cluster)
    y2_lst <- split(y, cluster)
    x1_u <- as.vector(tapply(x1, cluster, unique))
    y1_u <- as.vector(tapply(y1, cluster, unique))
    wgt2_lst <- split(wgt, cluster)
    wgt1_u <- as.vector(tapply(wgt1, cluster, unique))
    var_ind <- sapply(split(cluster, cluster), length) > 1

    # For each stage one sampling unit and each category, calculate an estimate
    # of the total of the stage two sampling unit residuals and the variance of
    # the total

    total2est <- matrix(0, ncluster, m)
    var2est <- matrix(0, ncluster, m)
    for (i in 1:ncluster) {

      # Calculate the weighted residuals matrix

      n <- length(z_lst[[i]])
      im <-
        ifelse(
          matrix(rep(z_lst[[i]], m), nrow = n) <=
            matrix(rep(val, n), nrow = n, byrow = TRUE),
          1, 0
        )
      rm <- im * matrix(rep(wgt2_lst[[i]], m), nrow = n)

      # Calculate the total estimate for each category

      total2est[i, ] <- apply(rm, 2, sum)

      # Adjust the variance estimator for small sample size

      if (var_ind[i] && n < 4) {
        warn_ind <- TRUE
        act <- "The simple random sampling variance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("There are less than four response values for stage one sampling unit \"", cluster_levels[i], "\"\nin stratum \"", stratum_level, "\", the simple random sampling variance estimator for an infinite population was used to calculate variance of the total of the residuals.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("There are less than four response values for stage one sampling unit \"", cluster_levels[i], "\", \nthe simple random sampling variance estimator for an infinite population was used to calculate variance of the total of the residuals.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        vartype <- "SRS"
      }

      # Calculate the variance estimate for each category

      if (var_ind[i]) {
        if (vartype == "Local") {
          weight_lst <- localmean_weight(
            x2_lst[[i]], y2_lst[[i]],
            1 / wgt2_lst[[i]]
          )
          if (is.null(weight_lst)) {
            warn_ind <- TRUE
            act <- "The simple random sampling variance estimator for an infinite population was used.\n"
            if (stratum_ind) {
              warn <- paste0("The local mean variance estimator cannot calculate valid estimates for stage one \nsampling unit \"", cluster_levels[i], "\" in stratum \"", stratum_level, "\", the simple random sampling \nvariance estimator for an infinite population was used to calculate variance of the \ntotal of the residuals.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = stratum_level,
                warning = I(warn), action = I(act)
              ))
            } else {
              warn <- paste0("The local mean variance estimator cannot calculate valid estimates for stage one \nsampling unit \"", cluster_levels[i], "\", the simple random sampling variance estimator for an \ninfinite population was used to calculate variance of the total of the residuals.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            }
            var2est[i, ] <- n * apply(rm, 2, var)
          } else {
            var2est[i, ] <- apply(rm, 2, localmean_var, weight_lst)
            if (any(var2est[i, ] < 0)) {
              warn_ind <- TRUE
              act <- "The simple random sampling variance estimator for an infinite population was used.\n"
              if (stratum_ind) {
                warn <- paste0("The local mean variance estimator produced one or more  negative variance estimates for \nstage one sampling unit \"", cluster_levels[i], "\" in stratum \"", stratum_level, "\", the simple random \nsampling variance estimator for an infinite population was used to calculate variance \nof the total of the residuals.\n")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1],
                  subpop = warn_vec[2], indicator = warn_vec[3],
                  stratum = stratum_level, warning = I(warn), action = I(act)
                ))
              } else {
                warn <- paste0("The local mean variance estimator produced one or more  negative variance estimates for \nstage one sampling unit \"", cluster_levels[i], "\", the simple random sampling variance estimator \nfor an infinite population was used to calculate variance of the total of the residuals.\n")
                warn_df <- rbind(warn_df, data.frame(
                  func = I(fname), subpoptype = warn_vec[1],
                  subpop = warn_vec[2], indicator = warn_vec[3], stratum = NA,
                  warning = I(warn), action = I(act)
                ))
              }
              var2est[i, ] <- n * apply(rm, 2, var)
            }
          }
        } else {
          var2est[i, ] <- n * apply(rm, 2, var)
          vartype <- "Local"
        }
      }
    }

    # Adjust the variance estimator for small sample size

    if (ncluster < 4) {
      warn_ind <- TRUE
      act <- "The simple random sampling variance estimator for an infinite population was used.\n"
      if (stratum_ind) {
        warn <- paste0("There are less than four stage one sampling units in stratum \"", stratum_level, "\", \nthe simple random sampling variance estimator for an infinite population was used \nto calculate variance of the CDF estimate.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
          stratum = I(stratum_level), warning = I(warn), action = I(act)
        ))
      } else {
        warn <- paste0("There are less than four stage one sampling units, the simple random sampling \nvariance estimator for an infinite population was used to calculate variance of the \nCDF estimate.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2], indicator = warn_vec[3],
          stratum = NA, warning = I(warn), action = I(act)
        ))
      }
      vartype <- "SRS"
    }

    # Calculate the variance estimate

    if (vartype == "Local") {
      weight_lst <- localmean_weight(x1_u, y1_u, 1 / wgt1_u)
      if (is.null(weight_lst)) {
        warn_ind <- TRUE
        act <- "The simple random sampling variance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("The local mean variance estimator cannot calculate valid estimates for stratum \n\"", stratum_level, "\", the simple random sampling variance estimator for an infinite \npopulation was used to calculate variance of the CDF estimate.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("The local mean variance estimator cannot calculate valid estimates, the simple random \nsampling variance estimator for an infinite population was used to calculate variance \nof the CDF estimate.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA,
            warning = I(warn), action = I(act)
          ))
        }
        vartype <- "SRS"
        varest <- 0
      } else {
        varest <-
          apply(
            total2est * matrix(rep(wgt1_u, m), nrow = ncluster), 2,
            localmean_var, weight_lst
          ) +
          apply(var2est * matrix(rep(wgt1_u, m), nrow = ncluster), 2, sum)
      }
    } else {
      varest <- 0
    }

    # End of section for a two-stage sample
  } else {

    # Begin the section for a single-stage sample

    # Calculate additional required values

    n <- length(z)
    m <- length(val)

    # Calculate the weighted residuals matrix

    im <-
      ifelse(
        matrix(rep(z, m), nrow = n) <=
          matrix(rep(val, n), nrow = n, byrow = TRUE),
        1, 0
      )
    rm <- im * matrix(rep(wgt, m), nrow = n)

    # Adjust the variance estimator for small sample size

    if (n < 4) {
      warn_ind <- TRUE
      act <- "The simple random sampling variance estimator for an infinite population was used.\n"
      if (stratum_ind) {
        warn <- paste0("There are less than four response values in stratum \"", stratum_level, "\", the simple random \nsampling variance estimator for an infinite population was used to calculate variance \nof the CDF estimate.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2],
          indicator = warn_vec[3], stratum = I(stratum_level), warning = I(warn),
          action = I(act)
        ))
      } else {
        warn <- paste0("\nThere are less than four response values, the simple random sampling variance estimator \nfor an infinite population was used to calculate variance of the CDF estimate.\n")
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname),
          subpoptype = warn_vec[1], subpop = warn_vec[2],
          indicator = warn_vec[3], stratum = NA, warning = I(warn),
          action = I(act)
        ))
      }
      vartype <- "SRS"
    }

    # Calculate the variance estimate

    if (vartype == "Local") {
      weight_lst <- localmean_weight(x, y, 1 / wgt)
      if (is.null(weight_lst)) {
        warn_ind <- TRUE
        act <- "The simple random sampling variance estimator for an infinite population was used.\n"
        if (stratum_ind) {
          warn <- paste0("The local mean variance estimator cannot calculate valid estimates for stratum \n\"", stratum_level, "\", the simple random sampling variance estimator for an infinite \npopulation was used to calculate variance of the CDF estimate.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = stratum_level, warning = I(warn),
            action = I(act)
          ))
        } else {
          warn <- paste0("The local mean variance estimator cannot calculate valid estimates, the simple random \nsampling variance estimator for an infinite population was used to calculate variance \nof the CDF estimate.\n")
          warn_df <- rbind(warn_df, data.frame(
            func = I(fname), subpoptype = warn_vec[1], subpop = warn_vec[2],
            indicator = warn_vec[3], stratum = NA, warning = I(warn),
            action = I(act)
          ))
        }
        vartype <- "SRS"
        varest <- 0
      } else {
        varest <- apply(rm, 2, localmean_var, weight_lst)
      }
    } else {
      varest <- 0
    }

    # End of section for a single-stage sample
  }

  # Return the indicator for type of variance estimator, the variance estimate,
  # the warning message indicator, and the warn_df data frame

  list(
    vartype = vartype, varest = varest, warn_ind = warn_ind,
    warn_df = warn_df
  )
}

###############################################################################
# Function: pd_summary (exported)
# Programmer: Tony Olsen
# Date: March 14, 2019
#'
#' Summary characteristics of a panel revisit design
#'
#' Panel revisit design characteristics are summarized: number of panels, number
#' of time periods, total number of sample events for the revisit design, total
#' number of sample events for each panel, total number of sample events for
#' each time period and cumulative number of unique units sampled by time
#' periods.
#'
#' @param  object Two-dimensional array from \code{panel_design} and dimnames specifying revisit
#'   panel design. Typically, array is output from \code{revisit_dsgn}, \code{revisit_bibd} or
#'   \code{revisit_rand} functions.
#'
#' @param  visitdsgn Two-dimensional array with same dimensions as \code{paneldsgn}
#'   specifying the number of times a sample unit is sampled at each time
#'   period. Default is \code{visitdsgn=NULL}, where default assumes that a sample unit
#'   will be sampled only once at each time period.
#'
#' @param ... Additional arguments (S3 consistency)
#'
#' @details The revisit panel design and the visit design (if present) are
#'   summarized. Summaries can be useful to know the effort required to complete
#'   the survey design. See the values returned for the summaries that are
#'   produced.
#'
#' @return List of six elements.
#'   \describe{
#'     \item{\code{n_panel}}{ number of panels in revisit design}
#'     \item{\code{n_period}}{ number of time periods in revisit design}
#'     \item{\code{n_total}}{ total number of sample events across all panels and all
#'       time periods, accounting for \code{visitdsgn}, that will be sampled in the revisit
#'       design}
#'     \item{\code{n_periodunit}}{ vector of the number of time periods a unit will be
#'       sampled in each panel}
#'     \item{\code{n_unitpnl}}{ vector of the number of sample units, accounting for
#'       \code{visitdsgn}, that will be sampled in each panel}
#'     \item{\code{n_unitperiod}}{ vector of the number of sample units, accounting for
#'       \code{visitdsgn}, that will be sampled during each time period}
#'     \item{\code{ncum_unit}}{ vector of the cumulative number of unique units that will
#'   be sampled in time periods up to and including the current time period.}
#'   }
#'
#' @export
#'
#' @author Tony Olsen \email{Olsen.Tony@@epa.gov}
#'
#' @keywords survey
#'
#' @examples
#' # Serially alternating panel revisit design summary
#' sa_dsgn <- revisit_dsgn(20, panels = list(SA60N = list(
#'   n = 60, pnl_dsgn = c(1, 4),
#'   pnl_n = NA, start_option = "None"
#' )), begin = 1)
#' pd_summary(sa_dsgn)
#' # Add visit design where first panel is sampled twice at every time period
#' sa_visit <- sa_dsgn
#' sa_visit[sa_visit > 0] <- 1
#' sa_visit[1, sa_visit[1, ] > 0] <- 2
#' pd_summary(sa_dsgn, sa_visit)
pd_summary <- function(object, visitdsgn = NULL, ...) {
  if (!inherits(object, "paneldesign")) {
    stop("object must be output from spsurvey::revisit_bibd(),
         spsurvey::revisit_dsgn(), or spsurvey::revisit_rand()")
  }
  paneldsgn <- object
  n_pan <- dim(paneldsgn)[1]
  n_period <- dim(paneldsgn)[2]

  # determine the cumulative number of unique sample units by sampling occasion
  used <- rep(FALSE, n_pan)
  tot <- vector("numeric", length = n_period)
  for (i in 1:n_period) {
    units <- paneldsgn[, i] > 0
    new <- used + units
    tot[i] <- sum(paneldsgn[new == 1, i])
    used <- new
  }
  n_unique_com <- cumsum(tot)
  names(n_unique_com) <- dimnames(paneldsgn)[[2]]

  # summarize number of sample results by panel and by time period
  # incorporate multiple times unit is sampled if sample units for a time period are
  # sampled more than once.
  ifelse(!is.null(visitdsgn), vis <- visitdsgn * paneldsgn, vis <- paneldsgn)
  n_unitpnl <- apply(vis, 1, sum)
  n_unitperiod <- apply(vis, 2, sum)
  n_total <- sum(n_unitperiod)
  # number of times a sample unit is visited in each panel
  tmp <- array(0, c(n_pan, n_period))
  tmp[paneldsgn > 0] <- 1
  ifelse(!is.null(visitdsgn), vis <- visitdsgn * tmp, vis <- tmp)
  n_periodunit <- apply(vis, 1, sum)
  names(n_periodunit) <- dimnames(paneldsgn)[[1]]

  # create list of results
  rslt <- list(
    n_panel = n_pan,
    n_period = n_period,
    n_total = n_total,
    n_periodunit = n_periodunit,
    n_unitpnl = n_unitpnl,
    n_unitperiod = n_unitperiod,
    ncum_unit = n_unique_com
  )
  return(rslt)
}

################################################################################
# Function: localmean_weight (not exported)
# Programmers: Don Stevens and Tom Kincaid
# Date: February 6, 2020
# Revised: April 28, 2021 to use the modified object returned by the
#          localmean_weight2 function and to return a NULL object when the ginv
#          function fails to return valid output
# Revised: April 29, 2021 to eliminate use of the while loop to achieve valid
#          output from the ginv function, which means that the vincr argument
#          and the localmean_weight2 function are no longer needed
#
#' Internal Function: Local Mean Variance Neighbors and Weights
#'
#' This function calculates the index values of neighboring points and
#' associated weights required by the local mean variance estimator.
#'
#' @param x Vector of x-coordinates for location of the sample points.
#'
#' @param y Vector of y-coordinates for location of the sample points.
#'
#' @param prb Vector of inclusion probabilities for the sample points.
#'
#' @param nbh Number of neighboring points to use in the calculations.
#'
#' @return If ginv fails to return valid output, a NULL object.  Otherwise, a
#'   list containing two elements: a matrix named \code{ij} composed of the
#'   index values of neighboring points and a vector named \code{gwt}
#'   composed of weights.
#'
#' @author  Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @export
################################################################################

localmean_weight <- function(x, y, prb, nbh = 4) {

  # Assign tne number of points

  n <- length(x)

  # Calculate indices of nearest neighbors

  dst <- as.matrix(dist(cbind(x, y), diag = TRUE, upper = TRUE))
  idx <- apply(dst, 2, order)[1:nbh, ]

  # Make neighbors symmetric

  jdx <- rep(1:n, rep(nbh, n))
  kdx <- unique(c((jdx - 1) * n + idx, (idx - 1) * n + jdx)) - 1
  ij <- cbind((kdx) %/% n + 1, (kdx) %% n + 1)
  ij <- ij[order(ij[, 1], dst[ij]), ]

  # Apply linear taper to the  inverse probability weights

  gct <- tabulate(ij[, 1])
  gwt <- numeric(0)
  for (i in 1:n) {
    gwt <- c(gwt, 1 - (1:gct[i] - 1) / (gct[i]))
  }
  gwt <- gwt / prb[ij[, 2]]

  # Normalize to make true average

  smwt <- sapply(split(gwt, ij[, 1]), sum)
  gwt <- gwt / smwt[ij[, 1]]
  smwt <- sapply(split(gwt, ij[, 2]), sum)

  # Make weights doubly stochastic

  hij <- matrix(0, n, n)
  hij[ij] <- 0.5
  a22 <- try(ginv(diag(gct / 2) - hij %*% diag(2 / gct) %*% hij), TRUE)
  if ("try-error" %in% class(a22)) {
    return(NULL)
  }
  a21 <- -diag(2 / gct) %*% hij %*% a22
  lm <- a21 %*% (1 - smwt)
  gm <- a22 %*% (1 - smwt)
  gwt <- (lm[ij[, 1]] + gm[ij[, 2]]) / 2 + gwt

  # Return the results

  list(ij = ij, gwt = gwt)
}

############################################################################
# Function: adjwgtNR (exported)
# Programmer: Tony Olsen
# Date: April 5, 2022
#
#' Adjust survey design weights for non-response by categories
#'
#' @description Adjust weights for target sample units that do not respond
#' and are missing at random within categories. The missing at random
#' assumption implies that their sample weight may be assigned to
#' specific categories of units that have responded (i.e., have been
#' sampled). This is a class-based method for non-response adjustment.
#'
#' @param wgt vector of weights for each sample unit that will be adjusted
#'  for non-response. Weights must be weights for the design as implemented.
#'  All weights must be greater than zero.
#'
#' @param MARClass vector that identifies for each sample unit the class
#'  (i.e., category, level)
#'  that will be used in non-response weight adjustment for sample units
#'  that are known to be target. Within each missing at random (MAR)
#'  class, the missing sample units that are not sampled are assumed to
#'  be missing at random. If MARClass is not specified, all sample units
#'  are assumed to be from the same MAR class.
#'
#' @param EvalStatus vector of the evaluation status for each sample unit.
#'  Values must include the values given in TNRclass and TRClass. May
#'  include other values not required for the non-response adjustment.
#'
#' @param TNRClass subset of values in EvalStatus that identify sample units
#'  whose target status is known and that do not respond (i.e., are not
#'  sampled).
#'
#' @param TRClass Subset of values in EvalStatus that identify sample units
#'  whose target status is known and that respond (i.e., are target and
#'  sampled).
#'
#' @return Vector of sample unit weights that are adjusted for non-response
#'  and that is the same length of input weights. Weights for sample
#'  units that did not response but were known to be eligible are set
#'  to zero. Weights for all other sample units are also set to zero.
#'
#' @export
#'
#' @author Tony Olsen \email{olsen.tony@epa.gov}
#'
#' @keywords survey non-response weight adjustment
#'
#' @examples
#' set.seed(5)
#' wgt <- runif(40)
#' MARClass <- rep(c("A", "B"), rep(20, 2))
#' EvalStatus <- sample(c("Not_Target", "Target_Sampled", "Target_Not_Sampled"), 40, replace = TRUE)
#' TNRClass <- "Target_Not_Sampled"
#' TRClass <- "Target_Sampled"
#' adjwgtNR(wgt, MARClass, EvalStatus, TNRClass, TRClass)
#' # function that has an error check
adjwgtNR  <- function(wgt, MARClass = NULL, EvalStatus, TNRClass, TRClass){
  if (is.null(MARClass)) {
    MARClass <- rep("1", length(wgt))
  }
  tstTNRClass <- EvalStatus %in% c(TNRClass)
  tstTRClass <- EvalStatus %in% c(TRClass)
  num <- tapply(wgt[tstTNRClass | tstTRClass],
                MARClass[tstTNRClass | tstTRClass], sum)
  den <- tapply(wgt[tstTRClass], MARClass[tstTRClass], sum)

  ar <- num/den[match(names(num), names(den))]
  wgt[tstTRClass] <- wgt[tstTRClass] *
    ar[match(MARClass, names(ar))][tstTRClass]
  wgt[!tstTRClass] <- 0
  # error/warning check
  # could use any(! unique(MARClass[tstTNRClass]) %in% unique(MARClass[tstTRClass]))
  if (length(num) > length(den)) {
    warning("At least one level of MARClass does not have any EvalStatus values in
         TRClass; for those MARClass level(s), non-response weight adjustment(s) cannot be performed.
         Consider aggregating so that all levels of MARClass have at least one TRClass value in EvalStatus.", call. = FALSE)
    wgt[is.na(ar[match(MARClass, names(ar))])] <- NA
  }
  as.vector(wgt)
}

context("weight adjustment non-response")

set.seed(5)

test_that("adjust weight non-response works no extra Eval", {
  wgt <- runif(30)
  MARClass <- rep(c("A", "B"), rep(15, 2))
  EvalStatus <- rep(c("Target_Sampled", "Target_Not_Sampled"), 15)
  TNRClass <- "Target_Not_Sampled"
  TRClass <- "Target_Sampled"
  wgt_new <- adjwgtNR(wgt, MARClass, EvalStatus, TNRClass, TRClass)
  wgtsums <- sum(wgt_new[EvalStatus == "Target_Sampled"])
  wgtzeros <- sum(wgt_new[EvalStatus == "Target_Not_Sampled"])
  expect_equal(sum(wgt), wgtsums)
  expect_equal(0, wgtzeros)

  # error returned
  EvalStatus[EvalStatus == "Target_Sampled"] <- "Target_Not_Sampled"
  # expect_error(adjwgtNR(wgt, MARClass, EvalStatus, TNRClass, TRClass))
  expect_warning(adjwgtNR(wgt, MARClass, EvalStatus, TNRClass, TRClass))
})

test_that("adjust weight non-response works extra Eval", {

  wgt <- runif(30)
  MARClass <- rep(c("A", "B"), rep(15, 2))
  EvalStatus <- rep(c("Not_Target", "Target_Sampled", "Target_Not_Sampled"), 10)
  TNRClass <- "Target_Not_Sampled"
  TRClass <- "Target_Sampled"
  wgt_new <- adjwgtNR(wgt, MARClass, EvalStatus, TNRClass, TRClass)
  wgtsums <- sum(wgt_new[EvalStatus == "Target_Sampled"])
  wgtzeros <- sum(wgt_new[EvalStatus == "Target_Not_Sampled"])
  expect_equal(sum(wgt[EvalStatus %in% c("Target_Not_Sampled", "Target_Sampled")]), wgtsums)
  expect_equal(0, wgtzeros)
  wgtzeros_extra <- sum(wgt_new[!EvalStatus %in% c("Target_Not_Sampled", "Target_Sampled")])
  expect_equal(0, wgtzeros)

  # error returned
  EvalStatus[EvalStatus == "Target_Sampled"] <- "Not_Target"
  # expect_error(adjwgtNR(wgt, MARClass, EvalStatus, TNRClass, TRClass))
  expect_warning(adjwgtNR(wgt, MARClass, EvalStatus, TNRClass, TRClass))
})

################################################################################
# Function: bootfcn (not exported)
# Programmer: Tom Kincaid
# Date: February 23, 2021
#
#' Bootstrap Function for Trend Parameter Estimation
#'
#' This function calculates trend parameter estimates using bootstrap replicates.
#'
#' @param dframe Data frame containing survey design variables, response
#'   variables, and subpopulation (domain) variables.
#'
#' @param indices Vector of indices referencing rows in the dframe data frame.
#'
#' @param ivar Character value providing name of the response variable in the
#'   dframe data frame.
#'
#' @param siteID Character value providing name of the site ID variable in
#'   the dframe data frame.
#'
#' @param yearID Character value providing name of the year variable in
#'   the dframe data frame.
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @noRd
################################################################################

bootfcn <- function(dframe, indices, ivar, siteID, yearID) {
  dframe <- dframe[indices, ]
  eval(parse(text = paste0(
    "regest <- lmer(", ivar, " ~ Wyear + (1 + Wyear|",
    siteID, ") + (1|", yearID, "), data = dframe,
    control = lmerControl(check.nobs.vs.nRE = 'warning'))"
  )))
  coeff <- summary(regest)$coefficients
  vcor <- as.data.frame(VarCorr(regest))
  rslt <- c(
    siteint = coeff[1, "Estimate"],
    siteint_stderr = coeff[1, "Std. Error"],
    siteslope = coeff[2, "Estimate"],
    siteslope_stderr = coeff[2, "Std. Error"],
    var_siteint = vcor[1, 4],
    var_siteslope = vcor[2, 4],
    corr_site = vcor[3, 5],
    var_year = vcor[4, 4],
    var_resid = vcor[5, 4],
    AIC = extractAIC(regest)[2]
  )
  rslt
}

################################################################################
# Function: relrisk_analysis (exported)
# Programmer: Tom Kincaid
# Date: July 16, 2020
# Revised: August 14, 2020 to allow use of an sf object as the input data
#          argument (dframe)
# Revised: December 15, 2020 to use a new function named survey_design to
#          create the survey design object
# Revised: March 2, 2021 to revise the process for creating unique site ID
#          values
# Revised: April 7, 2021 to ensure that the dframe argument does not contain
#          zero rows
# Revised: April 29, 2021 to ensure that the dframe argument only belongs to
#          class "data.frame"
# Revised: May 4, 2021 to avoid warning messages being generated during creation
#          of help files
# Revised: May 6, 2021 to ensure that sf objects do not belong to class tbl_df
# Revised: June 8, 2021 to simplify specification of the values required for
#          calculation of the finite population correction factor and to
#          eliminate use of the finite population correction factor with the
#          local mean variance estimator
# Revised: June 11, 2021 to modify the approach for specification of arguments
#          response_levels and stressor_levels
# Revised: July 28, 2021 to improve documentation
# Revised: September 9, 2021 to revise the documentation for argument popsize
#
#' Relative risk analysis
#'
#' This function organizes input and output for relative risk analysis (of
#' categorical variables).  The analysis data,
#' \code{dframe}, can be either a data frame or a simple features (\code{sf}) object.  If an
#' \code{sf} object is used, coordinates are extracted from the geometry column in the
#' object, arguments \code{xcoord} and \code{ycoord} are assigned values
#' \code{"xcoord"} and \code{"ycoord"}, respectively, and the geometry column is
#' dropped from the object.
#'
#' @inherit attrisk_analysis params
#'
#' @section Details:
#' Relative risk measures the relative strength of association between
#' conditional probabilities defined for a response variable and a stressor
#' variable, where the response and stressor variables are classified as either
#' good (i.e., reference condition) or poor (i.e., different from reference
#' condition). Relative risk is defined as the ratio of two conditional
#' probabilities.  The numerator of the ratio is the probability that the
#' response variable is in poor condition given that the stressor variable is in
#' poor condition.   The denominator of the ratio is the probability that the
#' response variable is in poor condition given that the stressor variable is in
#' good condition. A relative risk value equal to one indicates that the
#' response variable is independent of the stressor variable.  Relative risk
#' values greater than one measure the extent to which poor condition of the
#' stressor variable is associated with poor condition of the response variable.
#'
#' @return The analysis results. A data frame of population estimates for all combinations of
#'   subpopulations, categories within each subpopulation, response variables,
#'   and categories within each response variable.  Estimates are provided for
#'   proportion and size of the population plus standard error, margin of error,
#'   and confidence interval estimates. The data frame contains the following
#'   variables:
#'   \describe{
#'     \item{Type}{subpopulation (domain) name}
#'     \item{Subpopulation}{subpopulation name within a domain}
#'     \item{Response}{response variable}
#'     \item{Stressor}{stressor variable}
#'     \item{nResp}{sample size}
#'     \item{Estimate}{relative risk estimate}
#'     \item{Estimate_num}{relative risk numerator estimate}
#'     \item{Estimate_denom}{relative risk denominator estimate}
#'     \item{StdError}{relative risk standard error}
#'     \item{MarginofError}{relative risk margin of error}
#'     \item{LCBxxPct}{xx\% (default 95\%) lower confidence bound}
#'     \item{UCBxxPct}{xx\% (default 95\%) upper confidence bound}
#'     \item{WeightTotal}{sum of design weights}
#'     \item{Count_RespPoor_StressPoor}{number of observations in the poor response and poor stressor group}
#'     \item{Count_RespPoor_StressGood}{number of observations in the poor response and good stressor group}
#'     \item{Count_RespGood_StressPoor}{number of observations in the good response and poor stressor group}
#'     \item{Count_RespGood_StressGood}{number of observations in the good response and good stressor group}
#'     \item{Prop_RespPoor_StressPoor}{weighted proportion of observations in the poor response and poor stressor group}
#'     \item{Prop_RespPoor_StressGood}{weighted proportion of observations in the poor response and good stressor group}
#'     \item{Prop_RespGood_StressPoor}{weighted proportion of observations in the good response and poor stressor group}
#'     \item{Prop_RespGood_StressGood}{weighted proportion of observations in the good response and good stressor group}
#'   }
#'
#' @author Tom Kincaid \email{Kincaid.Tom@@epa.gov}
#'
#' @keywords survey univar
#'
#' @seealso
#'   \describe{
#'   \item{\code{\link{attrisk_analysis}}}{ for attributable risk analysis}
#'   \item{\code{\link{diffrisk_analysis}}}{ for risk difference analysis}
#'   }
#'
#' @examples
#' dframe <- data.frame(
#'   siteID = paste0("Site", 1:100),
#'   wgt = runif(100, 10, 100),
#'   xcoord = runif(100),
#'   ycoord = runif(100),
#'   stratum = rep(c("Stratum1", "Stratum2"), 50),
#'   RespVar1 = sample(c("Poor", "Good"), 100, replace = TRUE),
#'   RespVar2 = sample(c("Poor", "Good"), 100, replace = TRUE),
#'   StressVar = sample(c("Poor", "Good"), 100, replace = TRUE),
#'   All_Sites = rep("All Sites", 100),
#'   Resource_Class = rep(c("Agr", "Forest"), c(55, 45))
#' )
#' myresponse <- c("RespVar1", "RespVar2")
#' mystressor <- c("StressVar")
#' mysubpops <- c("All_Sites", "Resource_Class")
#' relrisk_analysis(dframe,
#'   vars_response = myresponse,
#'   vars_stressor = mystressor, subpops = mysubpops, siteID = "siteID",
#'   weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
#'   stratumID = "stratum"
#' )
#' @export
################################################################################

relrisk_analysis <- function(dframe, vars_response, vars_stressor, response_levels = NULL,
                             stressor_levels = NULL, subpops = NULL, siteID = NULL, weight = "weight",
                             xcoord = NULL, ycoord = NULL, stratumID = NULL, clusterID = NULL,
                             weight1 = NULL, xcoord1 = NULL, ycoord1 = NULL, sizeweight = FALSE,
                             sweight = NULL, sweight1 = NULL, fpc = NULL, popsize = NULL,
                             vartype = "Local", conf = 95, All_Sites = FALSE) {

  # Create a vector for error messages

  error_ind <- FALSE
  error_vec <- NULL

  # Create a data frame for warning messages

  warn_ind <- FALSE
  warn_df <- NULL
  fname <- "relrisk_analysis"

  # Ensure that the dframe argument was provided

  if (missing(dframe) | is.null(dframe)) {
    stop("\nThe dframe argument must be provided.\n")
  }

  # If the dframe argument is an sf object, extract coordinates from the
  # geometry column, assign values "xcoord" and "ycoord" to arguments xcoord and
  # ycoord, respectively, and drop the geometry column from the object

  if ("sf" %in% class(dframe)) {
    temp <- st_coordinates(dframe)
    xcoord <- "xcoord"
    dframe$xcoord <- temp[, "X"]
    ycoord <- "ycoord"
    dframe$ycoord <- temp[, "Y"]
    dframe <- st_set_geometry(dframe, NULL)
  }

  # If the dframe argument is a tibble or does not belong to class
  # "data.frame", coerce the argument to class "data.frame"

  if ("tbl_df" %in% class(dframe) | !("data.frame" %in% class(dframe))) {
    dframe <- as.data.frame(dframe)
  }

  # Ensure that the dframe argument does not contain zero rows

  if (nrow(dframe) == 0) {
    stop("\nThe dframe argument contains zero rows.\n")
  }

  # Ensure that unused levels are dropped from factor variables in the dframe
  # data frame

  dframe <- droplevels(dframe)

  # If no siteID is provided, set one that assumes each row is a unique site

  if (is.null(siteID)) {
    siteID <- "siteID"
    dframe$siteID <- paste("site", seq_len(nrow(dframe)), sep = "-")
  }

  # Ensure that the dframe data frame contains the site ID variable

  if (!(siteID %in% names(dframe))) {
    ind <- FALSE
    error_ind <- TRUE
    msg <- paste0("The name provided for the siteID argument, \"", siteID, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  } else {
    ind <- TRUE
  }

  # Check site IDs for repeat values and, as necessary, create unique site IDs
  # and output a warning message

  if (ind) {
    IDs <- dframe[, siteID]
    temp <- sapply(split(IDs, IDs), length)
    if (any(temp > 1)) {
      warn_ind <- TRUE
      temp.str <- vecprint(names(temp)[temp > 1])
      warn <- paste("The following site ID values occur more than once among the values that were \ninput to the function:\n", temp.str)
      act <- "Unique site ID values were created.\n"
      warn_df <- rbind(warn_df, data.frame(
        func = I(fname), subpoptype = NA,
        subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
      ))
      dframe[, siteID] <- uniqueID(dframe[, siteID])
    }
  }

  # Ensure that the dframe data frame contains the survey weight variable

  if (!(weight %in% names(dframe))) {
    error_ind <- TRUE
    msg <- paste0("The name provided for the weight argument, \"", weight, "\", does not occur among \nthe names for the dframe data frame.\n")
    error_vec <- c(error_vec, msg)
  }

  # Assign names to the variables required for calculation of the finite
  # population correction factor

  if (is.null(fpc)) {
    fpcfactor_ind <- FALSE
    fpcsize <- NULL
    Ncluster <- NULL
    stage1size <- NULL
  } else {
    fpcfactor_ind <- TRUE
    if (is.null(clusterID)) {
      fpcsize <- "fpcsize"
      Ncluster <- NULL
      stage1size <- NULL
    } else {
      fpcsize <- NULL
      Ncluster <- "Ncluster"
      stage1size <- "stage1size"
    }
  }

  # Create a list containing names of survey design variables

  design_names <- list(
    siteID = siteID,
    weight = weight,
    xcoord = xcoord,
    ycoord = ycoord,
    stratumID = stratumID,
    clusterID = clusterID,
    weight1 = weight1,
    xcoord1 = xcoord1,
    ycoord1 = ycoord1,
    sweight = sweight,
    sweight1 = sweight1,
    fpcsize = fpcsize,
    Ncluster = Ncluster,
    stage1size = stage1size
  )

  # Ensure that a value was provided for the vars_response (response variable
  # names) argument

  if (missing(vars_response)) {
    error_ind <- TRUE
    msg <- "A value must be provided for the vars_response (response variable names) argument.\n"
    error_vec <- c(error_vec, msg)
  }

  # Ensure that a value was provided for the vars_stressor (stressor variable
  # names) argument

  if (missing(vars_stressor)) {
    error_ind <- TRUE
    msg <- "A value must be provided for the vars_stressor (stressor variable names) argument.\n"
    error_vec <- c(error_vec, msg)
  }

  # If a value was not provided for the subpops (subpopulation names) argument,
  # assign the value "All_Sites" to the subpops argument and create a factor
  # named "All_Sites" in the dframe data frame that takes the value "All Sites"

  if (is.null(subpops)) {
    subpops <- "All_Sites"
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # If the user wants information for all sites together in addition to the
  # subpops, add the value "All_Sites" to the subpops argument and create a
  # factor named "All_Sites" in the dframe data frame that takes the value
  # "All Sites"

  if (!is.null(subpops) && All_Sites) {
    subpops <- c(subpops, "All_Sites")
    dframe$All_Sites <- "All Sites"
    dframe$All_Sites <- factor(dframe$All_Sites)
  }

  # Check input arguments

  temp <- input_check(dframe, design_names, vars_response, NULL, vars_stressor,
    NULL, subpops, sizeweight, fpc, popsize, vartype, NULL, conf,
    error_ind = error_ind, error_vec = error_vec
  )
  dframe <- temp$dframe
  vars_response <- temp$vars_cat
  vars_stressor <- temp$vars_stressor
  subpops <- temp$subpops
  popsize <- temp$popsize
  vartype <- temp$vartype
  error_ind <- temp$error_ind
  error_vec <- temp$error_vec

  # Check arguments response_levels and stressor_levels

  if (is.null(response_levels)) {
    response_levels <- rep(list(c("Poor", "Good")), length(vars_response))
    names(response_levels) <- vars_response
  } else {
    if (!is.list(response_levels)) {
      error_ind <- TRUE
      msg <- "Argument response_levels must be a list.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (length(response_levels) != length(vars_response)) {
        error_ind <- TRUE
        msg <- "Argument response_levels must be the same length as argument vars_response.\n"
        error_vec <- c(error_vec, msg)
      }
      if (is.null(names(response_levels))) { # set default names if none provided
        names(response_levels) <- vars_response
      }
      if (any(sapply(response_levels, function(x) length(x) != 2))) {
        error_ind <- TRUE
        msg <- "Each element of argument response_levels must contain only two values.\n"
        error_vec <- c(error_vec, msg)
      }
      if (any(sapply(response_levels, function(x) !is.character(x)))) {
        error_ind <- TRUE
        msg <- "Each element of argument response_levels must contain character values.\n"
        error_vec <- c(error_vec, msg)
      }
      if (all(vars_response %in% names(dframe))) {
        tst <- logical(length(vars_response))
        for (i in 1:length(vars_response)) {
          if (!all(response_levels[[i]] %in% levels(dframe[, vars_response[i]]))) {
            tst[i] <- TRUE
          }
        }
        if (any(tst)) {
          temp_str <- vecprint(vars_response[tst])
          error_ind <- TRUE
          msg <- paste0("\nCategory names for the following response variables do not match category names \nin the response_levels argument:\n", temp_str)
          error_vec <- c(error_vec, msg)
        } else {
          if (!all(names(response_levels) %in% vars_response)) {
            error_ind <- TRUE
            msg <- "Names for the response_levels list do not match the values in the vars_response \nargument."
            error_vec <- c(error_vec, msg)
          } else {
            indx <- match(vars_response, names(response_levels))
            response_levels <- response_levels[indx]
          }
        }
      }
    }
  }

  if (is.null(stressor_levels)) {
    stressor_levels <- rep(list(c("Poor", "Good")), length(vars_stressor))
    names(stressor_levels) <- vars_stressor
  } else {
    if (!is.list(stressor_levels)) {
      error_ind <- TRUE
      msg <- "Argument stressor_levels must be a list.\n"
      error_vec <- c(error_vec, msg)
    } else {
      if (length(stressor_levels) != length(vars_stressor)) {
        error_ind <- TRUE
        msg <- "Argument stressor_levels must be the same length as argument vars_stressor.\n"
        error_vec <- c(error_vec, msg)
      }
      if (is.null(names(stressor_levels))) { # set default names if none provided
        names(stressor_levels) <- vars_stressor
      }
      if (any(sapply(stressor_levels, function(x) length(x) != 2))) {
        error_ind <- TRUE
        msg <- "Each element of argument stressor_levels must contain only two values.\n"
        error_vec <- c(error_vec, msg)
      }
      if (any(sapply(stressor_levels, function(x) !is.character(x)))) {
        error_ind <- TRUE
        msg <- "Each element of argument stressor_levels must contain character values.\n"
        error_vec <- c(error_vec, msg)
      }
      if (all(vars_stressor %in% names(dframe))) {
        tst <- logical(length(vars_stressor))
        for (i in 1:length(vars_stressor)) {
          if (!all(stressor_levels[[i]] %in% levels(dframe[, vars_stressor[i]]))) {
            tst[i] <- TRUE
          }
        }
        if (any(tst)) {
          temp_str <- vecprint(vars_stressor[tst])
          error_ind <- TRUE
          msg <- paste0("\nCategory names for the following stressor variables do not match category names \nin the stressor_levels argument:\n", temp_str)
          error_vec <- c(error_vec, msg)
        } else {
          if (!all(names(stressor_levels) %in% vars_stressor)) {
            error_ind <- TRUE
            msg <- "Names for the stressor_levels list do not match the values in the vars_stressor \nargument."
            error_vec <- c(error_vec, msg)
          } else {
            indx <- match(vars_stressor, names(stressor_levels))
            stressor_levels <- stressor_levels[indx]
          }
        }
      }
    }
  }

  # As necessary, output a message indicating that error messages were generated
  # during execution of the program

  if (error_ind) {
    error_vec <<- error_vec
    if (length(error_vec) == 1) {
      message("During execution of the program, an error message was generated.  The error \nmessage is stored in a vector named 'error_vec'.  Enter the following command \nto view the error message: errorprnt()\n")
    } else {
      message(paste("During execution of the program,", length(error_vec), "error messages were generated.  The error \nmessages are stored in a vector named 'error_vec'.  Enter the following \ncommand to view the error messages: errorprnt()\n"))
    }

    if (warn_ind) {
      warn_df <<- warn_df
      if (nrow(warn_df) == 1) {
        message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
      } else {
        message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
      }
    }
    stop("See the preceding message(s).")
  }

  # Assign a logical value to the indicator variable for a stratified sample

  stratum_ind <- !is.null(stratumID)

  # For a stratified sample, remove strata that contain a single site

  if (stratum_ind) {
    dframe[, stratumID] <- factor(dframe[, stratumID])
    stratum_levels <- levels(dframe[, stratumID])
    nstrata <- length(stratum_levels)
    ind <- FALSE
    for (i in 1:nstrata) {
      tst <- dframe[, stratumID] == stratum_levels[i]
      if (sum(tst) == 1) {
        warn_ind <- TRUE
        warn <- paste("The stratum named \"", stratum_levels[i], "\" contains a single value and was removed from the analysis.\n")
        act <- "Stratum was removed from the analysis.\n"
        warn_df <- rbind(warn_df, data.frame(
          func = I(fname), subpoptype = NA,
          subpop = NA, indicator = NA, stratum = NA, warning = I(warn), action = I(act)
        ))
        dframe <- dframe[!tst, ]
        ind <- TRUE
      }
    }
    if (ind) {
      dframe[, stratumID] <- factor(dframe[, stratumID])
      stratum_levels <- levels(dframe[, stratumID])
      nstrata <- length(stratum_levels)
    }
  }

  # Assign a logical value to the indicator variable for a two-stage sample

  cluster_ind <- !is.null(clusterID)

  # Create the survey design object

  design <- survey_design(
    dframe, siteID, weight, stratum_ind, stratumID, cluster_ind, clusterID,
    weight1, sizeweight, sweight, sweight1, fpcfactor_ind, fpcsize, Ncluster,
    stage1size, vartype, NULL
  )

  # If popsize is not equal to NULL, then call either the postStratify or
  # calibrate function, as appropriate

  if (!is.null(popsize)) {
    if (all(class(popsize) %in% c("data.frame", "table", "xtabs"))) {
      if ("data.frame" %in% class(popsize)) {
        pnames <- names(popsize)[-ncol(popsize)]
      } else {
        pnames <- names(dimnames(popsize))
      }
      design <- postStratify(design, make.formula(pnames), popsize)
    } else {
      cnames <- cal_names(make.formula(names(popsize)), design)
      pop_totals <- numeric(length(cnames))
      names(pop_totals) <- cnames
      pop_totals[1] <- sum(popsize[[1]])
      k <- 2
      for (i in names(popsize)) {
        temp <- popsize[[i]]
        for (j in 2:length(temp)) {
          pop_totals[k] <- temp[j]
          k <- k + 1
        }
      }
      design <- calibrate(design, make.formula(names(popsize)), pop_totals)
    }
  }

  # If popsize is not equal to NULL and vartype equals "Local", then assign
  # adjusted weights to the appropriate weight variable(s) in the
  # design$variables data frame

  if (!is.null(popsize) && vartype == "Local") {
    if (cluster_ind) {
      design$variables$wgt2 <- weights(design) / design$variables$wgt1
    } else {
      design$variables$wgt <- weights(design)
    }
  }

  # For variables that exist in the design$variables data frame, assign survey
  # design variables

  dframe <- design$variables
  for (i in names(design_names)) {
    if (is.null(design_names[[i]])) {
      eval(parse(text = paste0(i, " <- NULL")))
    } else {
      eval(parse(text = paste0(i, " <- dframe[, \"", design_names[[i]], "\"]")))
    }
  }

  # Assign values to weight variables

  if (cluster_ind) {
    wgt1 <- dframe$wgt1
    wgt2 <- dframe$wgt2
  } else {
    wgt <- dframe$wgt
  }

  # Create the rrsum (results) data frame

  rrsum <- NULL

  # Assign the confidence bound multiplier

  mult <- qnorm(0.5 + (conf / 100) / 2)

  # Loop through all subpopulations (domains)

  for (itype in subpops) {
    lev_itype <- levels(dframe[, itype])
    nlev_itype <- length(lev_itype)

    # Loop through all response variables (vars_response)

    for (ivar_r in vars_response) {

      # Loop through all stressor variables (vars_stressor)

      for (ivar_s in vars_stressor) {

        # Loop through all levels of the subpopulation

        for (isubpop in lev_itype) {
          tst <- !is.na(dframe[, itype]) & dframe[, itype] == isubpop &
            !is.na(dframe[, ivar_r]) & !is.na(dframe[, ivar_s])

          # Assign response variable values

          response <- dframe[, ivar_r]
          nresp <- length(response)

          # Assign stressor variable values

          stressor <- dframe[, ivar_s]

          # Create the warn_vec object

          warn_vec <- c(itype, isubpop, paste(ivar_r, "and", ivar_s))

          #
          # Branch to handle stratified and unstratified data
          #

          if (stratum_ind) {

            #
            # Begin the section for stratified data
            #

            # Initialize variables for all strata combined

            wgt_total <- 0
            varest <- 0

            #
            # Begin the subsection for individual strata
            #

            for (i in 1:nstrata) {

              # Calculate required values

              stratum_i <- tst & stratumID == stratum_levels[i]
              response_st <- response[stratum_i]
              stressor_st <- stressor[stratum_i]
              if (cluster_ind) {
                wgt1_st <- wgt1[stratum_i]
                wgt2_st <- wgt2[stratum_i]
              } else {
                wgt_st <- wgt[stratum_i]
              }

              # Compute the 2x2 table of weight totals

              wgt_total_st <- svytable(make.formula(paste(
                ivar_r, "+",
                ivar_s
              )), design = subset(design, stratum_i))

              # Calculate the variance-covariance estimate for the cell and
              # marginal totals

              if (cluster_ind) {
                temp <- relrisk_var(
                  response[stratum_i], stressor[stratum_i],
                  response_levels[[ivar_r]], stressor_levels[[ivar_s]],
                  wgt2[stratum_i], xcoord[stratum_i], ycoord[stratum_i],
                  stratum_ind, stratum_levels[i], cluster_ind,
                  clusterID[stratum_i], wgt1[stratum_i], xcoord1[stratum_i],
                  ycoord1[stratum_i], vartype, warn_ind, warn_df, warn_vec
                )
              } else {
                temp <- relrisk_var(response[stratum_i], stressor[stratum_i],
                  response_levels[[ivar_r]], stressor_levels[[ivar_s]],
                  wgt[stratum_i], xcoord[stratum_i], ycoord[stratum_i],
                  stratum_ind, stratum_levels[i], cluster_ind,
                  vartype = vartype, warn_ind = warn_ind, warn_df = warn_df,
                  warn_vec = warn_vec
                )
              }
              varest_st <- temp$varest
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df

              # Add estimates to the variables for all strata combined

              wgt_total <- wgt_total + wgt_total_st
              varest <- varest + varest_st

              #
              # End the subsection for individual strata
              #
            }

            # Add margins to the wgt_total table

            wgt_total <- addmargins(wgt_total)

            # Assign cell and marginal weight totals

            total1 <- wgt_total[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][1]
            ]
            total2 <- wgt_total["Sum", stressor_levels[[ivar_s]][1]]
            total3 <- wgt_total[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][2]
            ]
            total4 <- wgt_total["Sum", stressor_levels[[ivar_s]][2]]

            # Calculate the estimate of relative risk for all strata combined

            if (total2 == 0 || total4 == 0) {
              rr <- NA
              rr_num <- NA
              rr_denom <- NA
              warn_ind <- TRUE
              temp <- ifelse(total2 == 0, stressor_levels[[ivar_s]][1],
                stressor_levels[[ivar_s]][2]
              )
              warn <- paste0("Since there are no observations for level \"", temp, "\" of the stressor \nvariable, the relative risk estimate and its standard error cannot be \ncalculated for stratum \"", stratum_levels[i], "\".  Also, the stratum \nwas removed from the analysis.\n")
              act <- paste0("The relative risk estimate and its standard error were not calculated for \nstratum \"", stratum_levels[i], "\".  Also, the stratum was removed from \nthe analysis.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if (total1 == 0 && total3 != 0) {
              rr <- 0
              rr_num <- 0
              rr_denom <- total3 / total4
              warn_ind <- TRUE
              warn <- paste0("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][1], "\" of the stressor \nvariable, the relative risk estimate is zero and standard error of the relative \nrisk estimate cannot be calculated for stratum \"", stratum_levels[i], "\".  \nAlso, the stratum was removed from the analysis.\n")
              act <- paste0("Standard error of the relative risk estimate was not calculated for stratum \n\"", stratum_levels[i], "\".  Also, the stratum was removed from the \nanalysis.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if (total1 == 0 && total3 == 0) {
              rr <- NA
              rr_num <- total1 / total2
              rr_denom <- total3 / total4
              warn_ind <- TRUE
              warn <- paste0("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][1], "\" of the stressor \nvariable and for the cell defined by level \"", response_levels[[ivar_r]][1], "\" of the \nresponse variable and level \"", stressor_levels[[ivar_s]][2], "\" of the stressor variable, \nthe relative risk estimate and its standard error cannot be calculated for \nstratum \"", stratum_levels[i], "\".  Also, the stratum was removed from \nthe analysis.\n")
              act <- paste0("The relative risk estimate and its standard error were not calculated for \nstratum \"", stratum_levels[i], "\".  Also, the stratum was removed from \nthe analysis.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if (total3 == 0) {
              rr <- NA
              rr_num <- total1 / total2
              rr_denom <- total3 / total4
              warn_ind <- TRUE
              warn <- paste0("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][2], "\" of the stressor \nvariable, the relative risk estimate and its standard error cannot be \ncalculated for stratum \"", stratum_levels[i], "\".  Also, the stratum \nwas removed from the analysis.\n")
              act <- paste0("The relative risk estimate and its standard error were not calculated for \nstratum \"", stratum_levels[i], "\".  Also, the stratum was removed from \nthe analysis.\n")
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else {
              rr <- (total1 * total4) / (total2 * total3)
              rr_num <- total1 / total2
              rr_denom <- total3 / total4
            }

            # Calculate the standard error estimate of the log of relative risk
            # for all strata combined

            if (any(c(total1, total2, total3, total4) == 0)) {
              rrlog_se <- NA
            } else {
              pder <- 1 / c(total1, -total2, -total3, total4)
              rrlog_se <- sqrt(t(pder) %*% varest %*% pder)
            }

            #
            # End the section for stratified data
            #
          } else {

            #
            # Begin the section for unstratified data
            #

            # Check whether the vector of response values contains a single
            # element

            if (nresp == 1) {
              stop("\nEstimates cannot be calculated since the vector of response values contains a \nsingle element.")
            }

            # Compute the 2x2 table of weight totals

            wgt_total <- addmargins(svytable(
              make.formula(paste(ivar_r, "+", ivar_s)),
              design = subset(design, tst)
            ))

            # Assign cell and marginal weight totals

            total1 <- wgt_total[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][1]
            ]
            total2 <- wgt_total["Sum", stressor_levels[[ivar_s]][1]]
            total3 <- wgt_total[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][2]
            ]
            total4 <- wgt_total["Sum", stressor_levels[[ivar_s]][2]]

            # Calculate the estimate of relative risk

            if (total2 == 0 || total4 == 0) {
              rr <- NA
              rr_num <- NA
              rr_denom <- NA
              warn_ind <- TRUE
              temp <- ifelse(total2 == 0, stressor_levels[[ivar_s]][1],
                stressor_levels[[ivar_s]][2]
              )
              warn <- paste0("Since there are no observations for level \"", temp, "\" of the stressor \nvariable, the relative risk estimate and its standard error cannot be \ncalculated.\n")
              act <- "The relative risk estimate and its standard error were not calculated.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if (total1 == 0 && total3 != 0) {
              rr <- 0
              rr_num <- 0
              rr_denom <- total3 / total4
              warn_ind <- TRUE
              warn <- paste0("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][1], "\" of the stressor variable, \nthe relative risk estimate is zero and standard error of the relative risk \nestimate cannot be calculated.\n")
              act <- "Standard error of the relative risk estimate was not calculated.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if (total1 == 0 && total3 == 0) {
              rr <- NA
              rr_num <- total1 / total2
              rr_denom <- total3 / total4
              warn_ind <- TRUE
              warn <- paste0("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][1], "\" of the stressor \nvariable and for the cell defined by level \"", response_levels[[ivar_r]][1], "\" of the \nresponse variable and level \"", stressor_levels[[ivar_s]][2], "\" of the stressor variable, \nthe relative risk estimate and its standard error cannot be calculated.\n")
              act <- "The relative risk estimate and its standard error were not calculated.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else if (total3 == 0) {
              rr <- NA
              rr_num <- total1 / total2
              rr_denom <- total3 / total4
              warn_ind <- TRUE
              warn <- paste0("Since there are no observations for the cell defined by level \"", response_levels[[ivar_r]][1], "\" \nof the response variable and level \"", stressor_levels[[ivar_s]][2], "\" of the stressor \nvariable, the relative risk estimate and its standard error cannot be \ncalculated.\n")
              act <- "The relative risk estimate and its standard error were not calculated.\n"
              warn_df <- rbind(warn_df, data.frame(
                func = I(fname),
                subpoptype = warn_vec[1], subpop = warn_vec[2],
                indicator = warn_vec[3], stratum = NA, warning = I(warn),
                action = I(act)
              ))
            } else {
              rr <- (total1 * total4) / (total2 * total3)
              rr_num <- total1 / total2
              rr_denom <- total3 / total4
            }

            # Determine whether the standard error can be calculated

            if (any(c(total1, total2, total3, total4) == 0)) {
              rrlog_se <- NA
            } else {

              # Calculate the variance-covariance estimate for the cell and
              # marginal totals

              if (cluster_ind) {
                temp <- relrisk_var(
                  response[tst], stressor[tst],
                  response_levels[[ivar_r]], stressor_levels[[ivar_s]],
                  wgt2[tst], xcoord[tst], ycoord[tst], stratum_ind, NULL,
                  cluster_ind, clusterID[tst], wgt1[tst], xcoord1[tst],
                  ycoord1[tst], vartype, warn_ind, warn_df, warn_vec
                )
              } else {
                temp <- relrisk_var(response[tst], stressor[tst],
                  response_levels[[ivar_r]], stressor_levels[[ivar_s]],
                  wgt[tst], xcoord[tst], ycoord[tst], stratum_ind, NULL,
                  cluster_ind,
                  vartype = vartype, warn_ind = warn_ind,
                  warn_df = warn_df, warn_vec = warn_vec
                )
              }
              varest <- temp$varest
              warn_ind <- temp$warn_ind
              warn_df <- temp$warn_df

              # Calculate the standard error estimate of the log of relative
              # risk

              pder <- 1 / c(total1, -total2, -total3, total4)
              rrlog_se <- sqrt(t(pder) %*% varest %*% pder)
            }

            #
            # End section for unstratified data
            #
          }

          # Calculate confidence limits for the estimate of relative risk

          if (is.na(rrlog_se)) {
            cl <- NA
          } else {
            cl <- c(exp(log(rr) - rrlog_se * mult), exp(log(rr) + rrlog_se *
              mult))
          }

          # Calculate the table of cell and margin counts

          cc <- addmargins(table(list(
            response = response[tst],
            stressor = stressor[tst]
          )))

          # Calculate the table of cell and margin proportion estimates

          cp <- wgt_total / wgt_total["Sum", "Sum"]

          # Append results to the rrsum data frame

          rrsum <- rbind(rrsum, data.frame(
            Type = itype,
            Subpopulation = isubpop,
            Response = ivar_r,
            Stressor = ivar_s,
            nResp = cc[3, 3],
            Estimate = rr,
            Estimate_num = rr_num,
            Estimate_denom = rr_denom,
            StdError_log = rrlog_se,
            MarginofError_log = mult * rrlog_se,
            LCB = cl[1],
            UCB = cl[2],
            WeightTotal = wgt_total["Sum", "Sum"],
            CellCounts_11 = cc[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][1]
            ],
            CellCounts_12 = cc[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][2]
            ],
            CellCounts_21 = cc[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][1]
            ],
            CellCounts_22 = cc[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][2]
            ],
            CellProportions_11 = cp[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][1]
            ],
            CellProportions_12 = cp[
              response_levels[[ivar_r]][1],
              stressor_levels[[ivar_s]][2]
            ],
            CellProportions_21 = cp[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][1]
            ],
            CellProportions_22 = cp[
              response_levels[[ivar_r]][2],
              stressor_levels[[ivar_s]][2]
            ]
          ))

          # End of the loop for levels of the subpopulation
        }

        # End of the loop for stressor variables
      }

      # End of the loop for response variables
    }

    # End of the loop for subpopulations
  }

  # As necessary, output a message indicating that warning messages were
  # generated during execution of the program

  if (warn_ind) {
    warn_df <<- warn_df
    if (nrow(warn_df) == 1) {
      message("During execution of the program, a warning message was generated.  The warning \nmessage is stored in a data frame named 'warn_df'.  Enter the following command \nto view the warning message: warnprnt()\n")
    } else {
      message(paste("During execution of the program,", nrow(warn_df), "warning messages were generated.  The warning \nmessages are stored in a data frame named 'warn_df'.  Enter the following \ncommand to view the warning messages: warnprnt() \nTo view a subset of the warning messages (say, messages number 1, 3, and 5), \nenter the following command: warnprnt(m=c(1,3,5))\n"))
    }
  }

  # Assign dimension names to the rrsum data frame

  cell_lbl <- character(4)
  cell_lbl[1] <- "RespPoor_StressPoor"
  cell_lbl[2] <- "RespPoor_StressGood"
  cell_lbl[3] <- "RespGood_StressPoor"
  cell_lbl[4] <- "RespGood_StressGood"
  dimnames(rrsum) <- list(1:nrow(rrsum), c(
    "Type", "Subpopulation", "Response", "Stressor", "nResp", "Estimate",
    "Estimate_num", "Estimate_denom", "StdError_log", "MarginofError_log",
    paste0("LCB", conf, "Pct"), paste0("UCB", conf, "Pct"), "WeightTotal",
    paste0("Count_", cell_lbl[1]), paste0("Count_", cell_lbl[2]),
    paste0("Count_", cell_lbl[3]), paste0("Count_", cell_lbl[4]),
    paste0("Prop_", cell_lbl[1]), paste0("Prop_", cell_lbl[2]),
    paste0("Prop_", cell_lbl[3]), paste0("Prop_", cell_lbl[4])
  ))

  # Return the rrsum data frame

  rrsum
}

context("risk difference analysis")

#
# Section for data input
#

# Read the data frame containing survey design and analysis variables
load(system.file("extdata", "NLA_IN.rda", package = "spsurvey"))

# Create a population size data frame
popsize <- data.frame(
  LAKE_ORGN = c("MAN_MADE", "NATURAL"),
  Total = c(6000, 14000)
)

# Create finite population correction factor objects
fpc1 <- 20000
fpc2a <- list(
  Urban = 5000,
  "Non-Urban" = 15000
)
fpc2b <- list(
  MAN_MADE = 6000,
  NATURAL = 14000
)
fpc3 <- c(
  Ncluster = 200,
  clusterID_1 = 100,
  clusterID_2 = 100,
  clusterID_3 = 100,
  clusterID_4 = 100
)
fpc4a <- list(
  Urban = c(
    Ncluster = 75,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  "Non-Urban" = c(
    Ncluster = 125,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)
fpc4b <- list(
  NATURAL = c(
    Ncluster = 130,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  MAN_MADE = c(
    Ncluster = 70,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)

#
# Section for the risk difference data analysis function
#

# Assign response variable names and stressor variable names to the
# vars_response and  vars_stressor vectors, respectively
vars_response <- c("BENT_MMI_COND_2017")
vars_stressor <- c("PTL_COND", "NTL_COND")

# Assign subpopulation variable names to the subpops vector
subpops <- c("All_Sites", "LAKE_ORGN")

# Perform tests
DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD"
)

test_that("Risk Difference: Unstratified single-stage analysis", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD",
  response_levels = list("BENT_MMI_COND_2017" = c("Poor", "Good")),
  stressor_levels = list("PTL_COND" = c("Poor", "Good"), "NTL_COND" = c("Poor", "Good"))
)

test_that("Risk Difference: Unstratified single-stage analysis (specify response/stressor levels)", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD",
  response_levels = list(c("Poor", "Good")),
  stressor_levels = list(c("Poor", "Good"), c("Poor", "Good"))
)

test_that("Risk Difference: Unstratified single-stage analysis (unnamed response/stressor levels)", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", popsize = popsize
)

test_that("Risk Difference: with known population sizes", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", fpc = fpc1
)

test_that("Risk Difference: with finite population correction factor", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17"
)

test_that("Risk Difference: Stratified single-stage analysis", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", popsize = popsize
)

test_that("Risk Difference: with known population sizes", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", fpc = fpc2a
)

test_that("Risk Difference: with finite population correction factor", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", vartype = "SRS"
)

test_that("Risk Difference: Unstratified two-stage analysis", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", popsize = popsize
)

test_that("Risk Difference: with known population sizes", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", fpc = fpc3, vartype = "SRS"
)

test_that("Risk Difference: with finite population correction factor", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", clusterID = "clusterID",
  weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
  vartype = "SRS"
)

test_that("Risk Difference: Stratified two-stage analysis", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", clusterID = "clusterID",
  weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
  popsize = popsize, vartype = "SRS"
)

test_that("Risk Difference: with known population sizes", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

DiffRisk_Estimates <- diffrisk_analysis(
  dframe = NLA_IN,
  vars_response = vars_response, vars_stressor = vars_stressor,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", clusterID = "clusterID",
  weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
  fpc = fpc4a, vartype = "SRS"
)

test_that("Risk Difference: with finite population correction factor", {
  expect_true(exists("DiffRisk_Estimates"))
  expect_equal(attributes(DiffRisk_Estimates)$class, "data.frame")
  expect_equal(nrow(DiffRisk_Estimates), 6)
})

test_that("A warning (in message form) is produced", {
  expect_message(expect_error(diffrisk_analysis(
    dframe = NLA_IN,
    vars_response = vars_response, vars_stressor = vars_stressor,
    subpops = subpops, siteID = "SITE_ID", weight = "XYZ", xcoord = "XCOORD",
    ycoord = "YCOORD", stratumID = "URBN_NLA17", clusterID = "clusterID",
    weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
    fpc = fpc4a, vartype = "SRS"
  )))
})

context("categorical variable analysis")

#
# Section for data input
#

# Read the data frame containing survey design and analysis variables
load(system.file("extdata", "NLA_IN.rda", package = "spsurvey"))

# Create a population size data frame
popsize <- data.frame(
  LAKE_ORGN = c("MAN_MADE", "NATURAL"),
  Total = c(6000, 14000)
)

# Create finite population correction factor objects
fpc1 <- 20000
fpc2a <- list(
  Urban = 5000,
  "Non-Urban" = 15000
)
fpc2b <- list(
  MAN_MADE = 6000,
  NATURAL = 14000
)
fpc3 <- c(
  Ncluster = 200,
  clusterID_1 = 100,
  clusterID_2 = 100,
  clusterID_3 = 100,
  clusterID_4 = 100
)
fpc4a <- list(
  Urban = c(
    Ncluster = 75,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  "Non-Urban" = c(
    Ncluster = 125,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)
fpc4b <- list(
  NATURAL = c(
    Ncluster = 130,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  ),
  MAN_MADE = c(
    Ncluster = 70,
    clusterID_1 = 50,
    clusterID_2 = 50,
    clusterID_3 = 50,
    clusterID_4 = 50
  )
)

#
# Section for the categorical data analysis function
#

# Assign response variable names to the vars vector
vars <- c("BENT_MMI_COND_2017")

# Assign subpopulation variable names to the subpops vector
subpops <- c("All_Sites", "LAKE_ORGN")

# Perform tests

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD"
)

test_that("Categorical: Unstratified single-stage analysis, local mean variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", popsize = popsize
)

test_that("Categorical: with known population sizes", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", fpc = fpc1
)

test_that("Categorical: with finite population correction factor", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17"
)

test_that("Categorical: Stratified single-stage analysis, local mean variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", popsize = popsize
)

test_that("Categorical: with known population sizes", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", fpc = fpc2a
)

test_that("Categorical: with finite population correction factor", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", vartype = "SRS"
)

test_that("Categorical: Unstratified two-stage analysis, SRS variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", popsize = popsize,
  vartype = "SRS"
)

test_that("Categorical: with known population sizes", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", clusterID = "clusterID", weight1 = "weight1",
  xcoord1 = "xcoord1", ycoord1 = "ycoord1", fpc = fpc3, vartype = "SRS"
)

test_that("Categorical: with finite population correction factor", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", clusterID = "clusterID",
  weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
  vartype = "SRS"
)

test_that("Categorical: Stratified two-stage analysis, SRS variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", clusterID = "clusterID",
  weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
  popsize = popsize, vartype = "SRS"
)

test_that("Categorical: with known population sizes", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", xcoord = "XCOORD",
  ycoord = "YCOORD", stratumID = "URBN_NLA17", clusterID = "clusterID",
  weight1 = "weight1", xcoord1 = "xcoord1", ycoord1 = "ycoord1",
  fpc = fpc4a, vartype = "SRS"
)

test_that("Categorical: with finite population correction factor", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", vartype = "HT"
)

test_that("Categorical: Unstratified single-stage analysis, HT-Overton variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", vartype = "HT",
  jointprob = "hr"
)

test_that("Categorical: Unstratified single-stage analysis, HT-HR variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", vartype = "HT",
  jointprob = "brewer"
)

test_that("Categorical: Unstratified single-stage analysis, HT-Brewer variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP",
  stratumID = "URBN_NLA17", vartype = "HT", jointprob = "brewer"
)

test_that("Categorical: Stratified single-stage analysis, HT-Brewer variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP",
  clusterID = "clusterID", weight1 = "weight1", vartype = "HT",
  jointprob = "brewer"
)

test_that("Categorical: Unstratified two-stage analysis, HT-Brewer variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP",
  stratumID = "URBN_NLA17", clusterID = "clusterID", weight1 = "weight1",
  vartype = "HT", jointprob = "brewer"
)

test_that("Categorical: Stratified two-stage analysis, HT-Brewer variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", vartype = "YG"
)

test_that("Categorical: Unstratified single-stage analysis, YG-Overton variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

Condition_Estimates <- cat_analysis(
  dframe = NLA_IN, vars = vars,
  subpops = subpops, siteID = "SITE_ID", weight = "WGT_TP", vartype = "YG",
  jointprob = "hr"
)

test_that("Categorical: Unstratified single-stage analysis, YG-HR variance", {
  expect_true(exists("Condition_Estimates"))
  expect_equal(attributes(Condition_Estimates)$class, "data.frame")
  expect_equal(nrow(Condition_Estimates), 9)
})

test_that("A warning (in message form) is produced", {
  expect_message(expect_error(cat_analysis(
    dframe = NLA_IN, vars = vars,
    subpops = subpops, siteID = "SITE_ID", weight = "XYZ", vartype = "YG",
    jointprob = "hr"
  )))
})

context("Test that package data has the appropriate structure")

data("NE_Lakes")
test_that("NE_Lakes (point design frame) is an sf object with the correct geometry type", {
  expect_s3_class(NE_Lakes, c("sf", "data.frame"), exact = TRUE)
  expect_s3_class(NE_Lakes$geometry, c("sfc_POINT", "sfc"), exact = TRUE)
  expect_equal(st_crs(NE_Lakes)$input, "EPSG:5070")
})

data("Illinois_River")
test_that("Illinois_River (linear design frame) is an sf object with the correct geometry type", {
  expect_s3_class(Illinois_River, c("sf", "data.frame"), exact = TRUE)
  expect_s3_class(Illinois_River$geometry, c("sfc_MULTILINESTRING", "sfc"), exact = TRUE)
  expect_equal(st_crs(Illinois_River)$input, "EPSG:5070")
})

data("Illinois_River_Legacy")
test_that("Illinois_River_Legacy (linear legacy design frame) is an sf object with the correct geometry type", {
  expect_s3_class(Illinois_River_Legacy, c("sf", "data.frame"), exact = TRUE)
  expect_s3_class(Illinois_River_Legacy$geometry, c("sfc_POINT", "sfc"), exact = TRUE)
  expect_equal(st_crs(Illinois_River_Legacy)$input, "EPSG:5070")
})

data("Lake_Ontario")
test_that("Lake Ontario (point design frame) is an sf object with the correct geometry type", {
  expect_s3_class(Lake_Ontario, c("sf", "data.frame"), exact = TRUE)
  expect_s3_class(Lake_Ontario$geometry, c("sfc_MULTIPOLYGON", "sfc"), exact = TRUE)
  expect_equal(st_crs(Lake_Ontario)$input, "EPSG:5070")
})

data("NLA_PNW")
test_that("NLA_PNW (point analysis frame) is an sf object with the correct geometry type", {
  expect_s3_class(NLA_PNW, c("sf", "data.frame"), exact = TRUE)
  expect_s3_class(NLA_PNW$geometry, c("sfc_POINT", "sfc"), exact = TRUE)
  expect_equal(st_crs(NLA_PNW)$input, "EPSG:5070")
})

data("NRSA_EPA7")
test_that("NRSA_EPA7 (linear design frame) is an sf object with the correct geometry type", {
  expect_s3_class(NRSA_EPA7, c("sf", "data.frame"), exact = TRUE)
  expect_s3_class(NRSA_EPA7$geometry, c("sfc_POINT", "sfc"), exact = TRUE)
  expect_equal(st_crs(NRSA_EPA7)$input, "EPSG:5070")
})

---
title: "Start Here"
author: "Michael Dumelle, Tom Kincaid, Anthony R. Olsen, and Marc Weber"
output: 
  html_document:
    theme: flatly
    number_sections: true
    highlighted: default 
    toc: yes
    toc_float:
      collapsed: no
      smooth_scroll: no
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Start Here}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>", 
  warning = FALSE, 
  message = FALSE
)
```

# Installing and loading `spsurvey`

If this is your first time using the `spsurvey` package, run
```{r, eval = FALSE}
install.packages("spsurvey")
```
to install the package. You only need to run this code once per version of R.

After the `spsurvey` package is installed, load it into R each new R session by running
```{r setup}
library(spsurvey)
```

# Citation information

If you used `spsurvey` in your work, please cite it. You can view the most recent citation by running
```{r}
citation("spsurvey")
```

# `spsurvey` terminology

`spsurvey` implements a design-based approach to statistical inference, with a focus on spatial data. There are a few terms helpful to define before we move forward with `spsurvey`, as these terms will be used throughout the vignettes and documentation:

* Survey design: All aspects of a survey from establishment of a need for data to the production of final results.
* Sampling frame: The set of all sites (i.e. units) from which a sample is selected.
* Sampling design: The process by which sites are selected from the sampling frame.
* Design sites: The set of sites selected from the sampling frame according to the sampling design. Sometimes the term "sample" is used to refer to a collection of design sites (e.g. the term "GRTS sample" more formally means "the collection of design sites selected using the GRTS algorithm according to the sampling design").
* Analysis data: Data collected at the design sites. This includes information from the survey design such as design weights, stratification variables, subpopulation variables, etc.
* Analysis results: The results from an appropriate statistical analysis of the analysis data.

# Vignettes in `spsurvey`

There are five additional vignettes in ``spsurvey`` that are available on our website [here](https://usepa.github.io/spsurvey/) under the "Articles" tab:

1. An Overview of Statistical Surveys for Ecological Applications
2. Exploratory Data Analysis: Summarizing and Visualizing Sampling Frames, Design Sites, and Analysis Data
    * Use the `summary()` and `plot()` functions to summarize and visualize sampling frames, design sites, and analysis data
3. Spatially Balanced Sampling
    * Use the `grts()` function to implement the Generalized Random Tessellation Stratified (GRTS) algorithm (Stevens and Olsen, 2004) to select spatially balanced samples
4. Weight Adjustments in Practice
    * Use the `adjwgt()` and `adjwgtNR()` functions to adjust weights for changing sample sizes and nonresponse, respectively.
5. Analyzing Data
    * Use several analysis functions to analyze data

These vignettes cover some of the core functions (and arguments within those functions) in `spsurvey`. To learn more about features of `spsurvey` that are not covered in these vignettes, we encourage you to read `spsurvey`'s documentation available for download [here](https://cran.r-project.org/package=spsurvey) or viewable interactively on our website [here](https://usepa.github.io/spsurvey/). Help files for a particular function are viewable by running `?function_name` after loading `spsurvey`. For example, to learn more about the `grts()` function, run `?grts`.

# Installing a previous version of `spsurvey`

The version 5.0.0 update to `spsurvey` implemented many significant changes to existing functions. As a result, some of your old code may not run properly while using version 5.0.0. Though we recommend adapting your code to work with the version 5.0.0 update (and later), you may also install a previous version of `spsurvey`. For information regarding the installation of previous version of R packages, please see the RStudio support page [here](https://support.posit.co/hc/en-us/articles/219949047-Installing-older-versions-of-packages). Additionally, old versions of `spsurvey` are also available for download in the release tags section of our GitHub repository [here](https://github.com/USEPA/spsurvey/releases).

# `sf` objects

The sampling functions in `spsurvey` (`grts()` and `irs()`) require that your sampling frame is an `sf` object. An `sf` object (shorthand for a "simple features" object) is an R object with a unique structure used to conveniently store spatial data. `sf` objects are constructed using the sf package (Pebesma, 2018). The sf package is loaded and installed alongside the `spsurvey` package, so you do not need to run `install.packages("sf")` or `library(sf)` to access the sf package if `spsurvey` is already installed and loaded. For more on the sf package, see [here](https://cran.r-project.org/package=sf).

Next we discuss a few ways to construct `sf` objects in R. The first is to read a shapefile directly into R using `sf::read_sf()`. The second is to use the `sf::st_sf()` function or the `sf::st_as_sf()` function to combine an appropriate R object (most commonly a data frame) and an appropriate geometry object into an `sf` object. To illustrate one approach for turning a data frame into an `sf` object, we start with `NE_Lakes_df`, a data frame in `spsurvey` that contains variables and geographic coordinates (latitude and longitude coordinates) for lakes in the Northeastern United States. To turn `NE_Lakes_df` into `NE_Lakes_geo`, an `sf` object with geographic coordinates, run
```{r}
NE_Lakes_geo <- st_as_sf(NE_Lakes_df, coords = c("XCOORD", "YCOORD"), crs = 4326)
NE_Lakes_geo
```

The `coords` argument to `sf::st_as_sf` specifies the columns in `NE_Lakes_df` that are the x-coordinates and y-coordinates. The `crs` argument specifies the coordinate reference system, which we discuss in more detail next.


# Coordinate reference systems

Spatial data and `sf` objects rely on coordinate reference systems. A coordinate reference system (CRS) provides a structure by which to identify unique locations on the Earth's surface. CRSs are either geographic or projected. A geographic CRS uses longitude (east-west direction) and latitude (north-south direction) coordinates to represent location with respect to a specific ellipsoid or spheroid surface. Geographic CRSs are measured in degrees, *not units like meters or feet* -- this has important consequences. For example, a one degree difference in latitude is different at different longitudes. Projected CRSs are measured in standard Cartesian coordinates with respect to a flat surface. They have x and y locations, an origin, and a unit of measurement (like meters or feet).

You can move between coordinate systems using `sf::st_transform()`. For example, we can transform `NE_Lakes_geo` (which uses a geographic CRS) to `NE_Lakes` (which uses a projected CRS) by running
```{r}
NE_Lakes <- st_transform(NE_Lakes_geo, crs = 5070)
NE_Lakes
```

CRSs in R have traditionally been stored using EPSG codes or `proj4string` values. This meant that in order to transform your coordinates from one CRS to another, you needed two EPSG codes or `proj4string` values, one for each CRS. Recent updates to R's handling of spatial data follow GDAL and PROJ (more information available [here](https://r-spatial.org/r/2020/03/17/wkt.html)), and CRSs in `sf` objects are stored in R as lists with two components: `input`, which contains information regarding the EPSG code and `proj4string`; and `wkt`, an open geospatial standard format. For more information on CRSs and EPSG codes, see Pebesma (2018) and Lovelace et al. (2019). To search for various CRSs and EPSG codes, see [here](https://epsg.io/) and [here](https://spatialreference.org).

`spsurvey` will use the CRS from your `sf` object, so it is your responsibility to make sure the `sf` object has an appropriate CRS. If the CRS is not specified correctly, you may get misleading results.

# References

Lovelace, R., Nowosad, J., & Muenchow, J. (2019). Geocomputation with R. CRC Press.

Pebesma, E., (2018). Simple Features for R: Standardized Support for Spatial Vector Data. *The R Journal*, 10 (1):439-446. https://doi.org/10.32614/RJ-2018-009

Stevens Jr, D. L. and Olsen, A. R. (2004). Spatially balanced sampling of natural resources. *Journal of the American Statistical Association*, 99(465):262-278.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trend_analysis.R
\name{trend_analysis}
\alias{trend_analysis}
\title{Trend analysis}
\usage{
trend_analysis(
  dframe,
  vars_cat = NULL,
  vars_cont = NULL,
  subpops = NULL,
  model_cat = "SLR",
  cat_rhs = NULL,
  model_cont = "LMM",
  cont_rhs = NULL,
  siteID = "siteID",
  yearID = "year",
  weight = "weight",
  xcoord = NULL,
  ycoord = NULL,
  stratumID = NULL,
  clusterID = NULL,
  weight1 = NULL,
  xcoord1 = NULL,
  ycoord1 = NULL,
  sizeweight = FALSE,
  sweight = NULL,
  sweight1 = NULL,
  fpc = NULL,
  popsize = NULL,
  invprboot = TRUE,
  nboot = 1000,
  vartype = "Local",
  jointprob = "overton",
  conf = 95,
  All_Sites = FALSE
)
}
\arguments{
\item{dframe}{Data to be analyzed (analysis data). A data frame or
\code{sf} object containing survey design variables, response
variables, and subpopulation (domain) variables.}

\item{vars_cat}{Vector composed of character values that identify the names
of categorical response variables in \code{dframe}. If
argument \code{model_cat} equals "GLMM", the categorical variables in the
\code{dframe} data frame must be factors each of which has two levels,
where the second level will be assumed to specify "success".  The default
value is \code{NULL}.}

\item{vars_cont}{Vector composed of character values that identify the
names of continuous response variables in \code{dframe}.
The default value is \code{NULL}.}

\item{subpops}{Vector composed of character values that identify the
names of subpopulation (domain) variables in \code{dframe}.
If a value is not provided, the value \code{"All_Sites"} is assigned to the
subpops argument and a factor variable named \code{"All_Sites"} that takes
the value \code{"All Sites"} is added to \code{dframe}.  The
default value is \code{NULL}.}

\item{model_cat}{Character value identifying the analytical procedure used
for trend estimation for categorical variables.  The choices are:
\code{"SLR"} (simple linear regression), \code{"WLR"} (weighted linear
regression), and \code{"GLMM"} (generalized linear mixed-effects model).
The default value is \code{"SLR"}.}

\item{cat_rhs}{Character value specifying the right hand side of the formula
for a generalized linear mixed-effects model.  If a value is not provided,
the argument is assigned a value that specifies the Piepho and Ogutu (2002)
model.  The default value is \code{NULL}.}

\item{model_cont}{Character value identifying the analytical procedure used
for trend estimation for continuous variables.  The choices are:
\code{"SLR"} (simple linear regression), \code{"WLR"} (weighted linear
regression), and \code{"LMM"} (linear mixed-effects model).  The default
value is \code{"LMM"}.}

\item{cont_rhs}{Character value specifying the right hand side of the
formula for a linear mixed-effects model.  If a value is not provided, the
argument is assigned a value that specifies the Piepho and Ogutu (2002)
model.  The default value is \code{NULL}.}

\item{siteID}{Character value providing name of the site ID variable in
\code{dframe}.  If repeat visit sites are present, the site
ID value for each revisit site will be the same for each survey.  For a
two-stage sample, the site ID variable identifies stage two site IDs.  The
default value is \code{"siteID"}.}

\item{yearID}{Character value providing name of the time period variable in
\code{dframe}, which must be numeric and will be forced to
numeric if it is not.  The default assumption is that the time period
variable is years.  The default value is \code{"year"}.}

\item{weight}{Character value providing name of the design weight
variable in \code{dframe}.  For a two-stage sample, the
weight variable identifies stage two weights.  The default value is
\code{"weight"}.}

\item{xcoord}{Character value providing name of the x-coordinate variable in
\code{dframe}.  For a two-stage sample, the x-coordinate
variable identifies stage two x-coordinates.  Note that x-coordinates are
required  for calculation of the local mean variance estimator.  If \code{dframe}
is an \code{sf} object, this argument is not required (as the geometry column
in \code{dframe} is used to find the x-coordinate). The
default value is \code{NULL}.}

\item{ycoord}{Character value providing name of the y-coordinate variable in
\code{dframe}.  For a two-stage sample, the y-coordinate
variable identifies stage two y-coordinates.  Note that y-coordinates are
required for calculation of the local mean variance estimator.  If \code{dframe}
is an \code{sf} object, this argument is not required (as the geometry column
in \code{dframe} is used to find the y-coordinate). The default
value is \code{NULL}.}

\item{stratumID}{Character value providing name of the stratum ID variable in
\code{dframe}.  The default value is \code{NULL}.}

\item{clusterID}{Character value providing name of the cluster (stage one) ID
variable in \code{dframe}.  Note that cluster IDs are
required for a two-stage sample.  The default value is \code{NULL}.}

\item{weight1}{Character value providing name of the stage one weight
variable in \code{dframe}.  The default value is \code{NULL}.}

\item{xcoord1}{Character value providing name of the stage one x-coordinate
variable in \code{dframe}.  Note that x-coordinates are
required for calculation of the local mean variance estimator.  The default
value is \code{NULL}.}

\item{ycoord1}{Character value providing name of the stage one y-coordinate
variable in \code{dframe}.  Note that y-coordinates are
required for calculation of the local mean variance estimator.  The default
value is \code{NULL}.}

\item{sizeweight}{Logical value that indicates whether size weights should be
used during estimation, where \code{TRUE} = use size weights and
\code{FALSE} = do not use size weights. To employ size weights for a
single-stage sample, a value must be supplied for argument weight.  To
employ size weights for a two-stage sample, values must be supplied for
arguments \code{weight} and \code{weight1}. The default value is
\code{FALSE}.}

\item{sweight}{Character value providing name of the size weight variable in
\code{dframe}.  For a two-stage sample, the size weight
variable identifies stage two size weights.  The default value is
\code{NULL}.}

\item{sweight1}{Character value providing name of the stage one size weight
variable in \code{dframe}.  The default value is
\code{NULL}.}

\item{fpc}{Object that specifies values required for calculation of the
  finite population correction factor used during variance estimation. The
  object must match the survey design in terms of stratification and whether
  the design is single-stage or two-stage.  For an unstratified design, the
  object is a vector.  The vector is composed of a single numeric value for a
  single-stage design.  For a two-stage unstratified design, the object is a
  named vector containing one more than the number of clusters in the sample,
  where the first item in the vector specifies the number of clusters in the
  population and each subsequent item specifies the number of stage two units
  for the cluster.  The name for the first item in the vector is arbitrary.
  Subsequent names in the vector identify clusters and must match the cluster
  IDs.  For a stratified design, the object is a named list of vectors, where
  names must match the strata IDs.  For each stratum, the format of the
  vector is identical to the format described for unstratified single-stage
  and two-stage designs.  Note that the finite population correction factor
  is not used with the local mean variance estimator.

  Example fpc for a single-stage unstratified survey design:

  \verb{fpc <- 15000}

  Example fpc for a single-stage stratified survey design:

  \verb{fpc <- list(
    Stratum_1 = 9000,
    Stratum_2 = 6000)
   }

  Example fpc for a two-stage unstratified survey design:

  \verb{fpc <- c(
    Ncluster = 150,
    Cluster_1 = 150,
    Cluster_2 = 75,
    Cluster_3 = 75,
    Cluster_4 = 125,
    Cluster_5 = 75)
  }

  Example fpc for a two-stage stratified survey design:

  \verb{fpc <- list(
    Stratum_1 = c(
      Ncluster_1 = 100,
      Cluster_1 = 125,
      Cluster_2 = 100,
      Cluster_3 = 100,
      Cluster_4 = 125,
      Cluster_5 = 50),
    Stratum_2 = c(
      Ncluster_2 = 50,
      Cluster_1 = 75,
      Cluster_2 = 150,
      Cluster_3 = 75,
      Cluster_4 = 75,
      Cluster_5 = 125))
  }}

\item{popsize}{Object that provides values for the population argument of the
  \code{calibrate} or \code{postStratify} functions in the survey package. If
  a value is provided for popsize, then either the \code{calibrate} or
  \code{postStratify} function is used to modify the survey design object
  that is required by functions in the survey package.  Whether to use the
  \code{calibrate} or \code{postStratify} function is dictated by the format
  of popsize, which is discussed below.  Post-stratification adjusts the
  sampling and replicate weights so that the joint distribution of a set of
  post-stratifying variables matches the known population joint distribution.
  Calibration, generalized raking, or GREG estimators generalize
  post-stratification and raking by calibrating a sample to the marginal
  totals of variables in a linear regression model. For the \code{calibrate}
  function, the object is a named list, where the names identify factor
  variables in \code{dframe}.  Each element of the list is a
  named vector containing the population total for each level of the
  associated factor variable.  For the \code{postStratify} function, the
  object is either a data frame, table, or xtabs object that provides the
  population total for all combinations of selected factor variables in the
  \code{dframe} data frame.  If a data frame is used for \code{popsize}, the
  variable containing population totals must be the last variable in the data
  frame.  If a table is used for \code{popsize}, the table must have named
  \code{dimnames} where the names identify factor variables in the
  \code{dframe} data frame.  If the popsize argument is equal to \code{NULL},
  then neither calibration nor post-stratification is performed.  The default
  value is \code{NULL}.

  Example popsize for calibration:

  \verb{popsize <- list(
    Ecoregion = c(
      East = 750,
      Central = 500,
      West = 250),
    Type = c(
      Streams = 1150,
      Rivers = 350))
  }

  Example popsize for post-stratification using a data frame:

  \verb{popsize <- data.frame(
    Ecoregion = rep(c("East", "Central", "West"),
      rep(2, 3)),
    Type = rep(c("Streams", "Rivers"), 3),
    Total = c(575, 175, 400, 100, 175, 75))
  }

  Example popsize for post-stratification using a table:

  \verb{popsize <- with(MySurveyFrame,
    table(Ecoregion, Type))}

  Example popsize for post-stratification using an xtabs object:

  \verb{popsize <- xtabs(~Ecoregion + Type,
    data = MySurveyFrame)}}

\item{invprboot}{Logical value that indicates whether the inverse probability
bootstrap procedure is used to calculate trend parameter estimates.  This
bootstrap procedure is only available for the "LMM" option for continuous
variables.  Inverse probability references the design weights, which
are the inverse of the sample inclusion probabilities.  The default value
is \code{TRUE}.}

\item{nboot}{Numeric value for the number of bootstrap iterations.  The
default is \code{1000}.}

\item{vartype}{Character value providing choice of the variance estimator,
where \code{"Local"} = the local mean estimator, \code{"SRS"} = the simple
random sampling estimator, \code{"HT"} = the Horvitz-Thompson estimator,
and \code{"YG"} = the Yates-Grundy estimator.  The default value is
\code{"Local"}.}

\item{jointprob}{Character value providing choice of joint inclusion
probability approximation for use with Horvitz-Thompson and Yates-Grundy
variance estimators, where \code{"overton"} indicates the Overton
approximation, \code{"hr"} indicates the Hartley_Rao approximation, and
\code{"brewer"} equals the Brewer approximation.  The default value is
\code{"overton"}.}

\item{conf}{Numeric value for the Gaussian-based confidence level.  The default is
\code{95}.}

\item{All_Sites}{A logical variable used when \code{subpops} is not
\code{NULL}. If \code{All_Sites} is \code{TRUE}, then alongside the
subpopulation output, output for all sites (ignoring subpopulations) is
returned for each variable in \code{vars}. If \code{All_Sites} is
\code{FALSE}, then alongside the subpopulation output, output for all sites
(ignoring subpopulations) is not returned for each variable in \code{vars}.
The default is \code{FALSE}.}
}
\value{
The analysis results. A list composed of two data frames containing trend estimates for all
  combinations of population Types, subpopulations within Types, and response
  variables.  For categorical variables, trend estimates are calculated for
  each category of the variable.  The two data frames in the output list are:
  \describe{
    \item{\code{catsum}}{data frame containing trend estimates for
      categorical variables}
    \item{\code{contsum}}{data frame containing trend
      estimates for continuous variables}
   }

  For the SLR and WLR model options, the data frame contains the following
  variables:
  \describe{
    \item{Type}{subpopulation (domain) name}
    \item{Subpopulation}{subpopulation name within a domain}
    \item{Indicator}{response variable}
    \item{Trend_Estimate}{trend estimate}
    \item{Trend_Std_Error}{trend standard error}
    \item{Trend_LCBxxPct}{trend xx\% (default 95\%) lower confidence bound}
    \item{Trend_UCBxxPct}{trend xx\% (default 95\%) upper confidence bound}
    \item{Trend_p_Value}{trend p-value}
    \item{Intercept_Estimate}{intercept estimate}
    \item{Intercept_Std_Error}{intercept standard error}
    \item{Intercept_LCBxxPct}{intercept xx\% (default 95\%) lower confidence
      bound}
    \item{Intercept_UCBxxPct}{intercept xx\% (default 95\%) upper confidence
      bound}
    \item{Intercept_p_Value}{intercept p-value}
    \item{R_Squared}{R-squared value}
    \item{Adj_R_Squared}{adjusted R-squared value}
  }

  For the GLMM and LMM model options, contents of the data frames will vary
  depending on the model specified by arguments \code{cat_rhs} and
  \code{cont_rhs}.  For the default PO model, the data frame contains the
  following variables:
  \describe{
    \item{Type}{subpopulation (domain) name}
    \item{Subpopulation}{subpopulation name within a domain}
    \item{Indicator}{response variable}
    \item{Trend_Estimate}{trend estimate}
    \item{Trend_Std_Error}{trend standard error}
    \item{Trend_LCBxxPct}{trend xx\% (default 95\%) lower confidence bound}
    \item{Trend_UCBxxPct}{trend xx\% (default 95\%) upper confidence bound}
    \item{Trend_p_Value}{trend p-value}
    \item{Intercept_Estimate}{intercept estimate}
    \item{Intercept_Std_Error}{intercept standard error}
    \item{Intercept_LCBxxPct}{intercept xx\% (default 95\%) lower confidence
      bound}
    \item{Intercept_UCBxxPct}{intercept xx\% (default 95\%) upper confidence
      bound}
    \item{Intercept_p_Value}{intercept p-value}
    \item{Var_SiteInt}{variance of the site intercepts}
    \item{Var_SiteTrend}{variance of the site trends}
    \item{Corr_SiteIntSlope}{correlation of site intercepts and site trends}
    \item{Var_Year}{year variance}
    \item{Var_Residual}{residual variance}
    \item{AIC}{generalized Akaike Information Criterion}
  }
}
\description{
This function organizes input and output for estimation of trend across time
for a series of samples (for categorical and continuous variables). Trend is estimated using the
analytical procedure identified by the model arguments.  For categorical
variables, the choices for the \code{model_cat} argument are: (1) simple linear
regression, (2) weighted linear regression, and (3) generalized linear
mixed-effects model. For continuous variables, the choices for the
\code{model_cont} argument are: (1) simple linear regression, (2) weighted
linear regression, and (3)  linear mixed-effects model.  The analysis data,
\code{dframe}, can be either a data frame or a simple features (\code{sf}) object.  If an
\code{sf} object is used, coordinates are extracted from the geometry column in the
object, arguments \code{xcoord} and \code{ycoord} are assigned values
\code{"xcoord"} and \code{"ycoord"}, respectively, and the geometry column is
dropped from the object.
}
\section{Details}{

For the simple linear regression (SLR) model, a design-based estimate of the
category proportion (categorical variables) or the mean (continuous
variables) is calculated for each time period (year).  Four choices of
variance estimator are available for calculating variance of the design-based
estimates: (1) the local mean estimator, (2) the simple random sampling
estimator, (3) the Horvitz-Thompson estimator, and (4) the Yates-Grundy
estimator.  For the Horvitz-Thompson and Yates-Grundy estimators, there are
three choices for calculating joint inclusion probabilities: (1) the Overton
approximation, (2) the Hartley-Rao approximation, and (3) the Brewer
approximation.  The \code{lm} function in the stats package is used to fit a
linear model using a \code{formula} argument that specifies the proportion or
mean estimates as the response variable and years as the regressor variable.
For fitting the SLR model, the \code{yearID} variable from the \code{dframe}
argument is modified by subtracting the minimum value of years from all
values of the variable.  Parameter estimates are extracted from the object
returned by the \code{lm} function.  For the weighted linear regression (WLR)
model, the process is the same as the SLR model except that the inverse of
the variances of the proportion or mean estimates is used as the
\code{weights} argument in the call to the \code{lm} function.  For the LMM
option, the \code{lmer} function in the lme4 package is used to fit a linear
mixed-effects model for trend across years.  For both the GLMM and LMM
options, the default Piepho and Ogutu (PO) model includes fixed effects for
intercept and trend (slope) and random effects for intercept and trend for
individual sites, where the \code{siteID} variable from the \code{dframe}
argument identifies sites.  Correlation between the random effects for site
intercepts and site trends is included in the model. Finally, the PO model
contains random effects for year variance and residual variance. For the GLMM
and LMM options, arguments \code{cat_rhs} and \code{cont_rhs}, respectively,
can be used to specify the right hand side of the model formula. Internally,
a variable named \code{Wyear} is created that is useful for specifying the
\code{cat_rhs} and \code{cont_rhs} arguments.  The \code{Wyear} variable is
created by subtracting the minimum value of the \code{yearID} variable from
all values of the variable.  If argument \code{invprboot} is \code{FALSE},
parameter estimates are extracted from the object returned by the \code{lmer}
function. If argument \code{invprboot} is \code{TRUE}, the \code{boot}
function in the boot package is used to generate bootstrap replicates using a
function named \code{bootfcn} as the \code{statistic} argument passed to the
\code{boot} function.  For each bootstrap replicate, \code{bootfcn} calls the
\code{glmer} or \code{lmer} function, as appropriate, using the specified
model.  design weights identified by the \code{weight} argument for
the \code{trend_analysis} function are passed as the \code{weights} argument
for the \code{boot} function, which specifies importance weights.  Using the
design weights as the \code{weights} argument ensures that bootstrap
replicates are representative of the survey population.  Parameter estimates
are calculated using the object returned by the \code{boot} function.
}

\examples{
# Example using a categorical variable with three resource classes and a
# continuous variable
mydframe <- data.frame(
  siteID = rep(paste0("Site", 1:40), rep(5, 40)),
  yearID = rep(seq(2000, 2020, by = 5), 40),
  wgt = rep(runif(40, 10, 100), rep(5, 40)),
  xcoord = rep(runif(40), rep(5, 40)),
  ycoord = rep(runif(40), rep(5, 40)),
  All_Sites = rep("All Sites", 200),
  Region = sample(c("North", "South"), 200, replace = TRUE),
  Resource_Class = sample(c("Good", "Fair", "Poor"), 200, replace = TRUE),
  ContVar = rnorm(200, 10, 1)
)
myvars_cat <- c("Resource_Class")
myvars_cont <- c("ContVar")
mysubpops <- c("All_Sites", "Region")
trend_analysis(
  dframe = mydframe,
  vars_cat = myvars_cat,
  vars_cont = myvars_cont,
  subpops = mysubpops,
  model_cat = "WLR",
  model_cont = "SLR",
  siteID = "siteID",
  yearID = "yearID",
  weight = "wgt",
  xcoord = "xcoord",
  ycoord = "ycoord"
)
}
\seealso{
\describe{
  \item{\code{\link{change_analysis}}}{for change analysis}
  }
}
\author{
Tom Kincaid \email{Kincaid.Tom@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Illinois_River_Legacy}
\alias{Illinois_River_Legacy}
\title{Illinois River legacy data}
\format{
5 rows and 2 variables:
\describe{
  \item{\code{STATE_NAME}}{ State name.}
  \item{\code{geometry}}{ POINT geometry using the NAD83 / Conus Albers
    coordinate reference system (EPSG: 5070).}
}
}
\usage{
Illinois_River_Legacy
}
\description{
An (\code{sf}) POINT object of legacy sites for the Illinois
River data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spsurvey.R
\docType{package}
\name{spsurvey-package}
\alias{spsurvey}
\alias{spsurvey-package}
\title{spsurvey: Spatial Sampling Design and Analysis}
\description{
spsurvey implements a design-based approach to statistical inference,
  with a focus on spatial data.
  Spatially balanced samples are selected using the
  Generalized Random Tessellation Stratified (GRTS) algorithm.
  The GRTS algorithm can be applied to finite resources (point geometries) and
  infinite resources (linear / linestring and areal / polygon geometries) and flexibly
  accommodates a diverse set of sampling design features, including
  stratification, unequal inclusion probabilities, proportional (to size)
  inclusion probabilities, legacy (historical) sites, a minimum distance between
  sites, and two options for replacement sites (reverse hierarchical order and
  nearest neighbor). Data are analyzed using a wide
  range of analysis functions that perform categorical variable analysis, continuous
  variable analysis, attributable risk analysis, risk difference analysis, relative
  risk analysis, change analysis, and trend analysis. spsurvey can also be used to
  summarize objects, visualize objects, select samples that are not spatially balanced,
  select panel samples, measure the amount of spatial balance in a sample,
  adjust design weights, and more.
  This R package has been reviewed in accordance
  with U.S. Environmental Protection Agency policy and approved for publication.
  Mention of trade names or commercial products does not constitute endorsement or
  recommendation for use.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://usepa.github.io/spsurvey/}
  \item \url{https://github.com/USEPA/spsurvey}
  \item Report bugs at \url{https://github.com/USEPA/spsurvey/issues}
}

}
\author{
\strong{Maintainer}: Michael Dumelle \email{Dumelle.Michael@epa.gov} (\href{https://orcid.org/0000-0002-3393-5529}{ORCID})

Authors:
\itemize{
  \item Tom Kincaid
  \item Anthony (Tony) R. Olsen
  \item Marc Weber
}

Other contributors:
\itemize{
  \item Don Stevens [contributor]
  \item Denis White [contributor]
  \item Amanda M. Nahlik [contributor]
  \item Sarah Lehmann [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{NE_Lakes}
\alias{NE_Lakes}
\title{New England Lakes data}
\format{
195 rows and 5 variables:
\describe{
  \item{\code{AREA}}{ Lake area in hectares.}
  \item{\code{AREA_CAT}}{ Lake area categories based on a hectare cutoff.}
  \item{\code{ELEV}}{ Elevation in meters.}
  \item{\code{ELEV_CAT}}{ Elevation categories based on a meter cutoff.}
  \item{\code{geometry}}{ POINT geometry using the NAD83 / Conus Albers
    coordinate reference system (EPSG: 5070).}
}
}
\usage{
NE_Lakes
}
\description{
An \code{sf} POINT object of 195 lakes in the Northeastern
United States.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/localmean_cov.R
\name{localmean_cov}
\alias{localmean_cov}
\title{Internal Function: Variance-Covariance Matrix Based on Local Mean Estimator}
\usage{
localmean_cov(zmat, weight_1st)
}
\arguments{
\item{zmat}{Matrix of weighted response values or weighted residual values
for the sample points.}

\item{weight_1st}{List from the local mean weight function containing two
elements: a matrix named \code{ij} composed of the index values of neighboring
points and a vector named \code{gwt} composed of weights.}
}
\value{
The local mean estimator of the variance-covariance matrix.
}
\description{
This function calculates the variance-covariance matrix using the local mean
estimator.
}
\author{
Tom Kincaid \email{Kincaid.Tom@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/attrisk_analysis.R
\name{attrisk_analysis}
\alias{attrisk_analysis}
\title{Attributable risk analysis}
\usage{
attrisk_analysis(
  dframe,
  vars_response,
  vars_stressor,
  response_levels = NULL,
  stressor_levels = NULL,
  subpops = NULL,
  siteID = NULL,
  weight = "weight",
  xcoord = NULL,
  ycoord = NULL,
  stratumID = NULL,
  clusterID = NULL,
  weight1 = NULL,
  xcoord1 = NULL,
  ycoord1 = NULL,
  sizeweight = FALSE,
  sweight = NULL,
  sweight1 = NULL,
  fpc = NULL,
  popsize = NULL,
  vartype = "Local",
  conf = 95,
  All_Sites = FALSE
)
}
\arguments{
\item{dframe}{Data to be analyzed (analysis data). A data frame or
\code{sf} object containing survey design
variables, response variables, stressor variables, and subpopulation
(domain) variables.}

\item{vars_response}{Vector composed of character values that identify the
names of response variables in \code{dframe}. Each response
variable must have two category values (levels), where one level is
associated with poor condition and the other level is associated with good
condition.}

\item{vars_stressor}{Vector composed of character values that identify the
names of stressor variables in \code{dframe}. Each stressor
variable must have two category values (levels), where one level is
associated with poor condition and the other level is associated with good
condition.}

\item{response_levels}{List providing the category values (levels) for each
element in the \code{vars_response} argument.  Each element in the list
must contain two values, where the first value identifies poor condition,
and the second value identifies good condition.  This argument must be
named and must be the same length as argument \code{vars_response}.  Names
for this argument must match the values in the \code{vars_response}
argument. If this argument equals NULL, then a named list is created that
contains the values \code{"Poor"} and \code{"Good"} for the first and
second levels, respectively, of each element in the \code{vars_response}
argument and that uses values in the \code{vars_response} argument as names
for the list.  If \code{response_levels} is provided without names,
then the names of \code{response_levels} are set to \code{vars_response}.
The default value is NULL.}

\item{stressor_levels}{List providing the category values (levels) for each
element in the \code{vars_stressor} argument.  Each element in the list
must contain two values, where the first value identifies poor condition,
and the second value identifies good condition.  This argument must be
named and must be the same length as argument \code{vars_stressor}.  Names
for this argument must match the values in the \code{vars_stressor}
argument. If this argument equals NULL, then a named list is created that
contains the values \code{"Poor"} and \code{"Good"} for the first and
second levels, respectively, of each element in the \code{vars_stressor}
argument and that uses values in the \code{vars_stressor} argument as names
for the list.  If \code{stressor_levels} is provided without names,
then the names of \code{stressor_levels} are set to \code{vars_stressor}. 
The default value is NULL.}

\item{subpops}{Vector composed of character values that identify the
names of subpopulation (domain) variables in \code{dframe}.
If a value is not provided, the value \code{"All_Sites"} is assigned to the
subpops argument and a factor variable named \code{"All_Sites"} that takes
the value \code{"All Sites"} is added to \code{dframe}.  The
default value is \code{NULL}.}

\item{siteID}{Character value providing the name of the site ID variable in
\code{dframe}.  For a two-stage sample, the site ID variable
identifies stage two site IDs.  The default value is \code{NULL}, which
assumes that each row in \code{dframe} represents a unique site.}

\item{weight}{Character value providing the name of the design weight
variable in \code{dframe}.  For a two-stage sample, the
weight variable identifies stage two weights.  The default value is
\code{"weight"}.}

\item{xcoord}{Character value providing name of the x-coordinate variable in
\code{dframe}.  For a two-stage sample, the x-coordinate
variable identifies stage two x-coordinates.  Note that x-coordinates are
required for calculation of the local mean variance estimator.  If \code{dframe}
is an \code{sf} object, this argument is not required (as the geometry column
in \code{dframe} is used to find the x-coordinate). The default
value is \code{NULL}.}

\item{ycoord}{Character value providing name of the y-coordinate variable in
\code{dframe}.  For a two-stage sample, the y-coordinate
variable identifies stage two y-coordinates.  Note that y-coordinates are
required for calculation of the local mean variance estimator.  If \code{dframe}
is an \code{sf} object, this argument is not required (as the geometry column
in \code{dframe} is used to find the t-coordinate). The default
value is \code{NULL}.}

\item{stratumID}{Character value providing the name of the stratum ID
variable in \code{dframe}.  The default value is
\code{NULL}.}

\item{clusterID}{Character value providing the name of the cluster
(stage one) ID variable in \code{dframe}.  Note that cluster
IDs are required for a two-stage sample.  The default value is \code{NULL}.}

\item{weight1}{Character value providing the name of the stage one weight
variable in \code{dframe}.  The default value is
\code{NULL}.}

\item{xcoord1}{Character value providing the name of the stage one
x-coordinate variable in \code{dframe}.  Note that x
coordinates are required for calculation of the local mean variance
estimator.  The default value is \code{NULL}.}

\item{ycoord1}{Character value providing the name of the stage one
y-coordinate variable in \code{dframe}.  Note that
y-coordinates are required for calculation of the local mean variance
estimator.  The default value is \code{NULL}.}

\item{sizeweight}{Logical value that indicates whether size weights should be
used during estimation, where \code{TRUE} uses size weights and
\code{FALSE} does not use size weights. To employ size weights for a
single-stage sample, a value must be supplied for argument weight.  To
employ size weights for a two-stage sample, values must be supplied for
arguments \code{weight} and \code{weight1}. The default value is
\code{FALSE}.}

\item{sweight}{Character value providing the name of the size weight variable
in \code{dframe}.  For a two-stage sample, the size weight
variable identifies stage two size weights.  The default value is
\code{NULL}.}

\item{sweight1}{Character value providing the name of the stage one size
weight variable in \code{dframe}.  The default value is
\code{NULL}.}

\item{fpc}{Object that specifies values required for calculation of the
  finite population correction factor used during variance estimation. The
  object must match the survey design in terms of stratification and whether
  the design is single-stage or two-stage.  For an unstratified design, the
  object is a vector.  The vector is composed of a single numeric value for a
  single-stage design.  For a two-stage unstratified design, the object is a
  named vector containing one more than the number of clusters in the sample,
  where the first item in the vector specifies the number of clusters in the
  population and each subsequent item specifies the number of stage two units
  for the cluster.  The name for the first item in the vector is arbitrary.
  Subsequent names in the vector identify clusters and must match the cluster
  IDs.  For a stratified design, the object is a named list of vectors, where
  names must match the strata IDs.  For each stratum, the format of the
  vector is identical to the format described for unstratified single-stage
  and two-stage designs.  Note that the finite population correction factor
  is not used with the local mean variance estimator.

  Example fpc for a single-stage unstratified survey design:

  \verb{fpc <- 15000}

  Example fpc for a single-stage stratified survey design:

  \verb{fpc <- list(
    Stratum_1 = 9000,
    Stratum_2 = 6000)
   }

  Example fpc for a two-stage unstratified survey design:

  \verb{fpc <- c(
    Ncluster = 150,
    Cluster_1 = 150,
    Cluster_2 = 75,
    Cluster_3 = 75,
    Cluster_4 = 125,
    Cluster_5 = 75)
  }

  Example fpc for a two-stage stratified survey design:

  \verb{fpc <- list(
    Stratum_1 = c(
      Ncluster_1 = 100,
      Cluster_1 = 125,
      Cluster_2 = 100,
      Cluster_3 = 100,
      Cluster_4 = 125,
      Cluster_5 = 50),
    Stratum_2 = c(
      Ncluster_2 = 50,
      Cluster_1 = 75,
      Cluster_2 = 150,
      Cluster_3 = 75,
      Cluster_4 = 75,
      Cluster_5 = 125))
  }}

\item{popsize}{Object that provides values for the population argument of the
  \code{calibrate} or \code{postStratify} functions in the survey package. If
  a value is provided for popsize, then either the \code{calibrate} or
  \code{postStratify} function is used to modify the survey design object
  that is required by functions in the survey package.  Whether to use the
  \code{calibrate} or \code{postStratify} function is dictated by the format
  of popsize, which is discussed below.  Post-stratification adjusts the
  sampling and replicate weights so that the joint distribution of a set of
  post-stratifying variables matches the known population joint distribution.
  Calibration, generalized raking, or GREG estimators generalize
  post-stratification and raking by calibrating a sample to the marginal
  totals of variables in a linear regression model. For the \code{calibrate}
  function, the object is a named list, where the names identify factor
  variables in \code{dframe}.  Each element of the list is a
  named vector containing the population total for each level of the
  associated factor variable.  For the \code{postStratify} function, the
  object is either a data frame, table, or xtabs object that provides the
  population total for all combinations of selected factor variables in the
  \code{dframe} data frame.  If a data frame is used for \code{popsize}, the
  variable containing population totals must be the last variable in the data
  frame.  If a table is used for \code{popsize}, the table must have named
  \code{dimnames} where the names identify factor variables in the
  \code{dframe} data frame.  If the popsize argument is equal to \code{NULL},
  then neither calibration nor post-stratification is performed.  The default
  value is \code{NULL}.

  Example popsize for calibration:

  \verb{popsize <- list(
    Ecoregion = c(
      East = 750,
      Central = 500,
      West = 250),
    Type = c(
      Streams = 1150,
      Rivers = 350))
  }

  Example popsize for post-stratification using a data frame:

  \verb{popsize <- data.frame(
    Ecoregion = rep(c("East", "Central", "West"),
      rep(2, 3)),
    Type = rep(c("Streams", "Rivers"), 3),
    Total = c(575, 175, 400, 100, 175, 75))
  }

  Example popsize for post-stratification using a table:

  \verb{popsize <- with(MySurveyFrame,
    table(Ecoregion, Type))}

  Example popsize for post-stratification using an xtabs object:

  \verb{popsize <- xtabs(~Ecoregion + Type,
    data = MySurveyFrame)}}

\item{vartype}{Character value providing the choice of the variance
estimator, where \code{"Local"} indicates the local mean estimator and \code{"SRS"} indicates the
simple random sampling estimator.  The default value is \code{"Local"}.}

\item{conf}{Numeric value providing the Gaussian-based confidence level.  The default value
is \code{95}.}

\item{All_Sites}{A logical variable used when \code{subpops} is not
\code{NULL}. If \code{All_Sites} is \code{TRUE}, then alongside the
subpopulation output, output for all sites (ignoring subpopulations) is
returned for each variable in \code{vars}. If \code{All_Sites} is
\code{FALSE}, then alongside the subpopulation output, output for all sites
(ignoring subpopulations) is not returned for each variable in \code{vars}.
The default is \code{FALSE}.}
}
\value{
The analysis results. A data frame of population estimates for all combinations of
  subpopulations, categories within each subpopulation, response variables,
  and categories within each response variable.  Estimates are provided for
  proportion and size of the population plus standard error, margin of
  error, and confidence interval estimates. The data frame contains the following
  variables:
  \describe{
    \item{Type}{subpopulation (domain) name}
    \item{Subpopulation}{subpopulation name within a domain}
    \item{Response}{response variable}
    \item{Stressor}{stressor variable}
    \item{nResp}{sample size}
    \item{Estimate}{attributable risk estimate}
    \item{StdError_log}{attributable risk standard error (on the log scale)}
    \item{MarginofError_log}{attributable risk margin of error (on the log scale)}
    \item{LCBxxPct}{xx\% (default 95\%) lower confidence bound}
    \item{UCBxxPct}{xx\% (default 95\%) upper confidence bound}
    \item{WeightTotal}{sum of design weights}
    \item{Count_RespPoor_StressPoor}{number of observations in the poor response and poor stressor group}
    \item{Count_RespPoor_StressGood}{number of observations in the poor response and good stressor group}
    \item{Count_RespGood_StressPoor}{number of observations in the good response and poor stressor group}
    \item{Count_RespGood_StressGood}{number of observations in the good response and good stressor group}
    \item{Prop_RespPoor_StressPoor}{weighted proportion of observations in the poor response and poor stressor group}
    \item{Prop_RespPoor_StressGood}{weighted proportion of observations in the poor response and good stressor group}
    \item{Prop_RespGood_StressPoor}{weighted proportion of observations in the good response and poor stressor group}
    \item{Prop_RespGood_StressGood}{weighted proportion of observations in the good response and good stressor group}
  }
}
\description{
This function organizes input and output for the analysis of attributable risk (for
categorical variables).  The analysis data,
\code{dframe}, can be either a data frame or a simple features (\code{sf}) object.  If an
\code{sf} object is used, coordinates are extracted from the geometry column in the
object, arguments \code{xcoord} and \code{ycoord} are assigned values
\code{"xcoord"} and \code{"ycoord"}, respectively, and the geometry column is
dropped from the object.
}
\section{Details}{

Attributable risk measures the proportional reduction in the extent of poor
condition of a response variable that presumably would result from
eliminating a stressor variable, where the response and stressor variables
are classified as either good (i.e., reference condition) or poor (i.e.,
different from reference condition).  Attributable risk is defined as one
minus the ratio of two probabilities.  The numerator of the ratio is the
conditional probability that the response variable is in poor condition given
that the stressor variable is in good condition.   The denominator of the
ratio is the probability that the response variable is in poor condition.
Attributable risk values close to zero indicate that removing the stressor
variable will have little or no impact on the probability that the response
variable is in poor condition.  Attributable risk values close to one
indicate that removing the stressor variable will result in extensive
reduction of the probability that the response variable is in poor condition.
}

\examples{
dframe <- data.frame(
  siteID = paste0("Site", 1:100),
  wgt = runif(100, 10, 100),
  xcoord = runif(100),
  ycoord = runif(100),
  stratum = rep(c("Stratum1", "Stratum2"), 50),
  RespVar1 = sample(c("Poor", "Good"), 100, replace = TRUE),
  RespVar2 = sample(c("Poor", "Good"), 100, replace = TRUE),
  StressVar = sample(c("Poor", "Good"), 100, replace = TRUE),
  All_Sites = rep("All Sites", 100),
  Resource_Class = rep(c("Agr", "Forest"), c(55, 45))
)
myresponse <- c("RespVar1", "RespVar2")
mystressor <- c("StressVar")
mysubpops <- c("All_Sites", "Resource_Class")
attrisk_analysis(dframe,
  vars_response = myresponse,
  vars_stressor = mystressor, subpops = mysubpops, siteID = "siteID",
  weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
  stratumID = "stratum"
)
}
\references{
Sickle, J. V., & Paulsen, S. G. (2008). Assessing the attributable risks,
   relative risks, and regional extents of aquatic stressors.
   \emph{Journal of the North American Benthological Society}, 27(4), 920-931.
}
\seealso{
\describe{
  \item{\code{\link{relrisk_analysis}}}{ for relative risk analysis}
  \item{\code{\link{diffrisk_analysis}}}{ for risk difference analysis}
  }
}
\author{
Tom Kincaid \email{Kincaid.Tom@epa.gov}
}
\keyword{survey}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pd_summary.R
\name{pd_summary}
\alias{pd_summary}
\title{Summary characteristics of a panel revisit design}
\usage{
pd_summary(object, visitdsgn = NULL, ...)
}
\arguments{
\item{object}{Two-dimensional array from \code{panel_design} and dimnames specifying revisit
panel design. Typically, array is output from \code{revisit_dsgn}, \code{revisit_bibd} or
\code{revisit_rand} functions.}

\item{visitdsgn}{Two-dimensional array with same dimensions as \code{paneldsgn}
specifying the number of times a sample unit is sampled at each time
period. Default is \code{visitdsgn=NULL}, where default assumes that a sample unit
will be sampled only once at each time period.}

\item{...}{Additional arguments (S3 consistency)}
}
\value{
List of six elements.
  \describe{
    \item{\code{n_panel}}{ number of panels in revisit design}
    \item{\code{n_period}}{ number of time periods in revisit design}
    \item{\code{n_total}}{ total number of sample events across all panels and all
      time periods, accounting for \code{visitdsgn}, that will be sampled in the revisit
      design}
    \item{\code{n_periodunit}}{ vector of the number of time periods a unit will be
      sampled in each panel}
    \item{\code{n_unitpnl}}{ vector of the number of sample units, accounting for
      \code{visitdsgn}, that will be sampled in each panel}
    \item{\code{n_unitperiod}}{ vector of the number of sample units, accounting for
      \code{visitdsgn}, that will be sampled during each time period}
    \item{\code{ncum_unit}}{ vector of the cumulative number of unique units that will
  be sampled in time periods up to and including the current time period.}
  }
}
\description{
Panel revisit design characteristics are summarized: number of panels, number
of time periods, total number of sample events for the revisit design, total
number of sample events for each panel, total number of sample events for
each time period and cumulative number of unique units sampled by time
periods.
}
\details{
The revisit panel design and the visit design (if present) are
  summarized. Summaries can be useful to know the effort required to complete
  the survey design. See the values returned for the summaries that are
  produced.
}
\examples{
# Serially alternating panel revisit design summary
sa_dsgn <- revisit_dsgn(20, panels = list(SA60N = list(
  n = 60, pnl_dsgn = c(1, 4),
  pnl_n = NA, start_option = "None"
)), begin = 1)
pd_summary(sa_dsgn)
# Add visit design where first panel is sampled twice at every time period
sa_visit <- sa_dsgn
sa_visit[sa_visit > 0] <- 1
sa_visit[1, sa_visit[1, ] > 0] <- 2
pd_summary(sa_dsgn, sa_visit)
}
\author{
Tony Olsen \email{Olsen.Tony@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/revisit_dsgn.R
\name{revisit_dsgn}
\alias{revisit_dsgn}
\title{Create a panel revisit design}
\usage{
revisit_dsgn(n_period, panels, begin = 1, skip = 1)
}
\arguments{
\item{n_period}{Number of time periods for the panel design. For example,
number of periods if sampling occurs once per period or number of months if
sampling occurs once per month.}

\item{panels}{List of lists where each list specifies a revisit panel
structure. Each sublist consists of four components: \code{n} - sample size for
each panel in the sublist, \code{pnl_dsgn} - a vector with an even number of
elements specifying the panel revisit schedule in terms of the number of
consecutive time periods sample units will be sampled, followed by number
of consecutive time periods skipped, and then repeated as necessary. \code{pnl_n}
- number of panels in the sublist, and \code{start_option} - option for starting
the \code{revisit_dsgn} (\code{None}, \code{Partial_Begin}, or \code{Partial_End})
which must be the same a \code{pnl_dsgn}. Three basic panel
structures are possible: a) if \code{pnl_dsgn} ends in \code{0}, then the sample units
are visited on all subsequent time periods, b) if \code{pnl_dsgn} ends in \code{NA}, then
panel follows a rotating panel structure, and c) if \code{pnl_dsgn} ends in any
number > \code{0}, then panel follows a serially alternating panel structure. See
details for further information.}

\item{begin}{Numeric name of first sampling occasion, e.g. a specific
period.}

\item{skip}{Number of time periods to skip between planned sampling
periods, e.g., sampling will occur only every 5 periods if \code{skip = 5}.}
}
\value{
A two-dimensional array of sample sizes to be sampled at each
  combination of panel and time period.
}
\description{
Create a revisit design for panels in a survey that specifies the time
periods that members of each panel will be sampled. Three basic panel design
structures may be created: always revisit panel, serially alternating panels,
or rotating panels.
}
\details{
The function creates revisit designs using the concepts in McDonald
  (2003) to specify the revisit pattern across time periods for each panel.
  The panel revisit schedule is specified by a vector. Odd positions in
  vector specify the number of consecutive time periods when panel units are
  sampled. Even positions in vector specify the number of consecutive time
  periods when panel units are not sampled.

  If last even position is a \code{"0"}, then a single panel follows an always
  revisit panel structure.  After satisfying the initial revisit schedule
  specified prior to the \code{"0"}, units in a panel are always visited for rest of
  the time periods. The simplest always revisit panel design is to revisit
  every sample unit on every time period, specified as \code{pnl_dsgn = c(1,0)} or
  using McDonald's notation [1-0].

  If the last even position is \code{NA}, the panels follow a rotating panel
  structure. For example, \code{pnl_dsgn = c(1, NA)} designates that sample units in
  a panel will be visited once and then never again, [1-n] in McDonald's
  notation. \code{pnl_dsgn =c(1, 4, 1, NA)} designates that sample units in a panel
  will be visited once, then not sampled on next four time periods, then
  sampled again once at the next time period and then never sampled again,
  [1-4-1-n] in McDonald/s notation.

  If the last even position is > \code{0}, the panels follow a serially alternating
  panel structure. For example, \code{pnl_dsgn = c(1, 4)} designates that sample
  units in a panel will be visited once, then not sampled during the next
  four time periods, then sampled once and not sampled for next four time
  periods, and that cycle repeated until end of the number of time periods,
  [1-4] in McDonald's notation. \code{pnl_dsgn = c(2, 3, 1, 4)} designates that the
  cycle has sample units in a panel being visited during two consecutive time
  periods, not sampled for three consecutive time periods, sampled for one time
  period and then not sampled on next four time periods, and the cycle is
  repeated until end of the number of time periods, [2-3-1-4] in McDonald's
  notation.

  The number of panels in a single panel design is specified by \code{pnl_n}.  For
  an always revisit panel structure, a single panel is created and \code{pnl_n} is
  ignored. For a rotating panel structure, when \code{pnl_n = NA}, the number of
  panels is equal to n_period. Note that this should only be used when the
  rotating panel structure is the only panel design, i.e., no split panel
  design (see below for split panel details). If \code{pnl_n = m} is specified for a
  rotating panel design, then then number of panels will be \code{m}.  For example,
  \code{pnl_dsgn = c( 1, 4, 1, NA)} and and \code{pnl_n = 5} means that only 5 panels will
  be constructed and the last time period to be sampled will be time period
  10. In McDonald's notation the panel design structure is [(1-4-1-n)^5].  If
  the number of time periods, \code{n_period}, is 20 and no other panel design
  structure is specified, then the last 10 time periods will not be sampled.
  For serially alternating panels, when \code{pnl_n = NA}, the number of panels will
  be the sum of the elements in pan_dsgn (ignoring \code{NA}). If \code{pnl_n} is specified
  as \code{m}, then \code{m} panels will be created.  For example, \code{pnl_dsgn = c(1, 4, 1, 4)}
  and \code{pnl_n = 3}, [(1-4-1-4)^3] in McDonald's notation, will create first three
  panels of the 510 serially alternating panels specified by \code{pnl_dsgn}.

  A serially alternating or rotating panel revisit design may not result in
  the same number of units being sampled during each time period,
  particularly during the initial start up period.  The default is to not
  specify a startup option (\code{"None"}).  Start up option \code{"Partial_Begin"}
  initiates the revisit design at the last time period scheduled for sampling
  in the first panel. For example, a [2-3-1-4] design starts at time period 6
  instead of time period 1 under the Partial_Begin option. For a serially
  alternating panel structure, start up option \code{"Partial_End"} initiates the
  revisit design at the time period that begins the second serially
  alternating pattern. For example, a [2-3-1-4] design starts at time period
  11 instead of time period 1. For a rotating panel structure design, use of
  Partial_End makes the assumption that the number of panels equals the
  number of time periods and adds units to the last "m" panels for time
  periods \code{1} to \code{"m"} as if number of time periods was extended by \code{"m"} where \code{"m"}
  is one less than then the sum of the panel design.  For example, a
  [1-4-1-4-1-n] design would result in \code{m = 10}.  Note that some designs with
  \code{pnl_n} not equal to the number of sample occasions can produce unexpected
  panel designs.  See examples.

  Different types of panel structures can be combined, these are termed split
  panels by many authors, by specifying more than one list for the panels
  parameter.  The total number of panels is the sum of the number of panels
  in each of the panel structures specified by the split panel design.
}
\examples{
# One panel of  60 sample units sampled at every time period: [1-0]
revisit_dsgn(20, panels = list(
  Annual = list(
    n = 60, pnl_dsgn = c(1, 0), pnl.n = NA,
    start_option = "None"
  )
), begin = 1)

# Rotating panels of 60 units sampled once and never again: [1-n].  Number
# of panels equal n_period.
revisit_dsgn(20,
  panels = list(
    R60N = list(n = 60, pnl_dsgn = c(1, NA), pnl_n = NA, start_option = "None")
  ),
  begin = 1
)

# Serially alternating panel with three visits to sample unit then skip
# next two time periods: [3-2]
revisit_dsgn(20, panels = list(
  SA60PE = list(
    n = 20, pnl_dsgn = c(3, 2), pnl_n = NA,
    start_option = "Partial_End"
  )
), begin = 1)

# Split panel of sample units combining above two panel designs: [1-0, 1-n]
revisit_dsgn(n_period = 20, begin = 2017, panels = list(
  Annual = list(
    n = 60, pnl_dsgn = c(1, 0), pnl.n = NA,
    start_option = "None"
  ),
  R60N = list(n = 60, pnl_dsgn = c(1, NA), pnl_n = NA, start_option = "None")
))
}
\references{
McDonald, T. (2003). Review of environmental monitoring methods: survey
  designs. \emph{Environmental Monitoring and Assessment} \bold{85},
  277-292.
}
\seealso{
\describe{
    \item{\code{\link{revisit_bibd}}}{to create a balanced incomplete block
      panel revisit design}
    \item{\code{\link{revisit_rand}}}{to create a revisit design with random
      assignment to panels and time periods}
    \item{\code{\link{pd_summary}}}{ to summarize characteristics of a
      panel revisit design}
  }
}
\author{
Tony Olsen \email{Olsen.Tony@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/errorprnt.R
\name{errorprnt}
\alias{errorprnt}
\title{Print errors from analysis functions}
\usage{
errorprnt(error_vec = get("error_vec", envir = .GlobalEnv))
}
\arguments{
\item{error_vec}{Data frame that contains error messages.  The default is
\code{"error_vec"}, which is the name given to the error messages vector created
by functions in the spsurvey package.}
}
\value{
Printed errors.
}
\description{
This function prints the error messages vector in the analysis functions.
}
\author{
Tom Kincaid \email{Kincaid.Tom@epa.gov}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sp_balance.R
\name{sp_balance}
\alias{sp_balance}
\title{Calculate spatial balance metrics}
\usage{
sp_balance(
  object,
  sframe,
  stratum_var = NULL,
  ip = NULL,
  metrics = "pielou",
  extents = FALSE
)
}
\arguments{
\item{object}{An \code{sf} object containing some design sites.}

\item{sframe}{The sampling frame as an \code{sf} object. The coordinate
system for \code{sframe} must be one where distance for coordinates is meaningful.}

\item{stratum_var}{The name of the stratum variable in \code{object}
and \code{sframe}. If \code{NULL} (the default), no strata is assumed.
If a single character vector is provided, it is assumed this is the
name of the stratum variable in \code{object} and \code{sframe}. If
a two-dimensional character vector is provided, one element must be
named "object" and corresponds to the name of the stratum variable
in \code{object}, while the other element must be named "sframe" and
corresponds to the name of the stratum variable in \code{sframe}.}

\item{ip}{Inclusion probabilities associated with each row of \code{sframe}.
If these are not provided, an equal probability design is assumed (within
strata).}

\item{metrics}{A character vector of spatial balance metrics:
\describe{
  \item{\code{pielou}}{ Pielou's Evenness Index (the default). This statistic
    can take on a value between zero and one.}
  \item{\code{simpsons}}{ Simpsons Evenness Index. This statistic
    can take on a value between zero and logarithm of the sample size.}
  \item{\code{rmse}}{ Root-Mean-Squared Error. This statistic
    can take on a value between zero and infinity.}
  \item{\code{mse}}{ Mean-Squared Error. This statistic
    can take on a value between zero and infinity.}
  \item{\code{mae}}{ Median-Absolute Error. This statistic
    can take on a value between zero and infinity.}
  \item{\code{medae}}{ Mean-Absolute Error. This statistic
    can take on a value between zero and infinity.}
  \item{\code{chisq}}{ Chi-Squared Loss. This statistic
    can take on a value between zero and infinity.}
 }
  All spatial balance metrics have a lower bound of zero, which indicates perfect
  spatial balance. As the metric value increases, the spatial balance decreases.}

\item{extents}{Should the extent (total units) within each Voronoi polygon
be returned? Defaults to \code{FALSE}.}
}
\value{
A data frame with columns providing the stratum (\code{stratum}),
  spatial balance metric (\code{metric}), and spatial balance (\code{value}).
}
\description{
This function measures the spatial balance (with respect to the
sampling frame) of design sites using
Voronoi polygons (Dirichlet tessellations).
}
\examples{
\dontrun{
sample <- grts(NE_Lakes, 30)
sp_balance(sample$sites_base, NE_Lakes)
strata_n <- c(low = 25, high = 30)
sample_strat <- grts(NE_Lakes, n_base = strata_n, stratum_var = "ELEV_CAT")
sp_balance(sample_strat$sites_base, NE_Lakes, stratum_var = "ELEV_CAT", metric = "rmse")
}
}
\author{
Michael Dumelle \email{Dumelle.Michael@epa.gov}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sp_plot.R
\name{sp_plot}
\alias{sp_plot}
\alias{sp_plot.default}
\alias{sp_plot.sp_design}
\title{Plot sampling frames, design sites, and analysis data.}
\usage{
sp_plot(object, ...)

\method{sp_plot}{default}(
  object,
  formula = ~1,
  xcoord,
  ycoord,
  crs,
  var_args = NULL,
  varlevel_args = NULL,
  geom = FALSE,
  onlyshow = NULL,
  fix_bbox = TRUE,
  ...
)

\method{sp_plot}{sp_design}(
  object,
  sframe = NULL,
  formula = ~siteuse,
  siteuse = NULL,
  var_args = NULL,
  varlevel_args = NULL,
  geom = FALSE,
  onlyshow = NULL,
  fix_bbox = TRUE,
  ...
)
}
\arguments{
\item{object}{An object to plot. When plotting sampling frames or analysis data,
a data frame or \code{sf} object. When plotting design sites, an object created by \code{grts()} or
\code{irs()} (which has class \code{sp_design}).}

\item{...}{Additional arguments to pass to \code{plot.sf()}.}

\item{formula}{A formula. One-sided formulas are used to summarize the
distribution of numeric or categorical variables. For one-sided formulas,
variable names are placed to the right of \code{~} (a right-hand side variable).
Two sided formulas are
used to summarize the distribution of a left-hand side variable
for each level of each right-hand side categorical variable in the formula.
Note that only for two-sided formulas are numeric right-hand side variables
coerced to a categorical variables. If an intercept
is included as a right-hand side variable (whether the formula is one-sided or
two-sided), the total will also be summarized. When plotting sampling frames
or analysis data, the default formula is \code{~ 1}. When plotting design sites,
\code{siteuse} should be used in the formula, and the default formula is
\code{~ siteuse}.}

\item{xcoord}{Name of the x-coordinate (east-west) in \code{object} (only required if
\code{object} is not an \code{sf} object).}

\item{ycoord}{Name of y (north-south)-coordinate in \code{object} (only required if
\code{object} is not an \code{sf} object).}

\item{crs}{Projection code for \code{xcoord} and \code{ycoord} (only
required if \code{object} is not an \code{sf} object).}

\item{var_args}{A named list. The name of each list element corresponds to a
right-hand side variable in \code{formula}. Values in the list are composed of
graphical arguments that are to be passed to \strong{every} level of the
variable. To see all graphical arguments available, run \code{?plot.sf}.}

\item{varlevel_args}{A named list. The name of each list element corresponds to a
right-hand side variable in \code{formula}. The first element in this list
should be \code{"levels"} and contain all levels of the particular right-hand side variable. Subsequent
names correspond to graphical arguments that are to be passed to
the specified levels (in order) of the right-hand side variable. Values for each
graphical argument must be specified for each level of the right-hand side variable,
but applicable sf defaults will be matched by inputting the value \code{NA}.
To see all graphical arguments available, run \code{?plot.sf}}

\item{geom}{Should separate geometries for each level of the right-hand
side \code{formula} variables be plotted? Defaults to \code{FALSE}.}

\item{onlyshow}{A string indicating the single level of the single right-hand side
variable for which a summary is requested. This argument is only used when
a single right-hand side variable is provided.}

\item{fix_bbox}{Should the geometry bounding box be fixed across plots?
If a length-four vector with names "xmin", "ymin", "xmax", and "ymax" and values
indicating bounding box edges, the bounding box will be fixed as \code{fix_bbox}
across plots. If \code{TRUE}, the bounding box will be fixed across plots as the
bounding box of \code{object}. If \code{FALSE}, the bounding box will vary across
plots according to the unique geometry for each plot. Defaults to \code{TRUE}.}

\item{sframe}{The sampling frame (an \code{sf} object) to plot alongside design
sites. This argument is only used when \code{object} corresponds to the design sites.}

\item{siteuse}{A character vector of site types to include when plotting design sites.
It can only take on values \code{"sframe"} (sampling frame),
\code{"Legacy"} (for legacy sites), \code{"Base"} (for base sites),
\code{"Over"} (for \code{n_over} replacement sites), and \code{"Near"}
(for \code{n_near} replacement sites). The order of sites represents the
layering in the plot (e.g. \code{siteuse = c("Base", "Legacy")} will plot
legacy sites on top of base sites. Defaults to all non-\code{NULL} elements
in \code{x} and \code{y} with plot order \code{"sframe"}, \code{"Legacy"},
\code{"Base"}, \code{"Over"}, \code{"Near"}.}
}
\description{
This function plots sampling frames, design sites, and analysis data.
If the left-hand side of the formula is empty, plots
are of the distributions of the right-hand side variables. If the left-hand side
of the variable contains a variable, plots are of the left-hand size variable
for each level of each right-hand side variable.
This function is largely built on \code{plot.sf()}, and all spsurvey plotting
methods can supply additional arguments to \code{plot.sf()}. For more information on
plotting in \code{sf}, run \code{?sf::plot.sf()}. Equivalent to \code{spsurvey::plot()}; both
are currently maintained for backwards compatibility.
}
\examples{
\dontrun{
data("NE_Lakes")
sp_plot(NE_Lakes, formula = ~ELEV_CAT)
sample <- grts(NE_Lakes, 30)
sp_plot(sample, NE_Lakes)
data("NLA_PNW")
sp_plot(NLA_PNW, formula = ~BMMI)
}
}
\author{
Michael Dumelle \email{Dumelle.Michael@epa.gov}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary}
\alias{summary}
\alias{summary.sp_frame}
\alias{summary.sp_design}
\title{Summarize sampling frames, design sites, and analysis data.}
\usage{
\method{summary}{sp_frame}(object, formula = ~1, onlyshow = NULL, ...)

\method{summary}{sp_design}(object, formula = ~siteuse, siteuse = NULL, onlyshow = NULL, ...)
}
\arguments{
\item{object}{An object to summarize. When summarizing sampling frames,
an \code{sf}
object given the appropriate class using \code{sp_frame}.
When summarizing design sites, an object created by \code{grts()} or
\code{irs()} (which has class \code{sp_design}). When summarizing analysis data,
a data frame or an \code{sf} object given the appropriate class using \code{sp_frame}.}

\item{formula}{A formula. One-sided formulas are used to summarize the
distribution of numeric or categorical variables. For one-sided formulas,
variable names are placed to the right of \code{~} (a right-hand side variable).
Two sided formulas are
used to summarize the distribution of a left-hand side variable
for each level of each right-hand side categorical variable in the formula.
Note that only for two-sided formulas are numeric right-hand side variables
coerced to a categorical variables. If an intercept
is included as a right-hand side variable (whether the formula is one-sided or
two-sided), the total will also be summarized. When summarizing sampling frames
or analysis data, the default formula is \code{~ 1}. When summarizing design sites,
\code{siteuse} should be used in the formula, and the default formula is
\code{~ siteuse}.}

\item{onlyshow}{A string indicating the single level of the single right-hand side
variable for which a summary is requested. This argument is only used when
a single right-hand side variable is provided.}

\item{...}{Additional arguments to pass to \code{sp_summary()}. If the left-hand
side of the formula is empty, the appropriate generic arguments are passed
to \code{summary.data.frame}. If the left-hand side of the formula is provided,
the appropriate generic arguments are passed to \code{summary.default}.}

\item{siteuse}{A character vector indicating the design sites
for which summaries are requested in \code{object}. Defaults to computing summaries for
each non-\code{NULL} \code{sites_*} list in \code{object}.}
}
\value{
If the left-hand side of the formula is empty, a named list
containing summaries of the count distribution for each right-hand side
variable is returned. If the left-hand side of the formula contains a
variable, a named list containing five number
summaries (numeric left-hand side) or tables (categorical or factor left
hand side) is returned for each right-hand side variable.
}
\description{
\code{summary()} summarizes sampling frames, design sites, and analysis data. The right-hand of the
formula specifies the variables (or factors) to
summarize by. If the left-hand side of the formula is empty, the
summary will be of the distributions of the right-hand side variables. If the left-hand side
of the formula contains a variable, the summary will be of the left-hand size variable
for each level of each right-hand side variable. Equivalent to \code{sp_summary()}; both
are currently maintained for backwards compatibility.
}
\examples{
\dontrun{
data("NE_Lakes")
summary(NE_Lakes, ELEV ~ 1)
summary(NE_Lakes, ~ ELEV_CAT * AREA_CAT)
sample <- grts(NE_Lakes, 100)
summary(sample, ~ ELEV_CAT * AREA_CAT)
}
}
\author{
Michael Dumelle \email{Dumelle.Michael@epa.gov}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cont_cdfplot.R
\name{cont_cdfplot}
\alias{cont_cdfplot}
\title{Create a PDF file containing cumulative distribution functions (CDF) plots}
\usage{
cont_cdfplot(
  pdffile = "cdf2x2.pdf",
  cdfest,
  units_cdf = "Percent",
  ind_type = rep("Continuous", nind),
  log = rep("", nind),
  xlab = NULL,
  ylab = NULL,
  ylab_r = NULL,
  legloc = NULL,
  cdf_page = 4,
  width = 10,
  height = 8,
  confcut = 0,
  cex.main = 1.2,
  cex.legend = 1,
  ...
)
}
\arguments{
\item{pdffile}{Name of the PDF file.  The default is "cdf2x2.pdf".}

\item{cdfest}{Data frame with the same structure as the "CDF"
output from  \code{cont_analysis}.}

\item{units_cdf}{Indicator for the label utilized for the left side y-axis
and the values used for the left side y-axis tick marks, where "Percent"
means the label and values are in terms of percent of the population, and
"Units" means the label and values are in terms of units (count, length,
or area) of the population.  The default is "Percent".}

\item{ind_type}{Character vector consisting of the values "Continuous" or
"Ordinal" that controls the type of CDF plot for each indicator.  The
default is "Continuous" for every indicator.}

\item{log}{Character vector consisting of the values "" or "x" that
controls whether the x axis uses the original scale ("") or the base 10
logarithmic scale ("x") for each indicator.  The default is "" for every
indicator.}

\item{xlab}{Character vector consisting of the x-axis label for each
indicator. If this argument equals NULL, then indicator names are used as
the labels. The default is NULL.}

\item{ylab}{Character string providing the left side y-axis label.  If
argument units_cdf equals "Units", a value should be provided for this
argument.  Otherwise, the label will be "Percent".  The default is
"Percent".}

\item{ylab_r}{Character string providing the label for the right side y-axis
(and, hence, determining the values used for the right side y-axis tick
marks), where NULL means a right side y-axis is not created.  If this
argument equals "Same", the right side y-axis will have the same label and
tick mark values as the left side y-axis.  If this argument equals a
character string other than "Same", the right side y-axis label will be the
value provided for argument ylab_r, and the right side y-axis tick mark
values will be determined by the choice not utilized for argument
units_cdf, which means that the default value of argument units_cdf (i.e.,
"Percent") will result in the right side y-axis tick mark values being
expressed  in terms of units of the population (i.e., count, length, or
area).  The default is NULL.}

\item{legloc}{Indicator for location of the plot legend, where "BR" means
bottom right, "BL" means bottom left, "TR" means top right, "TL" means
top left, and NULL means no legend.  The default is NULL.}

\item{cdf_page}{Number of CDF plots on each page, which must be chosen from
the values: 1, 2, 4, or 6.  The default is 4.}

\item{width}{Width of the graphic region in inches.  The default is 10.}

\item{height}{Height of the graphic region in inches.  The default is 8.}

\item{confcut}{Numeric value that controls plotting confidence limits at
the CDF extremes.  Confidence limits for CDF values (percent scale) less
than confcut or greater than 100 minus confcut are not plotted.  A value of
zero means confidence limits are plotted for the complete range of the CDF.
The default is 0.}

\item{cex.main}{Expansion factor for the plot title.  The default is 1.2.}

\item{cex.legend}{Expansion factor for the legend title. The default is 1.}

\item{...}{Additional arguments passed to the \code{cdf_plot} function.}
}
\value{
A PDF file containing the CDF plots.
}
\description{
This function creates a PDF file containing CDF plots.  Input data for the
plots is provided by a data frame with the same structure as the "CDF"
output from  \code{cont_analysis}.  Plots are produced for every combination of \code{Type} of
population, \code{Subpopulation} within \code{Type}, and \code{Indicator} (every combination
of subpopulations, subpopulation levels, and variables).
}
\examples{
\dontrun{
dframe <- data.frame(
  siteID = paste0("Site", 1:100),
  wgt = runif(100, 10, 100),
  xcoord = runif(100),
  ycoord = runif(100),
  stratum = rep(c("Stratum1", "Stratum2"), 50),
  ContVar = rnorm(100, 10, 1),
  All_Sites = rep("All Sites", 100),
  Resource_Class = rep(c("Good", "Poor"), c(55, 45))
)
myvars <- c("ContVar")
mysubpops <- c("All_Sites", "Resource_Class")
mypopsize <- data.frame(
  Resource_Class = c("Good", "Poor"),
  Total = c(4000, 1500)
)
myanalysis <- cont_analysis(dframe,
  vars = myvars, subpops = mysubpops,
  siteID = "siteID", weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
  stratumID = "stratum", popsize = mypopsize
)
cont_cdfplot("myanalysis.pdf", myanalysis$CDF, ylab_r = "Stream Length (km)")
}

}
\seealso{
\describe{
  \item{\code{\link{cdf_plot}}}{for plotting a cumulative distribution
    function (CDF)}
  \item{\code{\link{cont_cdftest}}}{for CDF hypothesis testing}
  }
}
\author{
Tom Kincaid \email{Kincaid.Tom@epa.gov}
}
\keyword{plot}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/revisit_rand.R
\name{revisit_rand}
\alias{revisit_rand}
\title{Create a revisit design with random assignment to panels and time periods}
\usage{
revisit_rand(
  n_period,
  n_pnl,
  rand_control = "period",
  n_visit,
  nsamp,
  panel_name = "Random",
  begin = 1,
  skip = 1
)
}
\arguments{
\item{n_period}{Number of time periods for the survey design. Typically,
number of periods if sampling occurs once per period or number of months if
sampling occurs once per month. (v, number of varieties (or treatments) in
BIBD terms)}

\item{n_pnl}{Number of panels}

\item{rand_control}{Character value must be \code{"none"}, \code{"panel"}, or \code{"period"}.
Specifies whether the number of sample events will be fixed for each panel
(\code{"panel"}), for each sample occasion (\code{"occasion"}), or for total panel-period
combinations (\code{"none"}).  Default is \code{"panel"}.}

\item{n_visit}{If \code{rand_control} is \code{"panel"}, this is the number of panels that
will be sampled in each time period. If rand_control is \code{"period"}, this is
the number of time periods to be sampled in each panel. If \code{rand_control} is
\code{"none"}, this is the total number of panel-period combinations that will
have units sampled in the revisit design.}

\item{nsamp}{Number of samples in each panel.}

\item{panel_name}{Prefix for name of each panel}

\item{begin}{Numeric name of first sampling occasion, e.g. a specific period.}

\item{skip}{Number of sampling occasions to skip between planned sampling
periods, e.g., sampling will occur only every 5 periods if \code{skip = 5}.}
}
\value{
A two-dimensional array of sample sizes to be sampled for each panel
  and each time period.
}
\description{
Create a revisit design for a survey that specifies the panels and time
periods that will be sampled by random selection of panels and time periods.
Three options for random assignments are \code{"period"} where the number of time
periods to be sampled in a panel is fixed, \code{"panel"} where the number panels to
be sampled in a time period is fixed, and \code{"none"} where the number of
panel-period combinations is fixed.
}
\details{
The revisit design for a survey is created by random selection of
  panels and time periods that will have sample events.  The number of sample
  occasions that will be visited by a panel is random.
}
\examples{
revisit_rand(
  n_period = 20, n_pnl = 10, rand_control = "none", n_visit = 50,
  nsamp = 20
)
revisit_rand(
  n_period = 20, n_pnl = 10, rand_control = "panel", n_visit = 5,
  nsamp = 10
)
revisit_rand(
  n_period = 20, n_pnl = 10, rand_control = "period",
  n_visit = 5, nsamp = 10
)
}
\seealso{
\describe{
    \item{\code{\link{revisit_bibd}}}{create a balanced incomplete block
      panel revisit design}
    \item{\code{\link{revisit_dsgn}}}{create a panel revisit design}
    \item{\code{\link{pd_summary}}}{ to summarize characteristics of a
      panel revisit design}
  }
}
\author{
Tony Olsen \email{Olsen.Tony@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sp_frame.R
\name{sp_frame}
\alias{sp_frame}
\alias{sp_unframe}
\title{\code{sp_frame} objects}
\usage{
sp_frame(frame)

sp_unframe(sp_frame)
}
\arguments{
\item{frame}{A sampling frame or analysis data}

\item{sp_frame}{An \code{sp_frame} object.}
}
\value{
An \code{sp_frame} object.
}
\description{
Turn sampling frames or analysis data into an \code{sp_frame} object
  or transform \code{sp_frame} objects back into their original object.
}
\details{
The \code{sp_frame()} function assigns \code{frame} class \code{sp_frame}
  to be used by \code{summary()} and \code{plot()}. \code{sp_frame()} objects
  can sometimes clash with other sf and tidyverse generics, so \code{un_spframe()} removes
  class \code{sp_frame()}, leaving the original classes of \code{frame} intact.
}
\examples{
NE_Lakes <- sp_frame(NE_Lakes)
class(NE_Lakes)
NE_Lakes <- sp_unframe(NE_Lakes)
class(NE_Lakes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cov_panel_dsgn.R
\name{cov_panel_dsgn}
\alias{cov_panel_dsgn}
\title{Create a covariance matrix for a panel design}
\usage{
cov_panel_dsgn(
  paneldsgn = matrix(50, 1, 10),
  nrepeats = 1,
  unit_var = NULL,
  period_var = NULL,
  unitperiod_var = NULL,
  index_var = NULL,
  unit_rho = 1,
  period_rho = 0
)
}
\arguments{
\item{paneldsgn}{A matrix (dimensions: number of panels (rows) by number of
periods (columns)) containing the number of units visited for each
combination of panel and period. Default is matrix(50, 1, 10) which is a
single panel of 50 units visited 10 times, typical time is a period.}

\item{nrepeats}{Either \code{NULL} or a list of matrices the same length as
paneldsgn specifying the number of revisits made to units in a panel in the
same period for each design.  Specifying \code{NULL} indicates that number of
revisits to units is the same for all panels and for all periods and for
all panel designs. The default is \code{NULL}, a single visit. Names must match
list names in \code{paneldsgn}.}

\item{unit_var}{The variance component estimate for unit. The default is
\code{NULL}.}

\item{period_var}{The variance component estimate for period The default is
\code{NULL}.}

\item{unitperiod_var}{The variance component estimate for unit by period
interaction. The default is \code{NULL}.}

\item{index_var}{The variance component estimate for index error. The
default is \code{NULL}.}

\item{unit_rho}{Unit correlation across periods. The default is \code{1}.}

\item{period_rho}{Period autocorrelation. The default is \code{0}.}
}
\value{
A list containing the covariance matrix (\code{cov}) for the panel design,
  the input panel design (\code{paneldsgn}), the input \code{nrepeats} design
  (\code{nrepeats.dsgn}) and the function call.
}
\description{
Covariance structure accounts for the panel design and the four variance
components: unit variation, period variation, unit by period interaction
variation and index (or residual) variation. The model incorporates unit,
period, unit by period, and index variance components. It also includes a
provision for unit correlation and period autocorrelation.
}
\details{
Covariance structure accounts for the panel design and the four
  variance components: unit variation, period variation, unit by period
  interaction variation and index (or residual) variation. Uses the model
  structure defined by Urquhart 2012.

  If \code{nrepeats} is \code{NULL}, then no units sampled more than once in a specific
  panel, period combination) and then unit by period and index variances are
  added together or user may have only estimated unit, period and unit by
  period variance components so that index component is zero. It calculates
  the covariance matrix for the simple linear regression. The standard error
  for a linear trend coefficient is the square root of the variance.
}
\references{
Urquhart, N. S., W. S. Overton, et al. (1993) Comparing sampling designs
  for monitoring ecological status and trends: impact of temporal patterns.
  In: \emph{Statistics for the Environment.} V. Barnett and K. F. Turkman.
  John Wiley & Sons, New York, pp. 71-86.

  Urquhart, N. S. and T. M. Kincaid (1999). Designs for detecting trends
  from repeated surveys of ecological resources. \emph{Journal of
  Agricultural, Biological, and Environmental Statistics}, \bold{4(4)},
  404-414.

  Urquhart, N. S. (2012). The role of monitoring design in detecting trend in
  long-term ecological monitoring studies. In: \emph{Design and Analysis of
  Long-term Ecological Monitoring Studies.} R. A. Gitzen, J. J. Millspaugh,
  A. B. Cooper, and D. S. Licht (eds.). Cambridge University Press, New York,
  pp. 151-173.
}
\seealso{
\describe{
    \item{\code{\link{power_dsgn}}}{ for power calculations of multiple panel
      designs}
  }
}
\author{
Tony Olsen \email{Olsen.Tony@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{NLA_PNW}
\alias{NLA_PNW}
\title{NLA PNW data}
\format{
96 rows and 9 variables:
\describe{
  \item{\code{SITE_ID}}{ A unique lake identifier.}
  \item{\code{WEIGHT}}{ The sampling design weight.}
  \item{\code{URBAN}}{ Urban category.}
  \item{\code{STATE}}{ State name.}
  \item{\code{BMMI}}{ Benthic MMI value.}
  \item{\code{BMMI_COND}}{ Benthic MMI condition categories.}
  \item{\code{PHOS_COND}}{ Phosphorus condition categories.}
  \item{\code{NITR_COND}}{ Nitrogen condition categories.}
  \item{\code{geometry}}{ POINT geometry using the NAD83 / Conus Albers
    coordinate reference system (EPSG: 5070).}
}
}
\usage{
NLA_PNW
}
\description{
An \code{sf} POINT object of 96 lakes in the Pacific Northwest Region of the United
States during the year 2017, from a subset of the Environmental
Protection Agency's "National Lakes Assessment."
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjwgt.R
\name{adjwgt}
\alias{adjwgt}
\title{Adjust survey design weights by categories}
\usage{
adjwgt(wgt, wgtcat = NULL, framesize, sites = NULL)
}
\arguments{
\item{wgt}{Vector of initial weights for each site. These equal
the reciprocal of the site's inclusion probability.}

\item{wgtcat}{Vector containing each site's weight adjustment
category name. The default is \code{NULL}, which assumes every
site is in the same category.}

\item{framesize}{Vector containing the known size of the frame
for each category name in \code{wgtcat}. If \code{wgtcat} is provided,
the names in \code{framesize} must match the names in \code{wgtcat}.
If \code{wgtcat} is not provided, an unnamed scalar is given to
\code{framesize}.}

\item{sites}{Vector indicating site use; \code{TRUE} indicates the site
should be included in the weight adjustment and \code{FALSE} indicates
the site should not be included in the weight adjustment. The default is
\code{NULL}, which assumes every site should be included.}
}
\value{
Vector of adjusted weights, where the adjusted weight is set
  to \code{0} for sites whose value in the sites argument was set to
  \code{FALSE}.
}
\description{
Adjust initial survey design weights so that the
final weights sum to a desired frame size. Adjusted weights
proportionally scale the initial weights to sum to the desired frame size.
Separate adjustments are applied to each category specified in \code{wgtcat}.
}
\examples{
wgt <- runif(50)
wgtcat <- rep(c("A", "B"), c(30, 20))
framesize <- c(A = 15, B = 10)
sites <- rep(rep(c(TRUE, FALSE), c(9, 1)), 5)
adjwgt(wgt, wgtcat, framesize, sites)
}
\author{
Tony Olsen \email{olsen.tony@epa.gov}
}
\keyword{misc}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grts.R
\name{grts}
\alias{grts}
\title{Select a generalized random tessellation stratified (GRTS) sample}
\usage{
grts(
  sframe,
  n_base,
  stratum_var = NULL,
  seltype = NULL,
  caty_var = NULL,
  caty_n = NULL,
  aux_var = NULL,
  legacy_var = NULL,
  legacy_sites = NULL,
  legacy_stratum_var = NULL,
  legacy_caty_var = NULL,
  legacy_aux_var = NULL,
  mindis = NULL,
  maxtry = 10,
  n_over = NULL,
  n_near = NULL,
  wgt_units = NULL,
  pt_density = NULL,
  DesignID = "Site",
  SiteBegin = 1,
  sep = "-",
  projcrs_check = TRUE
)
}
\arguments{
\item{sframe}{A sampling frame as an \code{sf} object. The coordinate
system for \code{sframe} must projected (not geographic). If m or z values
are in \code{sframe}'s geometry, they are silently dropped (i.e., only x-coordinates
and y-coordinates are preserved).}

\item{n_base}{The base sample size required. If the sampling design is unstratified,
this is a single numeric value. If the sampling design is stratified, this is a named
vector or list whose names represent each stratum and whose values represent each
stratum's sample size. These names must match the values of the stratification
variable represented by \code{stratum_var}. Legacy sites are considered part
of the base sample, so the value for \code{n_base} should be equal to the number
of legacy sites plus the number of desired non-legacy sites.}

\item{stratum_var}{A character string containing the name of the column from
\code{sframe} that identifies stratum membership for each element in \code{sframe}.
If stratum equals \code{NULL}, the sampling design is unstratified and all elements in \code{sframe}
are eligible to be selected in the sample. The default is \code{NULL}.}

\item{seltype}{A character string or vector indicating the inclusion probability type,
which must be one of following: \code{"equal"} for equal inclusion probabilities;
\code{"unequal"} for unequal inclusion probabilities according to a categorical
variable specified by \code{caty_var}; and \code{"proportional"} for inclusion
probabilities proportional to a positive auxiliary variable specified by
\code{aux_var}. If the sampling design is unstratified, \code{seltype} is a single
character vector. If the sampling design is stratified, \code{seltype} is a named vector
whose names represent each stratum and whose values represent each stratum's
inclusion probability type. \code{seltype}'s default value tries to match the
intended inclusion probability type: If \code{caty_var} and \code{aux_var} are
not specified, \code{seltype} is \code{"equal"}; if \code{caty_var} is specified,
\code{seltype} is \code{"unequal"}; and if \code{aux_var} is specified, \code{seltype}
is \code{"proportional"}.}

\item{caty_var}{A character string containing the name of the column from
\code{sframe} that represents the unequal probability variable.}

\item{caty_n}{A character vector indicating the expected sample size for each
level of \code{caty_var}, the unequal probability variable. If the sampling design
is unstratified, \code{caty_n} is a named vector whose names represent each
level of \code{caty_var} and whose values represent each level's expected
sample size. The sum of \code{caty_n} must equal \code{n_base}. If the sampling design
is stratified and the expected sample sizes are the same among strata, \code{caty_n} is
a named vector whose names represent represent each
level of \code{caty_var} and whose values represent each level's expected
sample size -- these expected sample sizes are applied to all strata. The sum of
\code{caty_n} must equal each stratum's value in \code{n_base}.
If the sampling design is stratified and the expected sample sizes differ among strata,
\code{caty_n} is a list where each element  is named as a stratum in \code{n_base}.
Each stratum's list element is a named vector whose
names represent each level of \code{caty_var} and whose values represent each
level's expected sample size (within the stratum). The sum of the values in each stratum's
list element must equal that stratum's value in \code{n_base}.}

\item{aux_var}{A character string containing the name of the column from
\code{sframe} that represents the proportional (to size) inclusion probability
variable (auxiliary variable). This auxiliary variable must be positive, and the resulting
inclusion probabilities are proportional to the values of the auxiliary variable.
Larger values of the auxiliary variable result in higher inclusion probabilities.}

\item{legacy_var}{This argument can be used instead of \code{legacy_sites}
when \code{sframe} is a \code{POINT} or \code{MULTIPOINT}
geometry (i.e. a finite sampling frame),
When \code{legacy_var} is used, it is a character string containing the name of the column
from \code{sframe} that represents whether each site is a legacy site. For
legacy sites, the values of the \code{legacy_var} must contain character strings that
act as a legacy site identifier. For non-legacy sites, the values of the
\code{legacy_var} column must be \code{NA}. Using this approach,
\code{legacy_stratum_var}, \code{legacy_caty_var}, and \code{legacy_aux_var}
are not required and should not be used (because \code{legacy_var} represents a column
in \code{sframe}). \code{spsurvey} assumes that the legacy sites were selected from
a previous sampling design that incorporated randomness into site selection
and that the legacy sites are elements of the current sampling frame.}

\item{legacy_sites}{An sf object with a \code{POINT} or \code{MULTIPOINT}
geometry representing the legacy sites. spsurvey assumes that
the legacy sites were selected from a previous sampling design that
incorporated randomness into site selection and that the legacy sites
are elements of the current sampling frame. If \code{sframe} has a
\code{POINT} or \code{MULTIPOINT} geometry, the observations in \code{legacy_sites}
should not also be in \code{sframe} (i.e., duplicates are not removed). Thus, \code{sframe}
and \code{legacy_sites} together compose the current sampling frame. If m or z values
are in \code{legacy_sites}' geometry, they are silently dropped (i.e., only x-coordinates
and y-coordinates are preserved).}

\item{legacy_stratum_var}{A character string containing the name of the column from
\code{legacy_sites} that identifies stratum membership for each element of \code{legacy_sites}.
This argument is required when the sampling design is stratified and its levels
must be contained in the levels of the \code{stratum_var} variable. The default value of \code{legacy_stratum_var}
is \code{stratum_var}, so \code{legacy_stratum_var} need only be specified explicitly when
the name of the stratification variable in \code{legacy_sites} differs from \code{stratum_var}.}

\item{legacy_caty_var}{A character string containing the name of the column from
\code{legacy_sites} that identifies the unequal probability variable for each element of \code{legacy_sites}.
This argument is required when the sampling design uses unequal selection probabilities and its categories
must be contained in the levels of the \code{caty_var} variable. The default value of \code{legacy_caty_var}
is \code{caty_var}, so \code{legacy_caty_var} need only be specified explicitly when
the name of the unequal probability variable in \code{legacy_sites} differs from \code{caty_var}.}

\item{legacy_aux_var}{A character string containing the name of the column from
\code{legacy_sites} that identifies the proportional probability variable for each element of \code{legacy_sites}.
This argument is required when the sampling design uses proportional selection probabilities and the values of the
\code{legacy_aux_var} variable must be positive. The
default value of \code{legacy_aux_var} is \code{aux_var}, so \code{legacy_aux_var} need only be specified explicitly
when the name of the proportional probability variable in \code{legacy_sites} differs from \code{aux_var}.}

\item{mindis}{A numeric value indicating the desired minimum distance between sampled
sites. If the sampling design is stratified and \code{mindis} is an numeric value, the minimum
distance is applied to all strata. If the sampling design is stratified and different minimum distances
are desired among strata, then \code{mindis}
is a list whose names match the names of \code{n_base} and whose and values
are the minimum distance for the corresponding stratum.  If a minimum distance is not desired
for a particular stratum, then the corresponding value in \code{mindis} should be \code{0} or
 \code{NULL} (which is equivalent to \code{0}).
The units of \code{mindis} must represent the units in \code{sframe}. A warning is returned if the
minimum distance could not be reached after \code{maxtry} attempts. If legacy sites are used, the minimum distance
requirement (and subsequent warning if \code{maxtry} attempts are reached) is enforced for all base sites
that are not legacy sites (i.e., the minimum distance is enforced for these sites
by comparing distances against all base sites (legacy and non-legacy)).}

\item{maxtry}{The number of maximum attempts to apply the minimum distance algorithm to obtain
the desired minimum distance between sites. Each iteration takes roughly as long as the
standard GRTS algorithm. Successive iterations will always contain at least as many
sites satisfying the minimum distance requirement as the previous iteration. The algorithm stops
when the minimum distance requirement is met or there are \code{maxtry} iterations.
The default number of maximum iterations is \code{10}.}

\item{n_over}{The number of reverse hierarchically ordered (rho) replacement sites.
 If the sampling design is unstratified, then
 \code{n_over} is an integer specifying the number of rho replacement sites desired.
If the sampling design is stratified,
 then \code{n_over} is a vector (or list) whose names match the names of \code{n_base} and
 whose values indicate the number of rho replacement sites for each stratum.
 If replacement sites are not desired for a particular stratum, then the corresponding
 value in \code{n_over} should be \code{0} or \code{NULL} (which is equivalent to \code{0}).
 If the sampling design is stratified but the number of \code{n_over} sites is the same in each
 stratum, \code{n_over} can be a vector which is used for each stratum.
 If \code{n_over} is an unnamed, length-one vector, it's value is recycled
 and used for each stratum. Note that if the
 sampling design has unequal selection probabilities (\code{seltype = "unequal"}), then \code{n_over} sites
 are given the same proportion of \code{caty_n} values as \code{n_base}.}

\item{n_near}{The number of nearest neighbor (nn) replacement sites.
If the sampling design is unstratified, \code{n_near} is integer from \code{1}
to \code{10} specifying the number of
nn replacement sites to be selected for each base site. If the sampling design
is stratified but the same number of nn replacement sites is desired
for each stratum,  \code{n_near} is integer from \code{1}
to \code{10} specifying the number of
nn replacement sites to be selected for each base site. If the sampling design is
unstratified and a different number of nn replacement sites is
desired for each stratum, \code{n_near} is a vector (or list) whose names represent strata and whose
values is integer from \code{1}
to \code{10} specifying the number of
nn replacement sites to be selected for each base site in the stratum. If replacement sites
are not desired for a particular stratum, then the corresponding value in \code{n_over}
should be \code{0} or \code{NULL} (which is equivalent to \code{0}). For
infinite sampling frames, the distance between a site and its nn
depends on \code{pt_density}. The larger \code{pt_density}, the closer the nn neighbors.}

\item{wgt_units}{The units used to compute the design weights. These
units must be standard units as defined by the \code{set_units()} function in
the units package. The default units match the units of the sf object.}

\item{pt_density}{A positive integer controlling the density of the GRTS approximation
for infinite sampling frames. The GRTS approximation for infinite sample
frames vastly improves computational efficiency by generating many finite points and
selecting a sample from the points. \code{pt_density} represents the density
of finite points per unit to use in the approximation. More specifically,
for each stratum, the number of points used in the approximation equals
\code{pt_density * (n_base + n_over)}. A larger value of \code{pt_density}
means a closer approximation to the infinite sampling frame but less
computational efficiency. The default value of \code{pt_density} is \code{10}. Note that
when used with \code{caty_n}, the unequal inclusion probabilities generated from
this approach are also approximations.}

\item{DesignID}{A character string indicating the naming structure for each
site's identifier selected in the sample, which is matched with \code{SiteBegin} and
included as a variable in the
sf object in the function's output. Default is "Site".}

\item{SiteBegin}{A character string indicating the first number to use to match
with \code{DesignID} while creating each site's identifier selected in the sample.
Successive sites are given successive integers. The default starting number
is \code{1} and the number of digits is equal to number of digits in
\code{nbase + nover}.
For example, if \code{nbase} is 50 and \code{nover} is 0, then the default
site identifiers are \code{Site-01} to \code{Site-50}}

\item{sep}{A character string that acts as a separator between
\code{DesignID} and \code{SiteBegin}. The default is \code{"-"}.}

\item{projcrs_check}{A check for whether the coordinates are projected. If \code{TRUE},
an error is returned if coordinates are not projected  (i.e., they are geographic or NA). If \code{FALSE}, the
check is not performed, which means that the crs in \code{sframe} (and \code{legacy_sites} if provided) can be projected, geographic, or NA.}
}
\value{
The sampling design sites and additional information about the sampling design. More specifically, it is, a list with five elements:
  \itemize{
    \item \code{sites_legacy} An sf object containing legacy sites. This is
      \code{NULL} if legacy sites were not included in the sample.
    \item \code{sites_base} An sf object containing the base sites. This is \code{NULL}
    if \code{n_base} equals the number of legacy sites.
    \item \code{sites_over} An sf object containing the reverse hierarchically
      ordered replacement sites. This is \code{NULL} if no reverse hierarchically
      ordered replacement sites were included in the sample.
    \item \code{sites_near} An sf object containing the nearest neighbor
      replacement sites. This is \code{NULL} if no nearest neighbor replacement
      sites were included in the sample.
    \item \code{design} A list documenting the specifications of this sampling design.
      This can be checked to verify your sampling design ran as intended.
      \itemize{
        \item \code{call} The original function call.
        \item \code{stratum_var} The name of the stratification variable in \code{sframe}.
          This equals \code{NULL} if no stratification is used.
        \item \code{stratum} The unique strata. This equals \code{"None"} if
          the sampling design is unstratified.
        \item \code{n_base} The base sample size per stratum.
        \item \code{seltype} The selection type per stratum.
        \item \code{caty_var} The name of the unequal probability variable in \code{sframe}.
          This equals \code{NULL} if no unequal probability variable is used.
        \item \code{caty_n} The expected sample sizes for each level of the
          unequal probability grouping variable per stratum. This equals
          \code{NULL} when \code{seltype} is not \code{"unequal"}.
        \item \code{aux_var} The name of the proportional probability (auxiliary) variable in \code{sframe}.
          This equals \code{NULL} if no proportional probability variable is used.
        \item \code{legacy} A logical variable indicating whether legacy sites
          were included in the sample.
        \item \code{legacy_stratum_var} The name of the stratification variable in \code{legacy_sites}.
          Omitted if legacy sites are not used. This equals \code{NULL} if legacy sites were used but
          no stratification variable is used.
        \item \code{legacy_caty_var} The name of the unequal probability variable in \code{legacy_sites}.
          Omitted if legacy sites are not used. This equals \code{NULL} if legacy sites were used but
          no unequal probability variable is used.
        \item \code{legacy_aux_var} The name of the proportional probability (auxiliary)
          variable in \code{legacy_sites}.
          Omitted if legacy sites are not used. This equals \code{NULL} if legacy sites
          were used but no proportional probability variable is used.
        \item \code{mindis} The minimum distance requirement desired. This
          is \code{NULL} when no minimum distance requirement was applied.
        \item \code{n_over} The reverse hierarchically ordered replacement
          site sample sizes per stratum. If \code{seltype} is \code{unequal},
          this represents the expected sample sizes. This is \code{NULL}
          when no reverse hierarchically ordered replacement sites were selected.
        \item \code{n_near} The number of nearest neighbor replacement sites
          desired. This is \code{NULL} when no nearest neighbor replacement
          sites were selected.
      }
  }
  When non-\code{NULL}, the \code{sites_legacy}, \code{sites_base},
  \code{sites_over}, and \code{sites_near} objects contain the original columns
  in \code{sframe} and include a few additional columns. These additional columns
  are
  \itemize{
    \item \code{siteID} A site identifier (as named using the \code{DesignID}
      and \code{SiteBegin} arguments to \code{grts()}).
    \item \code{siteuse} Whether the site is a legacy site (\code{Legacy}), base
      site (\code{Base}), reverse hierarchically ordered replacement site
      (\code{Over}), or nearest neighbor replacement site (\code{Near}).
    \item \code{replsite} The replacement site ordering. \code{replsite} is
      \code{None} if the site is not a replacement site, \code{Next} if it is
      the next reverse hierarchically ordered replacement site to use, or
      \code{Near_}, where the word following \code{_} indicates the ordering of sites closest to
      the originally sampled site.
    \item \code{lon_WGS84} Longitude coordinates using the WGS84 coordinate
      system (EPSG:4326). Only given if coordinates are projected.
    \item \code{lat_WGS84} Latitude coordinates using the WGS84 coordinate
      system (EPSG:4326). Only given if coordinates are projected.
    \item \code{X} Longitude coordinates using the provided coordinate
      system. Only given if coordinates are not projected (i.e., they are geographic or NA).
    \item \code{Y} Latitude coordinates using the provided coordinate
      system. Only given if coordinates are not projected (i.e., they are geographic or NA).
    \item \code{stratum} A stratum indicator. \code{stratum} is \code{None}
      if the sampling design was unstratified. If the sampling design was \code{stratified},
      \code{stratum} indicates the stratum.
    \item \code{wgt} The design weight.
    \item \code{ip} The site's original inclusion probability (the reciprocal)
      of (\code{wgt}).
    \item \code{caty} An unequal probability grouping indicator. \code{caty}
      is \code{None} if the sampling design did not use unequal inclusion probabilities.
      If the sampling design did use unequal inclusion probabilities, \code{caty}
      indicates the unequal probability level.
    \item \code{aux} The auxiliary proportional probability variable. This
      column is only returned if \code{seltype} was \code{proportional} in the
      original sampling design.
  }
  If any columns in \code{sframe} contain these names, those columns
  from \code{sframe} will be automatically prefixed with \code{sframe_}
  in the \code{sites} object. When output is printed, a summary of site counts by
  the levels in \code{stratum_var} and \code{caty_var} is shown.
}
\description{
Select a spatially balanced sample from a point (finite), linear / linestring (infinite),
or areal / polygon (infinite) sampling frame using the Generalized Random Tessellation
Stratified (GRTS) algorithm. The GRTS algorithm accommodates unstratified and
stratified sampling designs and allows for equal inclusion probabilities, unequal
inclusion probabilities according to a categorical variable, and inclusion
probabilities proportional to a positive auxiliary variable. Several additional
sampling options are included, such as including legacy (historical) sites,
requiring a minimum distance between sites, and selecting replacement sites.
For technical details, see Stevens and Olsen (2004).
}
\details{
\code{n_base} is the number of sites used to calculate
  the design weights, which is typically the number of sites used in an analysis. When a panel sampling design is implemented, \code{n_base} is typically the
  number of sites in all panels that will be sampled in the same temporal period --
  \code{n_base} is not the total number of sites in all panels. The sum of \code{n_base} and
  \code{n_over} is equal to the total number of sites to be visited for all panels plus
  any replacement sites that may be required.
}
\examples{
\dontrun{
samp <- grts(NE_Lakes, n_base = 100)
print(samp)
strata_n <- c(low = 25, high = 30)
samp_strat <- grts(NE_Lakes, n_base = strata_n, stratum_var = "ELEV_CAT")
print(samp_strat)
samp_over <- grts(NE_Lakes, n_base = 30, n_over = 5)
print(samp_over)
}
}
\references{
Stevens Jr., Don L. and Olsen, Anthony R. (2004). Spatially balanced sampling
of natural resources. \emph{Journal of the American Statistical Association}, 99(465), 262-278.
}
\seealso{
\describe{
    \item{\code{\link{irs}}}{ to select a sample that is not spatially balanced}
 }
}
\author{
Tony Olsen \email{olsen.tony@epa.gov}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/localmean_weight.R
\name{localmean_weight}
\alias{localmean_weight}
\title{Internal Function: Local Mean Variance Neighbors and Weights}
\usage{
localmean_weight(x, y, prb, nbh = 4)
}
\arguments{
\item{x}{Vector of x-coordinates for location of the sample points.}

\item{y}{Vector of y-coordinates for location of the sample points.}

\item{prb}{Vector of inclusion probabilities for the sample points.}

\item{nbh}{Number of neighboring points to use in the calculations.}
}
\value{
If ginv fails to return valid output, a NULL object.  Otherwise, a
  list containing two elements: a matrix named \code{ij} composed of the
  index values of neighboring points and a vector named \code{gwt}
  composed of weights.
}
\description{
This function calculates the index values of neighboring points and
associated weights required by the local mean variance estimator.
}
\author{
Tom Kincaid \email{Kincaid.Tom@epa.gov}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdf_plot.R
\name{cdf_plot}
\alias{cdf_plot}
\title{Plot a cumulative distribution function (CDF)}
\usage{
cdf_plot(
  cdfest,
  var = NULL,
  subpop = NULL,
  subpop_level = NULL,
  units_cdf = "Percent",
  type_cdf = "Continuous",
  log = "",
  xlab = NULL,
  ylab = NULL,
  ylab_r = NULL,
  main = NULL,
  legloc = NULL,
  confcut = 0,
  conflev = 95,
  cex.main = 1.2,
  cex.legend = 1,
  ...
)
}
\arguments{
\item{cdfest}{Data frame with the same structure as the "CDF" output from
\code{cont_analysis}.}

\item{var}{If \code{cdfest} has multiple variables in the "Indicator" column,
then \code{var} is the single variable to be plotted. The default is
\code{NULL}, which assumes that only one variable is in the "Indicator"
column of \code{cdfest}.}

\item{subpop}{If \code{cdfest} has multiple variables in the "Type" column,
then \code{subpop} is the single variable to be plotted. The default is
\code{NULL}, which assumes that only one variable is in the "Type"
column of \code{cdfest}.}

\item{subpop_level}{If \code{cdfest} has multiple levels of \code{subpop}
in the "Subpopulation" column,
then \code{subpop_level} is the single level to be plotted. The default is
\code{NULL}, which assumes that only one level is in the "Subpopulation"
column of \code{cdfest}.}

\item{units_cdf}{Indicator for the label utilized for the left side y-axis
and the values used for the left side y-axis tick marks, where "Percent"
means the label and values are in terms of percent of the population, and
"Units" means the label and values are in terms of units (count, length,
or area) of the population.  The default is "Percent".}

\item{type_cdf}{Character string consisting of the value "Continuous" or
"Ordinal" that controls the type of CDF plot.  The default is "Continuous".}

\item{log}{Character string consisting of the value "" or "x" that
controls whether the x axis uses the original scale ("") or the base 10
logarithmic scale ("x").  The default is "".}

\item{xlab}{Character string providing the x-axis label.  If this argument
equals NULL, then the indicator name is used as the label.  The default is
NULL.}

\item{ylab}{Character string providing the left side y-axis label.  If
argument units_cdf equals "Units", a value should be provided for this
argument.  Otherwise, the label will be "Percent".  The default is
"Percent".}

\item{ylab_r}{Character string providing the label for the right side y-axis
(and, hence, determining the values used for the right side y-axis tick
marks), where NULL means a right side y-axis is not created.  If this
argument equals "Same", the right side y-axis will have the same label and
tick mark values as the left side y-axis.  If this argument equals a
character string other than "Same", the right side y-axis label will be the
value provided for argument ylab_r, and the right side y-axis tick mark
values will be determined by the choice not utilized for argument
units_cdf, which means that the default value of argument units_cdf (i.e.,
"Percent") will result in the right side y-axis tick mark values being
expressed  in terms of units of the population (i.e., count, length, or
area).  The default is NULL.}

\item{main}{Character string providing the plot title.  The default is
NULL.}

\item{legloc}{Indicator for location of the plot legend, where "BR" means
bottom right, "BL" means bottom left, "TR" means top right, "TL" means
top left, and NULL means no legend.  The default is NULL.}

\item{confcut}{Numeric value that controls plotting confidence limits at
the CDF extremes.  Confidence limits for CDF values (percent scale) less
than confcut or greater than 100 minus confcut are not plotted.  A value of
zero means confidence limits are plotted for the complete range of the CDF.
The default is 0.}

\item{conflev}{Numeric value of the confidence level used for confidence
limits. The default is 95.}

\item{cex.main}{Expansion factor for the plot title.  The default is 1.2.}

\item{cex.legend}{Expansion factor for the legend title. The default is 1.}

\item{...}{Additional arguments passed to the \code{plot.default} function
(aside from those already used and \code{ylim}).}
}
\value{
A plot of a variable's CDF estimates associated confidence limits.
}
\description{
This function creates a CDF plot.  Input data for the plots is provided by a
data frame with the same structure as the "CDF" output from  \code{cont_analysis}.
Confidence limits for the CDF also are plotted.
}
\examples{
\dontrun{
dframe <- data.frame(
  siteID = paste0("Site", 1:100),
  wgt = runif(100, 10, 100),
  xcoord = runif(100),
  ycoord = runif(100),
  stratum = rep(c("Stratum1", "Stratum2"), 50),
  ContVar = rnorm(100, 10, 1),
  All_Sites = rep("All Sites", 100),
  Resource_Class = rep(c("Good", "Poor"), c(55, 45))
)
myvars <- c("ContVar")
mysubpops <- c("All_Sites", "Resource_Class")
mypopsize <- data.frame(
  Resource_Class = c("Good", "Poor"),
  Total = c(4000, 1500)
)
myanalysis <- cont_analysis(dframe,
  vars = myvars, subpops = mysubpops,
  siteID = "siteID", weight = "wgt", xcoord = "xcoord", ycoord = "ycoord",
  stratumID = "stratum", popsize = mypopsize
)
keep <- with(myanalysis$CDF, Type == "Resource_Class" &
  Subpopulation == "Good")
par(mfrow = c(2, 1))
cdf_plot(myanalysis$CDF[keep, ],
  xlab = "ContVar",
  ylab = "Percent of Stream Length", ylab_r = "Stream Length (km)",
  main = "Estimates for Resource Class: Good"
)
cdf_plot(myanalysis$CDF[keep, ],
  xlab = "ContVar",
  ylab = "Percent of Stream Length", ylab_r = "Same",
  main = "Estimates for Resource Class: Good"
)
}
}
\seealso{
\describe{
  \item{\code{\link{cont_cdfplot}}}{for creating a PDF file containing CDF
    plots}
  \item{\code{\link{cont_cdftest}}}{for CDF hypothesis testing}
  }
}
\author{
Tom Kincaid \email{Kincaid.Tom@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Illinois_River}
\alias{Illinois_River}
\title{Illinois River data}
\format{
244 rows and 2 variables:
\describe{
  \item{\code{STATE_NAME}}{ State name.}
  \item{\code{geometry}}{ MULTILINESTRING geometry using the NAD83 / Conus Albers
    coordinate reference system (EPSG: 5070).}
}
}
\usage{
Illinois_River
}
\description{
An (\code{sf}) MULTILINESTRING object of 244 segments of the
Illinois River in Arkansas and Oklahoma.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ash1_wgt.R
\name{ash1_wgt}
\alias{ash1_wgt}
\title{Compute the average shifted histogram (ASH) for one-dimensional weighted data}
\usage{
ash1_wgt(
  x,
  wgt = rep(1, length(x)),
  m = 5,
  nbin = 50,
  ab = NULL,
  support = "Continuous"
)
}
\arguments{
\item{x}{Vector used to estimate the density. \code{NA} values are allowed.}

\item{wgt}{Vector of weights for each observation from a
probability sample. The default assigns equal weights (equal probability).}

\item{m}{Number of empty bins to add to the ends when the range is not
completely specified.  The default is \code{5}.}

\item{nbin}{Number of bins for density estimation.  The default is \code{50}.}

\item{ab}{Optional range for support associated with the density. Both
values may be equal to \code{NA}.  If equal to \code{NA}, then corresponding limit will
be based on \code{nicerange()}. The default is \code{NULL}.}

\item{support}{Type of support.  If equal to \code{"Continuous"}, then data are
from a continuous distribution.  If equal to \code{"Ordinal"}, then data are from
a discrete distribution defined for integers only.  The default is
\code{"Continuous"}.}
}
\value{
List containing the ASH density estimate.  List consists of
  \describe{
    \item{\code{tcen}}{ x-coordinate for center of bin}
    \item{\code{f}}{ y-coordinate for density estimate height}
  }
}
\description{
Calculate the average shifted histogram estimate of a density based on one-dimensional data
  from a survey design with weights.
}
\examples{
x <- rnorm(100, 10, sqrt(10))
wgt <- runif(100, 10, 100)
rslt <- ash1_wgt(x, wgt)
plot(rslt)
}
\references{
Scott, D. W. (1985). "Averaged shifted histograms: effective nonparametric
  density estimators in several dimensions." \emph{The Annals of Statistics} 13(3):
  1024-1040.
}
\author{
Tony Olsen \email{Olsen.tony@epa.gov}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{NRSA_EPA7}
\alias{NRSA_EPA7}
\title{NRSA EPA7 data}
\format{
353 rows and 10 variables:
\describe{
  \item{\code{SITE_ID}}{ A unique site identifier.}
  \item{\code{YEAR}}{ Year of design cycle.}
  \item{\code{WEIGHT}}{ Sampling design weights.}
  \item{\code{ECOREGION}}{ Ecoregion.}
  \item{\code{STATE}}{ State name.}
  \item{\code{BMMI}}{ Benthic MMI value.}
  \item{\code{BMMI_COND}}{ Benthic MMI categories.}
  \item{\code{PHOS_COND}}{ Phosphorus condition categories.}
  \item{\code{NITR_COND}}{ Nitrogen condition categories.}
  \item{\code{geometry}}{ POINT geometry using the NAD83 / Conus Albers
    coordinate reference system (EPSG: 5070).}
}
}
\usage{
NRSA_EPA7
}
\description{
An \code{sf} POINT object of 353 stream segments in the Central
United States during the years 2008 and 2013, from a subset of the Environmental
Protection Agency's "National Rivers and Streams Assessment."
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppd_plot.R
\name{ppd_plot}
\alias{ppd_plot}
\title{Plot power curves for panel designs}
\usage{
ppd_plot(
  object,
  plot_type = "standard",
  trend_type = "mean",
  xaxis_type = "period",
  comp_type = "design",
  dsgns = NULL,
  indicator = NULL,
  trend = NULL,
  period = NULL,
  alpha = NULL,
  ...
)
}
\arguments{
\item{object}{List object of class \code{powerpaneldesign}. Object provides
power calculated for a set of panel designs, set of indicators, set of
trend values, and set of alpha values. Expect input as list as output from
function \code{power_dsgn}.}

\item{plot_type}{Default is \code{"standard"} which plots standard power curve. If
equal to \code{"relative"}, then plot power of one panel design compared to one or
more other panel designs.}

\item{trend_type}{Character value for trend in mean (\code{"mean"}) or or percent
change in proportion (\code{"percent"}) of cumulative distribution function above
or below a fixed cut point.  Default is \code{"mean"}.}

\item{xaxis_type}{Character value equal to \code{"period"} or \code{"change"} which
designates the type of x-axis for power plot where power is plotted on
y-axis.  For \code{xaxis_type = "period"}, x-axis is periods in \code{dsgnpower}. If
\code{xaxis_type = "change"}, then x-axis is percent per period with secondary
x-axes for total percent per period and associated change in mean.
Default is \code{"period"}.  Note that \code{xaxis_type} controls how the input for
\code{"period"} and \code{"trend"} parameters is used.}

\item{comp_type}{Character value equal to \code{"design"} or \code{"indicator"} which
designates the type of power curve comparison that will occur on a single
plot.  If \code{comp_type = "design"}, then on a single plot of power curves all
panel designs specified in \code{"dsgns"} are plotted for a single indicator,
single trend value and single alpha.  If \code{comp_type = "indicator"}, then on a
single plot of power curves all indicators specified in \code{"indicator"} are
plotted for a single panel design, single trend value and single alpha.
Default is \code{"design"}.}

\item{dsgns}{Vector of names of panel designs that are to be plotted.  Names
must be all, or a subset of, names of designs in \code{dsgnpower}. Default is \code{NULL}
which results in only the first panel design in \code{dsgnpower} being used.}

\item{indicator}{Vector of indicator names contained in \code{dsgnpower} that are
to be plotted.  Indicator names must be all, or a subset of, indicator
names in \code{dsgnpower}. Default is \code{NULL} which results in only the first
indicator in \code{dsgnpower} being used.}

\item{trend}{\code{NULL}. A single value or vector of values contained in \code{dsgnpower}
that will be plotted. Values must be all, or a subset of, trend values in
\code{dsgnpower}. If \code{xaxis_type} is equal to \code{"period"}, then \code{NULL} results in maximum
trend value being used and a single value or vector of values results in a
separate plot for each value specified.  If \code{xaxis_type} is equal to
\code{"change"}, then \code{NULL} results in all trend values in \code{dsgnpower} being plotted
on x-axis and a vector of values results in all trend values in \code{dsgnpower}
from minimum value to maximum value specified being plotted on x-axis.}

\item{period}{\code{NULL}, a single value or vector of values contained in \code{dsgnpower}
that will be plotted. Values must be all, or a subset of, period values in
\code{dsgnpower}. If \code{xaxis_type} is equal to \code{"period"}, then \code{NULL} results in all
time periods in \code{dsgnpower} being plotted on x-axis and a vector of values
results in all period values in \code{dsgnpower} from minimum value to maximum
value specified being plotted on x-axis. If \code{xaxis_type} is equal to
\code{"change"}, then \code{NULL} results in all time periods in \code{dsgnpower} being plotted
in separate plots and a vector of values results in time periods specified
being plotted in separate plots.}

\item{alpha}{A single value or vector of significance levels (as proportion,
e.g. \code{0.05}) contained in \code{dsgnpower} to used for power plots. Specifying more
than a single value results in multiple plots. Default is \code{NULL} which
results in the minimum significance level in \code{dsgnpower} being used.}

\item{...}{Additional arguments (S3 consistency)}
}
\value{
One or more power curve plots are created and plotted.  User must
  specify output graphical device if more than one plot is created.  See
  Devices for graphical output options.
}
\description{
Plot power curves and relative power curves for trend detection for set of
panel designs, time periods, indicators, significance levels and trend.  Trend
may be based on percent change per period in mean or percent change in
proportion of cumulative distribution function above or below a fixed cut
point. Types of plots are combinations of standard/relative, mean/percent,
period/change and design/indicator.  Input must be be of class
powerpaneldesign and is normally the output of function power_dsgn.
}
\details{
By default the plot function produces a standard power curve at end
  of each time period on the x-axis with y-axis as power. When more than one
  panel design is in \code{dsgnpower}, the first panel design is used. When more than
  one indicator is in \code{dsgnpower}, the first indicator is used.  When more than
  one trend value is in \code{dsgnpower}, the maximum trend value is used. When more
  than one significance level, \code{alpha}, is in \code{dsgnpower}, the minimum
  significance level is used.

  Control of the type of plot produced is governed by \code{plot_type}, \code{trend_type},
  \code{xaxis_type} and \code{comp_type}. The number of plots produced is governed by the
  number of panel designs (\code{dsgn}) specified, the number of indicators
  (\code{indicator}) specified, the number of time periods (\code{period}) specifies, the
  number of trend values (trend) specified and the number of significance
  levels (\code{alpha}) specified.

  When the comparison type (\code{"comp_type"}) is equal to \code{"design"}, all power
  curves specified by dsgn are plotted on the same plot.  When \code{comp_type} is
  equal to \code{"indicator"}, all power curves specified by \code{"indicator"} are plotted
  on the same plot.  Typically, no more than 4-5 power curves should be
  plotted on same plot.
}
\examples{
\dontrun{
# Construct a rotating panel design with sample size of 60
R60N <- revisit_dsgn(20, panels = list(R60N = list(
  n = 60, pnl_dsgn = c(1, NA),
  pnl_n = NA, start_option = "None"
)), begin = 1)

# Construct a fixed panel design with sample size of 60
F60 <- revisit_dsgn(20, panels = list(F60 = list(
  n = 60, pnl_dsgn = c(1, 0),
  pnl_n = NA, start_option = "None"
)), begin = 1)

# Power for rotating panel with sample size 60
Power_tst <- power_dsgn("Variable_Name",
  ind_values = 43, unit_var = 280,
  period_var = 4, unitperiod_var = 40, index_var = 90,
  unit_rho = 1, period_rho = 0, paneldsgn = list(
    R60N = R60N, F60 = F60
  ), nrepeats = NULL,
  trend_type = "mean", trend = c(1.0, 2.0), alpha = 0.05
)
ppd_plot(Power_tst)
ppd_plot(Power_tst, dsgns = c("F60", "R60N"))
ppd_plot(Power_tst, dsgns = c("F60", "R60N"), trend = 1.0)
ppd_plot(Power_tst,
  plot_type = "relative", comp_type = "design",
  trend_type = "mean", trend = c(1, 2), dsgns = c("R60N", "F60"),
  indicator = "Variable_Name"
)
}
}
\author{
Tony Olsen \email{Olsen.Tony@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjwgtNR.R
\name{adjwgtNR}
\alias{adjwgtNR}
\title{Adjust survey design weights for non-response by categories}
\usage{
adjwgtNR(wgt, MARClass = NULL, EvalStatus, TNRClass, TRClass)
}
\arguments{
\item{wgt}{vector of weights for each sample unit that will be adjusted
for non-response. Weights must be weights for the design as implemented.
All weights must be greater than zero.}

\item{MARClass}{vector that identifies for each sample unit the class
(i.e., category, level)
that will be used in non-response weight adjustment for sample units
that are known to be target. Within each missing at random (MAR)
class, the missing sample units that are not sampled are assumed to
be missing at random. If MARClass is not specified, all sample units
are assumed to be from the same MAR class.}

\item{EvalStatus}{vector of the evaluation status for each sample unit.
Values must include the values given in TNRclass and TRClass. May
include other values not required for the non-response adjustment.}

\item{TNRClass}{subset of values in EvalStatus that identify sample units
whose target status is known and that do not respond (i.e., are not
sampled).}

\item{TRClass}{Subset of values in EvalStatus that identify sample units
whose target status is known and that respond (i.e., are target and
sampled).}
}
\value{
Vector of sample unit weights that are adjusted for non-response
 and that is the same length of input weights. Weights for sample
 units that did not response but were known to be eligible are set
 to zero. Weights for all other sample units are also set to zero.
}
\description{
Adjust weights for target sample units that do not respond
and are missing at random within categories. The missing at random
assumption implies that their sample weight may be assigned to
specific categories of units that have responded (i.e., have been
sampled). This is a class-based method for non-response adjustment.
}
\examples{
set.seed(5)
wgt <- runif(40)
MARClass <- rep(c("A", "B"), rep(20, 2))
EvalStatus <- sample(c("Not_Target", "Target_Sampled", "Target_Not_Sampled"), 40, replace = TRUE)
TNRClass <- "Target_Not_Sampled"
TRClass <- "Target_Sampled"
adjwgtNR(wgt, MARClass, EvalStatus, TNRClass, TRClass)
# function that has an error check
}
\author{
Tony Olsen \email{olsen.tony@epa.gov}
}
\keyword{adjustment}
\keyword{non-response}
\keyword{survey}
\keyword{weight}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/revisit_bibd.R
\name{revisit_bibd}
\alias{revisit_bibd}
\title{Create a balanced incomplete block panel revisit design}
\usage{
revisit_bibd(
  n_period,
  n_pnl,
  n_visit,
  nsamp,
  panel_name = "BIB",
  begin = 1,
  skip = 1,
  iter = 30
)
}
\arguments{
\item{n_period}{Number of time periods for the survey design. Typically,
number of periods if sampling occurs once per period or number of months if
sampling occurs once per month. (v, number of varieties/treatments in BIBD
terms)}

\item{n_pnl}{Number of panels (b, number of blocks in BIBD terms)}

\item{n_visit}{Number of time periods to be visited in a panel (k, block
size in BIBD terms)}

\item{nsamp}{Number of samples in each panel.}

\item{panel_name}{Prefix for name of each panel}

\item{begin}{Numeric name of first sampling occasion, e.g. a specific
period.}

\item{skip}{Number of sampling occasions to skip between planned sampling
periods, e.g., sampling will occur only every 5 periods if \code{skip = 5}.}

\item{iter}{Maximum number of iterations in search for D-optimal
Generalized Youden Design.}
}
\value{
A two-dimensional array of sample sizes to be sampled for each panel
  and each sampling occasion.
}
\description{
Create a revisit design for panels in a survey that specifies the time
periods for the units of each panel to be sampled based on searching for a
D-optimal block design that is a member of the class of generalized Youden
designs.  The resulting design need not be a balanced incomplete block
design.  Based on algorithmic idea by Cook and Nachtsheim (1989) and
implemented by Robert Wheeler.
}
\details{
The function uses \code{find.BIB} function from crossdes package to
search for a D-optimal block design.  crossdes uses package AlgDesign
to search balanced incomplete block designs.
}
\examples{
# Balanced incomplete block design with 20 sample occasions, 20 panels,
# 3 visits to each unit, and 20 units in each panel.
revisit_bibd(n_period = 20, n_pnl = 20, n_visit = 3, nsamp = 20)
}
\references{
Cook R. D.  and C. Nachtsheim. (1989). Computer-aided blocking of factorial
  and response-surface designs. \emph{Technometrics} \bold{31(3)}, 339-346.
}
\seealso{
\describe{
    \item{\code{\link{revisit_dsgn}}}{to create a panel revisit design}
    \item{\code{\link{revisit_rand}}}{to create a panel revisit design with random
      assignment to panels and time periods}
    \item{\code{\link{pd_summary}}}{ to summarize characteristics of a
      panel revisit design}
  }
}
\author{
Tony Olsen \email{Olsen.Tony@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sp_rbind.R
\name{sp_rbind}
\alias{sp_rbind}
\title{Combine rows from GRTS or IRS samples.}
\usage{
sp_rbind(object, siteuse = NULL)
}
\arguments{
\item{object}{The design sites (output from \code{grts()} or \code{irs()}).}

\item{siteuse}{A character vector of site types to return. Can contain
\code{"Legacy"} (for legacy sites), \code{"Base"} (for base sites),
\code{"Over"} (for \code{n_over} replacement sites), and \code{"Near"}
(for \code{n_near} replacement sites). The default is \code{NULL}, which
returns all non-\code{NULL} output from \code{object$sites_legacy},
\code{object$sites_base}, \code{object$sites_over}, and \code{object$sites_near}.}
}
\value{
A single \code{sf} object containing all requested design sites.
}
\description{
This function row binds the \code{sites_legacy}, \code{sites_base},
\code{sites_over}, and \code{sites_near} objects from a GRTS or IRS sample
into a single \code{sf} object. This function is most useful when a single
\code{sf} object that contains all design sites is desired
(e.g. writing out a single shapefile using \code{sf::write_sf()}).
}
\examples{
\dontrun{
sample <- grts(NE_Lakes, 50, n_over = 10)
sample <- sp_rbind(sample)
write_sf(sample, "mypath/sample.shp")
}
}
\author{
Michael Dumelle \email{Dumelle.Michael@epa.gov}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power_dsgn.R
\name{power_dsgn}
\alias{power_dsgn}
\title{Power calculation for multiple panel designs}
\usage{
power_dsgn(
  ind_names,
  ind_values,
  unit_var,
  period_var,
  unitperiod_var,
  index_var,
  unit_rho = 1,
  period_rho = 0,
  paneldsgn,
  nrepeats = NULL,
  trend_type = "mean",
  ind_pct = NULL,
  ind_tail = NULL,
  trend = 2,
  alpha = 0.05
)
}
\arguments{
\item{ind_names}{Vector of indicator names}

\item{ind_values}{Vector of indicator mean values}

\item{unit_var}{Vector of variance component estimates for unit variability
for the indicators}

\item{period_var}{Vector of variance component estimates for period
variability for the indicators}

\item{unitperiod_var}{Vector of variance component estimates for unit by
period interaction variability for the indicators}

\item{index_var}{Vector of variance component estimates for index (residual)
error for the indicators}

\item{unit_rho}{Correlation across units. Default is \code{1}.}

\item{period_rho}{Correlation across periods. Default is \code{0}.}

\item{paneldsgn}{A list of panel designs each as a matrix.  Each element of
the list is a matrix with \code{dimnames} (dimensions: number of panels (rows) by
number of periods (columns)) containing the number of units visited for
each combination of panel and period.  Dimnames for columns must be
able to be coerced into an integer (e.g., 2016).  All designs must span the same
number of periods.  Typically, the panel designs are the output of the
function \code{revisit_dsgn}.}

\item{nrepeats}{Either \code{NULL} or a list of matrices the same length as
\code{paneldsgn} specifying the number of revisits made to units in a panel in the
same period for each design.  Specifying \code{NULL} indicates that number of
revisits to units is the same for all panels and for all periods and for
all panel designs. The default is \code{NULL}, a single visit. Names must match
list names in \code{paneldsgn}.}

\item{trend_type}{Trend type is either \code{"mean"} where trend is applied as
percent trend in the indicator mean or \code{"percent"} where the trend is applied
as percent trend in the proportion (percent) of the distribution that is
below or above a fixed value. Default is \code{trend_type="mean"}}

\item{ind_pct}{When \code{trend_type} is equal to \code{"percent"}, a vector of the
values of the indicator fixed value that defines the percent.  Default is
NULL}

\item{ind_tail}{When trend_type is equal to \code{"percent"}, a character vector
with values of either \code{"lower"} or \code{"upper"} for each indicator.  \code{"lower"}
states that the percent is associated with the lower tail of the
distribution and \code{"upper"} states that the percent is associated with the
upper tail of the distribution. Default is \code{NULL}.}

\item{trend}{Single value or vector of assumed percent change from
initial value in the indicator for each period. Assumes the trend is
expressed as percent per period. Note that the trend may be either positive
or negative. The default is \code{2}.}

\item{alpha}{Single value or vector of significance level for linear
trend test, alpha, Type I error, level.  The default is \code{0.05}.}
}
\value{
A list with components \code{trend_type}, \code{ind_pct}, \code{ind_tail}, trend values
  across periods, periods (all periods included in one or more panel
  designs), significance levels, a five-dimensional array of power
  calculations (dimensions: panel, design names, periods, indicator names,
  trend names, \code{alpha_names}), an array of indicator mean values for each trend
  and the function call.
}
\description{
Calculates the power for trend detection for one or more variables, for one
or more panel designs, for one or more linear trends, and for one or more
significance levels.  The panel designs create a covariance model where the
model includes variance components for units, periods, the interaction of
units and periods, and the residual (or index) variance.
}
\details{
Calculates the power for detecting a change in the mean for
  different panel design structures. The model incorporates unit, period,
  unit by period, and index variance components as well as correlation across
  units and across periods.  See references for methods.
}
\examples{
# Power for rotating panel with sample size 60
power_dsgn("Variable_Name",
  ind_values = 43, unit_var = 280, period_var = 4,
  unitperiod_var = 40, index_var = 90, unit_rho = 1, period_rho = 0,
  paneldsgn = list(NoR60 = revisit_dsgn(20,
    panels = list(NoR60 = list(
      n = 60, pnl_dsgn = c(1, NA),
      pnl_n = NA, start_option = "None"
    )), begin = 1
  )),
  nrepeats = NULL, trend_type = "mean", trend = 1.0, alpha = 0.05
)
}
\references{
Urquhart, N. S., W. S. Overton, et al. (1993) Comparing sampling designs
  for monitoring ecological status and trends: impact of temporal patterns.
  In: \emph{Statistics for the Environment.} V. Barnett and K. F. Turkman.
  John Wiley & Sons, New York, pp. 71-86.

  Urquhart, N. S. and T. M. Kincaid (1999). Designs for detecting trends
  from repeated surveys of ecological resources. \emph{Journal of
  Agricultural, Biological, and Environmental Statistics}, \bold{4(4)},
  404-414.

  Urquhart, N. S. (2012). The role of monitoring design in detecting trend in
  long-term ecological monitoring studies. In: \emph{Design and Analysis of
  Long-term Ecological Monitoring Studies.} R. A. Gitzen, J. J. Millspaugh,
  A. B. Cooper, and D. S. Licht (eds.). Cambridge University Press, New York,
  pp. 151-173.
}
\seealso{
\describe{
    \item{\code{\link{ppd_plot}}}{ to plot power curves for
      panel designs}
  }
}
\author{
Tony Olsen \email{Olsen.Tony@epa.gov}
}
\keyword{survey}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/warnprnt.R
\name{warnprnt}
\alias{warnprnt}
\title{Print grts(), irs()), and analysis function warnings}
\usage{
warnprnt(warn_df = get("warn_df", envir = .GlobalEnv), m = 1:nrow(warn_df))
}
\arguments{
\item{warn_df}{Data frame that contains warning messages.  The default is
\code{"warn_df"}, which is the name given to the warnings data frame created by
functions in the spsurvey package.}

\item{m}{Vector of indices for warning messages that are to be printed. The
default is a vector containing the integers from 1 through the number of
rows in \code{warn_df}, which will print all warning messages in the data frame.}
}
\value{
Printed warnings.
}
\description{
This function prints the warnings messages from the \code{grts()}, \code{irs()},
  and analysis functions.
}
\author{
Tom Kincaid \email{Kincaid.Tom@epa.gov}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Lake_Ontario}
\alias{Lake_Ontario}
\title{Lake Ontario data}
\format{
187 rows and 5 variables:
\describe{
  \item{\code{COUNTRY}}{ Country.}
  \item{\code{RSRC_CLASS}}{ Bay class.}
  \item{\code{PSTL_CODE}}{ Postal code.}
  \item{\code{AREA_SQKM}}{ Area in square kilometers}
  \item{\code{geometry}}{ MULTIPOLYGON geometry using the NAD83 / Conus Albers
    coordinate reference system (EPSG: 5070).}
}
}
\usage{
Lake_Ontario
}
\description{
An \code{sf} MULTIPOLYGON object of 187 polygons consisting
of shore segments in Lake Ontario.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{NE_Lakes_df}
\alias{NE_Lakes_df}
\title{New England Lakes data (as a data frame)}
\format{
195 rows and 6 variables:
\describe{
  \item{\code{AREA}}{ Lake area in hectares.}
  \item{\code{AREA_CAT}}{ Lake area categories based on a hectare cutoff.}
  \item{\code{ELEV}}{ Elevation in meters.}
  \item{\code{ELEV_CAT}}{ Elevation categories based on a meter cutoff.}
  \item{\code{XCOORD}}{ x-coordinate using the WGS 84 coordinate reference system
    (EPSG: 4326)}
  \item{\code{YCOORD}}{ y-coordinate using WGS 84 coordinate reference system
    (EPSG: 4326)}
}
}
\usage{
NE_Lakes_df
}
\description{
An data frame of 195 lakes in the Northeastern
United States.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stopprnt.R
\name{stopprnt}
\alias{stopprnt}
\title{Print grts() and irs() errors.}
\usage{
stopprnt(stop_df = get("stop_df", envir = .GlobalEnv), m = 1:nrow(stop_df))
}
\arguments{
\item{stop_df}{Data frame that contains stop messages.  The default is
\code{stop_df}, which is the name given to the stop data frame created by
functions in the spsurvey package.}

\item{m}{Vector of indices for stop messages that are to be printed. The
default is a vector containing the integers from 1 through the number of
rows in \code{stop_df}, which will print all stop messages in the data frame.}
}
\value{
Printed errors
}
\description{
This function prints the error messages vector in the \code{grts}
and \code{irs} functions.
}
\author{
Tony Olsen \email{Olsen.Tony@epa.gov}
}
